#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"
#include "APIBacesArm.h"
#include "API_Baces/API_Baces_FcnDeclarations.h"
#include "API_Baces/DAPI_Baces_Startup_Settings.h"

APIBacesArm::APIBacesArm(void)
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;

	buttontime = 4294967295;
	LongDown = FALSE;

	app.saarm.CMMString = _T("Axxis");
	app.saarm.SettingsFileName = _T("SA_Axxis_ArmSettings.bin");
	LogFileName = _T("Axxis.log");
}

APIBacesArm::~APIBacesArm(void)
{
	DisconnectFromHardware();
}

bool APIBacesArm::ConnectToArm(const int dof)
{
	if(!app.saarm.Data.doconnection || ConnectedToArm)
		return true;	// simulation mode

	if(!CreateBacesInstance())
		return false;

	// get/set communications port and param file path
	DAPI_Baces_Startup_Settings dlg;
	dlg.port = app.saarm.Data.apiBacesComPort;
	dlg.paramPath = app.saarm.Data.apiBacesParamPath;
	if(dlg.DoModal() != IDOK)
		return false;
	app.saarm.Data.apiBacesComPort = dlg.port;
	app.saarm.Data.apiBacesParamPath = dlg.paramPath;

	// removed warning below... they changed tab file name such that warning no longer applies
	//if(app.saarm.Data.apiBacesParamPath.Find(_T("Arm")) < 0)
	//{
	//	CString m;
	//	m.Format(_T("The 6 DOF arm calibration (*.tab) file should contain the string 'Arm'.\n\nContinue Anyway?"));
	//	int ret = AfxMessageBox(m, MB_ICONQUESTION | MB_YESNO);
	//	if(ret!=IDYES)
	//	{
	//		return false;
	//	}
	//}
		
	DOF = dof;
	// 2013.04.03 - Removed the below intelligence because now, the user is allowed to put a probe on the 7th axis.  Used to be that the 7th axis was literally removed with the scanner.
	// So now, the user will have to be sure to pick the right arm in SA.  Will add an Axxis model that is 7 dof with a probe for this.
	//if(app.saarm.Data.apiBacesParamPath.Find(_T("Scanner")) > 0)
	//	DOF = 7;
	//else if(app.saarm.Data.apiBacesParamPath.Find(_T("Arm")) > 0)
	//	DOF = 6;

	J.Create(DOF, TRUE);	// create joint vector
	lastJ.Create(DOF, TRUE);
	JShift.Create(DOF, TRUE);

	// open the communications port
	if(app.saarm.Data.apiBacesComPort > -1)	// 2013.04.26 - This was checking for >0 (0 is the USB port), but as of baces dll v.4.4.1.4 it has to be called even in that case, or the "Baces 3D - Probes Management" in the pop-up from the Baces icon in the tool tray will be disabled (confirmed this with Pietro Pasqualetti @ Baces
	{
		CString port;
		port.Format(_T("COM%d"), app.saarm.Data.apiBacesComPort);
		if(OpenCommPort(port) != 0)
		{
			// could do error reporting here based on return value
			return false;
		}
	}

	// read in the parameter file...
	int ret = ReadParamFile(app.saarm.Data.apiBacesParamPath);
	if(ret != 0)
	{
		CString m;
		m.Format(_T("Could not read parameter file at location\n%s"), app.saarm.Data.apiBacesParamPath);
		AfxMessageBox(m, MB_ICONHAND);
		return false;
	}

	// reset the encoders...
	ret = ResetBacesEncoders();
	if(ret != 0)
	{
		CString m;
		if(ret == -1)
		{
			m.Format(_T("Could not reset Arm Encoders.\nIs the arm connected and on?"));
		}
		else
		{
			m.Format(_T("Could not reset Arm Encoders."));
		}
		AfxMessageBox(m, MB_ICONHAND);
		return false;
	}

	ConnectedToArm = TRUE;
	return true;
}
BOOL APIBacesArm::DisconnectFromHardware()
{
	if(!ConnectedToArm)
		return app.saarm.DisconnectFromHardware();

	// close communications port
	int closed = -9;
	closed = CloseCommPort();

	return ReleaseBacesInstance();
}

void APIBacesArm::DoPostInitOperations()
{
	if(app.saarm.InstrumentModel == 104 || app.saarm.InstrumentModel == 105 || app.saarm.InstrumentModel == 106 || app.saarm.InstrumentModel == 146) // Axxis arms
	{
		app.saarm.Data.autoCorrectModel = FALSE;	// we can't read the 7th axis!  Requested this 
	}
}

int APIBacesArm::GetButtonCount()
{
	if(DOF == 7)
		return 2;
	return 3;
}

BOOL APIBacesArm::GetAllData()
{
	if( !ConnectedToArm )
		return ArmBase::GetAllData();

	if(!GetPos())
		return FALSE;

	GetJoints();

	return TRUE;
}
BOOL APIBacesArm::GetPos()	// and button states
{
	if( !ConnectedToArm )
		return ArmBase::GetPos();

	if(ReadPositionAndButtonStatesAndProbeDiameter() != 0)
		return FALSE;

	return TRUE;
}
BOOL APIBacesArm::GetJoints(bool simulate/*=true*/)
{
	if( !ConnectedToArm )
		return ArmBase::GetJoints(simulate);

	if(ReadJointAngles() != 0)
		return FALSE;

	return TRUE;
}

void APIBacesArm::DoCal()
{
	if( !ConnectedToArm )
	{
		ArmBase::DoCal();
		return;
	}

	MessageBox(NULL,
		Tr("Just right click the arm icon in the tool bar for probe calibration/selection."),
		Tr("Probe Cal."),
		MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST);
	return;
}
void APIBacesArm::ProbeCalSet()
{
	if( !ConnectedToArm )
	{
		ArmBase::ProbeCalSet();
		return;
	}
	MessageBox(NULL,
		Tr("Just right click the arm icon in the tool bar for probe calibration/selection."),
		Tr("Probe Cal. Set"),
		MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST);
	return;
}

////////////////////////////////////////////////////////////////////////////
// Hardware interface level (see our declarations in 'API_Baces_FcnDeclarations.h'....

HINSTANCE bacesInst;

tOpenCommPortAddr OpenCommPortAddr;
tCloseCommPortAddr CloseCommPortAddr;
tReadParamFileAddr ReadParamFileAddr;
tResetEncodersAddr ResetEncodersAddr;
tReadPositionAndButtonStatesAddr ReadPositionAndButtonStatesAddr;
tReadPositionAndButtonStatesAndProbeDiameterAddr ReadPositionAndButtonStatesAndProbeDiameterAddr;
tReadJointAnglesAddr ReadJointAnglesAddr;
tReadJointAngles2Addr ReadJointAngles2Addr;


bool APIBacesArm::CreateBacesInstance()
{
	bacesInst = LoadLibrary(_T("KRKreonArm.dll"));
	if(bacesInst == NULL)
	{
		bacesInst = LoadLibrary(_T("baces.dll"));
	}

	if(bacesInst != NULL)
	{
		OpenCommPortAddr = (tOpenCommPortAddr) GetProcAddress(bacesInst, "AprePorta");
		CloseCommPortAddr = (tCloseCommPortAddr) GetProcAddress(bacesInst, "ChiudePorta");
		ReadParamFileAddr = (tReadParamFileAddr) GetProcAddress(bacesInst, "LeggeParametri");
		ResetEncodersAddr = (tResetEncodersAddr) GetProcAddress(bacesInst, "ResetEncoder");
		ReadPositionAndButtonStatesAddr = (tReadPositionAndButtonStatesAddr) GetProcAddress(bacesInst, "LeggeQuota");
		ReadPositionAndButtonStatesAndProbeDiameterAddr = (tReadPositionAndButtonStatesAndProbeDiameterAddr) GetProcAddress(bacesInst, "LeggeQuota2");
		ReadJointAnglesAddr = (tReadJointAnglesAddr) GetProcAddress(bacesInst, "VedeCinematica");
		ReadJointAngles2Addr = (tReadJointAngles2Addr) GetProcAddress(bacesInst, "VedeCinematica2");

		return true;
	}
	return false;
}

int APIBacesArm::OpenCommPort(CString port)
{
	if(NULL == OpenCommPortAddr)
		return -3;

	LPTSTR lpsz = new TCHAR[5];
	_tcscpy(lpsz, port);
	CT2A lpszA(lpsz);

	int ret = -10;

	if(OpenCommPortAddr)
		ret = OpenCommPortAddr(lpszA);
	else
		ret = -3;

	delete lpsz;
	return ret;
}
int APIBacesArm::CloseCommPort()
{
	if(NULL == CloseCommPortAddr)
		return -5;

	if(CloseCommPortAddr)
		return CloseCommPortAddr();
	else
		return -5;
}

BOOL APIBacesArm::ReleaseBacesInstance()
{
	return FreeLibrary(bacesInst);
}

int APIBacesArm::ReadParamFile(CString path)
{
	if(NULL == ReadParamFileAddr)
		return -3;

	LPTSTR lpsz = new TCHAR[path.GetLength()+1];
	_tcscpy(lpsz, path);
	CT2A lpszA(lpsz);

	int ret = -10;

	if(ReadParamFileAddr)
		ret = ReadParamFileAddr(lpszA);
	else
		ret = -3;

	delete lpsz;
	return ret;
}

int APIBacesArm::ResetBacesEncoders()
{
	if(NULL == ResetEncodersAddr)
		return -5;

	//NRKProgress nrkprog(_T("Axxis Arm Encoder Initialization"), FALSE);
	//nrkprog.SetMsg(_T("Please wait for encoder reset dialog to start..."));

	if(ResetEncodersAddr)
		return ResetEncodersAddr();
	else
		return -5;
}

int APIBacesArm::ReadPositionAndButtonStatesAndProbeDiameter()
{
	if(NULL == ReadPositionAndButtonStatesAndProbeDiameterAddr)
		return -5;

	double quota[4][4];
	int pulsanti = 0;
	double stilo = 0.0;

	int ret = -3;
	if(ReadPositionAndButtonStatesAndProbeDiameterAddr)
		ret = ReadPositionAndButtonStatesAndProbeDiameterAddr(quota, &pulsanti, &stilo);

	if(ret != 0)
	{
		// could do error reporting here based on return value
		return ret;
	}

	//TRACE1("buttons = %d\n", pulsanti);
	//return ret;

	app.saarm.Data.TipDia = stilo/1000.0;

	// Position and Orientation....
	X = quota[0][3]/1000.0;
	Y = quota[1][3]/1000.0;
	Z = quota[2][3]/1000.0;

	II = quota[0][2];
	JJ = quota[1][2];
	KK = quota[2][2];

	CurrentTransform.T[0][0] = quota[0][0];
	CurrentTransform.T[1][0] = quota[1][0];
	CurrentTransform.T[2][0] = quota[2][0];

	CurrentTransform.T[0][1] = quota[0][1];
	CurrentTransform.T[1][1] = quota[1][1];
	CurrentTransform.T[2][1] = quota[2][1];

	CurrentTransform.T[0][2] = quota[0][2];
	CurrentTransform.T[1][2] = quota[1][2];
	CurrentTransform.T[2][2] = quota[2][2];

	CurrentTransform.T[0][3] = X;
	CurrentTransform.T[1][3] = Y;
	CurrentTransform.T[2][3] = Z;

	// Button States...
	if(overrideArmButtons || !app.pA->armButtons.ButtonsEnabled())
		return ret;
	BYTE buttons = (BYTE)pulsanti;

	if(!(buttons&1) && !(buttons&2) && !(buttons&4))  //no buttons
	{  
		buttonstate = 0;
		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
		if( LastButton == 1 && !LongDown)
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			Beep(1000,100);
		}
		if( LastButton == 2 && !LongDown)
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			Beep(800,100);
		}
		if(LastButton == 3 && !LongDown)
		{
			buttonstate = 4;
			buttonstate += BUTTON_SHORT;
			Beep(600,100);
		}

		LastButton = 0;
		LongDown = FALSE;
	}
	if(DOF == 6)
	{
		if(buttons&1)	// front button (blue)
		{  
			TRACE0("Front blue button hit\n");
			buttonstate = 1;
			if( LastButton == 2 || LastButton == 3 )
			{
				buttonstate = 2;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 1;
			}
			if( LastButton != 1 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 1;
			}
			else if( LastButton == 1 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					MessageBeep(-1);
					Beep(1200,200);
					TRACE0("LongDown just set to TRUE, front button.\n");
				}
			}
		}
		else if(buttons&2)	//middle button (yellow)
		{  
			TRACE0("Middle yellow button hit\n");
			buttonstate = 2;
			if( LastButton == 1 || LastButton == 3 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 2;
			}
			if( LastButton != 2 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 2;
			}
			else if( LastButton == 2 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
		else if(buttons&4)	//rear button (green)
		{  
			TRACE0("Rear green button hit\n");
			buttonstate = 4;
			if( LastButton == 1 || LastButton == 2 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 3;
			}
			if( LastButton != 3 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 3;
			}
			else if( LastButton == 3 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
	}
	if(DOF == 7)
	{
		if(buttons&1)	// green
		{  
			TRACE0("Rear green button hit\n");
			buttonstate = 4;
			if( LastButton == 1 || LastButton == 2 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 3;
			}
			if( LastButton != 3 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 3;
			}
			else if( LastButton == 3 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
		else if(buttons&2)	// yellow
		{  
			TRACE0("Middle yellow button hit\n");
			buttonstate = 2;
			if( LastButton == 1 || LastButton == 3 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 2;
			}
			if( LastButton != 2 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 2;
			}
			else if( LastButton == 2 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
	}
	return ret;
}

int APIBacesArm::ReadPositionAndButtonStates()
{
	if(NULL == ReadPositionAndButtonStatesAddr)
		return -5;

	double quota[4][4];
	int pulsanti = 0;

	int ret = -3;
	if(ReadPositionAndButtonStatesAddr)
		ret = ReadPositionAndButtonStatesAddr(quota, &pulsanti);

	if(ret != 0)
	{
		// could do error reporting here based on return value
		return ret;
	}

	// Position and Orientation....
	X = quota[0][3]/1000.0;
	Y = quota[1][3]/1000.0;
	Z = quota[2][3]/1000.0;

	II = quota[0][2];
	JJ = quota[1][2];
	KK = quota[2][2];

	CurrentTransform.T[0][0] = quota[0][0];
	CurrentTransform.T[1][0] = quota[1][0];
	CurrentTransform.T[2][0] = quota[2][0];

	CurrentTransform.T[0][1] = quota[0][1];
	CurrentTransform.T[1][1] = quota[1][1];
	CurrentTransform.T[2][1] = quota[2][1];

	CurrentTransform.T[0][2] = quota[0][2];
	CurrentTransform.T[1][2] = quota[1][2];
	CurrentTransform.T[2][2] = quota[2][2];

	CurrentTransform.T[0][3] = X;
	CurrentTransform.T[1][3] = Y;
	CurrentTransform.T[2][3] = Z;


	// Button States...
	if(overrideArmButtons || !app.pA->armButtons.ButtonsEnabled())
		return ret;
	BYTE buttons = (BYTE)pulsanti;
	if(!(buttons&1) && !(buttons&2) && !(buttons&4))  //no buttons
	{  
		buttonstate = 0;
		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
		if( LastButton == 1 && !LongDown)
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			Beep(1000,100);
		}
		if( LastButton == 2 && !LongDown)
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			Beep(800,100);
		}
		if(LastButton == 3 && !LongDown)
		{
			buttonstate = 4;
			buttonstate += BUTTON_SHORT;
			Beep(600,100);
		}

		LastButton = 0;
		LongDown = FALSE;
	}

	if(DOF == 6)
	{
		if(buttons&1)	// front button (blue)
		{  
			TRACE0("Front button hit\n");
			buttonstate = 1;
			if( LastButton == 2 || LastButton == 3 )
			{
				buttonstate = 2;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 1;
			}
			if( LastButton != 1 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 1;
			}
			else if( LastButton == 1 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					MessageBeep(-1);
					Beep(1200,200);
					TRACE0("LongDown just set to TRUE, front button.\n");
				}
			}
		}
		else if(buttons&2)	//middle button (yellow)
		{  
			TRACE0("Middle button hit\n");
			buttonstate = 2;
			if( LastButton == 1 || LastButton == 3 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 2;
			}
			if( LastButton != 2 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 2;
			}
			else if( LastButton == 2 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
		else if(buttons&4)	//rear button (green)
		{  
			TRACE0("Rear button hit\n");
			buttonstate = 4;
			if( LastButton == 1 || LastButton == 2 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 3;
			}
			if( LastButton != 3 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 3;
			}
			else if( LastButton == 3 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
	}
	if(DOF == 7)
	{
		if(buttons&1)	// front green (blue)
		{  
			TRACE0("Front button hit\n");
			buttonstate = 1;
			if( LastButton == 2 || LastButton == 3 )
			{
				buttonstate = 2;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 1;
			}
			if( LastButton != 1 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 1;
			}
			else if( LastButton == 1 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					MessageBeep(-1);
					Beep(1200,200);
					TRACE0("LongDown just set to TRUE, front button.\n");
				}
			}
		}
		else if(buttons&2)	//back button (yellow)
		{  
			TRACE0("Back button hit\n");
			buttonstate = 2;
			if( LastButton == 1 || LastButton == 3 )
			{
				buttonstate = 1;
				buttonstate += BUTTON_SHORT;
				buttontime = 4294967295;
				LastButton = 2;
			}
			if( LastButton != 2 )
			{
				buttontime = ::GetCurrentTime();
				LastButton = 2;
			}
			else if( LastButton == 2 )
			{
				if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
				{
					buttonstate += BUTTON_LONG;
					buttontime = 4294967295;
					LastButton = 0;
					LongDown = TRUE;
					//	TRACE0("LongDown just set to TRUE, back button.\n");
				}
			}
		}
	}
	return ret;
}
int APIBacesArm::ReadJointAngles()
{
	//if(NULL == ReadJointAngles2Addr)
	if(NULL == ReadJointAngles2Addr)
		return -5;

/*
from Baces documentation...
Declaration:
int VedeCinematica2(int codice, int* nAssi, double* angoli, double* lunghezze, int* modo);

Parameters:
int codice -> value for the angles unit :
0 -> radian
1 -> degree
int * nAssi -> Pointer to the number of joints used by the active configuration.
double* angoli -> Pointer to the angles array.
double* lunghezze -> Pointer to the lenghts array.
int * modo -> Pointer to the modes integer.

Return value:
0 -> ok.
-1 -> the communication isnt opened.
-2 -> the parameters isnt loaded: LeggeParametri() function not called.
*/
	//double param[10]; for old 2-argument version of this function
	int maxAssi = DOF;
	double* param;
	param = new double[maxAssi];
	double lunghezze[4];
	int nAssi;
	int modo;

	int ret = -3;
	if(ReadJointAngles2Addr)
		ret = ReadJointAngles2Addr(1, &nAssi, param, lunghezze, &modo);	// 1st arg: 0 radian, 1 degree
	//if(ReadJointAnglesAddr)
	//	ret = ReadJointAnglesAddr(1, param);	// 1st arg: 0 radian, 1 degree

	if(ret != 0)
	{
		// could do error reporting here based on return value
		return ret;
	}

	J.x[0] = -param[0];
	J.x[1] = -90.0-param[1];
	J.x[2] = param[2];
	J.x[3] = -param[3];
	J.x[4] = param[4];
	J.x[5] = -param[5];

	if(DOF == 7)
		J.x[6] = param[6] + 90.0;

	delete [] param;

	//////////////////////////////////////////////
	// info for model....
	//double baseHeight, segOne, segTwo, probeLen;
	//baseHeight=segOne=segTwo=probeLen = 0.0;
	//baseHeight = param[6];
	//segOne = param[7];
	//segTwo = param[8];
	//probeLen = param[9];
	//TRACE1("baseHeight = %f\n", baseHeight);
	//TRACE1("segOne = %f\n", segOne);
	//TRACE1("segTwo = %f\n", segTwo);
	//TRACE1("probeLen = %f\n", probeLen);
	//////////////////////////////////////////////

	return ret;
}