// ArmBase.cpp: implementation of the ArmBase class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DPCMMDlg.h"
#include "ArmBase.h"
#include "DPointMeas.h"
#include "DStreamPts.h"
#include "DSphereMeas.h"
#include "DCircleMeas.h"
#include "DPlaneMeas.h"
#include "DLineMeas.h"
#include "DFrameMeas.h"
#include "DHoleMeas.h"
#include "DSlotMeas.h"
#include "DPatchMeas.h"
#include "DPreSection.h"
#include "DGuideMeas.h"
#include "DAvgPointsMeas.h"
#include "DSAReq_SinglePtMeas.h"
#include "DOffsetLine.h"
#include "DAutoProxMeas.h"
#include "ModelCorrection/ModelInfo.h"
#include "DGeomTriggerPts.h"
#include "DConeMeas.h"
#include "DSinglePtHole.h"

#include "SA Easy Pad/EasyPadArmInterface.h"
#include "DEasyPadLocPts.h"

#include "DStreamPtsSettings.h"

#include "DSimScannerMeas.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ArmBase::ArmBase()
: scannerCS(*new CriticalSectionWrapper)
, scannerPkt(*new SAIPCloud)
, scanStripePkt(*new SAIPScanStripeCloud)
, saModelInfo(*new ModelInfo(30))
, armButtons(*new ArmButtons)
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;
	StepIndex = 0;
	mainDlgIsInitialized = false;

	LogFileName = _T("PCMM");
	CMMSerialNum.Empty();

	DOF = 6;
	jtlimitbad = false;
	J.Create(DOF, TRUE);		// set default size of Joint GenVec
	lastJ.Create(DOF, TRUE);
	
	II = 0.0;
	JJ = 0.0;
	KK = -1.0;
	X = 1.0;
	Y = 0.0;
	Z = 0.0;
	probeValid = true;

	CurrentTransform.SetRotation(Vector(180.0,0.0,.0));
	CurrentTransform.SetTranslation(Vector(X,Y,Z));

	Temperature = 0.0;

	LastButton = 0;
	LastRecordedPoint = Vector(0.0,0.0,0.0);
	LastPos = Vector(0.0,0.0,0.0);
	CurrentButtons = 0;
	inspectreq = -1;

	simjtangle = 0.0;
	simCloudCtr = 1;
	buttonstate = 0;
	overrideArmButtons = false;

	isMeasuringFrames = FALSE;
	FrameUpdateRequested = FALSE;

	pAutoProxDlg = NULL;

	trackArmActive = false;

	armScannerSwitchOn = false;

	performingStdScannerMeas = true;
}

ArmBase::~ArmBase()
{
	ClearDataArray();
//	ClearGuidedPts();
	delete &scannerCS;
	scannerPkt.ClearCB();
	delete& scannerPkt;
	scanStripePkt.ClearData();
	delete& scanStripePkt;
	delete& saModelInfo;
	delete& armButtons;
}

void ArmBase::CheckPad()
{
	if(!app.controlPad.IsOn())
	{
		return;
	}
	Vector p(X,Y,Z);
	p.Scale(app.saarm.LocalToInches);
	
	app.controlPad.CheckPosition(p, app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0);
}

void ArmBase::UpdateToSpan(bool zeroOffset/*=false*/)
{
	if(!JointsMovedBeyondThreshold())
		return;

	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//if(d.measmode == 13) // line scanning
	//	return;

//	TRACE0("Making Update for Span"));
	SAIPMeasDataCMM* p =  new SAIPMeasDataCMM;
	
	p->Point.x = X;
	p->Point.y = Y;
	p->Point.z = Z;

	if(Y==0.0 && Z==0.0)	// WinRDS 7.1.4 sends a point ( 39.370079, 0.000000, 0.000000 ) at startup
		return;

	p->Point.Scale(app.saarm.LocalToInches);

	//TRACE0("Update to SA sending XYZ...\n");
	//p->Point.Debug();
	//TRACE0("\n");

	p->JointValues.Create(DOF,TRUE);
	//TRACE0("Sending XYZ and jt angles Update to SA\n");
	for( int i=0; i<DOF; i++)
	{
		p->JointValues.x[i] = J[i];
		//TRACE2("j%d=%f ", i, J[i]);
	}
//	TRACE0("\n"));

	//TRACE0("Update to SA, making SAIPUpdate...");
	SAIPUpdate updpkt;
	if(zeroOffset)
	{
		updpkt.TargetOffset = 0.0;
		updpkt.TargetOffset2 = 0.0;
	}
	else
	{
		updpkt.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
		updpkt.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	}

	Transform t(CurrentTransform);
	Vector pos;
	t.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	t.SetTranslation(pos);
	p->probeDir.SetProbingTransform(t);

	updpkt.MeasData.Add(p);

	//TRACE0("sending QS packet...\n");
	app.saarm.QS.SendPacket(updpkt);
	//TRACE0("QS packet sent?\n");

	app.saarm.Blast6dUDPData(t, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(p->Point, app.saarm.units.InchesToUnit());
}

void ArmBase::MeasToSpan(bool zeroOffset, bool sendJointValues/*=true*/)
{
	MeasToSpan(Vector(X,Y,Z), zeroOffset, sendJointValues);	// send the current point formed by global X,Y,Z
}
void ArmBase::MeasToSpan(const Vector& XYZ, bool zeroOffset/*=false*/, bool updateGraphicModel/*=false*/, bool sendJointValues/*=true*/)
{
	if(app.saarm.Data.sendAllPtsAsCloudPts)
	{
		Vector xyz = XYZ;
		xyz.Scale(app.saarm.LocalToInches);
		SAIPCloudBlockPt cbpt;
		cbpt.x = xyz.x;
		cbpt.y = xyz.y;
		cbpt.z = xyz.z;
		AddScannerCloudPt(cbpt);
		CheckAndSendScannerCloud(1);
		LastRecordedPoint = Vector(X,Y,Z);
		return;
	}

	SAIPMeasurement measpak;
	measpak.collection = app.saarm.Data.Collection;
	measpak.Group = app.saarm.Data.Group;
	measpak.Target = app.saarm.Data.Target;
	if(zeroOffset)
	{
		measpak.TargetOffset = 0.0;
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2010.04.16 - no need to zero radial offset!
	}
	else
	{
		measpak.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2001.04.11 INCHES, NOT METERS ANY MORE!!
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	}

//	TRACE("ArmBase::MeasToSpan, measpak.TargetOffset = %f, app.saarm.Data.TipDia = %f, app.saarm.LocalToInches = %f\n"),
//		measpak.TargetOffset, app.saarm.Data.TipDia, app.saarm.LocalToInches);
	
	SAIPMeasDataCMM& p = *new SAIPMeasDataCMM;
	Vector xyz = XYZ;
	xyz.Scale(app.saarm.LocalToInches);
	p.Point = xyz;

	// 2007.12.13 - set the new probing transform
	Transform t(CurrentTransform);
	Vector pos;
	t.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	t.SetTranslation(pos);
	p.probeDir.SetProbingTransform(t);

	if(sendJointValues)
	{
		p.JointValues.Create(DOF,TRUE);
		//TRACE0("Meas: ");
		for( int i=0; i<DOF; i++)
		{
			p.JointValues.x[i] = J[i];
			//TRACE2("j%d=%f ", i, J[i]);
		}
		//TRACE0("\n");
	}

	// 2006.12.05 - if conditions are correct, see if the SA model info should be updated
	// 2006.12.06 - now see CheckAndUpdateModelInfo method below, leaving this temporarily in case needed for testing...
	//if(updateGraphicModel && app.saarm.Data.autoCorrectModel)
	//{
	//	JointOffsetCalItem& jCal = *new JointOffsetCalItem;
	//	jCal.JointValues = p.JointValues;
	//	jCal.Point = p.Point;
	//	if(saModelInfo.CheckUpdate(jCal))
	//		saModelInfo.ModelInfoUpdateToSpan();
	//}

	measpak.MeasData.Add(&p);
	if(!ConnectedToArm)
	{
		CString append;
		append.Format(_T("Simulation:"));
		measpak.Info.Add(append);
	}
	measpak.Info.Append(app.saarm.Data.measCommentArray);
	app.saarm.QS.SendPacket(measpak);
	measpak.ClearMeasData();
	LastRecordedPoint = Vector(X,Y,Z);
	IncrementName(app.saarm.Data.Target, app.saarm.Data.ptnameinc);

	app.saarm.Blast6dUDPData(t, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(p.Point, app.saarm.units.InchesToUnit());
}
// 2004.10.19 -- for e.g. Multi-Plane cross
void ArmBase::MeasToSpan(const Vector& XYZ, const CString& groupSuffix, bool zeroOffset/*=false*/, bool sendJointValues/*=true*/)
{
	if(app.saarm.Data.sendAllPtsAsCloudPts)
	{
		Vector xyz = XYZ;
		xyz.Scale(app.saarm.LocalToInches);
		SAIPCloudBlockPt cbpt;
		cbpt.x = xyz.x;
		cbpt.y = xyz.y;
		cbpt.z = xyz.z;
		AddScannerCloudPt(cbpt);
		CheckAndSendScannerCloud(1);
		LastRecordedPoint = Vector(X,Y,Z);
		return;
	}

	SAIPMeasurement measpak;
	measpak.collection = app.saarm.Data.Collection;
	measpak.Group = app.saarm.Data.Group;
	if(!groupSuffix.IsEmpty())
		measpak.Group += _T(" ") + groupSuffix;
	measpak.Target = app.saarm.Data.Target;
	if(zeroOffset)
	{
		measpak.TargetOffset = 0.0;
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2010.04.16 - no need to zero radial offset!
	}
	else
	{
		measpak.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2001.04.11 INCHES, NOT METERS ANY MORE!!
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	}
	//	TRACE("ArmBase::MeasToSpan, measpak.TargetOffset = %f, app.saarm.Data.TipDia = %f, app.saarm.LocalToInches = %f\n"),
	//		measpak.TargetOffset, app.saarm.Data.TipDia, app.saarm.LocalToInches);

	SAIPMeasDataCMM* p = new SAIPMeasDataCMM;
	Vector xyz = XYZ;
	xyz.Scale(app.saarm.LocalToInches);
	p->Point = xyz;

	// 2007.12.13 - set the new probing transform
	Transform t(CurrentTransform);
	Vector pos;
	t.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	t.SetTranslation(pos);
	p->probeDir.SetProbingTransform(t);

	if(sendJointValues)
	{
		p->JointValues.Create(DOF,TRUE);
		TRACE0("Meas: ");
		for( int i=0; i<DOF; i++)
		{
			p->JointValues.x[i] = J[i];
			TRACE2("j%d=%f ", i, J[i]);
		}
		TRACE0("\n");
	}

	measpak.MeasData.Add(p);

	if(!ConnectedToArm)
	{
		CString append;
		append.Format(_T("Simulation:"));
		measpak.Info.Add(append);
	}
	measpak.Info.Append(app.saarm.Data.measCommentArray);

	app.saarm.QS.SendPacket(measpak);
	measpak.ClearMeasData();
	LastRecordedPoint = Vector(X,Y,Z);
	IncrementName(app.saarm.Data.Target, app.saarm.Data.ptnameinc);

	app.saarm.Blast6dUDPData(t, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(p->Point, app.saarm.units.InchesToUnit());
}
// 2001.03.23 -- for Guided Measurements
void ArmBase::MeasToSpan(const Vector& XYZ, CString& Group, CString& Target, bool zeroOffset/*=false*/, bool sendJointValues/*=true*/)
{
	if(app.saarm.Data.sendAllPtsAsCloudPts)
	{
		Vector xyz = XYZ;
		xyz.Scale(app.saarm.LocalToInches);
		SAIPCloudBlockPt cbpt;
		cbpt.x = xyz.x;
		cbpt.y = xyz.y;
		cbpt.z = xyz.z;
		AddScannerCloudPt(cbpt);
		CheckAndSendScannerCloud(1);
		LastRecordedPoint = Vector(X,Y,Z);
		return;
	}

	SAIPMeasurement measpak;
	measpak.Group = Group;
	measpak.Target = Target;
	if(zeroOffset)
	{
		measpak.TargetOffset = 0.0;
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2010.04.16 - no need to zero radial offset!
	}
	else
	{
		measpak.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2001.04.11 INCHES, NOT METERS ANY MORE!!
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	}
//	TRACE("ArmBase::MeasToSpan, measpak.TargetOffset = %f, app.saarm.Data.TipDia = %f, app.saarm.LocalToInches = %f\n"),
//		measpak.TargetOffset, app.saarm.Data.TipDia, app.saarm.LocalToInches);

	if(!ConnectedToArm)
	{
		CString append;
		append.Format(_T("Simulation:"));
		measpak.Info.Add(append);
	}
	measpak.Info.Append(app.saarm.Data.measCommentArray);
	SAIPMeasDataCMM* p = new SAIPMeasDataCMM;
	Vector xyz = XYZ;
	xyz.Scale(app.saarm.LocalToInches);
	p->Point = xyz;

	// 2007.12.13 - set the new probing transform
	Transform t(CurrentTransform);
	Vector pos;
	t.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	t.SetTranslation(pos);
	p->probeDir.SetProbingTransform(t);

	if(sendJointValues)
	{
		p->JointValues.Create(DOF,TRUE);
		TRACE0("Meas: ");
		for( int i=0; i<DOF; i++)
		{
			p->JointValues.x[i] = J[i];
			TRACE2("j%d=%f ", i, J[i]);
		}
		TRACE0("\n");
	}

	measpak.MeasData.Add(p);

	app.saarm.QS.SendPacket(measpak);
	measpak.ClearMeasData();
	LastRecordedPoint = Vector(X,Y,Z);

	app.saarm.Blast6dUDPData(t, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(p->Point, app.saarm.units.InchesToUnit());
}
void ArmBase::MeasToSpan(ArmPointArray* va, CString FstTarg, bool zeroOffset/*=false*/)
{
	if (va->P.GetSize() <= 0)
		return;

	if(app.saarm.Data.sendAllPtsAsCloudPts)
	{
		SAIPCloudBlockPt cbpt;
		ArmPointArray pts;
		ScaleArmPtsArrayToInches(*va, &pts);
		for( int i=0; i<pts.P.GetSize(); i++ )
		{
			cbpt.x = pts.P[i]->x;
			cbpt.y = pts.P[i]->y;
			cbpt.z = pts.P[i]->z;
			AddScannerCloudPt(cbpt);
		}
		CheckAndSendScannerCloud(1);
		LastRecordedPoint = Vector(X,Y,Z);
		return;
	}

//	CString Info;
	CString append;
	SAIPMeasurement measpak;

	if(!ConnectedToArm)
	{
		append.Format(_T("Simulation:"));
		measpak.Info.Add(append);
	}
	measpak.Info.Append(app.saarm.Data.measCommentArray);
	measpak.collection = app.saarm.Data.Collection;
	measpak.Group = app.saarm.Data.Group;
	measpak.Target = FstTarg;
	if(zeroOffset)
	{
		measpak.TargetOffset = 0.0;
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2010.04.16 - no need to zero radial offset!
	}
	else
	{
		measpak.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;	// 2001.04.11 INCHES, NOT METERS ANY MORE!!
		measpak.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	}
//	TRACE("ArmBase::MeasToSpan, measpak.TargetOffset = %f, app.saarm.Data.TipDia = %f, app.saarm.LocalToInches = %f\n"),
//		measpak.TargetOffset, app.saarm.Data.TipDia, app.saarm.LocalToInches);

	ArmPointArray pts;
	ScaleArmPtsArrayToInches(*va, &pts);

	for( int i=0; i<pts.P.GetSize(); i++ )
	{
		SAIPMeasDataCMM* p = new SAIPMeasDataCMM;
		p->Point = pts.P[i]->GetPoint();
		p->JointValues = pts.P[i]->Joints;
		p->probeDir.SetProbingDirection(pts.P[i]->Normal);
		measpak.MeasData.Add(p);
	}

	TRACE1("sending %d pts...\n", pts.P.GetSize());
	app.saarm.QS.SendPacket(measpak);
	measpak.ClearMeasData();
	LastRecordedPoint = Vector(X,Y,Z);

	app.saarm.Blast3dUDPData(*pts.P[0], app.saarm.units.InchesToUnit());
}

void ArmBase::FrameToSpan(const Transform& frame, const CString& name, BOOL UpdateOnly/*=FALSE*/) 
{
	SAIPGeomFrame QSF;
	QSF.UpdateOnly = UpdateOnly;
	QSF.Name = name;
	QSF.CollectionName = app.saarm.Data.Collection;

	//	CurrentTransform.Invert(&QSF.ObjectToInst);
	QSF.ObjectToInst = frame;

	QSF.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	QSF.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;

	Vector pos;
	QSF.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	QSF.ObjectToInst.SetTranslation(pos);
	CString note;
	note.Format(_T("Frame from %s interface"), app.saarm.CMMString);
	QSF.Notes.Add(note);
	TRACE0("Sending SAIPGeomFrame packet to SA\n");
	app.saarm.QS.SendPacket(QSF);

	app.saarm.Blast6dUDPData(QSF.ObjectToInst, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(pos, app.saarm.units.InchesToUnit());
}

void ArmBase::FrameToSpan(CString& Name, BOOL UpdateOnly) 
{
	SAIPGeomFrame QSF;
	QSF.UpdateOnly = UpdateOnly;
	QSF.Name = Name;
	QSF.CollectionName = app.saarm.Data.Collection;

//	CurrentTransform.Invert(&QSF.ObjectToInst);
	QSF.ObjectToInst = CurrentTransform;

	QSF.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	QSF.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;

	Vector pos;
	QSF.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	QSF.ObjectToInst.SetTranslation(pos);
	CString note;
	note.Format(_T("Frame from %s interface"), app.saarm.CMMString);
	QSF.Notes.Add(note);
	TRACE0("Sending SAIPGeomFrame packet to SA\n");
	app.saarm.QS.SendPacket(QSF);

	app.saarm.Blast6dUDPData(QSF.ObjectToInst, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(pos, app.saarm.units.InchesToUnit());
}

void ArmBase::FrameUpdateToSpan() 
{
	CurrentTransform.Debug();
	
	SAIPGeomFrame QSF;
	QSF.UpdateOnly = TRUE;
	QSF.Name.Empty();
	QSF.CollectionName = app.saarm.Data.Collection;

//	CurrentTransform.Invert(&QSF.ObjectToInst);
	QSF.ObjectToInst = CurrentTransform;

	QSF.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	QSF.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	
	Vector pos;
	QSF.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	QSF.ObjectToInst.SetTranslation(pos);
/*	if(CurrentButtons&LEFTBUTTON)
		QSF.Status = STAT_START;
	if(CurrentButtons&MIDBUTTON)
		QSF.Status = STAT_END;
*/
	//	send
	TRACE0("Sending SAIPGeomFrame update to SA\n");
	app.saarm.QS.SendPacket(QSF);
//	AfxMessageBox("Sending a frame update to SA."));

	app.saarm.Blast6dUDPData(QSF.ObjectToInst, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(pos, app.saarm.units.InchesToUnit());
}

void ArmBase::FrameandJointUpdateToSpan()
{
	// Frame
	CurrentTransform.Debug();
	
	SAIPGeomFrame QSF;
	QSF.UpdateOnly = TRUE;
	QSF.Name.Empty();
	QSF.CollectionName = app.saarm.Data.Collection;

	QSF.ObjectToInst = CurrentTransform;

	QSF.TargetOffset = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	QSF.TargetOffset2 = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	
	Vector pos;
	QSF.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	QSF.ObjectToInst.SetTranslation(pos);

	// Joints (and XYZ)
	SAIPMeasDataCMM p;
	
	p.Point.x = X;
	p.Point.y = Y;
	p.Point.z = Z;
	p.Point.Scale(app.saarm.LocalToInches);

	p.JointValues.Create(DOF,TRUE);
	TRACE0("Sending XYZ and jt angles Update to SA\n");
	for( int i=0; i<DOF; i++)
	{
		p.JointValues.x[i] = J[i];
//		TRACE2("j%d=%f "), i, J[i]);
	}


	CObArray pak;
	pak.Add(&p);
	pak.Add(&QSF);

	app.saarm.QS.SendPackets(pak);

	app.saarm.Blast6dUDPData(QSF.ObjectToInst, app.saarm.units.InchesToUnit());
	app.saarm.Blast3dUDPData(p.Point, app.saarm.units.InchesToUnit());
}
//////////////////////////////////////////////////////////
// Group and Target name stuff
BOOL ArmBase::FixBlankNames()
{	
	BOOL hit = FALSE;
	if (app.saarm.Data.Group.IsEmpty())
	{
		app.saarm.Data.Group = "Main";
		hit = TRUE;
	}
	if (app.saarm.Data.Target.IsEmpty())
	{
		app.saarm.Data.Target = _T("1");
		hit = TRUE;
	}
	return hit;
}
void ArmBase::IncrementName(CString& Target, int inc)
{
	//search to find first non-number or '.' character from end
	int FirstDigit = 0;
	int CharCount = Target.GetLength();			 
	for(int i=0;i<CharCount;i++)
	{
		if	(((!isdigit(Target[CharCount-i-1]))&&
				(Target[CharCount-i-1]!='-'))||
				(Target[CharCount-i-1]=='.'))
		{
			FirstDigit = CharCount-i;				   
			break;
		}		  
	}
	//disassemble the targetname
	CString TName =_T("");
	CString TNumber =_T("");
	for(int i=0;i<CharCount;i++)
	{
		if(i<FirstDigit)
			TName += Target[i];
		else
			TNumber += Target[i];
	}
	int PtNum = _ttoi(TNumber);
	TNumber.ReleaseBuffer();

	PtNum += inc;

	if(PtNum < 0)
		return;

	// reassemble the targetname
	TNumber.Format(_T("%d"), PtNum);
	if(PtNum == 0)
		TNumber = "";

	Target = TName;
	Target += TNumber;
	SetDisplayData();
}
// edit these 2 fcn's as appropriate for the user's main dlg, view, whatever
void ArmBase::SetDisplayData()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.GetDataParams();
	d.PDASendGT();	// only does something if PDA is connected!
}
void ArmBase::GetDisplayData()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.SetDataParams();
}
bool ArmBase::SetMeasMode(int measmode)
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(measmode < 0 || measmode > UPPERMODE)
		return false;
	if(measmode != 13 && (IsFaroUSB() || IsKreonScannerArm()))
	{
		StopScannerMeas();
		Delay(0.1, TRUE);
	}
	d.measmode = measmode;
	d.HighlightButton();
	return true;
}
bool ArmBase::SetMeasModeByName(const CString& name, CString& pErrorString)
{
	if((IsFaroUSB() || IsKreonScannerArm()) && !StringHelper::NameCompare(name, _T("Scanner")))
	{
		StopScannerMeas();
		Delay(0.1, TRUE);
	}
	int mode = -1;
	if(StringHelper::NameCompare(name, _T("Discrete")))
		mode = 0;
	else if(StringHelper::NameCompare(name, _T("Stream")))
		mode = 1;
	else if(StringHelper::NameCompare(name, _T("Patch")))
		mode = 2;
	else if(StringHelper::NameCompare(name, _T("Pin")))
		mode = 3;
	else if(StringHelper::NameCompare(name, _T("Hole")))
		mode = 4;
	else if(StringHelper::NameCompare(name, _T("Slot")))
		mode = 5;
	else if(StringHelper::NameCompare(name, _T("Line")))
		mode = 6;
	else if(StringHelper::NameCompare(name, _T("Circle")))
		mode = 7;
	else if(StringHelper::NameCompare(name, _T("Plane")))
		mode = 8;
	else if(StringHelper::NameCompare(name, _T("Sphere")))
		mode = 9;
	else if(StringHelper::NameCompare(name, _T("Section")))
		mode = 10;
	else if(StringHelper::NameCompare(name, _T("Frame")))
		mode = 11;
	else if(StringHelper::NameCompare(name, _T("Batch")))
		mode = 12;
	else if(StringHelper::NameCompare(name, _T("Scanner")))
		mode = 13;
	else if(StringHelper::NameCompare(name, _T("Average")))
		mode = 14;
	else if(StringHelper::NameCompare(name, _T("Geom Trigger")))
		mode = 15;
	else if(StringHelper::NameCompare(name, _T("Cone")))
		mode = 16;
	else if(StringHelper::NameCompare(name, _T("Single Pt. Hole")))
		mode = 17;

	if(mode < 0 || mode > UPPERMODE)
	{
		pErrorString.Format(_T("Could not find valid mode to match requested '%s'."), name);
		return false;
	}
	return SetMeasMode(mode);
}
CString ArmBase::CurrentMeasModeName()
{
	CString mode;
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	switch (d.measmode)
	{
	case 0:
		mode.Format(_T("Discrete"));
		break;
	case 1:
		mode.Format(_T("Stream"));
		break;
	case 2:
		mode.Format(_T("Patch"));
		break;
	case 3:
		mode.Format(_T("Pin"));
		break;
	case 4:
		mode.Format(_T("Hole"));
		break;
	case 5:
		mode.Format(_T("Slot"));
		break;
	case 6:
		mode.Format(_T("Line"));
		break;
	case 7:
		mode.Format(_T("Circle"));
		break;
	case 8:
		mode.Format(_T("Plane"));
		break;
	case 9:
		mode.Format(_T("Sphere"));
		break;
	case 10:
		mode.Format(_T("Section"));
		break;
	case 11:
		mode.Format(_T("Frame"));
		break;
	case 12:
		mode.Format(_T("Batch"));
		break;
	case 13:
		mode.Format(_T("Scanner"));
		break;
	case 14:
		mode.Format(_T("Average"));
		break;
	case 15:
		mode.Format(_T("Geom Trigger"));
		break;
	case 16:
		mode.Format(_T("Cone"));
		break;
	case 17:
		mode.Format(_T("Single Pt. Hole"));
		break;
	default:
		break;
	}
	return mode;
}
//////////////////////////////////////////////////////////////////////////////
// the arm
BOOL ArmBase::IsRomer()
{
	return FALSE;
}
BOOL ArmBase::IsFaro()
{
	return FALSE;
}
BOOL ArmBase::IsFaroUSB()
{
	return FALSE;
}
BOOL ArmBase::IsFrenchRomer()
{
	return FALSE;
}
BOOL ArmBase::IsSNL()
{
	return FALSE;
}
BOOL ArmBase::IsMetrisGarda()
{
	return FALSE;
}
BOOL ArmBase::IsAPIBacesAxxis()
{
	return FALSE;
}
BOOL ArmBase::IsKreonScannerArm()
{
	return FALSE;
}
BOOL ArmBase::IsHexagonRDS()
{
	return FALSE;
}

BOOL ArmBase::GetAllData()
{
	//TRACE0("Base class GetAllData()\n"));

	if(app.StopNow)
		return FALSE;

	GetPos();

	if(app.saarm.arm_type == SAArm::arm_Axxis_KreonScanner)
		return TRUE;

	GetJoints();
	GetButtonStates();
	GetTemp();
	return TRUE;
}

BOOL ArmBase::GetPos()
{
	//TRACE0("Base class GetPos()\n"));
	return TRUE;
}

BOOL ArmBase::GetJoints(bool simulate/*=true*/)
{
	//TRACE1("Base class GetJoints(), simjtangle=%f\n"), simjtangle);

	if(!simulate)
	{
		return FALSE;
	}

	simjtangle += 0.1;
	int InstrumentModel = app.saarm.InstrumentModel;

	if( InstrumentModel==13||InstrumentModel==14 ) // Romer 2500,2200
	{
		J.x[0] = simjtangle+90.0;
		J.x[1] = -simjtangle-90.0;
		J.x[2] = simjtangle;
		J.x[3] = simjtangle+270.0;
		J.x[4] = simjtangle;
		J.x[5] = simjtangle;
	}
	else if( InstrumentModel==10||InstrumentModel==25||InstrumentModel==26||
		InstrumentModel==27||InstrumentModel==34||InstrumentModel==35 )	// Faro S and G 08's and 12's
	{
		J.x[0] = simjtangle-90.0;
		J.x[1] = simjtangle-90.0;
		J.x[2] = simjtangle;
		J.x[3] = simjtangle-90.0;
		J.x[4] = -simjtangle;
		J.x[5] = -simjtangle;
	}
	else if( InstrumentModel==5 ) // old Faro
	{
		J.x[0] = simjtangle-90.0;
		J.x[1] = simjtangle-90.0;
		J.x[2] = simjtangle-90.0;
		J.x[3] = simjtangle;
		J.x[4] = simjtangle;
		J.x[5] = simjtangle+90.0;
	}
	else if( InstrumentModel==38 )	// Sandia Arm
	{
		J.x[0] = simjtangle;
		J.x[1] = simjtangle;
		J.x[2] = simjtangle+15.0;
	}
	else if( InstrumentModel>=47 && InstrumentModel <=56 )	// Faro USB
	{
		J.x[0] = simjtangle;
		J.x[1] = simjtangle+90.0;
		J.x[2] = simjtangle;
		J.x[3] = simjtangle+90.0;
		J.x[4] = -simjtangle;
		J.x[5] = simjtangle;
	}
	else if( InstrumentModel == 86 )  // Metris MCA (Garda Arm)
	{
		J.x[0] = 180.0 - simjtangle;
		J.x[1] = 90.0 - simjtangle;
		J.x[2] = simjtangle * -1.0;
		J.x[3] = 90.0 - simjtangle;
		J.x[4] = simjtangle;
		J.x[5] = simjtangle;
	}
	else if( (InstrumentModel>=79 && InstrumentModel<=84) || (InstrumentModel>=92 && InstrumentModel<=103) )	// Infinite (5000 series), and Infinite 2 (5100 series) arms
	{
		J.x[0] = simjtangle+90.0;
		J.x[1] = simjtangle+90.0;
		J.x[2] = simjtangle;
		J.x[3] = simjtangle+90.0;
		J.x[4] = -simjtangle;
		J.x[5] = -simjtangle;
	}
	else
	{
		J.x[0] = simjtangle+90.0;
		J.x[1] = simjtangle+90.0;
		J.x[2] = simjtangle+180.0;
		J.x[3] = simjtangle+90.0;
		J.x[4] = -simjtangle;
		J.x[5] = -simjtangle;
	}
	return TRUE;
}

int ArmBase::GetButtonStates()
{
	//TRACE0("Base class GetButtonStates()\n"));
	return 1;
}
BOOL ArmBase::CheckButton(int but, int* duration)
{
	// pure virtual
	TRACE("Base class IsButtonDown(int but)\n");
	return FALSE;
}
void ArmBase::ClearButtonState()
{
	buttonstate = 0;
}

BOOL ArmBase::GetTemp()
{
	//TRACE0("Base class GetTemp()\n"));
	return TRUE;
}

int ArmBase::GetProbeNumber()
{
	//TRACE0("Base class GetProbeNumber()\n"));
	return 0;
}

int ArmBase::GetButtonCount()
{
	//TRACE0("Base class GetButtonCount()\n"));
	return 2;
}

void ArmBase::SetProbeNumber(int ProbeNum)
{
	TRACE0("Base class SetProbeNumber(int)\n");
}

BOOL ArmBase::ValidateCurrentProbe()
{
	TRACE0("Base class ValidateCurrentProbe()\n");
	probeValid = true;
	return TRUE;
}

void ArmBase::SetProbeAutoDetect(BOOL ad)
{
	TRACE0("Base class SetProbeAutoDetect(BOOL)\n");
}

BOOL ArmBase::SetVolumeLevel( int VolumeLevel )
{
	app.saarm.Data.SpkrVolume = VolumeLevel;
	return TRUE;
}
void ArmBase::OutputSpeakerSound( int frequency, int duration )
{
	TRACE("Base class OutputSpeakerSound(int, int)\n");
	Beep(frequency, duration);
}
void ArmBase::ArmBeep(int sound)
{
}

/////////////////////////////////////////////////////////////////////////////
// Data Handling

void ArmBase::ClearDataArray()
{
	TRACE("Cleaaa!!\n");
	
	if( DataArray.GetSize() > 0)
	{
		for(int i=0; i<DataArray.GetSize(); i++)
		{
			DataArray[i]->Clear();
			delete DataArray[i];
		}
		DataArray.RemoveAll();
		StepIndex = 0;
	}

	if( BPoints.GetSize() > 0)
	{
		for(int i=0;i<BPoints.GetSize();i++)
			delete BPoints[i];
		BPoints.RemoveAll();
	}
}
void ArmBase::ClearVectorArray()
{
	if( DataArray[StepIndex]->P.GetSize() > 0 )
	{
		for( int i=0; i<DataArray[StepIndex]->P.GetSize(); i++ )
			DataArray[StepIndex]->Clear();
		DataArray.RemoveAt(StepIndex);
	}
}
void ArmBase::ClearData()
{
	if( DataArray.GetSize() > 0)
	{
		for(int i=0; i<DataArray.GetSize(); i++)
		{
			DataArray[i]->Clear();
		}
	}

	if( BPoints.GetSize() > 0)
	{
		for(int i=0;i<BPoints.GetSize();i++)
			delete BPoints[i];
		BPoints.RemoveAll();
	}
}
/*
void ArmBase::ClearGuidedPts()
{
	if( GPoints.GetSize() > 0)
	{
		for(int i=0;i<GPoints.GetSize();i++)
			delete GPoints[i];
		GPoints.RemoveAll();
	}
}
*/
INT_PTR ArmBase::AddPoint()
{
	ArmPoint ap(Vector(X,Y,Z), Vector(II,JJ,KK), J);
	ap.Scale(app.saarm.GetInstrumentScaleFactor());
	ap.Debug();
	return DataArray[StepIndex]->Add(ap);
}
INT_PTR ArmBase::AddPoint(const Vector& pos, const Vector& norm, const GenVec& jts)
{
	ArmPoint ap(pos, norm, jts);
	ap.Scale(app.saarm.GetInstrumentScaleFactor());
	return DataArray[StepIndex]->Add(ap);
}
INT_PTR ArmBase::RemovePoint()
{
	if( DataArray[StepIndex]->P.GetSize() > 0 )
	{
		delete DataArray[StepIndex]->P.GetAt(DataArray[StepIndex]->P.GetUpperBound());
		DataArray[StepIndex]->P.RemoveAt(DataArray[StepIndex]->P.GetUpperBound());
	}

	return DataArray[StepIndex]->P.GetUpperBound();
}
void ArmBase::ScaleDataArrays(double scaleFactor)
{
	if( DataArray.GetSize() > 0)
	{
		for(int i=0; i<DataArray.GetSize(); i++)
		{
			DataArray[i]->Scale(scaleFactor);
		}
	}
}
bool ArmBase::GetAvgPtAndRMSFromDataArray(int array, Vector& avg_xyz, double& rmserr)	// for discrete pts.
{
	if( DataArray.GetSize() <= array )
		return false;
	if( DataArray[array]->P.GetSize() <= 0 )
		return false;
	if( DataArray[array]->P.GetSize() == 1)
	{
		avg_xyz = *DataArray[array]->P[0];
		rmserr = 0.0;
		return true;
	}
	avg_xyz = DataArray[array]->Average();
	Vector stddev;
	if(!DataArray[array]->StandardDeviationMeanZero(&stddev))
		rmserr = -1;
	else
		rmserr = stddev.Length();

	return true;
}
/////////////////////////////////////////////////////////////////////////////
// Measurement Functions

BOOL ArmBase::DoEasyPadLocMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	TRACE0("MEASURE - DoPointMeas\n");
	TRACE0("call ValidateCurrentProbe?...\n");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		TRACE0("...yes, calling ValidateCurrentProbe\n");
		if(!app.pA->ValidateCurrentProbe())
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
	}
	d.StopQueryTimer();

	MessageBeep(MB_ICONINFORMATION);

	Delay(0.05,TRUE);
	Sleep(0);

	DEasyPadLocPts dlg;
	app.controlPad.SwitchOff();
	app.controlPad.BeginCal();
	
	BOOL ret = TRUE;
	if( dlg.DoModal() != IDOK )
	{
		app.controlPad.SwitchOff();
		ret = FALSE;
	}
	else
	{
		app.controlPad.SwitchOn();
	}
	ClearDataArray();

	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoPointMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	TRACE0("MEASURE - DoPointMeas\n");
	TRACE0("call ValidateCurrentProbe?...\n");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		TRACE0("...yes, calling ValidateCurrentProbe\n");
		if(!app.pA->ValidateCurrentProbe())
		{
			d.StopQueryTimer();
			Delay(0.2,TRUE);
			Sleep(200);
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	TRACE2("called Onmeasinit with: coll'%s', g'%s', ", app.saarm.Data.Collection, app.saarm.Data.Group);
	TRACE2(" targ'%s', mode'%s'\n", app.saarm.Data.Target, app.pA->CurrentMeasModeName());

	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DPointMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray[0]->GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoSAIPPointMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	TRACE0("MEASURE - DoSAIPPointMeas\n");

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		TRACE0("...yes, calling ValidateCurrentProbe\n");
		if(!app.pA->ValidateCurrentProbe())
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.

	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DSAReq_SinglePtMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
		ret = TRUE;
	if( DataArray[0]->GetSize() <= 0 )
	{
		d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
		return FALSE;
	}
	ScaleDataArrays(1.0/app.saarm.GetInstrumentScaleFactor());
	MeasToSpan(*DataArray[0]->P[0]);
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoAvgPtsMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DAvgPointsMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoGeomTriggerMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if(!app.saarm.geomTriggerWrap.HasTriggersDefined())
	{
		CString m;
		m.Format(_T("There are no Triggers defined.\nAdd triggers by right-clicking (or in Fit/Meas Options) and try again."));
		AfxMessageBox(m, MB_ICONEXCLAMATION);
		return FALSE;
	}
	else
	{
		if(!app.saarm.geomTriggerWrap.EnsureValidity_InstScaleFactorHasNotChanged())
			return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	double measTimerPeriod = app.saarm.Data.meastimerperiodsec;
	app.saarm.Data.meastimerperiodsec = 0.02;

	DGeomTriggerPts dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		ret = TRUE;
	}
	ClearDataArray();

	app.saarm.Data.meastimerperiodsec = measTimerPeriod;

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoStreamPtsMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DStreamPts dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray.GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoSphereMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DSphereMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoCircleMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DCircleMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoFrameMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE0("DoFrameMeas\n");
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DFrameMeas dlg;
	app.measDlg = &dlg;
	isMeasuringFrames = TRUE;
	BOOL ret=FALSE;
	if( dlg.DoModal() == IDOK )
		ret = TRUE;
	isMeasuringFrames = FALSE;
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoPlaneMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DPlaneMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = FALSE;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}	
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoLineMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	if(app.saarm.Data.lineMeasType == 0)
		return DoOffsetLineMeas();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DLineMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}	
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoOffsetLineMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DOffsetLine dlg;
	app.measDlg = &dlg;
	dlg.title.Format(_T("Offset Line Measurement"));
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray.GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}

		app.saarm.Data.measCommentArray.RemoveAll();
		app.saarm.Data.measCommentArray.Append(dlg.notes);

		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		if(!app.saarm.Data.geomName.IsEmpty())
			IncrementName(app.saarm.Data.geomName);
		ret = TRUE;
	}	
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoPipeProbeMeas()
{
	TRACE0("ArmBase DoPipeProbeMeas called...\n");
	return FALSE;
}
BOOL ArmBase::DoSectionMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DPreSection dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray.GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}

		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		ret = TRUE;
	}	
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoTheoMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DPreSection dlg;
	dlg.TheoPt = TRUE;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray.GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}

		if(app.saarm.Data.incrementgroup)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}
		if(!app.saarm.Data.geomName.IsEmpty())
			IncrementName(app.saarm.Data.geomName);
		ret = TRUE;
	}	
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

// 2001.03.23 -- fcn added
BOOL ArmBase::DoGuidedMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	// 2004.08.25 - now this starts the guide mode prompt in sa...
	SAIPMeasGroup pkt;
	pkt.promptForPoints = true;
	return app.saarm.QS.SendPacket(pkt);
}
BOOL ArmBase::MeasureGuidePts()
{
	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
		{
			AfxMessageBox(_T("Could not validate probe!"), MB_ICONEXCLAMATION);
			app.saarm.ProcessCompleted(MPResult::DoneFatalError);
			return FALSE;
		}
	}
	if( app.pA->GuidedMeasGroup.Points.GetSize() <= 0 )
	{
		AfxMessageBox(_T("No points in batch!"), MB_ICONEXCLAMATION);
		app.saarm.ProcessCompleted(MPResult::DoneFatalError);
		return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	double measTimerPeriodSec = app.saarm.Data.meastimerperiodsec;
	app.saarm.Data.meastimerperiodsec = 0.1;
	
	DGuideMeas dlg;
	app.measDlg = &dlg;
	//app.saarm.SetScreamerParams();
	//app.saarm.StartScreaming();
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
		ret = TRUE;
	IncrementName(GuidedMeasGroup.GroupSuffix);

	//app.saarm.StopScreaming();

	// stop the SA ServoView mode...
	SAIPServoView pkt;
	pkt.cancelMode = true;
	app.saarm.QS.SendPacket(pkt);

	app.saarm.ProcessCompleted(MPResult::DoneSuccess);

	app.saarm.Data.meastimerperiodsec = measTimerPeriodSec;

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::MeasureAutoProximityPoints(SAIPAutoMeasProximity &p)
{
	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
		{
			AfxMessageBox(_T("Could not validate probe!"), MB_ICONEXCLAMATION);
			app.saarm.ProcessCompleted(MPResult::DoneFatalError);
			return FALSE;
		}
	}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(p.processDONE)
	{
		app.saarm.StopScreaming();
		if(pAutoProxDlg)	// 2010.09.16 - we get 2 OnSAIPAutoMeasProximity's with processDONE=1 from SA when user ends this mode in SA
		{
			pAutoProxDlg->EndCancel();
			pAutoProxDlg = NULL;
		}

		if(app.saarm.units.toolBarModeOn)
		{
			CString err;
			SetMeasModeByName(app.saarm.lastToolBarMeasModeName, err);
			d.DoMeasurement();
		}
		else
		{
			d.HighlightButton();
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
		}

		return TRUE;
	}
	if(pAutoProxDlg)	// 2010.09.16 - we get 2 OnSAIPAutoMeasProximity's from SA when user starts this mode in SA
	{
		UpdateAutoProxSamp(p);
		double proxTol, axisTol, rampZone;
		p.GetTolerances(proxTol, axisTol, rampZone);
		app.saarm.Data.GoodProximity = proxTol * app.saarm.InchesToLocal;
		app.saarm.Data.proxWorkingField = rampZone * app.saarm.InchesToLocal;
		app.saarm.SetScreamerParams(proxTol * app.saarm.InchesToLocal, rampZone * app.saarm.InchesToLocal);

		return TRUE;
	}

	pAutoProxDlg = new DAutoProxMeas;
	app.measDlg = pAutoProxDlg;
	UpdateAutoProxSamp(p);
	double proxTol, axisTol, rampZone;
	p.GetTolerances(proxTol, axisTol, rampZone);
	app.saarm.Data.GoodProximity = proxTol * app.saarm.InchesToLocal;
	app.saarm.Data.proxWorkingField = rampZone * app.saarm.InchesToLocal;
	app.saarm.SetScreamerParams(proxTol * app.saarm.InchesToLocal, rampZone * app.saarm.InchesToLocal);
	app.saarm.StartScreaming();

	if(!ConnectedToArm)
	{
		VectorArray testDat;
		for (DWORD_PTR i=0;i<p.GetElementCount();i++)
		{
			CString name;
			Vector ptInInstInches;
			bool measured=false;
			double error = 0;
			p.GetElement(i,name,ptInInstInches, measured, error);
			Vector ptInInstMeters(ptInInstInches);
			ptInInstMeters.Scale(app.saarm.InchesToLocal);
			ptInInstMeters.Debug();
			testDat.Add(ptInInstMeters);
		}
		pAutoProxDlg->SetSimulationDataArray(testDat);
	}

	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	BOOL ret = FALSE;
	if( pAutoProxDlg->DoModal() == IDOK )
		ret = TRUE;
	app.saarm.StopScreaming();
	pAutoProxDlg = NULL;

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);

	return ret;
}
void ArmBase::UpdateAutoProxSamp(SAIPAutoMeasProximity &p)
{
	if(app.saarm.Data.useScannerForAutoProxMeas)
	{
		samp = p;
	}
	else if(pAutoProxDlg != NULL)
	{
		pAutoProxDlg->samp.Clear();
		pAutoProxDlg->samp = p;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
// 2000.01.09 not using these any more - they were for holes, slots, etc.
BOOL ArmBase::GetCircle(CString title, int* samples, double* radius, double* RMSErr, double* MaxErr)
{
	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	DCircleMeas dlg;
	app.measDlg = &dlg;
	dlg.title.Format(_T("%s"), title);
	dlg.ShowGpTarg = FALSE;
	app.saarm.Data.inoutface = 0;	// for holes and slots
	BOOL ret = FALSE;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	if( dlg.DoModal() == IDOK )
	{
		if( DataArray[StepIndex]->P.GetSize() <= 0 )
			return FALSE;

		*samples = dlg.ctr + 1;
		*radius = dlg.rad;
		*RMSErr = dlg.rmserr;
		*MaxErr = dlg.maxerr;
	
		ret = TRUE;
	}

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::GetPlane(CString title, int* samples, double* RMSErr, double* MaxErr)
{
	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DPlaneMeas dlg;
	app.measDlg = &dlg;
	dlg.title.Format(_T("%s"), title);
	dlg.ShowGpTarg = FALSE;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		if( DataArray[StepIndex]->P.GetSize() <= 0 )
		{
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}

		*samples = dlg.ctr + 1;
		*RMSErr = dlg.rmserr;
		*MaxErr = dlg.maxerr;

		ret = TRUE;
	}

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
/////////////////////////////////////////////////////////////////////////////////////////
BOOL ArmBase::DoHoleMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoHoleMeas!!\n");
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DHoleMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = FALSE;
	dlg.Pin = FALSE;
	dlg.title.Format(_T("Hole Measurement"));
	BOOL ret = FALSE;

	dlg.DoModal();
	ret = TRUE;
//	if( dlg.DoModal() == IDOK )
//	{
//		app.saarm.Data.Target.Format(_T("HoleCenter"));
//		MeasToSpan(dlg.center, true, false, false);
//
//		if( app.saarm.Data.SendGeom )
//			app.saarm.SockGeom.SendCircle(dlg.t, dlg.rad, dlg.name, dlg.notes);
//
//		if(app.saarm.Data.FirePointsToo)
//		{
//			MeasToSpan(DataArray[0], Tr("PlanePoint"));
//			MeasToSpan(DataArray[1], Tr("CirclePoint"));
//		}
//
//		if(app.saarm.Data.incrementgroup)
//		{
//			app.saarm.Data.Target.Format(_T("P1"));
//			IncrementName(app.saarm.Data.Group, 1);
//		}
//		SetDisplayData();
//
////		for(i=0; i<DataArray.GetSize(); i++)
////			for( int j=0; j<DataArray[i]->P.GetSize(); j++ )
////				TRACE("Array %d, pt %d:\n X=%f, Y=%f, Z=%f\n"), i, j, DataArray[i]->P[j]->x, DataArray[i]->P[j]->y, DataArray[i]->P[j]->z);
//
//		ret = TRUE;
//	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoPinMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoPinMeas!!\n");
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DHoleMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = FALSE;
	dlg.Pin = TRUE;
	dlg.title.Format(_T("Pin Measurement"));
	BOOL ret = FALSE;

	dlg.DoModal();
	ret = TRUE;
//	if( dlg.DoModal() == IDOK )
//	{
//		app.saarm.Data.Target.Format(_T("PinCenter"));
//		MeasToSpan(dlg.center);
//		
//		if( app.saarm.Data.SendGeom )
//			app.saarm.SockGeom.SendCircle(dlg.t, dlg.rad, dlg.name, dlg.notes);
//
//		if(app.saarm.Data.FirePointsToo)
//		{
//			MeasToSpan(DataArray[0], Tr("PlanePoint"));
//			MeasToSpan(DataArray[1], Tr("CirclePoint"));
//		}
//
//		if(app.saarm.Data.incrementgroup)
//		{
//			app.saarm.Data.Target.Format(_T("P1"));
//			IncrementName(app.saarm.Data.Group, 1);
//		}
//		SetDisplayData();
//
////		for(int i=0; i<DataArray.GetSize(); i++)
////			for( int j=0; j<DataArray[i]->P.GetSize(); j++ )
////				TRACE("Array %d, pt %d:\n X=%f, Y=%f, Z=%f\n"), i, j, DataArray[i]->P[j]->x, DataArray[i]->P[j]->y, DataArray[i]->P[j]->z);
//
//		ret = TRUE;
//	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoSlotMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoSlotMeas!!\n");
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DSlotMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = FALSE;
	BOOL ret = FALSE;

	dlg.DoModal();
	ret = TRUE;
//	if( dlg.DoModal() == IDOK )
//	{
//		app.saarm.Data.Target.Format(_T("SlotCenter"));
//		MeasToSpan(dlg.center);
//		
//		if( app.saarm.Data.SendGeom )
//		{
//			app.saarm.SockGeom.SendCircle(dlg.t, dlg.rad, dlg.name + Tr("A"), dlg.notes);
//			app.saarm.SockGeom.SendCircle(dlg.tB, dlg.radB, dlg.name + Tr("B"), dlg.notes);
//		}
//
//		if(app.saarm.Data.FirePointsToo)
//		{
//			MeasToSpan(DataArray[0], Tr("PlanePoint"));
//			MeasToSpan(DataArray[1], Tr("CircleAPoint"));
//			MeasToSpan(DataArray[2], Tr("CircleBPoint"));
//		}
//
//		if(app.saarm.Data.incrementgroup)
//		{
//			app.saarm.Data.Target.Format(_T("P1"));
//			IncrementName(app.saarm.Data.Group, 1);
//		}
//		SetDisplayData();
//
////		for(int i=0; i<DataArray.GetSize(); i++)
////			for( int j=0; j<DataArray[i]->P.GetSize(); j++ )
////				TRACE("Array %d, pt %d:\n X=%f, Y=%f, Z=%f\n"), i, j, DataArray[i]->P[j]->x, DataArray[i]->P[j]->y, DataArray[i]->P[j]->z);
//
//		ret = TRUE;
//	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}

BOOL ArmBase::DoPatchMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoPatchMeas!!\n");
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DPatchMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;

	dlg.DoModal();
	ret = TRUE;
//	if( dlg.DoModal() == IDOK )
//	{
//		if(DataArray[1]->GetSize() != 1)
//			return FALSE;
//		else
//		{
//			if( app.saarm.Data.SendGeom )
//			{
//				GetBoundaryPoints(dlg.t);
//				app.saarm.SockGeom.SendPlane(dlg.t, BPoints, dlg.name, dlg.notes);
//			}
//			
//			if(app.saarm.Data.FirePointsToo)
//				MeasToSpan(DataArray[0], Tr("PatchPoint"));
//
//			app.saarm.Data.Target.Format(Tr("ProjectedPoint"));
//			MeasToSpan(*DataArray[1]->P[0]);
//		}
//
//		if(app.saarm.Data.incrementgroup)
//		{
//			app.saarm.Data.Target.Format(_T("P1"));
//			IncrementName(app.saarm.Data.Group, 1);
//		}
//		SetDisplayData();
//
////		for(i=0; i<DataArray.GetSize(); i++)
////			for( int j=0; j<DataArray[i]->P.GetSize(); j++ )
////				TRACE("Array %d, pt %d:\n X=%f, Y=%f, Z=%f\n"), i, j, DataArray[i]->P[j]->x, DataArray[i]->P[j]->y, DataArray[i]->P[j]->z);
//
//		ret = TRUE;
//	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoConeMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DConeMeas dlg;
	app.measDlg = &dlg;
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
BOOL ArmBase::DoSinglePtHoleMeas()
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	app.saarm.OnMeasurementInitiated(app.saarm.Data.Collection, app.saarm.Data.Group, app.saarm.Data.Target, app.pA->CurrentMeasModeName());  // 2005.04.12 for learn mode in crib sheets.

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);

	DSinglePtHole dlg;
	app.measDlg = &dlg;
	dlg.title.Format(_T("Single Point Hole Measurement"));
	BOOL ret = FALSE;
	if( dlg.DoModal() == IDOK )
	{
		ret = TRUE;
	}
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
///////////////////////////////////////////////////////////////////////////////////
// Inspections

BOOL ArmBase::DoPlaneInspect(SAIPGeomPlane& gp)
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DPlaneMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = TRUE;
	dlg.ObName = gp.Name;
	Vector pos;
	gp.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.InchesToLocal);
	gp.ObjectToInst.SetTranslation(pos);
	dlg.NominalTrans = gp.ObjectToInst;
	dlg.title = _T("Plane Inspection: " + gp.Name);
	dlg.DoModal();
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return TRUE;
}
BOOL ArmBase::DoPinInspect(SAIPGeomCircle& gp)
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoPinInspect!!\n");
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DHoleMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = TRUE;
	dlg.Pin = TRUE;
	dlg.ObName = gp.Name;
	dlg.title = _T("Pin Inspection: " + gp.Name);
	dlg.NominalRadius = gp.Radius*app.saarm.InchesToLocal;
	Vector pos;
	gp.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.InchesToLocal);
	gp.ObjectToInst.SetTranslation(pos);
	dlg.NominalTrans = gp.ObjectToInst;
	dlg.DoModal();
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return TRUE;
}
BOOL ArmBase::DoHoleInspect(SAIPGeomCircle& gh)
{
	if(armScannerSwitchOn) // 2016.06.29
	{
		TRACE0("armScannerSwitchOn - retruning false from probing meas...\n");
		SetMeasMode(13);
		if(Units::toolBarModeOn)
			app.saarm.SendSAIRCAttributesToSA();
		MessageBeep(MB_ICONHAND);
		return FALSE;
	}

	if((app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.pA->ConnectedToArm)
	{
		if(!app.pA->ValidateCurrentProbe())
			return FALSE;
	}
	TRACE("DoHoleInspect!!\n");
	
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.2,TRUE);
	Sleep(200);
	
	DHoleMeas dlg;
	app.measDlg = &dlg;
	dlg.Inspect = TRUE;
	dlg.Pin = FALSE;
	dlg.ObName = gh.Name;
	dlg.title = _T("Hole Inspection: " + gh.Name);
	dlg.NominalRadius = gh.Radius*app.saarm.InchesToLocal;
	Vector pos;
	gh.ObjectToInst.GetTranslation(&pos);
	pos.Scale(app.saarm.InchesToLocal);
	gh.ObjectToInst.SetTranslation(pos);
	dlg.NominalTrans = gh.ObjectToInst;
	dlg.DoModal();
	ClearDataArray();

	d.HighlightButton();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////////////
// Utilities for Measurement functions
Vector ArmBase::ComputeAvgDirection()
{
	Vector AveDir;
	VectorArray Norms;

	DataArray[app.pA->StepIndex]->GetVectorArrayofNormals(Norms);

	Norms.Debug();
	Norms.Average(&AveDir);
	AveDir.Normalize();
	return AveDir;
}
void ArmBase::ScaleArmPtsArrayToInches(ArmPointArray& Input, ArmPointArray* FitPts)
{
	for(INT_PTR i=0; i<Input.P.GetSize(); i++)
	{
		ArmPoint aPt(Input.Get(i));
		aPt.Scale(app.saarm.LocalToInches);
		FitPts->Add(aPt);
	}
}
void ArmBase::ScaleArmPtsArrayToLocal(ArmPointArray& Input, ArmPointArray* FitPts)
{
	for(INT_PTR i=0; i<Input.P.GetSize(); i++)
	{
		ArmPoint aPt(Input.Get(i));
		aPt.Scale(app.saarm.InchesToLocal);
		FitPts->Add(aPt);
	}
}
bool ArmBase::JointsMovedBeyondThreshold()
{
	for(int i=0; i<DOF; i++)
	{
		if(fabs(J.x[i] - lastJ.x[i]) > app.saarm.Data.jtUpdateThreshold)
		{
			lastJ = J;
			return true;
		}
	}
	return false;
}
void ArmBase::GetBoundaryPoints(Transform base)
{	
	for(int i=0;i<BPoints.GetSize();i++)
		delete BPoints[i];
	BPoints.RemoveAll();

	//first drop all the points to get 2D points
	Vector PtInBase;
	Transform temp;
	base.Invert(&temp);

//	VectorArray pts;
//	ScaleVecArray(*DataArray[0], &pts);
	
	for (int i=0;i<DataArray[0]->P.GetSize();i++)
	{
		//first map to baseframe(of plane)
		temp.Multiply(*DataArray[0]->P[i],&PtInBase);
		BPoints.Add( new Vector2D( PtInBase.x, PtInBase.y ) );
	}

	CWordArray Edge;
	MakeConvexRepresentation(Edge);
	
	if (BPoints.GetSize() > 3)
	{
		CTypedPtrArray <CObArray, Vector2D*> VEdge;		// 2-D bounding points!
		for (int i=0;i<Edge.GetSize();i++)
			VEdge.Add( new Vector2D(*BPoints[Edge[i]]) );
		for(int i=0;i<BPoints.GetSize();i++)
			delete BPoints[i];
		BPoints.RemoveAll();
		for (int i=0;i<Edge.GetSize();i++)
			BPoints.Add( VEdge[i] );
	}
}

BOOL ArmBase::MakeConvexRepresentation(CWordArray& Edge)
{
	BYTE NoChange = TRUE;

	if (BPoints.GetSize() <= 3 )
		return TRUE;

	int MaxPt = 0;
	double dist = 0.0;
	for (int i=0;i<BPoints.GetSize();i++)
	{
		double len = BPoints[i]->Length();
		if (len > dist)
		{
			dist = len;
			MaxPt = i;
		}
	}
	Edge.Add( MaxPt );
	BYTE Looped = FALSE;
	for (int i=0;i<Edge.GetSize();i++)		// for each given anchor point.
	{
		int ap = Edge[ Edge.GetUpperBound() ];

		for (int j=0; j<BPoints.GetSize(); j++)
		{
			if (j==ap)
				continue;
			BYTE skip = FALSE;
			for (int n=0;n<Edge.GetSize();n++)
				if (j==Edge[n])
				{
					skip = TRUE;
					break;
				}
			if (skip)
				continue;

			int hp = j;			// we have an hypothesized edge.

			Vector2D ap_hp( *BPoints[hp] );
			ap_hp.Sub( *BPoints[ap] );

			Vector2D perp( -ap_hp.y, ap_hp.x );

			BYTE SignChanged = FALSE;
			int LastSign = 0;
			for (int k=0;k<BPoints.GetSize();k++)
			{
				if (k==hp || k==ap)		// don;'t test yourself or the previous anchor!
					continue;

				int op = k;				// we have the other point!

				Vector2D hp_op( *BPoints[op] );
				hp_op.Sub( *BPoints[hp] );

				double sign = hp_op.Dot( perp );
				int ts = 0;
				if (sign > 0 )
					ts = 1;
				else
					ts = -1;
				if (ts*LastSign < 0)
				{
					SignChanged = TRUE;
					break;
				}
				LastSign = ts;
			}
			if (SignChanged)
				continue;
			if (hp == Edge[0])
			{
				Looped = TRUE;// all done!;
				break;
			}
			Edge.Add( hp );
			break;
		}
		if (Looped)
			break;
	}
	return TRUE;
}

void ArmBase::Test()
{
}

//////////////////////////////////////////////////////////////////////////////
// Here's how DTrack simulates the geometry measurements
/*
// populate the vector array
BOOL GenTrack::MeasureTilDone(double SpatInc, double TFreq, int samples)
{
	DTrackDlg& d = *((DTrackDlg*)app.m_pMainWnd);
	d.OpProg->UpdateTopString("Simulatin Mode:\nMeasureTilDone"));

	Vector CurrPos, OldPos; 
	Vector CurrPosCart, OldPosCart, Delta;
	int ctr = 0;

	CurrPos = Vector(0.0,0.0,0.0);

	Vector DHV(D,H,V);
	Vector XYZ, newXYZ;
	DHV.RTPtoXYZ(&XYZ);
	double x, y, z, rho, theta, phi;

	while(ctr < samples)
	{
		ProcessWaitingMessages();
		Delay(0.0005);

		if(app.StopNow)
		{
			if(app.KillDataBuffer)
			{
				d.Span.KillBuffer();
				return FALSE;
			}
			else
				return TRUE;			
		}
		
		if(d.MeasMode == 3 || d.MeasMode == 11)	// sphere
		{			
			rho = 0.2+((double)rand())/((double)RAND_MAX)*0.01;		// the radius of the sphere!!
			phi = 180.0*((double)rand())/((double)RAND_MAX);
			theta = 360.0*((double)rand())/((double)RAND_MAX);

			Vector dhv(rho, theta, phi);
			Vector xyz;
			dhv.RTPtoXYZ(&xyz);

			newXYZ = XYZ;
			newXYZ.Add(xyz);
			newXYZ.XYZtoRTP(&CurrPos);
		}
		if(d.MeasMode == 4)	// circle
		{
			rho = 0.2;		// the radius of the circle!!
			phi = 180.0*((double)rand())/((double)RAND_MAX);
			theta = 0.0;

			Vector dhv(rho, theta, phi);
			Vector xyz;
			dhv.RTPtoXYZ(&xyz);

			newXYZ = XYZ;
			newXYZ.Add(xyz);
			newXYZ.XYZtoRTP(&CurrPos);
		}
		if(d.MeasMode == 5)	// plane
		{			
			Vector xyz;
			xyz.x = ((double)rand())/((double)RAND_MAX);
			xyz.y = ((double)rand())/((double)RAND_MAX);
			xyz.z = D*cos(V*DTR);

			newXYZ = XYZ;
			newXYZ.Add(xyz);
			newXYZ.XYZtoRTP(&CurrPos);
		}

		CurrPos.RTPtoXYZ(&CurrPosCart);
		OldPos.RTPtoXYZ(&OldPosCart);
		Delta = OldPosCart;
		Delta.Sub(CurrPosCart);

		if(Delta.Length() >= SpatInc)
		{
			ctr++;
			OldPos = CurrPos;
//			d.MeasurementToSpan(CurrPos);	//Testing only!!!!
			int spk = AvgVec.P.Add(new Vector(CurrPos));
			d.OpProg->UpdateProgressBar(ctr);
		}
	}
	return TRUE;
}
*/


// Faro Only
BOOL ArmBase::ProcessButtonsandUpdatePacket(JointPacket* Data)
{
	return TRUE;
}
BOOL ArmBase::ProcessXYZIJKPacket(IJKPacket* Data)
{
	return TRUE;
}
// Romer only
BOOL ArmBase::IsRemoteMouseEnabled()
{
	return FALSE;
}
void ArmBase::ToggleRemoteMouse()
{
}
void ArmBase::RemoteMouse(BOOL flag)
{
}
///////////////////////////////////////////////////////////////////////////////////////
bool ArmBase::StartPipeProbe() 
{
	TRACE0("Base class StartPipeProbe called.\n"); 
	//if(!app.saarm.scannerLicenseValid)AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
	return false;
}
bool ArmBase::StopPipeProbe()
{
	TRACE0("Base class StopPipeProbe called.\n");
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////
// Scanner fcns
BOOL ArmBase::StartScannerMeas() 
{
	TRACE0("Base class StartScannerMeas called.\n"); 
	if(!app.saarm.scannerLicenseValid)
	{
		AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
		return FALSE;
	}

	DSimScannerMeas dlg;
	dlg.DoModal();

	return TRUE;
}

void ArmBase::SendOnScanPassStarted()
{
	//return; // 2016.07.07 - this is no longer needed by SA, and was causing clouds to be created even though this sends a packet with no data
	// 2020.01.06 - XIT-1616 - put this back. Now to be used for meshing, and keeping track of passes.
	if(!app.saarm.Data.useScanStripeClouds || !performingStdScannerMeas)
		return;

	CriticalSectionLock lock( scannerCS );

	scanStripePkt.ClearData();
	scanStripePkt.name = app.saarm.Data.Group;
	scanStripePkt.collectionName = app.saarm.Data.Collection;
	scanStripePkt.stripes.Add( new SAIPScanStripe );

	//SAIPScanStripePoint sspt = *new SAIPScanStripePoint();
	//sspt.x = 0.0;
	//sspt.y = 0.0;
	//sspt.z = 0.0;
	//sspt.Flags = 0;
	//scanStripePkt.AddPoint(sspt);

	INT_PTR idx = scanStripePkt.stripes.GetUpperBound();
	scanStripePkt.stripes[idx]->scannerXform = CurrentTransform;
	scanStripePkt.stripes[idx]->scannerXform.Scale(app.saarm.LocalToInches);

	BOOL ret = app.saarm.QS.SendPacket(scanStripePkt);

	/////////////////////////////////////////////////////////////////////////////
	// 2016.07.07
	if(ret)
	{
		TRACE0("Sent ScanPassStarted Packet\n");
	}
	else
	{
		app.saarm.ActivateInstQuickSock();
		TRACE0("<<<<<<<<<<<<<<<<Could NOT SEND ScanPasStarted Packet!!!!\n");
	}
	/////////////////////////////////////////////////////////////////////////////

	//CString dbg;
	//dbg.Format(_T("Sending %ld stripes containing %ld pts. to Cloud %s"), scanStripePkt.stripes.GetSize(), scanStripePkt.GetTotalPointCount(), app.saarm.Data.Group);
	//TRACE1("%s\n", dbg);

	//CString statmsg;
	//statmsg.Format(_T("Sending ScanPass Started Event"));
	//PostUpdateScanDlg(statmsg);
}

void ArmBase::SendOnScanPassStopped()
{
	return; // 2016.07.07 - this is no longer needed by SA, and was causing clouds to be created even though this sends a packet with no data
	if(!app.saarm.Data.useScanStripeClouds || !performingStdScannerMeas)
		return;

	CriticalSectionLock lock( scannerCS );

	scanStripePkt.ClearData();
	scanStripePkt.name = app.saarm.Data.Group;
	scanStripePkt.collectionName = app.saarm.Data.Collection;
	scanStripePkt.stripes.Add( new SAIPScanStripe );

	//SAIPScanStripePoint sspt = *new SAIPScanStripePoint();
	//sspt.x = 0.0;
	//sspt.y = 0.0;
	//sspt.z = 0.0;
	//sspt.Flags = 0;
	//scanStripePkt.AddPoint(sspt);

	INT_PTR idx = scanStripePkt.stripes.GetUpperBound();
	scanStripePkt.stripes[idx]->scannerXform = CurrentTransform;
	scanStripePkt.stripes[idx]->scannerXform.Scale(app.saarm.LocalToInches);

	BOOL ret = app.saarm.QS.SendPacket(scanStripePkt);

	/////////////////////////////////////////////////////////////////////////////
	// 2016.07.07
	if(ret)
	{
		TRACE0("Sent ScanPassStopped Packet\n");
	}
	else
	{
		app.saarm.ActivateInstQuickSock();
		TRACE0("<<<<<<<<<<<<<<<<<<<Could NOT SEND ScanPasStopped Packet!!!!\n");
	}
	/////////////////////////////////////////////////////////////////////////////

	//CString dbg;
	//dbg.Format(_T("Sending %ld stripes containing %ld pts. to Cloud %s"), scanStripePkt.stripes.GetSize(), scanStripePkt.GetTotalPointCount(), app.saarm.Data.Group);
	//TRACE1("%s\n", dbg);

	//CString statmsg;
	//statmsg.Format(_T("Sending ScanPass Stopped Event"));
	//PostUpdateScanDlg(statmsg);
}

BOOL ArmBase::CheckAndSendScannerCloud(int minSize /*= 0*/, bool scanningEnded /*= false*/)
{
	CriticalSectionLock lock( scannerCS );
	//TRACE0("SENDING CLOUD packet to SA, check size first...\n");

	if(app.saarm.Data.useScanStripeClouds && performingStdScannerMeas)
	{
		if(scanStripePkt.stripes.GetSize() <= 0)
			return FALSE;

		DWORD_PTR pts = scanStripePkt.GetTotalPointCount();
		if(pts <= (DWORD_PTR)minSize)
		{
			if(scanningEnded && app.saarm.Data.autoIncCloudName)
			{
				app.saarm.Data.Target.Format(_T("P1"));
				IncrementName(app.saarm.Data.Group, 1);
				SetDisplayData();
			}
			return FALSE;
		}
	}

	if(app.saarm.Data.useScannerForAutoProxMeas && !performingStdScannerMeas)
	{
		VectorNamedArray cpts;
		CString targ;
		double dist = 0.0;
		double tol = 0.0;
		Vector resVecOrg, resVecEnd;
		bool isVecPos = false;
		Vector goalPointForWatch, measPointForWatch, closestNomPt, closestNomVector;
		double dummyProxTol, dummyAxisTol, workingFieldTol;
		for (INT_PTR i=0; i<scannerPkt.CB.GetSize(); i++)
		{
			SAIPCloudBlock* pB = scannerPkt.CB[i];
			for(INT_PTR j=0; j<pB->Count; j++)
			{
				SAIPCloudBlockPt& pib = pB->Block[j];
				Vector xyz(pib.x, pib.y, pib.z);

				bool ckOK = samp.CheckPoint(xyz, 0.0, 
					true, samp.measureOnce ? true : false, true,
					targ, dist, tol, closestNomPt,closestNomVector, resVecOrg, resVecEnd, isVecPos, 
					goalPointForWatch, measPointForWatch);
				//TRACE3("pt '%s', dist=%f, tol=%f\n", targ, dist, tol);
				
				samp.GetTolerances(dummyProxTol, dummyAxisTol, workingFieldTol);
				app.saarm.Data.GoodProximity = tol * app.saarm.InchesToLocal;
				app.saarm.Data.proxWorkingField = workingFieldTol * app.saarm.InchesToLocal;
				//app.saarm.SetScreamerParams();
				//app.saarm.ScreamThis(dist * app.saarm.InchesToLocal);
				if(ckOK)
				{
					//Vector xyzMeters = xyz;
					//xyzMeters.Scale(app.saarm.InchesToLocal);
					//X = xyzMeters.x;
					//Y = xyzMeters.y;
					//Z = xyzMeters.z;

					cpts.Add(targ, xyz);
				}
			}
		}
		scannerPkt.ClearCB();


		BOOL jtsOk = FALSE;
		if(app.pA->IsKreonScannerArm())
		{
			jtsOk = TRUE;	// arm xform (not joints actually) is updated during scanning, so we can send a xform that will be used if user is set to update graphics viewpoint w/arm xform. see KreonCmdWnd callback's KR_CMD_TRIGGERED_POSITION_NEEDED case
		}
		else
		{
			jtsOk = GetJoints(false);	// false argument means don't simulate!
		}

		CObArray pak;
		for(INT_PTR i=0; i<cpts.GetSize(); i++)
		{
			SAIPMeasurement& measpak = *new SAIPMeasurement;
			measpak.collection = app.saarm.Data.Collection;
			measpak.Group = samp.destGroup;
			measpak.Target = cpts[i].Name;
			measpak.TargetOffset = 0.0;
			measpak.TargetOffset2 = 0.0;

			SAIPMeasDataCMM& p = *new SAIPMeasDataCMM;
			p.Point = cpts[i];

			if(jtsOk)
			{
				p.JointValues.Create(DOF,TRUE);
				for( int i=0; i<DOF; i++)
				{
					p.JointValues.x[i] = J[i];
				}
			}

			Transform t(CurrentTransform);
			Vector pos;
			t.GetTranslation(&pos);
			pos.Scale(app.saarm.LocalToInches);
			t.SetTranslation(pos);
			p.probeDir.SetProbingTransform(t);

			measpak.MeasData.Add(&p);
			pak.Add(&measpak);
		}
		if(pak.GetSize())
		{
			app.saarm.QS.SendPackets(pak);
			TRACE1("sent %d paks\n", pak.GetSize());
		}
		ObArrayHelper::DeleteAll(pak);

		if(jtsOk)
			UpdateToSpan(true);

		if(scanningEnded && app.saarm.Data.autoIncCloudName)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}

		return TRUE;
	}

	if (!app.saarm.Data.useScanStripeClouds || !performingStdScannerMeas)
	{
		if (app.saarm.Data.voxelizationEnabled && performingStdScannerMeas)
		{
			BOOL ret = FALSE;
			try
			{
				UINT pts = scannerPkt.GetPointCount();
				if (!scanningEnded && pts <= (UINT) app.saarm.Data.minCloudPtsToVoxelize)
				{
					return FALSE;
				}
				if (scanningEnded)
				{
					scannerPkt.ClearCB(); // clear the buffer since we very likely did not get the exact minCloudPtsToVoxelize before ending

					if (app.saarm.Data.autoIncCloudName)
					{
						app.saarm.Data.Target.Format(_T("P1"));
						IncrementName(app.saarm.Data.Group, 1);
						SetDisplayData();
					}
					return true;
				}

				TRACE2("%d pts In, name: %s\n", pts, app.saarm.Data.Group);
				//scannerPkt.Name = app.saarm.Data.Group;
				//scannerPkt.collectionName = app.saarm.Data.Collection;

				//BOOL ret = app.saarm.QS.SendPacket(scannerPkt);

				SAIPCloud& voxCloud = *new SAIPCloud;
				voxelizer.VoxelizeCloudBlock(scannerPkt, app.saarm.Data.voxelSize, app.saarm.Data.minPtsPerVoxel, voxCloud);

				voxCloud.Name = app.saarm.Data.Group;
				voxCloud.collectionName = app.saarm.Data.Collection;
				ret = app.saarm.QS.SendPacket(voxCloud);

				TRACE1("%d pts Out\n\n", voxCloud.GetPointCount());
				
				scannerPkt.ClearCB();
				delete &voxCloud;
			}
			catch (CMemoryException* e)
			{
				CString m;
				m.Format(_T("ArmBase::CheckAndSendScannerCloud Memory Exception"));
				TRACE1("%s\n",m);
				e->Delete();
			}
			catch (CFileException* e)
			{
				CString m;
				m.Format(_T("ArmBase::CheckAndSendScannerCloud Archive File Exception = %d, OS error = %ld"),e->m_cause, e->m_lOsError);
				TRACE1("%s\n",m);
				e->Delete();
			}
			catch (CException* e)
			{
				CString m;
				m.Format(_T("ArmBase::CheckAndSendScannerCloud CException caught"));
				TRACE1("%s\n",m);
				e->Delete();
			}
			catch (...)
			{
				CString m;
				m.Format(_T("ArmBase::CheckAndSendScannerCloud General Exception caught"));
				TRACE1("%s\n",m);
			}

			return ret;
		}
		else
		{
			//TRACE1("%d pts\n", pts);
			scannerPkt.Name = app.saarm.Data.Group;
			scannerPkt.collectionName = app.saarm.Data.Collection;
			BOOL ret = app.saarm.QS.SendPacket(scannerPkt);
			scannerPkt.ClearCB();

			CString statmsg;
			statmsg.Format(_T("Sending %d pts. to Cloud %s"), scannerPkt.GetPointCount(), app.saarm.Data.Group);
			PostUpdateScanDlg(statmsg);

			if(scanningEnded && app.saarm.Data.autoIncCloudName)
			{
				app.saarm.Data.Target.Format(_T("P1"));
				IncrementName(app.saarm.Data.Group, 1);
				SetDisplayData();
			}

			BOOL jtsOk = FALSE;
			if(app.pA->IsKreonScannerArm())
			{
				jtsOk = TRUE; // arm xform (not joints actually) is updated during scanning, so we can send a xform that will be used if user is set to update graphics viewpoint w/arm xform. see KreonCmdWnd callback's KR_CMD_TRIGGERED_POSITION_NEEDED case
			}
			else
			{
				jtsOk = GetJoints(false);	// false argument means don't simulate!
			}
			if(jtsOk)
			{
				UpdateToSpan();
			}
			if(ret && !ConnectedToArm)
			{
				simCloudCtr++;
			}

			return ret;
		}
	}
	else
	{
		scanStripePkt.name = app.saarm.Data.Group;
		scanStripePkt.collectionName = app.saarm.Data.Collection;
		BOOL ret = FALSE;
		try
		{
			ret = app.saarm.QS.SendPacket(scanStripePkt);
			if(!ret)
			{
				// blt 2018.03.16 - take out packet skipping in case Steve's inst. threading has made it unnecessary
				////TRACE1("Last Socket Error = %d\n", app.saarm.QS.GetLastSockError());
				// // blt - 2018.02.08 - skip handling this packet after a network error to prevent data back up and freezing (See netError in QuickSock)
				//if(netError)
				//{
				//	//TRACE2("netError - Last Socket Error = %d. Stripe Pts in buffer = %ld\n", app.saarm.QS.GetLastSockError(), scanStripePkt.GetTotalPointCount());
				//	scanStripePkt.ClearData();
				//	//TRACE1("netError - After clearing - Stripe Pts in buffer = %ld\n", scanStripePkt.GetTotalPointCount());
				//	netError = false;
				//}
				//if(criticalNetError)
				//{
				//	//TRACE2("criticalNetError - Last Socket Error = %d. Stripe Pts in buffer = %ld\n", app.saarm.QS.GetLastSockError(), scanStripePkt.GetTotalPointCount());
				//	scanStripePkt.ClearData();
				//	//TRACE1("criticalNetError - After clearing - Stripe Pts in buffer = %ld\n", scanStripePkt.GetTotalPointCount());
				//	StopScannerMeas();
				//	criticalNetError = false;
				//}
				if(scanningEnded && app.saarm.Data.autoIncCloudName)
				{
					app.saarm.Data.Target.Format(_T("P1"));
					IncrementName(app.saarm.Data.Group, 1);
					SetDisplayData();
				}
				return ret;
			}
			//if(netError)
			//{
			//	netError = false;
			//}
		}
		catch (CMemoryException* e)
		{
			CString m;
			m.Format(_T("ArmBase::CheckAndSendScannerCloud Memory Exception"));
			TRACE1("%s\n",m);
			e->Delete();
		}
		catch (CFileException* e)
		{
			CString m;
			m.Format(_T("ArmBase::CheckAndSendScannerCloud Archive File Exception = %d, OS error = %ld"),e->m_cause, e->m_lOsError);
			TRACE1("%s\n",m);
			e->Delete();
		}
		catch (CException* e)
		{
			CString m;
			m.Format(_T("ArmBase::CheckAndSendScannerCloud CException caught"));
			TRACE1("%s\n",m);
			e->Delete();
		}
		catch (...)
		{
			CString m;
			m.Format(_T("ArmBase::CheckAndSendScannerCloud General Exception caught"));
			TRACE1("%s\n",m);
		}

		//CString statmsg;
		//statmsg.Format(_T("Sending %ld stripes containing %ld pts. to Cloud %s"), scanStripePkt.stripes.GetSize(), scanStripePkt.GetTotalPointCount(), app.saarm.Data.Group);
		//PostUpdateScanDlg(statmsg);

		scanStripePkt.ClearData();

		if(scanningEnded && app.saarm.Data.autoIncCloudName)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			IncrementName(app.saarm.Data.Group, 1);
			SetDisplayData();
		}

		BOOL jtsOk = FALSE;
		if(app.pA->IsKreonScannerArm())
		{
			jtsOk = TRUE; // arm xform (not joints actually) is updated during scanning, so we can send a xform that will be used if user is set to update graphics viewpoint w/arm xform. see KreonCmdWnd callback's KR_CMD_TRIGGERED_POSITION_NEEDED case
		}
		else
		{
			jtsOk = GetJoints(false);	// false argument means don't simulate!
		}
		if(jtsOk)
		{
			UpdateToSpan();
		}
		if(ret && !ConnectedToArm)
		{
			simCloudCtr++;
		}

		return ret;
	}
	return FALSE;
}
void ArmBase::AddScannerCloudPt(SAIPCloudBlockPt& cbpt)
{
	CriticalSectionLock lock( scannerCS );
	scannerPkt.AddPoint(cbpt);
}
void ArmBase::ClearScanClouds()
{
	scannerPkt.ClearCB();
	scanStripePkt.ClearData();
}
void ArmBase::AddScannerCloudStripe()
{
	scanStripePkt.AddStripe();
}
void ArmBase::AddScannerCloudStripePt(SAIPScanStripePoint& sp)
{
	CriticalSectionLock lock( scannerCS );
	scanStripePkt.AddPoint(sp);
}
void ArmBase::PostUpdateScanDlg(CString msg)
{
	//TRACE1("ArmBase::PostUpdateScanDlg called with text '%s'\n", msg);
}
///////////////////////////////////////////////////////////////////////////////
// Meas Settings
BOOL ArmBase::StreamPtsSettings()
{
	DStreamPtsSettings diag;
	diag.scan_spatial = app.saarm.Data.scan_spatial;
	diag.scan_spatial_inc = app.saarm.Data.scan_spatial_inc/app.saarm.units.UnitToMeters[app.saarm.units.distunits];
	diag.unitsLabel = app.saarm.units.DistUnitNames[app.saarm.units.distunits];

	if(diag.DoModal()!=IDOK)
		return FALSE;

	app.saarm.Data.scan_spatial = diag.scan_spatial;
	app.saarm.Data.scan_spatial_inc = diag.scan_spatial_inc*app.saarm.units.UnitToMeters[app.saarm.units.distunits];

	return FALSE;
}
///////////////////////////////////////////////////////////////////////////////
// SA Model Info
void ArmBase::ClearModelInfo() 
{
	saModelInfo.ClearInfo();
}
void ArmBase::CheckAndUpdateModelInfo()
{
	if(!app.saarm.Data.autoCorrectModel || !probeValid)
		return;

	JointOffsetCalItem jCal;

	Transform t(CurrentTransform);
	Vector pos;
	t.GetTranslation(&pos);
	pos.Scale(app.saarm.LocalToInches);
	t.SetTranslation(pos);
	if (!IsMetrisGarda())
		jCal.SetTransform(t);// no frames for funny arm
	jCal.Point = pos;
	jCal.JointValues.Create(DOF,TRUE);
	for( int i=0; i<DOF; i++)
	{
		jCal.JointValues.x[i] = J[i];
	}

	if(saModelInfo.CheckUpdate(jCal))
		saModelInfo.ModelInfoUpdateToSpan();
}
///////////////////////////////////////////////////////////////////////////////
