// ArmBase.h: interface for the ArmBase class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ARMBASE_H__8FE77831_ECBC_48BC_8A34_0FDB7983DB02__INCLUDED_)
#define AFX_ARMBASE_H__8FE77831_ECBC_48BC_8A34_0FDB7983DB02__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "LogFile.h"
#include "JtPak.h"
#include "ArmButtons.h"

class ModelInfo;
class DAutoProxMeas;


class ArmBase : public CObject
{
public:
	ArmBase();
	virtual ~ArmBase();

//	MainTimerWnd timerWnd;
	LogFile Log;
	CString LogFileName;

	void SetFileNames();

	BOOL ConnectedToArm;
	BOOL Measuring;
	bool mainDlgIsInitialized; // JIRA XIT-29 - 2016.05.17 - added for RDS arms - order of events from arm changed as of RDS v.4.1 - caused win10 pc's to freeze at startup if scanner was already on
	int StepIndex;
	int inspectreq;

	int DOF;
	GenVec J;	// joints
	GenVec lastJ;	// previous joint values
	Vector LastPos;
	int LastButton;
	WORD buttonstate;
	bool overrideArmButtons;
	Vector LastRecordedPoint;
	double X,Y,Z;
	double II,JJ,KK;
	Transform CurrentTransform;
	WORD CurrentButtons;
	Transform NativeArmToSA;
	double Temperature;
	CString CMMSerialNum;
	bool probeValid;
	bool performingStdScannerMeas;
	Voxelizer voxelizer;

	// storage for measurement arrays
	CTypedPtrArray <CObArray, ArmPointArray*> DataArray;
	void ScaleDataArrays(double scaleFactor);
	void ClearDataArray();
	void ClearVectorArray();
	void ClearData();
	INT_PTR AddPoint();	// returns index in current DataArray's VectorArray after adding point
	INT_PTR AddPoint(const Vector& pos, const Vector& norm, const GenVec& jts);	// returns index in current DataArray's VectorArray after adding point
	INT_PTR RemovePoint();	// returns index in current DataArray's VectorArray after removing last point
	bool GetAvgPtAndRMSFromDataArray(int array, Vector& avg_xyz, double& rmserr);
	// 2001.03.23 begin -- for guided measurement
//	CTypedPtrArray <CObArray, SAIPMeasGroupPt*> GPoints;
//	void ClearGuidedPts();
	SAIPMeasGroup GuidedMeasGroup;
//	CString gGpsuffix;
//	CString gTsuffix;
	// 2001.03.23 end

	// simulation mode
	double simjtangle;
	int simCloudCtr;
	friend class DSimScannerMeas;

	void SetDisplayData();
	void GetDisplayData();
	bool SetMeasMode(int measmode);
	bool SetMeasModeByName(const CString& name, CString& pErrorString);
	CString CurrentMeasModeName();

	void CheckPad();

	void UpdateToSpan(bool zeroOffset=false);
	void MeasToSpan(bool zeroOffset=false, bool sendJointValues=true);
	void MeasToSpan(const Vector& XYZ, bool zeroOffset=false, bool updateGraphicModel=false, bool sendJointValues=true);
	void MeasToSpan(ArmPointArray* va, CString FstTarg, bool zeroOffset=false);
	void MeasToSpan(const Vector& XYZ, CString& Group, CString& Target, bool zeroOffset=false, bool sendJointValues=true);	// 2001.03.23 for Guided Meas
	void MeasToSpan(const Vector& XYZ, const CString& groupSuffix, bool zeroOffset=false, bool sendJointValues=true);	// 2004.10.19 -- for e.g. Multi-Plane cross
	void FrameToSpan(CString& Name, BOOL UpdateOnly);
	void FrameToSpan(const Transform& frame, const CString& name, BOOL UpdateOnly=FALSE);
	void FrameUpdateToSpan();
	void FrameandJointUpdateToSpan();	// added 2001.11.12
	void SavenQuit();

	BOOL FixBlankNames();
	void IncrementName(CString& Target, int inc=1);

	virtual bool ConnectToArm(const int dof){TRACE0("ArmBase ConnectToArm called.\n"); J.Create(dof, TRUE); return TRUE;}	// overridden in derived arm classes - gets called by SAArm::ConnecToHardware (that's the SAInstBase derivation)
	virtual BOOL DisconnectFromHardware() {TRACE0("ArmBase DisconnectFromHardware called.\n"); return FALSE;}
	virtual BOOL GetAllData();
	virtual bool IsProbeValid(){TRACE0("ArmBase IsProbeValid called.\n"); return true;}
	//virtual bool IsTouchProbeWithButtonEvents(){TRACE0("ArmBase IsTouchProbeWithButtonEvents called.\n"); return false;}	// 2009.01.05 - added for Faro's Renishaw Sensor touch probe - to change button behavior for point scan mode
	virtual BOOL GetPos();
	virtual BOOL GetJoints(bool simulate=true);
	virtual BOOL GetTemp();
	virtual int GetButtonStates();
	virtual BOOL CheckButton(int but, int* duration);
	void ClearButtonState();

	virtual BOOL IsRomer();
	virtual BOOL IsFaro();
	virtual BOOL IsFaroUSB();
	virtual BOOL IsFrenchRomer();
	virtual BOOL IsSNL();
	virtual BOOL IsMetrisGarda();
	virtual BOOL IsAPIBacesAxxis();
	virtual BOOL IsKreonScannerArm();
	virtual BOOL IsHexagonRDS();
	virtual void Test();

	virtual void DoPostInitOperations() {}

	virtual BOOL ScannerSettings() {TRACE0("Base class ScannerSettings called.\n"); return FALSE;}	// 2004.06.02
	virtual BOOL StartScannerMeas();
	virtual BOOL StopScannerMeas() {TRACE0("Base class StopScannerMeas called.\n"); return FALSE;}	// 2004.06.15
	virtual void PostUpdateScanDlg(CString msg);
	ModelessDialogTracker trackScanDlg;

	virtual BOOL StreamPtsSettings();
	
	virtual bool StartPipeProbe();
	virtual bool StopPipeProbe();

	BOOL DoEasyPadLocMeas();

	BOOL DoPointMeas();
	BOOL DoSAIPPointMeas();	// 2005.01.10
	BOOL DoStreamPtsMeas();
	BOOL DoSphereMeas();
	BOOL DoCircleMeas();
	BOOL DoPlaneMeas();
	BOOL DoFrameMeas();
	BOOL DoHoleMeas();
	BOOL DoPinMeas();
	BOOL DoSlotMeas();
	BOOL DoLineMeas();
	BOOL DoOffsetLineMeas();	// the sheet metal routine
	BOOL DoPatchMeas();
	BOOL DoSectionMeas();
	BOOL DoTheoMeas();
	BOOL DoGuidedMeas();	// 2001.03.23 - added 2004.08.24 - now, this starts the pt. select prompt in SA
	BOOL MeasureGuidePts();	// 2004.08.24 - do the meas
	BOOL MeasureAutoProximityPoints(SAIPAutoMeasProximity &p);	// 2006.06.09
	void UpdateAutoProxSamp(SAIPAutoMeasProximity &p);
	DAutoProxMeas* pAutoProxDlg;
	BOOL DoAvgPtsMeas();
	virtual BOOL DoPipeProbeMeas();
	BOOL DoGeomTriggerMeas();
	BOOL DoConeMeas();
	BOOL DoSinglePtHoleMeas();

	BOOL DoPlaneInspect(SAIPGeomPlane& gp);
	BOOL DoPinInspect(SAIPGeomCircle& gp);
	BOOL DoHoleInspect(SAIPGeomCircle& gh);

	BOOL GetCircle(CString title, int* samples, double* radius, double* RMSErr, double* MaxErr);
	BOOL GetPlane(CString title, int* samples, double* RMSErr, double* MaxErr);

	Vector ComputeAvgDirection();	// given ArmPointArray Norms (array of ijk's)
	void ScaleArmPtsArrayToInches(ArmPointArray& Input, ArmPointArray* FitPts);
	void ScaleArmPtsArrayToLocal(ArmPointArray& Input, ArmPointArray* FitPts);
	bool JointsMovedBeyondThreshold();
	// more plane stuff
	BOOL MakeConvexRepresentation(CWordArray& Edge);
	void GetBoundaryPoints(Transform base);
	CTypedPtrArray <CObArray, Vector2D*> BPoints;

	// Romer Cimcore only
	bool jtlimitbad; 
	virtual int GetProbeNumber();
	virtual int GetButtonCount();
	virtual void SetProbeNumber(int ProbeNum);
	virtual BOOL ValidateCurrentProbe();
	virtual void SetProbeAutoDetect(BOOL ad);
	virtual BOOL SetVolumeLevel( int VolumeLevel );
	virtual void OutputSpeakerSound( int frequency, int duration );
	virtual void ArmBeep(int sound);
	virtual BOOL IsRemoteMouseEnabled();
	virtual void RemoteMouse(BOOL flag);
	virtual void ToggleRemoteMouse();

	BOOL FrameUpdateRequested;
	BOOL isMeasuringFrames;	// should not need these!

	// Added for Romer CimCore Articulator
	virtual bool ReadArmTransform(Transform& t) {TRACE0("Base class ReadArmTransform called.\n"); return false;}
	virtual bool DriveToArmTransform(const Transform& t) {TRACE0("Base class DriveToArmTransform called.\n"); return false;}
	virtual bool ReadJointValues(GenVec& jv) {TRACE0("Base class ReadJointValues called.\n"); return false;}
	virtual bool DriveToJointValues(const GenVec& jv) 
	{
		TRACE0("Base class DriveToJointValues called, values:\n"); 
		for(int i=0;i<jv.nx;i++)
			TRACE(_T("		%f \n"),jv.x[i]);
		return false;
	}
	virtual bool DriveToCommaDelimitedJointString(const CString& jv) {TRACE0("Base class DriveToCommaDelimitedJointString called.\n"); return false;}
	virtual bool DriveToNamedJtSet(const CString& name) {TRACE0("Base class DriveToNamedJtSet called.\n"); return false;}

	ArmButtons& armButtons;

	// FARO only
	GenVec JShift;
	BOOL firstshot;
	BOOL ContQueryMode;
	BOOL FirstTime;
	virtual BOOL ProcessButtonsandUpdatePacket(JointPacket* Data);
	virtual BOOL ProcessXYZIJKPacket(IJKPacket* Data);
	virtual void ToggleArmButtonBeep() {TRACE0("Base class ToggleArmButtonBeep called.\n");}
	virtual void SetAuxPort(BOOL on) {TRACE0("Base class ToggleAuxPort called.\n");}

	// Faro USB only
	virtual void DoCal() {TRACE0("ArmBase DoCal called.\n");}
	virtual void DoArmSettings() {TRACE0("ArmBase DoMouseSetttings called.\n");}
	virtual void ProbeCalSet() {TRACE0("ArmBase ProbeCalSet called.\n");}
	// TrackArm
	bool trackArmActive;
	bool GetTrackArmActive() const {return trackArmActive;}

	// KreonScannerArm only
	virtual void OpenKreonDongleLicenseDlg() {}

	// Hexagon RDS only
	bool armScannerSwitchOn;
	
	// Scanner fcns
	void SendOnScanPassStarted(); // 2016.03.28 - send an empty stripe packet (1 stripe with 1 all zero's pt. with current collection and cloud name and current transform) - may change later, but should work
	void SendOnScanPassStopped();	// 2016.03.28 - same as SendOnScanPassStarted
	virtual BOOL CheckAndSendScannerCloud(int minSize = 0, bool scanningEnded = false);	// returns true if something gets sent
	virtual void AddScannerCloudPt(SAIPCloudBlockPt& cbpt);
	virtual void AddScannerCloudStripe();
	virtual void AddScannerCloudStripePt(SAIPScanStripePoint& stripePt);
	void ClearScanClouds(); // 2020.04.07 - added to clear scan clouds after voxelization and/or scan line settings are changed by user. In case a buffer is waiting for a min number of points, but then buffer size changes due to settings change
	SAIPAutoMeasProximity samp; // for using the scanners for AutoProx Meas

	// SA Model Info
	void ClearModelInfo();
	void CheckAndUpdateModelInfo();

protected:   
	CriticalSectionWrapper& scannerCS;
	SAIPCloud& scannerPkt;
	SAIPScanStripeCloud& scanStripePkt;
	ModelInfo& saModelInfo;	// SA model
};

#endif // !defined(AFX_ARMBASE_H__8FE77831_ECBC_48BC_8A34_0FDB7983DB02__INCLUDED_)
