#include "stdafx.h"
#include "ArmPointArray.h"

ArmPointArray::ArmPointArray(void)
{
}

void ArmPointArray::Copy( ArmPointArray* pDest) const
{
	pDest->Clear();
	for (int i=0;i<GetSize();i++)
		pDest->P.Add( new ArmPoint(*P[i]) );
}

ArmPointArray& ArmPointArray::operator=(const ArmPointArray& aPts)
{
	aPts.Copy(this);
	return *this;
}

INT_PTR ArmPointArray::Add(const ArmPoint& aPt)
{
	return P.Add( new ArmPoint(aPt) );
}

void ArmPointArray::Append(const ArmPointArray& other)
{
	for (INT_PTR i=0;i<other.GetSize();i++)
		Add(*other[i]);
}

ArmPointArray::~ArmPointArray(void)
{
	Clear();
}
void ArmPointArray::Clear()
{
	for (int i=0;i<P.GetSize();i++)
		delete P[i];
	P.RemoveAll();
}
void ArmPointArray::Scale(double s)
{
	for (int i=0;i<P.GetSize();i++)
	{
		P[i]->Scale( s );
	}
}
void ArmPointArray::Average( Vector* avg ) const
{
	if (P.GetSize() < 1) 
		return;
	*avg = Vector( 0,0,0 );
	for (int i=0;i<P.GetSize();i++)
		avg->Add( P[i]->GetPoint() );
	avg->Scale( 1.0 / ( (double) P.GetSize() ) );
}
Vector ArmPointArray::Average(  ) const
{
	Vector avg;
	Average(&avg);
	return avg;
}
BOOL ArmPointArray::StandardDeviationMeanZero( Vector* stddev )
{
	if (P.GetSize() < 2) 
		return FALSE;
	Vector Mean,Temp;
	Average(&Mean);
	*stddev = Vector( 0,0,0 );
	Vector e2;
	for (int i=0;i<P.GetSize();i++)
	{
		Temp = *P[i];
		Temp.Sub(Mean);
		e2.x = Temp.x*Temp.x;
		e2.y = Temp.y*Temp.y;
		e2.z = Temp.z*Temp.z;
		stddev->Add( e2 );
	}
	stddev->Scale( 1.0 / ( (double) P.GetSize() - 1.0 ) );
	stddev->x = SafeSqrt( stddev->x);
	stddev->y = SafeSqrt( stddev->y);
	stddev->z = SafeSqrt( stddev->z);
	return TRUE;
}



INT_PTR ArmPointArray::GetVectorArrayofPoints(VectorArray& pts)
{
	pts.Clear(); // just in case
	for(INT_PTR i=0; i<GetSize(); i++)
	{
		pts.P.Add(new Vector(P[i]->GetPoint()));
	}
	//for(INT_PTR i=0; i<pts.P.GetSize(); i++)
	//{
	//	pts.P[i]->Debug();
	//}
	return pts.GetSize();
}
INT_PTR ArmPointArray::GetVectorArrayofNormals(VectorArray& norms)
{
	norms.Clear(); // just in case
	for(INT_PTR i=0; i<GetSize(); i++)
	{
		norms.P.Add(new Vector(P[i]->Normal));
	}
	//for(INT_PTR i=0; i<norms.P.GetSize(); i++)
	//{
	//	norms.P[i]->Debug();
	//}
	return norms.GetSize();
}
void ArmPointArray::SetFromVectorArray(const VectorArray& Input, const Transform& objectToInstrument)
{
	Clear();
	Vector point;
	for (int i=0;i<Input.GetSize();i++)
	{
		objectToInstrument.Multiply(*Input[i], &point);
		Vector norm;
		GenVec jts;
		ArmPoint aPt(point, norm, jts);
		Add(aPt);
	}
}

void ArmPointArray::Serialize(CArchive& ar)
{
	BYTE Schema = 0;
	BYTE LoadSchema = 0;
	if (ar.IsStoring())
	{ 
		ar << Schema;
	}
	else
	{
		ar >> LoadSchema;
	}
	P.Serialize(ar);
}


IMPLEMENT_SERIAL( ArmPointArray, CObject, 0 );