#include "stdafx.h"
#include ".\articulatedarmwrapper.h"
#include "JointValues.h"
#include "NamedJointSet.h"
#include "../DPCMM.h"
#include "../ArmBase.h"
#include "DJointDrive.h"
#include "DFrameDrive.h"
#include "DManageJtSets.h"

ArticulatedArmWrapper::ArticulatedArmWrapper(void)
{
}

ArticulatedArmWrapper::~ArticulatedArmWrapper(void)
{
	ClearJointSets();
}
////////////////////////////////////////////////////////////////
// drive
bool ArticulatedArmWrapper::GetArmTransform(Transform& t)
{
	return app.pA->ReadArmTransform(t);
}
bool ArticulatedArmWrapper::DriveToArmTransform(const Transform& t)
{
	return app.pA->DriveToArmTransform(t);
}
bool ArticulatedArmWrapper::GetArmJointValues(GenVec& jv)
{
	return app.pA->ReadJointValues(jv);
}
bool ArticulatedArmWrapper::DriveToArmJointValues(const GenVec& jv)
{
	return app.pA->DriveToJointValues(jv);
}
bool ArticulatedArmWrapper::DriveToNamedJtSet(const CString& name)
{
	NamedJointSet njs;
	TRACE1("drive to jt set %s...\n", name);
	if(!GetJointSetFromName(name, njs))
		return false;
	JointValues jts;
	jts = njs;
	TRACE0("...which means jt values:\n");
	jts.Debug();
	GenVec gv;
	gv.Create(7);
	jts.GetAsGenVec(gv);
	return app.pA->DriveToJointValues(gv);
}
void ArticulatedArmWrapper::PopJtDrive()
{
	DJointDrive dlg(*this);
	DOMODAL(dlg);
}
void ArticulatedArmWrapper::PopFrameDrive()
{
	DFrameDrive dlg(*this);
	DOMODAL(dlg);
}
void ArticulatedArmWrapper::ManageJtSets()
{
	DManageJtSets dlg(*this);
	DOMODAL(dlg);
}
//////////////////////////////////////////////////////////////////
// utilities
bool ArticulatedArmWrapper::GetJointSetNames(CStringArray& names)
{
	TRACE1("jt set array size = %d\n",JointSetArray.GetSize());
	if(JointSetArray.GetSize() <= 0)
		return false;
	for(int i=0; i<JointSetArray.GetSize(); i++)
	{
		TRACE2("joints %d name = %s\n", i, JointSetArray[i]->name);
		names.Add(JointSetArray[i]->name);
	}
	if(names.GetSize() > 0)
		return true;
	return false;
}
bool ArticulatedArmWrapper::AddNamedJtSet(const GenVec& jv)
{
	//DJointSetNames dlg;
	//CString name;
	//if(DOMODAL(dlg) == IDOK)
	//	name = dlg.newName;
	//else
	//	return false;
	CString name;
	if(!AskUser(_T("Enter a name for the new Joint Set"), &name))
		return false;
	NamedJointSet& njs = *new NamedJointSet;
	if(GetJointSetFromName(name, njs))
	{
		AfxMessageBox(Tr("Already a set with this name, pick a different name."), MB_ICONERROR);
		return false;
	}
	njs.SetFromGenVec(jv);
	njs.name = name;
	if(JointSetArray.Add(&njs) >= 0)
		return true;
	return false;
}
bool ArticulatedArmWrapper::GetJointSetFromName(const CString& name, NamedJointSet& njs) const
{
	if(JointSetArray.GetSize() <= 0)
		return false;
	for(int i=0; i<JointSetArray.GetSize(); i++)
	{
		if(StringHelper::NameCompare(name, JointSetArray[i]->name))
		{
			njs = *JointSetArray[i];
			return true;
		}
	}
	return false;
}
bool ArticulatedArmWrapper::GetJointSetIndexFromName(const CString& name, int& index) const
{
	index = -1;
	for(int i=0; i<JointSetArray.GetSize(); i++)
	{
		if(StringHelper::NameCompare(name, JointSetArray[i]->name))
		{
			index = i;
			return true;
		}
	}
	return false;
}
bool ArticulatedArmWrapper::RemoveJointSet(const int& index)
{
	if(index < 0 || index > JointSetArray.GetUpperBound())
		return false;
	delete JointSetArray[index];
	JointSetArray.RemoveAt(index);
	return true;
}
void ArticulatedArmWrapper::ClearJointSets()
{
	for (int i=0;i<JointSetArray.GetSize();i++)
		delete JointSetArray[i];
	JointSetArray.RemoveAll();
}
/////////////////////////////////////////////////////////////////////
void ArticulatedArmWrapper::Serialize(CArchive& ar)
{	
	JointSetArray.Serialize(ar);

	BYTE Schema = 0;
	BYTE LoadSchema = 0;
	if (ar.IsStoring())
	{ 
		ar << Schema;
	}
	else
	{
		ar >> LoadSchema;
	}
} 
IMPLEMENT_SERIAL( ArticulatedArmWrapper, CObject, 0 );