// C:\_Bruce\@Projects\PCMM Arms\Articulated\DJointDrive.cpp : implementation file
//

#include "stdafx.h"
#include "../DPCMM.h"
#include "DJointDrive.h"
#include ".\djointdrive.h"
#include "ArticulatedArmWrapper.h"

// DJointDrive dialog

IMPLEMENT_DYNAMIC(DJointDrive, CDialog)
DJointDrive::DJointDrive(ArticulatedArmWrapper& _parent)
: parent(_parent)
, jt1(0)
, jt2(0)
, jt3(0)
, jt4(0)
, jt5(0)
, jt6(0)
, jt7(0)
, jt_inc(5)
{
}

DJointDrive::~DJointDrive()
{
}

void DJointDrive::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_JT1, jt1);
	DDX_Text(pDX, IDC_EDIT_JT2, jt2);
	DDX_Text(pDX, IDC_EDIT_JT3, jt3);
	DDX_Text(pDX, IDC_EDIT_JT4, jt4);
	DDX_Text(pDX, IDC_EDIT_JT5, jt5);
	DDX_Text(pDX, IDC_EDIT_JT6, jt6);
	DDX_Text(pDX, IDC_EDIT_JT7, jt7);
	DDX_Text(pDX, IDC_EDIT_INC, jt_inc);
}


BEGIN_MESSAGE_MAP(DJointDrive, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_DEFAULT, OnBnClickedButtonDefault)
	ON_BN_CLICKED(IDC_BUTTON_APPLY_INC, OnBnClickedButtonApplyInc)
	ON_BN_CLICKED(IDC_BUTTON_READ, OnBnClickedButtonRead)
	ON_BN_CLICKED(IDC_BUTTON_WRITE, OnBnClickedButtonWrite)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN1, OnDeltaposSpin1)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN2, OnDeltaposSpin2)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN3, OnDeltaposSpin3)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN4, OnDeltaposSpin4)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN5, OnDeltaposSpin5)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN6, OnDeltaposSpin6)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN7, OnDeltaposSpin7)
	ON_BN_CLICKED(IDC_BUTTON_SAVE_JTS, OnBnClickedButtonSaveJts)
	ON_BN_CLICKED(IDC_BUTTON_MANAGE_JT_SETS, OnBnClickedButtonManageJtSets)
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// utilities...
void DJointDrive::GetValuesAndMakeGenVec(GenVec& jts)
{
	UpdateData();
	jts.x[0] = jt1;
	jts.x[1] = jt2;
	jts.x[2] = jt3;
	jts.x[3] = jt4;
	jts.x[4] = jt5;
	jts.x[5] = jt6;
	jts.x[6] = jt7;
}
void DJointDrive::SetValuesFromGenVec(const GenVec& jts)
{
	jt1 = jts.x[0];
	jt2 = jts.x[1];
	jt3 = jts.x[2];
	jt4 = jts.x[3];
	jt5 = jts.x[4];
	jt6 = jts.x[5];
	jt7 = jts.x[6];
	UpdateData(FALSE);
}
void DJointDrive::GetValuesAndmakeCommaDelimitedString(CString& jts)
{
	UpdateData();
	jts.Format(_T("%f, %f, %f, %f, %f, %f, %f\n"),
		jt1,jt2,jt3,jt4,jt5,jt6,jt7);
}
///////////////////////////////////////////////////////////
// DJointDrive message handlers

BOOL DJointDrive::OnInitDialog()
{
	CDialog::OnInitDialog();

	// set dlg icon to match main dlg...
	SetIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME), FALSE);

	//motors_button.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_STOP));

	OnBnClickedButtonRead();	// display current values

	//CStringArray velocities;
	//if(parent.GetTextForVelocitySet(velocities))
	//	for(int j=0; j<velocities.GetSize(); j++)
	//		velCombo.AddString(velocities.GetAt(j));

	//CString velUnit;
	//parent.GetTextForVelocitySetLabel(velUnit);
	//velSetLabel.SetWindowText(velUnit);

	//velCombo.SetCurSel(velocityIndex);

	TRANSLATE_DIALOG;

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DJointDrive::OnBnClickedButtonDefault()
{
	OnBnClickedButtonApplyInc();
}

void DJointDrive::OnBnClickedButtonApplyInc()
{
	UpdateData();
	//velocityIndex = velCombo.GetCurSel();
	//parent.SetJointVelocity(velocityIndex);
}

void DJointDrive::OnBnClickedButtonRead()
{
	GenVec jts;
	jts.Create(7);
	if(parent.GetArmJointValues(jts))
		SetValuesFromGenVec(jts);
}

void DJointDrive::OnBnClickedButtonWrite()
{
	GenVec jts;
	jts.Create(7);
	GetValuesAndMakeGenVec(jts);
	if(!parent.DriveToArmJointValues(jts))
		AfxMessageBox(_T("Could not drive to joint set."), MB_ICONERROR);
}

void DJointDrive::OnDeltaposSpin1(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt1 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin2(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt2 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin3(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt3 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin4(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt4 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin5(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt5 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin6(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt6 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnDeltaposSpin7(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	UpdateData();
	jt7 -= (double)pNMUpDown->iDelta*jt_inc;
	UpdateData(FALSE);
	*pResult = 0;
}

void DJointDrive::OnBnClickedButtonSaveJts()
{
	// read current values
	GenVec jts;
	jts.Create(7);
	//if(parent.GetArmJointValues(jts))
	//	SetValuesFromGenVec(jts);
	//else
	//	AfxMessageBox(_T("Could not read joints."), MB_ICONERROR);
	GetValuesAndMakeGenVec(jts);

	// tell arm wrapper to prompt for a name and save
	parent.AddNamedJtSet(jts);
}

void DJointDrive::OnBnClickedButtonManageJtSets()
{
	parent.ManageJtSets();
}
