#include "StdAfx.h"
#include ".\jointvalues.h"

JointValues::JointValues(void)
{
	val[0] = 0.0;
	SetAll(0.0);
}

JointValues::JointValues( const JointValues& jnt )
{
	operator=(jnt);
}

JointValues::~JointValues(void)
{
}

void JointValues::SetAll(double value)
{
	for (int i=0;i<num_joints;i++)
		SetValue(i, value);
}

JointValues& JointValues::operator=(const JointValues& jnt)
{
	for (int i=0;i<num_joints;i++)
		SetValue(i,jnt.GetValue(i));
	return *this;
}
void JointValues::operator-=(const JointValues& jnt)
{
	for (int i=0;i<num_joints; i++)
		val[i] -= jnt[i];
}
void JointValues::operator+=(const JointValues& jnt)
{
	for (int i=0;i<num_joints; i++)
		val[i] += jnt[i];
}
void JointValues::operator*=(const double& mult)
{
	for (int i=0;i<num_joints; i++)
		val[i] *= mult;
}
double JointValues::operator[](int idx) const
{
	return GetValue(idx);
}

double JointValues::GetValue(int idx) const
{
	return val[idx];
}
void JointValues::SetValue(int idx, double value)
{
	val[idx] = value;
}
int JointValues::GetSize() const
{
	return num_joints;
}
void JointValues::GetAsGenVec(GenVec& gv)
{
	gv.Create(num_joints);
	for(int i=0; i<num_joints; i++)
		gv.x[i] = val[i];
}
void JointValues::SetFromGenVec(const GenVec& gv)
{
	for(int i=0; i<num_joints; i++)
		val[i] = gv.x[i];
}
void JointValues::GetAsCommaDelimitedCstring(CString& jts)
{
	CString append;
	int i=0;
	for(i=0; i<num_joints-1; i++)
	{
		append.Format(_T("%f, "), val[i]);
		jts += append;
	}
	append.Format(_T("%f\n"), val[i]);
	jts += append;
}
bool JointValues::SetFromCommaDelimitedCString(const CString& jts)
{
	int numEntries=0;
	double v[20];
	numEntries = _stscanf(jts,_T("%lf, %lf, %lf, %lf, %lf, %lf"),&v[0],&v[1],&v[2],&v[3],&v[4],&v[5]);
	if(numEntries != num_joints)
		return false;
	for(int i=0; i<num_joints; i++)
		val[i] = v[i];
	return true;
}
void JointValues::Debug()
{
	TRACE("J[");
	for (int i=0;i<num_joints;i++)
	{
		if (i==num_joints-1)
			TRACE("%f",GetValue(i));
		else
			TRACE("%f, ",GetValue(i));
	}
	TRACE("] \n");
}
void JointValues::Serialize(CArchive& ar)
{	
	CObject::Serialize(ar);

	BYTE Schema = 0;
	BYTE LoadSchema = 0;
	if (ar.IsStoring())
	{ 
		ar << Schema;
		for (int i=0;i<num_joints;i++)
			ar << val[i];
	}
	else
	{
		ar >> LoadSchema;
		for (int i=0;i<num_joints;i++)
			ar >> val[i];
	}
} 
IMPLEMENT_SERIAL( JointValues, CObject, 0 );