// DAvgPointsMeas.cpp : implementation file
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DAvgPointsMeas.h"
#include ".\davgpointsmeas.h"


// DAvgPointsMeas dialog

IMPLEMENT_DYNAMIC(DAvgPointsMeas, DGenMeas)
DAvgPointsMeas::DAvgPointsMeas(CWnd* pParent /*=NULL*/)
	: DGenMeas(DAvgPointsMeas::IDD, pParent)
	, tip(_T(""))
{
	pointsMeasured = 0;
	samplesTaken = 0;
}

DAvgPointsMeas::~DAvgPointsMeas()
{
}

void DAvgPointsMeas::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BUTTON_ONESHORT, B1short_button);
	DDX_Control(pDX, IDC_BUTTON_ONELONG2, B1long_button);
	DDX_Control(pDX, IDC_BUTTON_TWOSHORT, B2short_button);
	DDX_Control(pDX, IDC_BUTTON_TWOLONG, B2long_button);
	DDX_Text(pDX, IDC_TIP, tip);
	DDX_Control(pDX, IDC_STRMINDICATOR, StreamIndicator);
	DDX_Control(pDX, IDC_SAMPLES, pointInfoStatic);
	DDX_Control(pDX, IDC_PROGRESS_SAMPLES, prog_samples);
	DDX_Control(pDX, IDC_STATIC_SAMPLES, samplesStatic);
	DDX_Control(pDX, IDC_CRMSERR, rmsErrStatic);
}


BEGIN_MESSAGE_MAP(DAvgPointsMeas, DGenMeas)
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTON_ONESHORT, OnBnClickedButtonOneshort)
	ON_BN_CLICKED(IDC_BUTTON_ONELONG2, OnBnClickedButtonOnelong2)
	ON_BN_CLICKED(IDC_BUTTON_TWOSHORT, OnBnClickedButtonTwoshort)
	ON_BN_CLICKED(IDC_BUTTON_TWOLONG, OnBnClickedButtonTwolong)
	ON_BN_CLICKED(IDC_DEFAULT, OnBnClickedDefault)
END_MESSAGE_MAP()


// DAvgPointsMeas message handlers

BOOL DAvgPointsMeas::Init()
{
	if(!app.pA->ConnectedToArm)
	{
		B1long_button.ShowWindow(SW_SHOW);
		B1short_button.ShowWindow(SW_SHOW);
		B2long_button.ShowWindow(SW_SHOW);
		B2short_button.ShowWindow(SW_SHOW);
	}
	tip.Format(_T("Tip Dia: %f %s"), 
		app.saarm.Data.TipDia/(app.saarm.units.UnitToMeters[app.saarm.units.distunits]),
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	streaming = TRUE;
	sendfromdlg = TRUE;
	avgptsmeas = TRUE;	// see button hit handler in DGenMeas
	SetStreamIndicator(FALSE);

	prog_samples.SetRange32(0, app.saarm.Data.avgPointSamples);

	DGenMeas::Init();	// call Base class Init for translation, etc.

	return TRUE;
}

void DAvgPointsMeas::ProcessAccept()
{
	if(samplesTaken < app.saarm.Data.avgPointSamples)
		return;
	app.pA->DataArray[0]->Clear();

	CString cs1, cs2;
	cs1.Format(_T("Averaged Point Measurement"));
	cs2.Format(_T("%d samples"),app.saarm.Data.avgPointSamples);
	app.saarm.Data.measCommentArray.RemoveAll();
	app.saarm.Data.measCommentArray.Add(cs1);
	app.saarm.Data.measCommentArray.Add(cs2);

	currentPoint.Scale(1.0/app.saarm.GetInstrumentScaleFactor());
	app.pA->MeasToSpan(currentPoint);
	ClearFields();
	++pointsMeasured;
	CString pointinfo;
	pointinfo.Format(_T("%d Pts Sent"), pointsMeasured);
	pointInfoStatic.SetWindowText(pointinfo);
	//app.pA->IncrementName(Target, app.saarm.Data.ptnameinc);
	//streaming = TRUE;
	ProcessingAccept = FALSE;
}
BOOL DAvgPointsMeas::ProcessNewData(int inc)
{
	PauseMeasTimer();
	//app.pA->ClearButtonState();

	if(inc < 0 && samplesTaken >= app.saarm.Data.avgPointSamples)	// reject the point
	{
		streamstarted = FALSE;
		SetStreamIndicator(FALSE);
		app.pA->DataArray[0]->Clear();
		ClearFields();
	}
	else if(samplesTaken >= app.saarm.Data.avgPointSamples)
	{
		streamstarted = FALSE;
		SetStreamIndicator(FALSE);
		// average the point and get the error info
	/*	Vector Current(app.pA->DataArray[app.pA->StepIndex]->P[0]->x,
			app.pA->DataArray[app.pA->StepIndex]->P[0]->y,
			app.pA->DataArray[app.pA->StepIndex]->P[0]->z);
		currentPoint = Current;*/

		double rmserr;
		app.pA->GetAvgPtAndRMSFromDataArray(0, currentPoint, rmserr);
		rmserr /= app.saarm.units.UnitToMeters[app.saarm.units.distunits];

		Vector CurrWork;
		app.saarm.InstToWorking.Multiply(currentPoint,&CurrWork);
		CurrWork.Scale(1.0/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);
		AvgXText.Format(_T("%6.3f"), CurrWork.x);
		AvgYText.Format(_T("%6.3f"), CurrWork.y);
		AvgZText.Format(_T("%6.3f"), CurrWork.z);
		CString rmstxt;
		rmstxt.Format(_T("RMS Err = %3.6f"), rmserr);
		rmsErrStatic.SetWindowText(rmstxt);
		UpdateFields();
	}
	else
	{
		samplesTaken = app.pA->AddPoint()+1;
		UpdateSamplesInfo();
	}
	ResumeMeasTimer();
	return TRUE;
}

void DAvgPointsMeas::UpdateSamplesInfo()
{
	prog_samples.SetPos(samplesTaken);
	CString s;
	s.Format(_T("%d"), samplesTaken);
	samplesStatic.SetWindowText(s);
}

void DAvgPointsMeas::ClearFields()
{
	samplesTaken = 0;
	UpdateSamplesInfo();
	AvgXText.Format(_T(""));
	AvgYText.Format(_T(""));
	AvgZText.Format(_T(""));
	CString rmstxt;
	rmstxt.Format(_T("RMS Error: "));
	rmsErrStatic.SetWindowText(rmstxt);
	UpdateFields();
}
void DAvgPointsMeas::UpdateFields()
{
	UpdateData(FALSE);
	SetDlgItemText(IDC_AVG_X,AvgXText);
	SetDlgItemText(IDC_AVG_Y,AvgYText);
	SetDlgItemText(IDC_AVG_Z,AvgZText);
}

void DAvgPointsMeas::SetStreamIndicator(BOOL strm)
{
	if(strm)
	{
		TRACE0("Setting stream indicator ON\n");
		StreamIndicator.SetBkColor(RGB(0,255,0));
		StreamIndicator.SetTextColor(RGB(0,0,0));
		StreamIndicator.SetWindowText(_T("ON"));
	}
	else
	{
		TRACE0("Setting stream indicator OFF\n");
		StreamIndicator.SetBkColor(RGB(255,0,0));
		StreamIndicator.SetTextColor(RGB(0,0,0));
		StreamIndicator.SetWindowText(_T("OFF"));
	}
}

///////////////////////////////////////////////////////////////////////////
// simulation mode

BOOL DAvgPointsMeas::MakeFakeData(int step)
{
	if(app.StopNow)
		return FALSE;			

	app.pA->X = ((double)rand())/((double)RAND_MAX);
	app.pA->Y = ((double)rand())/((double)RAND_MAX);
	app.pA->Z = (-(double)rand())/((double)RAND_MAX);

	app.pA->CurrentTransform.SetTranslation(Vector(app.pA->X, app.pA->Y, app.pA->Z));

	return TRUE;
}

void DAvgPointsMeas::OnShowWindow(BOOL bShow, UINT nStatus)
{
	DGenMeas::OnShowWindow(bShow, nStatus);

	if (bShow)
		PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_Arms_DAvgPointsMeas"),this);	
}

void DAvgPointsMeas::OnDestroy()
{
	PersistentWnd::SaveWndRect(_T("windows"),_T("SA_Arms_DAvgPointsMeas"),this);

	DGenMeas::OnDestroy();
}

void DAvgPointsMeas::OnBnClickedButtonOneshort()
{
	SetB1Short();
}

void DAvgPointsMeas::OnBnClickedButtonOnelong2()
{
	SetB1Long();
}

void DAvgPointsMeas::OnBnClickedButtonTwoshort()
{
	SetB2Short();
}

void DAvgPointsMeas::OnBnClickedButtonTwolong()
{
	SetB2Long();
}

void DAvgPointsMeas::OnBnClickedDefault()
{
	
}
