// DConeMeas.cpp : implementation file
//

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "stdafx.h"
#include "DPCMM.h"
#include "DConeMeas.h"

// DConeMeas dialog

DConeMeas::DConeMeas(CWnd* pParent /*=NULL*/)
	: DGenMeas(DConeMeas::IDD, pParent)
	, tip(_T(""))
	, sampleinfo(_T(""))
	, apexInfo(_T(""))
	, includedAngleInfo(_T(""))
	, maxErrInfo(_T(""))
	, rmsErrInfo(_T(""))
	, fs(*new FitSummary)
	, axisInfo(_T(""))
{
	haveFakeApexForSim = false;
	validFit = false;
}

void DConeMeas::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDOK, OKButton);
	DDX_Control(pDX, IDC_BUTTON_TWOSHORT, B2short_button);
	DDX_Control(pDX, IDC_BUTTON_TWOLONG, B2long_button);
	DDX_Control(pDX, IDC_BUTTON_ONESHORT, B1short_button);
	DDX_Control(pDX, IDC_BUTTON_ONELONG, B1long_button);
	DDX_Control(pDX, IDC_STATIC_EVAL, eval_message);
	DDX_Text(pDX, IDC_TIP_CONE, tip);
	DDX_Text(pDX, IDC_SAMPLES_CONE, sampleinfo);
	DDX_Text(pDX, IDC_APEX_CONE, apexInfo);
	DDX_Text(pDX, IDC_INC_ANGLE_CONE, includedAngleInfo);
	DDX_Text(pDX, IDC_MAXERR_CONE, maxErrInfo);
	DDX_Text(pDX, IDC_RMSERR_CONE, rmsErrInfo);
	DDX_Text(pDX, IDC_AXIS_CONE, axisInfo);
}


BEGIN_MESSAGE_MAP(DConeMeas, DGenMeas)
	ON_BN_CLICKED(IDC_BUTTON_ONESHORT, &DConeMeas::OnBnClickedButtonOneshort)
	ON_BN_CLICKED(IDC_BUTTON_ONELONG, &DConeMeas::OnBnClickedButtonOnelong)
	ON_BN_CLICKED(IDC_BUTTON_TWOSHORT, &DConeMeas::OnBnClickedButtonTwoshort)
	ON_BN_CLICKED(IDC_BUTTON_TWOLONG, &DConeMeas::OnBnClickedButtonTwolong)
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDOK, &DConeMeas::OnBnClickedOk)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// DConeMeas message handlers

BOOL DConeMeas::Init()
{
	if(!app.pA->ConnectedToArm)
	{
		B1long_button.ShowWindow(SW_SHOW);
		B1short_button.ShowWindow(SW_SHOW);
		B2long_button.ShowWindow(SW_SHOW);
		B2short_button.ShowWindow(SW_SHOW);
	}
	OKButton.EnableWindow(FALSE);
	tip.Format(_T("Tip Dia: %f %s"), 
		app.saarm.Data.TipDia/(app.saarm.units.UnitToMeters[app.saarm.units.distunits]),
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	
	apexInfo.Format(_T("Apex"));
	axisInfo.Format(_T("Axis"));
	includedAngleInfo.Format(_T("Included Angle"));
	maxerrinfo.Format(_T("Max Error"));
	rmserrinfo.Format(_T("RMS Error"));
	sampleinfo.Format(_T("Samples: (12 required)"));
	OKButton.EnableWindow(FALSE);
	enoughdata = FALSE;
	
	SetGTNames();

	DGenMeas::Init();	// call Base class Init for translation, etc.

	return TRUE;
}
void DConeMeas::SetGTNames()
{
	Group = app.saarm.Data.Group;
	Target = app.saarm.Data.Target;
	UpdateData(FALSE);
}

void DConeMeas::ProcessAccept()
{
	if(!enoughdata)
	{
		app.saarm.TTS.TextToSpeech(_T("Not enough data to fit."));
		ProcessingAccept = FALSE;
		return;
	}
	if(!validFit)
		return;

	///////////////////////////////////////////////////////////////////
	// do the report stuff
	name.Format(_T("%s_%s"), Group,Target);

	CString s;
	notes.RemoveAll();

	s.Format(_T("User: %s"), app.saarm.Data.UserName);//, app.saarm.Data.SpanPort);
	notes.Add(s);

	s.Format(_T("Cone fit %s  %d samples"), name,
		app.pA->DataArray[app.pA->StepIndex]->P.GetSize());
	notes.Add(s);

	CStringArray statres;
	CStringArray ptres;
	CStringArray geomres;
	CStringArray inputparams;
	CStringArray ptsused;
	CString frameandunits;
	app.saarm.Data.cf.ReportStats(&statres);
	app.saarm.Data.cf.ReportPointResults(&ptres);
	app.saarm.Data.cf.ReportGeom(&geomres);
	app.saarm.Data.cf.ReportInputParams(&inputparams);
	app.saarm.Data.cf.ReportPtsUsed(&ptsused);
	app.saarm.Data.cf.ReportWorkingFrameAndUnits(&frameandunits);

	PhCone& cone = *((PhCone*)app.saarm.Data.cf.GetGeometry());

	s.Format(_T("Included Angle = %f deg."), cone.includedAngle);
	notes.Add(s);

	Vector Apex, ApexDisplay;
	cone.t.GetTranslation(&Apex);
	app.saarm.InstToWorking.Multiply(Apex, &ApexDisplay);
	ApexDisplay.Scale(1.0/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);
	s.Format(_T("Vertex: %f, %f, %f %s"), ApexDisplay.x, ApexDisplay.y, ApexDisplay.z,
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	notes.Add(s);

	Vector zaxis = cone.t.GetZAxis();
	Vector zaxisw;
	app.saarm.InstToWorking.MultiplyRot(zaxis, &zaxisw);
	s.Format(_T("Axis: %f %f %f"), zaxisw.x, zaxisw.y, zaxisw.z);
	notes.Add(s);

	//TRACE0("");
	//TRACE0("*****GEOMETRY RESULTS*****\n");
	//for(INT_PTR i=0; i<geomres.GetSize(); i++ )
	//{
	//	TRACE1("%s\n", geomres.GetAt(i));
	//	notes.Add(geomres.GetAt(i));
	//}
	TRACE0("*****STAT RESULTS*****\n");
	for(INT_PTR i=0; i<statres.GetSize(); i++ )
	{
		TRACE1("%s\n", statres.GetAt(i));
		notes.Add(statres.GetAt(i));
	}
	TRACE0("*****POINT RESULTS*****\n");
	for(INT_PTR i=0; i<ptres.GetSize(); i++ )
	{
		TRACE1("%s\n", ptres.GetAt(i));
	}
	TRACE0("*****INPUT PARAMS*****\n");
	for(INT_PTR i=0; i<inputparams.GetSize(); i++ )
	{
		TRACE1("%s\n", inputparams.GetAt(i));
		notes.Add(inputparams.GetAt(i));
	}
	TRACE0("*****PTS USED*****\n");
	for(INT_PTR i=0; i<ptsused.GetSize(); i++ )
	{
		TRACE1("%s\n", ptsused.GetAt(i));
	}
	TRACE0("*****WORKING FRAME AND UNITS*****\n");
	TRACE1("%s\n", frameandunits);
	TRACE0("");
	///////////////////////////////////////////////////////////

	app.saarm.Data.measCommentArray.RemoveAll();
	app.saarm.Data.measCommentArray.Append(notes);

	CString name, tName;
	if(app.saarm.Data.geomName.IsEmpty())
	{
		name.Format(_T("%s_%s"), app.saarm.Data.Group, app.saarm.Data.Target);
	}
	else
	{
		name = app.saarm.Data.geomName;
		app.pA->IncrementName(app.saarm.Data.geomName);
	}
	tName.Format(_T("%s"), app.saarm.Data.Target);
	if( app.saarm.Data.SendGeom )
	{
		app.saarm.SockGeom.SendCone(cone.t, cone.includedAngle, app.saarm.Data.cf.graphicalLength, name, notes);
	}

	VectorNamedArray cpts;
	if(app.saarm.Data.cf.GetCardinalPts(true, cpts))
	{
		for(int i=0; i<cpts.GetSize(); i++)
		{
			cpts.P[i]->Scale(app.saarm.units.UnitToMeters[app.saarm.units.distunits]);
		}
		if(cpts.GetSize() == 2)
		{
			CString tg;
			tg.Format(_T("Vertex"));
			app.pA->MeasToSpan(cpts.Get(0), app.saarm.Data.Group, tg, true, false);
			tg.Format(_T("Pt. On Axis"));
			app.pA->MeasToSpan(cpts.Get(1), app.saarm.Data.Group, tg, true, false);
		}
	}

	if( app.saarm.Data.FirePointsToo )
	{
		CString tempGName = app.saarm.Data.Group;
		app.saarm.Data.Group.Format(_T("%s_%s-Meas"), tempGName, tName);
		app.pA->ScaleDataArrays(1.0/app.saarm.GetInstrumentScaleFactor());
		app.pA->MeasToSpan(app.pA->DataArray[0], tName);
		app.saarm.Data.Group = tempGName;
	}

	if(app.saarm.Data.incrementgroup)
	{
		app.saarm.Data.Target.Format(_T("P1"));
		app.pA->IncrementName(app.saarm.Data.Group, 1);
	}

	app.saarm.Data.cf.ClearData();
	validFit = false;

	app.pA->SetDisplayData();
	haveFakeApexForSim = false;

	app.pA->DataArray[0]->Clear();
	ctr = -1;
	ProcessNewData(0);

	ProcessingAccept = FALSE;
}
BOOL DConeMeas::ProcessNewData(int inc)
{
	sampleinfo.Format(_T("%d Samples"), ctr+1);
	app.pA->IncrementName(Target, inc);

	if( app.pA->DataArray[app.pA->StepIndex]->P.GetSize() < 12 )//|| inc <= 0 )
	{
		apexInfo.Format(_T("Vertex"));
		axisInfo.Format(_T("Axis"));
		includedAngleInfo.Format(_T("Included Angle"));
		maxerrinfo.Format(_T("Max Error"));
		rmserrinfo.Format(_T("RMS Error"));
		OKButton.EnableWindow(FALSE);
		enoughdata = FALSE;
		if(app.pA->DataArray[app.pA->StepIndex]->P.GetSize() <= 0)
			sampleinfo.Format(_T("Samples: (12 required)"));
		UpdateData(FALSE);
		return FALSE;
	}
	enoughdata = TRUE;
	OKButton.EnableWindow(TRUE);

	PauseMeasTimer();
	eval_message.SetWindowText(_T("Fitting Cone, Please Wait"));

	if(app.saarm.Data.geomName.IsEmpty())
	{
		name.Format(_T("%s_%s"), Group, Target);
	}
	else
	{
		name = app.saarm.Data.geomName;
		app.pA->IncrementName(app.saarm.Data.geomName);
	}

	app.saarm.Data.cf.ClearData();
	Vector AveDir = app.pA->ComputeAvgDirection();
	double offsetInches = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	if(!app.pA->ConnectedToArm && !haveFakeApexForSim)
	{
		Vector apex(0.7, 0.0, 0.0);
		app.saarm.Data.cf.AddData(Target, apex, offsetInches, offsetInches);
		haveFakeApexForSim = true;
	}
	for( int i=0; i<app.pA->DataArray[0]->P.GetSize(); i++)
	{
		Vector pt(*app.pA->DataArray[0]->P[i]);
		pt.Scale(app.saarm.LocalToInches);
		TRACE1("%s, ", name);
		pt.Debug();
		app.saarm.Data.cf.AddData(name, pt, offsetInches, offsetInches);
	}

	TRACE("tip dia = %f in\n", app.saarm.Data.TipDia*app.saarm.LocalToInches);
	
	if( !app.saarm.Data.cf.Fit(TRUE, &fs) )
	{
		validFit = false;
		eval_message.SetWindowText(app.saarm.Data.cf.GetLastError());

		UpdateData(FALSE);
		ResumeMeasTimer();
		return FALSE;
	}
	validFit = true;
	
	app.saarm.Data.cf.geom.t.Scale(app.saarm.units.UnitToMeters[app.saarm.units.distunits]);
	app.saarm.Data.cf.graphicalLength *= app.saarm.units.UnitToMeters[app.saarm.units.distunits];
	PhCone& cone = *((PhCone*)app.saarm.Data.cf.GetGeometry());

	Vector Apex, ApexDisplay;
	cone.t.GetTranslation(&Apex);
	app.saarm.InstToWorking.Multiply(Apex, &ApexDisplay);
	ApexDisplay.Scale(1.0/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);

	Vector zaxis = cone.t.GetZAxis();
	Vector zaxisw;
	app.saarm.InstToWorking.MultiplyRot(zaxis, &zaxisw);
	axisInfo.Format(_T("Axis: %f %f %f"), zaxisw.x, zaxisw.y, zaxisw.z);

	apexInfo.Format(_T("Vertex: %f, %f, %f %s"), ApexDisplay.x, ApexDisplay.y, ApexDisplay.z,
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);

	includedAngleInfo.Format(_T("Included Angle: %f %s"), cone.includedAngle, app.saarm.units.DistUnitNames[app.saarm.units.distunits]);

	rmsErrInfo.Format(_T("RMS Err %f"), fs.rms);
	maxErrInfo.Format(_T("Max Err %f"), fs.absmax);

	UpdateData(FALSE);
	eval_message.SetWindowText(_T(""));
	ResumeMeasTimer();
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////
// simulation mode
BOOL DConeMeas::MakeFakeData(int step)
{
	TRACE0("Make fake cone data!!\n");
	Vector XYZ(0.7, 0.7, 0.0), newXYZ;
	double rho, theta, phi;

	ProcessWaitingMessages();
	Delay(0.0005);

	if(app.StopNow)
		return FALSE;			

	rho = 0.2;		// the radius of the circle!!
	phi = 180.0*((double)rand())/((double)RAND_MAX);
	theta = 0.0;

	Vector dhv(rho, theta, phi);
	Vector xyz;
	dhv.RTPtoXYZ(&xyz);

	newXYZ = XYZ;
	newXYZ.Add(xyz);
	app.pA->X = newXYZ.x;
	app.pA->Y = newXYZ.y;
	app.pA->Z = newXYZ.z;

	app.pA->CurrentTransform.SetTranslation(Vector(app.pA->X, app.pA->Y, app.pA->Z));

	return TRUE;
}

void DConeMeas::OnBnClickedButtonOneshort()
{
	SetB1Short();
}

void DConeMeas::OnBnClickedButtonOnelong()
{
	SetB1Long();
}

void DConeMeas::OnBnClickedButtonTwoshort()
{
	SetB2Short();
}

void DConeMeas::OnBnClickedButtonTwolong()
{
	SetB2Long();
}

void DConeMeas::OnShowWindow(BOOL bShow, UINT nStatus)
{
	DGenMeas::OnShowWindow(bShow, nStatus);

	if (bShow)
		PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_Arms_DConeMeas"),this);
}

void DConeMeas::OnDestroy()
{
	PersistentWnd::SaveWndRect(_T("windows"),_T("SA_Arms_DConeMeas"),this);

	delete &fs;

	DGenMeas::OnDestroy();
}

void DConeMeas::OnBnClickedOk()
{
	ProcessAccept();
}
