// DFitOptions.cpp : implementation file
//

#include "stdafx.h"
#include "nrkapp.h"
#include "DFitOptions.h"
#include ".\dfitoptions.h"
#include "GeomTriggerWrapper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DFitOptions dialog


DFitOptions::DFitOptions(CWnd* pParent /*=NULL*/)
	: CDialog(DFitOptions::IDD, pParent)
	, scan_spatial(FALSE)
	, scan_spatial_inc(0)
	, m_Units3(_T(""))
	, cloudDataInterval(0.25)
	, m_proxThresh(0)
	, m_Units4(_T(""))
	, avgPointSamples(50)
	, lineMeasType(1)
	, autoIncCloudName(FALSE)
	, thinningFactor(1)
	, proxWorkingField(0)
	, units5(_T(""))
	, startUpMeasMode(0)
	, startUpInMeasMode(FALSE)
	, UseScannerForAutoProxMeas(FALSE)
{
	//{{AFX_DATA_INIT(DFitOptions)
	m_LockCircles = FALSE;
	m_LockSpheres = FALSE;
	m_SphereLockVal = 0.0;
	m_CircleLockVal = 0.0;
	m_Units1 = _T("");
	m_Units2 = _T("");
	//}}AFX_DATA_INIT
}


void DFitOptions::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DFitOptions)
	DDX_Check(pDX, IDC_CIRCLOCKCHECK, m_LockCircles);
	DDX_Check(pDX, IDC_SPHERELOCKCHECK, m_LockSpheres);
	DDX_Text(pDX, IDC_SPHEREEDIT, m_SphereLockVal);
	DDX_Text(pDX, IDC_CIRCEDIT, m_CircleLockVal);
	DDX_Text(pDX, IDC_UNIT1STATIC, m_Units1);
	DDX_Text(pDX, IDC_UNIT2STATIC, m_Units2);
	//}}AFX_DATA_MAP
	DDX_Radio(pDX, IDC_RADIO1, scan_spatial);
	DDX_Text(pDX, IDC_EDIT_SPAT_INC, scan_spatial_inc);
	DDX_Text(pDX, IDC_UNIT3STATIC, m_Units3);
	DDX_Control(pDX, IDC_EDIT_SPAT_INC, scan_spatial_inc_Edit);
	DDX_Text(pDX, IDC_EDIT_CLOUD_DATA_INTERVAL, cloudDataInterval);
	DDX_Text(pDX, IDC_PROXEDIT, m_proxThresh);
	DDX_Text(pDX, IDC_UNIT1STATIC2, m_Units4);
	DDX_Text(pDX, IDC_EDIT_AVG_PT_SAMPLES, avgPointSamples);
	DDX_Radio(pDX, IDC_RADIO_EDGE_LINE, lineMeasType);
	DDX_Check(pDX, IDC_CHECK_AUTO_INC_CLOUD_NAME, autoIncCloudName);
	DDX_Control(pDX, IDC_BUTTON_SCANNER_SETTINGS, scanner_settings_button);
	DDX_Text(pDX, IDC_EDIT_SCANNER_THINNING_FACTOR, thinningFactor);
	DDX_Text(pDX, IDC_PROXFIELDEDIT, proxWorkingField);
	DDX_Text(pDX, IDC_UNIT1STATIC3, units5);
	DDX_CBIndex(pDX, IDC_COMBO_STARTUP_MEAS_MODE, startUpMeasMode);
	DDX_Check(pDX, IDC_CHECK_STARTUP_IN_MEAS_MODE, startUpInMeasMode);
	DDX_Control(pDX, IDC_COMBO_STARTUP_MEAS_MODE, startUpMeasModeCombo);
	DDX_Check(pDX, IDC_CHECK_USE_SCANNER_FOR_AUTO_PROX, UseScannerForAutoProxMeas);
}


BEGIN_MESSAGE_MAP(DFitOptions, CDialog)
	//{{AFX_MSG_MAP(DFitOptions)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_RADIO1, OnBnClickedRadioTemp)
	ON_BN_CLICKED(IDC_RADIO2, OnBnClickedRadioSpat)
	ON_BN_CLICKED(IDC_BUTTON_SCANNER_SETTINGS, OnBnClickedButtonScannerSettings)
	ON_BN_CLICKED(IDC_CHECK_STARTUP_IN_MEAS_MODE, &DFitOptions::OnBnClickedCheckStartupInMeasMode)
	ON_BN_CLICKED(IDC_BUTTON_SET_GEOM_TRIGGERS, &DFitOptions::OnBnClickedButtonSetGeomTriggers)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DFitOptions message handlers

void DFitOptions::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	if (bShow)
		PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_Arms_DFitOptions"),this);	
}

void DFitOptions::OnDestroy()
{
	PersistentWnd::SaveWndRect(_T("windows"),_T("SA_Arms_DFitOptions"),this);

	CDialog::OnDestroy();
}
void DFitOptions::CheckSpatRadioStatus()
{
	if(scan_spatial)
		scan_spatial_inc_Edit.EnableWindow();
	else
		scan_spatial_inc_Edit.EnableWindow(FALSE);
}
void DFitOptions::OnBnClickedRadioTemp()
{
	scan_spatial_inc_Edit.EnableWindow(FALSE);
}

void DFitOptions::OnBnClickedRadioSpat()
{
	scan_spatial_inc_Edit.EnableWindow();
}

BOOL DFitOptions::OnInitDialog()
{
	CDialog::OnInitDialog();

	if(!app.pA->IsFaroUSB() && !app.pA->IsKreonScannerArm())
		scanner_settings_button.ShowWindow(SW_HIDE);

	SetIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME), FALSE);
	TRANSLATE_DIALOG;

	UpdateData(FALSE);

	CheckSpatRadioStatus();
	OnBnClickedCheckStartupInMeasMode();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DFitOptions::OnBnClickedButtonScannerSettings()
{
	app.pA->ScannerSettings();
}

void DFitOptions::OnBnClickedCheckStartupInMeasMode()
{
	UpdateData();
	if(startUpInMeasMode)
		startUpMeasModeCombo.EnableWindow(TRUE);
	else
		startUpMeasModeCombo.EnableWindow(FALSE);
}

void DFitOptions::OnBnClickedButtonSetGeomTriggers()
{
	app.saarm.geomTriggerWrap.SetGeomTriggers();

	while(app.saarm.geomTriggerWrap.WantSelectedPlanes())
	{
		app.saarm.SelectMultiplePlanesFromSA();
		app.saarm.geomTriggerWrap.TurnOffPlaneSelection();
		app.saarm.geomTriggerWrap.SetGeomTriggers();
	}

	while(app.saarm.geomTriggerWrap.WantSelectedCylinders())
	{
		app.saarm.SelectMultipleCylindersFromSA();
		app.saarm.geomTriggerWrap.TurnOffCylinderSelection();
		app.saarm.geomTriggerWrap.SetGeomTriggers();
	}

	while(app.saarm.geomTriggerWrap.WantSelectedSpheres())
	{
		app.saarm.SelectMultipleSpheresFromSA();
		app.saarm.geomTriggerWrap.TurnOffSphereSelection();
		app.saarm.geomTriggerWrap.SetGeomTriggers();
	}
}
