// DFrameMeas.cpp : implementation file
//

#include "stdafx.h"
#include "nrkapp.h"
#include "DFrameMeas.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// DFrameMeas dialog


DFrameMeas::DFrameMeas(CWnd* pParent /*=NULL*/)
	: DGenMeas(DFrameMeas::IDD, pParent)
{
	//{{AFX_DATA_INIT(DFrameMeas)
	name = _T("");
	CurrentText = _T("");
	LastText = _T("");
	SampleText = _T("");
	TipText = _T("");
	//}}AFX_DATA_INIT
}


void DFrameMeas::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DFrameMeas)
	DDX_Control(pDX, IDC_BUTTON_TWOSHORT, B2short_button);
	DDX_Control(pDX, IDC_BUTTON_TWOLONG, B2long_button);
	DDX_Control(pDX, IDC_BUTTON_ONESHORT, B1short_button);
	DDX_Control(pDX, IDC_BUTTON_ONELONG, B1long_button);
	DDX_Text(pDX, IDC_NAMEEDIT, name);
	DDX_Text(pDX, IDC_CURRENT, CurrentText);
	DDX_Text(pDX, IDC_LAST, LastText);
	DDX_Text(pDX, IDC_SAMPLES, SampleText);
	DDX_Text(pDX, IDC_TIP, TipText);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DFrameMeas, DGenMeas)
	//{{AFX_MSG_MAP(DFrameMeas)
	ON_BN_CLICKED(IDC_BUTTON_ONELONG, OnButtonOnelong)
	ON_BN_CLICKED(IDC_BUTTON_ONESHORT, OnButtonOneshort)
	ON_BN_CLICKED(IDC_BUTTON_TWOLONG, OnButtonTwolong)
	ON_BN_CLICKED(IDC_BUTTON_TWOSHORT, OnButtonTwoshort)
	ON_BN_CLICKED(IDC_DEFAULT, OnDefault)
	//}}AFX_MSG_MAP
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DFrameMeas message handlers

BOOL DFrameMeas::Init()
{
	app.pA->ClearVectorArray();	// DGenMeas creates a new array for data, this mode doesn't use it

	if(!app.pA->ConnectedToArm)
	{
		B1long_button.ShowWindow(SW_SHOW);
		B1short_button.ShowWindow(SW_SHOW);
		B2long_button.ShowWindow(SW_SHOW);
		B2short_button.ShowWindow(SW_SHOW);
	}
	TipText.Format(_T("Tip Dia: %f %s"), 
		app.saarm.Data.TipDia/(app.saarm.units.UnitToMeters[app.saarm.units.distunits]),
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	sendfromdlg = TRUE;
	sendupdates = TRUE;
	framemode = TRUE;
	name = app.saarm.Data.Target;	// 2013.11.11 - Mantis #6567
	UpdateData(FALSE);

	DGenMeas::Init();	// call Base class Init for translation, etc.

	return TRUE;
}

void DFrameMeas::ProcessAccept()
{
	ProcessingAccept = FALSE;
	BaseOnOK();
}
BOOL DFrameMeas::ProcessNewData(int inc)
{
	app.pA->ClearButtonState();
	
	if( inc < 0 )
		return FALSE;
	if( inc == 0 )
	{
		if(app.pA->FrameUpdateRequested)
		{
//			app.pA->UpdateToSpan();
//			app.pA->FrameUpdateToSpan();
			app.pA->FrameandJointUpdateToSpan();
			app.pA->FrameUpdateRequested = FALSE;
		}
		return TRUE;
	}

	UpdateData(TRUE);
	LastText = CurrentText;
	ProcessUpdate();
	UpdateData(FALSE);
	SetDlgItemText(IDC_SAMPLES,SampleText);
	SetDlgItemText(IDC_TIP,TipText);
	SetDlgItemText(IDC_CURRENT,CurrentText);
	SetDlgItemText(IDC_LAST,LastText);

	app.pA->FrameToSpan(name, FALSE);
	app.pA->IncrementName(name);

	UpdateData(FALSE);
	return TRUE;
}
void DFrameMeas::ProcessUpdate()
{
	SampleText.Format(_T("%d Frames Recorded"),ctr+1);
	TipText.Format(_T("Tip = %5.2f"),app.saarm.Data.TipDia);
	Vector Current(app.pA->X,app.pA->Y,app.pA->Z); //in meters
	Vector CurrWork;
	Transform TWork;
	app.saarm.InstToWorking.Multiply(Current,&CurrWork);
	TWork.Multiply(app.pA->CurrentTransform,app.saarm.InstToWorking);
	Vector VOrient;
	TWork.GetRotation(&VOrient);

	CurrentText.Format(_T("%6.*f %6.*f %6.*f\n%6.*f %6.*f %6.*f"),
		app.saarm.Data.Precision,CurrWork.x/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.Data.Precision,CurrWork.y/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.Data.Precision,CurrWork.z/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.Data.Precision,VOrient.x,
		app.saarm.Data.Precision,VOrient.y,
		app.saarm.Data.Precision,VOrient.z);
	UpdateData(FALSE);
	SetDlgItemText(IDC_SAMPLES,SampleText);
	SetDlgItemText(IDC_TIP,TipText);
	SetDlgItemText(IDC_CURRENT,CurrentText);
	SetDlgItemText(IDC_LAST,LastText);

	CString note;
	notes.RemoveAll();
	note.Format(_T("Frame from %s interface"), app.saarm.CMMString);
	notes.Add(note);
}

///////////////////////////////////////////////////////////////////////////
// simulation mode

BOOL DFrameMeas::MakeFakeData(int step)
{
	if(app.StopNow)
		return FALSE;			
	
	app.pA->X = ((double)rand())/((double)RAND_MAX);
	app.pA->Y = ((double)rand())/((double)RAND_MAX);
	app.pA->Z = -0.3;

	app.pA->CurrentTransform.SetRotation(Vector(0.0,0.0,-1.0));
	app.pA->CurrentTransform.SetTranslation(Vector(app.pA->X,app.pA->Y,app.pA->Z));

	return TRUE;
}

void DFrameMeas::OnButtonOnelong() 
{
	SetB1Long();
}

void DFrameMeas::OnButtonTwolong() 
{
//	SetB2Long();
}

void DFrameMeas::OnButtonTwoshort() 
{
//	SetB2Short();
}

void DFrameMeas::OnButtonOneshort() 
{
	SetB1Short();
}


void DFrameMeas::OnDefault() 
{
	UpdateData(TRUE);
}

void DFrameMeas::OnShowWindow(BOOL bShow, UINT nStatus)
{
	DGenMeas::OnShowWindow(bShow, nStatus);

	if (bShow)
		PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_Arms_DFrameMeas"),this);	
}

void DFrameMeas::OnDestroy()
{
	PersistentWnd::SaveWndRect(_T("windows"),_T("SA_Arms_DFrameMeas"),this);

	DGenMeas::OnDestroy();
}
