// DGeomTriggerPts.cpp : implementation file
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DGeomTriggerPts.h"
#include "GeomTriggerWrapper.h"

// DGeomTriggerPts dialog

DGeomTriggerPts::DGeomTriggerPts(CWnd* pParent /*=NULL*/)
	: DGenMeas(DGeomTriggerPts::IDD, pParent)
	, tip(_T(""))
	, sampleinfo(_T(""))
{
	groupInc.Icon = IDI_GROUPINCICON;
	targInc.Icon = IDI_TARGETINCICON;
	refPosSet = false;
}


void DGeomTriggerPts::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_STRMINDICATOR_GT, StreamIndicator);
	DDX_Text(pDX, IDC_TIP_GT, tip);
	DDX_Text(pDX, IDC_SAMPLES_GT, sampleinfo);
	DDX_Control(pDX, IDC_BUTTON_ONESHORT_GT, B1short_button);
	DDX_Control(pDX, IDC_BUTTON_ONELONG_GT, B1long_button);
	DDX_Control(pDX, IDC_BUTTON_TWOSHORT_GT, B2short_button);
	DDX_Control(pDX, IDC_BUTTON_TWOLONG_GT, B2long_button);
	DDX_Control(pDX, IDC_INCG_GT, groupInc);
	DDX_Control(pDX, IDC_INCT_GT, targInc);
}


BEGIN_MESSAGE_MAP(DGeomTriggerPts, DGenMeas)
	ON_BN_CLICKED(IDC_BUTTON_ONESHORT_GT, &DGeomTriggerPts::OnBnClickedButtonOneshortGt)
	ON_BN_CLICKED(IDC_BUTTON_ONELONG_GT, &DGeomTriggerPts::OnBnClickedButtonOnelongGt)
	ON_BN_CLICKED(IDC_BUTTON_TWOSHORT_GT, &DGeomTriggerPts::OnBnClickedButtonTwoshortGt)
	ON_BN_CLICKED(IDC_BUTTON_TWOLONG_GT, &DGeomTriggerPts::OnBnClickedButtonTwolongGt)
	ON_BN_CLICKED(IDC_INCG_GT, &DGeomTriggerPts::OnBnClickedIncgGt)
	ON_BN_CLICKED(IDC_INCT_GT, &DGeomTriggerPts::OnBnClickedInctGt)
	ON_WM_SHOWWINDOW()
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_DEFAULT, &DGeomTriggerPts::OnBnClickedDefault)
END_MESSAGE_MAP()


BOOL DGeomTriggerPts::Init()
{
	if(!app.pA->ConnectedToArm)
	{
		B1long_button.ShowWindow(SW_SHOW);
		B1short_button.ShowWindow(SW_SHOW);
		B2long_button.ShowWindow(SW_SHOW);
		B2short_button.ShowWindow(SW_SHOW);
	}
	tip.Format(_T("Tip Dia: %f %s"), 
		app.saarm.Data.TipDia/(app.saarm.units.UnitToMeters[app.saarm.units.distunits]),
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	streaming = TRUE;
	sendfromdlg = TRUE;
	SetGTNames();
	SetStreamIndicator(FALSE);
	CString title;
	title.Format(_T("Geometry Trigger Measurement"));
	app.saarm.Data.measCommentArray.RemoveAll();
	app.saarm.Data.measCommentArray.Add(title);
	SetWindowText(title);

	DGenMeas::Init();	// call Base class Init for translation, etc.

	return TRUE;
}
void DGeomTriggerPts::SetGTNames()
{
	Group = app.saarm.Data.Group;
	Target = app.saarm.Data.Target;
	UpdateData(FALSE);
}

void DGeomTriggerPts::ProcessAccept()
{
	name.Format(_T("%s_%s"), Group,Target);

	CString s;
	notes.RemoveAll();
	s.Format(_T("User: %s"), app.saarm.Data.UserName);//, app.saarm.Data.SpanPort);
	notes.Add(s);

	DGenMeas::OnOK();
}
BOOL DGeomTriggerPts::ProcessNewData(int inc)
{
	PauseMeasTimer();
	Vector CurrentPos(app.pA->X,app.pA->Y,app.pA->Z);
	Vector CurrentNorm(app.pA->II,app.pA->JJ,app.pA->KK);
	//CurrentPos.Scale(app.saarm.LocalToInches);

	if(!refPosSet)
	{
		refPos = CurrentPos;
		refPosSet = true;
		ResumeMeasTimer();
		return TRUE;
	}

	VectorNamedArray res;
	if(!app.saarm.geomTriggerWrap.Check(refPos, CurrentPos, app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0, res))
	{
		refPos = CurrentPos;
		app.pA->UpdateToSpan(); // 2015.08.19 - let's update SA during geom trig meas, not just when we get crossing(s)
		ResumeMeasTimer();
		return FALSE;
	}

	if( (app.pA->IsRomer() || app.pA->IsFrenchRomer()) && app.saarm.Data.SectionSound)
	{
		app.pA->ArmBeep(REC_OK);
	}
	else
	{
		if(MessageBeep(MB_ICONASTERISK) == 0)
			Beep(700, 200);
	}

	refPos = CurrentPos;
	for(INT_PTR i=0; i<res.GetSize(); i++)
	{		
		Vector xyz(res.Get(i));
		app.pA->IncrementName(Target, inc);
	

		//ctr = app.pA->AddPoint(xyz,CurrentNorm);
		sampleinfo.Format(_T("%d Pts Measured"), ctr+1);

		//Vector CurrWork;
		//app.saarm.InstToWorking.Multiply(xyz,&CurrWork);
		//CurrWork.Scale(1.0/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);
		
		//2011.01.05 - no longer send joint values, since this could be an interpolated point, and that could slightly throw off the model correction, etc.
		app.pA->MeasToSpan(xyz, res[i].Name, false, false);	// 2010.02.05 - Mantis 2614 - Arm graphics are not updating when using geometry Triggers 
	}
	ResumeMeasTimer();
	return TRUE;
}

void DGeomTriggerPts::SetStreamIndicator(BOOL strm)
{
	refPosSet = false;

	if(strm)
	{
		TRACE0("Setting stream indicator ON\n");
		StreamIndicator.SetBkColor(RGB(0,255,0));
		StreamIndicator.SetTextColor(RGB(0,0,0));
		StreamIndicator.SetWindowText(_T("ON"));
	}
	else
	{
		TRACE0("Setting stream indicator OFF\n");
		StreamIndicator.SetBkColor(RGB(255,0,0));
		StreamIndicator.SetTextColor(RGB(0,0,0));
		StreamIndicator.SetWindowText(_T("OFF"));
	}
}

///////////////////////////////////////////////////////////////////////////
// simulation mode

BOOL DGeomTriggerPts::MakeFakeData(int step)
{
	if(app.StopNow)
		return FALSE;			

	app.pA->X = ((double)rand())/((double)RAND_MAX);
	app.pA->Y = ((double)rand())/((double)RAND_MAX);
	app.pA->Z = (-(double)rand())/((double)RAND_MAX);

	app.pA->CurrentTransform.SetTranslation(Vector(app.pA->X, app.pA->Y, app.pA->Z));

	return TRUE;
}

// DGeomTriggerPts message handlers

void DGeomTriggerPts::OnBnClickedButtonOneshortGt()
{
	SetB1Short();
}

void DGeomTriggerPts::OnBnClickedButtonOnelongGt()
{
	SetB1Long();
}

void DGeomTriggerPts::OnBnClickedButtonTwoshortGt()
{
	SetB2Short();
}

void DGeomTriggerPts::OnBnClickedButtonTwolongGt()
{
	SetB2Long();
}

void DGeomTriggerPts::OnBnClickedIncgGt()
{
	app.pA->IncrementName(app.saarm.Data.Group, 1);
}

void DGeomTriggerPts::OnBnClickedInctGt()
{
	app.pA->IncrementName(app.saarm.Data.Target, app.saarm.Data.ptnameinc);
	app.pA->SetDisplayData();
}

void DGeomTriggerPts::OnShowWindow(BOOL bShow, UINT nStatus)
{
	DGenMeas::OnShowWindow(bShow, nStatus);

	if (bShow)
		PersistentWnd::RestoreWndRect(_T("windows"),_T("SA_Arms_DGeomTrigger"),this);	
}

void DGeomTriggerPts::OnDestroy()
{
	PersistentWnd::SaveWndRect(_T("windows"),_T("SA_Arms_DGeomTrigger"),this);

	DGenMeas::OnDestroy();
}

void DGeomTriggerPts::OnBnClickedDefault()
{
	UpdateData(TRUE);
	app.saarm.Data.Group = Group;
	app.saarm.Data.Target = Target;
	app.pA->SetDisplayData();
}
