// DPCMM.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "ArmBase.h"

#include "RomerArm.h"
#include "FaroArm.h"
#include "FaroUSBArm.h"
#include "SNLArm.h"
#include "Articulated/ArticulatedArmWrapper.h"
#include "RomerEuropeArm.h"
#include "MetrisGardaArm.h"
#include "APIBacesArm.h"
#include "KreonScannerArm.h"
#include "HexagonRDS.h"

#include "SA Easy Pad/EasyPadArmInterface.h"
#include "DGenMeas.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDPCMMApp

BEGIN_MESSAGE_MAP(CDPCMMApp, SAIBWinApp)
	//{{AFX_MSG_MAP(CDPCMMApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDPCMMApp construction

CDPCMMApp::CDPCMMApp()
: saarm(*new SAArm)
, controlPad(*new EasyPadArmInterface())
{
	pArtic = NULL;
	keyCtrl = false;
	measDlg = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CDPCMMApp object
CDPCMMApp app;

/////////////////////////////////////////////////////////////////////////////
// CDPCMMApp initialization

void CDPCMMApp::EstablishArm_API_Baces()
{
	delete pA; 
	pA=NULL; 
	pA = new APIBacesArm;
}

void CDPCMMApp::EstablishArm_Hexagon_RDS()
{
	delete pA;
	pA=NULL;
	pA = new HexagonRDS;
}

void CDPCMMApp::EstablishArm_API_Baces_Kreon_Scanner()
{
	delete pA; 
	pA=NULL; 
	pA = new KreonScannerArm;
}

void CDPCMMApp::EstablishArm_FaroUSB() 
{
	delete pA; 
	pA=NULL; 
	pA = new FaroUSBArm;
}
void CDPCMMApp::EstablishArm_Romer()
{
	delete pA; 
	pA=NULL; 
	//////////////////////////////////////////////////////////////////////////////////////////////////
	// 2004.09.07 - Project properties (Linker->Input) are set to Delay Load all arm specific dll's.  
	// What's cool is that we can still do LoadLibrary to make sure e.g. winrds exists somewhere 
	// (whatever RomerPathSet put in the registry).  If the dll is not there, we leave gracefully, 
	// if it is, we just unload the dll (FreeLibrary) and move on.  
	// We have delay load set, so when the first call to a dll fcn is made, the dll gets loaded again!
	HMODULE rdsMod = LoadLibrary(_T("WINRDS.DLL"));
	if(rdsMod == NULL)
	{
		AfxMessageBox(_T("Could not load WinRDS.dll\nYou must run RomerPathSet for this arm install."),MB_ICONHAND);
		int ret = AfxMessageBox(_T("Would you like to run in simulation?"),MB_ICONQUESTION | MB_YESNO);
		if(ret==IDYES)
		{
			pA = new ArmBase;
		}
		else
		{
			CleanUp();
			exit(2);
		}
	}
	else
	{
		FreeLibrary(rdsMod);
		pA = new RomerArm;
	}
	//////////////////////////////////////////////////////////////////////////////////////////////////
}
void CDPCMMApp::EstablishArm_RomerArticulator()
{
	pArtic = new ArticulatedArmWrapper;
	delete pA; 
	pA=NULL; 
	//////////////////////////////////////////////////////////////////////////////////////////////////
	// 2004.09.07 - Project properties (Linker->Input) are set to Delay Load all arm specific dll's.  
	// What's cool is that we can still do LoadLibrary to make sure e.g. winrds exists somewhere 
	// (whatever RomerPathSet put in the registry).  If the dll is not there, we leave gracefully, 
	// if it is, we just unload the dll (FreeLibrary) and move on.  
	// We have delay load set, so when the first call to a dll fcn is made, the dll gets loaded again!
	HMODULE rdsMod = LoadLibrary(_T("WINRDS.DLL"));
	if(rdsMod == NULL)
	{
		AfxMessageBox(_T("Could not load WinRDS.dll\nYou must run RomerPathSet for this arm install."),MB_ICONHAND);
		int ret = AfxMessageBox(_T("Would you like to run in simulation?"),MB_ICONQUESTION | MB_YESNO);
		if(ret==IDYES)
		{
			pA = new ArmBase;
		}
		else
		{
			CleanUp();
			exit(2);
		}
	}
	else
	{
		FreeLibrary(rdsMod);
		pA = new RomerArm;
	}
	//////////////////////////////////////////////////////////////////////////////////////////////////
}
void CDPCMMApp::EstablishArm_FaroSerial()
{
	delete pA; 
	pA=NULL; 
	pA = new FaroArm;
}
void CDPCMMApp::EstablishArm_SNL()
{
	delete pA; 
	pA=NULL; 
	pA = new SNLArm;
}
void CDPCMMApp::EstablishArm_RomerEurope()
{
	delete pA;
	pA = NULL;
	pA = new RomerEuropeArm;
}
void CDPCMMApp::EstablishArm_MetrisMCA()
{
	delete pA;
	pA = NULL;
	pA = new MetrisGardaArm;
}

void CDPCMMApp::CleanUp()
{
	delete &saarm;
	delete& controlPad;
	delete pArtic;
}
BOOL CDPCMMApp::InitInstance()
{
	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer(); // 2005.10.20 - for html help - uses MSWebBrowser ocx

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.
	SetRegistryKey(_T("New River Kinematics"));

	//GXInit();	// 2003.12.19 - migrating to 2003 environment, grid v.7.0 not supported, and not used here anyway
	
	StopNow = FALSE;
	VString.Format(_T("2019.08.07"));

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	ParseCommandLine(saarm.AutoSocket);

	// Do language setup prior to logon so that language changes are acknowledged
	if (saarm.IsRegLanguageEnforcerSet())
	{
		saarm.EnableLanguageEnforcer(_T("ArmOptiParam.tvo"), _T("aRmS"));
		if (!saarm.EnforceLanguage(true))
		{
			PostQuitMessage(0);
		}
	}

	saarm.DoInitInstanceStuff();

	if(saarm.AutoSocket.Valid)
	{
		saarm.Data.doconnection = saarm.AutoSocket.RunInSimulationMode() ? FALSE : TRUE;
		//if(saarm.Data.doconnection)
		//	AfxMessageBox(_T("Will do connection"));
		//else
		//	AfxMessageBox(_T("Will Run In Sim"));
	}

	if(!saarm.Logon()) // select arm from browse tree, connect to SA, and tell this app to establish selected arm
	{
		CleanUp();
		return FALSE;
	}

	saarm.SaveSettings();
	
	CDPCMMDlg dlg;
	m_pMainWnd = &dlg;	
	int nResponse = dlg.DoModal();
	
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}

	saarm.SaveSettings();
	CleanUp();

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}

BOOL CDPCMMApp::PreTranslateMessage(MSG* pMsg) 
{
	if(m_pMainWnd == NULL)
		return CWinApp::PreTranslateMessage(pMsg);

	int nVirtKey = 0;
	bool SkipStandardHandling = false;

	if (pMsg->message == WM_KEYDOWN)
	{
		nVirtKey = (int) pMsg->wParam;
		if( nVirtKey == VK_ESCAPE )	// don't close when escape is hit!!
		{
			SkipStandardHandling = TRUE;
		}
		else if (nVirtKey == VK_CONTROL)
		{
			keyCtrl = true;
		}
	}
	if (SkipStandardHandling)
		return TRUE;

	// 2001.06.15 - following added for keyboaArd control of Sandia N. L. 3 dof Arm
	// 2007.09.19 - and opened up for all arms - Scott requested keyboard measurement control
	if( pMsg->message == WM_KEYUP )
	{
		CDPCMMDlg& d = *((CDPCMMDlg*)m_pMainWnd);
		if(d.FocusOnKeyboardInput())
			return CWinApp::PreTranslateMessage(pMsg);

		nVirtKey = (int) pMsg->wParam;
		TRACE("%d, %c\n", nVirtKey, nVirtKey);

		if(pA->IsSNL())
		{
			if( nVirtKey == 'R' )	// record
			{
				pA->buttonstate = 1;
				pA->buttonstate += BUTTON_SHORT;
			}
			else if( nVirtKey == 'A')	// accept
			{
				pA->buttonstate = 1;
				pA->buttonstate += BUTTON_LONG;
			}
			else if( nVirtKey == 'D')	// delete last
			{
				pA->buttonstate = 2;
				pA->buttonstate += BUTTON_SHORT;
			}
			else if( nVirtKey == 'C')	// cancel
			{
				pA->buttonstate = 2;
				pA->buttonstate += BUTTON_LONG;
			}
		}
		else
		{
			if (nVirtKey == VK_CONTROL)
			{
				keyCtrl = false;
			}
			if(nVirtKey == VK_LEFT && keyCtrl)
			{
				SAIPInspectionNavigation in;
				in.navMode = SAIPInspectionNavigation::NAV_PREV;
				app.saarm.SendInspectionNavigationToSA(in);
			}
			else if(nVirtKey == VK_RIGHT && keyCtrl)
			{
				SAIPInspectionNavigation in;
				in.navMode = SAIPInspectionNavigation::NAV_NEXT;
				app.saarm.SendInspectionNavigationToSA(in);
			}
			else if( (nVirtKey == '1' || nVirtKey == VK_NUMPAD1) && !keyCtrl)	// button 1 short hit
			{
				TRACE0("MEASURE - 1 hit\n");
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 1;
				pA->buttonstate += BUTTON_SHORT;
			}
			else if( (nVirtKey == '1' || nVirtKey == VK_NUMPAD1) && keyCtrl)	// button 1 long hit
			{
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 1;
				pA->buttonstate += BUTTON_LONG;
			}
			else if( (nVirtKey == '2' || nVirtKey == VK_NUMPAD2) && !keyCtrl)	// button 2 short hit
			{
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 2;
				pA->buttonstate += BUTTON_SHORT;
			}
			else if( (nVirtKey == '2' || nVirtKey == VK_NUMPAD2) && keyCtrl)	// button 2 long hit
			{
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 2;
				pA->buttonstate += BUTTON_LONG;
			}
			else if( (nVirtKey == '3' || nVirtKey == VK_NUMPAD3) && !keyCtrl)	// button 3 short hit
			{
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 3;
				pA->buttonstate += BUTTON_SHORT;
			}
			else if( (nVirtKey == '3' || nVirtKey == VK_NUMPAD3) && keyCtrl)	// button 3 long hit
			{
				pA->overrideArmButtons = true;
				pA->ClearButtonState();
				pA->buttonstate = 3;
				pA->buttonstate += BUTTON_LONG;
			}
		}
	}
	return CWinApp::PreTranslateMessage(pMsg);
}
BOOL CDPCMMApp::FindSADir( LPCTSTR Key, CString* pDir)
{
	TRACE("find dir for SA: %s\n",Key);
	// attempt to locate MoluCAD
	HKEY hKey;
	if (!RegOpenKeyEx(HKEY_CURRENT_USER,_T("Software\\New River Kinematics\\Spatial Analyzer\\Directories"),0,KEY_READ,&hKey) == ERROR_SUCCESS)
		return FALSE;

	DWORD reg_type;
	DWORD buf_size = MAX_PATH;
	TCHAR buf[MAX_PATH];
	buf[0]=NULL;
	if (!RegQueryValueEx(hKey,Key,0,&reg_type,(LPBYTE)buf,&buf_size) == ERROR_SUCCESS)
	{
		RegCloseKey(hKey);
		return FALSE;
	}

	*pDir = buf;

	TRACE("SA %s dir = %s\n",Key,*pDir );

	RegCloseKey(hKey);
	return TRUE;
}
