// DSimScannerMeas.cpp : implementation file
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "DSimScannerMeas.h"
#include "ArmBase.h"

// DSimScannerMeas dialog

IMPLEMENT_DYNAMIC(DSimScannerMeas, CDialog)

DSimScannerMeas::DSimScannerMeas(CWnd* pParent /*=NULL*/)
	: CDialog(DSimScannerMeas::IDD, pParent)
	, scannerCS(*new CriticalSectionWrapper)
	, ptsPerLine(512)
	, ptsPerLineVariation(32)
	, linesPerSecond(100)
{
	simScanning = false;
	stopSimScanning = false;
}

DSimScannerMeas::~DSimScannerMeas()
{
	delete &scannerCS;
}

void DSimScannerMeas::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_SIM_PTS_PER_LINE, ptsPerLine);
	DDX_Text(pDX, IDC_EDIT_SIM_VARY_PTS_PER_LINE, ptsPerLineVariation);
	DDX_Text(pDX, IDC_EDIT_SIM_LINES_PER_SEC, linesPerSecond);
}


BEGIN_MESSAGE_MAP(DSimScannerMeas, CDialog)
	ON_BN_CLICKED(IDC_DEFAULT, &DSimScannerMeas::OnBnClickedDefault)
	ON_BN_CLICKED(IDC_BUTTON_START_SS, &DSimScannerMeas::OnBnClickedButtonStartSs)
	ON_BN_CLICKED(IDC_BUTTON_STOP_SS, &DSimScannerMeas::OnBnClickedButtonStopSs)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDOK, &DSimScannerMeas::OnBnClickedOk)
END_MESSAGE_MAP()

static double shiftX = 5.0;
static double shiftY = 0.0;
void DSimScannerMeas::SimulateScan()
{
	simScanning = true;
	SAIPCloudBlockPt cPt;

	UpdateData(TRUE);

	app.pA->SendOnScanPassStarted();

	DWORD ctr = 0;
	while(1)
	{
		//Delay(0.01,TRUE);
		ProcessWaitingMessages();	// user needs to be able to abort meas

		if(stopSimScanning)
		{
			TRACE0("\n***Aborting SimulateScan***\n\n");
			if (app.saarm.Data.voxelizationEnabled)
				app.pA->ClearScanClouds();
			simScanning = false;
			app.pA->SendOnScanPassStopped();
			return;
		}

		if(app.saarm.Data.useScanStripeClouds && app.pA->performingStdScannerMeas)
		{
			if(app.pA->simCloudCtr > 100)
			{
				app.pA->simCloudCtr = 1;
				shiftX = 5.0;
				shiftY += 3.0;
				//app.pA->SendOnScanPassStopped();
				app.pA->SendOnScanPassStarted();
			}

			app.pA->AddScannerCloudStripe();

			SAIPScanStripePoint cPt;
			Transform stripeXform;
			stripeXform.SetRotation(Vector(180.0, 0.0, 0.0));

			double period = 1.0/(double)linesPerSecond;
			Delay(period, TRUE);
			//TRACE2("%d lines per sec -> period of %f sec\n", linesPerSecond, period);

			RandomDataGenerator rdg;
			DWORD shave = (DWORD)rdg.RandomInt(0, ptsPerLineVariation);
			DWORD pts = (DWORD)ptsPerLine - shave;

			//TRACE3("pts/line = %d, variation = %d, lines/sec = %d, ", pts, shave, linesPerSecond);
			//TRACE1("thinning factor = %d\n", app.saarm.Data.thinningFactor);

			ctr = 0;
			shiftX += 0.1;
			for(DWORD k=0; k<pts; k++) 
			{
				++ctr;
				//TRACE2("ctr=%ld, thinningFactor=%d\n", ctr, theApp.saTScan.thinningFactor);
				if(ctr % app.saarm.Data.thinningFactor != 0)
				{
					//TRACE0("skipping pt.\n");
					continue;
				}
				//TRACE0("not skipping pt.\n");

				cPt.x = shiftX + (0.01*((double)rand())/((double)RAND_MAX));//+shiftX);
				cPt.y = shiftY + (double)(k)/200.0 + (1.0*((double)rand())/((double)RAND_MAX));//+shiftY);
				cPt.z = 0.5 + (double)((-1.0*ctr)/ctr) * (0.01*((double)rand())/((double)RAND_MAX)); // blt - 2016.04.15 - Olga requested some noise, cPt.z was just set to 0.5 originally	

				if(k == ptsPerLine/2)
				{
					stripeXform.SetTranslation(Vector(cPt.x, cPt.y, cPt.z));
				}

				app.pA->AddScannerCloudStripePt(cPt);
			}
			//CriticalSectionLock lock( scannerCS );

			//stripeXform.Scale(app.saarm.LocalToInches);
			app.pA->scanStripePkt.SetCurrentLineTransform(stripeXform);
		}
		else
		{
			if(app.pA->simCloudCtr > 100)
			{
				app.pA->simCloudCtr = 1;
				shiftX = 5.0;
				shiftY += 3.0;
				//app.pA->SendOnScanPassStopped();
				app.pA->SendOnScanPassStarted();
			}

			double period = 1.0/(double)linesPerSecond;
			Delay(period, TRUE);
			//TRACE2("%d lines per sec -> period of %f sec\n", linesPerSecond, period);

			RandomDataGenerator rdg;
			DWORD shave = (DWORD)rdg.RandomInt(0, ptsPerLineVariation);
			DWORD pts = (DWORD)ptsPerLine - shave;

			//TRACE3("pts/line = %d, variation = %d, lines/sec = %d, ", pts, shave, linesPerSecond);
			//TRACE1("thinning factor = %d\n", app.saarm.Data.thinningFactor);

			ctr = 0;
			shiftX += 0.1;
			for(DWORD k=0; k<pts; k++) 
			{
				++ctr;
				//TRACE2("ctr=%ld, thinningFactor=%d\n", ctr, theApp.saTScan.thinningFactor);
				if(ctr % app.saarm.Data.thinningFactor != 0)
				{
					//TRACE0("skipping pt.\n");
					continue;
				}
				//TRACE0("not skipping pt.\n");

				cPt.x = shiftX + (0.01*((double)rand())/((double)RAND_MAX));//+shiftX);
				cPt.y = shiftY + (double)(k)/200.0 + (1.0*((double)rand())/((double)RAND_MAX));//+shiftY);
				cPt.z = 0.5 + (double)((-1.0*ctr)/ctr) * (0.01*((double)rand())/((double)RAND_MAX)); // blt - 2016.04.15 - Olga requested some noise, cPt.z was just set to 0.5 originally	

				app.pA->AddScannerCloudPt(cPt);
			}
			//CriticalSectionLock lock( scannerCS );
		}
	}
	simScanning = false;
}

// DSimScannerMeas message handlers

void DSimScannerMeas::OnBnClickedDefault()
{
	return;
}

void DSimScannerMeas::OnBnClickedButtonStartSs()
{
	if(!simScanning)
	{
		stopSimScanning = false;

		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		d.StopQueryTimer();

		Delay(0.5,TRUE);
		d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

		SimulateScan();
	}
}

void DSimScannerMeas::OnBnClickedButtonStopSs()
{
	stopSimScanning = true;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
}

BOOL DSimScannerMeas::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME), FALSE);
	TRANSLATE_DIALOG;

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DSimScannerMeas::OnClose()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	OnBnClickedButtonStopSs();

	CDialog::OnClose();
}

void DSimScannerMeas::OnBnClickedOk()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	OnBnClickedButtonStopSs();

	OnOK();
}
