// DVoxelSettings.cpp : implementation file
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DVoxelSettings.h"
#include "DVoxelDefaultSettings.h"

// DVoxelSettings dialog

IMPLEMENT_DYNAMIC(DVoxelSettings, CDialog)

DVoxelSettings::DVoxelSettings(CWnd* pParent /*=NULL*/)
	: CDialog(DVoxelSettings::IDD, pParent)
	, voxEnabled(FALSE)
	, voxelSize(0)
	, minPtsPerVoxel(0)
	, numPtsPerVoxelization(0)
	, voxelSizeUnitsVal(_T(""))
{

}

DVoxelSettings::~DVoxelSettings()
{
}

void DVoxelSettings::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_VOXEL_SIZE, voxelSizeEditBox);
	DDX_Control(pDX, IDC_EDIT_MIN_PTS_PER_VOXEL, ptsPerVoxelEditBox);
	DDX_Control(pDX, IDC_EDIT_NUM_CLOUD_PTS_PER_VOXELIZATION, numCloudPtsPerVoxelizationEditBox);
	DDX_Check(pDX, IDC_CHECK_ENABLE_VOXELIZATION, voxEnabled);
	DDX_Text(pDX, IDC_EDIT_VOXEL_SIZE, voxelSize);
	DDX_Text(pDX, IDC_EDIT_MIN_PTS_PER_VOXEL, minPtsPerVoxel);
	DDX_Text(pDX, IDC_EDIT_NUM_CLOUD_PTS_PER_VOXELIZATION, numPtsPerVoxelization);
	DDX_Text(pDX, IDC_STATIC_VOX_SIZE_UNITS, voxelSizeUnitsVal);
	DDX_Control(pDX, IDC_BUTTON_SET_DEFAULTS, defaultsButton);
}


BEGIN_MESSAGE_MAP(DVoxelSettings, CDialog)
	ON_BN_CLICKED(IDC_CHECK_ENABLE_VOXELIZATION, &DVoxelSettings::OnBnClickedCheckEnableVoxelization)
	ON_BN_CLICKED(IDC_BUTTON_SET_DEFAULTS, &DVoxelSettings::OnBnClickedButtonSetDefaults)
END_MESSAGE_MAP()


// DVoxelSettings message handlers

BOOL DVoxelSettings::OnInitDialog()
{
	CDialog::OnInitDialog();

	voxelSizeUnitsVal = app.saarm.units.DistUnitNames[app.saarm.units.distunits];

	UpdateData(FALSE);
	OnBnClickedCheckEnableVoxelization();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DVoxelSettings::OnBnClickedCheckEnableVoxelization()
{
	UpdateData(TRUE);

	if (voxEnabled)
	{
		// enable controls
		voxelSizeEditBox.EnableWindow(TRUE);
		ptsPerVoxelEditBox.EnableWindow(TRUE);
		numCloudPtsPerVoxelizationEditBox.EnableWindow(TRUE);
		defaultsButton.EnableWindow(TRUE);
	}
	else
	{
		// disable controls
		voxelSizeEditBox.EnableWindow(FALSE);
		ptsPerVoxelEditBox.EnableWindow(FALSE);
		numCloudPtsPerVoxelizationEditBox.EnableWindow(FALSE);
		defaultsButton.EnableWindow(FALSE);
	}
}

void DVoxelSettings::OnBnClickedButtonSetDefaults()
{
	DVoxelDefaultSettings dVoxDefs(*this);
	if (DOMODAL(dVoxDefs) != IDOK)
	{
		voxelSize = app.saarm.Data.voxelSize * app.saarm.InchesToLocal/app.saarm.units.UnitToMeters[app.saarm.units.distunits]; // stored in inches since that's native to our voxelization code
		minPtsPerVoxel = app.saarm.Data.minPtsPerVoxel;
		numPtsPerVoxelization = app.saarm.Data.minCloudPtsToVoxelize;
		UpdateData(FALSE);
	}
}

void DVoxelSettings::SetValues(double voxSize, int minPtsPerVox, int minPtsPerVoxelization)
{
	double inchesToCurrentUnit = app.saarm.InchesToLocal/app.saarm.units.UnitToMeters[app.saarm.units.distunits];

	voxelSize = voxSize * inchesToCurrentUnit; // stored in inches since that's native to our voxelization code
	minPtsPerVoxel = minPtsPerVox;
	numPtsPerVoxelization = minPtsPerVoxelization;
	UpdateData(FALSE);
}
