// DataStore.cpp : implementation file
//

#include "stdafx.h"
#include "nrkapp.h"
#include "PlaneObj.h"
#include "DataStore.h"
#include "Articulated/ArticulatedArmWrapper.h"	// for serialization
#include "GeomTriggerWrapper.h"
#include "Kreon/KreonProbeCalibrations.h"
#include "SA Easy Pad/EasyPadArmInterface.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////
Store::Store()
: ds_plane(*new PlaneObj)
, sheetMetalPlane(*new PlaneObj)
, KreonProbeCals(*new KreonProbeCalibrations)
, cf(*new ConeFit)
, singlePtPlane(*new PlaneObj)
{
	cf.fp.Name = _T("Cone Fit Settings");
	ds_planeSet = false;
	sm_planeSet = false;
	singlePtPlaneSet = false;
	SAComputer = _T("200.200.200.200");
	MeasComments = _T("");
	SAPort = 0;
	CommPort = 1;
	Collection = _T("");
	Group = _T("Main");
	Target = _T("P1");
	geomName = _T("");
	ButtonOneMode = app.saarm.bf_record_accept;
	ButtonTwoMode = app.saarm.bf_deleteLast_cancel;
	ButtonThreeMode = app.saarm.bf_record_accept;
	buttonOneMain = app.saarm.bf_record_accept;
	buttonTwoMain = app.saarm.bf_deleteLast_cancel;
	buttonThreeMain = app.saarm.bf_record_accept;
	TargetNameStartNumber = 0;
	QueryRate = QUERYRATESLOW;        //in millisecs
	SlackCount =0;
	AutoDetectProbe = TRUE;
	TipDia = 0.006;
	NoSound = FALSE;
	ComputerSound = TRUE;
	DisplayUnits = 0;  //index to inches
	Interval = 0.0254;
	SpkrVolume = 40;
	MEProximity = 0.10; //meters	// 2004.08.24 - no longer used (still serialized for compatibility)
	GoodProximity = 0.0254;	
	proxWorkingField = 0.0889;
	GuideSound = TRUE;
	SectionSound = TRUE;
	FirePointsToo = FALSE;
	SendGeom = TRUE;
	sendAllPtsAsCloudPts = FALSE;
	Precision = 3;
	DoRealTimeGraphics = TRUE;
	AddExtraPatchShift = FALSE;
	ExtraPatchShift = 0.0;
	GuideToPatchPoints = FALSE;
	LockSpheres = FALSE;
	SphereLockVal = 0.0254;
	LockCircles = FALSE;
	CircleLockVal = 0.0254;
	FrameRequest = FALSE;

	SphereTol = 0.000127;
	CircleTol = 0.000127;
	PlaneTol = 0.000127;

	inoutface = 0;
	alwaysupdateSA = TRUE;

	meastimerperiodsec = 0.05;
	updatetimerperiodsec = 0.03125;	// 2010.09.07 changed from 16Hz to 32Hz
	PDAtimerperiodsec = 1.0;

	doconnection = TRUE;

	// Faro serial only...
	CommBaud = 3;
	FaroAuxOn = FALSE;
	mouseGain = 650;
	backButtonCentersMouse = TRUE;
	commParams = 0;

	ptnameinc = 1;
	incrementgroup = TRUE;

	scan_spatial = FALSE;	// 2003.11.17
	scan_spatial_inc = 0.0254;

	scannerTimerPeriodSec = 0.25;	// seconds, 2004.06.16
	autoIncCloudName = FALSE;	// 2005.09.28
	thinningFactor = 1;	// 2006.03.02

	avgPointSamples = 50;

	lineMeasType = 1;	// 0 - offset line, 1 - probe center

	measmode = 0;

	autoCorrectModel = TRUE;

	jtUpdateThreshold = 0.5;	// degrees

	startupInMeasMode = FALSE;
	startupMeasMode = 0;

	scaleFromInstBase = 1.0;

	// API_Baces (Axxis) settings
	apiBacesComPort = 0;
	apiBacesParamPath = _T("C:\\BACES\\M022_5C_2.TAB");

	controlPadEnabled = false;

	useScannerForAutoProxMeas = false;
	useScanStripeClouds = true;

	// 2020.04.03 - voxelize before sending to SA
	voxelizationEnabled = false;
	voxelSize = 0.15;
	minPtsPerVoxel = 3;
	minCloudPtsToVoxelize = 150000;

	useShankForAutoProxMeas = false;
	shankRadius = 0.003; // 6mm seems to be the most commonly used probe tip size. Guessing at 3mm being the most common stylus radius
}

Store::~Store()
{
	Clear();
}

void Store::Clear()
{
	delete &ds_plane;
	delete &sheetMetalPlane;
	delete &KreonProbeCals;
	delete &cf;
	delete &singlePtPlane;
}

void Store::InvalidateReferencePlanes()
{
	ds_planeSet = false;
	sm_planeSet = false;
	singlePtPlaneSet = false;
}

void Store::Serialize(CArchive& ar) 
{
	BYTE Schema;
	//Schema = 4;
	//Schema = 5;	// 2003.02.11 - added ptnameinc and incrementgroup
	//Schema = 6;	// 2003.11.17 - added scan_spatial and scan_spatial_inc
	//Schema = 7; // 2004.06.16 - added scannerTimerPeriodSec for laser line scanners
	//Schema = 8;
	//Schema = 9;	// 2004.09.09 - added measmode to save last measmode button state in main dlg
	//Schema = 10;	// 2004.09.10 - added avg point samples for new meas mode
	//Schema = 11;	// 2004.09.27 - Faro serial mouse mode support
	//Schema = 12;	// 2004.10.12 - Faro serial commParams (parity, data bits, stop bits)
	//Schema = 13;	// 2005.04.28 - added lineMeasType for offset line (sheet metal)
	//Schema = 14;	// 2005.05.01 - added sheetMetalPlane
	//Schema = 15;	// 2005.08.24 - Articulated Arm Wrapper can save stuff
	//Schema = 15;	// 2005.09.28 - autoIncCloudName
	//Schema = 16;	// 2006.03.02 - scanner thinningFactor
	//Schema = 17;	// 2006.06.07 - proxWorkingField for Dave's new srceaming weasel
	//Schema = 18;	// 2006.12.05 - added autoCorrectModel for SA graphic model correction (see ModelInfo cpp & h)
	//Schema = 19;	// 2007.04.12 - SA joint update threshold
	//Schema = 20;	// 2007.08.17 - send all pts. as cloud pts. option
	//Schema = 21;	// 2008.06.18 - start up meas mode option
	//Schema = 22;	// 2008.11.17 - added user editable geometry name
	//Schema = 23;	// 2009.01.14 - added scaleFromInstBase, so reference planes can be invalidated when scale changes
	//Schema = 24;	// 2009.02.12 - added armbase's geomtriggerwrapper
	//Schema = 25;	// 2009.04.07 - added stuff for API_Baces (Axxis) arm
	//Schema = 26;	// 2009.04.27 - added stuff for Kreaon Scanner amr (supported on API Baces Arm)
	//Schema = 27;	// 2010.02.26 - added Cone Fit meas
	//Schema = 28;	// 2010.08.11 - added useScannerForAutoProxMeas setting for Mantis 3328 - Update Auto-Correspond with Proximity trigger to handle point cloud data 
	//Schema = 29;	// 2010.09.03 - easy pad - added to serialize pad calibration
	//Schema = 30;	// 2011.12.22 - single point plane object and set indicator
	//Schema = 31;	// 2013.10.04 - added button modes for main interface  to restore when returning from toolbar mode
	//Schema = 32;	// 2016.06.30 - added option so user can go back to original pt. cloud format for when they don't need meshing but need max. performance
	//Schema = 33;	// 2019.06.18 - added option to use shank (probe stylus) for Auto-Prox measurements, and added shankRadius
	Schema = 34;   // 2020.04.03 - added options for voxelization
	if (ar.IsStoring())
	{
		ar << Schema;
		
		ar << SAComputer;
		ar << UserName;
		ar << MeasComments;
		ar << SAPort;
		ar << CommPort;
		ar << Group;
		ar << Target;
		ar << ButtonOneMode;
		ar << ButtonTwoMode;
		ar << ButtonThreeMode;
		ar << TargetNameStartNumber;
		ar << QueryRate;
		ar << SlackCount;
		ar << AutoDetectProbe;
		ar << TipDia;
		ar << NoSound;
		ar << ComputerSound;
		ar << DisplayUnits;
		ar << Interval;
		ar << SpkrVolume;
		ar << MEProximity;
		ar << GoodProximity;
		ar << GuideSound;
		ar << SectionSound;
		ar << FirePointsToo;
		ar << Precision;
		ar << DoRealTimeGraphics;
		ar << AddExtraPatchShift;
		ar << ExtraPatchShift;
		ar << GuideToPatchPoints;
		ar << LockSpheres;
		ar << SphereLockVal;
		ar << LockCircles;
		ar << CircleLockVal;
		ar << FrameRequest;
		ar << inoutface;
		ar << alwaysupdateSA;
		ar << meastimerperiodsec;
		ar << updatetimerperiodsec;
		ar << PDAtimerperiodsec;
		ar << CommBaud;
		ar << SendGeom;
		ar << FaroAuxOn;
		ar << ptnameinc;
		ar << incrementgroup;
		ar << scan_spatial;	// 2003.11.17
		ar << scan_spatial_inc;
		ar << scannerTimerPeriodSec;
		ds_plane.Serialize(ar);
		ar << ds_planeSet;
		ar << measmode;
		ar << avgPointSamples;
		ar << mouseGain;
		ar << backButtonCentersMouse;
		ar << commParams;
		ar << lineMeasType;
		sheetMetalPlane.Serialize(ar);
		ar << sm_planeSet;
		//app.pArtic->Serialize(ar);
		ar << autoIncCloudName;
		ar << thinningFactor;
		ar << proxWorkingField;
		ar << autoCorrectModel;
		ar << jtUpdateThreshold;
		ar << sendAllPtsAsCloudPts;
		ar << startupInMeasMode;
		ar << startupMeasMode;
		ar << geomName;
		ar << scaleFromInstBase;
		app.saarm.geomTriggerWrap.Serialize(ar);
		ar << apiBacesComPort;
		ar << apiBacesParamPath;
		kreonScannerCalMatrix.Serialize(ar);
		KreonProbeCals.Serialize(ar);
		cf.Serialize(ar);
		ar << useScannerForAutoProxMeas;
		app.controlPad.Serialize(ar);
		singlePtPlane.Serialize(ar);
		ar << singlePtPlaneSet;
		ar << buttonOneMain;
		ar << buttonTwoMain;
		ar << buttonThreeMain;
		ar << useScanStripeClouds;
		ar << useShankForAutoProxMeas;
		ar << shankRadius;
		ar << voxelizationEnabled;
		ar << voxelSize; // inches
		ar << minPtsPerVoxel;
		ar << minCloudPtsToVoxelize;
	}
	else
	{
		ar >> LoadSchema;
		
		ar >> SAComputer;
		ar >> UserName;
		ar >> MeasComments;
		ar >> SAPort;
		ar >> CommPort;
		ar >> Group;
		ar >> Target;
		ar >> ButtonOneMode;
		ar >> ButtonTwoMode;
		ar >> ButtonThreeMode;
		ar >> TargetNameStartNumber;
		ar >> QueryRate;
		ar >> SlackCount;
		ar >> AutoDetectProbe;
		ar >> TipDia;
		ar >> NoSound;
		ar >> ComputerSound;
		ar >> DisplayUnits;
		ar >> Interval;
		ar >> SpkrVolume;
		ar >> MEProximity;
		ar >> GoodProximity;
		ar >> GuideSound;
		ar >> SectionSound;
		ar >> FirePointsToo;
		ar >> Precision;
		ar >> DoRealTimeGraphics;
		ar >> AddExtraPatchShift;
		ar >> ExtraPatchShift;
		ar >> GuideToPatchPoints;
		ar >> LockSpheres;
		ar >> SphereLockVal;
		ar >> LockCircles;
		ar >> CircleLockVal;
		ar >> FrameRequest;
		ar >> inoutface;
		ar >> alwaysupdateSA;

		if( LoadSchema >= 1 )
		{
			ar >> meastimerperiodsec;
			ar >> updatetimerperiodsec;
			ar >> PDAtimerperiodsec;
		}

		if( LoadSchema >= 2 )
			ar >> CommBaud;

		if( LoadSchema >= 3 )
			ar >> SendGeom;

		if( LoadSchema >= 4)
			ar >> FaroAuxOn;

		if( LoadSchema >= 5)
		{
			ar >> ptnameinc;
			ar >> incrementgroup;
		}

		if( LoadSchema >= 6 )
		{
			ar >> scan_spatial;	// 2003.11.17
			ar >> scan_spatial_inc;
		}

		if( LoadSchema >= 7 )	// 2004.06.16
			ar >> scannerTimerPeriodSec;

		if( LoadSchema >= 8 )	// 2004.08.24
		{
			ds_plane.Serialize(ar);
			ar >> ds_planeSet;
		}

		if( LoadSchema >= 9 )	// 2004.09.09
			ar >> measmode;

		if( LoadSchema >= 10 )	// 2004.09.10
			ar >> avgPointSamples;

		if( LoadSchema >= 11 )
		{
			ar >> mouseGain;
			ar >> backButtonCentersMouse;
		}

		if( LoadSchema >= 12 )
			ar >> commParams;

		if( LoadSchema >= 13 )	// 2005.04.28
			ar >> lineMeasType;

		if( LoadSchema >= 14 )	// 2005.05.01
		{
			sheetMetalPlane.Serialize(ar);
			ar >> sm_planeSet;
		}

		//if( LoadSchema >= 15 )	// 2005.08.24
		//{
		//	app.pArtic->Serialize(ar);
		//}

		if( LoadSchema >= 15 )	// 2005.09.28
		{
			ar >> autoIncCloudName;
		}

		if( LoadSchema >= 16 )	// 2006.03.02
		{
			ar >> thinningFactor;
		}
		
		if( LoadSchema >= 17 )	// 2006.06.07
		{
			ar >> proxWorkingField;
		}

		if( LoadSchema >= 18 )	// 2006.12.05
		{
			ar >> autoCorrectModel;
		}

		if( LoadSchema >= 19 )
		{
			ar >> jtUpdateThreshold;
		}

		if( LoadSchema >= 20 )
		{
			ar >> sendAllPtsAsCloudPts;
		}

		if( LoadSchema >= 21 )
		{

			ar >> startupInMeasMode;
			ar >> startupMeasMode;
		}

		if( LoadSchema >= 22 )
		{
			ar >> geomName;
		}

		if( LoadSchema >= 23 )
		{
			ar >> scaleFromInstBase;
		}

		if( LoadSchema >= 24 )
		{
			app.saarm.geomTriggerWrap.Serialize(ar);
		}

		if(LoadSchema >= 25)
		{
			ar >> apiBacesComPort;
			ar >> apiBacesParamPath;
		}

		if(LoadSchema >= 26)
		{
			kreonScannerCalMatrix.Serialize(ar);
			KreonProbeCals.Serialize(ar);
		}

		if( LoadSchema >= 27 )
		{
			cf.Serialize(ar);
		}

		if( LoadSchema >= 28 )
		{
			ar >> useScannerForAutoProxMeas;
		}

		if(LoadSchema >= 29)
		{
			app.controlPad.Serialize(ar);
		}

		if(LoadSchema >= 30)
		{
			singlePtPlane.Serialize(ar);
			ar >> singlePtPlaneSet;
		}

		if(LoadSchema >= 31)
		{
			ar >> buttonOneMain;
			ar >> buttonTwoMain;
			ar >> buttonThreeMain;
		}

		if(LoadSchema >= 32)
		{
			ar >> useScanStripeClouds;
		}

		if (LoadSchema >= 33)
		{
			ar >> useShankForAutoProxMeas;
			ar >> shankRadius;
		}

		if (LoadSchema >= 34)
		{
			ar >> voxelizationEnabled;
			ar >> voxelSize; // inches
			ar >> minPtsPerVoxel;
			ar >> minCloudPtsToVoxelize;
		}
	}
}

IMPLEMENT_SERIAL( Store, CObject, 0 );

