// Faro.cpp: implementation of the Faro class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"
#include "FaroArm.h"
#include "DSerialPortSettings.h"
#include "DSettingsFaroSerial.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FaroArm::FaroArm()
{
//	JShift.Create(DOF,TRUE);	// 2001.04.12 moved to ConnectToHardware
	ConnectedToArm = FALSE;
	Measuring = FALSE;

	app.saarm.Data.ButtonTwoMode = 3;	// backup/cancel, Romer defaults to 2, backup/mousetoggle
	firstshot = FALSE;
	buttontime = 4294967295;
	LongDown = FALSE;

	app.saarm.CMMString = _T("Faro");
	app.saarm.SettingsFileName = _T("SA_FaroSerial_ArmSettings.bin");
	LogFileName = _T("Faro.log");

	mouseModeOn = FALSE;
	leftWasDown=false;
	rightWasDown=false;
}

FaroArm::~FaroArm()
{

}											  
bool FaroArm::PopCommSettingsDlg()
{
	DSerialPortSettings dlg(app.saarm.arm_FaroSerial);
	dlg.m_CommPort = app.saarm.Data.CommPort-1;
	dlg.CommBaud = app.saarm.Data.CommBaud;
	dlg.faro_com_param_comboval = app.saarm.Data.commParams;
	if(DOMODAL(dlg) != IDOK)
		return false;
	app.saarm.Data.CommPort = dlg.m_CommPort+1;
	app.saarm.Data.CommBaud = dlg.CommBaud;
	app.saarm.Data.commParams = dlg.faro_com_param_comboval;
	return true;
}
bool FaroArm::ConnectToArm(const int dof)
{
	if(!app.saarm.Data.doconnection)	// 2001.03.23
		return true;	// simulation mode

	if(!app.saarm.AutoSocket.Valid)
		if(!PopCommSettingsDlg())
			return false;

	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);
	JShift.Create(dof,TRUE);

	int InstrumentModel = app.saarm.InstrumentModel;

	if(InstrumentModel == 5)	// old faro
	{
		Vector rot( 90,0,180);
		NativeArmToSA.SetRotation(rot);

		JShift.x[0] = 2.3388;
		JShift.x[1] = 5.0;
		JShift.x[2] = -132.6490;
		JShift.x[3] =  -87.4549;
		JShift.x[4] = -13.3394;
		JShift.x[5] = 2.0178-180.0;
	}
	if(InstrumentModel == 10)	// S08
	{
		Vector rot( 90,0,180);
		NativeArmToSA.SetRotation(rot);

		JShift.x[0] = -0.472591;
		JShift.x[1] = -25.707538;
		JShift.x[2] = 88.640503;
		JShift.x[3] = 191.557312;
		JShift.x[4] = -89.740216;
		JShift.x[5] = 6.564743;
	}
	if(InstrumentModel == 26)	// S12
	{
		Vector rot( 90,0,180);
		NativeArmToSA.SetRotation(rot);
		//done roughly for SN S12029600495
		JShift.x[0] = -0.472591;
		JShift.x[1] = -25.707538;
		JShift.x[2] = 95.0;//88.640503;
		JShift.x[3] = 191.557312;
		JShift.x[4] = -89.740216;
		JShift.x[5] = 6.564743;
	}
	if(InstrumentModel == 25 || InstrumentModel == 27 || InstrumentModel == 40)	// G08-02 or G12-02 or G04-02
	{
		Vector rot( 90,0,180);
		NativeArmToSA.SetRotation(rot);
		//gold series G08 RJS  7/7/99
		JShift.x[0] = 1.382011;
		JShift.x[1] = -23.159181;
		JShift.x[2] = 89.622977;
		JShift.x[3] = 207.977470;
		JShift.x[4] = -92.084127;
		JShift.x[5] = 5.271514;
	}
	if(InstrumentModel == 34 || InstrumentModel == 35 || InstrumentModel == 41)	// G08-05 or G12-05 or G04-05(7 dof)
	{
		Vector rot( 90,0,180);
		NativeArmToSA.SetRotation(rot);
		JShift.x[0] = 1.382011;
		JShift.x[1] = -23.159181;
		JShift.x[2] = 89.622977;
		JShift.x[3] = 207.977470;
		JShift.x[4] = -92.084127;
		JShift.x[5] = 9.5;//5.271514;
		JShift.x[6] = -80.0;
	}

	return true;
}
BOOL FaroArm::DisconnectFromHardware()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.CloseComm();

	return TRUE;
}
BOOL FaroArm::ProcessXYZIJKPacket(IJKPacket* Data)	// 2002.03.20 - just IJK now, XYZ in button processor
{
//	OutputSpeakerSound(1000,200);
/*	sscanf(Data->Ppacket.X,"%lf",&X);
	sscanf(Data->Ppacket.Y,"%lf",&Y);
	sscanf(Data->Ppacket.Z,"%lf",&Z);
//	TRACE3("IJKPacket, From Faro: X=%f,Y=%f,Z=%f millimeters\n",X,Y,Z);
	X /= 1000.0f;
	Y /= 1000.0f;
	Z /= 1000.0f;

	Vector FaroCoord( X,Y,Z ); //raw data
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	NativeArmToSA.Multiply( FaroCoord, &AnalyzerCoord );
	X = AnalyzerCoord.x;
	Y = AnalyzerCoord.y;
	Z = AnalyzerCoord.z;
//	TRACE3("IJKPacket, SA coords: X=%f, Y=%f, Z=%f inches\n", X*app.saarm.LocalToInches,Y*app.saarm.LocalToInches,Z*app.saarm.LocalToInches);
*/
	double A,B,C;
//	TRACE0("ProcessXYZIJKPacket\n");
	sscanf(Data->Ppacket.A,"%lf",&A);
	sscanf(Data->Ppacket.B,"%lf",&B);
	sscanf(Data->Ppacket.C,"%lf",&C);
//	TRACE3("A=%f,B=%f,C=%f\n",A,B,C);

	Transform FaroXForm;
	FaroXForm.SetZXZEulerRotation(Vector(A,B,C));
	CurrentTransform.MultiplyRot(NativeArmToSA, FaroXForm);
	CurrentTransform.SetTranslation(Vector(X,Y,Z));

	Vector ijk, k(0.0,0.0,1.0);
	CurrentTransform.MultiplyRot(k, &ijk);
	
	// Testing!!
//	MeasToSpan(ijk);	// should make pts on a 1 meter radius sphere centered around the instrument
							// each pt. will be located such that it 'points' in the same direction as the probe tip

	II = ijk.x;
	JJ = ijk.y;
	KK = ijk.z;

//	TRACE3("IJK: i=%f, j=%f, k=%f\n", II,JJ,KK);

	Delay(0.005, TRUE);	// 2001.04.13 upsetting, but I need this for the 'dual' sampling
								// i.e. joints, points, and IJK's to work!
	return TRUE;
}
BOOL FaroArm::ProcessButtonsandUpdatePacket(JointPacket* AData)
{
	//OutputSpeakerSound(2000,50);
//	TRACE0("ProcessButtonsandUpdatePacket\n");

	// mouse
	if(mouseModeOn)
	{
		oldFaroCoord = FaroCoord;
	}
	DWORD mouseFlags = 0L;

	sscanf(AData->Opacket.X,"%lf",&X);
	sscanf(AData->Opacket.Y,"%lf",&Y);
	sscanf(AData->Opacket.Z,"%lf",&Z);
	//TRACE3("JointPacket, From Faro: X=%f,Y=%f,Z=%f millimeters\n",X,Y,Z);
	X /= 1000.0f;
	Y /= 1000.0f;
	Z /= 1000.0f;

	FaroCoord = Vector( X,Y,Z ); //raw data
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	NativeArmToSA.Multiply( FaroCoord, &AnalyzerCoord );
	X = AnalyzerCoord.x;
	Y = AnalyzerCoord.y;
	Z = AnalyzerCoord.z;
	TRACE3("JointPacket, SA coords: X=%f, Y=%f, Z=%f inches\n", X*app.saarm.LocalToInches,Y*app.saarm.LocalToInches,Z*app.saarm.LocalToInches);

//	double J7;
	sscanf(AData->Opacket.J0,"%lf",&J.x[0]);
	sscanf(AData->Opacket.J1,"%lf",&J.x[1]);
	sscanf(AData->Opacket.J2,"%lf",&J.x[2]);
	sscanf(AData->Opacket.J3,"%lf",&J.x[3]);
	sscanf(AData->Opacket.J4,"%lf",&J.x[4]);
	sscanf(AData->Opacket.J5,"%lf",&J.x[5]);
	if( DOF == 7 )
		sscanf(AData->Opacket.J6,"%lf",&J.x[6]);
	sscanf(AData->Opacket.Temp,"%lf",&Temperature);

//	TRACE1("Joint 7 angle = %f\n", J.x[6]);

	if( firstshot )
	{
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		firstshot = FALSE;
		CString m;
		m.Format(_T("%s %s"), app.saarm.CMMString, app.VString);
		d.SetWindowText(m);
		ConnectedToArm = TRUE;
	}

//	TRACE1("Faro says button is %c\n", Data->Opacket.Error[6]);
	// move this to GetButtonStates
	if(!overrideArmButtons && app.pA->armButtons.ButtonsEnabled())
	{
		switch(AData->Opacket.Error[6])  //check for button hits
		{
		case '0':  //no buttons
			{  
				buttonstate = 0;
				buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
				
				if(leftWasDown)
				{
					leftWasDown = false;
					mouseFlags |= MOUSEEVENTF_LEFTUP;
				}
				if(rightWasDown)
				{
					rightWasDown = false;
					mouseFlags |= MOUSEEVENTF_RIGHTUP;
				}
				if( LastButton == 1 && !LongDown)
				{
					buttonstate = 1;
					if(!IsRemoteMouseEnabled())
					{
						buttonstate += BUTTON_SHORT;
						Beep(900,100);
					}
				}
				if( LastButton == 2 && !LongDown)
				{
					buttonstate = 2;
					if(!IsRemoteMouseEnabled())
					{
						buttonstate += BUTTON_SHORT;
						Beep(600,100);
					}
				}
				LastButton = 0;
				LongDown = FALSE;
	//			TRACE0("LongDown just set to FALSE!\n");
				break;
			}
		case '1':  // rear button
			{  
				buttonstate = 2;
				if( LastButton == 1 && !mouseModeOn)
				{
					buttonstate = 1;
					if(!IsRemoteMouseEnabled())
					{
						buttonstate += BUTTON_SHORT;
					}
					buttontime = 4294967295;
					LastButton = 2;
				}
				if( LastButton != 2 )
				{
					buttontime = ::GetCurrentTime();
					LastButton = 2;
					if(mouseModeOn)
					{
						rightWasDown = true;
						mouseFlags |= MOUSEEVENTF_RIGHTDOWN;
						if(app.saarm.Data.backButtonCentersMouse)
						{
							TRACE0("Call moouse_event to center cursor...\n");
							mouse_event(
								MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE,
								32767,	// center - see documentation
								32767,	// center - see documentation
								0x0,		// unused, reserved for future use, set to zero
								0x0 		// 32 bits of application-defined information	
								);
						}
					}
				}
				else if( LastButton == 2 )
				{
					if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
					{
						buttonstate += BUTTON_LONG;
						buttontime = 4294967295;
						LastButton = 0;
						LongDown = TRUE;
	//					TRACE0("LongDown just set to TRUE, back button.\n");
					}
				}
				break;
			}
		case '2':	//front button
			{  
				// 2002.03.19 begin hack - get data with probe 'down'
				// Renshaw works nicely without this...
	//			if( app.saarm.Data.FaroAuxOn && Measuring )
	//			{
	//				buttonstate = 2;
	//				buttonstate += BUTTON_SHORT;
	//				buttontime = 4294967295;
	//				LastButton = 1;
	//				break;
	//			}
				// 2002.03.19 end hack
				buttonstate = 1;
				if( LastButton == 2 )
				{
					buttonstate = 2;
					if(!IsRemoteMouseEnabled())
					{
						buttonstate += BUTTON_SHORT;
					}
					buttontime = 4294967295;
					LastButton = 1;
				}
				if( LastButton != 1 )
				{
					buttontime = ::GetCurrentTime();
					LastButton = 1;
					// Cannot handle the left mouse button here, it freezes the arm,
					// had to move left button handling to DPCMMDlg button handler
					//if(mouseModeOn && !leftWasDown)
					//{
					//	leftWasDown = true;
					//	mouseFlags |= MOUSEEVENTF_LEFTDOWN;
					//}
				}
				else if( LastButton == 1 )
				{
	//				TRACE("is it a long button hit?\n");
	//				if( LongDown )
	//					TRACE("LongDown = TRUE!!\n");
					if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
					{
						buttonstate += BUTTON_LONG;
						buttontime = 4294967295;
						LastButton = 0;
						LongDown = TRUE;
						MessageBeep(-1);
						Beep(1200,200);
	//					TRACE("LongDown just set to TRUE, front button.\n");
	//					TRACE("Yes, it is.\n");
					}
				}
				break;
			}
		}
	}

//	TRACE1("buttonstate = %d\n", buttonstate);

	J.x[1] = -J.x[1]; //the only one that doesn't reverse
	if( DOF == 7 )		// 2001.04.17, this one also does not reverse
		J.x[6] = -J.x[6];
	for(int i=0;i<DOF;i++)	// 2001.04.12 was hard coded to i<6
		J.x[i] = -J.x[i] ;    //to correct for shift and sign of joints
	
//	TRACE0("Joint Angles:\n");
	for(int i=0;i<DOF;i++)	// 2001.04.12 was hard coded to i<6
	{
		J.x[i] += JShift[i];    //to correct for shift and sign of joints
//		TRACE2("J[%d] = %f\n", i, J.x[i]);
	}
//	TRACE1("Temperature = %f C\n", Temperature);
//	TRACE0("\n");

	//SEND EVENTS TO MOUSE DRIVER
	if(mouseModeOn)
	{
		//TRACE0("Call moouse_event to move cursor...\n");
		mouse_event(
			MOUSEEVENTF_MOVE | mouseFlags,	// flags specifying various motion/click variants    	
			DWORD((FaroCoord.z-oldFaroCoord.z)*app.saarm.Data.mouseGain),
			DWORD((oldFaroCoord.y-FaroCoord.y)*app.saarm.Data.mouseGain),
			0x0,	// unused, reserved for future use, set to zero
			0x0 	// 32 bits of application-defined information	
			);
	}

	Delay(0.005, TRUE);	// 2001.04.13 upsetting, but I need this for the 'dual' sampling
								// i.e. joints, points, and IJK's to work!

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////////////
BOOL FaroArm::GetAllData()
{
	if( !ConnectedToArm )
		return ArmBase::GetAllData();

	if(app.StopNow)
		return FALSE;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	d.WriteHead("_O");
	d.HoldForResponse(0.1f);

	if(app.StopNow)
		return FALSE;
	d.WriteHead("_P");
	d.HoldForResponse(0.1f);

	return TRUE;
}
BOOL FaroArm::GetPos()
{
	if( !ConnectedToArm )
		return ArmBase::GetPos();
	
	return TRUE;
}
BOOL FaroArm::GetJoints(bool simulate/*=true*/)
{
	if(app.StopNow)
		return FALSE;
	if( !ConnectedToArm )
		return ArmBase::GetJoints(simulate);
	
	return TRUE;
}
//void FaroArm::MoveTheMouse()
//{
//	DWORD dwFlag = 0L;
//
//	//INITIALIZE BUTTON FLAGS	
//	static BYTE bFrontWasDown=FALSE;
//	static BYTE bBackWasDown=FALSE;
//
//	//PUT POINT INTO PROBE'S SYSTEM AND COMPUTE TRAVEL VECTOR
//	static Vec3 vOldPoint=(Vec3)pPosUpdate->lfPosition / theRemoteConfig.scale_unit;
//	Vec3 vTest, vProbe;
//
//	vProbe.p[0] = (sin(   (PosUpdate.lfPosition[B]*(M_PI/180.0)) )*sin(  (PosUpdate.lfPosition[A]*(M_PI/180.0)) ) );
//	vProbe.p[1] = (-sin(PosUpdate.lfPosition[B]*(M_PI/180.0))*cos(PosUpdate.lfPosition[A]*(M_PI/180.0)) );
//	vProbe.p[2] = (cos(PosUpdate.lfPosition[B]*(M_PI/180.0)) );
//
//	int nAxis;
//
//	if((theRemoteConfig.armdata[0]=='S')&&(theRemoteConfig.armdata[1]=='A')){
//		vTest.p[2] = 1.0;
//		Vec3 vHorz = crossprd(vProbe,vTest);
//		ProbeSystem.Matrix.col(0,vHorz);
//		ProbeSystem.Matrix.col(1,crossprd(vTest,vHorz));
//		ProbeSystem.Matrix.col(2,vTest);
//		nAxis = 2;
//	}
//	else{
//		vTest.p[1] = 1.0;
//		Vec3 vHorz = crossprd(vProbe,vTest);
//		ProbeSystem.Matrix.col(0,vHorz);
//		ProbeSystem.Matrix.col(1,vTest);
//		ProbeSystem.Matrix.col(2,crossprd(vHorz,vTest));
//		nAxis = 1;
//	}
//
//	ProbeSystem.Matrix=transpose(ProbeSystem.Matrix);	
//	Vec3 vDistPoint=product(ProbeSystem.Matrix,((Vec3)pPosUpdate->lfPosition / theRemoteConfig.scale_unit)-vOldPoint);
//	long dwMoveX = (int)((double)(vDistPoint.p[0]*((double)(theLocalConfig.dwArmSensitivity)/6.5535)));
//	long dwMoveY = (int)((-1.0)*(double)(vDistPoint.p[nAxis]*((double)(theLocalConfig.dwArmSensitivity)/6.5535)));
//	vOldPoint=(Vec3)pPosUpdate->lfPosition / theRemoteConfig.scale_unit;
//
//	//BACK BUTTON
//	switch(theLocalConfig.bBackButtonMode){
//		case BACKBUTTON_CENTERS:
//			if(BackButton){
//				mouse_event(
//					MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE,	// flags specifying various motion/click variants
//					32767,	// horizontal mouse position or position change
//					32767,	// vertical mouse position or position change
//					0x0,		// unused, reserved for future use, set to zero
//					0x0 		// 32 bits of application-defined information	
//					);
//				return;
//			}
//			break;
//		case BACKBUTTON_AS_RIGHT:
//			if(PosUpdate.cBackButton && !bBackWasDown){
//				dwFlag|=MOUSEEVENTF_RIGHTDOWN;
//				bBackWasDown=TRUE;
//			}
//			else if(bBackWasDown && !PosUpdate.cBackButton){
//				dwFlag|=MOUSEEVENTF_RIGHTUP;
//				bBackWasDown=FALSE;
//			}
//			break;
//		case BACKBUTTON_AS_CENTER:
//			if(PosUpdate.cBackButton && !bBackWasDown){
//				dwFlag|=MOUSEEVENTF_MIDDLEDOWN;
//				bBackWasDown=TRUE;
//			}
//			else if(bBackWasDown && !PosUpdate.cBackButton){
//				dwFlag|=MOUSEEVENTF_MIDDLEUP;
//				bBackWasDown=FALSE;
//			}
//			break;
//	};
//
//	//FRONT BUTTON								
//	if(PosUpdate.cFrontButton && !bFrontWasDown){ 
//		dwFlag|=MOUSEEVENTF_LEFTDOWN;
//		bFrontWasDown=TRUE;
//	}
//	else if(bFrontWasDown && !PosUpdate.cFrontButton){ 
//		dwFlag|=MOUSEEVENTF_LEFTUP;
//		bFrontWasDown=FALSE;
//	}
//	else if((abs(dwMoveX)>1)||(abs(dwMoveY)>1)){	
//		dwFlag|=MOUSEEVENTF_MOVE;		
//	}
//
//	//SEND EVENTS TO MOUSE DRIVER
//	mouse_event(
//		dwFlag,	// flags specifying various motion/click variants    	
//		(theLocalConfig.bFlipAxis&SWAP_AXIS?(((theLocalConfig.bFlipAxis & FLIP_HORZ) ? (dwMoveY) : (-dwMoveY))):(((theLocalConfig.bFlipAxis & FLIP_HORZ) ? (-dwMoveX) : (dwMoveX)))),
//		(theLocalConfig.bFlipAxis&SWAP_AXIS?(((theLocalConfig.bFlipAxis & FLIP_VERT) ? (-dwMoveX) : (dwMoveX))):(((theLocalConfig.bFlipAxis & FLIP_VERT) ? (-dwMoveY) : (dwMoveY)))),
//		0x0,	// unused, reserved for future use, set to zero
//		0x0 	// 32 bits of application-defined information	
//		);
//}
// not calling this (& therefore CheckButton) any more,
// no need unless I can figure out how to handle long duration
// for now, buttonstate is set in ProcessButtonsandUpdatePacket
int FaroArm::GetButtonStates()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonStates();

	int duration=0;
	buttonstate = 0;

	if( CheckButton(1, &duration) )
	{
		buttonstate = 1;
		buttonstate += duration;
	}
	if( CheckButton(2, &duration) )
	{
		buttonstate = 2;
		buttonstate += duration;
	}
	return buttonstate;
}
BOOL FaroArm::CheckButton(int but, int* duration)
{
	if( !ConnectedToArm )
		return ArmBase::CheckButton(but, duration);

//	if(buttondown == but)
//	{
//		DWORD Start = ::GetCurrentTime();
//		while((buttondown == but)&&((::GetCurrentTime() - Start) < LONG_DURATION))
//			ProcessWaitingMessages();
//		if(buttondown == but)
//			*duration = BUTTON_LONG;
//		else
			*duration = BUTTON_SHORT;
//	}
//	else
//		return FALSE;
	return TRUE;
}

BOOL FaroArm::GetTemp()
{
	if( !ConnectedToArm )
		return ArmBase::GetTemp();
	
	return TRUE;
}

void FaroArm::ToggleArmButtonBeep()
{
	TRACE0("Toggle Arm Button Beep.\n");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.WriteHead("_S0 1 0100 0700");
	Delay(0.5,TRUE);
}

void FaroArm::SetAuxPort(BOOL on)
{
	TRACE0("Set Aux port.\n");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(on)
		d.WriteHead("_L1");
	else
		d.WriteHead("_L0");
	d.HoldForResponse(0.1f);
}

int FaroArm::GetProbeNumber()
{
	return 0;
}

int FaroArm::GetButtonCount()
{
	return 2;
}
///////////////////////////////////////////////
// mouse mode
BOOL FaroArm::IsRemoteMouseEnabled()
{
	return mouseModeOn;
}
void FaroArm::ToggleRemoteMouse()
{
	RemoteMouse(!mouseModeOn);
}
void FaroArm::RemoteMouse(BOOL flag)
{
	mouseModeOn = flag;
}
////////////////////////////////////////////////////////////////////////////
// 2004.09.21 - settings
void FaroArm::DoArmSettings()
{
	if(!ConnectedToArm)
	{
		ArmBase::DoArmSettings();
		return;
	}
	DSettingsFaroSerial dlg;
	dlg.mouseModeOn = mouseModeOn;
	dlg.sliderSetting = app.saarm.Data.mouseGain;
	dlg.backButtonCentersMouse = app.saarm.Data.backButtonCentersMouse;
	if(DOMODAL(dlg) == IDOK)
	{
		mouseModeOn= dlg.mouseModeOn;
		app.saarm.Data.mouseGain = dlg.sliderSetting;
		app.saarm.Data.backButtonCentersMouse = dlg.backButtonCentersMouse;
	}
}
/////////////////////////////////////////////////////////////////////////////////
BOOL FaroArm::IsRomer()
{
	return FALSE;
}
BOOL FaroArm::IsFaro()
{
	return TRUE;
}
BOOL FaroArm::IsFaroUSB()
{
	return FALSE;
}
BOOL FaroArm::IsFrenchRomer()
{
	return FALSE;
}
BOOL FaroArm::IsSNL()
{
	return FALSE;
}
BOOL FaroArm::IsMetrisGarda()
{
	return FALSE;
}