#include "stdafx.h"
#include "../DPCMM.h"
#include ".\farousbcallback.h"

FaroUSBCallBack::FaroUSBCallBack(FaroUSBArm& _parent)
: parent(_parent)
, scannerCS(*new CriticalSectionWrapper)

{
}

FaroUSBCallBack::~FaroUSBCallBack(void)
{
	delete &scannerCS;
}

BEGIN_MESSAGE_MAP(FaroUSBCallBack, CStatic)
	//{{AFX_MSG_MAP(CPreviewWnd)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// FaroUSBCallBack message handlers
void FaroUSBCallBack::OnStart()
{
	TRACE0(">>>>>>FaroUSBCallBack::OnStart()\n");
}

void FaroUSBCallBack::OnEnd()
{
	TRACE0(">>>>>>FaroUSBCallBack::OnEnd() Leaving scan mode.\n");
	if(parent.lineScanning)
	{
		//TRACE0("Send ScanPassStopped\n");
		parent.lineScanning = false;
		//parent.SendOnScanPassStopped(); // but this is really when we have left scan mode, does not happen at end of each pass

		if(app.saarm.Data.autoIncCloudName)
		{
			app.saarm.Data.Target.Format(_T("P1"));
			app.pA->IncrementName(app.saarm.Data.Group, 1);
		}

		//app.pA->SetMeasMode(0);

		//if(Units::toolBarModeOn)
		//{
		//	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		//	d.RestartMeasurement();
		//}
	}
}

void FaroUSBCallBack::OnPositionSampled(const CScanData& ScanData)
{
	if(ScanData.m_nPoints <= 0)	// 2010.02.25 - have not seen this w/previous drivers, but this one (v.5.6.4.3) can trigger this with no points
		return;

	static long tick = 0;
	static long count = 0;
	int n=10;
	count ++;
	//TRACE1("have %d pts. 1st one is...\n", ScanData.m_nPoints);
	//TRACE3("x=%f, y=%f, z=%f\n", ScanData.m_parrPointData->m_dData[0], ScanData.m_parrPointData->m_dData[1], ScanData.m_parrPointData->m_dData[2]);
	//Vector xyzFaro(ScanData.m_parrPointData[0].m_dData[0], ScanData.m_parrPointData[0].m_dData[1], ScanData.m_parrPointData[0].m_dData[2]);
	//xyzFaro.Scale(1.0/1000.0);
	//Vector xyzSA;
	//parent.NativeArmToSA.Multiply( xyzFaro, &xyzSA );
	//parent.MeasToSpan(xyzSA);

	if(app.saarm.Data.useScanStripeClouds && parent.performingStdScannerMeas)
	{
		parent.AddScannerCloudStripe();

		SAIPScanStripePoint sspt;
		Vector xyzFaro, xyzSA;

		for(int j=0; j<ScanData.m_nPoints; j++)
		{
			if(j % app.saarm.Data.thinningFactor != 0)	// 2006.03.02
				continue;
			xyzFaro.x = ScanData.m_parrPointData[j].m_dData[0];
			xyzFaro.y = ScanData.m_parrPointData[j].m_dData[1];
			xyzFaro.z = ScanData.m_parrPointData[j].m_dData[2];
			xyzFaro.Scale(1.0/25.4);
			parent.NativeArmToSA.Multiply( xyzFaro, &xyzSA );

			sspt.x = xyzSA.x;
			sspt.y = xyzSA.y;
			sspt.z = xyzSA.z;
			parent.AddScannerCloudStripePt(sspt);
		}
	}
	else
	{
		SAIPCloudBlockPt cbpt;
		Vector xyzFaro, xyzSA;

		//TRACE1("adding %d pts to cloud\n", ScanData.m_nPoints);

		for(int j=0; j<ScanData.m_nPoints; j++)
		{
			if(j % app.saarm.Data.thinningFactor != 0)	// 2006.03.02
				continue;
			xyzFaro.x = ScanData.m_parrPointData[j].m_dData[0];
			xyzFaro.y = ScanData.m_parrPointData[j].m_dData[1];
			xyzFaro.z = ScanData.m_parrPointData[j].m_dData[2];
			xyzFaro.Scale(1.0/25.4);
			parent.NativeArmToSA.Multiply( xyzFaro, &xyzSA );
			cbpt.x = xyzSA.x;
			cbpt.y = xyzSA.y;
			cbpt.z = xyzSA.z;
			parent.AddScannerCloudPt(cbpt);
		}
	}

	Vector FaroCoord( ScanData.m_ArmXYZ[0]/1000.0f,
		ScanData.m_ArmXYZ[1]/1000.0f,
		ScanData.m_ArmXYZ[2]/1000.0f );
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	parent.NativeArmToSA.Multiply( FaroCoord, &AnalyzerCoord );
	parent.X = AnalyzerCoord.x;
	parent.Y = AnalyzerCoord.y;
	parent.Z = AnalyzerCoord.z;
	//TRACE3("Arm XYZ from Faro Scanner callback...X=%f, Y=%f, Z=%f\n", parent.X, parent.Y, parent.Z);

	Transform FaroXForm;
	FaroXForm.SetZXZEulerRotation(Vector(ScanData.m_ArmABC[0], ScanData.m_ArmABC[1], ScanData.m_ArmABC[2]));
	parent.CurrentTransform.MultiplyRot(parent.NativeArmToSA, FaroXForm);
	parent.CurrentTransform.SetTranslation(Vector(parent.X, parent.Y, parent.Z));

	Vector ijk, k(0.0,0.0,1.0);
	parent.CurrentTransform.MultiplyRot(k, &ijk);

	if(app.saarm.Data.useScanStripeClouds && parent.performingStdScannerMeas)
	{
		CriticalSectionLock lock( scannerCS );

		Transform scannerXform = parent.CurrentTransform;
		scannerXform.Scale(app.saarm.LocalToInches);
		parent.scanStripePkt.SetCurrentLineTransform(scannerXform);
	}
}

// 2016.05.13 - added guts to the following to update SA graphics view when in scanning mode, but not actively scanning...
static int ctr = 0;
void FaroUSBCallBack::OnPositionChanged(const CScanData& ScanData)
{
	ctr++;
	if(ctr < 50)
	{
		return;
	}
	else
	{
		ctr = 0;
	}
	//TRACE0("OnPositionChanged\n");

	Vector FaroCoord( ScanData.m_ArmXYZ[0]/1000.0f,
		ScanData.m_ArmXYZ[1]/1000.0f,
		ScanData.m_ArmXYZ[2]/1000.0f );
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	parent.NativeArmToSA.Multiply( FaroCoord, &AnalyzerCoord );
	parent.X = AnalyzerCoord.x;
	parent.Y = AnalyzerCoord.y;
	parent.Z = AnalyzerCoord.z;
	//TRACE3("Arm XYZ from Faro Scanner callback...X=%f, Y=%f, Z=%f\n", parent.X, parent.Y, parent.Z);

	Transform FaroXForm;
	FaroXForm.SetZXZEulerRotation(Vector(ScanData.m_ArmABC[0], ScanData.m_ArmABC[1], ScanData.m_ArmABC[2]));
	parent.CurrentTransform.MultiplyRot(parent.NativeArmToSA, FaroXForm);
	parent.CurrentTransform.SetTranslation(Vector(parent.X, parent.Y, parent.Z));

	Vector ijk, k(0.0,0.0,1.0);
	parent.CurrentTransform.MultiplyRot(k, &ijk);

	parent.GetJoints();
	parent.UpdateToSpan();

	//CDC* pDC = GetDC();
	//RECT rect;
	//GetClientRect(&rect);
	//pDC->SelectObject(GetStockObject(BLACK_BRUSH));
	//pDC->Rectangle(&rect);

	//if(ScanData.m_nPoints)
	//{
	//	CPointData Center;
	//	for(int k=0; k<3; k++)
	//	{
	//		Center.m_dData[k] = 0;
	//	}
	//	for(int i=0; i<ScanData.m_nPoints; i++)
	//	{
	//		CPointData* pPointData = &ScanData.m_parrPointData[i];
	//		for(int k=0; k<3; k++)
	//		{
	//			Center.m_dData[k] += pPointData->m_dData[k];
	//		}
	//	}
	//	for(k=0; k<3; k++)
	//	{
	//		Center.m_dData[k] /= ScanData.m_nPoints;
	//	}
	//	for(i=0; i<ScanData.m_nPoints; i++)
	//	{
	//		CPointData* pPointData = &ScanData.m_parrPointData[i];
	//		for(int k=0; k<3; k++)
	//		{
	//			pPointData->m_dData[k] -= Center.m_dData[k];
	//		}
	//		int x = pPointData->m_dData[0] * 5 + (rect.left + rect.right) / 2;
	//		int y = pPointData->m_dData[2] * 5 + (rect.top + rect.bottom) / 2;
	//		pDC->SetPixel(x, y, RGB(255, 0, 0));
	//	}
	//}

	//ReleaseDC(pDC);
}

void FaroUSBCallBack::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	// TODO: Add your message handler code here
	RECT rect;
	GetClientRect(&rect);
	dc.SelectObject(GetStockObject(BLACK_BRUSH));
	dc.Rectangle(&rect);

	// Do not call CStatic::OnPaint() for painting messages
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

FaroUSBRawEvents::FaroUSBRawEvents(FaroUSBArm& _parent)
: parent(_parent)
{
}
FaroUSBRawEvents::~FaroUSBRawEvents(void)
{

}

// Overrides
void FaroUSBRawEvents::OnDeviceRawEvent(const CScanData& ScanData, const BUTTON_STATUS& eButtonStatus)
{
	if(!parent.IsLineScanning())
		return;

	switch(eButtonStatus)
	{
		case BUTTON_NONE:
		{
			//TRACE0("NO BUTTON\n");
			break;
		}
		case BUTTON_BACK:
		{
			//TRACE0("BUTTON BACK\n");
			break;
		}
		case BUTTON_FRONT:
		{
			//TRACE0("BUTTON FRONT\n");
			break;
		}
	}
}
