//////////////////////////////////////////////////////////////////////////////
//  File:      CmmSpecific.h
//
//  Summary:   Contains definition of structures that are passed as
//			   parameters to the CmmSpecific() method.
//---------------------------------------------------------------------
//  This file is part of the USB FaroArm SDK Code Samples.
//
//  Copyright (C) 2006 FARO Technologies, Inc.  All rights reserved.
//
//  FARO Technologies, Inc. grants you a non-exclusive, royalty-free
//  license to copy, modify, and distribute the software solely for
//  the purpose of facilitating interoperability between your software
//  and the FARO Technologies portable measurement devices.
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE
// AND NON-INFRINGEMENT.  WE DISCLAIM ALL LIABILITY FOR ANY LOSS OR DAMAGE
// SUFFERED BY YOUR OR ANY THIRD PARTY AS A RESULT OF USING OR DISTRIBUTING
// THIS SOFTWARE, INCLUDING DIRECT, INDIRECT, INCIDENTAL, SPECIAL OR
// CONSEQUENTIAL DAMAGES (INCLUDING LOST SAVINGS, LOST PROFIT OR ATTORNEY
// FEES), HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, EVEN IF
// FARO HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//////////////////////////////////////////////////////////////////////////////

#ifndef _CMMSPECIFIC_H
#define _CMMSPECIFIC_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif


#define BIT00 0x00000001
#define BIT01 0x00000002
#define BIT02 0x00000004
#define BIT03 0x00000008
#define BIT04 0x00000010
#define BIT05 0x00000020

#define CMM_UPDATE_DETECT			BIT00        
#define CMM_TIMEOUT_DETECT			BIT01
#define CMM_ERROR_DETECT			BIT02
#define CMM_MOUSE_DETECT			BIT03
#define CMM_INACCURATE_DATA_DETECT	BIT04
#define CMM_AUX_BUTTON_PUSH			BIT05

// distance measure mode values
#define DISTANCE_MODE_INTERFEROMETER_ONLY			1
#define DISTANCE_MODE_INTERFEROMETER_SET_BY_ADM		2
#define DISTANCE_MODE_ADM_ONLY						3
#define DISTANCE_MODE_USER_SPECIFIED				4

// Laser Tracker Models
#define Tracker4000			1
#define Tracker4500			2		//  4000 with adm 
#define TrackerKeystoneI	3
#define TrackerKeystoneSI	4		// keystone with adm
#define TrackerKeystoneS	5
#define TrackerKeystoneX	6
#define TrackerKeystoneXI	7

#define MAX_POINTS_FOR_A_SCAN 1024  // needed for laser line scanner, scr-1854
#define GOOD_KEY					0xc1c2c3c4 // used to tell the device that there is enough space in the buffer

#define DLL_OPEN_CMM				1		//Connect
#define DLL_CLOSE_CMM				2		//Disconnect
#define DLL_ARM_INFO				3
//#define DLL_PROBE_DIAM			4
#define DLL_POS_UPDATE				5
#define DLL_END_UPDATE				6
#define DLL_DIALOG_CMM				7
#define DLL_PROBE_DIALOG			9 
#define DLL_CALIBRATE_ENTER			10
//#define DLL_CALIBRATE_HOLE			11
//#define DLL_CALIBRATE_SPHERE		12
//#define DLL_SET_PROBE				13
#define DLL_MOUSE_MODE_ON			14
#define DLL_MOUSE_MODE_OFF			15
#define DLL_KEY_MODE_ON				16
#define DLL_KEY_MODE_OFF			17
#define DLL_SETTINGS_DIALOG			18		//HardwareConfig
#define DLL_TEMPERATURE_DIALOG		19
#define DLL_ANGLES_DIALOG			20
#define DLL_TERMINAL_DIALOG			21
#define DLL_HELP_DIALOG				22
#define DLL_UNITS_IN				23
#define DLL_UNITS_MM				24
#define DLL_READ_POINTS				25
#define DLL_SoftCheck               26
#define DLL_START_MEASUREMENT		27
#define DLL_STOP_MEASUREMENT		28
#define DLL_TRACKERPAD				29	//start tracker pad application
#define DLL_SAMPLINGSETUP_DIALOG	30	
#define DLL_TRACKERRESET			31
#define DLL_TRACKERAIM				32
//#define DLL_SEARCH_DIALOG			33	// NOT USED
#define DLL_TRACKERHOME				34	//When select, positions the tracker to home position
#define DLL_TRACKERHEALTHCHECK		35	//start health check application 
#define DLL_TRACKEROPERATIONALCHECK	36	//start operational check application 
#define DLL_TRACKERCOMPIT			37	//start compit application.
#define DLL_GET_PROBE_INFO			38
#define DLL_SET_PROBE_INFO			39
#define DLL_GET_SAMPLE_SETUP_INFO	40
#define DLL_SET_SAMPLE_SETUP_INFO	41
#define DLL_GET_SEARCH_INFO			42
#define DLL_SET_SEARCH_INFO			43
#define DLL_SEARCH					44
#define DLL_GET_AUTO_RESET_INFO		45
#define DLL_SET_AUTO_RESET_INFO		46
#define DLL_GET_HOME_INFO			47
#define DLL_CLOSURECHECK			48
#define DLL_MEASURE_LEVEL			49
#define DLL_SET_DISTANCE_MODE		50
#define DLL_SET_HARDWARE_CONFIG		51
#define DLL_GET_HARDWARE_CONFIG		52
#define DLL_INITIALIZE				53  // open without connecting
#define DLL_START_MEASUREMENT_CMD	54
#define DLL_STOP_MEASUREMENT_CMD	55
#define DLL_GET_DEVICE_SETUP		56	// what model, adm?
#define DLL_POINT_FRONT_BACK_SIGHT	58
#define DLL_READ_DEVICE				60
#define DLL_GET_AUXILIARY_PROBE		61
#define DLL_SET_AUXILIARY_PROBE		62
#define DLL_GET_MACHINE_MODE		63
#define DLL_SET_MACHINE_MODE		64
#define DLL_DEVICE_INFO				66


// 100 to 200 are reserved for the Laser Tracker
#define DLL_SET_INSTALLATION_DIRECTORY 100


// 1001-2000: reserved for laser scanner 
#define DLL_SCANNER_CONTROL			1001		// invoke scanner control dialog
#define DLL_GET_PROBE_TYPE			1002		// get probe type.: 0: hard probe, 1, touch probe, 2, line laser probe
#define DLL_GET_PROBE_CALIBRATION_INFO	1003	// get probe calibration info
#define DLL_SET_PROBE_CALIBRATION_INFO	1004	// set probe calibration info 
#define DLL_MEASUREMENT_MODE		1005		// enter reguallar measurement mode. ( i.e, EnableProbe(TRUE))
#define DLL_CALIBRATION_MODE		1006		// enter calibration mode (i.e. EnablbeProbe(FALSE))
#define DLL_GET_LASER_ON			1007		// get laser on/off
#define DLL_SET_LASER_ON			1008		// set laser on/off
#define DLL_GET_LINE_SCANNER_ENABLED 1009		// get scanner enabled
#define DLL_SET_LINE_SCANNER_ENABLED 1010		// set scanner enabled, if enabled, the active probe is laser probe, otherwise, the active probe is hard probe
//#define DLL_CALIBRATE_PLANE			 1011

// 2001-3000 reserved for Tracker Arm Combination
#define DLL_GET_COMBO_CONFIG		2001
#define DLL_GET_TRANSFORMATION_MATRIX		 2002

#pragma pack(push, 8)

//EXPORTED FUNCTIONS
int CmmSpecific(void *);


//*********
//TYPE DEFS
//*********

struct vTransformationMatrix
{
	int		nFuncType; // set to DLL_GET_TRANSFORMATION_MATRIX
	double dMatrix[3][3]; // angular 
	double dOrigin[3]; // origin 
	double dScale[4]; // not used, but included just in case

	vTransformationMatrix():
		 nFuncType(DLL_GET_TRANSFORMATION_MATRIX)
	{
	}
};

struct vAuxiliaryProbe
{
	int		nFuncType;	// = DLL_GET_AUXILIARY_PROBE or DLL_SET_AUXILIARY_PROBE
	long	bEnable;
	long	lDebounce;
	double	dMinAssertTime;
};

typedef struct vDeviceSetup
{
	int		nFuncType;
	BOOL	bAdmCapable;
	BOOL	bAdmOnly;		// does the tracker require ADM mode
	int		nModel;			// 1 is 4000, 2 is 4500( 4000 with adm ), 3 is keystone i, 4 is keystone si ( has adm )
	int		nDistanceMode;	// same as SetDistanceMeasurementMode
	vDeviceSetup() :
		nFuncType(DLL_GET_DEVICE_SETUP),
		bAdmCapable(FALSE),
		bAdmOnly(FALSE),
		nModel(0),
		nDistanceMode(0)
	{
	}
} vDeviceSetup;

///////////////////////////////////////////////////////////////////////////////
typedef struct vGetHardwareConfig
{
	int nFuncType;
	BOOL bInvokeHardwareConfig; 
	WCHAR sConfig[500];
	vGetHardwareConfig() :
		nFuncType(DLL_GET_HARDWARE_CONFIG),
		bInvokeHardwareConfig(FALSE)
	{
		ZeroMemory(sConfig, 500*sizeof(WCHAR));
	}
} vGetHardwareConfig;
///////////////////////////////////////////////////////////////////////////////
typedef struct vSetHardwareConfig
{
	int nFuncType;
	BOOL bInvokeHardwareConfig; 
	WCHAR sConfig[500];
	vSetHardwareConfig() :
		nFuncType(DLL_SET_HARDWARE_CONFIG),
		bInvokeHardwareConfig(FALSE)
	{
		ZeroMemory(sConfig, 500*sizeof(WCHAR));
	}
} vSetHardwareConfig;
///////////////////////////////////////////////////////////////////////////////
typedef struct vDistanceMode
{
	int nFuncType;
	int nMode;
	vDistanceMode() :
		nFuncType(DLL_SET_DISTANCE_MODE),
		nMode(0)
	{
	}
} vDistanceMode;
//////////////////////////////////////////////////////////////////////////////////
typedef struct vMeasureLevel
{
	int nFuncType;
	double	dX;			// x point of the plane
	double	dY;			// y point of the plane
	double	dZ;			// z point of the plane
	double	dI;			// i of the plane
	double	dJ;			// j of the plane
	double	dK;			// k of the plane
	vMeasureLevel() :
		nFuncType(DLL_MEASURE_LEVEL),
		dX(0.0),
		dY(0.0),
		dZ(0.0),
		dI(0.0),
		dJ(0.0),
		dK(0.0)
	{
	}
} vMeasureLevel;

typedef enum
{
	enSightNA = 0,				// point not taken by a laser
	enSightFront = 1,			// fore ground point for a measurement, use front sight
	enSightFrontAndBack = 2,	// fore ground point for a measurement, use front and back
	enSightBack = 3,			// fore ground point for a measurement, use back sight
	enCompPoint = 4,			// fore ground comp point
	enTimeOfReading = 5			// time that reading was taken ( scr-3310 )
}PointType;

typedef enum
{
	enPositionUpdate = 0,		// just a plane position, not a reading nor comp point
	enReading,
	enCompensationPoint,
	enUnknown					// an error of some sort
}UpdateType;

// CAUTION: the USB Wrapper has put two longs into dPosition[7] ( the 8th location ), the number of end
// stops and lTime
typedef struct vPositionUpdate
{
	int 		nFuncType;
	char		cUpdateFlag;
	unsigned	uFnumber;
	char		cFrontButton;
	char		cBackButton;
	double		dPosition[8];
	HANDLE		hPositionUpdateEvent;
	UINT		unKey;
	int			nPnts;
	double		dPoints[MAX_POINTS_FOR_A_SCAN][3];
	PointType	ePointSightType[MAX_POINTS_FOR_A_SCAN];
	vPositionUpdate() :
		nFuncType(DLL_POS_UPDATE),
		cUpdateFlag(0),
		uFnumber(0),
		cFrontButton(0),
		cBackButton(0),
		unKey(GOOD_KEY),
		hPositionUpdateEvent(NULL)
	{
			for(int i=0;i<8;i++)
			{
				dPosition[i] = 0.0;
			}
			// CAUTION: the USB driver does not use the point sight type array, so we must initialize it here
			for(int i=0; i < MAX_POINTS_FOR_A_SCAN; i++)
			{
				ePointSightType[i] = enSightNA;
			}
	}
} vPositionUpdate;

typedef struct vArmInfo
{
	int		nFuncType;
	char	sArmData[32];
	char	sCertificationDate[16];
	char	sCalibrationDate[16];
	int		nSerindVersion;
	float	fUnits;
	vArmInfo():
		nFuncType(DLL_ARM_INFO),
		nSerindVersion(0),
		fUnits(0.0f)
		{
			ZeroMemory(sArmData, 32*sizeof(char));
			ZeroMemory(sCertificationDate, 16*sizeof(char));
			ZeroMemory(sCalibrationDate, 16*sizeof(char));
		}
} vArmInfo;

typedef struct vOpenInfo
{
	int		nFuncType;
	HWND	hWnd;
	TCHAR	sSaveDir[128];
	void (*pHelpCallBack)();
	vOpenInfo():
		nFuncType(DLL_OPEN_CMM),
		hWnd(NULL),
		pHelpCallBack(NULL)
	{
		ZeroMemory(sSaveDir, 128*sizeof(TCHAR));
	}
		
} vOpenInfo;

typedef struct vInitDevice
{
	int  nFuncType;
	HWND hWnd;
	TCHAR sSaveDir[128];
	void (*pHelpCallBack)();
	vInitDevice() :
		nFuncType(DLL_INITIALIZE),
		hWnd(NULL),
		pHelpCallBack(NULL)
	{
		ZeroMemory(sSaveDir, 128*sizeof(TCHAR));
	}
} vInitDevice;

typedef struct vKeyMode
{
	int nFuncType;
	int nFrontButton;
	int nBackButton;
	vKeyMode():
		nFuncType(DLL_KEY_MODE_ON),
		nFrontButton(VK_RETURN),
		nBackButton(VK_CANCEL)
		{
		}
} vKeyMode;

typedef struct vStartMeasurement
{
	int			nFuncType;
	int			nPointsToTake;  // 1 for regular measure, more than 1 for scan
	PointType	nPointType;
	vStartMeasurement() :
		nFuncType(DLL_START_MEASUREMENT),
		nPointsToTake(0),
		nPointType(enSightNA)
	{
	}
} vStartMeasurement;

typedef struct vXYZPosition 
{
	int		nFuncType;
	double	dX;
	double	dY;
	double	dZ;
	BOOL	bIsBackSight;	//Pass in FALSE for now.
	BOOL	bIsRelative;	//Pass in FALSE for now.
	vXYZPosition():
		nFuncType(-1),		// user must fill this in
		dX(0.0),
		dY(0.0),
		dZ(0.0),
		bIsBackSight(FALSE),
		bIsRelative(FALSE)
		{
		}
} vXYZPosition;

typedef struct vGetSearchRadius
{
	int nFuncType;
	double dRadius;
	vGetSearchRadius() :
		nFuncType(DLL_GET_SEARCH_INFO),
		dRadius(0.0)
	{
	}
} vGetSearchRadius;

typedef struct vSearch
{
	int nFuncType;
	double dRadius;
	vSearch() :
		nFuncType(DLL_SEARCH),
		dRadius(0.0)
	{
	}
} vSearch;

typedef struct vSampleSetup {
	int		nFuncType;
	int		nWatchWinUpdate;
	int		nMeasurement;
	int		nCompPoint;
	int		nScanPoint;
	vSampleSetup():
		nFuncType( DLL_GET_SAMPLE_SETUP_INFO),
		nWatchWinUpdate(0),
		nMeasurement(0),
		nCompPoint(0),
		nScanPoint(0)
		{
		}
} vSampleSetup;

/* vGetProbeInfo
	if the Laser Tracker is running. 
	1. SMR only:
	dProbeDiameter = diameter of the SMR
	dAdapterHeight = 0 because no adapter was used. You can test if you are using an adapter by
					 checking if dAdapterHeight == 0 

	2. SMR and flat nest:
	dProbeDiameter = diameter of the adapter. This diameter is from the center of SMR to the 
					 outside edge of adapter.
	dAdapterHeight = height of adapter. This height is from the center of the SMR to the
					 bottom of the adapter.

	3. SMR and cylindrical pin nest:
	dProbeDiameter = diameter of the pin on the adapter.
	dAdapterHeight = height of adapter. This height is from the center of the SMR to the
					 bottom of the adapter. NOTE! this is not the bottom of the pin.

	If the faro arm is running.
	dProbeDiameter = diameter of the probe.
	dAdapterHeight = 0
	sProbeName = name of the probe, not the index.
	sAdapterName = "" 
*/
typedef struct vGetProbeInfo
{
	int nFuncType;
	double dProbeDiameter;
	double dAdapterHeight;
	double dEndEffectorError;
	TCHAR sProbeName[128];
	TCHAR sAdapterName[128];
	vGetProbeInfo() :
		nFuncType(DLL_GET_PROBE_INFO),
		dProbeDiameter(0.0),
		dAdapterHeight(0.0),
		dEndEffectorError(0.0)
	{
		ZeroMemory(sProbeName, 128*sizeof(TCHAR));
		ZeroMemory(sAdapterName, 128*sizeof(TCHAR));
	}
} vGetProbeInfo;

/*
	dProbeDiameter and dAdapterHeight. If you pass in values, you want to overwrite the values
	found in the probe file. If they are zero, the defaults from the file are used.
*/
typedef struct vSetProbeInfo {
	int nFuncType;
	double dProbeDiameter;
	double dAdapterHeight;
	TCHAR sProbeName[128];
	TCHAR sAdapterName[128];
	vSetProbeInfo():
		nFuncType(DLL_SET_PROBE_INFO),
		dProbeDiameter(0.0),
		dAdapterHeight(0.0)
	{
		ZeroMemory(sProbeName, 128*sizeof(TCHAR));
		ZeroMemory(sAdapterName, 128*sizeof(TCHAR));
	}
} vSetProbeInfo;

struct vProbeDlgInfo {
	int nFuncType;
	TCHAR sProbeFileName[300];
	struct vSetProbeInfo setProbe;
};

typedef struct vGetAutoResetInfo
{
	int nFuncType;
	BOOL bAutoReset;   // true if Auto Reset is ON
	BOOL bHomeIsReset; // true if Reset Point is HOME, may or may not be auto reset
	struct vXYZPosition xyz;
	vGetAutoResetInfo() :
		nFuncType(DLL_GET_AUTO_RESET_INFO),
		bAutoReset(FALSE),
		bHomeIsReset(FALSE)
	{
	}
} vGetAutoResetInfo;

typedef struct vSetAutoResetInfo
{
	int nFuncType;
	BOOL bAutoReset;   // true if Auto Reset is ON
	BOOL bHomeIsReset; // true if Reset Point is HOME, may or may not be auto reset
	struct vXYZPosition xyz;
	vSetAutoResetInfo() :
		nFuncType(DLL_SET_AUTO_RESET_INFO),
		bAutoReset(FALSE),
		bHomeIsReset(FALSE)
	{
	}
} vSetAutoResetInfo;


typedef struct vHome
{
	int nFuncType;
	BOOL bUseBacksight;
	vHome():
		nFuncType(DLL_TRACKERHOME),
		bUseBacksight(FALSE)
		{
		}

} vHome;

typedef struct vDeviceInfo	
{
	int nFuncType;
	TCHAR VersionInfo[_MAX_PATH];
	TCHAR DeviceDescription[_MAX_PATH];
	vDeviceInfo() :
		nFuncType(DLL_DEVICE_INFO)
	{
		ZeroMemory(VersionInfo, _MAX_PATH * sizeof(TCHAR));
		ZeroMemory(DeviceDescription, _MAX_PATH * sizeof(TCHAR));
	}
}vDeviceInfo,* LPDeviceInfo;
typedef const LPDeviceInfo LPCDeviceInfo;


typedef struct vComboConfig 
{
	int nFuncType;
	BOOL bTrackerIsActive;
	char sArmData[32];    // from USB arm for SCR-4681
	vComboConfig():
		nFuncType( DLL_GET_COMBO_CONFIG),
		bTrackerIsActive(FALSE)
		{
			ZeroMemory(sArmData, 32*sizeof(char));
		}
} vComboConfig;

#define ProbeCalBadError 12344.678 
struct vProbeCalibrationInfo 
{
	int nFuncType;// =DLL_GET_PROBE_CALIBRATION_INFO or DLL_SET_PROBE_CALIBRATION_INFO
	double dX;
	double dY;
	double dZ;
	double dA;
	double dB;
	double dC;
	double dError; // pStruct->dError = g_ProbeInfoFromServer.d2SigmaError/2.0;
	long lTime;
	vProbeCalibrationInfo():
		dX(0.0),
		dY(0.0),
		dZ(0.0),
		dA(0.0),
		dB(0.0),
		dC(0.0),
		dError(ProbeCalBadError+1.0),
		lTime(0)
		{
		}
};

struct vMouseMode
{
	int nFuncType;
	vMouseMode():
		nFuncType(DLL_MOUSE_MODE_ON)
		{
		}
};

////////////////////////////////////////////////////////////////////////////////
typedef struct vSetInstallationDirectory	//DLL_SET_INSTALLATION_DIRECTORY
{
	int nFuncType;
	TCHAR InstallationDirectory[_MAX_PATH];

	vSetInstallationDirectory() :
		nFuncType(DLL_SET_INSTALLATION_DIRECTORY)
	{
		ZeroMemory(InstallationDirectory,_MAX_PATH * sizeof(TCHAR));
	}
}SetInstallationDirectory,* LPSetInstallationDirectory;
typedef const LPSetInstallationDirectory LPCSetInstallationDirectory;

//================================================================================================
#pragma pack(pop)

#endif //_CMMSPECIFIC_H

