// \PCMM Arms\FaroUSB\TrackArm\DTrackArm.cpp : implementation file
//

#include "stdafx.h"
#include "../../nrkapp.h"
#include "../../DPCMM.h"
#include "../../ArmBase.h"
#include "DTrackArm.h"
#include "TrackArmInterface.h"

// DTrackArm dialog

IMPLEMENT_DYNAMIC(DTrackArm, CDialog)

DTrackArm::DTrackArm(CWnd* pParent /*=NULL*/)
: CDialog(DTrackArm::IDD, pParent)
, trackArm(*new TrackArmInterface(*app.pA))
, makeCopyOfArm(FALSE)
{

}

DTrackArm::~DTrackArm()
{
	delete &trackArm;
}

void DTrackArm::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_BUTTON_CONNECT, connectButton);
	DDX_Control(pDX, IDC_BUTTON_LOCATE, locateButton);
	DDX_Control(pDX, IDC_BUTTON_TRACKER_CONTROL, trackerButton);
	DDX_Control(pDX, IDC_BUTTON_SELECT_TRACKARM_TRACKER, selectTrackerButton);
	DDX_Radio(pDX, IDC_RADIO_COPY_INST, makeCopyOfArm);
}


BEGIN_MESSAGE_MAP(DTrackArm, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, &DTrackArm::OnBnClickedButtonConnect)
	ON_BN_CLICKED(IDC_BUTTON_LOCATE, &DTrackArm::OnBnClickedButtonLocate)
	ON_BN_CLICKED(IDC_BUTTON_TRACKER_CONTROL, &DTrackArm::OnBnClickedButtonTrackerControl)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BUTTON_SELECT_TRACKARM_TRACKER, &DTrackArm::OnBnClickedButtonSelectTrackarmTracker)
END_MESSAGE_MAP()


// DTrackArm message handlers

BOOL DTrackArm::OnInitDialog()
{
	CDialog::OnInitDialog();

	if(app.saarm.trackArmTrackerInstSerialNumber == 0)
	{
		connectButton.EnableWindow(FALSE);
		makeCopyOfArm = FALSE;
	}
	else
	{
		connectButton.EnableWindow(TRUE);
		makeCopyOfArm = TRUE;
	}

	UpdateData(FALSE);

	locateButton.EnableWindow(FALSE);
	trackerButton.EnableWindow(FALSE);

	SetIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME), FALSE);
	TRANSLATE_DIALOG;


	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DTrackArm::OnBnClickedButtonSelectTrackarmTracker()
{
	if(app.saarm.SelectTrackArmTrackerFromSA())
	{
		// light up the connect button
		connectButton.EnableWindow(TRUE);
	}
}

void DTrackArm::OnBnClickedButtonConnect()
{
	connectButton.EnableWindow(FALSE);
	locateButton.EnableWindow(FALSE);
	trackerButton.EnableWindow(FALSE);

	//CWaitCursor wait;
	if(!TurnTrackArmOn())
	{
		connectButton.EnableWindow(TRUE);
	}
	else
	{
		locateButton.EnableWindow(TRUE);
		trackerButton.EnableWindow(TRUE);
	}
}

void DTrackArm::OnBnClickedButtonLocate()
{
	if(app.pA->trackArmActive)
	{
		Transform tkArmXform;
		if(trackArm.LocateArm(tkArmXform))
		{
			selectTrackerButton.EnableWindow(FALSE);

			Transform adjustForSA, ToTrackerFrame(0.0, 0.0, 180.0);
			app.pA->NativeArmToSA.Invert(&adjustForSA);
			// map Faro's transform to SA arm 
			tkArmXform = tkArmXform * adjustForSA;
			// now map new SA arm into SA tracker
			tkArmXform = ToTrackerFrame * tkArmXform;

			UpdateData(TRUE);
			app.saarm.OnTrackArmTransformChanged(tkArmXform, makeCopyOfArm);
		}
		return;
	}
}

void DTrackArm::OnBnClickedButtonTrackerControl()
{
	if(app.pA->trackArmActive)
	{
		trackArm.DoTrackerSettings();
		return;
	}
}

void DTrackArm::OnClose()
{
	connectButton.EnableWindow(FALSE);
	locateButton.EnableWindow(FALSE);
	trackerButton.EnableWindow(FALSE);
	if(!TurnTrackArmOff())
	{
		connectButton.EnableWindow(TRUE);
	}
	else
	{
		Sleep(250);
		CDialog::OnClose();
	}
}
////////////////////////////////////////////////////////////////////////////
// TrackArm
bool DTrackArm::TurnTrackArmOn()
{
	if(app.pA->trackArmActive)
		return true;

	NRKProgress prog(Tr("Connecting TrackArm"));
	prog.Start(Tr("Disconnecting Arm..."));
	prog.SetMsg(Tr("Disconnecting Arm..."));

	if(app.pA->DisconnectFromHardware())
	{
		app.pA->ConnectedToArm = false;
		prog.SetMsg(Tr("Connecting TrackArm..."));
		if(trackArm.ConnectTrackArm())
		{
			app.pA->trackArmActive = true;
			return true;
		}
	}

	return false;
}

bool DTrackArm::TurnTrackArmOff()
{
	if(!app.pA->trackArmActive)
		return true;

	NRKProgress prog(Tr("Disconnecting TrackArm"));
	prog.Start(Tr("Disconnecting TrackArm..."));
	prog.SetMsg(Tr("Disconnecting TrackArm..."));

	if(trackArm.DisconnectTrackArm())
	{
		app.pA->trackArmActive = false;
		prog.SetMsg(Tr("Connecting Arm..."));
		if(app.pA->ConnectToArm(app.pA->DOF))
		{
			app.pA->ConnectedToArm = true;
			return true;
		}
	}

	return false;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
