#include "stdafx.h"
#include "../../nrkapp.h"
#include "../../DPCMMDlg.h"
#include "../../SAArm.h"
#include "TrackArmInterface.h"
#include "../../FaroUSBArm.h"
/*
blt 2009.09.25
Did not need to add NEARLY all the functionality that is in this class.
It supports measurement from the arm or even the tracker via the old C-style
cmmspecific that Faro implemented for this.  (same as was used for the old
serial arms).  But anyway, it's all here in case it is ever needed.....
*/

TrackArmInterface::TrackArmInterface(ArmBase& _parent)
: m_hArmLib(NULL)
, m_pCmmSpecific(NULL)
, m_bOpened(FALSE)
, m_nPrev(0)
, m_bUseBackSight(FALSE)
, m_nDistanceMode(DISTANCE_MODE_INTERFEROMETER_SET_BY_ADM)	// defined in the new cmmspecific
, parent(_parent)
, m_nInvalidUpdates(0)
{
}

TrackArmInterface::~TrackArmInterface(void)
{
}

bool TrackArmInterface::ConnectTrackArm()
{
	// see if the devices directory exists; if it does not, then the link output directory of the application
	// is not appropriate
	CString strDevicesPathName( app.saarm.ProgramDirectory + _T("Devices"));
	if( !PathFileExists(strDevicesPathName))
	{
		AfxMessageBox(_T("Devices directory not found. Invalid installation for TrackArm."));
		return false;
	}

	// load the TrackArm DLL ( which was originally called the Combo Device
	// The TrackArm DLL does a LoadLibrary on Devices\FaroArmUsbWrapper.dll and Devices\FaroTracker.dll
	// Unfortunately, if OPEN fails, there's not much you can tell about why it failed, or even which device failed
	// Just invoke LoadDevice with the suspect DLL, and see what happens.
	//LoadDevice(_T("Devices\\FaroArmUsbWrapper.dll"));
	//LoadDevice(_T("Devices\\FaroTracker.dll"));
	CString FaroComboPath(strDevicesPathName + _T("\\FaroCombo.dll"));
	if( LoadDevice(FaroComboPath) )
	{		
		// give the driver the shared buffer; it will put in the event
		// m_UpdateStruct.unKey == GOOD_KEY tells USB devices that the dPositions array is large and we want it filled
		if(!m_pCmmSpecific((void *)&m_UpdateStruct))
		{
			AfxMessageBox(_T("BAD POSITION UPDATE"));
		}

		m_bOpened = TRUE;
	}

	return true;
}
bool TrackArmInterface::LoadDevice( LPCTSTR DllName )
{
	m_hArmLib = LoadLibrary( DllName );

	if(NULL == m_hArmLib)
	{
		LPVOID lpMsgBuf;
		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			0, // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
			);
		AfxMessageBox( (LPCTSTR)lpMsgBuf, MB_ICONEXCLAMATION | MB_OK );
		LocalFree( lpMsgBuf );


		//CString errMess;
		//errMess.Format(_T("Could not load library '%s'."), DllName);
		//AfxMessageBox(errMess, MB_ICONERROR);
		return false;
	}
	else
	{
		// get the procedure address of the one and only procedure, CmmSpecific. 
		// Just like old Faro serial stuff -- takes an address of the parameter structure
		m_pCmmSpecific = (BOOL (*)(VOID *))GetProcAddress(m_hArmLib, "CmmSpecific");
		if(NULL == m_pCmmSpecific)
		{
			AfxMessageBox(_T("Could not find CmmSpecific Procedure"));
			return false;
		}
	}

	SetInstallationDirectory SID;

	// will let the user browse for this, C:\FaroJRE is our default location...
	//CString tkrInstallDir(_T("C:\\FaroJRE"));
	//_tcscpy(SID.InstallationDirectory, tkrInstallDir );

	// Cannot let user browse for the tracker JRE -- we have to have our own special local copy
	// This will make deployment fun...
	CString tkrInstallDir;
	int nIndex = app.saarm.ProgramDirectory.ReverseFind(_T('\\'));
	if(-1 != nIndex)
	{
		tkrInstallDir = app.saarm.ProgramDirectory.Left(nIndex+1);
	}

	_tcscpy(SID.InstallationDirectory, tkrInstallDir );
	if( !m_pCmmSpecific((void *)&SID))
	{
		AfxMessageBox(_T("BAD SET INSTALL DIRECTORY IN OPEN"));
	}

	// must invoke initialize to set up all the other directories
	vInitDevice vInit;	
	_tcscpy(vInit.sSaveDir, app.saarm.ProgramDirectory.GetBuffer(128) );
	if(!m_pCmmSpecific((void *)&vInit))
	{
		AfxMessageBox(_T("BAD INIT"));
		return FALSE;
	}

	// MUST set inches in order to initialize the probe
	if( !SetUnits( TRUE ))
	{
		AfxMessageBox(_T("BAD INCHES IN LoadDevice"));
	}

	// the first time this executes, the IP address of the tracker will be wrong
	// this creates the LaserTracker.xml file in the Devices directory
	if(!DoTrackerSettings())
	{
		AfxMessageBox(_T("BAD Configure"));
	}

	// OPEN takes the IP address of the tracker and actually connects to it
	vOpenInfo vOpen;
	_tcscpy(vOpen.sSaveDir, app.saarm.ProgramDirectory.GetBuffer(128) );		
	if(!m_pCmmSpecific((void *)&vOpen))
	{
		AfxMessageBox(_T("BAD OPEN"));

		// try a close just in case one of them opened
		DisconnectTrackArm();
		return false;
	}

	// MUST set inches again
	if( !SetUnits(true))
	{
		AfxMessageBox(_T("BAD INCHES IN LoadDevice (2)"));
	}

	// get the tracker's information
	vDeviceInfo di;
	if( m_pCmmSpecific((void *)&di))
	{
		//m_UpdatesListBox.AddString( di.DeviceDescription);
		//m_UpdatesListBox.AddString( di.VersionInfo);
	}
	else
	{
		AfxMessageBox(_T("BAD Device INFO"));
	}

	// set tracker to ifm set by adm - unusable if it happens to be adm only
	vDistanceMode vDM;
	vDM.nMode = DISTANCE_MODE_INTERFEROMETER_SET_BY_ADM;
	m_pCmmSpecific((void *)&vDM);

	m_bTrackerActive = IsTrackerActive();
	return true;
}

bool TrackArmInterface::DisconnectTrackArm()
{
	if(NULL != m_pCmmSpecific)
	{
		// Since there are no parameters, just pass the address of the integer
		int nCmmRoutine = DLL_CLOSE_CMM;
		if( !m_pCmmSpecific( (void *)&nCmmRoutine ))
		{
			AfxMessageBox(_T("Failed to close the TrackArm Driver"), MB_ICONSTOP);
		}
		else
		{
			if( NULL != m_hArmLib  )
			{
				if(!FreeLibrary( m_hArmLib ))
					return false;
			}
		}
	}
	return true;
}

// did not need to do all this work, but here it is, in case it's ever needed
BOOL TrackArmInterface::GetAllData()
{
	// reset the event so that the devices can fill the update buffer
	ResetEvent(m_UpdateStruct.hPositionUpdateEvent);

	m_bTrackerActive = IsTrackerActive();

	// wait for the device to update the data and set the event. Usually, the devices do this
	// several hundred times a second, but there may be a significant delay on the first one, and it
	// depends on how many laser observations are used to generate an update		
	int nWaitTime(1000);
	// we're always in single point mode. user may be using many laser observations to make one update, 
	//nWaitTime += nMeasurementUpdates; // approximately 1 millisecond per laser observation

	PointType pt(enSightFront);

	// tell the usb arm to enable the "End Stop" dialog
	// the tracker always fails this command, so don't test for failure; or test for Tracker
	if( !m_bTrackerActive )
	{
		if( !StartMeasurementCommand())
		{
			AfxMessageBox(_T("Could not start measurement command"));
			return 0;
		}
	}

	// if the active device is a USB Arm, StartMeasurement fails
	if( m_bTrackerActive )
	{
		Delay(0.5,TRUE);
		if( !GetSampleSetup())
		{
			AfxMessageBox(_T("BAD GetSampleUpdate"));
			return 0;
		}
		if( !StartMeasurement( 1, pt ))
		{
			Beep(1000,50);
			//AfxMessageBox(_T("Could not start a measurement"));
			return 0;
		}
	}

	int nPointsTaken(0);
	BOOL bStopMeasurement(TRUE);

	double dCurrentX, dCurrentY, dCurrentZ;
	dCurrentX=dCurrentY=dCurrentZ = 0.0;

	Vector arm, sa;

	bool goodMeas = false;
	while(!goodMeas)
	{
		DWORD dwWaitResult = WaitForSingleObject( m_UpdateStruct.hPositionUpdateEvent, nWaitTime );

		// make a local copy to minimize the time the buffer is locked
		vPositionUpdate TmpPosUpdate;
		memcpy(&TmpPosUpdate, &m_UpdateStruct, sizeof(vPositionUpdate));
		m_UpdateStruct.cUpdateFlag = 0; // clear the flag just in case the device does not

		// reset the event so that the active device can refill the update buffer
		ResetEvent(m_UpdateStruct.hPositionUpdateEvent); 
		Sleep(0); // give up the thread

		if( dwWaitResult == WAIT_TIMEOUT )
		{
			// the USB Arm might be at an end stop, so just skip this one
			// if this is a tracker, then the update has not changed, so skip it
			//MessageBox( _T("Time out waiting for position update event"), _T("Internal ERROR"), MB_OK);
			break;
		}
		else if( dwWaitResult != WAIT_OBJECT_0 )
		{
			//CString strError;strError.Format(_T("Wait Result NEQ WAIT_OBJECT_0 %ld"),dwWaitResult);
			//AfxMessageBox(strError);
			Beep(1000,50);
			break;
			//continue;
		}

		// ensure that processing the buffer takes less than 3 milliseconds so that the next buffer
		// does not "stack up"

		if( !CheckForValidity( TmpPosUpdate ))
		{
			if( m_nInvalidUpdates++ > 10 )
			{
				break; // just get out
			}
			//CString strTemp; strTemp.Format(_T("Invalid update buffer %ld"), m_nInvalidUpdates);
			//m_UpdatesListBox.AddString(strTemp);
			continue; // some errors are transient, so ignore them
		}

		goodMeas = true;

		dCurrentX = TmpPosUpdate.dPosition[0];
		dCurrentY = TmpPosUpdate.dPosition[1];
		dCurrentZ = TmpPosUpdate.dPosition[2];

		//TRACE3("Arm ABC:\n A=%f, B=%f, C=%f\n", TmpPosUpdate.dPosition[3], TmpPosUpdate.dPosition[4], TmpPosUpdate.dPosition[5]);

		parent.II = (sin(TmpPosUpdate.dPosition[4]) * sin(TmpPosUpdate.dPosition[3]));
		parent.JJ = (-sin(TmpPosUpdate.dPosition[4]) * cos(TmpPosUpdate.dPosition[3]));
		parent.KK = cos(TmpPosUpdate.dPosition[4]);

		TRACE3("SA ijk:\n i=%f, j=%f, k=%f\n", parent.II, parent.JJ, parent.KK);

		// get their transform here, and put the data back into the arm's coordinate system
		
		arm.x = dCurrentX;
		arm.y = dCurrentY;
		arm.z = dCurrentZ;

		//transform to analyzer coords
		//parent.NativeArmToSA.Multiply( arm, &sa );
		sa = arm;
		parent.X = sa.x/app.saarm.LocalToInches;
		parent.Y = sa.y/app.saarm.LocalToInches;
		parent.Z = sa.z/app.saarm.LocalToInches;

		TRACE3("SA xyz:\n x=%f, y=%f, z=%f\n\n", sa.x, sa.y, sa.z);

		// Button States...
		// implemented for the 'parent' to be the FaroUSB object...
	//	if(parent.overrideArmButtons)
	//		return TRUE;
	//	if(!(TmpPosUpdate.cFrontButton) && !(TmpPosUpdate.cBackButton))  //no buttons
	//	{  
	//		parent.buttonstate = 0;
	//		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
	//		if( parent.LastButton == 1 && !LongDown)
	//		{
	//			parent.buttonstate = 1;
	//			//if(!IsRemoteMouseEnabled())
	//			//{
	//				parent.buttonstate += BUTTON_SHORT;
	//				Beep(900,100);
	//			//}
	//		}
	//		if( parent.LastButton == 2 && !LongDown)
	//		{
	//			//parent.buttonstate = 2;
	//			//if(!IsRemoteMouseEnabled())
	//			//{
	//				parent.buttonstate += BUTTON_SHORT;
	//				Beep(600,100);
	//			//}
	//		}
	//		if(parent.LastButton == 3 && parent.Measuring)	// to support pausing a stream (see DGenMeas::ButtonHit)
	//		{
	//			parent.buttonstate = 4;
	//			//if(!IsRemoteMouseEnabled())
	//			//{
	//				parent.buttonstate += BUTTON_LONG;
	//			//}
	//		}

	//		parent.LastButton = 0;
	//		LongDown = FALSE;
	//	}
	//	if(TmpPosUpdate.cBackButton)
	//	{  
	//		TRACE0("Rear button hit\n");
	//		parent.buttonstate = 2;
	//		if( parent.LastButton == 1 || parent.LastButton == 3 )
	//		{
	//			parent.buttonstate = 1;
	//			//if(!IsRemoteMouseEnabled())
	//			//{
	//				parent.buttonstate += BUTTON_SHORT;
	//			//}
	//			buttontime = 4294967295;
	//			parent.LastButton = 2;
	//		}
	//		if( parent.LastButton != 2 )
	//		{
	//			buttontime = ::GetCurrentTime();
	//			parent.LastButton = 2;
	//		}
	//		else if( parent.LastButton == 2 )
	//		{
	//			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
	//			{
	//				parent.buttonstate += BUTTON_LONG;
	//				buttontime = 4294967295;
	//				parent.LastButton = 0;
	//				LongDown = TRUE;
	//				//	TRACE0("LongDown just set to TRUE, back button.\n");
	//			}
	//		}
	//	}
	//	else if(TmpPosUpdate.cFrontButton)
	//	{  
	//		TRACE0("Front button hit\n");
	//		parent.buttonstate = 1;
	//		if( parent.LastButton == 2 || parent.LastButton == 3 )
	//		{
	//			parent.buttonstate = 2;
	//			//if(!IsRemoteMouseEnabled())
	//			//{
	//				parent.buttonstate += BUTTON_SHORT;
	//			//}
	//			buttontime = 4294967295;
	//			parent.LastButton = 1;
	//		}
	//		if( parent.LastButton != 1 )
	//		{
	//			buttontime = ::GetCurrentTime();
	//			parent.LastButton = 1;
	//		}
	//		else if( parent.LastButton == 1 )
	//		{
	//			if( !parent.LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
	//			{
	//				parent.buttonstate += BUTTON_LONG;
	//				buttontime = 4294967295;
	//				parent.LastButton = 0;
	//				parent.LongDown = TRUE;
	//				MessageBeep(-1);
	//				Beep(1200,200);
	//				TRACE0("LongDown just set to TRUE, front button.\n");
	//			}
	//		}
	//	}
	}

	// tell the USB arm to disable the "End Stop" dialog
	if( !m_bTrackerActive )
	{
		StopMeasurementCommand();
	}

	if( bStopMeasurement )
	{
		StopMeasurement();
	}

	return TRUE;
}
BOOL TrackArmInterface::GetJoints(bool simulate/*=true*/)
{

	return FALSE;
}

bool TrackArmInterface::LocateArm(Transform& trackArmTransform)	
{
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//d.StopQueryTimer();

	int nFunc(DLL_PROBE_DIALOG);
	if(!m_pCmmSpecific((void *)&nFunc))
	{
		return false;
	}

	return GetTrackArmTransform(trackArmTransform);

	// maybe the user changed devices
	//m_bTrackerActive = IsTrackerActive();

	//if(m_bTrackerActive)
	//{
	//	SetSampleSetup(1,1,1,1);
	//}

	//if(IsProbeValid())
	//	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
}
bool TrackArmInterface::SetSampleSetup( int nWatchWindowUpdates, int nMeasurementUpdates, int nCompensationUpdates, int nScanUpdates )
{
	vSampleSetup vSS;
	vSS.nFuncType = DLL_SET_SAMPLE_SETUP_INFO;
	vSS.nCompPoint = nCompensationUpdates;
	vSS.nMeasurement = nMeasurementUpdates;
	vSS.nScanPoint = nScanUpdates;
	vSS.nWatchWinUpdate = nWatchWindowUpdates;

	return m_pCmmSpecific((void *)&vSS) > 0;
}
bool TrackArmInterface::GetSampleSetup()
{
	vSampleSetup vSS;
	if(!m_pCmmSpecific((void *)&vSS))
	{
		return false;
	}

	TRACE2("SampleSetup:\nwatch updates = %d, meas = %d, ", vSS.nWatchWinUpdate, vSS.nMeasurement);
	TRACE2("nCompPoint = %d, nScanPoint = %d\n\n", vSS.nCompPoint, vSS.nScanPoint);

	return true;
}
bool TrackArmInterface::GetTrackArmTransform(Transform& trackArmTransform)
{
	bool bRes(false);
	vTransformationMatrix vMatrix;
	vMatrix.nFuncType = DLL_GET_TRANSFORMATION_MATRIX;
	if( m_pCmmSpecific((void *)&vMatrix))
	{
		trackArmTransform.T[0][0] = vMatrix.dMatrix[0][0];   trackArmTransform.T[0][1] = vMatrix.dMatrix[0][1];   trackArmTransform.T[0][2] = vMatrix.dMatrix[0][2];
		trackArmTransform.T[1][0] = vMatrix.dMatrix[1][0];   trackArmTransform.T[1][1] = vMatrix.dMatrix[1][1];   trackArmTransform.T[1][2] = vMatrix.dMatrix[1][2];
		trackArmTransform.T[2][0] = vMatrix.dMatrix[2][0];   trackArmTransform.T[2][1] = vMatrix.dMatrix[2][1];   trackArmTransform.T[2][2] = vMatrix.dMatrix[2][2];
		
		Vector translation(vMatrix.dOrigin[0], vMatrix.dOrigin[1], vMatrix.dOrigin[2]);
		trackArmTransform.SetTranslation(translation);
		
		TRACE0("Transform:\n");
		trackArmTransform.Debug();
		
		if(trackArmTransform.T[0][0] != 1.0  && trackArmTransform.T[1][1] != 1.0  &&  trackArmTransform.T[2][2] != 1.0)
			bRes = true;
		//TRACE3("Matrix:\n%f %f %f\n", vMatrix.dMatrix[0][0], vMatrix.dMatrix[0][1], vMatrix.dMatrix[0][2]);
	}
	return bRes;
}
bool TrackArmInterface::IsProbeValid()
{
	// tracker probe info
	//bool bRes(false);
	//vGetProbeInfo vTkrProbeInfo;
	//vTkrProbeInfo.nFuncType = DLL_GET_PROBE_INFO;
	//if( m_pCmmSpecific((void *)&vTkrProbeInfo))
	//{
	//	bRes = vTkrProbeInfo.dEndEffectorError != 0.0;
	//}

	// arm Cal. info
	bool bRes = (bool)IsTrackerActive();
	if(!bRes)
	{
		vProbeCalibrationInfo vArmCalInfo;
		vArmCalInfo.nFuncType = DLL_GET_PROBE_CALIBRATION_INFO;
		if( m_pCmmSpecific((void *)&vArmCalInfo))
		{
			bRes = vArmCalInfo.dError < (double)ProbeCalBadError;
		}
	}
	return bRes;
}


BOOL TrackArmInterface::DoTrackerSettings()
{
	// bring up device's configuration dialog
	int nFunc(DLL_SETTINGS_DIALOG);
	return m_pCmmSpecific((void *)&nFunc);
}


////////////////////////////////////////////////////////////////////////
BOOL TrackArmInterface::SetUnits( const bool bInches )
{
	int nFunc(DLL_UNITS_MM);
	if( bInches )
	{
		nFunc = DLL_UNITS_IN;
	}
	return m_pCmmSpecific((void *)&nFunc);
}

BOOL TrackArmInterface::IsTrackerActive()
{
	BOOL bRes(FALSE);
	vComboConfig vConfig;
	if( m_pCmmSpecific(&vConfig))
	{
		bRes = vConfig.bTrackerIsActive;
	}

	return bRes;
}

BOOL TrackArmInterface::StartMeasurementCommand()
{
	int nCmmRoutine = DLL_START_MEASUREMENT_CMD;
	return m_pCmmSpecific( (void *)&nCmmRoutine );
}

BOOL TrackArmInterface::StopMeasurementCommand()
{	
	int nCmmRoutine = DLL_STOP_MEASUREMENT_CMD;
	return m_pCmmSpecific( (void *)&nCmmRoutine );
}

BOOL TrackArmInterface::StartMeasurement( const int nCount, const PointType nType )
{
	vStartMeasurement StartMeasurement;
	StartMeasurement.nPointsToTake = nCount;
	StartMeasurement.nPointType = nType; 

	return m_pCmmSpecific( (void *)&StartMeasurement );
}

BOOL TrackArmInterface::StopMeasurement()
{
	int nFunc(DLL_STOP_MEASUREMENT);
	return m_pCmmSpecific((void *)&nFunc);

}

// CAUTION: Laser Tracker SCAN MODE assumes that all values are from the front sight
//
// NOTE: the Tracker always sends just one position update per reading. The USB arm sends updates
//       continuously, and the button flag is ON if the user is holding it down. Since it takes three
//       milliseconds ( approximately ) to generate one update, and the average human holds a button down
//       for many milliseconds, the USB arm will send many updates with the button push flag. The application
//       needs to detect leading or trailing edge for this situation. The problem is that if the tracker 
//       is in scan mode, all updates are both valid and have the front button flag ON. The application 
//		 must know if it's scan mode so it can detect this situation. If the laser tracker does
//       a "front and back sight" reading, it is still one update buffer with two values.

UpdateType TrackArmInterface::DecodePositionUpdate(vPositionUpdate& TmpPosUpdate)
{
	UpdateType eUpdateType(enPositionUpdate);

	if( TmpPosUpdate.cFrontButton )
	{
		eUpdateType = enReading;
	}								
	else if(TmpPosUpdate.cBackButton )
	{	
		eUpdateType = enCompensationPoint;		
	}	

	return eUpdateType;
}

/*

typedef struct vPositionUpdate
{
int 		nFuncType;			// must be DLL_POS_UPDATE
char		cUpdateFlag;		// 0, CMM_TIMEOUT_DETECT, CMM_INACCURATE_DATA_DETECT, CMM_ERROR_DETECT
unsigned	uFnumber; 
if(TmpPosUpdate.uFnumber & 0xfffc) ERROR IN HARDWARE						
m_bCurrFrontButton	= TmpPosUpdate.uFnumber & 2;
m_bCurrBackButton	= TmpPosUpdate.uFnumber & 1;
char		cFrontButton;		// 0 if not pushed, 1 if button pushed
char		cBackButton;		// 0 if not pushed, 1 if button pushed
double		dPosition[8];
0 is the X value
1 is the Y value
2 is the Z value
3,4, and 5 are the eigenvalues of the angle of the USB probe handle
6 temperature of the USB probe; ignored for Tracker
7 not used by tracker; USB arm does end stops and time in two longs
HANDLE		hPositionUpdateEvent;
UINT		unKey;
int			nPnts;				// number of points in dPoints
double		dPoints[MAX_POINTS_FOR_A_SCAN][3];
PointType	ePointSightType[MAX_POINTS_FOR_A_SCAN]; // the PointType of the value in dPoints
} vPositionUpdate;

*/
BOOL TrackArmInterface::CheckForValidity( vPositionUpdate TmpPosUpdate )
{
	BOOL bRes(TRUE);
	CString strTmp;

	int nUpdateFlag = TmpPosUpdate.cUpdateFlag;		
	if( nUpdateFlag == 0 )
	{	
		// sometimes, this value does not get set; tacky, but OK, just position data
	}
	else if( nUpdateFlag & CMM_UPDATE_DETECT )
	{
		// OK, an update
	}
	else if( nUpdateFlag & CMM_TIMEOUT_DETECT ) 
	{
		AfxMessageBox(  _T("Time out flag set in position update; device malfunction"));			

	}
	else if( nUpdateFlag & CMM_INACCURATE_DATA_DETECT )
	{
		// this is quite common for the Tracker;
		bRes = FALSE;

	}
	else if( TmpPosUpdate.cUpdateFlag & CMM_ERROR_DETECT )
	{
		AfxMessageBox(_T(" Error in Data"));

	}
	else
	{
		CString strTemp; 
		strTemp.Format(_T("Invalid cUpdateFlag %ld"), nUpdateFlag);
		AfxMessageBox(strTemp);
		bRes = FALSE;
	}

	unsigned	uFnumber = TmpPosUpdate.uFnumber; 
	if(	uFnumber & 0xfffc)
	{
		// ERROR IN HARDWARE; this happens a lot in the USB arm, so just ignore the buffer
		bRes = FALSE;
	}

	// for historical reasons, there are two flags for a button push
	if( (uFnumber & 2) && !TmpPosUpdate.cFrontButton)
	{
		AfxMessageBox(_T("Mismatch in front button pushed flags"));
		bRes = FALSE;
	}
	if( (uFnumber & 1) && !TmpPosUpdate.cBackButton)
	{
		AfxMessageBox(_T("Mismatch in back button pushed flags"));
		bRes = FALSE;
	}

	// SCAN mode is defined as taking many fore ground readings as fast as possible for a Laser Tracker; sometimes,
	// as few as 5 laser observations may be used to create one update. See the Tracker configuration dialog to see how this is set up
	// if the device is a USB arm, all points have the same accuracy, so there is no such thing as Foreground or Background
	// Single Point mode for a tracker means that many laser observations will be used for one readings. This may
	// be as high as 2000. The SDK tells the laser tracker to be in SCAN or Single point mode by the number of
	// readings requested ( see TakePoints ). If the number is one, then, obviously, it is single point mode; if
	// the number is greater than 1, then use SCAN mode

	// The Laser Tracker uses the dPoints array to store the back sight reading and the time of the reading
	// PAY ATTENTION: if the Tracker is in SCAN mode, it cannot be using both sights, and the location
	// of the SMR is stored only in dPosition and the only value in dPoints is the time. If the tracker is in
	// single point mode, the the location of the SMR is in both dPosition and in the zero'th position of
	// dPoints. The USB device puts the location of the probe in both dPoints and dPosition if GOOD_KEY was set

	int nPoints = TmpPosUpdate.nPnts;
	if( nPoints == 0 )
	{
		// just a position update, so don't check any more
		return bRes;
	}

	// should never be negative, but check anyways; zero was handled already
	if( nPoints < 1 || nPoints > MAX_POINTS_FOR_A_SCAN )
	{
		strTmp.Format(_T("Invalid number of points %ld"), nPoints);
		AfxMessageBox(strTmp);
		bRes = FALSE;
	}

	for( int nPointIndex = 0; nPointIndex < nPoints; nPointIndex++ )
	{
		int nPointType = TmpPosUpdate.ePointSightType[nPointIndex];
		switch(nPointType)
		{

		case enSightNA: 
			// USB point, so the number of points must be one. Disable this check if you are using the 
			// FARO Laser Scanner, which might have several hundred points in dPoints
			if( nPoints != 1 )
			{
				strTmp.Format(_T("USB Device has an invalid number of points %ld"), nPoints);
				AfxMessageBox(strTmp);
				bRes = FALSE;
			}
			break;

		case enSightFront:
			// Laser Tracker front sight point for a measurement ( ie, foreground laser observation )
		case enSightBack:
			// Laser Tracker back sight point for a measurement ( ie, foreground laser observation )

			if( !(nPoints == 2 || nPoints == 4 ))
			{
				strTmp.Format(_T("Laser Tracker Device has an invalid number of points %ld"), nPoints);
				AfxMessageBox(strTmp);
				bRes = FALSE;
			}
			break;

		case enTimeOfReading:
			break;

		case enSightFrontAndBack:
			// this type is a command to the Laser Tracker, not a return type
			AfxMessageBox(_T("Invalid reading type( cannot be both front and back"));
			bRes = FALSE;
			break;

		case enCompPoint:
			// this type is a command to the Laser Tracker, not a return type
			AfxMessageBox(_T("Invalid reading type( COMP POINT)"));
			bRes = FALSE;
			break;

		default:
			strTmp.Format(_T("Invalid point type %8X"), nPointType);
			AfxMessageBox(strTmp);
			break;
		}
	}

	return bRes;
}
