#ifndef _FARO_SCANNER_API_H
#define _FARO_SCANNER_API_H

#ifdef _FARO_LASER_SCANNER_API
	#define FARO_LASER_SCANNER_API __declspec(dllexport)
#else
	#define FARO_LASER_SCANNER_API __declspec(dllimport)
#endif

enum BUTTON_STATUS
{
	BUTTON_NONE, 
	BUTTON_BACK, 
	BUTTON_FRONT
};

class FARO_LASER_SCANNER_API CPointData  
{
public:
	CPointData();
	virtual ~CPointData();
	double m_dData[3];

};

class FARO_LASER_SCANNER_API CScanData  
{
public:
	CScanData();
	virtual ~CScanData();

	CPointData* m_parrPointData;
	int m_nPoints;
	double m_ArmXYZ[3];
	double m_ArmABC[3];
};

class FARO_LASER_SCANNER_API CCloudData  
{
public:
	CCloudData();
	virtual ~CCloudData();
	
	CScanData* m_parrScanData;
	int m_nScans;
};

class FARO_LASER_SCANNER_API CCalibrationMatrix  
{
public:
	CCalibrationMatrix();
	virtual ~CCalibrationMatrix();

	double m_dData[4][4];
};

class FARO_LASER_SCANNER_API CDataCaptureCallBack  
{
public:
	virtual void OnUndoLast();
	virtual void OnPositionSampled(const CScanData& ScanData);
	virtual void OnPositionChanged(const CScanData& ScanData);
	virtual void OnEnd();
	virtual void OnStart();
	CDataCaptureCallBack();
	virtual ~CDataCaptureCallBack();

};

class FARO_LASER_SCANNER_API CDeviceRawEventCallBack  
{
public:
	CDeviceRawEventCallBack();
	virtual ~CDeviceRawEventCallBack();
	virtual void OnDeviceRawEvent(const CScanData& ScanData, const BUTTON_STATUS& eButtonStatus );
};

class FARO_LASER_SCANNER_API CFaroLaserScanner  
{
public:
	void SetCalibrationMatrix(const CCalibrationMatrix& CalibrationMatrix);
	void GetCalibrationMatrix(CCalibrationMatrix& CalibrationMatrix);
	void CalibrationMode();
	void MeasurementMode();
	void SetDataCaputreCallBack(const CDataCaptureCallBack* pDataCaptureCallBack);
	void SetDeviceRawEventCallBack(const CDeviceRawEventCallBack* pDeviceRawEventCallBack);
	void ScannerControlUI();
	BOOL CaptureData(CCloudData& CloudData);
	void SelectProbeUI();
	void StopDevice();
	BOOL StartDevice();
	// stop data capture
	void EndDataCapture();
// Laser on/off function disabled in v5.0.7
	// get laser on/off status;
//	BOOL GetLaserOn();
	// turn laser on/off
//	void SetLaserOn(BOOL bOn);
	BOOL GetScannerEnabled();
	// switch between hard probe and laser probe: 
	void SetScannerEnabled(BOOL bEnabled);
	// get active probe information
	void GetProbeInfo(BSTR& bstrProbeName, double& dProbeDiameter, BOOL& bIsLaserProbe);
	// sphere calibration
	BOOL SphereCalibrationUI();
	// hole calibration
	BOOL HoleCalibrationUI();
	// plane calibration, for laser probe only
	BOOL PlaneCalibrationUI();

	CFaroLaserScanner();
	virtual ~CFaroLaserScanner();

};

#endif
