#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"
#include "farousbarm.h"

#include "FaroUSB/FaroScannerAPI.h"
#include "FaroUSB/FaroUSBCallBack.h"
#include "FaroUSB/DPickFaroUSB.h"
#include "FaroUSB/ProbeData.h"
#include "FaroUSB/Scanner/ScannerControlDlg.h"

#import "FaroUSB/FaroAtlServerApp5.tlb" no_namespace
#import "FaroUSB/FaroArmUsb5.tlb" no_namespace
IFAROConnectPoint5Ptr pFaroArm;
long lDeviceHandle;

FaroUSBArm::FaroUSBArm(void)
: scanner(*new CFaroLaserScanner)
, scannerCallBack(*new FaroUSBCallBack(*this))
, scannerEvents(*new FaroUSBRawEvents(*this))
//, trackArm(*new TrackArmInterface(*this))
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;
	scannerStarted = false;
	lineScanning = false;

	app.saarm.Data.ButtonTwoMode = 3;	// backup/cancel, Romer defaults to 2, backup/mousetoggle
	firstshot = FALSE;
	buttontime = 4294967295;
	LongDown = FALSE;

	app.saarm.SettingsFileName = _T("SA_FaroUSB_ArmSettings.bin");
	LogFileName = _T("FaroUSB.log");

	lDeviceHandle = 0;
	mouseModeOn = FALSE;

	dataCtr = 0;
	canSendData = false;
}

FaroUSBArm::~FaroUSBArm(void)
{
	delete &scanner;
	delete &scannerCallBack;
	delete &scannerEvents;
	//delete &trackArm;
}

bool FaroUSBArm::ConnectToArm(const int dof)
{
	TRACE1("FaroUSBArm::ConnectToArm dof=%d...\n", dof);
	if(!app.saarm.Data.doconnection || ConnectedToArm)	// 2001.03.23
		return true;	// simulation mode

	J.Create(dof, TRUE);	// create joint vector
	FJ.Create(dof, TRUE);
	lastJ.Create(dof, TRUE);
	JShift.Create(dof,TRUE);

	// 2006.02.20 - the offsets are unique PER ARM.  Paul Atwell is looking into adding a query to the sdk for us...

	JShift.x[0] = 90.0;//-90.0;
	JShift.x[1] = 45.0;
	JShift.x[2] = 0.0;//180.0;
	JShift.x[3] = 65.0;
	JShift.x[4] = 0.0;
	JShift.x[5] = 70.0;

	DOF = dof;
	if(DOF == 7)	// 2003.11.17 - guessing!  haven't seen a 7 dof yet
		JShift.x[6] = 0.0;

	//if(app.saarm.InstrumentModel == 47 )	// Faro USB 4 ft. Arm OR USB 8 ft. Arm (7 dof)
	//{
	//	JShift.x[0] = -91.0;
	//	JShift.x[1] = 45.0;
	//	JShift.x[2] = 135.0;
	//	JShift.x[3] = 65.0;
	//	JShift.x[4] = 198.0;
	//	JShift.x[5] = 70.0;
	//}
	//if(app.saarm.InstrumentModel == 54) // Faro USB 8 ft. Arm (7 dof)
	//{
	//	JShift.x[0] = 90.0;//-90.0;
	//	JShift.x[1] = 45.0;
	//	JShift.x[2] = 0.0;//180.0;
	//	JShift.x[3] = 65.0;
	//	JShift.x[4] = 0.0;
	//	JShift.x[5] = 70.0;
	//}
	//if(app.saarm.InstrumentModel == 49)	// Faro USB 8 ft. Arm
	//{
	//	JShift.x[0] = -90.0;
	//	JShift.x[1] = 45.0;
	//	JShift.x[2] = 180.0;
	//	JShift.x[3] = 85.0;
	//	JShift.x[4] = 198.0;
	//	JShift.x[5] = 70.0;
	//}
	if(app.saarm.InstrumentModel == 124 )	// Faro USB 4 ft. Arm OR USB 8 ft. Arm (7 dof)
	{
		JShift.x[0] = 0.0;
		JShift.x[1] = 0.0;
		JShift.x[2] = 0.0;
		JShift.x[3] = 180.0;
		JShift.x[4] = 180.0;
		JShift.x[5] = 180.0;
	}
	if(app.saarm.CMMString.Find(_T("Quantum")) != -1) // 2017.11.02 - new models made by Jeremy
	{
		JShift.x[1] = 30.0;
		JShift.x[2] = -270.0;
		JShift.x[3] = 50.0;
		JShift.x[5] = 180.0;
	}

	Vector rot(90,0,0);
	NativeArmToSA.SetRotation(rot);

	//if(trackArm.ConnectTrackArm())
	//{
	//	ConnectedToArm = true;
	//	trackArmActive = true;
	//	return true;
	//}

	TRACE0("FaroUSB low level connection stuff...\n");
	//CoInitialize(NULL);
	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	HRESULT hr = pFaroArm.CreateInstance(__uuidof(FAROConnectPoint5));
	if(SUCCEEDED(hr))
	{
		VARIANT comSN;
		CStringArray sns;
		pFaroArm->GetSerialNumbers(&comSN);
		FromCOM(comSN, sns);

		if(sns.GetSize() == 1)
			lDeviceHandle = pFaroArm->OpenFirstDetected();
		else
		{
			DPickFaroUSB dlg(sns);

			if(DOMODAL(dlg) == IDOK)
			{
				CComBSTR serialNum(dlg.selectedSN);
				lDeviceHandle = pFaroArm->Open(serialNum.m_str);
			}
		}
		if(lDeviceHandle)
		{
			int armJts = pFaroArm->GetNumberOfJoints(lDeviceHandle);
			if(DOF != armJts)
			{
				CString Msg;
				Msg.Format(Tr("Number of Arm Joints (%d) does not match model in SA (%d joints).\nAuto model correction will not work correctly."), armJts, DOF);
				int ans = AfxMessageBox(Msg + Tr("\n\nContinue anyway?"), MB_YESNO);
				if(ans==IDNO)
				{
					DisconnectFromHardware();
					PostQuitMessage(0);
					return false;
				}
			}

			//double unitScale;
			//pFaroArm->GetUnitScale(lDeviceHandle, &unitScale);
			//TRACE1(">>>> unitScale = %f\n", unitScale);

			// following is for external hardware - does not apply to us (at the moment)
			//int mode;
			//pFaroArm->GetMachineMode(lDeviceHandle, &mode);
			//TRACE1("FaroArm machine mode = %d\n", mode);
			//pFaroArm->SetMachineMode(lDeviceHandle,);

			ConnectedToArm = TRUE;
 
			// 2006.02.16 - sdk 5.1.1.2 - see faroscannerapi.h - "Laser on/off function disabled in v5.0.7"
			// whenever the scanner is the 'current probe', the laser is turned on and the range finder is run.
			// the following will shut off the range finder, but unfortunately, the laser will still be on until 
			// the user runs the scanner, and then stops the scanner mode...
			if(!scannerStarted)
			{
				if(scanner.StartDevice()) // start the scanner
				{
					scanner.SetScannerEnabled(FALSE); // "unset" the scanner as the active probe
					scannerStarted = true;
				}
			}

			RemoteMouse(mouseModeOn);

			//2006.02.20 - If SetScannerEnabled is called BEFORE EnableReferenceDlg, the reference dialog does not pop, even when needed!!
			// Paul Atwell is looking into this, meanwhile, enable the reference dialog LAST...
			pFaroArm->EnableReferenceDlg(lDeviceHandle,TRUE);

			BSTR bstrSerialNumber;
			BOOL bStatus = (BOOL)pFaroArm->GetSerialNumber(lDeviceHandle, &bstrSerialNumber);
			if(bStatus)
			{
				CString sn(bstrSerialNumber);
				CMMSerialNum = sn;
			}

			// 2008.11.17 - the following 3 strings, arguments for the Probes calibration ui, and the specific cal. ui's (hole, plane, and sphere)
			// are NOT USED by Faro - this is confirmed for Faro SDK v.5.4.0.2, and has apparently been the case for some time -- will leave them in anyway...
			strProbeFile = _T("FaroArmProbes4_") + CMMSerialNum + _T(".xml");
			strLogFile = _T("CalibrationLog_") + CMMSerialNum + _T(".xml");
			strProbeSettingsFile = _T("FaroLaserScannerSettings_") + CMMSerialNum + _T(".xml");
			//double dAccSpec = 0.0;
			//bStatus = (BOOL)pFaroArm->GetAccuracySpec(lDeviceHandle, &dAccSpec);
			//if(bStatus)
			//{
			//	WCHAR prompt[128];
			//	swprintf(prompt, L"\n   Accuracy Spec: %+.4lf", dAccSpec);
			//	wprintf(prompt);
			//}
			//long CalDate;
			//bStatus = (BOOL)pFaroArm->GetCalibrationDate(lDeviceHandle, &CalDate);
			//if(bStatus)
			//{
			//	wprintf( L"\nCalibration Date: ");
			//	wprintf(_tctime(&CalDate));
			//}

			IsProbeValid();

			return true;
		}
	}

	CString Msg;
	Msg.Format(Tr("Faro USB Connection failed."));
	int ans = AfxMessageBox(Msg + Tr("\n\nRun in Simulation mode?"),MB_YESNO);
	if(ans==IDNO)
	{
		PostQuitMessage(0);
		return false;
	}
	else
		return true;	// simulation mode
	return false;
}

void FaroUSBArm::DoPostInitOperations()
{
	if(app.saarm.CMMString.Find(_T("Quantum")) != -1) // 2017.11.02 - new models made by Jeremy
	{
		app.saarm.Data.autoCorrectModel = FALSE;
	}
}

void FaroUSBArm::DoArmSettings()
{
	if(!ConnectedToArm)
	{
		ArmBase::DoArmSettings();
		return;
	}

	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	IFaroArmUsbDialogs5Ptr	theDialogs;
	HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	if(!SUCCEEDED(hr))
	{
		::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
		return;
	}
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	LONG lWnd = (LONG)d.GetSafeHwnd();
	LONG lFaroRail = 0;
	//if(theApp.m_pDeviceThread->m_pFaroRail)
	//{
	//	lFaroRail = theApp.m_pDeviceThread->m_pFaroRail->lDeviceHandle;
	//}
	theDialogs->HardwareConfig(lDeviceHandle, lFaroRail, lWnd);
	theDialogs = NULL;
	CoUninitialize();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Calibration...
#define CALIBRATE_HOLE			11
#define CALIBRATE_SPHERE		12
#define CALIBRATE_PLANE			1011
void FaroUSBArm::DoCal()
{
	if(!ConnectedToArm)
	{
		ArmBase::DoCal();
		return;
	}

	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	int nRes = IDCANCEL;
	//vProbeDlgInfo * pStruct=NULL;
	IFaroArmUsbDialogs5Ptr	theDialogs;
	HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	if(!SUCCEEDED(hr))
	{
		::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
		return;
	}
	//CString strPath = GetModulePath(_NOT("FaroArmUsbWrapper.dll"));
	CComBSTR bstrProbeFile(strProbeFile);
	CComBSTR bstrLogFile(strLogFile);
	CComBSTR bstrProbeSettingsFile(strProbeSettingsFile);
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	LONG lWnd = (LONG)d.GetSafeHwnd();
	double dDiameter = app.saarm.Data.TipDia*1000.0f;	// mm
	BSTR bstrCurProbe;
	
	// 2008.05.06 - scale cal. display to whatever current units are...
	double dScale = 1.0/25.4;
	if(app.saarm.units.distunits == DISTUNITS_IN)
		dScale = 1.0/25.4;
	else if(app.saarm.units.distunits == DISTUNITS_MM)
		dScale = 1.0;
	else if(app.saarm.units.distunits == DISTUNITS_M)
		dScale = 0.001;
	else if(app.saarm.units.distunits == DISTUNITS_CM)
		dScale = 0.1;
	else if(app.saarm.units.distunits == DISTUNITS_FT)
		dScale = 1.0/(25.4*12.0);

	nRes = theDialogs->Probes(lDeviceHandle, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str, bstrProbeSettingsFile.m_str, dScale, &dDiameter, &bstrCurProbe);
	TRACE1("nRes = %d\n", nRes);
	//CString curProbe(bstrCurProbe);
	//TRACE2("tip dia = %f mm, current probe name = %s\n", dDiameter, curProbe);
	//app.saarm.Data.TipDia = dDiameter/1000.0f; // m

	// 2014.04.29 - no longer call separate functions for the calibrations, when theDialogs COM object unloads, it loses the units multiplier, dScale
	if(nRes == CALIBRATE_HOLE)
	{
		//OnHoleCalibration();
		theDialogs->HoleCalibration(lDeviceHandle, TRUE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	}
	else if(nRes == CALIBRATE_SPHERE)
	{
		//OnSphereCalibration();
		theDialogs->SphereCalibration(lDeviceHandle, FALSE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	}
	else if(nRes == CALIBRATE_PLANE)
	{
		//OnPlaneCalibration();
		theDialogs->PlaneCalibration(lDeviceHandle, FALSE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	}

	//nRes = theDialogs->Probes(lDeviceHandle, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str, bstrProbeSettingsFile.m_str, dScale, &dDiameter, &bstrCurProbe);
	//TRACE1("nRes = %d\n", nRes);
	
	CString curProbe(bstrCurProbe);
	TRACE2("tip dia = %f mm, current probe name = %s\n", dDiameter, curProbe);
	app.saarm.Data.TipDia = dDiameter/1000.0f; // m

	theDialogs = NULL;
	CoUninitialize();

	// 2009.01.22 - they leave the arm in scanning mode if the scanner is the selected probe, we don't want to do that...
	BOOL scEnabled = FALSE;
	if(GetLineScannerEnabled(scEnabled))
	{
		if(scEnabled)
			SetLineScannerEnabled(FALSE);
	}
}
void FaroUSBArm::OnHoleCalibration()
{
	if(!ConnectedToArm)
		return;

	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	IFaroArmUsbDialogs5Ptr	theDialogs;
	HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	if(!SUCCEEDED(hr))
	{
		::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
	}
	CComBSTR bstrLogFile(strLogFile);
	CComBSTR bstrProbeFile(strProbeFile);

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	LONG lWnd = (LONG)d.GetSafeHwnd();
	theDialogs->HoleCalibration(lDeviceHandle, TRUE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	theDialogs = NULL;
	CoUninitialize();
}
void FaroUSBArm::OnSphereCalibration()
{
	if(!ConnectedToArm)
		return;

	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	IFaroArmUsbDialogs5Ptr	theDialogs;
	HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	if(!SUCCEEDED(hr))
	{
		::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
		return;
	}
	CComBSTR bstrLogFile(strLogFile);
	CComBSTR bstrProbeFile(strProbeFile);

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	LONG lWnd = (LONG)d.GetSafeHwnd();
	theDialogs->SphereCalibration(lDeviceHandle, FALSE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	theDialogs = NULL;
	CoUninitialize();
}
void FaroUSBArm::OnPlaneCalibration()
{
	if(!ConnectedToArm)
		return;

	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	IFaroArmUsbDialogs5Ptr	theDialogs;
	HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	if(!SUCCEEDED(hr))
	{
		::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
		return;
	}
	CComBSTR bstrLogFile(strLogFile);
	CComBSTR bstrProbeFile(strProbeFile);

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	LONG lWnd = (LONG)d.GetSafeHwnd();
	theDialogs->PlaneCalibration(lDeviceHandle, FALSE, lWnd, bstrProbeFile.m_str, bstrLogFile.m_str);
	theDialogs = NULL;
	CoUninitialize();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::GetJoints(bool simulate/*=true*/)
{
	if( trackArmActive )
		return FALSE; //trackArm.GetJoints();
	if(!ConnectedToArm)
		return ArmBase::GetJoints(simulate);

	double x,y,z;
	x=y=z = 0.0;
	double A,B,C,dAng7;
	int nButtons=0, nEndStop, nErrors, nWaiting;
	long lTimeStamp;
	if( !(BOOL)pFaroArm->ReadDevice(lDeviceHandle, &x, &y, &z, &A, &B, &C,
		&J.x[0], &J.x[1], &J.x[2], &J.x[3], &J.x[4], &J.x[5], &dAng7,
		&Temperature, &lTimeStamp, &nEndStop, &nButtons, &nErrors, &nWaiting) )
		return FALSE;

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// 2020.01.07 - XIT-1616 - added the following button state handling for SendOnScanPassStarted to SA for new meshing with resharper 
	//TRACE1("buttons = %d\n", nButtons);
	BYTE buttons = (BYTE)nButtons;
	if (lineScanning)
	{
		if(buttons&2)	//front button
		{  
			if (LastButton != 1)
			{
				LastButton = 1;
				SendOnScanPassStarted();
			}
		}
		else if (buttons&1)  // rear button
		{
			LastButton = 2;
		}
		else if (buttons&4)  // touch probe
		{
			LastButton = 3;
		}
		else
		{
			if (lineScanning && LastButton == 1)
			{
				if (app.saarm.Data.voxelizationEnabled)
					ClearScanClouds();
			}

			LastButton = 0;
		}
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	if(app.saarm.CMMString.Find(_T("Quantum")) != -1) // 2017.11.02 - new models made by Jeremy
	{
		if(DOF == 7)
			J.x[6] = dAng7;

		//J.Debug();

		for(int i=0; i<DOF; i++)
		{
			//TRACE3("J[%d] = %f, shift = %f\n", i, J.x[i], JShift[i]);
			J.x[i] += JShift[i];    //to correct for shift and sign of joints
		}
	}
	else
	{
		J.x[1] = -J.x[1];
		J.x[3] = -J.x[3];
		J.x[5] = -J.x[5];
		if(DOF == 7)
			J.x[6] = dAng7;

		//J.Debug();

		for(int i=0; i<DOF; i++)
		{
			//TRACE3("J[%d] = %f, shift = %f\n", i, J.x[i], JShift[i]);
			J.x[i] += JShift[i];    //to correct for shift and sign of joints
		}
	}

	return TRUE;
}

bool FaroUSBArm::IsProbeValid()
{
	if( !ConnectedToArm )
		return ArmBase::IsProbeValid();

	BSTR bstrName;
	double lfX = 0.0;
	double lfY = 0.0;
	double lfZ = 0.0;
	double lfDiameter = 0.0;
	double lf2SigmaError = 0.0;
	double lfMaxError = 0.0;
	long lDebounce = 0;
	ProbeType eProbeType = HardProbe;
	double lfTemperature = 0.0;
	double lfTempCoeff = 0.0;
	long lCalibrationTimeDate = 0;
	double dMinimumAssertTime = 0.0;
	long ret = pFaroArm->GetProbe(lDeviceHandle, &bstrName, &lfX, &lfY, &lfZ, &lfDiameter, &lf2SigmaError, &lfMaxError, &lDebounce, &eProbeType, &lfTemperature, &lfTempCoeff, &lCalibrationTimeDate, &dMinimumAssertTime );
	double acSpec;
	ret = pFaroArm->GetAccuracySpec(lDeviceHandle, &acSpec);
	
	CString probeName(bstrName);
	CTime dTime(lCalibrationTimeDate);
	CString a;
	a.Format(_T("cal time = %s"),dTime.Format(_T("(%c)")));
	TRACE1("%s\n", a);
	app.saarm.Data.TipDia = lfDiameter/1000.0f; // m
	//if(lCalibrationTimeDate <= 0 || lf2SigmaError > acSpec)	// 2008.12.30 - forget basing this on the accuracy spec, user had to forcibly ok that in Faro UI, and may want to use the probe anyway
	if(lCalibrationTimeDate <= 0)
		probeValid = false;
	else
		probeValid = true;
	return probeValid;
}
//bool FaroUSBArm::IsTouchProbeWithButtonEvents()
//{
//	BSTR bstrName;
//	double lfX = 0.0;
//	double lfY = 0.0;
//	double lfZ = 0.0;
//	double lfDiameter = 0.0;
//	double lf2SigmaError = 0.0;
//	double lfMaxError = 0.0;
//	long lDebounce = 0;
//	ProbeType eProbeType = HardProbe;
//	double lfTemperature = 0.0;
//	double lfTempCoeff = 0.0;
//	long lCalibrationTimeDate = 0;
//	double dMinimumAssertTime = 0.0;
//	long ret = pFaroArm->GetProbe(lDeviceHandle, &bstrName, &lfX, &lfY, &lfZ, &lfDiameter, &lf2SigmaError, &lfMaxError, &lDebounce, &eProbeType, &lfTemperature, &lfTempCoeff, &lCalibrationTimeDate, &dMinimumAssertTime );
//	CString probeName(bstrName);
//	return StringHelper::NameCompare(_T("Sensor"), probeName) ? true : false;
//}
BOOL FaroUSBArm::GetAllData()
{
	if( trackArmActive )
		return FALSE; //trackArm.GetAllData();
	if( !ConnectedToArm )
		return ArmBase::GetAllData();

	if(app.StopNow)
		return FALSE;

	//IsProbeValid();

	double x,y,z;
	x=y=z = 0.0;
	double A,B,C,dAng7;
	int nButtons, nEndStop, nErrors, nWaiting;
	nButtons=nEndStop=nErrors=nWaiting = 0;
	long lTimeStamp=0;	// 2004.11.15 - this is ALWAYS 0 as of Faro's firmware v. 128...
	// used to check it for 'fresh' data, but no more, it broke our detection of button hits!!
	wprintf(L"\n\nPress a button on the FaroArm to continue...\n");
	if( !(BOOL)pFaroArm->ReadDevice(lDeviceHandle, &x, &y, &z, &A, &B, &C,
			&FJ.x[0], &FJ.x[1], &FJ.x[2], &FJ.x[3], &FJ.x[4], &FJ.x[5], &dAng7,
			&Temperature, &lTimeStamp, &nEndStop, &nButtons, &nErrors, &nWaiting) )
	{
		buttonstate = 0;
		LastButton = 0;
		LongDown = FALSE;
		return FALSE;
	}

	if(x==0.0 || y==0.0 || z==0.0)
		return FALSE;

	//TRACE1("Faro Arm Time Stamp = %ld\n", lTimeStamp);

	if(!canSendData)
	{
		dataCtr++;
		if(dataCtr > 10)
			canSendData = true;
		return FALSE;
	}
	

	J = FJ;

	// trying to find a way to ask arm if it's ok to get data.... are encoders intialized, etc.....
	//long status = pFaroArm->GetCurrentStatus(lDeviceHandle);	// 2008.01.14 - usb dvr v.5.4.0.2 - returns zero no matter what... checking with Paul Atwell on it
	//TRACE1("status = %ld\n", status);

	// Position app.saarm.Data...
	Vector FaroCoord( x/1000.0f,
							y/1000.0f,
							z/1000.0f );
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	NativeArmToSA.Multiply( FaroCoord, &AnalyzerCoord );
	X = AnalyzerCoord.x;
	Y = AnalyzerCoord.y;
	Z = AnalyzerCoord.z;
	//TRACE3("Arm XYZ from Faro GetAllData...X=%f, Y=%f, Z=%f\n", X, Y, Z);

	// Rotation app.saarm.Data...
	Transform FaroXForm;
	FaroXForm.SetZXZEulerRotation(Vector(A, B, C));
	CurrentTransform.MultiplyRot(NativeArmToSA, FaroXForm);
	CurrentTransform.SetTranslation(Vector(X,Y,Z));

	Vector ijk, k(0.0,0.0,1.0);
	CurrentTransform.MultiplyRot(k, &ijk);

	// Testing!!
	//	MeasToSpan(ijk);	// should make pts on a 1 meter radius sphere centered around the instrument
	// each pt. will be located such that it 'points' (line from arm center through pt.) in the same direction as the probe tip

	II = ijk.x;
	JJ = ijk.y;
	KK = ijk.z;

	if(app.saarm.CMMString.Find(_T("Quantum")) != -1) // 2017.11.02 - new models made by Jeremy
	{
		if(DOF == 7)
			J.x[6] = dAng7;

		//J.Debug();

		for(int i=0; i<DOF; i++)
		{
			//TRACE3("J[%d] = %f, shift = %f\n", i, J.x[i], JShift[i]);
			J.x[i] += JShift[i];    //to correct for shift and sign of joints
		}
	}
	else
	{
		J.x[1] = -J.x[1];
		J.x[3] = -J.x[3];
		J.x[5] = -J.x[5];
		if(DOF == 7)
			J.x[6] = dAng7;

		//J.Debug();

		for(int i=0; i<DOF; i++)
		{
			//TRACE3("J[%d] = %f, shift = %f\n", i, J.x[i], JShift[i]);
			J.x[i] += JShift[i];    //to correct for shift and sign of joints
			
		}
	}

	//CString test;
	//test.Format("POS: %lf\t%lf\t%lf\nROT: %lf\t%lf\t%lf\nTEMP: %lf\n", X,Y,Z,II,JJ,KK,
	//	theUpdateStruct.dPosition[6]);
	//TRACE1("%s\n",test);

	// Button States...
	if(overrideArmButtons || !app.pA->armButtons.ButtonsEnabled())
		return TRUE;
	BYTE buttons = (BYTE)nButtons;
	if(!(buttons&1) && !(buttons&2))  //no buttons
	{  
		buttonstate = 0;
		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
		if( LastButton == 1 && !LongDown)
		{
			buttonstate = 1;
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_SHORT;
				Beep(900,100);
			}
		}
		if( LastButton == 2 && !LongDown)
		{
			buttonstate = 2;
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_SHORT;
				Beep(600,100);
			}
		}
		if(LastButton == 3 && Measuring && !(buttons&4))	// to support pausing a stream (see DGenMeas::ButtonHit)
		{
			buttonstate = 4;
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_LONG;
			}
		}

		LastButton = 0;
		LongDown = FALSE;
	}
	if(buttons&1)	// rear button
	{  
		TRACE0("Rear button hit\n");
		buttonstate = 2;
		if( LastButton == 1 || LastButton == 3 )
		{
			buttonstate = 1;
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_SHORT;
			}
			buttontime = 4294967295;
			LastButton = 2;
		}
		if( LastButton != 2 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 2;
		}
		else if( LastButton == 2 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				//	TRACE0("LongDown just set to TRUE, back button.\n");
			}
		}
	}
	else if(buttons&2)	//front button
	{  
		TRACE0("Front button hit\n");
		buttonstate = 1;
		if( LastButton == 2 || LastButton == 3 )
		{
			buttonstate = 2;
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_SHORT;
			}
			buttontime = 4294967295;
			LastButton = 1;
		}
		if( LastButton != 1 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 1;
		}
		else if( LastButton == 1 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				MessageBeep(-1);
				Beep(1200,200);
				TRACE0("LongDown just set to TRUE, front button.\n");
			}
		}
	}
	else if(buttons&4)	//touch probe
	{  
		TRACE0("Touch Probe hit\n");
		buttonstate = 4;
		if(LastButton != 3)
		{
			if(!IsRemoteMouseEnabled())
			{
				buttonstate += BUTTON_SHORT;
				//Beep(900,100);
			}
			buttontime = 4294967295;
			LastButton = 3;
			LongDown = FALSE;
		}

	}
	return TRUE;
}

BOOL FaroUSBArm::IsRemoteMouseEnabled()
{
	return mouseModeOn;
}
void FaroUSBArm::ToggleRemoteMouse()
{
	if(!ConnectedToArm)
		return;
	RemoteMouse(!mouseModeOn);
}
void FaroUSBArm::RemoteMouse(BOOL flag)
{
	if(!ConnectedToArm)
	{
		ArmBase::RemoteMouse(flag);
		return;
	}
	pFaroArm->EnableMouseMove(lDeviceHandle,flag);
	mouseModeOn = flag;
}

BOOL FaroUSBArm::IsRomer()
{
	return FALSE;
}
BOOL FaroUSBArm::IsFaro()
{
	return FALSE;
}
BOOL FaroUSBArm::IsFaroUSB()
{
	return TRUE;
}
BOOL FaroUSBArm::IsFrenchRomer()
{
	return FALSE;
}
BOOL FaroUSBArm::IsSNL()
{
	return FALSE;
}
BOOL FaroUSBArm::IsMetrisGarda()
{
	return FALSE;
}
//////////////////////////////////////////////////////////////////////////
// for scanner...
BOOL FaroUSBArm::ScannerSettings()	// old way - this calls Faro's ScannerControl dialog 
// 2014.01.06 - Mantis #6678 - brought this back (ScannerSettings Method below shows controls, but no preview window) to provide right-click access from main interface, and access from settings ui in SA toolbar
// 2016.03.30 - all new way, one liner: scanner.ScannerControlUI();  also added setting active probe (via SetScannerEnabled) back to what it was before running this
{
	TRACE0("FaroUSBArm::ScannerSettings...\n");
	if(!ConnectedToArm)
		return ArmBase::ScannerSettings();

	// 2006.02.16 - with sdk 5.1.1.2, the scanner MUST be the 'current probe' to run the scanner settings,
	// otherwise, the Faro stuff will just pop a message box, and will not run the settings ui...
	if(!scannerStarted)
	{
		if(!scanner.StartDevice())
		{
			AfxMessageBox(_T("Could not start scanner.  Is it connected and on?"));
			return FALSE;
		}
		scannerStarted = true;
	}

	BOOL scannerWasActiveProbe = scanner.GetScannerEnabled();	// 2016.03.30 - see if scanner was already active,

	if(!scannerWasActiveProbe)
		scanner.SetScannerEnabled(TRUE); // set the scanner as the active probe

	scanner.ScannerControlUI(); // 2016.03.30 - use this one liner instead of the mess below!

	//CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	//int nRes = IDCANCEL;
	//IFaroArmUsbDialogs5Ptr	theDialogs;
	//HRESULT hr = theDialogs.CreateInstance(__uuidof(FaroArmUsbDialogs5));
	//if(!SUCCEEDED(hr))
	//{
	//	::MessageBox(NULL, _T("CreateInstance of FaroArmUsbDialogs5 failed."), _T("Installation problem"), MB_OK);
	//	return FALSE;
	//}
	////CString strPath = GetModulePath(_NOT("FaroArmUsbWrapper.dll"));
	//CString strProbeSettingsFile = _T("FaroLaserScannerSettings.xml");
	//CComBSTR bstrProbeSettingsFile(strProbeSettingsFile);
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//LONG lWnd = (LONG)d.GetSafeHwnd();
	//nRes = theDialogs->ScannerControl(lDeviceHandle, lWnd, bstrProbeSettingsFile.m_str);
	//TRACE1("nRes = %d\n", nRes);
	//theDialogs = NULL;
	//CoUninitialize();

	scanner.SetScannerEnabled(scannerWasActiveProbe); // 2016.03.30 - now set scanner to its active state before we started this
	return TRUE;
}
//BOOL FaroUSBArm::ScannerSettings()
//{
//	// 2008.11.12 - just guide user to Faro scanner cal. - have to dig into ui a bit, but settings are there....
//	//AfxMessageBox(_T("Go to Arm Calibration button in main dialog.\nSelect Laser Probe >> Edit >> Laser Probe Settings."), MB_ICONINFORMATION);
//	//return TRUE;
//	// 2008.11.12 - have to handle ui ourselves, but we can add auto-exposure algorithm control, etc....
//	CScannerControlDlg dlg(*this);
//	if(DOMODAL(dlg) == IDOK)
//		return TRUE;
//	return FALSE;
//}
BOOL FaroUSBArm::StopScannerMeas()
{
	if(!ConnectedToArm)
		return ArmBase::StopScannerMeas();

	if(!lineScanning)
		return TRUE;

	TRACE0("Stop FaroUSB scanner meas...\n");
	scanner.EndDataCapture(); // end the scanner date capture mode
	lineScanning = false;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);

	// return mouse mode to previous state - Faro stuff keeps turning it on...
	RemoteMouse(mouseModeOn);
	scanner.SetScannerEnabled(FALSE); // "unset" the scanner as the active probe

	TRACE0("send scan pass stopped\n");
	SendOnScanPassStopped();

	//if(app.saarm.Data.autoIncCloudName)
	//{
	//	app.saarm.Data.Target.Format(_T("P1"));
	//	app.pA->IncrementName(app.saarm.Data.Group, 1);
	//}

	//SetMeasMode(0);

	//if(Units::toolBarModeOn)
	//{
	//	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//	d.RestartMeasurement();
	//}

	return TRUE;
}
BOOL FaroUSBArm::StartScannerMeas()
{
	if(!ConnectedToArm)
		return ArmBase::StartScannerMeas();

	if(!app.saarm.scannerLicenseValid)
	{
		AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
		return FALSE;
	}
	if(lineScanning)
		return TRUE;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.5,TRUE);
	d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

	TRACE0("FaroUSBArm::StartScannerMeas...");
	if(!scannerStarted)
	{
		if(!scanner.StartDevice())
		{
			d.StopScannerTimer();
			MessageBox(NULL,
				Tr("Could not start scanner.  Is it connected and on?"),
				Tr("Scanner"),
				MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST);
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		scannerStarted = true;
	}

	// be sure the scanner is the current probe...
	if(!scanner.GetScannerEnabled())
		scanner.SetScannerEnabled(TRUE);

	// make sure there really is a scanner present - fortunately, GetScannerEnabled will return false if there's no scanner...
	if(!scanner.GetScannerEnabled())
	{
		d.StopScannerTimer();
		MessageBox(NULL,
			Tr("Could not start scanner.  Is it connected and on?"),
			Tr("Scanner"),
			MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST);
		d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
		return FALSE;
	}

	lineScanning = true;
	CCloudData CloudData;
	scanner.SetDataCaputreCallBack((CDataCaptureCallBack*) &scannerCallBack);
	scanner.SetDeviceRawEventCallBack((FaroUSBRawEvents*) &scannerEvents);

	scanner.CaptureData(CloudData);

	scanner.SetDataCaputreCallBack(NULL);
	scanner.SetDeviceRawEventCallBack(NULL);

	scanner.EndDataCapture();
	lineScanning = false;
	d.StopScannerTimer();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);

	// return mouse mode to previous state - Faro stuff keeps turning it on...
	RemoteMouse(mouseModeOn);

	//SAIPCloud pkt;
	//pkt.Name = app.saarm.Data.Group;
	//SAIPCloudBlockPt cbpt;
	//Vector xyzFaro, xyzSA;

	//int pakidx = 0;
	//for(int i=0; i<Cloudapp.saarm.Data.m_nScans; i++)
	//{
	//	for(int j=0; j<Cloudapp.saarm.Data.m_parrScanData[i].m_nPoints; j++)
	//	{
	//		xyzFaro.x = Cloudapp.saarm.Data.m_parrScanData[i].m_parrPointData[j].m_dData[0];
	//		xyzFaro.y = Cloudapp.saarm.Data.m_parrScanData[i].m_parrPointData[j].m_dData[1];
	//		xyzFaro.z = Cloudapp.saarm.Data.m_parrScanData[i].m_parrPointData[j].m_dData[2];
	//		xyzFaro.Scale(1.0/25.4);
	//		NativeArmToSA.Multiply( xyzFaro, &xyzSA );
	//		cbpt.x = xyzSA.x;
	//		cbpt.y = xyzSA.y;
	//		cbpt.z = xyzSA.z;
	//		pkt.AddPoint(cbpt);
	//	}

	//}
	//TRACE0("SENDING CLOUD packet sent to SA...\n");
	//QS.SendPacket(pkt);

	scanner.SetScannerEnabled(FALSE); // "unset" the scanner as the current probe
	CheckAndSendScannerCloud(0, true);
	return TRUE;
}
//////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::DisconnectFromHardware()
{
	if(!ConnectedToArm)
		return app.saarm.DisconnectFromHardware();

	if(scannerStarted)
	{
		scanner.EndDataCapture();
		scanner.StopDevice();
	}

	RemoteMouse(FALSE);
	//close
	if(ConnectedToArm)
	{
		pFaroArm->Close(lDeviceHandle);
		//CoUninitialize();
	}
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// COM Utilities
//////////////////////////////////////////////////////////////////////////////
VARIANT FaroUSBArm::ToCOM (const double *pdValue, long lSize)
{
	VARIANT doublelist;
	VariantInit (&doublelist);

	try
	{
		doublelist.vt = VT_ARRAY | VT_R8;

		SAFEARRAYBOUND rgsabound = {lSize, 0};

		doublelist.parray = SafeArrayCreate (VT_R8, 1, &rgsabound);

		double* pdList = NULL;

		SafeArrayAccessData (doublelist.parray, (void **) &pdList);

		for (int i = 0; i < lSize; i++)
		{
			pdList[i] = pdValue[i];
		}

		SafeArrayUnaccessData (doublelist.parray);
	}
	catch (...)
	{
		doublelist.vt = VT_EMPTY;
	}

	return doublelist;
}

//////////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::FromCOM (double **ppdData, long &lSize, const VARIANT& vValue)
{
	BOOL bRet = TRUE;

	double *pdData = NULL;

	try
	{
		double* pdList = NULL;

		if (vValue.vt == (VT_ARRAY | VT_R8) || vValue.vt == (VT_ARRAY | VT_VARIANT))
		{
			SafeArrayAccessData (vValue.parray, (void **) &pdList);

			lSize = vValue.parray->rgsabound->cElements;

			pdData = new double[lSize];

			if (pdData)
			{
				for (unsigned long i = 0; i < vValue.parray->rgsabound->cElements; i++)
				{
					pdData[i] = pdList[i];
				}
			}
			else
			{
				bRet = FALSE;  //  Bail if we can't allocate enough memory
			}

			SafeArrayUnaccessData (vValue.parray);
		}

		else
			if (vValue.vt == (VT_ARRAY | VT_R8 | VT_BYREF) || vValue.vt == (VT_ARRAY | VT_VARIANT | VT_BYREF))
			{
				SafeArrayAccessData (*vValue.pparray, (void **) &pdList);

				lSize = (*vValue.pparray)->rgsabound->cElements;

				pdData = new double[lSize];

				if (pdData)
				{
					for (unsigned long i = 0; i < (*vValue.pparray)->rgsabound->cElements; i++)
					{
						pdData[i] = pdList[i];
					}
				}
				else
				{
					bRet = FALSE;  //  Bail if we can't allocate enough memory
				}

				SafeArrayUnaccessData (*vValue.pparray);
			}
	}
	catch (...)
	{
		bRet = FALSE;
	}

	*ppdData = pdData;
	return bRet;
}
//////////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::FromCOM (long **pplData, long &lSize, const VARIANT& vValue)
{
	BOOL bRet = TRUE;

	long *plData = NULL;

	try
	{
		long* plList = NULL;

		if (vValue.vt == (VT_ARRAY | VT_I4) || vValue.vt == (VT_ARRAY | VT_VARIANT))
		{
			SafeArrayAccessData (vValue.parray, (void **) &plList);

			lSize = vValue.parray->rgsabound->cElements;

			plData = new long[lSize];

			if (plData)
			{
				for (unsigned long i = 0; i < vValue.parray->rgsabound->cElements; i++)
				{
					plData[i] = plList[i];
				}
			}
			else
			{
				bRet = FALSE;  //  Bail if we can't allocate enough memory
			}

			SafeArrayUnaccessData (vValue.parray);
		}

		else
			if (vValue.vt == (VT_ARRAY | VT_I4 | VT_BYREF) || vValue.vt == (VT_ARRAY | VT_VARIANT | VT_BYREF))
			{
				SafeArrayAccessData (*vValue.pparray, (void **) &plList);

				lSize = (*vValue.pparray)->rgsabound->cElements;

				plData = new long[lSize];

				if (plData)
				{
					for (unsigned long i = 0; i < (*vValue.pparray)->rgsabound->cElements; i++)
					{
						plData[i] = plList[i];
					}
				}
				else
				{
					bRet = FALSE;  //  Bail if we can't allocate enough memory
				}

				SafeArrayUnaccessData (*vValue.pparray);
			}
	}
	catch (...)
	{
		bRet = FALSE;
	}

	*pplData = plData;
	return bRet;
}

//////////////////////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::FromCOM(const VARIANT v, CStringArray& a)
{
	BOOL bRet = FALSE;
	try
	{
		if(v.vt == (VT_BSTR | VT_ARRAY))
		{
			a.RemoveAll();
			BSTR* bstrArray = NULL;
			SafeArrayAccessData(v.parray, (void**)&bstrArray);
			USES_CONVERSION;
			for(ULONG i=0;i<v.parray->rgsabound->cElements;i++)
			{
				a.Add(OLE2T(CComBSTR(bstrArray[i])));
			}
			SafeArrayUnaccessData(v.parray);
			bRet = TRUE;
		}
	}
	catch(...)
	{
		bRet = FALSE;
	}
	return bRet;
}

//////////////////////////////////////////////////////////////////////////////
BOOL FaroUSBArm::FromCOM(BYTE **ppdData, long &lSize, const VARIANT& vValue)
{
	BOOL bRet = TRUE;

	BYTE *pdData = NULL;

	try
	{
		BYTE* pdList = NULL;

		if (vValue.vt == (VT_ARRAY | VT_UI1) || vValue.vt == (VT_ARRAY | VT_VARIANT))
		{
			SafeArrayAccessData (vValue.parray, (void **) &pdList);

			lSize = vValue.parray->rgsabound->cElements;

			pdData = new BYTE[lSize];

			if (pdData)
			{
				for (unsigned long i = 0; i < vValue.parray->rgsabound->cElements; i++)
				{
					pdData[i] = pdList[i];
				}
			}
			else
			{
				bRet = FALSE;  //  Bail if we can't allocate enough memory
			}

			SafeArrayUnaccessData (vValue.parray);
		}

		else
			if (vValue.vt == (VT_ARRAY | VT_UI1 | VT_BYREF) || vValue.vt == (VT_ARRAY | VT_VARIANT | VT_BYREF))
			{
				SafeArrayAccessData (*vValue.pparray, (void **) &pdList);

				lSize = (*vValue.pparray)->rgsabound->cElements;

				pdData = new BYTE[lSize];

				if (pdData)
				{
					for (unsigned long i = 0; i < (*vValue.pparray)->rgsabound->cElements; i++)
					{
						pdData[i] = pdList[i];
					}
				}
				else
				{
					bRet = FALSE;  //  Bail if we can't allocate enough memory
				}

				SafeArrayUnaccessData (*vValue.pparray);
			}
	}
	catch (...)
	{
		bRet = FALSE;
	}

	*ppdData = pdData;
	return bRet;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Scanner Settings/Control

BOOL FaroUSBArm::GetAvailableScanRates( CStringArray& strRates )
{
	_variant_t vt = pFaroArm->GetAvailableScanRates ( lDeviceHandle );
	FromCOM (vt, strRates );
	BOOL bRet = TRUE;

	return bRet;
}

BOOL FaroUSBArm::GetAvailableScanDensities( CStringArray& strDensities )
{
	_variant_t vt = pFaroArm->GetAvailableScanDensities( lDeviceHandle );
	FromCOM (vt, strDensities );
	BOOL bRet = TRUE;
	return bRet;
}

BOOL FaroUSBArm::GetAvailableAlgorithms( CStringArray& strAlgorithms)
{
	_variant_t vt = pFaroArm->GetAvailableImageProcessingAlgorithms( lDeviceHandle );
	FromCOM (vt, strAlgorithms );
	BOOL bRet = TRUE;
	return bRet;
}
void FaroUSBArm::GetLineScannerTransformation(CProbeData& Prb)
{
	pFaroArm->GetLineScannerTransformation(
		lDeviceHandle,			// long lHandle
		&Prb.m_Offset[0],		// double* lfX
		&Prb.m_Offset[1],		// double* lfY
		&Prb.m_Offset[2],		// double* lfZ
		&Prb.m_ABC[0],			// double* lfA
		&Prb.m_ABC[1],			// double* lfB
		&Prb.m_ABC[2],			// double* lfC
		&Prb.m_d2Sigma,			// double* lf2SigmaError
		&Prb.m_dMaxError,		// double* lfMaxError
		&Prb.m_dTemperature,		// double* lfTemperature
		&Prb.m_dExpansionCoef,		// double* lfTempCoef
		&Prb.m_lCalibrationDate	// long* lCalibrationTimeDate
		);
}
void FaroUSBArm::SetLineScannerTransformation(const CProbeData& Prb)
{
	pFaroArm->SetLineScannerTransformation(
		lDeviceHandle,			// long lHandle
		Prb.m_Offset[0],		// double* lfX
		Prb.m_Offset[1],		// double* lfY
		Prb.m_Offset[2],		// double* lfZ
		Prb.m_ABC[0],			// double* lfA
		Prb.m_ABC[1],			// double* lfB
		Prb.m_ABC[2],			// double* lfC
		Prb.m_d2Sigma,			// double* lf2SigmaError
		Prb.m_dMaxError,		// double* lfMaxError
		Prb.m_dTemperature,		// double* lfTemperature
		Prb.m_dExpansionCoef,		// double* lfTempCoef
		Prb.m_lCalibrationDate	// long* lCalibrationTimeDate
		);
}
void FaroUSBArm::GetCameraInfo(CProbeData& Prb)
{
	CComBSTR bstrProbeName = NULL;
	CComBSTR bstrScanRate = NULL;
	CComBSTR bstrScanDensity = NULL;
	CComBSTR bstrAlgorithm = NULL;
	pFaroArm->GetCameraInfo(lDeviceHandle,
		&bstrProbeName,
		&bstrScanRate,
		&bstrScanDensity,
		&Prb.m_nScanNoiseThreshold, 
		&Prb.m_nScanPeakThreshold, 
		&Prb.m_nScanWidthThreshold, 
		&Prb.m_nScanFrameMode, 
		&Prb.m_nScanLaserOn, 
		&bstrAlgorithm, 
		&Prb.m_nScanExposure);
	Prb.m_strName = bstrProbeName;
	Prb.m_strScanRate = bstrScanRate;
	Prb.m_strScanDensity = bstrScanDensity;
	Prb.m_strScanAlgorithm = bstrAlgorithm;
}
void FaroUSBArm::SetCameraInfo(const CProbeData& Prb)
{
	CComBSTR bstrProbeName = Prb.m_strName;
	CComBSTR bstrScanRate = Prb.m_strScanRate;
	CComBSTR bstrScanDensity = Prb.m_strScanDensity;
	CComBSTR bstrAlgorithm = Prb.m_strScanAlgorithm;
	pFaroArm->SetCameraInfo(lDeviceHandle, 
		bstrProbeName.m_str,
		bstrScanRate.m_str,
		bstrScanDensity.m_str,
		Prb.m_nScanNoiseThreshold, 
		Prb.m_nScanPeakThreshold, 
		Prb.m_nScanWidthThreshold, 
		Prb.m_nScanFrameMode, 
		Prb.m_nScanLaserOn, 
		bstrAlgorithm.m_str, 
		Prb.m_nScanExposure);
}
BOOL FaroUSBArm::SetRangeFinderEnabled( const BOOL bOn )
{
	//long lEnabled; 
	//pFaroArm->GetRangeFinderEnabled ( lDeviceHandle, &lEnabled );
	BOOL bRet = pFaroArm->SetRangeFinderEnabled ( lDeviceHandle, bOn );

	return bRet;
}

BOOL FaroUSBArm::GetRangeFinderEnabled( BOOL& bOn )
{
	bOn = FALSE;
	long nOn = 0;
	BOOL bRet = pFaroArm->GetRangeFinderEnabled ( lDeviceHandle, &nOn );
	if( bRet )
	{
		bOn = static_cast<BOOL>(nOn);
	}

	return bRet;
}

BOOL FaroUSBArm::GetLineScannerEnabled(BOOL& bEnabled)
{
	bEnabled = FALSE;
	long lEnabled = 0;
	BOOL bRet = pFaroArm->GetLineScannerEnabled(lDeviceHandle, &lEnabled);
	if(bRet)
	{
		bEnabled = static_cast<BOOL>(lEnabled);
	}
	return bRet;
}

BOOL FaroUSBArm::SetLineScannerEnabled(BOOL bEnabled)
{
	BOOL bRet = pFaroArm->SetLineScannerEnabled(lDeviceHandle, bEnabled);
	return bRet;
}
