#pragma once
#include "armbase.h"

class CFaroLaserScanner;
class FaroUSBCallBack;
class CProbeData;
class FaroUSBRawEvents;

class FaroUSBArm :
	public ArmBase
{
public:
	FaroUSBArm(void);
	virtual ~FaroUSBArm(void);

	friend class FaroUSBCallBack;

	DWORD buttontime;
	BOOL LongDown;

	virtual bool ConnectToArm(const int dof);
	virtual BOOL DisconnectFromHardware();

	virtual void DoPostInitOperations();

	virtual BOOL GetAllData();
	virtual BOOL GetJoints(bool simulate=true);

	virtual void DoCal();
	virtual bool IsProbeValid();
	//virtual bool IsTouchProbeWithButtonEvents();

	virtual BOOL IsRemoteMouseEnabled();
	virtual void ToggleRemoteMouse();
	virtual void RemoteMouse(BOOL flag);
	virtual void DoArmSettings();

	virtual BOOL ScannerSettings();	// 2004.06.02
	virtual BOOL StartScannerMeas();	// 2004.06.02
	virtual BOOL StopScannerMeas();	// 2016.03.11

	virtual BOOL IsRomer();
	virtual BOOL IsFaro();
	virtual BOOL IsFaroUSB();
	virtual BOOL IsFrenchRomer();
	virtual BOOL IsSNL();
	virtual BOOL IsMetrisGarda();

	////////////////////////////////////////////////////////////////////
	// Scanner Settings/Control
	BOOL GetLineScannerEnabled(BOOL& bEnabled);
	BOOL SetLineScannerEnabled(BOOL bEnabled);
	BOOL GetProbeTemperature(double& dTemperature);
	BOOL AreStressStopsAvailable();
	void EnableProbe(BOOL bEnable);
	long EnableMouseMove(BOOL bEnable);
	long EnableReferenceDlg(BOOL bEnable);
	BOOL GetBatteryVoltage(double& dVoltage, double& dCurrent, double& dPercent);
	BOOL GetCalibrationDate(long& lDate);
	BOOL GetFirmwareVersion(int& nProductId, int& nFirmwareVersion);
	BOOL GetSerialNumber(CString& strSerialNumber);
	int GetNumberOfJoints();

	BOOL GetMachineMode(int& nMode);
	BOOL SetMachineMode(int nMode);
	BOOL GetMouseParameters(long& lSensitivity, int& nFlipAxis, int& nBackButtonMode);
	BOOL SetMouseParameters(long lSensitivity, int nFlipAxis, int nBackButtonMode);
	BOOL GetStressStopsEnable(BOOL& bEnabled);
	BOOL SetStressStopsEnable(BOOL bEnable);
	BOOL GetStressStopsMultiplier(double& dStressMultiplier);
	BOOL SetStressStopsMultiplier(double dStressMultiplier, BOOL bSaveSetting);
	BOOL IsValidProbe(double dMaxError);

	//BOOL ReadDevice(CPositionUpdateData& update);
	//BOOL ReadRawDevice(struct PositionPacket& data);
	//WORD AddListener(CFaroArmUsbListener* pListener);
	//void RemoveListener(WORD ListenerId);
	//void Fire_OnPositionChanged(const CPositionUpdateData& update);
	//void Fire_OnConnect();
	//void Fire_OnDisconnect();
	void Fire_OnImageUpdate(BYTE* pData, int nRows, int nColumns);
	void Fire_OnCGUpdate(struct StatusPacket *pStatus, double* pCGs, long* pCols, long& lCGs);

	void GetLineScannerTransformation(CProbeData& Prb);
	void SetLineScannerTransformation(const CProbeData& Prb);
	void GetCameraInfo(CProbeData& Prb);
	void SetCameraInfo(const CProbeData& Prb);

	BOOL GetAvailableScanRates( CStringArray& strRates );
	BOOL GetAvailableScanDensities( CStringArray& strDensities );
	BOOL GetAvailableAlgorithms( CStringArray& strAlgorithms);

	BOOL SetRangeFinderEnabled( const BOOL bOn );
	BOOL GetRangeFinderEnabled( BOOL& bOn );

	bool IsLineScanning() {return lineScanning;}
	////////////////////////////////////////////////////////////////////

private:
	CFaroLaserScanner& scanner;
	FaroUSBCallBack& scannerCallBack;
	FaroUSBRawEvents& scannerEvents; // 2016.05.13 - added for scanner start/stop events, but does not work for that (see notes in FaroUSBRawEvents class definition
	bool scannerStarted;
	bool lineScanning;
	BOOL mouseModeOn;
	void OnHoleCalibration();
	void OnSphereCalibration();
	void OnPlaneCalibration();
	GenVec FJ;
	int dataCtr;
	bool canSendData;

	// cal. files
	CString strProbeFile;
	CString strLogFile;
	CString strProbeSettingsFile;

	// COM Utilities
	VARIANT ToCOM (const double *pdValue, long lSize);
	BOOL FromCOM (double **ppdData, long &lSize, const VARIANT& vValue);
	BOOL FromCOM(BYTE **ppdData, long &lSize, const VARIANT& vValue);
	BOOL FromCOM (long **pplData, long &lSize, const VARIANT& vValue);
	BOOL FromCOM(const VARIANT v, CStringArray& a);
};
