// FitLine.cpp: implementation of the FitLine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FitLine.h" 
#include "FitLineMatrix.h"
#include "..\\DPCMM.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


FitLine::FitLine()
{
	Theta = 10;
	Phi = 10;
	pPoints = NULL;
	maxgdist = 1.0;
}
BOOL FitLine::Calculate(  )
{
	if (pPoints->GetSize() == 2)
	{
		A = *pPoints->P[0];
		B = *pPoints->P[1];
		Begin = A;
		Normal = B;
		Normal.Sub(A);
		Normal.Normalize();

		Resid.Add(_T("Line Created Using 2 Points."));
		return TRUE;
	}
	// make a local set of points at the average.
	LocPoints.Clear();
	pPoints->Average(&LocCenter);
	for (int k=0;k<pPoints->GetSize();k++)
	{
		Vector* p = new Vector(*pPoints->P[k]);
		p->Sub(LocCenter);
		LocPoints.P.Add( p );
	}
	// compute a guess from the local points!
	int f1=0;
	int f2=0;
	LocPoints.Find2FarthestPoints(&f1,&f2);
	Begin = PtA(f1);
	Normal = PtA(f2);
	Normal.Sub( Begin );
	maxgdist = Normal.Length();	// get the dist to use for other scaling operations.
	if (maxgdist < 1.0)
		maxgdist = 1.0;
	Normal.Normalize();
	// now, make the normal go in the direction from first to last point.
	Vector dir(PtA(pPoints->GetUpperBound()));
	dir.Sub(PtA(0));
	dir.Normalize();
	if (Normal.Dot(dir)<0)
		Normal.Scale(-1.0);

	// convert to the 4 variable line representation.
	double iX=0.0;
	double iY=0.0;
	Transform::ConvertBeginNormalToThetaPhiXY(Begin,Normal,&Theta,&Phi,&iX,&iY);
	TRACE("NORM1  ");
	Normal.Debug();

	FitLineMatrix sys;
	sys.ShowProgress = FALSE;
	sys.Prepare( pPoints->GetSize(), 4 );
	sys.SetParameters( *pOpt );
	sys.pObj = this;

	int index = 0;
	double dpos = pOpt->PosDelta;
	double drot = pOpt->AngDelta;
	sys.x[index] = iX;
	sys.dx[index++] = dpos;
	sys.x[index] = iY;
	sys.dx[index++] = dpos;
	sys.x[index] = Theta;
	sys.dx[index++] = drot;
	sys.x[index] = Phi;
	sys.dx[index++] = drot;

	sys.Solve();

	Begin.Add(LocCenter);		// remove the average shift!
	
	// search through all points and find the min and max along the line direction!
	//
	TRACE("NORM2  ");
	Normal.Debug();
	Vector OffsetNormal(Normal);
	OffsetNormal.Scale(maxgdist);
	OffsetNormal.Add(Begin);

	Vector MaxPt(Begin);
	double MaxDist = -1e6;
	Vector MinPt(Begin);
	double MinDist = 1e6;
	
	for (int i=0;i<pPoints->GetSize();i++)
	{
		double dist = pPoints->P[i]->DistanceAlongAxisToPoint( Begin, OffsetNormal );
		if (dist > MaxDist)
		{
			MaxDist = dist;
			Vector temp(Normal);
			temp.Scale( dist );
			temp.Add( Begin );
			MaxPt = temp;
		}
		if (dist < MinDist)
		{
			MinDist = dist;
			Vector temp(Normal);
			temp.Scale( dist );
			temp.Add( Begin );
			MinPt = temp;
		}
	}

	A = MinPt;
	B = MaxPt;

	// bob--- send back whatever info you need about the line fit... rms maybe,,,?
	// just steal rms code from below.... OR:
	// RMS = SafeSqrt(sys.SumRhs());
	
#ifdef USE_REPORTING
	// now, report the error!
	Resid.RemoveAll();
	double MaxError = 0.0;
	for (i=0;i<pPoints->GetSize();i++) 
	{
		CString fullname,t;
		if (pArray != NULL)
		{
			d.GetTargetFullName(pArray->GetAt(i), &fullname);
			t.Format(_T("%s  Dist from line %f "),fullname,sys.rhs[i]);
		}
		else
			t.Format(_T("%d  Dist from line  %f "),i,sys.rhs[i]);
			
		Resid.Add( t );

		MaxError = max(MaxError, sys.rhs[i]);
	}
	CString t;
	t.Format(_T("         Maximum error   %f  RMS %f"),MaxError, SafeSqrt(sys.SumRhs()));
	Resid.Add( t );
	t.Format(_T("---------------------------------------"));
	Resid.Add(t);
	t.Format(_T("Results in working frame:  %s"),d.WorkingFrameName);
	Resid.Add(t);
	Vector vec(A),vec1(B);
	Vector vw;
	d.GetPointInWorking( vec, &vw );
	t.Format(_T(" Point A:  %f  %f  %f"),vw.x,vw.y,vw.z);
	Resid.Add(t);
	d.GetPointInWorking( vec1, &vw );
	t.Format(_T(" Point B:  %f  %f  %f"),vw.x,vw.y,vw.z);
	Resid.Add(t);
	t.Format(_T(" Line Length  %f"),A.Distance(B));
	Resid.Add(t);
	d.GetVectorInWorking( Normal, &vw );
	t.Format(_T(" Direction:  %f  %f  %f"),vw.x,vw.y,vw.z);
	Resid.Add(t);


	// display residuals.
	DResiduals diag;
	diag.Title = "Line Fit Results:";
	diag.a = &Resid;
	if (diag.DoModal()!=IDOK)
		return FALSE;
#endif
	return TRUE;
}
void FitLine::Evaluate(double *e)
{  
	Vector OffsetNormal(Normal);
	OffsetNormal.Scale(maxgdist);
	OffsetNormal.Add( Begin );
	int index = 0;
	for ( int i=0;i<pPoints->GetSize(); i++)
		e[index++] = PtA(i).DistanceFromAxisToPoint(Begin, OffsetNormal) ; 
}
