// GeomCoverage.cpp : implementation file
//

#include "stdafx.h"
#include "GeomCoverage.h"
#include "..\\DPCMM.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// GeomCoverage

GeomCoveragePCMM::GeomCoveragePCMM()
{
}
GeomCoveragePCMM::~GeomCoveragePCMM()
{
}

double GeomCoveragePCMM::GetSpherePercent(Vector &center, double radius, VectorArray *measured)
{
	double coverage = 0.0;

	// make a unit test sphere with 20 triangles
	sphere.Generate();
	// subdivide the sphere into 180 triangles (9*20)
	sphere.SubDivideB();
	// 2002.12.09 - now make it 4*180 = 720
	sphere.SubDivideA();

	// scale test sphere points (referenced by facet indices) to match calculated sphere radius
	for(int i=0; i<sphere.Points.GetSize(); i++)
		sphere.PointA(i)->Scale(radius);

	// decide which facet each measured point is in, then find number of populated facets
	double ang1, ang2, ang3, res;
	ang1=ang2=ang3=res = 0.0;
	int Index1, Index2, Index3;
	Index1=Index2=Index3 = 0;
	Vector spot, v1, v2, v3, meastr, v[3];
	//	double resmax = 0.0;	// for testing only
	Transform Frame, ToFrame;

	TRACE1("GeomCoverage::GetSpherePercent  we have %d facets...\n", sphere.Facets.GetSize());
	TRACE2("Input: %d points, radius = %f, center =\n", measured->GetSize(), radius);
	center.Debug();


	for( int j=0; j < measured->P.GetSize(); j++ )
		//for( int j=0; j < 13; j++ )
	{
		Vector meas = *measured->P[j];
		meas.Sub(center);
		for( int k=0; k < sphere.Facets.GetSize(); k++)
			//for( int k=0; k < 13; k++)
		{
			if( sphere.FacetA(k)->covered )
				continue;

			Index1 = sphere.FacetA(k)->PointIndex[0];
			Index2 = sphere.FacetA(k)->PointIndex[1];
			Index3 = sphere.FacetA(k)->PointIndex[2];

			// make a frame with the origin at this facet's 0 corner and the facet lying in the XY plane of the frame
			v[0] = *sphere.PointA( Index1 );
			v[1] = *sphere.PointA( Index2 );
			v[2] = *sphere.PointA( Index3 );
			Frame.Set(v);
			Frame.Invert(&ToFrame);
			//TRACE0("v[0]...\n");
			//v[0].Debug();
			//TRACE0("ToFrame...\n");
			//ToFrame.Debug();
			//Frame.Debug();

			// now get the facet pts. and the measured point in the new frame
			Vector v1, v2, v3, meastr;
			//v1 = ToFrame * v[0];
			ToFrame.Multiply(v[0], &v1);
			ToFrame.Multiply(v[1], &v2);
			ToFrame.Multiply(v[2], &v3);
			ToFrame.Multiply(meas, &meastr);

			//TRACE0("v1...\n");
			//v1.Debug();
			//TRACE0("\n");

			// if the z coord. of the measured point isn't really close to the facet plane, move on
			if(meastr.z > radius/10.0)
				continue;

			// now make 3 vectors, from the measured point to each facet corner
			Vector t1=meastr-v1;
			Vector t2=meastr-v2;
			Vector t3=meastr-v3;

			// get the angles between the 3 new vectors
			ang1 = t1.AngleBetween(t2, &spot);
			ang2 = t2.AngleBetween(t3, &spot);
			ang3 = t3.AngleBetween(t1, &spot);

			res = ang1 + ang2 + ang3;

			//TRACE(_T("point %d facet %d:   meas.z = %f ang1=%f, ang2=%f, ang3=%f, angle sum = %f\n"),
			//	j, k, meastr.z, ang1, ang2, ang3, res);

			//			if(res > resmax)
			//				resmax = res;

			//TRACE(_T("point %d: x=%f,y=%f,z=%f, facet %d: v1: x=%f,y=%f,z=%f, v2: x=%f,y=%f,z=%f, v3: x=%f,y=%f,z=%f,  ang1=%f, ang2=%f, ang3=%f, angle sum = %f\n"),
			//	j, meastr.x, meastr.y, meastr.z, k, v1.x, v1.y, v1.z, v2.x, v2.y, v2.z, v3.x, v3.y, v3.z, ang1, ang2, ang3, res);

			// NEEDS REFINING: measured point needs to be projected INTO the facet plane,
			// this should be == 360.0 with a slight margin for round off error
			if( res >= 330.0 )
			{
				TRACE0("...covered\n");
				sphere.FacetA(k)->covered = 1;
			}
		}
	}

	//	TRACE("Max angle sum = %f\n", resmax);

	int ccount = 0;
	for( int k=0; k < sphere.Facets.GetSize(); k++)
	{
		if(sphere.FacetA(k)->covered)
			ccount++;
	}
	coverage = 100.0*double(ccount)/720.0;	// 720 is the number of facets we made

	TRACE1("\nCOVERAGE = %f\n\n", coverage);
	return coverage;
}

void GeomCoveragePCMM::GetPtInTriangle(Vector &pt, Vector *ptintr)
{
	app.saarm.InstToWorking.Multiply(pt, ptintr);
}