#include "stdafx.h"
#include <math.h>
#include "sphere.h"	

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif   
SphTriangle::SphTriangle()
{
	covered = 0;
	PointIndex[0] = PointIndex[1] = PointIndex[2] = 0;
}
SphTriangle::SphTriangle(int a, int b, int c)
{
	covered = 0;
	PointIndex[0] = a;
	PointIndex[1] = b;
	PointIndex[2] = c;
}

//////////////////////////////   PRIMITIVE  /////////////////////////////////////
Sphere::Sphere()
{     
	Resolution  = 1; 
}
Sphere::~Sphere()
{
	Clear();
}  
void Sphere::Clear()
{
	for (int i=0;i<Facets.GetSize(); i++)
		delete Facets.GetAt(i);
	Facets.RemoveAll();
	for (int i=0;i<Points.GetSize(); i++)
		delete Points.GetAt(i);
	Points.RemoveAll();
	Lines.RemoveAll();
}
#define X 0.525731112119
#define Z 0.850650808352    
void Sphere::Generate()
{
	Clear();

	// build the unit sphere
	Points.Add( new Vector( -X, 0,  Z ) );
	Points.Add( new Vector(  X, 0,  Z ) );
	Points.Add( new Vector( -X, 0, -Z ) );
	Points.Add( new Vector(  X, 0, -Z ) );
	Points.Add( new Vector(  0, Z,  X ) );
	Points.Add( new Vector(  0, Z, -X ) );
	Points.Add( new Vector(  0,-Z,  X ) );
	Points.Add( new Vector(  0,-Z, -X ) );
	Points.Add( new Vector(  Z, X,  0 ) );
	Points.Add( new Vector( -Z, X,  0 ) );
	Points.Add( new Vector(  Z,-X,  0 ) );
	Points.Add( new Vector( -Z,-X,  0 ) );
	// now, the facets.
	Facets.Add( new SphTriangle( 0, 4, 1) );
	Facets.Add( new SphTriangle( 0, 9, 4) );
	Facets.Add( new SphTriangle( 9, 5, 4) );
	Facets.Add( new SphTriangle( 4, 5, 8) );
	Facets.Add( new SphTriangle( 4, 8, 1) );
	Facets.Add( new SphTriangle( 8, 10, 1) );
	Facets.Add( new SphTriangle( 8, 3, 10) );
	Facets.Add( new SphTriangle( 5, 3, 8) );
	Facets.Add( new SphTriangle( 5, 2, 3) );
	Facets.Add( new SphTriangle( 2, 7, 3) );
	Facets.Add( new SphTriangle( 7, 10, 3) );
	Facets.Add( new SphTriangle( 7, 6, 10) );
	Facets.Add( new SphTriangle( 7, 11, 6) );
	Facets.Add( new SphTriangle( 11, 0, 6) );
	Facets.Add( new SphTriangle( 0, 1, 6) );
	Facets.Add( new SphTriangle( 6, 1, 10) );
	Facets.Add( new SphTriangle( 9, 0, 11) );
	Facets.Add( new SphTriangle( 9, 11, 2) );
	Facets.Add( new SphTriangle( 9, 2, 5) );
	Facets.Add( new SphTriangle( 7, 2, 11) );
	
//	GenerateLines();
}
void Sphere::SubDivideA()
{
	int facet_count = Facets.GetSize();
	for(int i =0 ; i<facet_count;i++)
	{
		// get indices of three points.
		int Index1 = FacetA(i)->PointIndex[0];
		int Index2 = FacetA(i)->PointIndex[1];
		int Index3 = FacetA(i)->PointIndex[2];
	 	Vector Vertex1(*PointA( Index1 ));
	 	Vector Vertex2(*PointA( Index2 ));
	 	Vector Vertex3(*PointA( Index3 ));


		Vector p12;
		Vector p23;
		Vector p13;
	 
		p12 = Vertex2;
		p12.Add(Vertex1);
		p12.Normalize();
	
		p23 = Vertex3;
		p23.Add(Vertex2);
		p23.Normalize();

		p13 = Vertex3;
		p13.Add(Vertex1);
		p13.Normalize();

		int Index12 = Points.Add( new Vector(p12) );
		int Index23 = Points.Add( new Vector(p23) );
		int Index13 = Points.Add( new Vector(p13) );

		// replace origional facet with first sub-facet
		FacetA(i)->PointIndex[0] = Index12;
		FacetA(i)->PointIndex[1] = Index23;
		FacetA(i)->PointIndex[2] = Index13;
		//facet insert - insert three new facets
		Facets.Add( new SphTriangle( Index2, Index23, Index12 ) );
		Facets.Add( new SphTriangle( Index1, Index12, Index13 ) );
		Facets.Add( new SphTriangle( Index3, Index13, Index23 ) );

	}
	GenerateLines();
}


void Sphere::SubDivideB()
{
	int facet_count = Facets.GetSize();
	for(int i =0 ; i<facet_count;i++)
	{
		// get indices of three points.
		int Index1 = FacetA(i)->PointIndex[0];
		int Index2 = FacetA(i)->PointIndex[1];
		int Index3 = FacetA(i)->PointIndex[2];
	 	Vector Vertex1(*PointA( Index1 ));
	 	Vector Vertex2(*PointA( Index2 ));
	 	Vector Vertex3(*PointA( Index3 ));


		Vector p4;
		Vector p5;
		Vector p6;
		Vector p7;
		Vector p8;
		Vector p9;
		Vector p10; 

		p4 = Vertex1;
		p4.Add(Vertex1);
		p4.Add(Vertex3);
		p4.Normalize();
	
		p5 = Vertex1;
		p5.Add(Vertex1);
		p5.Add(Vertex2);
		p5.Normalize();
	
		p6 = Vertex1;
		p6.Add(Vertex3);
		p6.Add(Vertex3);
		p6.Normalize();
	
		p7 = Vertex1;
		p7.Add(Vertex2);
		p7.Add(Vertex2);
		p7.Normalize();
	
		p8 = Vertex2;
		p8.Add(Vertex2);
		p8.Add(Vertex3);
		p8.Normalize();
	
		p9 = Vertex2;
		p9.Add(Vertex3);
		p9.Add(Vertex3);
		p9.Normalize();
	
		p10 = Vertex1;
		p10.Add(Vertex2);
		p10.Add(Vertex3);
		p10.Normalize();
	

		int Index4 = Points.Add( new Vector(p4) );
		int Index5 = Points.Add( new Vector(p5) );
		int Index6 = Points.Add( new Vector(p6) );
		int Index7 = Points.Add( new Vector(p7) );
		int Index8 = Points.Add( new Vector(p8) );
		int Index9 = Points.Add( new Vector(p9) );
		int Index10 = Points.Add( new Vector(p10) );

		// replace origional facet with first sub-facet
		FacetA(i)->PointIndex[0] = Index1;
		FacetA(i)->PointIndex[1] = Index5;
		FacetA(i)->PointIndex[2] = Index4;

		//facet insert - insert three new facets
		Facets.Add( new SphTriangle( Index5, Index7, Index10 ) );
		Facets.Add( new SphTriangle( Index7, Index2, Index8 ) );
		Facets.Add( new SphTriangle( Index4, Index5, Index10 ) );
		Facets.Add( new SphTriangle( Index7, Index8, Index10 ) );
		Facets.Add( new SphTriangle( Index6, Index4, Index10 ) );
		Facets.Add( new SphTriangle( Index8, Index9, Index10 ) );
		Facets.Add( new SphTriangle( Index9, Index6, Index10 ) );
		Facets.Add( new SphTriangle( Index3, Index6, Index9 ) );

	}
	GenerateLines();
}

void Sphere::GenerateLines()
{
	Lines.RemoveAll();
	int lin[3][2];
	// given the current faceting of the sphere, generate the line list for
	// wireframe drawing
	for (int i=0;i<Facets.GetSize();i++)
	{
		lin[0][0] = FacetA(i)->PointIndex[0];
		lin[0][1] = FacetA(i)->PointIndex[1];
		lin[1][0] = FacetA(i)->PointIndex[1];
		lin[1][1] = FacetA(i)->PointIndex[2];
		lin[2][0] = FacetA(i)->PointIndex[2];
		lin[2][1] = FacetA(i)->PointIndex[0];
		for (int n = 0;n<3;n++)
		{
			BYTE AlreadyThere = FALSE;
			for (int j=0;j<Lines.GetSize();j++)
			{
				if ((LOWORD( Lines[j] ) == lin[n][0]  && HIWORD( Lines[j] ) == lin[n][1]	)
					||	(LOWORD( Lines[j] ) == lin[n][1]  && HIWORD( Lines[j] ) == lin[n][0]	))
				{
					AlreadyThere = TRUE;
					break;
				}
			}
			if (!AlreadyThere)
			{
				Lines.Add( MAKELONG( lin[n][0], lin[n][1] ) );
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
Light::Light()
{
	On = FALSE;
	Ambient = RGB( 20, 20, 20 );
	AmbientAlpha = 255;
	Diffuse = RGB( 190, 190, 190 );
	DiffuseAlpha = 255;
	Specular = RGB(  30, 30, 30 );
	SpecularAlpha = 255;
	Position = Vector( 0, 0, 1 );
	ConstantAttenuation = 1.0;
	LinearAttenuation = 0.0;
	QuadraticAttenuation = 0.0;
	SpotExponent = 0.0;
	SpotCutoff = 180.0;
	SpotDirection = Vector( 0.0, 0.0, -1.0 );
}	
void Light::GenerateGLFormat( float amb[4], float diff[4], float spec[4] )
{
	amb[0] = ((float)GetRValue(Ambient))/255.0f;
	amb[1] = ((float)GetGValue(Ambient))/255.0f;
	amb[2] = ((float)GetBValue(Ambient))/255.0f;
	amb[3] = ((float)AmbientAlpha)/255.0f;
	diff[0] = ((float)GetRValue(Diffuse))/255.0f;
	diff[1] = ((float)GetGValue(Diffuse))/255.0f;
	diff[2] = ((float)GetBValue(Diffuse))/255.0f;
	diff[3] = ((float)DiffuseAlpha)/255.0f;
	spec[0] = ((float)GetRValue(Specular))/255.0f;
	spec[1] = ((float)GetGValue(Specular))/255.0f;
	spec[2] = ((float)GetBValue(Specular))/255.0f;
	spec[3] = ((float)SpecularAlpha)/255.0f;
}

void Light::Serialize(CArchive& ar)
{
	BYTE LoadSchema;
	BYTE Schema = 0;	// angles are all in degrees!
	if (ar.IsStoring())
	{         
		ar << Schema; 
		ar << On; 
		ar << Ambient;
		ar << AmbientAlpha;
		ar << Diffuse;
		ar << DiffuseAlpha;
		ar << Specular;
		ar << SpecularAlpha;
		ar << SpotExponent;
		ar << SpotCutoff;
		ar << ConstantAttenuation;
		ar << LinearAttenuation;
		ar << QuadraticAttenuation;
	}
	else
	{
		ar >> LoadSchema; 
		ar >> On; 
		ar >> Ambient;
		ar >> AmbientAlpha;
		ar >> Diffuse;
		ar >> DiffuseAlpha;
		ar >> Specular;
		ar >> SpecularAlpha;
		ar >> SpotExponent;
		ar >> SpotCutoff;
		ar >> ConstantAttenuation;
		ar >> LinearAttenuation;
		ar >> QuadraticAttenuation;
	}  
	Position.Serialize( ar );
	SpotDirection.Serialize( ar );
}
IMPLEMENT_SERIAL( Light, CObject, 0 );
///////////////////////////////////////////////////////////////////////////////
Material::Material()
{
	Ambient = RGB( 0, 0, 255);
	AmbientAlpha = 255;
	Specular = RGB( 255, 255, 255 );
	SpecularAlpha = 255;
	Shininess = 10;
	Emission = RGB( 0, 0, 0 );
	EmissionAlpha = 255;

	Name = "";			// user given name of the material
}
void Material::GenerateGLFormat( float amb[4], float spec[4], float shine[1], float emis[4] )
{
	amb[0] = ((float)GetRValue(Ambient))/255.0f;
	amb[1] = ((float)GetGValue(Ambient))/255.0f;
	amb[2] = ((float)GetBValue(Ambient))/255.0f;
	amb[3] = ((float)AmbientAlpha)/255.0f;
	spec[0] = ((float)GetRValue(Specular))/255.0f;
	spec[1] = ((float)GetGValue(Specular))/255.0f;
	spec[2] = ((float)GetBValue(Specular))/255.0f;
	spec[3] = ((float)SpecularAlpha)/255.0f;
	shine[0] = (float) Shininess;
	emis[0] = ((float)GetRValue(Emission))/255.0f;
	emis[1] = ((float)GetGValue(Emission))/255.0f;
	emis[2] = ((float)GetBValue(Emission))/255.0f;
	emis[3] = ((float)EmissionAlpha)/255.0f;
}
BOOL Material::Compare( Material& m )
{
	// check all components except the name!  if the same, return true!
	if (Ambient != m.Ambient) return FALSE;
	if (AmbientAlpha != m.AmbientAlpha) return FALSE;
	if (Specular != m.Specular) return FALSE;
	if (SpecularAlpha != m.SpecularAlpha) return FALSE;
	if (Shininess != m.Shininess) return FALSE;
	if (Emission != m.Emission) return FALSE;
	if (EmissionAlpha != m.EmissionAlpha) return FALSE;
	return TRUE;
}
void Material::CopyProperties( Material& m )
{
	Ambient = m.Ambient;
	AmbientAlpha = m.AmbientAlpha;
	Specular = m.Specular;
	SpecularAlpha = m.SpecularAlpha;
	Emission = m.Emission;
	EmissionAlpha = m.EmissionAlpha;
	Name = m.Name;
}
void Material::WriteNAX(FILE* f )
{
	fprintf(f,"%s\n",Name);
	fprintf(f,"%d %d %d %d %d %d %d\n",
		Ambient,
		AmbientAlpha,
		Specular,
		SpecularAlpha,
		Shininess,
		Emission,
		EmissionAlpha);
}
void Material::ReadNAX(FILE* f )
{
	int aa,sa,ea,s;

	fscanf(f,"%d%d%d%d%d%d%d%*[^\n]",
		&Ambient,
		&aa,
		&Specular,
		&sa,
		&s,
		&Emission,
		&ea);

	AmbientAlpha = aa;
	SpecularAlpha = sa;
	EmissionAlpha = ea;
	Shininess = s;

/*
	TRACE(" loaded material %d %d %d %d %d %d %d\n",
		Ambient,
		AmbientAlpha,
		Specular,
		SpecularAlpha,
		Shininess,
		Emission,
		EmissionAlpha);
*/
}

void Material::Serialize(CArchive& ar)
{
	BYTE LoadSchema;
	BYTE Schema = 0;	// angles are all in degrees!
	if (ar.IsStoring())
	{         
		ar << Schema; 
		ar << Name; 
		ar << Ambient;
		ar << AmbientAlpha;
		ar << Specular;
		ar << SpecularAlpha;
		ar << Shininess;
		ar << Emission;
		ar << EmissionAlpha;
	}
	else
	{
		ar >> LoadSchema; 
		ar >> Name;
		ar >> Ambient;
		ar >> AmbientAlpha;
		ar >> Specular;
		ar >> SpecularAlpha;
		ar >> Shininess;
		ar >> Emission;
		ar >> EmissionAlpha;
	}      
}
IMPLEMENT_SERIAL( Material, CObject, 0 );

  

