// sphere class
#pragma once

class SphTriangle: public CObject
{
public:
	SphTriangle();
	SphTriangle( int a, int b, int c );
	int PointIndex[3];
	BYTE covered;	// blt, for % sphere coverage
};

class Sphere : public CObject
{
public:
	Sphere();
	~Sphere();
			
	int Resolution;
	CObArray Facets;
	CObArray Points;

	CDWordArray Lines;

	SphTriangle* FacetA(int i)
		{return (SphTriangle*) Facets.GetAt(i);}
	Vector* PointA(int i)
		{return (Vector*) Points.GetAt(i);}

	void Generate();			// fills the members using the set resolution.
	void Clear();
	void SubDivideA();
	void SubDivideB();
	void GenerateLines();

protected:

};

class Light : public CObject
{
public:
	BYTE On;

	COLORREF Ambient;
	BYTE AmbientAlpha;
	COLORREF Diffuse;
	BYTE DiffuseAlpha;
	COLORREF Specular;
	BYTE SpecularAlpha;
	Vector Position;
	Vector SpotDirection;
	
	double SpotExponent;
	double SpotCutoff;
	double ConstantAttenuation;
	double LinearAttenuation;
	double QuadraticAttenuation;

	Light();

	void GenerateGLFormat( float amb[4], float diff[4], float spec[4] );
	virtual void Serialize(CArchive& ar);	// overridden for document i/o  
protected:
	DECLARE_SERIAL( Light );
};

// material properties class.
class Material : public CObject
{
public:
	COLORREF Ambient;
	BYTE AmbientAlpha;
	COLORREF Specular;
	BYTE SpecularAlpha;
	BYTE Shininess;
	COLORREF Emission;
	BYTE EmissionAlpha;

	CString Name;			// user given name of the material

	Material();

	void GenerateGLFormat( float amb[4], float spec[4], float shine[1], float emis[4] );
	BOOL Compare( Material& m );
	void CopyProperties( Material& m );
	void WriteNAX(FILE* f );
	void ReadNAX(FILE* f );

	virtual void Serialize(CArchive& ar);	// overridden for document i/o  
protected:
	DECLARE_SERIAL( Material );
};


