
#include "stdafx.h"
#include <math.h>
#include "fitball.h"
#include "ballfitmatrix.h"
#include "..\\DPCMM.h"

////////////////////////// ball fitting
///////////////////////////////
FitBall::FitBall()
{
//	Perturb = 0.1;
//	PerturbScale = 1.0;
//	PerturbLimit = 1e-5;
//	FailGain = 0.8;
	outsideRMStol = 0;
	pct = 0.0;
	rmserr = 0.0;		// blt
	VaryRad = TRUE;	// blt
}
void FitBall::SetOld()
{
	OldCenter = Center;
	OldRadius = Radius;
}
void FitBall::GetOld()
{
	Center = OldCenter;
	Radius = OldRadius;
}

#define SMALLNUM 1e-6
BOOL FitBall::GetMidPlane(Vector& P1,Vector& P2,Vector* Norm,double* D)
{
	//get the midplane normal
	*Norm = P2;
	Norm->Sub(P1);
	if(Norm->Length()<SMALLNUM)
		return FALSE;
	Norm->Normalize();
	//get the midpoint
	Vector Mid(P1);
	Mid.Add(P2);
	Mid.Scale(0.5);  //now its the midpoint
	//evaluate D
	*D = -(Norm->Dot(Mid));
	return TRUE;
}
BOOL FitBall::FourPtSphere(Vector& P1,Vector& P2,Vector& P3,Vector& P4, Vector* Cent, double* Rad)
{
//*******************
   EquationSystem3x4 eq;   
	Vector Norm;
	double D=0.0;
	if(!GetMidPlane(P1,P2,&Norm,&D))
		return FALSE;
	eq.A[0][0] = Norm.x;
   eq.A[0][1] = Norm.y;
   eq.A[0][2] = Norm.z;
   eq.b[0]  = - D;
	if(!GetMidPlane(P2,P3,&Norm,&D))
		return FALSE;
	eq.A[1][0] = Norm.x;
   eq.A[1][1] = Norm.y;
   eq.A[1][2] = Norm.z;
   eq.b[1]  = - D;
	if(!GetMidPlane(P3,P4,&Norm,&D))
		return FALSE;
	eq.A[2][0] = Norm.x;
   eq.A[2][1] = Norm.y;
   eq.A[2][2] = Norm.z;
   eq.b[2]  = - D;
	Vector P;
   if (!eq.Cramer( &P ))		// intersect the three planes!
   {        
      //CString msg("Error Intersecting 3 planes to form a point.");  
      //msg += "\nMake sure the planes are not parallel.";
      //AfxMessageBox( msg );	            	
      return FALSE;
   }	
	*Cent = P;
	*Rad = P1.Distance(P);
	return TRUE;
}
BOOL FitBall::Guess()
{
	// make only one guess using the 4 farthest points.
	VectorArray Points;
	for (int i=0;i<pPoints->GetSize();i++)
		Points.P.Add(new Vector(PtA(i)) );
	int f1 = 0;
	int f2 = 0;
	int f3 = 0;
	int f4 = 0;
	if (!Points.Find4FarthestPoints(&f1,&f2,&f3,&f4))
		return FALSE;
	TRACE("farthest 4 = %d %d %d %d \n",f1,f2,f3,f4);
	if(!FourPtSphere(PtA(f1),PtA(f2),PtA(f3),PtA(f4),&Center,&Radius))
		return FALSE;

	return TRUE;
}


BOOL FitBall::Calculate(int* retcount, double* lockrad, double* caserad)
{
//	double ProgressSpan = log(PerturbLimit) / log(FailGain);	// jmcprogress
//	int ProgressFailures = 0;
//	NRKProgress prog(CString("Fitting a Sphere..."));		// jmcprogress
	*retcount=0;
	TRACE("***beginning Ball fit, lockrad = %f in, caserad = %f in\n\n", *lockrad,*caserad);
	if(!Guess())
	{
		TRACE("Guess Failed\n");
		//use old Guess at the center
		Center = Vector(0,0,0);
		for (int i=0;i<pPoints->P.GetSize(); i++)
		{
			Center.Add( PtA(i) );
		}
		Center.Scale( 1.0/pPoints->P.GetSize() );			// determine average!
		// guess at the radius!
		Vector v =  PtA(0);
		v.Sub( Center );		  
		Radius = v.Length();          // get vector length
	}
	TRACE("Rad = %f   Cent = %f  %f  %f \n",Radius,Center.x,Center.y,Center.z);
	/////////////////
	//temp for testing the guess
	//pBall->PrimitivePos = Center;
	//pBall->Radius = Radius;
	//return TRUE;
	/////////////////
	//we now have an initial guess based on the 
	//Avg as the center and the dist to the 1st point as the radius
	//  May want to modify later
	TRACE("Out of Guess\n");
	TRACE("		Cent = %lf %lf %lf \n",Center.x,Center.y,Center.z);
	TRACE("		Rad = %lf\n",Radius);

	BallFitMatrix sys;
	sys.ShowProgress = FALSE;	// BLT 2000.11.20
	sys.VaryRad = VaryRad;
	sys.pObj = this;
	sys.Prepare( pPoints->P.GetSize(), (sys.VaryRad) ? 4 : 3);
	sys.SetParameters( *pOpt );

	int index = 0;
	double dpos = pOpt->PosDelta;
	double drot = pOpt->AngDelta;
	sys.x[index] = Center.x;
	sys.dx[index++] = dpos;
	sys.x[index] = Center.y;
	sys.dx[index++] = dpos;
	sys.x[index] = Center.z;
	sys.dx[index++] = dpos;
	if (sys.VaryRad)
	{
		sys.x[index] = Radius;
		sys.dx[index++] = dpos;
	}
	else
		Radius = *lockrad + *caserad;

	sys.Solve();

	// Get Percent Coverage
	pct = Coverage.GetSpherePercent(Center, Radius, pPoints);

	// build residual text array!
	Resid.RemoveAll();
	double MaxError = -1e6;
	// 2000.11.21 took out all the report stuff, it was for trackers, the arms
	// present a report as they go, calling this fit as new data is acquired
//	CStringArray ResidPts;
	for (int i=0;i<pPoints->P.GetSize(); i++)
	{
		double err = sys.rhs[index++];
		TRACE("err = %f\n",err);
/*		CString t;
		t.Format("%3d  error %9.5f %s",i+1,
			err*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
		ResidPts.Add( t );
*/
		MaxError = max(MaxError, fabs(err));
	} 
//	CString txt;
	rmserr = SafeSqrt( sys.SumRhs() );
	rmserr /= app.saarm.units.UnitToMeters[app.saarm.units.distunits];
/*	txt.Format("  Maximum error   %f %s.  RMS %f %s.",MaxError*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits], rmserr,
			app.saarm.units.DistUnitNames[app.saarm.units.distunits] );
	Resid.Add( txt );

	// Percent Coverage
	txt.Format("Percent Coverage = %f", pct);
	Resid.Add( txt );

// following added for Boeing 2000.07.27
	if(SafeSqrt( sys.SumRhs() ) > Tolerance)
	{
		outsideRMStol = 1;
		txt.Format("Outside RMS rejection tolerance of %f %s.",
		app.saarm.Data.SphereTol/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
		Resid.Add( txt );
	}
*/
	// get the results into the Working Frame!! blt 9/14/99
	Vector center, CenterW;
	center = Center;
	center.Debug();
	center.Scale(app.saarm.units.UnitToMeters[DISTUNITS_IN]);	// transform is in meters
	app.saarm.InstToWorking.Multiply(center, &CenterW);
	CenterW.Scale(1.0f/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);	// now go to working units
//	txt.Format("Center = %f %f %f %s.", CenterW.x, CenterW.y, CenterW.z,
//		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
//	Resid.Add(txt);

	Radius -= *caserad;
	double radmeters;
	if(sys.VaryRad)
	{
		radmeters = Radius*app.saarm.units.UnitToMeters[DISTUNITS_IN];
	}
	else
		radmeters = *lockrad*app.saarm.units.UnitToMeters[DISTUNITS_IN];
/*	
	txt.Format("Radius = %f %s.", radmeters/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
											app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	Resid.Add( txt );

	txt.Format("Sphere Fit Radius offset by -%f %s (probe tip radius)",
		app.saarm.Data.TipDia/(2.0 * app.saarm.units.UnitToMeters[app.saarm.units.distunits]),
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	Resid.Add(txt);

	if(!sys.VaryRad)
	{
		txt.Format("Radius locked at %f %s during fit.", 
			radmeters/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
		Resid.Add(txt);
	}

	txt.Format("-----------------------------------------");
	Resid.Add( txt );

	// Now report points AFTER summary
	for(i=0; i<ResidPts.GetSize(); i++)
		Resid.Add(ResidPts[i]);

	////////////////////////////////
*/	
	return TRUE;
}

void FitBall::EvaluateVec( double* e )
{
	int index = 0;
	for ( int i=0;i<pPoints->P.GetSize(); i++)
	{
		PtA(i).Debug();
		pPoints->P[i]->Debug();
		Center.Debug();
		TRACE("radius = %f\n", Radius);
		double err = PtA(i).Distance(Center) - Radius ; 
		e[index++] = err;
	}   
}
