
#include "stdafx.h"
#include "fitcircle.h"
#include "fitcirclematrix.h"
#include "..\\DPCMM.h"

////////////////////////// circle fitting
///////////////////////////////
FitCircle::FitCircle()
{
	rejctr = 0;
	Theta = 10;
	Phi = 10;
	VaryRad = TRUE;	// blt
}
BOOL FitCircle::Guess()
{
	Vector CenterGuess;
	Vector NormalGuess;
	
	Vector InitNormal;
	if(!ThreePtCircle(PtA(0),PtA(1),PtA(2),&CenterGuess,&InitNormal,&Radius))
		InitNormal=Vector(0,0,1);
	NormalGuess = InitNormal;

	int fari=0,fari2=0,fari3=0;
	if (pPoints->Find3FarthestPoints(&fari,&fari2,&fari3))
	{
		if (ThreePtCircle(PtA(fari),PtA(fari2),PtA(fari3),&CenterGuess,&NormalGuess,&Radius))
		{
			if(NormalGuess.Dot(InitNormal)<0.0) // match first 3 direction
				NormalGuess.Scale(-1.0);
		}
		TRACE("3 farthest points = %d %d %d\n",fari,fari2,fari3);
	}

	Vector rtp;
	NormalGuess.XYZtoRTP( &rtp );
	Theta = rtp.y;
	Phi = rtp.z;
	Center = CenterGuess;
	return TRUE;
}

#define SMALLNUM 1e-6
BOOL FitCircle::ThreePtCircle(Vector& P1,Vector& P2,Vector& P3, Vector* Cent,Vector* Norm, double* Rad)
{
	Vector L1(P2);
	L1.Sub(P1);
	L1.Normalize();

	Vector L2(P3);
	L2.Sub(P2);
	L2.Normalize();
		
	//need to make this return false if colinear
	double temp = L1.Dot(L2);
	TRACE("\nTemp = %lf\n",temp);
	if(fabs(temp)>(1.0-SMALLNUM))
		return FALSE;


	double d = P3.DistanceAlongAxisToPoint( P1, P2 );
	Vector diff1(L1);
	diff1.Scale( d );
	diff1.Add( P1 );
	diff1.Sub( P3 );
	//get midpoint
	Vector K1(P1);
	K1.Add(P2);
	K1.Scale(0.5);

	d = P1.DistanceAlongAxisToPoint( P2, P3 );
	Vector diff2(L2);
	diff2.Scale( d );
	diff2.Add( P2 );
	diff2.Sub( P1 );
	//get midpoint
	Vector K2(P2);
	K2.Add(P3);
	K2.Scale(0.5);

	Ray r1, r2;
	r1.Normal= diff1;
	r1.StartPoint = K1;
	r2.Normal= diff2;
	r2.StartPoint = K2;
	Vector IntA,IntB;

	TRACE0("Vectors Into Ray Compare:\n");
	TRACE0("r1:\n");
	r1.Debug();
	TRACE0("r2:\n");
	r2.Debug();

	double e = RayCompareForFit( r1, r2, &IntA, &IntB );

	TRACE0("Vectors After Ray Compare:\n");
	TRACE0("IntA:\n");
	IntA.Debug();
	TRACE0("IntB:\n");
	IntB.Debug();
	TRACE("Mutual Perp Dist = %lf %lf\n\n",e,IntA.Distance(IntB));

	*Cent = IntA;
	Cent->Add(IntB);
	Cent->Scale(0.5);

	Norm->Cross(diff1,diff2);
	*Rad = Cent->Distance(P1);
	return TRUE;	
}
double FitCircle::RayCompareForFit(Ray& RayA, Ray& RayB, Vector* pPA, Vector* pPB, bool* pAnyNeg)
{
	Vector CommonNormal;	//
	CommonNormal.Cross(RayA.Normal,RayB.Normal);		//   Compute Mutual Perpendicular Direction

	Vector PlaneANorm;			 	//
	PlaneANorm.Cross(RayA.Normal,CommonNormal);		//   compute AB&C for first intersecting plane
	double PlaneAD = -(PlaneANorm.Dot(RayA.StartPoint));	 // D for first plane

	Vector PlaneBNorm;				//
	PlaneBNorm.Cross(RayB.Normal,CommonNormal);		//   compute AB&C for second intersecting plane
	double PlaneBD = -(PlaneBNorm.Dot(RayB.StartPoint));	  // D for second plane

	// evaluate first theo points in second plane
	double Distance1 = PlaneBNorm.Dot(RayA.StartPoint)+PlaneBD;
	Vector Point2 = RayA.StartPoint;
	Point2.Add(RayA.Normal); 
	double Distance2 = PlaneBNorm.Dot(Point2)+PlaneBD;

	double originDistance = RayA.StartPoint.Distance(RayB.StartPoint)/2.0;

	double DistanceToMoveA = originDistance;
	double denom = Distance1-Distance2;
	if (denom!=0.0)		// jmc 2007.12.06 - fixed div / zero
		DistanceToMoveA = Distance1/denom;

	Vector PointA(RayA.Normal);
	PointA.Scale(DistanceToMoveA);
	PointA.Add(RayA.StartPoint);

	// evaluate second theo points in first plane
	Distance1 = PlaneANorm.Dot(RayB.StartPoint)+PlaneAD;
	Point2 = RayB.StartPoint;
	Point2.Add(RayB.Normal); 
	Distance2 = PlaneANorm.Dot(Point2)+PlaneAD;

	double DistanceToMoveB = originDistance;
	denom = Distance1-Distance2;
	if (denom!=0.0)		// jmc 2007.12.06 - fixed div / zero
		DistanceToMoveB = Distance1/denom;

	Vector PointB(RayB.Normal);
	PointB.Scale(DistanceToMoveB);
	PointB.Add(RayB.StartPoint);

	*pPA = PointA;
	*pPB = PointB;

	if (pAnyNeg)
	{
		if (DistanceToMoveA<0 || DistanceToMoveB<0)
			*pAnyNeg = true;
		else
			*pAnyNeg = false;
	}

	return(PointA.Distance(PointB));
}

BOOL FitCircle::Calculate(Vector* pStat )
{
	// Guess the normal!  
	if(!Guess())
	{
		AfxMessageBox(_T("Colinear Points Selected\n\t Circle Fit Aborted\n"));
		return FALSE;
	}

	TRACE("Out of Guess\n");
	TRACE("		Cent = %lf %lf %lf \n",Center.x,Center.y,Center.z);
	TRACE("		Rad = %lf  Theta = %lf   Phi = %lf\n",Radius,Theta,Phi);

	FitCircleMatrix sys;
	sys.ShowProgress = FALSE;	// 2000.12.29
	sys.VaryRad = VaryRad;
	sys.pObj = this;
	sys.Prepare( pPoints->P.GetSize()*2, (sys.VaryRad) ? 6 : 5 );
	sys.SetParameters( *pOpt );
	
	int index = 0;
	double dpos = pOpt->PosDelta;
	double drot = pOpt->AngDelta;
	sys.x[index] = Center.x;
	sys.dx[index++] = dpos;
	sys.x[index] = Center.y;
	sys.dx[index++] = dpos;
	sys.x[index] = Center.z;
	sys.dx[index++] = dpos;
	if (sys.VaryRad)
	{
		sys.x[index] = Radius;
		sys.dx[index++] = dpos;
	}
	sys.x[index] = Theta;
	sys.dx[index++] = drot;
	sys.x[index] = Phi;
	sys.dx[index++] = drot;

	sys.Solve();

	// build residual text array!
	Resid.RemoveAll();
	double MaxError = -1e6;
	double TotError = 0.0;
	rejctr=0;
	// 2000.11.21 took out all the report stuff, it was for trackers, the arms
	// present a report as they go, calling this fit as new data is acquired
//	CStringArray ResidPts;
	int rhsIdx = 0;
	for ( int i=0;i<pPoints->P.GetSize(); i++)
	{
		double radial = sys.rhs[rhsIdx++];
		double planar = sys.rhs[rhsIdx++];
		Vector temp( radial, planar, 0.0 );
		double err = temp.Length();
/*		CString t;
		t.Format(_T("%d  Radial %f %s.  Plane %f %s.  Magnitude %f %s."),
			i+1,radial*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits],
			planar*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits],
			err*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
*/		if(fabs(err*app.saarm.units.UnitToMeters[DISTUNITS_IN])>app.saarm.Data.CircleTol)
		{
			(rejctr)++;
//			t += "   -----  Outside Rejection Limit!";
		}
//		ResidPts.Add( t );

		TotError += err;
		MaxError = max(MaxError, err);
	} 
	pStat->x = MaxError;
	pStat->y = TotError / pPoints->P.GetSize();
	pStat->z = SafeSqrt( sys.SumRhs() );

/*	CString txt;
	txt.Format(_T("  Maximum error   %f %s.  RMS %f %s."),
			MaxError*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits], 
			SafeSqrt( sys.SumRhs() )*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits] );
	Resid.Add( txt );
	txt.Format(_T("%d points outside rejection tolerance of %f %s."),
		rejctr, app.saarm.Data.CircleTol/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	Resid.Add( txt );
	double radmeters = Radius*app.saarm.units.UnitToMeters[DISTUNITS_IN];
	txt.Format(_T("Radius = %f %s."),
		radmeters/app.saarm.units.UnitToMeters[app.saarm.units.distunits], app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	Resid.Add( txt );
	if (!sys.VaryRad)
	{
		txt.Format(_T("  The radius was locked during the fit."));
		Resid.Add( txt );
	}
*/
	// get the results into the Working Frame!! blt 9/14/99
	Vector center, CenterW, NormalW;

	center = Center;
	center.Scale(app.saarm.units.UnitToMeters[DISTUNITS_IN]);	// transform is in meters
	app.saarm.InstToWorking.Multiply(center, &CenterW);
	CenterW.Scale(1.0f/app.saarm.units.UnitToMeters[app.saarm.units.distunits]);	// now go to working units

	// Normal is a vector w/ origin at Center, it's coordinates do not get translated, only rotated
	app.saarm.InstToWorking.MultiplyRot(Normal, &NormalW);
/*	
	txt.Format(_T("Center = %f %f %f %s."), CenterW.x, CenterW.y, CenterW.z,
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	Resid.Add(txt);

//	txt.Format(_T("Normal = %f %f %f"), Normal.x, Normal.y, Normal.z);	in World
//	Resid.Add( txt );

	txt.Format(_T("Normal = %f %f %f"), NormalW.x, NormalW.y, NormalW.z);
	Resid.Add( txt );
	
	txt.Format(_T("-----------------------------------------"));
	Resid.Add( txt );
	
	for(i=0; i<ResidPts.GetSize(); i++)
		Resid.Add(ResidPts[i]);
*/
	////////////////////////////////

	return TRUE;
}
void FitCircle::EvaluateVec( double* e )
{
	Vector rtp(1.0,Theta,Phi);
	rtp.RTPtoXYZ( &Normal );
	Vector OffsetNormal(Normal);
	OffsetNormal.Add( Center );
	int index = 0;
	for ( int i=0;i<pPoints->P.GetSize(); i++)
	{
		double RadialDist = PtA(i).DistanceFromAxisToPoint(Center, OffsetNormal) ; 
		double RadialError = Radius - RadialDist;
		double PlaneError = PtA(i).DistanceAlongAxisToPoint(Center, OffsetNormal) ;
		e[index++] = RadialError;
		e[index++] = PlaneError;
	}   
}
