// Plane.cpp Plane Primitive class and dialog class.
// 
#include "stdafx.h"
#include "FitPlane.h" 
#include "FitPlaneMatrix.h"
#include "..\\DPCMM.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif   
////////////////////////// Plane fitting
///////////////////////////////
FitPlane::FitPlane()
{
	Theta = 10;
	Phi = 10;
	D = 0.0;
	ShowResults = TRUE;
	//pPlane = NULL;
	pArray = NULL;
	//pKnowns = NULL;
}
BOOL FitPlane::Guess(Vector* FitNorm,Vector* FitPoint)
{
	if (pPoints->GetSize() <3)
		return FALSE;
	Vector RefNorm;
	if(!ThreePtPlane(PtA(0),PtA(1),PtA(2),FitNorm,FitPoint))
	{
		RefNorm = Vector(0.0,0.0,1.0);
		*FitPoint = PtA(0);
		*FitNorm = Vector(0,0,1);
	}
	else
		RefNorm = *FitNorm;
	
	if (pPoints->GetSize()==3)		// no need to get fancy!
		return TRUE;

	int fari=0,fari2=0,fari3=0;
	Vector fn,fp;
	if (pPoints->Find3FarthestPoints(&fari,&fari2,&fari3))
	{
		if(ThreePtPlane(PtA(fari),PtA(fari2),PtA(fari3),&fn,&fp))
		{
			*FitNorm = fn;
			*FitPoint = fp;
		}	
		////////////////// use these three points to form a plane.
		TRACE("3 farthest points = %d %d %d\n",fari,fari2,fari3);
	}
	// flip the guess to match the first 3 points...
	if(RefNorm.Dot(*FitNorm)<0.0) //check whether to reverse or not
		FitNorm->Scale(-1.0);
	return TRUE;

}

#define SMALLVAL 1e-3
BOOL FitPlane::ThreePtPlane(Vector& P0,Vector& P1,Vector& P2,Vector* Normal,Vector* Point)
{
	Vector Diff1(P1);       // Get first side vector Diff1 = P1-P0
	Diff1.Sub(P0); 
	Vector Diff2(P2);       // Get second side vector Diff2 = P2-P1
	Diff2.Sub(P1);
	Vector dummy;
	if(Diff1.AngleBetween(Diff2,&dummy)<SMALLVAL)
		return FALSE;
	Normal->Cross(Diff1,Diff2);               // Find normal vector direction as Diff1 X Diff2
	Normal->Normalize();               // Normalize it

   *Point = P0;
	Point->Add(P1);
	Point->Add(P2);
	Point->Scale(1.0/3.0);
	return TRUE;
}


BOOL FitPlane::Calculate(  )
{
	LocPoints.Clear();
	pPoints->Average(&LocCenter);
	for (int k=0;k<pPoints->GetSize();k++)
	{
		Vector* p = new Vector(*pPoints->P[k]);
		p->Sub(LocCenter);
		LocPoints.P.Add( p );
	}

	MaxError = 0.0;
	RMSError = 0.0;
	// make the initial guess for the normal.
	Vector ImagPoint;
	if(!Guess(&FittedNormal,&ImagPoint))
	{
		AfxMessageBox(_T("Insufficient Data To Fit"));
		return FALSE;
	}

	Vector ImagPointWorld(ImagPoint);
	ImagPointWorld.Add(LocCenter);			// take out the center shift.
	Trans.Set( ImagPointWorld, FittedNormal );

   if (pPoints->P.GetSize() > 3)
   {
		TRACE("real fit!\n");
		// place guess in the proper format. 
		Vector rtp;
		FittedNormal.XYZtoRTP( &rtp); 
		Theta = rtp.y;          // theta and phi from the xyz vector... hack... hack... hack
		Phi = rtp.z;
		
		// determine the center point of the plane (average in 3-d)
		Vector Total;
		for (int i=0;i<pPoints->P.GetSize();i++)
			Total.Add( PtA(i) );   
		Total.Scale( 1.0/((double)pPoints->P.GetSize()) );
		ImagPoint = Total;                      // this is the point!
		D = -ImagPoint.Dot( FittedNormal );		// determine d to start the optimization.


		FitPlaneMatrix sys;
		sys.ShowProgress = FALSE;	// 2000.12.29
		sys.Prepare( pPoints->P.GetSize(), 3 );
		sys.SetParameters( *pOpt );
		sys.pObj = this;

		int index = 0;
		double dpos = pOpt->PosDelta;
		double drot = pOpt->AngDelta;
		sys.x[index] = D;
		sys.dx[index++] = dpos;
		sys.x[index] = Theta;
		sys.dx[index++] = drot;
		sys.x[index] = Phi;
		sys.dx[index++] = drot;

		sys.Solve();

		// now, convert the D value to a "Imag Point" that is on the plane... so we can use a transform
		double err = FittedNormal.Dot( ImagPoint ) + D;
		Vector norm(FittedNormal);
		norm.Scale( -err );		// apply lambda
		ImagPoint.Add(norm);

		// take out the average center shift for moment arm reduction:
		ImagPoint.Add( LocCenter );

		
		Trans.Set( ImagPoint, FittedNormal );
//		if (pPlane != NULL)
//			pPlane->SetTransform(Trans);
		
		// now, report the error!
		Resid.RemoveAll();
		CString t;
		CStringArray ti;
		double total = 0.0;
		retcount = 0;
		MaxError = 0.0;
		for (int i=0;i<pPoints->P.GetSize();i++) 
		{
			CString fullname;
			if (pArray != NULL)
			{
				t.Format(_T("Dist from plane %f %s."),fullname,
															sys.rhs[i]*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
															app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
			}
			else
				t.Format(_T("%d  Dist from plane %f %s."),i,
																sys.rhs[i]*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
																app.saarm.units.DistUnitNames[app.saarm.units.distunits]);

			if(fabs(sys.rhs[i])*app.saarm.units.UnitToMeters[DISTUNITS_IN]>app.saarm.Data.PlaneTol)
			{
				TRACE("Big One!!!!!!!!!!!!!!\n");
				retcount++;
				t += "   -----  Outside Rejection Limit!";
			}
			ti.Add( t );

			MaxError = max(MaxError, fabs(sys.rhs[i]));
		}
		RMSError = SafeSqrt(sys.SumRhs());

		t.Format(_T("  Maximum error   %f %s.  RMS %f %s."),
			MaxError*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits],
			SafeSqrt( sys.SumRhs() )*app.saarm.units.UnitToMeters[DISTUNITS_IN]/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits] );
		Resid.Add(t);
		t.Format(_T("%d points outside rejection tolerance of %f %s."),
			retcount, app.saarm.Data.PlaneTol/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
			app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
		Resid.Add(t);

		t.Format(_T("-----------------------------------------"));
		Resid.Add(t);

		for(int i=0; i<ti.GetSize(); i++)
			Resid.Add(ti[i]);
		
/*		CString t;
		t.Format(_T("         Maximum error   %f  RMS %f"),MaxError, SafeSqrt(sys.SumRhs()));
		Resid.Add( t );
		t.Format(_T("---------------------------------------"));
		Resid.Add(t);
		t.Format(_T("Results in working frame:  %s"),d.WorkingFrameName);
		Resid.Add(t);

		// you can remove this if you need to.... Just for reporting coeff in right frame.
		double a,b,c,d1;
		Plane ptemp;
		ptemp.pDoc = CTDoc::GetDoc();
		ptemp.SetTransform(Trans);
		ptemp.GetCoeffInWorking(&a,&b,&c,&d1);
		t.Format(_T("A = %f  B = %f  C = %f  D = %f"),a,b,c,d1);
		Resid.Add(t);

		if (ShowResults)
		{
			// display residuals.
			DResiduals diag;
			diag.Title = "Plane Fit Results";
			d.Log(diag.Title);
			d.Log(Resid);
			diag.a = &Resid;
			if (diag.DoModal()!=IDOK)
			{
				d.Log("Results NOT accepted!"));
				return FALSE;
			}
			d.Log("Results accepted!"));
		}*/
	}

	return TRUE;
}
void FitPlane::Evaluate(double e[])
{ 
	// convert from theta phi to normal vector.
	Vector rtp( 1.0, Theta, Phi );
	rtp.RTPtoXYZ( &FittedNormal) ; 
	FittedNormal.Normalize();    

	TRACE("EVAL plane\n");
	int index=0;
	for (int i=0;i<pPoints->P.GetSize();i++) 
	{
		e[index] = FittedNormal.Dot( PtA(i) ) + D; 
/*		if (pKnowns != NULL)		// offset the point!
		{
			e[index] = e[index] - pKnowns->x[i];
			TRACE("%d  %f\n"),index,e[index]);
		}*/
		index++;
	}
}

