#include <time.h>
#include <afxdisp.h>
#include "stdafx.h"
#include "dmcwin.h"
#include <stdio.h>
#include "VDCPos.h"

const double pi=4.0*atan(1.0);
const double deg2rad=pi/180.0;
const double rad2deg=1/deg2rad;

void VDCPosition::VDCZeroValue(void)
{	
	/*******************
	This VDCPosition class member function initializes all of the 
	VDC variables to zero.

	It returns nothing.
	*******************/

	int i,j,k;

	NumCalPose=0;
	NumCalCalcPnts=0;

	for (k=0; k<20; ++k )
	{	for (j=0; j<3; ++j )
		{	for (i=0; i<3; ++i )
				CalibdThetaMat[k][i][j]=0.0;
		}
	}

	for (j=0; j < 3; ++j )
	{	EncCnts[j]=0;
		EncOS[j]=0.0;
		dTheta[j]=0.0;
		Theta[j]=0.0;
		CartPos[j]=0.0;
		S[j]=0.0;
		EncConst[j]=0.0;
	}

	for (j=0; j<3; ++j)
	{	for (i=0; i<2; ++i)
		{	a[i][j]=0.0;
			alpha[i][j]=0.0;
		}
	}
}

void VDCPosition::VDCLoadKP(char *filename)
{	
	/*******************
	This VDCPosition class member function reads data from a datafile supplied
	in the function call. The data in the file includes parameters for initialization,
	kinematic parameters, as well as offsets for the home position.

	It returns nothing.
	*******************/

	FILE *KPdat;
	char DummyTxt[25];
	int i;
	
	BOOL open = TRUE;	// blt 2001.06.13 -- bad news if the file wasn't found!
	if ((KPdat=fopen(filename,"r"))==NULL)
	{
		TRACE("Parameter file did not open OK.\n");
		open = FALSE;	// blt
	}
	else
	{	fscanf(KPdat,"%s %d",DummyTxt, &NumCalPose);
		fscanf(KPdat,"%s %d",DummyTxt, &NumCalCalcPnts);
		fscanf(KPdat,"%s %lf,%lf",DummyTxt, &a[0][1],&a[1][2]);
		fscanf(KPdat,"%s %lf,%lf",DummyTxt, &alpha[0][1],&alpha[1][2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &S[0],&S[1],&S[2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &Ptool3[0],&Ptool3[1],&Ptool3[2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &EncConst[0],&EncConst[1],&EncConst[2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &HomePos[0],&HomePos[1],&HomePos[2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &EncOS[0],&EncOS[1],&EncOS[2]);
		fscanf(KPdat,"%s %lf,%lf,%lf",DummyTxt, &InterTargDist[0][1],&InterTargDist[0][2],&InterTargDist[1][2]);
	}
	if( open )	// blt
		fclose(KPdat);
	
	for (i=0; i<3; i++)
	{
		EncConst[i]=EncConst[i]*3600.0*180.0/(4*atan(1.0));
		TRACE("EncConst %d = %f -- EncOS %d = %f\n", i, EncConst[i], i, EncOS[i]);
	}

	alpha[0][1]=alpha[0][1]*deg2rad;
	alpha[1][2]=alpha[1][2]*deg2rad;
}


void VDCPosition::VDCLoadCnts(char *szMessage, char *Enc1CntsTxt, char *Enc2CntsTxt, char *Enc3CntsTxt)
{

	/*******************
	This VDCPosition class member function receives a text string originally
	sent by the Galil motion controller that contains the total number of 
	encoder counts read by the Galil motion controller. This functnion parses
	the message, then assigns the constituent data contained in the message to 
	the appropriate variables. 

	It returns text values for the encouder count text display boxes in the 
	user interface.
	*******************/

	char seps[]   = ",: ";

	sprintf(Enc1CntsTxt,"%s",strtok( szMessage, seps ));
	sprintf(Enc2CntsTxt,"%s",strtok( NULL, seps ));
	sprintf(Enc3CntsTxt,"%s",strtok( NULL, seps ));
	
	EncCnts[0] = atol(Enc1CntsTxt); 
	EncCnts[1] = atol(Enc2CntsTxt);
	EncCnts[2] = atol(Enc3CntsTxt);
}


void VDCPosition::VDCLoadOS(char *Enc1OSTxt,char *Enc2OSTxt,char *Enc3OSTxt)
{
	/*******************
	This VDCPosition class member function converts three text values read from the 
	GUI for the encoder offsets into double values and stores them in the ecnoder offset
	class member variables.
	
	It returns nothing.
	*******************/

	char *stopstring;

	EncOS[0]=strtod(Enc1OSTxt,&stopstring)*deg2rad;
	EncOS[1]=strtod(Enc2OSTxt,&stopstring)*deg2rad;
	EncOS[2]=strtod(Enc3OSTxt,&stopstring)*deg2rad;
}

void VDCPosition::VDCDispOS(char *Enc1OSTxt,char *Enc2OSTxt,char *Enc3OSTxt)
{
	/*******************
	This VDCPosition class member constructs text strings from double encoder offset
	values from the class member variables.

	It returns these values to the calling program to for display in the GUI.
	*******************/

	sprintf(Enc1OSTxt,"%11.6f",EncOS[0]*rad2deg);
	sprintf(Enc2OSTxt,"%11.6f",EncOS[1]*rad2deg);
	sprintf(Enc3OSTxt,"%11.6f",EncOS[2]*rad2deg);
}

void VDCPosition::VDCCalcAng(char *Theta1Txt, char *Theta2Txt, char *Theta3Txt)
{
	/*******************
	This VDCPosition class member function calculates and stores the joints' 
	angluar positions from the angular encoder offsets and the number of recorded
	encoders pulses.

	It returns three text strings to be displayed in the GUI.
	*******************/

	int j;

	for (j=0; j<3; j++)
	{	dTheta[j] = (double)EncCnts[j]/EncConst[j]; 
		Theta[j] = dTheta[j]+EncOS[j];
	}
	
	sprintf(Theta1Txt,"%11.6f",Theta[0]*rad2deg);
	sprintf(Theta2Txt,"%11.6f",Theta[1]*rad2deg);
	sprintf(Theta3Txt,"%11.6f",Theta[2]*rad2deg);
}

void VDCPosition::VDCCalcPos(char *XTxt, char *YTxt, char *ZTxt)
{
	/*******************
	This VDCPosition class member function calcualtes the Cartesian coordinates of the
	probe tip. It stores the values in the class member variables.
	
	It returns three text strings for display in the GUI.
	*******************/

	double s3,s4,s2,s1;

	s3 = ((cos(Theta[0])*cos(Theta[1])-sin(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*cos(Theta[2])+(-cos(Theta[0])*sin(Theta[1])-sin(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(Theta[2])*cos(alpha[1][2])+sin(Theta[0])*sin(alpha[0][1])*sin(Theta[2])*sin(alpha[1][2]))*Ptool3[0];
    s4 = (-(cos(Theta[0])*cos(Theta[1])-sin(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*sin(Theta[2])+(-cos(Theta[0])*sin(Theta[1])-sin(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*cos(Theta[2])*cos(alpha[1][2])+sin(Theta[0])*sin(alpha[0][1])*cos(Theta[2])*sin(alpha[1][2]))*Ptool3[1];
    s2 = s3+s4;
    s1 = s2+(-(-cos(Theta[0])*sin(Theta[1])-sin(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(alpha[1][2])+sin(Theta[0])*sin(alpha[0][1])*cos(alpha[1][2]))*Ptool3[2]+(cos(Theta[0])*cos(Theta[1])-sin(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*a[1][2];
    CartPos[0] = s1-(-cos(Theta[0])*sin(Theta[1])-sin(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(alpha[1][2])*S[2]+sin(Theta[0])*sin(alpha[0][1])*cos(alpha[1][2])*S[2]+cos(Theta[0])*a[0][1]+sin(Theta[0])*sin(alpha[0][1])*S[1];
	
	s3 = ((sin(Theta[0])*cos(Theta[1])+cos(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*cos(Theta[2])+(-sin(Theta[0])*sin(Theta[1])+cos(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(Theta[2])*cos(alpha[1][2])-cos(Theta[0])*sin(alpha[0][1])*sin(Theta[2])*sin(alpha[1][2]))*Ptool3[0];
	s4 = (-(sin(Theta[0])*cos(Theta[1])+cos(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*sin(Theta[2])+(-sin(Theta[0])*sin(Theta[1])+cos(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*cos(Theta[2])*cos(alpha[1][2])-cos(Theta[0])*sin(alpha[0][1])*cos(Theta[2])*sin(alpha[1][2]))*Ptool3[1];
    s2 = s3+s4;
    s1 = s2+(-(-sin(Theta[0])*sin(Theta[1])+cos(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(alpha[1][2])-cos(Theta[0])*sin(alpha[0][1])*cos(alpha[1][2]))*Ptool3[2]+(sin(Theta[0])*cos(Theta[1])+cos(Theta[0])*sin(Theta[1])*cos(alpha[0][1]))*a[1][2];
    CartPos[1] = s1-(-sin(Theta[0])*sin(Theta[1])+cos(Theta[0])*cos(Theta[1])*cos(alpha[0][1]))*sin(alpha[1][2])*S[2]-cos(Theta[0])*sin(alpha[0][1])*cos(alpha[1][2])*S[2]+sin(Theta[0])*a[0][1]-cos(Theta[0])*sin(alpha[0][1])*S[1];

	s1 = (sin(Theta[1])*sin(alpha[0][1])*cos(Theta[2])+cos(Theta[1])*sin(alpha[0][1])*sin(Theta[2])*cos(alpha[1][2])+cos(alpha[0][1])*sin(Theta[2])*sin(alpha[1][2]))*Ptool3[0]+(-sin(Theta[1])*sin(alpha[0][1])*sin(Theta[2])+cos(Theta[1])*sin(alpha[0][1])*cos(Theta[2])*cos(alpha[1][2])+cos(alpha[0][1])*cos(Theta[2])*sin(alpha[1][2]))*Ptool3[1]+(-cos(Theta[1])*sin(alpha[0][1])*sin(alpha[1][2])+cos(alpha[0][1])*cos(alpha[1][2]))*Ptool3[2]+sin(Theta[1])*sin(alpha[0][1])*a[1][2];
    CartPos[2] = s1-cos(Theta[1])*sin(alpha[0][1])*sin(alpha[1][2])*S[2]+cos(alpha[0][1])*cos(alpha[1][2])*S[2]+cos(alpha[0][1])*S[1]+S[0];
	
	sprintf(XTxt,"%11.6f",CartPos[0]);
	sprintf(YTxt,"%11.6f",CartPos[1]);
	sprintf(ZTxt,"%11.6f",CartPos[2]);
  
}

void VDCPosition::VDCCalibRecAng(int PoseNum, int TargNum)
{
	/*******************
	This VDCPosition class member function stores the dTheta angular displacements
	collected during the initialization routine. The data is stored in a 3D matrix
	that will be passed to the offset calculculation routine.
	
	It returns nothing.
	*******************/

	int i;

	for (i=0; i<3; i++)
		CalibdThetaMat[PoseNum][i][TargNum]=dTheta[i];
}


int VDCPosition::VDCGetNumCalPose(void)
{
	/*******************
	This VDCPosition class member function returns the number of calibration 
	artifact poses.
	*******************/

		
	return(NumCalPose);
}

void VDCPosition::VDCGetPos(double Joint1, double Joint2, double Joint3, double X, double Y, double Z)
{
	/*******************
	This VDCPosition class member function returns angular positions of the three 
	angular encoders (in RADIANS) and the Cartesian position of the probe tip (IN MM).

	(TO FACILITATE INTERFACE TO SPATIAL ANALYZER SOFTWARE)
	*******************/

	X=CartPos[0];
	Y=CartPos[1];
	Z=CartPos[2];

	Joint1=Theta[0];
	Joint2=Theta[1];
	Joint3=Theta[2];
}
