#include "stdafx.h"
#include "nrkapp.h"
#include "GeomTriggerWrapper.h"

GeomTriggerWrapper::GeomTriggerWrapper(void)
: geomTrigger(*new GeometryTrigger)
{
}

GeomTriggerWrapper::~GeomTriggerWrapper(void)
{
	delete &geomTrigger;
}

bool GeomTriggerWrapper::HasTriggersDefined()
{
	return geomTrigger.HasTriggersDefined();
}

void GeomTriggerWrapper::AddSelectedPlane(SAIPGeomPlane& p)
{
	geomTrigger.AddSelectedPlane(p);
}
bool GeomTriggerWrapper::WantSelectedPlanes()
{
	return geomTrigger.WantSelectedPlanes();
}
void GeomTriggerWrapper::TurnOffPlaneSelection()
{
	geomTrigger.TurnOffPlaneSelection();
}

void GeomTriggerWrapper::AddSelectedCylinder(SAIPGeomCylinder& c)
{
	geomTrigger.AddSelectedCylinder(c, app.saarm.InchesToLocal);
}
bool GeomTriggerWrapper::WantSelectedCylinders()
{
	return geomTrigger.WantSelectedCylinders();
}
void GeomTriggerWrapper::TurnOffCylinderSelection()
{
	geomTrigger.TurnOffCylinderSelection();
}

void GeomTriggerWrapper::AddSelectedSphere(SAIPGeomSphere& s)
{
	geomTrigger.AddSelectedSphere(s, app.saarm.InchesToLocal);
}
bool GeomTriggerWrapper::WantSelectedSpheres()
{
	return geomTrigger.WantSelectedSpheres();
}
void GeomTriggerWrapper::TurnOffSphereSelection()
{
	geomTrigger.TurnOffSphereSelection();
}

bool GeomTriggerWrapper::EnsureValidity_InstScaleFactorHasNotChanged()
{
	if(geomTrigger.HasTriggersDefined())
	{
		TRACE2("inst scale in existing triggers = %f  current inst scale = %f\n", geomTrigger.GetInstScale(), app.saarm.GetInstrumentScaleFactor()*app.saarm.units.UnitToMeters[DISTUNITS_IN]);
		if(geomTrigger.GetInstScale() != app.saarm.GetInstrumentScaleFactor()*app.saarm.units.UnitToMeters[DISTUNITS_IN])
		{
			::MessageBox(NULL, Tr("Instrument Scale Factor has been changed since existing triggers were created.\nALL TRIGGERS WILL BE DELETED."),  Tr("Inst Scale Changed"), MB_ICONERROR | MB_SETFOREGROUND | MB_TOPMOST);
			geomTrigger.ClearAllTriggers();
			return false;
		}
	}
	return true;
}
void GeomTriggerWrapper::SetGeomTriggers()
{
	EnsureValidity_InstScaleFactorHasNotChanged();

	bool drawPlanesInSA = false;
	bool drawCylindersInSA = false;
	Transform IToW(app.saarm.InstToWorking);
	IToW.Scale(app.saarm.LocalToInches);
	geomTrigger.SetOperatingFrame(IToW);
	geomTrigger.SetInstScale(app.saarm.units.UnitToMeters[DISTUNITS_IN]*app.saarm.GetInstrumentScaleFactor());
	double inchesToUnit = app.saarm.InchesToLocal/app.saarm.units.UnitToMeters[app.saarm.units.distunits];
	geomTrigger.SetUnitsScaleAndName(inchesToUnit, app.saarm.units.DistUnitNames[app.saarm.units.distunits]);

	if(geomTrigger.SetTriggers(drawPlanesInSA, drawCylindersInSA))
	{
		if(drawPlanesInSA)
		{
			CStringArray notes;
			for(DWORD_PTR i=0; i<geomTrigger.triggers.GetSize(); i++)
			{
				if(geomTrigger.triggers[i].IsKindOf(RUNTIME_CLASS(TriggerPlane)))
				{
					TriggerPlane& tp = *(TriggerPlane*)&geomTrigger.triggers[i];
					//tp.ObjectToOperating.Debug();
					//tp.PerimPts.Debug();
					app.saarm.SockGeom.SendPlane(tp.ObjectToOperating, tp.PerimPts.P, tp.GetDescription(), notes);
				}
			}
		}
		if(drawCylindersInSA)
		{
			CStringArray notes;
			for(DWORD_PTR i=0; i<geomTrigger.triggers.GetSize(); i++)
			{
				if(geomTrigger.triggers[i].IsKindOf(RUNTIME_CLASS(TriggerCylinder)))
				{
					TriggerCylinder& tp = *(TriggerCylinder*)&geomTrigger.triggers[i];
					app.saarm.SockGeom.SendCylinder(tp.ptA, tp.ptB, tp.radius*2.0, tp.GetDescription(), notes);
				}
			}
		}
	}
}

bool GeomTriggerWrapper::Check(const Vector& previous, const Vector& current, const double& offset, VectorNamedArray& out)
{
	Vector prev(previous), curr(current);
	prev.Scale(1.0/app.saarm.units.UnitToMeters[DISTUNITS_IN]);
	curr.Scale(1.0/app.saarm.units.UnitToMeters[DISTUNITS_IN]);
	geomTrigger.Check(prev, curr, offset, out);
	out.Scale(app.saarm.units.UnitToMeters[DISTUNITS_IN]);
	return (out.GetSize() > 0) ? true : false;
}

/////////////////////////////////////////////////////////////////////
void GeomTriggerWrapper::Serialize(CArchive& ar)
{	
	BYTE Schema = 0;
	BYTE LoadSchema = 0;
	if (ar.IsStoring())
	{ 
		ar << Schema;
		geomTrigger.Serialize(ar);
	}
	else
	{
		ar >> LoadSchema;
		geomTrigger.Serialize(ar);
	}
} 
IMPLEMENT_SERIAL( GeomTriggerWrapper, CObject, 0 );