// ******************************************************************************
// BalloonHelp.h : header file
// Heiko Eichhorn

#ifndef _BALLOON_HELP_H_
#define _BALLOON_HELP_H_

//
#ifndef _M_IX86
	#pragma message("_ThunkImpl/ is implemented for X86 only!")
#endif

#pragma pack(push, 1)

template <class T>
class _ThunkImpl
{
private:

	BYTE	m_mov;			// mov ecx, %pThis
	DWORD	m_this; 		//
	BYTE	m_jmp;			// jmp func
	DWORD	m_relproc;		// relative jmp
protected:
	typedef void (T::*TMFP)();
	void InitThunk(TMFP method, const T* pThis)
	{
		union { DWORD func; TMFP method; } addr;
		addr.method = (TMFP)method;
		m_mov  = 0xB9;
		m_this = (DWORD)pThis;
		m_jmp  = 0xE9;
		m_relproc = addr.func - (DWORD)(this+1);
		::FlushInstructionCache(GetCurrentProcess(), this, sizeof(*this));
	}
	FARPROC GetThunk() const 
	{
		_ASSERTE(m_mov == 0xB9);
		return (FARPROC)this; 
	}
};
#pragma pack(pop)

template <class T>
class BHMouseHookThunk: public _ThunkImpl<T> {};

template <class T>
class BHKeybHookThunk: public _ThunkImpl<T> {};

template <class T>
class BHCallWndRetHookThunk: public _ThunkImpl<T> {};

class CBalloonHelp : public CWnd,
                     public BHKeybHookThunk<CBalloonHelp>,
                     public BHMouseHookThunk<CBalloonHelp>,
                     public BHCallWndRetHookThunk<CBalloonHelp>
{
public:
	CBalloonHelp();
	virtual ~CBalloonHelp();

	// options
	static const unsigned int unCLOSE_ON_LBUTTON_UP;   // closes window on WM_LBUTTON_UP
	static const unsigned int unCLOSE_ON_MBUTTON_UP;   // closes window on WM_MBUTTON_UP
	static const unsigned int unCLOSE_ON_RBUTTON_UP;   // closes window on WM_RBUTTON_UP
	static const unsigned int unCLOSE_ON_LBUTTON_DOWN; // closes window on WM_LBUTTON_DOWN
	static const unsigned int unCLOSE_ON_MBUTTON_DOWN; // closes window on WM_MBUTTON_DOWN
	static const unsigned int unCLOSE_ON_RBUTTON_DOWN; // closes window on WM_RBUTTON_DOWN
	static const unsigned int unCLOSE_ON_MOUSE_MOVE;   // closes window when user moves mouse past threshhold
	static const unsigned int unCLOSE_ON_KEYPRESS;     // closes window on the next keypress message sent to this thread.
	static const unsigned int unCLOSE_ON_ANYTHING;     // all of the above
	static const unsigned int unDELAY_CLOSE;           // when a user action triggers the close, begins timer.  closes when timer expires.
	static const unsigned int unDELETE_THIS_ON_CLOSE;  // deletes object when window is closed.  Used by LaunchBalloon(), use with care
	static const unsigned int unSHOW_CLOSE_BUTTON;     // shows close button in upper right
	static const unsigned int unSHOW_INNER_SHADOW;     // draw inner shadow in balloon
	static const unsigned int unSHOW_TOPMOST;          // place balloon above all other windows
	static const unsigned int unDISABLE_XP_SHADOW;     // disable Windows XP's drop-shadow effect (overrides system and user settings)
	static const unsigned int unDISABLE_FADEIN;        // disable the fade-in effect (overrides system and user settings)
	static const unsigned int unDISABLE_FADEOUT;       // disable the fade-out effect (overrides system and user settings)
	static const unsigned int unDISABLE_FADE;          // disable the fade-in/fade-out effects (overrides system and user settings)
public:
	BOOL Create(const CString& strTitle,
			   const CString& strContent,
			   const CPoint& ptAnchor,
			   unsigned int unOptions,
			   CWnd* pParentWnd = NULL,
			   const CString strURL = _T(""),       
			   unsigned int unTimeout = 0,      
			   HICON hIcon = NULL);             
	static void LaunchBalloon(const CString& strTitle, const CString& strContent, 
			   const CPoint& ptAnchor, 
			   LPCTSTR szIcon = IDI_EXCLAMATION,
			   unsigned int unOptions = unSHOW_CLOSE_BUTTON,
			   CWnd* pParentWnd = NULL, 
			   const CString strURL = _T(""),
			   unsigned int unTimeout = 10000);
	void SetTitleFont(CFont* pFont);
	void SetContentFont(CFont* pFont);
	void SetIcon(HICON hIcon);
	void SetIconScaled(HICON hIcon, int cx, int cy);
	void SetIcon(HBITMAP hBitmap, COLORREF crMask);
	void SetIcon(HBITMAP hBitmap, HBITMAP hMask);
	void SetIcon(CImageList* pImageList, int nIconIndex);
	void SetURL(const CString& strURL);
	void SetTimeout(unsigned int unTimeout);
	void SetMouseMoveTolerance(int nTolerance);
	void SetAnchorPoint(CPoint ptAnchor, CWnd* pWndAnchor = NULL);
	void SetTitle(const CString& strTitle);
	void SetContent(const CString& strContent);
	void SetForegroundColor(COLORREF crForeground);
	void SetBackgroundColor(COLORREF crBackground);
private:
	static const int nTIP_TAIL;
	static const int nTIP_MARGIN;
	enum BALLOON_QUADRANT { BQ_TOPRIGHT, BQ_TOPLEFT, BQ_BOTTOMRIGHT, BQ_BOTTOMLEFT };
	typedef BOOL (WINAPI* FN_ANIMATE_WINDOW)(HWND,DWORD,DWORD);
private:
	CPoint				GetAnchorPoint();
	void				GetAnchorScreenBounds(CRect& rect);
	BALLOON_QUADRANT	GetBalloonQuadrant();
	virtual void		DrawNonClientArea(CDC* pDC);
	virtual void		DrawClientArea(CDC* pDC);
	virtual CSize		DrawHeader(CDC* pDC, bool bDraw = TRUE);
	virtual CSize		DrawContent(CDC* pDC, int nTop, bool bDraw = TRUE);
	CSize				CalcHeaderSize(CDC* pDC) { return DrawHeader(pDC, FALSE); }
	CSize				CalcContentSize(CDC* pDC) { return DrawContent(pDC, 0, FALSE); }
	CSize				CalcWindowSize();
	CSize				CalcClientSize();
	void				PositionWindow();
	void				ShowBalloon(void);
	void				HideBalloon(void);
	static ATOM			GetClassAtom(BOOL bShadowed);
private:
	afx_msg void	OnShowWindow(BOOL bShow, UINT nStatus);
	afx_msg LRESULT OnPrint(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnPrintClient(WPARAM wParam, LPARAM lParam);
	afx_msg BOOL	OnEraseBkgnd(CDC* pDC);
	afx_msg void	OnPaint();
	afx_msg void	OnNcPaint();
	afx_msg void	OnLButtonDown(UINT, CPoint point);
	afx_msg void	OnLButtonUp(UINT, CPoint point);
	afx_msg void	OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS FAR* lpncsp);
	afx_msg void	OnTimer(UINT nIDEvent);
	afx_msg void	OnMouseMove(UINT, CPoint point);
	afx_msg UINT	OnNcHitTest(CPoint point);
	afx_msg void	OnClose();
	afx_msg void	OnDestroy();
	virtual void	PostNcDestroy();

	DECLARE_MESSAGE_MAP()
private:
	void	SetKeyboardHook();
	void	RemoveKeyboardHook();
	void	SetMouseHook();
	void	RemoveMouseHook();
	void	SetCallWndRetHook();
	void	RemoveCallWndRetHook();
	LRESULT KeyboardHookProc( int code, WPARAM wParam, LPARAM lParam);
	LRESULT MouseHookProc(int code, WPARAM wParam, LPARAM lParam);
	LRESULT CallWndRetProc(int code, WPARAM wParam, LPARAM lParam);

private:
   FN_ANIMATE_WINDOW	m_fnAnimateWindow;
   HHOOK				m_hKeyboardHook;
   HHOOK				m_hMouseHook;
   HHOOK				m_hCallWndRetHook;
   unsigned int			m_unOptions;
   unsigned int			m_unTimeout;     
   unsigned int			m_unTimerClose;   
   CString				m_strContent;
   CString				m_strURL;       
   HWND					m_hwndAnchor;  
   CPoint				m_ptAnchor;   
   CImageList			m_ilIcon;     
   CFont*				m_pTitleFont;  
   CFont*				m_pContentFont; 
   COLORREF				m_crBackground;  
   COLORREF				m_crForeground;   
   CRect				m_screenRect;     
   CRgn					m_rgnComplete;   
   CPoint				m_ptMouseOrig;    
   UINT					m_uCloseState;    
   int					m_nMouseMoveTolerance; 
   static ATOM			s_ClassAtom;
   static ATOM			s_ClassAtomShadowed;
};

#endif // _BALLOON_HELP_H_