#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"
#include "HexagonRDS.h"
#include "RDSWindowsMessenger.h"
#include "HexagonRDS/RDS_API.h"	// for enums

// Perceptron ScanWorks tcp/ip interface...
#include "ScanWorks_tcpip/tools.h"
#include "ScanWorks_tcpip/tcpip.h"
#include "DExternalScannerMeas.h"

HexagonRDS::HexagonRDS()
: wm(*new RDSWindowsMessenger(this))
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;
	lineScanning=false;
	armScannerSwitchOn = false;

	//app.saarm.Data.ButtonTwoMode = 3;	// backup/cancel, Romer defaults to 2, backup/mousetoggle

	if(app.saarm.arm_type == SAArm::arm_Hexagon_MultiGage)
	{
		NativeArmToSA.SetIdentity();

		app.saarm.CMMString = _T("Multi-Gage");
		app.saarm.SettingsFileName = _T("SA_Multi-Gage_ArmSettings.bin");
		LogFileName = _T("Multi-Gage.log");
	}
	else if(app.saarm.arm_type == SAArm::arm_Hexagon_Absolute_SI)
	{
		Vector rot(0,0,-90);
		NativeArmToSA.SetRotation(rot);

		app.saarm.CMMString = _T("Absolute SI/SE");
		app.saarm.SettingsFileName = _T("SA_Absolute_SI_SE_ArmSettings.bin");
		LogFileName = _T("Absolute_SI_SE.log");
	}
	else if(app.saarm.arm_type == SAArm::arm_Hexagon_Absolute)
	{
		NativeArmToSA.SetIdentity();

		app.saarm.CMMString = _T("Absolute");
		app.saarm.SettingsFileName = _T("SA_Absolute_ArmSettings.bin");
		LogFileName = _T("Absolute.log");
	}
	else
	{
		app.saarm.CMMString = _T("HexagonRDS");
		app.saarm.SettingsFileName = _T("SA_HexagonRDS_ArmSettings.bin");
		LogFileName = _T("HexagonRDS.log");
	}

	Vector rot(00.0, -90.0, 180.0);
	scannerXform.SetRotation(rot);
}

HexagonRDS::~HexagonRDS()
{
	delete &wm;
}

bool HexagonRDS::ConnectToArm(const int dof)
{
	if(!app.saarm.Data.doconnection)	// 2012.12.03 - Mantis#5697 - Starting Hexagon Absolute arm in simulation mode still tries to connect to Hardware 
		return true;	// simulation mode

	DOF = dof;

	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);

	if(!wm.windowValid)	// 2005.09.27 - added for scanner dlg
	{
		if(!wm.MakeValidWindow())
		{
			CString m;
			m.Format(Tr("Critical Error.\nCould not start RDS Message Window.\n\nPlease try again."));
			AfxMessageBox(m, MB_ICONHAND);
			return false;
		}
		else	// wait till our message handler gets the connected event...
		{
			DWORD timeOut = 5000;	// only wait 5 sec.
			DWORD Start = ::GetCurrentTime();
			while((::GetCurrentTime() - Start) < timeOut)
			{
				if(ConnectedToArm)
					break;
				Delay(0.01, TRUE);
			}
		}
	}

	if(!ConnectedToArm)
	{
		if(wm.IsConfiguredForTracker())
		{
			CString msg;
			msg.Format(Tr("RDS is configured for the tracker.\n\nOpen RDS Control Panel and set the Connection for the Arm and try again."));
			::MessageBox(NULL, msg,  Tr("RDS Connection"), MB_ICONERROR | MB_SETFOREGROUND | MB_TOPMOST);
			return false;
		}
		CString Msg;
		Msg.Format(Tr("Hexagon ROMER Connection failed."));
		int ans = AfxMessageBox(Msg + Tr("\n\nRun in Simulation mode?"),MB_YESNO);
		if(ans==IDNO)
		{
			return false;
		}
		else
			return true;	// simulation mode
	}
	return ConnectedToArm ? true : false;
}
void HexagonRDS::DoCal()
{
	if(!ConnectedToArm)
	{
		ArmBase::DoCal();
		return;
	}

	wm.RunProbeCal();
}
void HexagonRDS::DoArmSettings()
{
	wm.DoArmSettings();
}

int HexagonRDS::GetButtonCount()
{
	//if(app.saarm.arm_type == SAArm::arm_Hexagon_MultiGage)
		return 3;

	return 3;
}

void HexagonRDS::OutputSpeakerSound( int frequency, int duration )
{
	wm.OutputSpeakerSound(frequency, duration);
}
void HexagonRDS::ArmBeep(int sound)
{
	wm.ArmBeep(sound);
}

BOOL HexagonRDS::IsRemoteMouseEnabled()
{
	return wm.IsRemoteMouseEnabled();
}
void HexagonRDS::ToggleRemoteMouse()
{
	wm.ToggleRemoteMouse();
}
void HexagonRDS::RemoteMouse(BOOL flag)
{
	wm.RemoteMouse(flag);
}

BOOL HexagonRDS::DisconnectFromHardware()
{
	return TRUE;
}

void HexagonRDS::DoPostInitOperations()
{
	wm.OnPostInit();
}

BOOL HexagonRDS::GetAllData()
{
	if(!overrideArmButtons && app.pA->armButtons.ButtonsEnabled()) // 2019.03.15 - took out !lineScanning here - allowing side buttons to e.g. operate Next Feature during line scanning
	//if(!lineScanning && !overrideArmButtons && app.pA->armButtons.ButtonsEnabled())
		GetButtonStates();
	return TRUE;
}
BOOL HexagonRDS::GetPos()
{

	return TRUE;
}
BOOL HexagonRDS::GetJoints(bool simulate/*=true*/)
{

	return TRUE;
}
int HexagonRDS::GetButtonStates()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonStates();

	int duration=0;
	buttonstate = 0;

	if( CheckButton(1, &duration) )
	{
		buttonstate = 1;
		buttonstate += duration;
	}
	else if( !lineScanning && CheckButton(2, &duration) ) // 2019.03.15 - added !lineScanning here - allowing side buttons to e.g. operate Next Feature during line scanning, but have to allow RDS to keep control of the trigger button during scanning
	{
		buttonstate = 2;
		buttonstate += duration;
	}
	else if( CheckButton(3, &duration) )
	{
		buttonstate = 4;
		buttonstate += duration;
	}	
	return buttonstate;
}

BOOL HexagonRDS::CheckButton(int but, int* duration)
{
	if( !ConnectedToArm )
		return ArmBase::CheckButton(but, duration);
	if(but == 1)
	{
		if(wm.m_bButton1State)
		{
			TRACE1("Button %d down\n",but);
			DWORD Start = ::GetCurrentTime();
			while((wm.m_bButton1State)&&((::GetCurrentTime() - Start) < LONG_DURATION))
			{
				Delay(0.01, TRUE);
			}
			if(wm.m_bButton1State)
			{
				*duration = BUTTON_LONG;
				OutputSpeakerSound(3000,100);
				while(wm.m_bButton1State)
				{
					Delay(0.01, TRUE);
				}
				return TRUE;
			}
			else if(!IsRemoteMouseEnabled())
				*duration = BUTTON_SHORT;
		}
		else
			return FALSE;
		return TRUE;
	}
	if(but == 2)
	{
		if(wm.m_bButton2State)
		{
			TRACE1("Button %d down\n",but);
			DWORD Start = ::GetCurrentTime();
			while((wm.m_bButton2State)&&((::GetCurrentTime() - Start) < LONG_DURATION))
			{
				Delay(0.01, TRUE);
			}
			if(wm.m_bButton2State)
			{
				*duration = BUTTON_LONG;
				OutputSpeakerSound(3000,100);
				while(wm.m_bButton2State)
				{
					Delay(0.01, TRUE);
				}
				return TRUE;
			}
			else if(!IsRemoteMouseEnabled())
				*duration = BUTTON_SHORT;
		}
		else
			return FALSE;
		return TRUE;
	}
	if(but == 3)
	{
		if(wm.m_bButton3State)
		{
			TRACE1("Button %d down\n",but);
			DWORD Start = ::GetCurrentTime();
			while((wm.m_bButton3State)&&((::GetCurrentTime() - Start) < LONG_DURATION))
			{
				Delay(0.01, TRUE);
			}
			if(wm.m_bButton3State)
			{
				*duration = BUTTON_LONG;
				OutputSpeakerSound(3000,100);
				while(wm.m_bButton3State)
				{
					Delay(0.01, TRUE);
				}
				return TRUE;
			}
			else if(!IsRemoteMouseEnabled())
				*duration = BUTTON_SHORT;
		}
		else
			return FALSE;
		return TRUE;
	}
}

void HexagonRDS::OnTouchTriggerPressed() // 2015.01.28 - for Ute Burton @ Hexagon
{
	MeasToSpan();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SI or SE with CMS scanner...
BOOL HexagonRDS::ScannerSettings()
{
	wm.DoArmSettings();
	return TRUE;
}

BOOL HexagonRDS::StartScannerMeas()
{
	if(wm.GetProbeType() == PT_EXTERNAL_TRIGGER)
		return StartPerceptronScannerMeas();

	if(!ConnectedToArm)
		return ArmBase::StartScannerMeas();

	if(!armScannerSwitchOn)
	{
		MessageBox(NULL,
			Tr("Arm Scanner switch is OFF.\nJust use the arm's scanner switch to turn on the line scanner."),
			Tr("Scanner"),
			MB_ICONHAND | MB_SETFOREGROUND | MB_TOPMOST);
		return FALSE;
	}

	if(!app.saarm.scannerLicenseValid)
	{
		MessageBox(NULL,
			Tr("No Scanner License found.  Please contact your SA sales representative."),
			Tr("Scanner"),
			MB_ICONHAND | MB_SETFOREGROUND | MB_TOPMOST);
		return FALSE;
	}
	if(lineScanning)
		return TRUE;

	app.saarm.EndMeasurement();
	app.saarm.SendSAIRCAttributesToSA();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();

	Delay(0.5,TRUE);
	lineScanning = true;
	d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

	return TRUE;
}
BOOL HexagonRDS::StopScannerMeas()
{
	if(!lineScanning)
		return FALSE;
		
	if(wm.GetProbeType() == PT_EXTERNAL_TRIGGER)
		return StopPerceptronScannerMeas();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	d.StopScannerTimer();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);

	lineScanning = false;

	// return mouse mode to previous state 
	//RemoteMouse(mouseModeOn);

	CheckAndSendScannerCloud(0, true);
	return TRUE;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// for scanner with tcp/ip to ScanWorks...
SOCKET		hSWSocket;
CWinThread* pSWThread;
void ScanWorksListener(LPVOID ptr);

//BOOL HexagonRDS::ScannerSettings()
//{
//	TRACE0("HexagonRDS::ScannerSettings...\n");
//	if(!ConnectedToArm)
//		return ArmBase::ScannerSettings();
//
//	return TRUE;
//}
BOOL HexagonRDS::StartPerceptronScannerMeas()
{
	TRACE0("HexagonRDS::StartPerceptronScannerMeas...");

	if(!armScannerSwitchOn)
	{
		MessageBox(NULL,
			Tr("Arm Scanner switch is OFF.\nJust use the arm's scanner switch to turn on the line scanner."),
			Tr("Scanner"),
			MB_ICONHAND | MB_SETFOREGROUND | MB_TOPMOST);
		return FALSE;
	}

	if(!app.saarm.scannerLicenseValid)
	{
		MessageBox(NULL,
			Tr("No Scanner License found.  Please contact your SA sales representative."),
			Tr("Scanner"),
			MB_ICONHAND | MB_SETFOREGROUND | MB_TOPMOST);
		return FALSE;
	}
	if(lineScanning)
		return TRUE;

	if (trackScanDlg.IsAlreadyPopped())
	{
		//TRACE0("trackScanDlg.IsAlreadyPopped() = TRUE, returning...\n");
		return TRUE;
	}

	if(!ConnectedToArm)
		return ArmBase::StartScannerMeas();

	//////////////////////////////////////////////////////////////////////////////////////////////////
	// 2008.07.21
	// Ralph Stephan @ ScanWorks says: "We are moving away from using only probe number 8 and this is 
	// probably the safest way to detect if the scanner is active.  So, we'll verify the scanner once
	// ScanWorks has control - see ScanWorksCheckForProbe().

	// make sure user does not try to scan when scanner is not connected...
	//probeValid = RDS_ValidateCurrentProbe() ? true : false;
	//int probenum = RDS_GetProbeNumber();
	//if(probenum != 8)	// the scanner probe number is ALWAYS 8 according to Mike Hunwick @ Perceptron
	//{
	//	NRKProgress noScanner;
	//	noScanner.Start(_T("No Scanner Connected (Probe #8)"));
	//	noScanner.SetMsg(_T("You must connect the scanner to the arm."));
	//	RDS_OutputSpeakerSound(3000,250);
	//	Delay(0.1,TRUE);
	//	RDS_OutputSpeakerSound(2500,250);
	//	Delay(2.0,TRUE);
	//	noScanner.Kill();
	//	return FALSE;
	//}
	//////////////////////////////////////////////////////////////////////////////////////////////////


	// 2007.10.29 - reg key no longer used as of ScanWorks v5
	//if(!scannerRegKeySet)
	//{
	//	if(!SetScannerRegKey(true))
	//		return FALSE;
	//}

	app.saarm.EndMeasurement();
	app.saarm.SendSAIRCAttributesToSA();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.1,TRUE);
	wm.OutputSpeakerSound(1400,120);
	//DisconnectFromHardware();

	d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

	DExternalScannerMeas* pnewdlg = new DExternalScannerMeas((ArmBase&)*this, trackScanDlg);
	DOMODELESS(pnewdlg); // OR the normal way:  pnewdlg->Create(pnewdlg->IDD,NULL);

	if(!pSWThread)
	{
		TRACE0("call begin thread...\n");
		LPVOID sptr = (LPVOID)this;
		pSWThread = AfxBeginThread((AFX_THREADPROC)ScanWorksListener, sptr, THREAD_PRIORITY_NORMAL, 0, 0, NULL);
		Sleep(250);
	}
	if(!hSWSocket || hSWSocket == INVALID_SOCKET)
	{
		TRACE0("Call TCPIPConnect...\n");
		if (!TCPIPConnect(&hSWSocket, _T("localhost"), COMM_PORT))
		{
			CString m;
			m.Format(Tr("Unable to connect tcp/ip.  Is ScanWorks running??\n\nBe sure to set 'Start ScanWorks on Console Startup' in Console Options if running ScanWorks v.5."));
			AfxMessageBox(m, MB_OK|MB_ICONERROR);
			trackScanDlg.CloseDialog();
			Sleep(100);
			d.StopScannerTimer();
			Sleep(100);
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		Sleep(250);
		// 2005.10.03 - added support for ScanWorks v. 4.4 - it allows setting ScanWorks units...
		SetScanWorksUnitsToInches();
	}
	if(pSWThread && hSWSocket != INVALID_SOCKET)
	{
		Sleep(200);
		if(!ScanWorksMoverConnect())
		{
			trackScanDlg.CloseDialog();
			Sleep(100);
			d.StopScannerTimer();
			Sleep(100);
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		if(!ScanWorksCheckForProbe())	// 2008.07.21 - insure user has the scanner active
		{
			ScanWorksMoverDisconnect();
			Sleep(100);
			trackScanDlg.CloseDialog();
			d.StopScannerTimer();
			Sleep(100);
			d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
			return FALSE;
		}
		Sleep(2000);
		TRACE0("Call StartModel...\n");
		StartModel();
		lineScanning = true;
		return TRUE;
	}
	trackScanDlg.CloseDialog();
	AfxMessageBox(_T("Could not begin ScanWorks listening thread."), MB_ICONERROR);
	return FALSE;
}
BOOL HexagonRDS::StopPerceptronScannerMeas()
{
	TRACE0("HexagonRDS::StopPerceptronScannerMeas - stop scan");
	if(!lineScanning)
		return TRUE;

	int		command;
	long	Written;
	command = SCANWORKS_STOP_SCAN;
	if(!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
		return false;
	}
	TRACE0("HexagonRDS::StopPerceptronScannerMeas - stop model");
	StopModel();
	//AfxMessageBox(_T("model stopped?"));
	CString statmsg;
	statmsg.Format(_T("Model Stopped"));
	PostUpdateScanDlg(statmsg);
	HideScanWorksToolBar();
	Sleep(200);
	ScanWorksMoverDisconnect();
	//AfxMessageBox(_T("mover diconnected?"));
	statmsg.Format(_T("Disconnected from Mover"));
	PostUpdateScanDlg(statmsg);
	Sleep(200);
	TRACE0("HexagonRDS::StopPerceptronScannerMeas - close socket");
	TCPIPClose(hSWSocket);
	hSWSocket = INVALID_SOCKET;
	//AfxMessageBox(_T("tcpip closed?"));
	Delay(0.5,TRUE);
	lineScanning = false;
	return SendBufferedCloudDataAndReconnect();
}
bool HexagonRDS::SendBufferedCloudDataAndReconnect()
{
	TRACE0("CimCoreArm::SendBufferedCloudDataAndReconnect...");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	CheckAndSendScannerCloud(0, true);

	Sleep(100);

	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return true;
}
void HexagonRDS::PostUpdateScanDlg(CString msg)
{
	if( !ConnectedToArm )
		return ArmBase::PostUpdateScanDlg(msg);
	TRACE1("Post Message to update scan dlg with text '%s'\n", msg);
	scanMsg = msg;
	wm.PostMessage(UPDATE_SCAN_DLG, 0, 0);
}
void HexagonRDS::OnUpdateScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	if(((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.GetCount() > 100)
		((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.ResetContent();
	int idx = ((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.AddString(scanMsg);
	((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.SetTopIndex(idx);
}
void HexagonRDS::CloseScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	TRACE0("Post Message to close scan dlg...\n");
	wm.PostMessage(CLOSE_SCAN_DLG);
}

///////////////////////////////////////////////////////////////////
// Perceptron specific
// 2007.10.29 - DelayConnect key no longer used as of SW v5
bool HexagonRDS::SetScannerRegKey(bool reportError/*=false*/)
{
	// create/open the key...
	HKEY hk; 
	DWORD dwDisp = 0; 
	if( RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Perceptron\\GSE\\Settings"), 
		0, NULL, REG_OPTION_NON_VOLATILE,KEY_WRITE, NULL, &hk, &dwDisp) ) 
	{
		if(reportError)
		{
			CString temp;
			StringHelper::GetLastErrorString(temp);
			AfxMessageBox(_T("Could not create/open Perceptron registry key.\n")+temp, MB_OK|MB_ICONERROR);
		}
		return false;
	}

	// set the key value...
	DWORD delayConnect = 1;
	// if set to 0, scanworks will act just like it does now
	// if set to 1, scanworks will NOT connect to the arm at startup unless TCPIP is disabled. 
	// when you check the TCPIP box, it will connect to the arm - if you uncheck the box it will disconnect from the arm. it will also connect/disconnect to the arm as you start/stop the model.
	bool ret = true;
	if( RegSetValueEx(hk, _T("DelayConnect"), 0, REG_DWORD, (LPBYTE) &delayConnect, sizeof(DWORD)) )
	{
		if(reportError)
		{
			CString temp;
			StringHelper::GetLastErrorString(temp);
			AfxMessageBox(_T("Could not Set Perceptron registry key.\n")+temp, MB_OK|MB_ICONERROR);
		}
		ret = false;
	}
	RegCloseKey(hk); 
	return ret;
}
void HexagonRDS::ResetSocket() 

{
	if (hSWSocket == INVALID_SOCKET)
	{
		return;
	}
	TCPIPClose(hSWSocket);
	hSWSocket = INVALID_SOCKET;
}
bool HexagonRDS::ScanWorksMoverConnect()
{
	int		command;
	long	Written;

	command = SCANWORKS_MOVER_CONNECT;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(Tr("Unable to connect ScanWorks to mover.  Is ScanWorks running??\n\nBe sure to set 'Start ScanWorks on Console Startup' in Console Options if running ScanWorks v.5."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
bool HexagonRDS::ScanWorksMoverDisconnect()
{
	int		command;
	long	Written;

	command = SCANWORKS_MOVER_DISCONNECT;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("unable to disconnect ScanWorks from mover"), MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
void HexagonRDS::ShowScanWorksToolBar()
{
	int		command;
	long	Written;

	command = SCANWORKS_SHOW_TOOLBAR;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to show ScanWorks toolbar"), MB_OK|MB_ICONERROR);
		return;
	}

}
void HexagonRDS::HideScanWorksToolBar()
{
	int		command;
	long	Written;

	command = SCANWORKS_HIDE_TOOLBAR;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to hide ScanWorks toolbar"), MB_OK|MB_ICONERROR);
		return;
	}
}

bool HexagonRDS::SetScanWorksUnitsToInches()
{
	int		command;
	long	Written;

	command = SCANWORKS_SET_UNITS;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(_T("Unable to set ScanWorks units.\nEither upgrade to ScanWorks v. 4.4 or later,\nor set units to Inches in ScanWorks"));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}

	stSWUnits unit;
	unit.state = 0;	// 1 is mm

	if (!TCPIPWrite(hSWSocket, (LPVOID) &unit, (long) sizeof(unit), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to set ScanWorks units.\nEnsure units are set to Inches in ScanWorks."), MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
bool HexagonRDS::ScanWorksCheckForProbe()
{
	int command;
	long Written;
	long nNumberRead;

	command = SCANWORKS_GET_PROBE;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(_T("Unable to check for ScanWorks probe settings."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);

		return false;
	}

	stSWProbeSettings settings;
	TCPIPRead(hSWSocket, &settings, sizeof(settings), &nNumberRead);
	if (nNumberRead < sizeof(settings))
	{
		CString m;
		m.Format(_T("Unable to get ScanWorks probe settings."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}

	if (settings.type == TOUCH_PROBE_TYPE)
	{
		CString m;
		m.Format(_T("ScanWorks: Touch Probe is active or Sensor Power is Off.\nNeed to activate the Scanner to run ScanWorks."));
		AfxMessageBox(m, MB_OK|MB_ICONINFORMATION);
		return false;
	}
	else if(settings.type == SCAN_PROBE_TYPE)
	{
		return true;
	}
	return false;
}
bool HexagonRDS::StartModel()
{
	int		command;
	long	Written;

	command = SCANWORKS_MODEL_START;
	if (!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("ScanWorks unable to start model."), MB_OK|MB_ICONERROR);
		trackScanDlg.CloseDialog();
		return false;
	}
	return true;
}
bool HexagonRDS::StopModel()
{
	int		command;
	long	Written;

	command = SCANWORKS_MODEL_STOP;
	if(!TCPIPWrite(hSWSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
		return false;
	}
	return true;
}
void HexagonRDS::CleanUpPerceptron()
{
	if(pSWThread)
		delete pSWThread;
}
// the ScanWorks tcp/ip thread function...
void ScanWorksListener(LPVOID ptr)
{
	TRACE0("ScanWorksListener thread started.\n");
	HexagonRDS* pParent = (HexagonRDS*)ptr;

	stSWPartConnect			Part;
	//stSWTouchprobeConnect	Touchprobe;
	stSWPhase				Phase;
	stSWScanRegion			Region;
	stSWScanLine			Line;
	//stSWGridHeader			Grid;

	stSWPartConnectHS		PartHS;
	//stSWTouchprobeConnectHS	TouchprobeHS;
	stSWPhaseHS				PhaseHS;
	stSWScanRegionHS		RegionHS;

	int				nTotalXYZs = 0;
	int				nTotalReadXYZs = 0;
	int				nTotalLines = 0;
	int				nXYZs = 0;
	float			szScanData[12000][3];
	int				nData;
	sockaddr_in		sin;
	bool			loop=true;
	//char			lpszBuf[256];


	bool m_bHandshakes = false;
	bool m_bTagdata = true;

	while(1)
	{
		SOCKET		hListenSocket;

		if (!TCPIPInitialize())	// 2005.10.12 - initiate use of WS2_32.DLL for ScanWorks connection - only once now (See ConnectToArm)
		{
			TRACE0("TCPIPInitialize failed.\n");
			return;
		}

		hListenSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (hListenSocket == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Cannot create socket to ScanWorks. ") + err);
			TRACE1("Cannot create socket to ScanWorks. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		int i = TRUE;	// 2005.10.12 - set to reuse address in case user would like to scan more than once during the life of this app!
		i = setsockopt(hListenSocket, SOL_SOCKET, SO_REUSEADDR, (LPSTR)&i, sizeof(i));
		if (i == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Failed to set sock option - SO_REUSEADDR ") + err);
			TRACE1("Failed to set sock option - SO_REUSEADDR. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		memset(&sin, 0, sizeof(sin));
		sin.sin_family = AF_INET;
		sin.sin_port   = htons(DATA_PORT);
		if (bind(hListenSocket, (sockaddr*)&sin, sizeof(sin)) == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Could not bind to ScanWorks port. ") + err);
			TRACE1("Cannot bind to ScanWorks port. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		bool connected = true;
		//pParent->PostUpdateScanDlg(_T("Start sock set thread loop"));
		while (1)
		{
			TRACE("Top of while loop\n");
			//Beep(900,100);
			if (listen(hListenSocket, SOMAXCONN) == SOCKET_ERROR)	// 2005.10.13 - set backlog of pending connections to the max value
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("ScanWorks Could not listen. ") + err);
				TRACE1("ScanWorks Could not listen for tcp/ip packets. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			sockaddr_in		sinfrom;
			int				nLength = sizeof(sinfrom);

			TRACE("1\n");

			SOCKET	hSock;
			hSock = accept(hListenSocket, (sockaddr*)&sinfrom, &nLength);
			if (hSock == INVALID_SOCKET )
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Cannot create linkage socket to ScanWorks. ") + err);
				TRACE1("Cannot create linkage socket to ScanWorks. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			TRACE("1a\n");

			//int ret = TRUE;	// 2005.10.12 - set to reuse address in case user would like to scan more than once during the life of this app!
			//ret = setsockopt(hSock, SOL_SOCKET, SO_REUSEADDR, (LPSTR)&ret, sizeof(ret));
			//if (ret == SOCKET_ERROR)
			//{
			//	CString err;
			//	err.Format(_T("[%d]"), WSAGetLastError());
			//	pParent->PostUpdateScanDlg(_T("Failed to set sock option for linkage socket - SO_REUSEADDR ") + err);
			//	TRACE1("Failed to set sock option for linkage socket - SO_REUSEADDR. [%d]\n", WSAGetLastError());
			//	Delay(5.0);
			//	pParent->DoEndScanOperations();
			//	return;
			//}

			TRACE("2\n");

			if (hSock == INVALID_SOCKET)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("ScanWorks Failed to accept socket connection. ") + err);
				TRACE1("scanworks Failed to accept. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				TCPIPCleanup();
				AfxEndThread(0);
			}

			DWORD dwAddr = ntohl(sinfrom.sin_addr.s_addr);
			int   i      = 0;

			i = 8 * 1024;
			i = setsockopt(hSock, SOL_SOCKET, SO_SNDBUF, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - SendBuffer ") + err);
				TRACE0("scanworks Failed to set sock options.\n");
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			i = 8 * 1024;
			i = setsockopt(hSock, SOL_SOCKET, SO_RCVBUF, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - ReceiveBuffer ") + err);
				TRACE0("scanworks Failed to set sock options.\n");
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			i = TRUE;
			i = setsockopt(hSock, IPPROTO_TCP, TCP_NODELAY, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - TCP_NODELAY ") + err);
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			ULONG j = 0; // Zero means block
			if (ioctlsocket(hSock, FIONBIO, &j) == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - Blocking ") + err);
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			SAIPCloudBlockPt cbpt;
			Vector xyzCimCore, xyzArm, xyzSA;

			TRACE("Bottom of while loop\n");
			//Beep(1000,100);
			loop = true;
			while (loop)
			{
				int nRead = recv(hSock, (char*)&nData, sizeof(nData), 0);

				if (nRead == SOCKET_ERROR)
				{
					pParent->PostUpdateScanDlg(_T("Error reading data"));
					break;
				}

				long nNumberRead=0;
				long nNumberWritten=0;
				int command;
				CString str;

				switch (nData)
				{
				case (SCANWORKS_PART_CONNECT): 
					TCPIPRead(hSock, &Part, sizeof(stSWPartConnect), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("ScanWorks Part Connected"));
					TRACE0("Part connect...\n");

					command = SCANWORKS_PART_CONNECT_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PartHS.model_number = Part.model_number;
					strcpy(PartHS.model_name, Part.model_name);
					PartHS.serial_number = Part.serial_number;
					PartHS.status = ALL_OK;
					TCPIPWrite(hSock, &PartHS, sizeof(stSWPartConnectHS), &nNumberWritten);

					break;


				case (SCANWORKS_PART_DISCONNECT):
					TCPIPRead(hSock, &Part, sizeof(stSWPartConnect), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("ScanWorks Closing, Please Wait..."));
					TRACE0("Part disconnect.\n");

					command = SCANWORKS_PART_DISCONNECT_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PartHS.model_number = Part.model_number;
					strcpy(PartHS.model_name, Part.model_name);
					PartHS.serial_number = Part.serial_number;
					PartHS.status = ALL_OK;
					TCPIPWrite(hSock, &PartHS, sizeof(stSWPartConnectHS), &nNumberWritten);

					loop = false;
					break;


				case (SCANWORKS_PHASE_START):  
					TCPIPRead(hSock, &Phase, sizeof(stSWPhase), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("ScanWorks Connected..."));
					TRACE0("Phase start.\n");

					command = SCANWORKS_PHASE_START_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PhaseHS.phase_number = Phase.phase_number;
					strcpy(PhaseHS.phase_name, Phase.phase_name);
					PhaseHS.status = ALL_OK;
					TCPIPWrite(hSock, &PhaseHS, sizeof(stSWPhaseHS), &nNumberWritten);

					break;


				case (SCANWORKS_PHASE_STOP):
					TCPIPRead(hSock, &Phase, sizeof(stSWPhase), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("ScanWorks Phase Stop"));
					TRACE0("Phase stop.\n");

					command = SCANWORKS_PHASE_STOP_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PhaseHS.phase_number = Phase.phase_number;
					strcpy(PhaseHS.phase_name, Phase.phase_name);
					PhaseHS.status = ALL_OK;
					TCPIPWrite(hSock, &PhaseHS, sizeof(stSWPhaseHS), &nNumberWritten);

					break;


				case (SCANWORKS_SCAN_REGION_START): 
					TCPIPRead(hSock, &Region, sizeof(stSWScanRegion), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("Scan Start"));

					command = SCANWORKS_SCAN_REGION_START_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					RegionHS.scan_number = Region.scan_number;
					RegionHS.process_switch = Region.process_switch;
					strcpy(RegionHS.scan_name, Region.scan_name);
					RegionHS.status = ALL_OK;
					TCPIPWrite(hSock, &RegionHS, sizeof(stSWScanRegionHS), &nNumberWritten);

					break;


				case (SCANWORKS_SCAN_REGION_STOP):
					TCPIPRead(hSock, &Region, sizeof(stSWScanRegion), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("Scan Stop"));

					command = SCANWORKS_SCAN_REGION_STOP_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					RegionHS.scan_number = Region.scan_number;
					RegionHS.process_switch = Region.process_switch;
					strcpy(RegionHS.scan_name, Region.scan_name);
					RegionHS.status = ALL_OK;
					TCPIPWrite(hSock, &RegionHS, sizeof(stSWScanRegionHS), &nNumberWritten);

					nTotalXYZs =0;
					nTotalReadXYZs =0;

					break;

				case (SCANWORKS_SCAN_LINE):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);

					//str.Format(_T("lines: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					//TRACE1("%s\n", str);
					//str.Format(_T("lines: %d"), nTotalLines++);
					//pDlg->SetDlgItemText(IDC_TOTALLINES, str);
					TRACE1("%s\n", str);
					if (m_bHandshakes)
					{
						command = SCANWORKS_SCAN_LINE_HS;
						TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
					}
					break;

				case (SCANWORKS_CURSOR_SCANLINE):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);
					//str.Format(_T("lines: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					break;

				case (SCANWORKS_SCAN_DATA):
					if (m_bTagdata)
					{
						TCPIPRead(hSock, szScanData, sizeof(float)*(Line.num_xyzs+1)*3, &nNumberRead);
						nTotalXYZs += Line.num_xyzs;
						nTotalReadXYZs += nNumberRead/3/(sizeof(float));
						//str.Format(_T("points: %d (%d) (#%d)"), nTotalXYZs, nTotalReadXYZs, (int) szScanData[Line.num_xyzs][0]);
						//pDlg->SetDlgItemText(IDC_POINTS, str);
						//TRACE1("%s\n", str);
						nXYZs += nTotalXYZs;
						//str.Format(_T("points: %d"), nXYZs);
						//pDlg->SetDlgItemText(IDC_TOTALPOINTS, str);
						//TRACE1("%s\n", str);
						for(int j=0; j<Line.num_xyzs-20; j++)	// 2005.10.04 - Yes, the -20 is deliberate.  These last points in the line tend to be bogus.
						{
							if(j % app.saarm.Data.thinningFactor != 0)	// 2006.03.02
								continue;
							if(j >= 12000)
								continue;

							try
							{
								xyzCimCore.x = szScanData[j][1];
								xyzCimCore.y = szScanData[j][2];
								xyzCimCore.z = szScanData[j][3];
							}
							catch(...)
							{
								continue;
							}

							//xyzCimCore.Scale(1.0/25.4);	2005.10.03 - we can set units to inches in ScanWorks as of ScanWorks v. 4.4
							pParent->NativeArmToSA.Multiply( xyzCimCore, &xyzArm );
							pParent->scannerXform.Multiply(xyzArm, &xyzSA);
							cbpt.x = xyzSA.x;
							cbpt.y = xyzSA.y;
							cbpt.z = xyzSA.z;

							pParent->AddScannerCloudPt(cbpt);
						}
						TRACE3("Scan Line XYZ: x=%f, y=%f, z=%f\n",
							Line.machine_X,
							Line.machine_Y,
							Line.machine_Z);
						TRACE3("Scan Line Roll, Pitch, Yaw: r=%f, p=%f, y=%f\n",
							Line.machine_Roll,
							Line.machine_Pitch,
							Line.machine_Yaw);
						//pParent->CheckAndSendScannerCloud(2000);
						if (m_bHandshakes)
						{
							command = SCANWORKS_SCAN_DATA_HS;
							TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
						}
					}
					else
					{
						TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
						nTotalXYZs += Line.num_xyzs;
						nTotalReadXYZs += nNumberRead/3/(sizeof(float));
						str.Format(_T("points: %d (%d)"), nTotalXYZs, nTotalReadXYZs);
						//pDlg->SetDlgItemText(IDC_POINTS, str);
						nXYZs += nTotalXYZs;
						str.Format(_T("points: %d"), nXYZs);
						//pDlg->SetDlgItemText(IDC_TOTALPOINTS, str);
						if (m_bHandshakes)
						{
							command = SCANWORKS_SCAN_DATA_HS;
							TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
						}
					}
					break;

				case (SCANWORKS_SCANLINE_CURSOR_DATA):
					TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
					str.Format(_T("points: %d"), Line.num_xyzs);
					//pDlg->SetDlgItemText(IDC_POINTS, str);
					break;

				case (SCANWORKS_TOUCH_HEADER):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);
					str.Format(_T("touch: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					break;

				case (SCANWORKS_TOUCH_DATA):
					TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
					str.Format(_T("touch: %d"), Line.num_xyzs);
					//pDlg->SetDlgItemText(IDC_POINTS, str);
					break;

				case (SCANWORKS_STOP_SCAN):
					pParent->PostUpdateScanDlg(_T("ScanWorks button press (stop)"));
					//pDlg->SetDlgItemText(IDC_POINTS, "");
					TRACE0("Stop Scan.\n");
					break;

				}
				if (!loop)
					break;
			}
			//Sleep(500);
			TRACE("out of loop - calling TCPIPClose...\n");
			//pParent->PostUpdateScanDlg(_T("Closing ScanWorks Connection, Please Wait..."));
			pParent->StopPerceptronScannerMeas();
			Sleep(2000);
			//TCPIPClose(hSock);	
			//pParent->SendBufferedCloudDataAndReconnect();
			pParent->CloseScanDlg();
			//break;	// 2005.10.12
		}
		TRACE("Out of while 2...\n");
		//break;	// 2005.10.12
	}
	TCPIPCleanup();	// 2005.10.12
	//pParent->DoEndScanOperations();
	TRACE0(">>>>>>>>Out of while 1 - leaving ScanWorks thread.\n");
	//AfxEndThread(0);
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
