#pragma once

#ifdef UNDER_CE
	/// Under CE, Compact.Net framework expects 8 bytes alignment
	#pragma pack(push,8)
#else
	/// Default VC++ packing value is 8, overriding of this value to be compliant with other environment like VB6
	#pragma pack(push,4)
#endif

#include "RDS_API_Tools.h"

#ifdef LINUX
	#include "CrossPlatformUtil.h"  
	#include "CommonTypes.h"
#else
	/// Types definition
	#ifndef byte
		typedef unsigned char byte;
	#endif

	#ifndef word
		typedef unsigned short word;
	#endif

	#ifndef dword
		typedef unsigned long dword;
	#endif

	#ifndef qword
		typedef unsigned __int64 qword;
	#endif

	#ifndef datetime
		typedef	SYSTEMTIME			datetime;
	#endif

#endif


#ifndef MAKEDWORDUSINGBYTES
	#define	MAKEDWORDUSINGBYTES(b0, b1, b2, b3)	((b0<<24) + (b1<<16) + (b2<<8) + (b3<<0))
#endif

#ifndef MAKEQWORDUSINGBYTES
	#define	MAKEQWORDUSINGBYTES(b0, b1, b2, b3)	(((__int64)b0<<48) + ((__int64)b1<<32) + ((__int64)b2<<16) + ((__int64)b3<<0))
#endif
/// Macro used to check structure size compliance at building time
/// If one of lines below returns an error, it could mean two things:
///		- you've got an issue with structure packing value.
///		- a structure field has been added or changed and structure size doesn't match reference sizes anymore.
/// In both case, please advise Romer R&D team.
///
#define	RDS_CHECK_STRUCT_SIZE(struct_name, struct_size) static char check_##struct_name[ sizeof(struct_name) == struct_size ];


// **** Constants ****

/// Defines below identify current RDS API version.
/// Please read "version.txt" to know version content.
#define	RDS_API_VERSION_STRING	_T("4.1.2.0")
#define	RDS_API_VERSION_VALUE		MAKEDWORDUSINGBYTES(4,1,2,0)
#define	RDS_API_VERSION_DATE		_T("2016-02-29")

#ifndef MSGFLT_ADD
#define MSGFLT_ADD 1
#endif

#define RDS_MAX_AXIS							8
#define RDS_MSG_1									_T("RDS_SERVERAVAILABLE")
#define	RDS_TM_CHECK_STATE_ID			(WM_USER + 0x100)		// = 0x500 = 1280
#define	RDS_TM_CHECK_STATE_TIMER	(500)								// Check RDS/Client link every 500 ms

// Every tube and encoder composing an arm embeds an identification chipset.
// Maximum number of components is: 10 encoders + 2 tubes
#define	MAX_COMPONENT_IDS			(10 + 2) 

#define	LEN_SERIAL_NUMBER			39
#define	LEN_ARM_STATUS_TEXT		100
#define LEN_PROFILE_NAME			40

#define	LEN_PROBE_TYPE				40
#define	LEN_PROBE_NAME				80
#define	LEN_PROBE_PART_NUMBER	32
#define MAX_PROBE_CALCULATION_RESULTS	14

#define	MAX_PART_NAME_LENGTH	64

#define RDS_REMOTE_MOUSE_OFF	0
#define RDS_REMOTE_MOUSE_ON		1

// List of available languages in RDS: These codes are based on ISO Standard 3166
// Use these values with sRDSSetting::ID set to RDSSET_LANGUAGE

/// \note JPDL 2011-10-12 Languages up to date
#define	RDSLANG_USER_DEFAULT				""							// Get from Windows current UI language, If it not supported, english is set
#define	RDSLANG_FRENCH_FRANCE				"fr-FR"
#define	RDSLANG_ENGLISH_US					"en-US"
#define	RDSLANG_SPANISH_SPAIN				"es-ES"
#define	RDSLANG_GERMAN_GERMANY			"de-DE"
#define	RDSLANG_ITALIAN_ITALY				"it-IT"
#define	RDSLANG_PORTUGUESE_BRAZIL		"pt-BR"
#define	RDSLANG_SWEDISH_SWEDEN			"sv-SE"
#define	RDSLANG_CHINESE_SIMPLIFIED	"zh-CHS"
#define	RDSLANG_CHINESE_TRADITIONAL	"zh-CHT"
#define	RDSLANG_JAPANESE						"ja"	
#define	RDSLANG_RUSSIAN							"ru"	
#define	RDSLANG_CZECH								"cs-CZ"
#define	RDSLANG_HUNGARIAN						"hu-HU"
#define	RDSLANG_KOREAN							"ko-KR"
#define	RDSLANG_POLISH							"pl-PL"
#define	RDSLANG_TURKISH							"tr-TR"

#define	LEN_SETTING_VALUE	256

/// Defines the maximum number of points that can be sent through RDS_SCANNER_DATA callback
#define	RDS_MAX_SCANNER_POINTS	5001		/// \note JPDL 2013-06-19 : increased to 5001 due to new CMS scanner specifications.
//#define	RDS_MAX_SCANNER_POINTS	2048	/// \note JPDL 02/06/2010 : increased to 2048 due to new CMS scanner specifications.
//#define	RDS_MAX_SCANNER_POINTS	1024

#define	LEN_AVAILABLE_VALUE_NAME	31


// **************************************************************************************************************************************************************************
// ***************************************************************************** Enums **************************************************************************************
// **************************************************************************************************************************************************************************


/// Available chunks. Thid party doesn't have to use directly this enum
typedef enum _eRDSChunkID
{
	// Chunks Service <-> Clients
	RDSCI_PING = 0,														// Internal usage. Used to known if link is not broken

	// Chunks Service -> Clients
	RDSCI_ARM = 1000,													// Information regarding arm
	RDSCI_PROBE,															// Information regarding probe
	RDSCI_POSITION,														// Calculated XYZ position, button states and so on
	RDSCI_SCANNER_DATA,												// Data coming from plugged in scanner
	RDSCI_RESULT,															// Every request made by a client leads to a result sent back to it
	RDSCI_SERVER_SHUTTING_DOWN,								// Service is shutting down
	RDSCI_REFERENCE_EVENT,										/// \note BJF 07/22/2009 : A referencing event has occurred
	RDSCI_CAMERA,															/// \note DRR 2/11/2014: A camera spec.'s event has occurred

	RDSCI_VIDEO_CAPTURE_FRAME = 1020,					/// \note JPDL 03/17/2009 : New frame coming from video capture device

	// Chunks Clients -> Service
	RDSCI_CONNECT = 2000,											// Client wants to connect to service
	RDSCI_DISCONNECT,													// Client disconnects itself

	RDSCI_PLAY_MELODY,												// Ask to service to play a melody
	RDSCI_CONTROL_REMOTE_MOUSE,								// Client controls remote mouse with this chunk
	RDSCI_EXECUTE_SCRIPT,											// Calibration/Certification script execution request
	RDSCI_DISPLAY_CONTROL_PANEL,							// Display control panel
	RDSCI_DISPLAY_ADVANCED_CONTROL_PANEL,			// Display specific arm control panel
	RDSCI_UPDATE_SETTING,											// Update setting value (i.e UI language)
	RDSCI_RESET_ARM,													// Reset arm encoders
	RDSCI_GET_SETTING,												/// \note JPDL 10/10/2008 : Client asks for a setting
	RDSCI_GET_AVAILABLE_PROBES,								/// \note JPDL 10/10/2008 : Client asks for the list of available probes
	RDSCI_FORCE_PROBE,												/// \note JPDL 12/02/2008 : Client manually forces a probe
	RDSCI_RESET_TRIGGER_COUNTER,							/// \note JPDL 30/06/2009 : If available, reset internal arm trigger counter
	RDSCI_GET_ARM_DATA,												/// \note JPDL 16/04/2009 : Ask for arm data structure
	RDSCI_GET_PROBE_DATA,											/// \note JPDL 16/04/2009 : Ask for probe data structure
	RDSCI_GET_SETTING_V2,											/// \note JPDL 27/07/2010 : Replaces RDSCI_GET_SETTING
	RDSCI_UPDATE_SETTING_V2,									/// \note JPDL 27/07/2010 : Replaces RDSCI_UPDATE_SETTING
	RDSCI_FILTER_SOUND_ON_EVENT,							/// \note JPDL 2012-09-13 : 3rd parties are allowed to select sounds to play
	RDSCI_GET_CAMERA_SPECS,										/// \note DRR 2014-02-11 : 3rd parties can request the camera calibration specs from the arm

	RDSCI_VIDEO_CAPTURE_START = 2020,					/// \note JPDL 01/07/2009 : Starts video capture if available
	RDSCI_VIDEO_CAPTURE_STOP,									/// \note JPDL 01/07/2009 : Stops video capture
	RDSCI_VIDEO_CAPTURE_GRAB_FRAME,						/// \note JPDL 01/07/2009 : Asks for a frame
	RDSCI_VIDEO_CAPTURE_SUBSCRIBE,						/// \note JPDL 01/07/2009 : Subscribe to video capture service
	RDSCI_VIDEO_CAPTURE_UNSUBSCRIBE,					/// \note JPDL 03/17/2009 : Unsubscribe

	RDSCI_HEAD_LIGHT_ON = 2030,								/// \note JPDL 01/07/2010 : Switch on head light if available
	RDSCI_HEAD_LIGHT_OFF,											/// \note JPDL 01/07/2009 : Switch off head light if available

	RDSCI_HAPTIC = 2040,											/// \note BJF 09/09/2013 : Query for haptic motor
}eRDSChunkID;


/// Codes returned by RDS Service functions
/// Use GetRDSAPIErrorDescription function to get the corresponding description
typedef enum _eRDSReturnCode
{
	RDSRC_SUCCESS = 0,												// Success
	RDSRC_UNKNOWN_ERROR = 1,									// Unexpected error
	RDSRC_NOT_CONNECTED = 2,									// Client must be registered before executing a request
	RDSRC_EXECUTION_FAILED = 3,								// Execution failed
	RDSRC_INVALID_HANDLE = 4,									// An invalid handle has been provided.
	RDSRC_INVALID_PARAMETER = 5,							// A provided parameter has a bad value
	RDSRC_ALREADY_REGISTERED = 6,							// Client is already registered
	RDSRC_INTERNAL_ERROR = 7,									// RDS Service internal error
	RDSRC_ARM_NOT_CONNECTED = 8,							// An arm must be connected in order to execute this request
	RDSRC_NOT_IMPLEMENTED = 9,								// This feature is not available on this version of RDS
	RDSRC_NOT_INITIALIZED = 10,								// Something is not correctly initialized
	RDSRC_NOT_SUPPORTED_BY_ARM = 11,					// This request is not supported by the arm.
	RDSRC_UNSUPPORTED_VALUE = 12,							// Given value is not supported
	RDSRC_INVALID_VERSION = 13,								// Version issue
	RDSRC_NOT_SUPPORTED_BY_PROBE = 14,				// This request is not supported by the probe.
	RDSRC_DEVICE_NOT_AVAILABLE = 15,					// The device is not available
	RDSRC_INFINTESPECS_UPDATED = 16,
	RDSRC_NO_ANSWER_FROM_HARDWARE = 17,
	RDSRC_BAD_ANSWER_FROM_HARDWARE = 18,
}eRDSReturnCode;


/// This enum defines the arm event returned to RDS client. See sRDSPosition::eventType
typedef enum _eRDSEvent
{
	/// \note JPDL 11/13/2008 : Not used anymore
	//	UNSYNCED_DATA = 0x8000000,							// Data is not reliable, use it just for information

	EVENT_SYNC =											0x00000001,		// Position is reliable
	EVENT_PROBE_1 =										0x00000002,		// Probe triggered its first event (TP-20 for instance)
	EVENT_PROBE_2 =										0x00000004,		// Probe triggered its second event
	EVENT_BEAM1_BLCK_FWD =						0x00000008,		// Beam probe: BEAM1 cut when going down
	EVENT_BEAM1_UNBLCK_FWD =					0x00000010,		// Beam probe: BEAM1 released when going down
	EVENT_BEAM1_BLCK_BCKWD =					0x00000020,		// Beam probe: BEAM1 cut when going up
	EVENT_BEAM1_UNBLCK_BCKWD =				0x00000040,		// Beam probe: BEAM1 released when going up
	EVENT_BEAM2_BLCK_FWD =						0x00000080,		// Beam probe: BEAM2 cut when going down
	EVENT_BEAM2_UNBLCK_FWD =					0x00000100,		// Beam probe: BEAM2 released when going down
	EVENT_BEAM2_BLCK_BCKWD =					0x00000200,		// Beam probe: BEAM2 cut when going up
	EVENT_BEAM2_UNBLCK_BCKWD =				0x00000400,		// Beam probe: BEAM2 released when going up
	EVENT_OVER_LIMIT =								0x00000800,		// Axis over limit
	EVENT_BUTTON_1_WENT_UP =					0x00001000,		// Arm button 1 up
	EVENT_BUTTON_2_WENT_UP =					0x00002000,		// Arm button 2 up
	EVENT_BUTTON_3_WENT_UP =					0x00004000,		// Arm button 3 up
	EVENT_BUTTON_1_WENT_DOWN =				0x00008000,		// Arm button 1 down
	EVENT_BUTTON_2_WENT_DOWN =				0x00010000,		// Arm button 2 down
	EVENT_BUTTON_3_WENT_DOWN =				0x00020000,		// Arm button 3 down
	EVENT_BUTTON_1_DOWN_LONG =				0x00040000,		// Arm button 1 long down
	EVENT_BUTTON_2_DOWN_LONG =				0x00080000,		// Arm button 2 long down
	EVENT_BUTTON_3_DOWN_LONG =				0x00100000,		// Arm button 3 long down

	/// \note JPDL 07/24/2008 : EVENT_EASY flag usage has been rehabilitated
	EVENT_EASY =											0x00200000,		// Third party must check this flag to know if it's a measure or not.
																									// This flag is set depending on current probe type (sRDSProbe.ProbeType):
																									//		- Contact probe: EVENT_SYNC is set, EVENT_BUTTON_2_WENT_DOWN is set
																									//		- Trigger probe: EVENT_SYNC is set, EVENT_PROBE_1 is set
																									//		- External trigger: EVENT_SYNC is set, EVENT_EXTERNAL_TRIGGER is set
																									//		- Hexagon scanner: EVENT_SYNC is set, EVENT_SCANNING is set
																									//		- CMS scanner: EVENT_SYNC is set, EVENT_SCANNING is set

	/// \note JPDL 05/20/2008 : New values
	EVENT_EXTERNAL_SWITCH_WENT_UP =		0x00400000,		// External switch up
	EVENT_EXTERNAL_SWITCH_WENT_DOWN = 0x00800000,		// External switch down
	EVENT_EXTERNAL_SWITCH_DOWN_LONG = 0x01000000,		// External switch long down

	EVENT_EXTERNAL_TRIGGER =					0x02000000,		// External trigger detected

	/// \note JPDL 11/13/2008 : New values
	EVENT_SCANNING_START =						0x04000000,		// Scanning sequence started (Not set if preview mode)
	EVENT_SCANNING =									0x08000000,		// Scanning in progress (Not set if preview mode)
	EVENT_SCANNING_STOP =							0x10000000,		// Scanning sequence stopped (Not set if preview mode)

	/// \note JPDL 01/27/2010 : New value
	EVENT_PREVIEWING =								0x20000000,		// Scanner is in preview mode
}eRDSEvent;


/// \note JPDL 2012-09-12 To simplify writing
#define	EVENT_ALL											0xFFFFFFFF
#define	EVENT_BEAM_BLCK								(EVENT_BEAM1_BLCK_FWD|EVENT_BEAM1_BLCK_BCKWD|EVENT_BEAM2_BLCK_FWD|EVENT_BEAM2_BLCK_BCKWD)
#define	EVENT_BEAM_UNBLCK							(EVENT_BEAM1_UNBLCK_FWD|EVENT_BEAM1_UNBLCK_BCKWD|EVENT_BEAM2_UNBLCK_FWD|EVENT_BEAM2_UNBLCK_BCKWD)
#define	EVENT_BEAM_ALL								(EVENT_BEAM_BLCK|EVENT_BEAM_UNBLCK)
#define	EVENT_BEAM_SIMPLE_PRECISION		(EVENT_BEAM1_BLCK_FWD|EVENT_BEAM2_BLCK_FWD|EVENT_BEAM1_UNBLCK_BCKWD|EVENT_BEAM2_UNBLCK_BCKWD)
#define EVENT_BUTTON_ALL							(EVENT_BUTTON_1_WENT_UP|EVENT_BUTTON_2_WENT_UP|EVENT_BUTTON_3_WENT_UP|EVENT_BUTTON_1_WENT_DOWN|EVENT_BUTTON_2_WENT_DOWN|EVENT_BUTTON_3_WENT_DOWN|EVENT_BUTTON_1_DOWN_LONG|EVENT_BUTTON_2_DOWN_LONG|EVENT_BUTTON_3_DOWN_LONG)
#define EVENT_BUTTON_1_ALL						(EVENT_BUTTON_1_WENT_UP|EVENT_BUTTON_1_WENT_DOWN|EVENT_BUTTON_1_DOWN_LONG)
#define EVENT_BUTTON_2_ALL						(EVENT_BUTTON_2_WENT_UP|EVENT_BUTTON_2_WENT_DOWN|EVENT_BUTTON_2_DOWN_LONG)
#define EVENT_BUTTON_3_ALL						(EVENT_BUTTON_3_WENT_UP|EVENT_BUTTON_3_WENT_DOWN|EVENT_BUTTON_3_DOWN_LONG)
#define EVENT_EXTERNAL_SWITCH_ALL			(EVENT_EXTERNAL_SWITCH_WENT_UP|EVENT_EXTERNAL_SWITCH_WENT_DOWN|EVENT_EXTERNAL_SWITCH_DOWN_LONG)



/// Used with RDSCI_EXECUTE_SCRIPT chunk
/// \note JPDL 13/05/2009 : List of available calibration methods changed
typedef enum _eRDSCalibMethod
{
	CM_UNKNOWN =				0,									// Let RDS choose for you the most adapted method according probe type
	CM_SINGLE_POINT_1=	10,									// Several points taken using a cone
	CM_SINGLE_POINT_2,											// Warning: This method has not been implemented yet
	CM_SPHERE_1=				20,									// Sphere method without reference
	CM_SPHERE_2,														// Sphere method with reference
	CM_PLANE_1=					30,									// Plane method without reference
	CM_PLANE_2,															// Plane method with reference
	CM_T=								40,									// TBar method for Tube probes
}eRDSCalibMethod;

#define	RDSCalibMethodList	_T("10=Single point 1, 11=Single point 2, 20=Sphere 1, 21= Sphere 2, 30=Plane 1, 31=Plane 2, 40=T")


/// Used with RDSCI_EXECUTE_SCRIPT chunk
typedef enum _eRDSScriptType
{
	ST_UNKNOWN = 0,
	ST_PROBE_CALIBRATION,
	ST_CERTIFICATION,
	ST_LEAPFROG,
	ST_PROBE_CHECK,										/// \note JPDL 2012-09-28 New item added
}eRDSScriptType;



/// Identifies arm status, see sRDSArm structure
/// \note JPDL 07/09/2008 : eArmStatus enum: AS_CONNECTED_WARNING had a bad value.
typedef enum _eArmStatus
{
	AS_NOT_CONNECTED = 0,								// Service is not connected to arm.
																			// Check ArmStatusText for more detail
	AS_CONNECTED = 1,										// Connected to arm. No error.	
	AS_CONNECTED_WARNING = 2,						// Connected but some warnings remain
																			// Check ArmStatusText for more detail
}eArmStatus;


/// Identify type of arm configured using RDS Control Panel
typedef enum _eRDSArmType
{
	RDSAT_UNKNOWN = -1,
	RDSAT_NCAUSB = 0,										// Absolute/Multi Gage, USB link
	RDSAT_NCAWIFI,											// Absolute/Multi Gage, WiFi link
	RDSAT_SIGMAETHERNET,								// Sigma arm, Wired ethernet link
	RDSAT_SIGMAWIFI,										// Sigma arm, WiFi link
	RDSAT_INFINITEUSB,									// Infinite arm, USB link
	RDSAT_INFINITEWIFI,									// Infinite arm, WiFi link,
	RDSAT_NOARM,												// No arm configured
	RDSAT_NCATCP,												// Absolute/Multi Gage, Wired Ethernet link
	RDSAT_LEICA_TRACKER_EMSCON = 20,		// Leica tracker using Emscon API (AT401, AT901). Only for internal purpose, not available for integrators
	RDSAT_LEICA_TRACKER_LMF,						// Leica tracker using LMF (AT960/AT930)
}eRDSArmType;						


/// Identifies which type of NCA arm is connected
typedef enum _eArmType
{
	NCAAT_UNKNOWN = 0,
	NCAAT_MULTIGAGE,										// Multi-gage
	NCAAT_RA7,													// RA7 = 6 axis
	NCAAT_RA7i,													// RA7i = RA7si = 7 axis with RS1 integrated scanner
	NCAAT_RA7m,													// RA7m = RA7se = 7 axis with two TKJ adapters
	NCAAT_INFINITE,											// 6 axis infinite
	NCAAT_INFINITE7,										// 7 axis infinite
	NCAAT_STINGER,											// Stinger
	NCAAT_ARMTUBE,											/// \note JPDL 2011-11-22 New arm type (ArmTube) = 6axis dedicated to tubing
	NCAAT_RA7sei,												/// \note HJ 2013-9-18 : addition of RA7sei
	NCAAT_LASTVALUE = NCAAT_RA7sei,
	NCAAT_NBVALUES = NCAAT_LASTVALUE+1,
}eArmType;

#define	ArmTypeList	_T("0=Unknown, 1=Multi-Gage, 2=RA7, 3=RA7si, 4=RA7se, 5=Infinite, 6=Infinite7, 7=Stinger, 8=ArmTube, 9=RA7sei")		/// \note JPDL 2012-02-02 


typedef enum _ePerformanceLevel
{
	PF_UNKNOWN = 0,
	PF_ENTRY_LEVEL=1,			/// \note JPDL 2011-11-22 New performance level (1=Entry level)
	PF_GOOD = 3,
	PF_BETTER = 5,
	PF_BEST = 7
}ePerformanceLevel;

#define	PerformanceLevelList	_T("0=Unknown, 1=71, 3=73, 5=75, 7=77")					/// \note JPDL 2014-09-09 Performance level descriptions changed
//#define	PerformanceLevelList	_T("1=Entry level, 3=Good, 5=Better, 7=Best")


/// \note JPDL 2012-02-01 Arm version management
typedef enum _eArmVersion
{ 
	AV_RA7 = 0,
	AV_RA7_2011,
	AV_KRONOS,
	AV_RA7_2011_PH2,			/// \note HJ 2013-09-18 Version 3 addition
	AV_RA7_V4,						/// \note JPDL 2014-07-24 Version 4 added
	AV_LASTVALUE = AV_RA7_V4,	
	AV_NBVALUES = AV_LASTVALUE+1,
}eArmVersion;

#define	ArmVersionList	_T("0=RA7 V1, 1=RA7 V2, 2=71, 3=RA7 V3, 4=RA7 V4")	/// \note JPDL 2014-07-24 Version 4 added


/// Identifies type of probe plugged in arm. See sRDSProbe::ProbeType. 
typedef enum _eProbeType
{
	PT_UNKNOWN = 0,							// Unknown type
	PT_V = 1,										// "V" probe
	PT_CONTACT = 2,							// Contact probe
	PT_PLA = 3,									// Managed by Sigma arms only
	PT_RENISHAW = 4,						// Managed by Sigma arms only
	PT_UDT = 5,									// Managed by Sigma arms only
	PT_VERTEX = 6,							// Managed by Sigma arms only
	PT_ROTATIF = 7,							// Managed by Sigma arms only
	PT_VIDEO_PROBE_V2 = 8,			// Managed by Sigma arms only
	PT_VIDEO_PROBE_V3 = 9,			// Managed by Sigma arms only
	PT_TRIGGER_PROBE = 10,			// Trigger probe
	PT_EXTERNAL_TRIGGER = 11,		// External trigger probe
	PT_HEXAGON_SCANNER = 12,		// Hexagon scanner. Covers RS1, RS2, RS3 or HP-L-8.9
	PT_CMS_SCANNER = 13,				// Covers CMS108 or HP-L-20.8 scanner

	/// \note JPDL 2015-03-03 Probe types below have been introduced with RDS API v4.1.
	/// For compatibility reasons, if client uses an API lower than 4.1, type will be automatically converted to PT_CONTACT or PT_HEXAGON_SCANNER.
	PT_REFLECTOR = 20,					// Reflector. For Leica Absolute Tracker only
	PT_T_PROBE = 21,						// T-Probe. For Leica Absolute Tracker only
	PT_T_SCAN = 22,							// T-Scan. Not supported by RDS. For Leica Absolute Tracker only
	PT_T_MAC = 23,							// T-MAC. For Leica Absolute Tracker only
	PT_LAS = 24,								// Leica Absolute Tracker. For Leica Absolute Tracker only
}eProbeType;


/// Identifies probe status, see sRDSProbe structure
typedef enum _eProbeStatusBits
{
	PSB_SELECTED = (1<<0),												// = 1 if probe is currently selected
	PSB_FORCED = (1<<1),													// = 1 if probe has been forced

	PSB_QUALIFIED = (1<<2),												// = 1 if probe has already been qualified
	PSB_ACCEPTABLE_QUALIFICATION_RESULT = (1<<3),	// = 1 if qualification result is good enough to accept measurement

	PSB_INITIALIZED = (1<<4),												/// \note JPDL 2013-03-19 Bit is set if the probe has been correctly initialized
	PSB_SENSOR_OFF = (1<<5),												/// \note JPDL 2014-07-10 Bit is set when the sensor has been switched off by user. Available for HPL-20.8 sensor only.

}eProbeStatusBits;


// PCOD meaning
typedef enum _ePCOD
{
		/// \note JPDL 2011-12-12 According to Brian Frohlich, PCalBit can be used because not use anymore by WinRDS.
		PNonRepeatableBit = 0x80,		// When set to 1, the probe is considered as non-repeatable (screwed one for instance).
		PCalBit = 0x80,							// b7 = 1 for calibration probe
		PCustBit = 0x40,						// b6 = 1 for custom probe kit
																// b5-b0 is PCOD (typically 1-22)
}ePCOD;


typedef enum _eProbeCapability
{
	PC_SUPPORTS_DOUBLE_PRECISION = 0x00000001,			// Probe supports double precision mode. Can be set if probe is a Tube probe
}eProbeCapability;


typedef enum _eRefType
{
	REV_GRID_LOK = -1,
	NO_REF = 0,
	GRID_LOK = 1,
	SPACE_LOK = 2,
	TOO_LOK = 4,
	PART_LOK = 8,

	VPS_REF = 0x0100,						// VPS is not supported by RDS
	LPF_REF,
}eRefType;


/// Can be used to access sRDSPosition::ProbeXYZ, ProbeMatrix, ArmXYZ and ArmMatrix fields
typedef enum _eMatrixComponent
{
	RDS_X = 0,
	RDS_Y = 1,
	RDS_Z = 2,

	RDS_I = 0,
	RDS_J = 1,
	RDS_K = 2,
} eMatrixComponent;


/// This enum defines the list of predefines melodies RDS can play
typedef enum _eRDSSounds
{
	/// \note JPDL 06/23/2008 : RDS_CUSTOM_SOUND renamed to RDS_DEFAULT_SOUND
	RDS_DEFAULT_SOUND = 0,				// Default sound
	RDS_DATA_TAKEN_SOUND,					// Point has been captured
	RDS_ENTER_OVER_LIMIT_SOUND,		// An arm encoder is over limit
	RDS_EXIT_OVER_LIMIT_SOUND,		// Arm is no more over limit
	RDS_DATA_ERASED_SOUND,				// Point has been deleted
	RDS_LONG_BUTTON_SOUND,				// Long press done with an arm button
	RDS_CONNECTED_TO_ARM,					// Service is connected to arm
	RDS_PROBE_INSERTED,						// Probe has been plugged in
	RDS_PROBE_PULLED_OUT,					// Probe has been plugged out
	RDS_LOW_POWER_WARNING,				// Battery level is under low power threshold
	RDS_SUCCESS_SOUND,						/// \note JPDL 06/23/2008 : Operation successful
	RDS_ERROR_SOUND,							/// \note JPDL 14/05/2009 : Played if error (ie probe conflict)
	RDS_SCANNING_START,						/// \note JPDL 13/08/2009 : Start of scanning
	RDS_SCANNING_STOP,						/// \note JPDL 13/08/2009 : End of scanning

	/// \note BJF 7/30/2009 : New sounds
	RDS_REFERENCE_ENGAGED,
	RDS_REFERENCE_DISENGAGED,
	RDS_REFERENCE_POINT_HELD,

	/// \note JPDL 19/05/2010 : New sounds for tube probe
	RDS_EVENT_BEAM1_BLCK_FWD_SOUND,
	RDS_EVENT_BEAM1_UNBLCK_FWD_SOUND,
	RDS_EVENT_BEAM1_BLCK_BCKWD_SOUND,
	RDS_EVENT_BEAM1_UNBLCK_BCKWD_SOUND,
	RDS_EVENT_BEAM2_BLCK_FWD_SOUND,
	RDS_EVENT_BEAM2_UNBLCK_FWD_SOUND,
	RDS_EVENT_BEAM2_BLCK_BCKWD_SOUND,
	RDS_EVENT_BEAM2_UNBLCK_BCKWD_SOUND,

	RDS_NC_READY = 25,
	RDS_NC_MEASURE_SPECIAL,
	RDS_NC_SUCCESS,
	RDS_NC_MOVE_BACK,

}eRDSSounds;


typedef enum _eRDSPlaySoundPriority
{
	RDSPSP_PLAY_NOW = 0,					// Custom sound is played immediately
	RDSPSP_ENQUEUE = 1,						// Custom sound is euqueued and will be played as soon as possible
}eRDSPlaySoundPriority;


typedef enum _eRDSVideoCaptureFrameDestination
{
	RDSFD_MESSAGE = 0,																// Captured frame is sent back to client using a Windows message
	RDSFD_HARD_DISK_AND_MESSAGE,											// Captured frame is saved to RDS captured frames directory and sent by message
}eRDSVideoCaptureFrameDestination;


typedef enum _eRDSVideoCaptureImageFormat
{
	VCIF_JPEG = 0,
	VCIF_BMP
}eRDSVideoCaptureImageFormat;


typedef enum _eRDSControlPanelSection
{
	RDSCPS_UNDEFINED = -1,						// Let RDS choose the appropriate section
	RDSCPS_SUMMARY,										// Summary
	RDSCPS_CONNECTION,								// Connection
	RDSCPS_GENERAL_PARAMETERS,				// General parameters
	RDSCPS_PROBE,											// List of probe
	RDSCPS_SCANNER,										// Scanner (Only when a scanner probe is plugged in)
	RDSCPS_FEATURES,									// Features
	RDSCPS_ADVANCED_SETTINGS,					// Advanced settings
	RDSCPS_VIDEO_CAPTURE,							// Video capture (RA7 6 axis arms only)
	RDSCPS_PROBE_VIEW,								/// \note JPDL 05/10/2010 : Probe view
	RDSCPS_PROFILE,										/// \note JPDL 2011-10-12 Profiles management screen
	RDSCPS_ACCESS_CONTROL,						/// \note JPDL 2011-10-12 Access control screen
	RDSCPS_NB_ITEMS,
}eRDSControlPanelSection;


typedef enum _eRDSRemoteMouseBits
{
	RDSRMB_SILENT = (1<<0),							// If set, don't display OSD message, useful for GPad.
}eRDSRemoteMouseBits;


/// Tells how to start and stop a scanning sequence
typedef enum _eScanningCaptureMode
{
	SCCM_PRESS_BUTTON_ONCE_TO_START_AND_TO_STOP=0,			// Use measurement button as a trigger
	SCCM_CAPTURE_WHILE_BUTTON_PRESSED,									// Capture as long as measurement button is pressed
	SCCM_CAPTURE_N_STRIPES,															// Capture a limited amount of stripes
}eScanningCaptureMode;																/// \note JPDL 2013-11-26 Enum values renamed to avoid confusion with those of eScannerMode


/// \note JPDL 11/01/2011 : New
typedef enum _eAxisOrientation
{
	AO_UNKNOWN = -1,
	AO_NEG_X,
	AO_POS_X,
	AO_NEG_Y,
	AO_POS_Y,
	AO_NEG_Z,
	AO_POS_Z,
}eAxisOrientation;

/// \note JPDL 11/01/2011 : New
typedef enum _eKeyStoneMode
{
	KSM_UNKNOWN = -1,
	KSM_TRAPEZOID,
	KSM_RECTANGLE,
}eKeyStoneMode;


/// This enum defines the list of settings that can be retrieved and sometimes changed
/// Please check setting comment to know:
///   - if setting can be set or not.
///   - if the request needs a parameter (ie RDSSET_SCANNING_MODE_PARAMETER needs Mode as parameter).
///   - Which version of RDS_GET_SETTING/RDS_UPDATE_SETTING is needed with this setting
///   - the type of the manipulated value.
///
/// Note that some scanner settings are not available for now.
typedef enum _eRDSSetting
{
	RDSSET_UNKNOWN = 0,	

	// Description																				Operation	Param		Version	Value type			Supported values
	// RDS UI language																		Get/Set						1, 2		unicode string	See below. Default is an empty nul terminated string
	RDSSET_LANGUAGE,																				

	// Enable/Disable "Long press on arm buttons"					Get/Set						1, 2		int							0=disabled, 1=enabled (default)
	RDSSET_MANAGE_LONG_PRESS_ON_ARM_BUTTONS,								

	// Arm playback volume																Get/Set						1, 2		int							0 to 100%
	RDSSET_PLAYBACK_VOLUME,																	

	// Way to connect to the arm													Get/Set						1, 2		int							0=USB (default), 1=Wi Fi
	RDSSET_LINK_TYPE,																												
																																					
	// Arm IP address used if link type is WiFi						Get/Set						1, 2		unicode string	Default is an empty nul terminated string
	// NCA arms: default address is 192.168.0.3
	// Should not be changed
	RDSSET_IP_ADDRESS,																											
																																					
	// Enable/Disable "Arm stops"													Get/Set						1, 2		int							0=disabled, 1=enabled (default)
	// As arm stops ensures that user doesn't measure
	// outside arm calibration limits, this setting
	// shouldn't be changed
	RDSSET_ENABLE_STOPS,																		

	// Tells if remote mouse can be used									Get/Set						1, 2		int							0=disabled, 1=enabled (default)
	RDSSET_REMOTE_MOUSE_AVAILABLE,																					
																																					
	// Access to rds control panel option									Get/Set						1, 2		int							0=disabled, 1=enabled (default)
	RDSSET_AUTO_SUPPRESS_REMOTE_MOUSE_IF_NO_MOVEMENT,												
																																					
	// Max arm capture frequency.													Get/Set						1, 2		int							0 to 100Hz. Default is 100Hz

	// Allow slow computers (ie WinCE devices to play
	// on CPU usage consumed by RDS. Default is 100Hz
	RDSSET_MAX_CAPTURE_FREQUENCY,											

	// Enable/Disable "sounds on events"									Get/Set						1, 2		int							0=disabled, 1=enabled (default)
	// Arm has the ability to automatically play sound
	// when special events occurre. 
	RDSSET_SOUNDS_ON_EVENTS_ENABLED,																				
																																					
	// Tells how to start and to stop scanning sequence		Get/Set						1, 2		eScanningCaptureMode value			Default is SCM_PRESS_BUTTON_ONCE_TO_START_AND_TO_STOP
	RDSSET_SCANNING_CAPTURE_MODE,																						
																																					
	// Optional parameter used with scanning capture mode	Get/Set						1, 2		int							1 to ?				
	// If SCM_CAPTURE_N_STRIPES, int (number of not empty 
	// stripes to capture)
	RDSSET_SCANNING_CAPTURE_MODE_PARAMETER,						

	// Enable/Disable "sound guide" during scanning				Get/Set						1, 2		int							0=disabled (default), 1=enabled
	RDSSET_SCANNING_SOUND_GUIDE_ENABLED,							

	/// \note JPDL 27/07/2010 : New settings. They are available only using RDSCI_UPDATE_SETTING_V2 and RDSCI_GET_SETTING_V2 chunks

	// List of supported modes supported by scanner				Get								2 only	List of sRDSAvailableValue structures
	RDSSET_SCANNING_AVAILABLE_MODES = 2000,						

	// Current scanning mode															Get/Set						2 only	int							Contains one of RDSSET_SCANNING_AVAILABLE_MODES values
	RDSSET_SCANNING_CURRENT_MODE,											

	// List of available mode parameters									Get								2 only	List of sRDSAvailableValue structures
	RDSSET_SCANNING_AVAILABLE_MODE_PARAMETERS,				

	// Gives access to a mode parameter										Get				Mode		2 only	double					Value depends on given parameter
	RDSSET_SCANNING_MODE_PARAMETER,										

	// List of supported sensitivities										Get								2 only	List of sRDSAvailableValue structures
	RDSSET_SCANNING_AVAILABLE_SENSITIVITIES,					

	// Current sensitivity																Get/Set						2 only	int							Contains one of RDSSET_SCANNING_AVAILABLE_SENSITIVITIES values
	RDSSET_SCANNING_CURRENT_SENSITIVITY,	

	// List of supported filter types											Get								2 only	List of sRDSAvailableValue structures
	RDSSET_SCANNING_AVAILABLE_FILTER_TYPES,

	// Current value of a filter													Get/Set		Filter	2 only	int							Meaning of value depends on scanner type
	RDSSET_SCANNING_CURRENT_FILTER_VALUE,			 				

	// Maximum size of keystone for current scanning mode	Get								2 only	sRDSKeyStone structure
	RDSSET_SCANNING_MAX_KEYSTONE,										

	// Current size of keystone														Get(/Set)					2 only	sRDSKeyStone structure
	RDSSET_SCANNING_CURRENT_KEYSTONE,												

	// Keystone usage on/off (Set available soon)					Get(/Set)					2 only	int							0=disabled (default), 1=enabled
	RDSSET_SCANNING_USE_KEYSTONE,										

	// List of supported key stone modes									Get								2 only	List of sRDSAvailableValue structures
	RDSSET_SCANNING_AVAILABLE_KEYSTONE_MODES,				

	// Current key stone mode (Set available soon)				Get(/Set)					2 only	int							contains one of RDSSET_SCANNING_AVAILABLE_KEYSTONE_MODES values
	RDSSET_SCANNING_CURRENT_KEYSTONE_MODE,

	// Maximum size of keystone for current scanning mode	Get								2 only	sRDSKeyStoneV2 structure
	RDSSET_SCANNING_MAX_KEYSTONE_V2,										

	// Current size of keystone														Get(/Set)					2 only	sRDSKeyStoneV2 structure
	RDSSET_SCANNING_CURRENT_KEYSTONE_V2,												

	/// \note JPDL 2013-03-19 Returns RDS version					Get								2 only	unicode string	
	RDSSET_RDS_VERSION,

	/// \note JPDL 2013-03-19 Returns Arm calibration date Get							2 only	unicode string	
	RDSSET_ARM_CALIBRATION_DATE,

	// Current scanning mode															Get								2 only	int							
	RDSSET_SCANNING_SYSTEM_CAPABILITIES,											

	// Enable/Disable Haptic															Get/Set						1, 2		int							0=disabled (default), 1=enabled
	RDSSET_HAPTIC,

}eRDSSetting;


typedef enum _eRDSSettingValueType
{
	SVT_UNKNOWN = 0,
	SVT_WSTRING,
	SVT_INT,
	SVT_DOUBLE,
	SVT_KEYSTONE,
	SVT_NVALUES,
	SVT_KEYSTONE_V2,
} eRDSSettingValueType;


typedef enum _CalibrationDetailedResult
{
	CDR_Unknown = -1,
	CDR_InitialDeviation = 0,
	CDR_ResidualDeviation,
	CDR_StandardDeviation,
	CDR_MinDeviation,
	CDR_MaxDeviation,
	CDR_MaxAbsoluteDeviation,
	CDR_AverageDeviation,
	CDR_DiameterDeviation,
	CDR_CenterDeviation,
	CDR_CalculationTime,
	CDR_LeastSquaresDividedByNbPoints,
}eCalibrationDetailedResult;


#ifndef datetime
	typedef	SYSTEMTIME			datetime;
#endif

#ifdef _WIN64		/// \note JPDL 09/06/2010 : Bad constant used preventing building for x64
//#ifdef WIN64
		#pragma message("RDS API - Compiled for x64 CPU")
		typedef	UINT	RDSHANDLE;
#else
		#pragma message("RDS API - Compiled for x86 CPU")
		typedef	HWND	RDSHANDLE;
#endif


																						 
// **************************************************************************************************************************************************************************
// *************************************************************************** Structures ***********************************************************************************
// **************************************************************************************************************************************************************************


/// Internal RDS structure
/// Size = 32 bytes
struct sRDSChunk
{
	eRDSChunkID	ChunkID;									// See eRDSChunkID for a list of available chunks
	long				Parameter1;								// Optional parameter, use depends on chunk ID:
																				// RDSCI_RESULT -> Parameter1 = RequestID
	long				Parameter2;								// Optional parameter, use depends on chunk ID:
																				// RDSCI_RESULT -> Parameter2 = Request return code
	long				ChunkSize;								// Size of data coming after structure

	byte				Reserved[16];							// Reserved for further usage

	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSChunk, 32)


/// sRDSClient identifies which client sent the request to service
/// Size = 28 bytes
struct sRDSClient
{
	/// \note JPDL 13/10/2009 : HWND is a 64bit under 64bit environments
	RDSHANDLE		hwnd;												// handle of client window which will receive back msgs from service.
//	HWND		hwnd;													

	UINT		extendedData;									/// \note JPDL 06/19/2008 : Not used anymore
	BOOL		RemoteMouseOn;								/// \note JPDL 06/19/2008 : Not used anymore
	UINT		Sound;												/// \note JPDL 06/19/2008 : Not used anymore
	UINT		freq;													/// \note JPDL 06/19/2008 : Not used anymore
	UINT		dur;													/// \note JPDL 06/19/2008 : Not used anymore
	UINT		reserved;											/// \note JPDL 06/19/2008 : Not used anymore

	void Init(void)	{	memset(this, 0, sizeof(*this));	}
};

RDS_CHECK_STRUCT_SIZE(sRDSClient, 28)


/// \note JPDL 06/19/2008 : Used since RDS API v0.3.3
/// Structure sent by client to service to identify itself.
/// It is sent during connection to RDS Service. See RDSCI_CONNECT command
/// Size = 272 bytes
struct sRDSConnect
{
	UINT		RDSAPIVersion;								// RDS API version used by client
	byte		AuthenticationValue[8];				// Encrypted authentication value describing which features client has access to
																				// If field is = {0,0,0,0,0,0,0,0} -> Client is unidentified
	UINT		ExtendedData;									// For internal use only
	bool		UnicodeCharset;								/// \note JPDL 01/20/2009 : TRUE if structures have been build in a unicode environment
																				// In this case, client will receive unicode structures instead ansi ones.
	byte		Reserved[252];								// Reserved for further usage

	void Init(void)	
	{	
		memset(this, 0, sizeof(*this));	
		#ifdef RDS_API_UNICODE
			UnicodeCharset = TRUE;
		#endif
	}
};

RDS_CHECK_STRUCT_SIZE(sRDSConnect, 272)

	
/// Used with RDSCI_EXECUTE_SCRIPT chunk
// size = 280 bytes
struct sRDSScript
{
	int								X, Y, Width, Height;
	eRDSScriptType		ScriptType;
	eRDSCalibMethod		Method;

	byte	Reserved[256];							// Reserved for further usage

	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSScript, 280)
	

// **************************************************************************************************************************************************************************


/// This structure describes the arm currently connected to service.
/// Used with RDSCI_ARM chunk
/// This structure is sent from service to client when a property changes
/// Size = 420 bytes
struct sRDSArmA
{
	byte				HardwareVersion;											// Hardware version, v1 -> Multi-Gage
	byte				MajorFWVersion;												// Version of firmware used by arm motherboard
	byte				MinorFWVersion;												// Version of firmware used by arm motherboard
	dword				NbAxis;																// Number of axis composing the arm
	dword				Tube1Length;													// First tube length (in millimeters)
	dword				Tube2Length;													// Second tube length (in millimeters)
	byte				BatteryLevel;													// Level of battery
																										// 0xF0 -> 0xFF: status values:
																										//	0xF0 -> battery charging
																										//	0xF1 -> power supply detected
																										//	0xFF -> Invalid battery level
																										// Other values: 0 to 100%
	byte				ExternalTemperature;									// 0 -> Value over limit
																										// 0xFF -> Sensor error or missing
																										// Multiply correct value by 0.5 for C temperature (+0.5C to +85C)
	byte				SpeakerVolume;												// Speaker volume: 0 to 100%
	char				SerialNumber[LEN_SERIAL_NUMBER+1];		// Readable arm serial number
	eRDSArmType	ArmType;															/// \note JPDL 03/13/2008 : See eRDSArmType enum for a list of available values
	dword				NbComponents;													/// \note JPDL 04/01/2008 : Number of components
	qword				Components[MAX_COMPONENT_IDS];				/// \note JPDL 04/01/2008 : Component IDs
	eArmStatus	ArmStatus;														/// \note JPDL 04/02/2008 : See eArmStatus enum for a list of available values
																										///													Check ArmStatusText if ArmStatus != AS_NO_ERROR
	dword				ArmID;																/// \note JPDL 05/27/2008 : New fieldArm serial number
	char				ArmStatusText[LEN_ARM_STATUS_TEXT+1];	/// \note JPDL 06/27/2008 : Contains arm status description
	eArmType		NCAArmType;														/// \note JPDL 07/25/2008 : Given if an NCA arm is connected
	bool				IsRemoteMouseON;											/// \note JPDL 09/24/2008 : TRUE if remote mouse ON
	bool				ArmReset;															/// \note JPDL 10/27/2008 : TRUE if arm is reset, FALSE -> examine AxisReset array to known which axis is in fault
	bool				AxisReset[RDS_MAX_AXIS];							/// \note JPDL 10/27/2008 : TRUE if axis is reset
	bool				bVideoCaptureRunning;									/// \note JPDL 01/08/2009 : Video capture is running, be ready to receive grabbed frames
	bool				bHeadLightOn;													/// \note JPDL 01/07/2010 : TRUE if head light is on, RA7 USB only
	dword				dwVolume;															/// \note JPDL 09/07/2010 : in mm ie: 1500, 2500, ...
	char				ProfileName[LEN_PROFILE_NAME+1];			/// \note JPDL 2011-08-25 : Current activated calibration profile 
	ePerformanceLevel	PerformanceLevel;								/// \note JPDL 2012-02-02 : PerformanceLevel added to sRDSArmx
	eArmVersion	ArmVersion;														/// \note JPDL 2012-02-02 : ArmVersion added to sRDSArmx

	byte				Reserved[72];													// Reserved for further usage

	void Init(void)	{	memset(this, 0, sizeof(*this)); ArmType = RDSAT_UNKNOWN;	}

	bool IsANCAArm(void) { return (ArmType == RDSAT_NCATCP) || (ArmType == RDSAT_NCAUSB) || (ArmType == RDSAT_NCAWIFI); }
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSArmA, 420)
#endif


/// Size = 560 bytes
struct sRDSArmW
{
	byte				HardwareVersion;											// Hardware version, v1 -> Multi-Gage
	byte				MajorFWVersion;												// Version of firmware used by arm motherboard
	byte				MinorFWVersion;												// Version of firmware used by arm motherboard
	dword				NbAxis;																// Number of axis composing the arm
	dword				Tube1Length;													// First tube length (in millimeters)
	dword				Tube2Length;													// Second tube length (in millimeters)
	byte				BatteryLevel;													// Level of battery
																										// 0xF0 -> 0xFF: status values:
																										//	0xF0 -> battery charging
																										//	0xF1 -> power supply detected
																										//	0xFF -> Invalid battery level
																										// Other values: 0 to 100%
	byte				ExternalTemperature;									// 0 -> Value over limit
																										// 0xFF -> Sensor error or missing
																										// Multiply correct value by 0.5 for C temperature (+0.5C to +85C)
	byte				SpeakerVolume;												// Speaker volume: 0 to 100%
	wchar_t			SerialNumber[LEN_SERIAL_NUMBER+1];		// Readable arm serial number
	eRDSArmType	ArmType;															/// \note JPDL 03/13/2008 : See eRDSArmType enum for a list of available values
	dword				NbComponents;													/// \note JPDL 04/01/2008 : Number of components
	qword				Components[MAX_COMPONENT_IDS];				/// \note JPDL 04/01/2008 : Component IDs
	eArmStatus	ArmStatus;														/// \note JPDL 04/02/2008 : See eArmStatus enum for a list of available values
																										///													Check ArmStatusText if ArmStatus != AS_NO_ERROR
	dword				ArmID;																/// \note JPDL 05/27/2008 : New fieldArm serial number
	wchar_t			ArmStatusText[LEN_ARM_STATUS_TEXT+1];	/// \note JPDL 06/27/2008 : Contains arm status description
	eArmType		NCAArmType;														/// \note JPDL 07/25/2008 : Given if an NCA arm is connected
	bool				IsRemoteMouseON;											/// \note JPDL 09/24/2008 : TRUE if remote mouse ON
	bool				ArmReset;															/// \note JPDL 10/27/2008 : TRUE if arm is reset, FALSE -> examine AxisReset array to known which axis is in fault
	bool				AxisReset[RDS_MAX_AXIS];							/// \note JPDL 10/27/2008 : TRUE if axis is reset
	bool				bVideoCaptureRunning;									/// \note JPDL 01/08/2009 : Video capture is running, be ready to receive grabbed frames
	bool				bHeadLightOn;													/// \note JPDL 01/07/2010 : TRUE if head light is on, RA7 USB only
	dword				dwVolume;															/// \note JPDL 09/07/2010 : in mm ie: 1500, 2500, ...
	wchar_t			ProfileName[LEN_PROFILE_NAME+1];			/// \note JPDL 2011-08-25 : Current activated calibration profile 
	ePerformanceLevel	PerformanceLevel;								/// \note JPDL 2012-02-02 : PerformanceLevel added to sRDSArmx
	eArmVersion	ArmVersion;														/// \note JPDL 2012-02-02 : ArmVersion added to sRDSArmx

	byte				Reserved[32];													// Reserved for further usage

	void Init(void)	{	memset(this, 0, sizeof(*this)); ArmType = RDSAT_UNKNOWN;	}

	bool IsANCAArm(void) { return (ArmType == RDSAT_NCATCP) || (ArmType == RDSAT_NCAUSB) || (ArmType == RDSAT_NCAWIFI); }
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSArmW, 560)
#endif

#ifdef RDS_API_UNICODE
	typedef struct sRDSArmW	sRDSArm;
#else
	typedef struct sRDSArmA	sRDSArm;
#endif


// **************************************************************************************************************************************************************************


/// This structure describes the probe currently plugged in the arm.
/// It's used with RDSCI_PROBE chunk. This structure is sent from service to client every time probe selection changes
/// Important: There are two particular cases to take in account:
///              - If user is calibrating a new probe. This one is considered as missing in third-party side.
///              Probe information will be sent again to third party at end of calibration.
///              - Calibration calculation result is too bad (> 1.0). No probe data sent to third-party. 
///              Probe must be recalibrated.
/// Size = 444 bytes
struct sRDSProbeA
{
	double				BallDiameter;									// Probe ball diameter in mm. Beam diameter for tube probes
	double				Length;												// Probe length in mm
	qword					ID;														// Unique 64bits probe ID, comes from chipset embed in probe
	eProbeType		ProbeType;										// Probe type, see eProbeType enum for a list of available types
	char					TypeName[LEN_PROBE_TYPE+1];		// Readable probe type
	char					Name[LEN_PROBE_NAME+1];				// Probe name
	SYSTEMTIME		CreationDateTime;							// Says when probe has been created in arm specs
	SYSTEMTIME		UpdateDateTime;								// Indicates last time probe specs have been changed (i.e. probe calibration)
	float					CalculationResult;						// Result of probe calculation. Warning! It's not a result in millimeters.
	dword					StatusBits;										// It's a combination of eProbeStatusBits enum bits
	int						Index;												// Probe index: -1 if no probe selected, 0 for first probe, 1 for second probe and so on
	dword					PLOC;													// TKJ connector gender on probe side: 0 = female TKJ, 1 = male TKJ. 
	double				b1rx;													/// \note BJF 2010\3\3 - added b1rx and b2rx to probe data for DOCS.
	double				b2rx;													/// \note BJF 2010\3\3 - added b1rx and b2rx to probe data for DOCS.
	dword					PCOD;													/// \note JPDL 05/10/2010 : Internal probe attributes
	double				XDeviation, YDeviation;				/// \note JPDL 06/10/2010 : Probe center X and Y deviations
	int						SubType;											/// \note JPDL 2012-06-28 : Meaning depends on probe connected. Today only useful for Hexagon scanners. 
	char					PartNumber[LEN_PROBE_PART_NUMBER+1];		/// \note JPDL 2012-09-13 Part Number
	dword					Version;											/// \note JPDL 2012-09-13 Probe version. Tube probe: 0=First version (2009?), 1=2012 version
	float					Size;													/// \note JPDL 2012-09-13 Can be tube size or probe diameter. This information will be displayed on RDS screens
	dword					Capabilities;									/// \note JPDL 2012-09-13 Indicates some probe capabilities. See eProbeCapability enum.
	eRDSCalibMethod	CalibrationMethod;																		/// \note JPDL 2015-09-17 Method that has been used to qualify the probe
	float						CalculationResults[MAX_PROBE_CALCULATION_RESULTS];		/// \note JPDL 2015-09-17 Detailed calculation results. Its content depends on probe type and method used
																																				/// See eCalibrationDetailedResult enum

	byte					Reserved[96];								// Reserved for further usage

	void Init(void) { memset(this, 0, sizeof(*this)); BallDiameter = 0; Length = 0; CalculationResult = 0; Index = -1; b1rx = b2rx = 0; XDeviation = YDeviation = 0; SubType = -1; Size = 0;  }

	bool IsAScanner(void) { return (ProbeType == PT_HEXAGON_SCANNER) || (ProbeType == PT_CMS_SCANNER) || (ProbeType == PT_T_SCAN) || (ProbeType == PT_LAS); }
	bool IsAContactProbe(void) { return (ProbeType == PT_TRIGGER_PROBE) || (ProbeType == PT_CONTACT) || (ProbeType == PT_REFLECTOR) || (ProbeType == PT_T_MAC) || (ProbeType == PT_T_PROBE); }
	bool IsANonContactProbe(void) { return (ProbeType == PT_V); }
	bool IsACustomProbe(void) { return (PCOD & PCustBit) != 0; }
	bool IsANonRepeatableProbe(void) { return (PCOD & PNonRepeatableBit) != 0; }		/// \note JPDL 2011-12-14 Returns true if the probe is a non repeatable one (ie Kronos screwed probe)
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSProbeA, 444)
#endif

/// Size = 564 bytes
struct sRDSProbeW
{
	double				BallDiameter;									// Probe ball diameter in mm. Beam diameter for tube probes
	double				Length;												// Probe length in mm
	qword					ID;														// Unique 64bits probe ID, comes from chipset embed in probe
	eProbeType		ProbeType;										// Probe type, see eProbeType enum for a list of available types
	wchar_t				TypeName[LEN_PROBE_TYPE+1];		// Readable probe type
	wchar_t				Name[LEN_PROBE_NAME+1];				// Probe name
	SYSTEMTIME		CreationDateTime;							// Says when probe has been created in arm specs
	SYSTEMTIME		UpdateDateTime;								// Indicates last time probe specs have been changed (i.e. probe calibration)
	float					CalculationResult;						// Result of probe calculation. Warning! It's not a result in millimeters.
	dword					StatusBits;										// It's a combination of eProbeStatusBits enum bits
	int						Index;												// Probe index: -1 if no probe selected, 0 for first probe, 1 for second probe and so on
	dword					PLOC;													// TKJ connector gender on probe side: 0 = female TKJ, 1 = male TKJ. 
	double				b1rx;													/// \note BJF 2010\3\3 - added b1rx and b2rx to probe data for DOCS.
	double				b2rx;													/// \note BJF 2010\3\3 - added b1rx and b2rx to probe data for DOCS.
	dword					PCOD;													/// \note JPDL 05/10/2010 : Internal probe attributes
	double				XDeviation, YDeviation;				/// \note JPDL 06/10/2010 : Probe center X and Y deviations
	int						SubType;											/// \note JPDL 2012-06-28 : Meaning depends on probe connected. Today only useful for Hexagon scanners. -1 by default.
	wchar_t				PartNumber[LEN_PROBE_PART_NUMBER+1];		/// \note JPDL 2012-09-13 Part Number
	dword					Version;											/// \note JPDL 2012-09-13 Probe version. Tube probe: 0=First version (2009?), 1=2012 version
	float					Size;													/// \note JPDL 2012-09-13 Can be tube size or probe diameter. This information will be displayed on RDS screens
	dword					Capabilities;									/// \note JPDL 2012-09-13 Indicates some probe capabilities. See eProbeCapability enum.
	eRDSCalibMethod	CalibrationMethod;																		/// \note JPDL 2015-09-17 Method that has been used to qualify the probe
	float						CalculationResults[MAX_PROBE_CALCULATION_RESULTS];		/// \note JPDL 2015-09-17 Detailed calculation results. Its content depends on probe type and method used
																																				/// See eCalibrationDetailedResult enum

	byte					Reserved[64];								// Reserved for further usage

	void Init(void)	{	memset(this, 0, sizeof(*this)); BallDiameter = 0; Length = 0; CalculationResult = 0; Index = -1; b1rx = b2rx = 0; XDeviation = YDeviation = 0; SubType=-1; }

	bool IsAScanner(void) { return (ProbeType == PT_HEXAGON_SCANNER) || (ProbeType == PT_CMS_SCANNER) || (ProbeType == PT_T_SCAN) || (ProbeType == PT_LAS); }
	bool IsAContactProbe(void) { return (ProbeType == PT_TRIGGER_PROBE) || (ProbeType == PT_CONTACT) || (ProbeType == PT_REFLECTOR) || (ProbeType == PT_T_MAC) || (ProbeType == PT_T_PROBE); }
	bool IsANonContactProbe(void) { return (ProbeType == PT_V); }
	bool IsACustomProbe(void) { return (PCOD & PCustBit) != 0; }
	bool IsANonRepeatableProbe(void) { return (PCOD & PNonRepeatableBit) != 0; }		/// \note JPDL 2011-12-14 Returns true if the probe is a non repeatable one (ie Kronos screwed probe)
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSProbeW, 564)
#endif


#ifdef RDS_API_UNICODE
	typedef struct sRDSProbeW	sRDSProbe;
#else
	typedef struct sRDSProbeA	sRDSProbe;
#endif


// **************************************************************************************************************************************************************************


/// This structure describes the reference event that has occurred.
/// It's used with RDSCI_REFERENCE_EVENT chunk. This structure is sent from service to client every a reference event occurs
/// Size = 528 bytes
struct sRDSReferenceA
{
	qword			ID;																		// Unique 64bits
	bool			Engaged;															// 1 = true
	eRefType	RefType;															// Referencing tool type, see eRefType enum for a list of available types
	char			PartName[MAX_PART_NAME_LENGTH + 1];		// TooLOK part name
	byte			Reserved[447];												// Reserved for further usage

	void Init(void)	{memset(this, 0, sizeof(*this)); }
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSReferenceA, 528)
#endif


/// Size = 528 bytes
struct sRDSReferenceW
{
	qword			ID;																		// Unique 64bits
	bool			Engaged;															// 1 = true
	eRefType	RefType;															// Referencing tool type, see eRefType enum for a list of available types
	wchar_t		PartName[MAX_PART_NAME_LENGTH + 1];		// TooLOK part name
	byte			Reserved[382];												// Reserved for further usage

	void Init(void)	{	memset(this, 0, sizeof(*this));}
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSReferenceW, 528)
#endif


#ifdef RDS_API_UNICODE
	typedef struct sRDSReferenceW	sRDSReference;
#else
	typedef struct sRDSReferenceA	sRDSReference;
#endif


/// This structure is sent from service to client every time a position has been captured from arm
/// Used with RDSCI_POSITION chunk
/// Be sure to analyze EventType to know if event is valid (AND operation with EVENT_SYNC value)
/// Size = 300 bytes
struct sRDSPosition
{
	dword		EventType;											// Event type. Combination of eRDSEvent enum.
	dword		RecordTime;											// 24 bits value, in milliseconds. Value wraps around to zero every 46 minutes
	double	ProbeXYZ[3];										// Probe XYZ position (ie. probe ball center)
	double	ProbeMatrix[3][3];							// Probe matrix (Rows=XYZ, Columns=IJK)
	double	ArmXYZ[3];											// Arm XYZ position (at probe attachment)
	double	ArmMatrix[3][3];								// Arm matrix (Rows=XYZ, Columns=IJK)
	double	Angles[RDS_MAX_AXIS];						// Calculated angles
	dword		VertexID;												// ID captured by Vertex probe
	dword		AnalogValues[3];								// Analog values (UDTX, UDTY, PLA)
	dword		NbLatches;											// Number of latches used to calculate XYZ position
	bool		ButtonsStates[3];								// Status of the 3 buttons
	bool		ExternalSwitchState;						// =TRUE if external switch is ON
	dword		TriggerIndex;										/// \note JPDL 24/06/2009 : Index of trigger signal received by arm and associated to this arm position.
																					/// Mainly needed for scanning. Starts at 1. 0 means no trigger associated
	double	ProbeMovementSpeed;							/// \note JPDL 13/08/2009 : Computed speed at ProbeXYZ. mm/s

	void Init(void)	{	memset(this, 0, sizeof(*this));	}
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSPosition, 300)
#endif

/// Extended data when a scanner is enabled
/// Size = 292 bytes
struct sRDSScannerHeader
{
	dword		NbPoints;								// Number of sRDSScannerPoint structures following this header
	double	CameraVector[3];				// Not used yet
	float		ExposureTime;						// Current exposure time in ms
	float		LaserPower;							// Current laser power
	double	AverageScannerXYZ[3];		/// \note JPDL 01/07/2009 : Coordinates of average point in scanner referential
	double	AverageSystemXYZ[3];		/// \note JPDL 01/07/2009 : Coordinates of average point in system (arm) referential
	byte		Reserved[208];					// Reserved for further usage

	void Init(void)	{ memset(this, 0, sizeof(*this));	}
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSScannerHeader, 292)
#endif


/// Extended data when a scanner is enabled. Sent just after sRDSScannerHeader
/// Size = 96 bytes
typedef struct _sRDSScannerPoint
{
	double	ScannerXYZ[3];				// Coordinates of scanner points in scanner referential
	double	SystemXYZ[3];					// Coordinates of scanner points in system (arm) referential
	byte		Reserved[48];					// Reserved for further usage

	void Init(void)	{ memset(this, 0, sizeof(*this));	}
}sRDSScannerPoint;

RDS_CHECK_STRUCT_SIZE(sRDSScannerPoint, 96)



/// This structure is sent to service when client wants to play a custom sound or a predefined melody
/// Used with RDSCI_SOUND chunk
/// Size = 84 bytes
struct sRDSSound
{
	eRDSSounds		Sound;									// See eRDSSounds for a list of available melodies
	bool					bStored;								// If TRUE, Sound is the index of sound to play
																				// If FALSE, Frequency, Duration and Volume are used
	UINT					Frequency;							
	UINT					Duration;								
	UINT					Volume;			

	/// \note JPDL 06/05/2009 : New parameter
	eRDSPlaySoundPriority	Priority;				// Describes when to play a custom sound

	byte					Reserved[60];						// Reserved for further usage

	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSSound, 84)


/// Used with RDSCI_FORCE_PROBE chunk
/// Size = 72 bytes
struct sRDSForceProbe
{
	qword	ProbeID;
	byte	Reserved[64];

	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSForceProbe, 72)


/// Size = 260 bytes
typedef struct _sRDSVideoCaptureGrabFrame
{
	eRDSVideoCaptureFrameDestination	Destination;

	byte	byReserved[256];

	void Init(void) { memset(this, 0, sizeof(*this)); }
}sRDSVideoCaptureGrabFrame;

RDS_CHECK_STRUCT_SIZE(sRDSVideoCaptureGrabFrame, 260)


/// \note JPDL 03/18/2009 : Added in RDS API v2.0
/// Size = 264 bytes
typedef struct _sRDSVideoCaptureFrame
{
	eRDSVideoCaptureImageFormat	ImageFormat;
	dword												dwFrameSize;				// Size of frame following this structure

	byte		byReserved[256];

	void Init(void) { memset(this, 0, sizeof(*this)); }
}sRDSVideoCaptureFrame;

RDS_CHECK_STRUCT_SIZE(sRDSVideoCaptureFrame, 264)

/// This structure is sent from service to client every time camera specs are requested
/// Used with RDSCI_POSITION chunk
/// Size = 256 bytes
struct sRDSCamera
{
	float TranslateAxisXYZ[3];
	float RotateAxisXYZ[3];
	float FocalLengthXY[2];
	float PixelSkew;
	float LensOffsetUV[2];
	float LensK1;
	float LensK2;
	float ImageOffsetXY[2];

	byte  byReserved[196];

	void Init(void)	{	memset(this, 0, sizeof(*this));	}
};

#ifndef UNDER_CE
RDS_CHECK_STRUCT_SIZE(sRDSCamera, 256)
#endif

// Used with RDSSET_SCANNING_MAX_KEYSTONE and RDSSET_SCANNING_KEYSTONE
typedef struct _sRDSKeyStone
{
	double	XLeft, XRight;
	double	ZNear, ZFar;
	byte		Reserved[16];
}sRDSKeyStone;

RDS_CHECK_STRUCT_SIZE(sRDSKeyStone, 48)

// Used with RDSSET_SCANNING_MAX_KEYSTONE_V2 and RDSSET_SCANNING_KEYSTONE_V2
typedef struct _sRDSKeyStoneV2
{
	bool					bValid;
	eKeyStoneMode	KeyStoneMode;

	double				dLeftTop[2], dRightTop[2];
	double				dLeftBottom[2], dRightBottom[2];

	// Binding between 2D orientations and coordinates coming from scanner
	eAxisOrientation	LeftOri, RightOri, TopOri, BottomOri;

	byte		Reserved[16];

	void Init(void) 
	{ 
		memset(this, 0, sizeof(*this)); 
		KeyStoneMode = KSM_UNKNOWN;

		dLeftTop[0] = dLeftTop[1] = dRightTop[0] = dRightTop[1] = 0;
		dLeftBottom[0] = dLeftBottom[1] = dRightBottom[0] = dRightBottom[1] = 0;

		LeftOri = RightOri = TopOri = BottomOri = AO_UNKNOWN;
	}

}sRDSKeyStoneV2;

RDS_CHECK_STRUCT_SIZE(sRDSKeyStoneV2, 104)

typedef struct _sRDSAvailableValue
{
	int			iValue;
	WCHAR		sName[LEN_AVAILABLE_VALUE_NAME+1];
	byte		Reserved[28];
}sRDSAvailableValue;

RDS_CHECK_STRUCT_SIZE(sRDSAvailableValue, 96)

/// Used with RDSCI_GET_SETTING and RDSCI_UPDATE_SETTING chunks
/// Size = 264 bytes
struct sRDSSettingA
{
	eRDSSetting	ID;															// Setting we want to retrieve or update
	char			Value[LEN_SETTING_VALUE+1];				// Check eRDSSetting enum comment for field meaning
	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSSettingA, 264)


/// Size = 520 bytes
struct sRDSSettingW
{
	eRDSSetting	ID;															// Setting we want to retrieve or update
	wchar_t			Value[LEN_SETTING_VALUE+1];			// Check eRDSSetting enum comment for field meaning
	
	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSSettingW, 520)


#ifdef RDS_API_UNICODE
	typedef struct sRDSSettingW	sRDSSetting;
#else
	typedef struct sRDSSettingA	sRDSSetting;
#endif


	
// **************************************************************************************************************************************************************************
// ******************************************************************* Commonly used macros *********************************************************************************
// **************************************************************************************************************************************************************************



/// Sends one to three structures to other partner and waits for result
#define RDS_SEND_STRUCTS																																\
	LRESULT	RDS_SendStructs(HWND hWnd, LPVOID pArg1, DWORD dwArg1Size,										\
													LPVOID pArg2=NULL, DWORD dwArg2Size=0,												\
													LPVOID pArg3=NULL, DWORD dwArg3Size=0)												\
	{																																											\
		DWORD	dwSize = dwArg1Size + dwArg2Size + dwArg3Size;																\
																																												\
		if(hWnd == 0)																																				\
			return RDSRC_INVALID_HANDLE;																											\
																																												\
		if(pArg1==NULL || dwArg1Size==0)																										\
			return RDSRC_INVALID_PARAMETER;																										\
																																												\
		char* pBuffer = new char[dwSize];																										\
		char* pCurrent = pBuffer; 																													\
																																												\
		memcpy(pCurrent, pArg1, dwArg1Size); 																								\
		pCurrent += dwArg1Size;																															\
																																												\
		if(pArg2)																																						\
		{																																										\
			memcpy(pCurrent, pArg2, dwArg2Size);																							\
			pCurrent += dwArg2Size;																														\
																																												\
			if(pArg3)																																					\
			{																																									\
				memcpy(pCurrent, pArg3, dwArg3Size);																						\
				pCurrent += dwArg3Size;																													\
			}																																									\
		}																																										\
																																												\
		COPYDATASTRUCT dStruct;																															\
		dStruct.dwData = 0;																																	\
		dStruct.cbData = dwSize;																														\
		dStruct.lpData = pBuffer;																														\
		LRESULT lresult = ::SendMessage(hWnd, WM_COPYDATA, (WPARAM)0, (LPARAM) &dStruct);		\
																																												\
		delete [] pBuffer;																																	\
																																												\
		if(lresult == FALSE)																																\
			return RDSRC_EXECUTION_FAILED;																										\
		else																																								\
			return RDSRC_SUCCESS;																															\
	}																																											


/// Sends a request to other partner and waits for result
#define RDS_SEND_REQUEST																																	\
	LRESULT	RDS_SendRequest(eRDSChunkID ChunkID, LPVOID pArg1=NULL, DWORD dwArg1Size=0,			\
													long lChunkParameter1=0, long lChunkParameter2=0)								\
	{																																												\
		sRDSChunk chunk;																																			\
		sRDSClient client;																																		\
																																													\
		chunk.Init();																																					\
		chunk.ChunkID = (eRDSChunkID)ChunkID;																									\
		chunk.Parameter1 = lChunkParameter1;																									\
		chunk.Parameter2 = lChunkParameter2;																									\
		chunk.ChunkSize = sizeof(client) + dwArg1Size;																				\
																																													\
		client.Init();																																				\
		client.hwnd = (RDSHANDLE) m_hWnd;																											\
																																													\
		return RDS_SendStructs(m_hRDSWindow, &chunk, sizeof(chunk), &client, sizeof(client), pArg1, dwArg1Size);	\
	}

#define RDS_MSGS \
	ON_REGISTERED_MESSAGE(m_iMyPrivateWindowMessage1, OnServerAvailable) \
	ON_WM_COPYDATA() 


#define RDS_ADD_PRIVATE_MESSAGES \
	const UINT m_iMyPrivateWindowMessage1 = RegisterWindowMessage(RDS_MSG_1);

#define RDS_INIT_VARS																											\
	m_bRDSServerAvailable = FALSE;																					\
	m_bRDSConnected = FALSE;																								\
	memset(&m_stConnectionDateTime, 0, sizeof(m_stConnectionDateTime));			\
	memset(&m_stLastMessageDateTime, 0, sizeof(m_stLastMessageDateTime));		\
	m_uiNumberOfMessagesReceived = 0;																				\
	m_hRDSWindow = 0;																												

/// \note JPDL 2012-01-16 Since 3.4, RDSService is ran with no admin credentials. On Vista and Win7 operating systems, user Interface Privilege Isolation (UIPI) prevents processes 
/// from sending selected window messages and other USER APIs to processes running with higher integrity. The result is that RDSService was not able to send messages to admin applications.
/// To solve this potential issue, RDS_ACCEPT_RDS_MESSAGES is automatically called by RDS_INITIALIZE and RDS_INITIALIZE_EXT macros
/// RDS_ACCEPT_RDS_MESSAGES macro will allow 3rd party applications to receive RDSService messages even they are ran with admin credentials
/// \note JPDL 2016-02-29 GetVersionEx has been deprecated. Test of Windows Vista or greater has been removed, indeed RDS and its API doesn't support Windows XP anymore
#define RDS_ACCEPT_RDS_MESSAGES																																																	\
{																																																																\
	HMODULE hDLL = LoadLibrary(_T("user32.dll"));																																					    \
	if(hDLL != NULL)																																																					\
	{																																																													\
		typedef BOOL (_stdcall* tChangeWindowMessageFilter)(UINT message, DWORD dwFlag);																				\
		tChangeWindowMessageFilter	pChangeWindowMessageFilter;																																	\
		pChangeWindowMessageFilter = (tChangeWindowMessageFilter)GetProcAddress(hDLL, "ChangeWindowMessageFilter");							\
		if(pChangeWindowMessageFilter != NULL)																																									\
		{																																																												\
			if(pChangeWindowMessageFilter(WM_COPYDATA, MSGFLT_ADD) != FALSE)																											\
				pChangeWindowMessageFilter(m_iMyPrivateWindowMessage1, MSGFLT_ADD);																									\
		}																																																												\
																																																														\
		FreeLibrary(hDLL);																																																			\
	}																																																															\
}

/// By default, API will try to connect to server if available
#define RDS_INITIALIZE																										\
	RDS_INIT_VARS;																													\
	RDS_ACCEPT_RDS_MESSAGES;																								\
	m_uiExtendedData = 0;																										\
	memset(&m_ucAuthenticationKey[0], 0, sizeof(m_ucAuthenticationKey));		\
	m_bRDSInitOK = TRUE;																										\
	m_bRDSCommEnabled = TRUE;

/// By default, API will try to connect to server if available
/// \note JPDL 06/19/2008, RDS API v0.3.3 : New macro
/// \note JPDL 16/04/2009 : m_uiExtendedData init missing
#define RDS_INITIALIZE_EXT(AuthenticationKey)																									\
	RDS_INIT_VARS;																																							\
	RDS_ACCEPT_RDS_MESSAGES;																																		\
	m_uiExtendedData = 0;																																				\
	memcpy(&m_ucAuthenticationKey[0], &AuthenticationKey[0], sizeof(m_ucAuthenticationKey));		\
	m_bRDSInitOK = TRUE;																																				\
	m_bRDSCommEnabled = TRUE;

/// \note JPDL 29/05/2009 : New variables m_bRDSInitOK and m_bRDSCommEnabled
#define RDS_ADD_VARIABLES																																	\
	RDS_SEND_REQUEST																																				\
	RDS_SEND_STRUCTS																																				\
	LRESULT OnServerAvailable(WPARAM w, LPARAM l);																					\
	void		CheckState(void);																																\
	LRESULT RemoveFromRDS(void);																														\
	virtual BOOL OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult);		\
	BOOL		OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct);												\
	void RDS_SYNCED_DATA(sRDSPosition& p);																									\
	void RDS_UNSYNCED_DATA(sRDSPosition& p);																								\
	void RDS_PROBE_DATA(sRDSProbe& p);																											\
	void RDS_REFERENCE_DATA(sRDSReference& p); \
	void RDS_ARM_DATA(sRDSArm& a);																													\
	void RDS_SCANNER_DATA(sRDSScannerHeader* pHeader, sRDSScannerPoint* pPoints);						\
	void RDS_SERVER_DOWN(void);																															\
	void RDS_SERVER_UP(void);																																\
	void RDS_ON_RESULT(eRDSChunkID RequestID, eRDSReturnCode ReturnCode, void* lpResult, dword dwResultSize);\
	void RDS_VIDEO_CAPTURE_FRAME(sRDSVideoCaptureFrame* pFrame, byte* pFrameData);					\
	void RDS_CAMERA_SPECS(sRDSCamera& c);																													\
	BOOL				m_bRDSConnected;										\
	HWND				m_hRDSWindow;												\
	BOOL				m_bRemoteMouseIsOn;									\
	SYSTEMTIME	m_stConnectionDateTime;							\
	SYSTEMTIME	m_stLastMessageDateTime;						\
	UINT				m_uiNumberOfMessagesReceived;				\
	UINT				m_uiExtendedData;										\
	uchar				m_ucAuthenticationKey[8];						\
	BOOL				m_bRDSInitOK;												\
	BOOL				m_bRDSCommEnabled;									\
	BOOL				m_bRDSServerAvailable;							\
public:


// **************************************************************************************************************************************************************************
// *************************************************************** Callbacks management - Internal use  *********************************************************************
// **************************************************************************************************************************************************************************


/// OnCopyData is called every time RDS Service sends a message to client. 
/// OnCopyData is responsible for analyzing this message and call the right user function.
/// By using this macro, you let RDS Service determine if the position is a measurement. You don't need to check arm status (buttons for instance)
/// to know if the position has to been saved or not.
/// It means every time RDS_SYNCED_DATA is called, you can store received position to your set of measurements.
///
/// We strongly recommend the use of this macro.
///
#define RDS_ON_COPY_DATA_EASY(Class, Base)																\
BOOL Class::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct)				\
{																																					\
	if(pCopyDataStruct->cbData != 0)																				\
	{																																				\
		char* p = new char[pCopyDataStruct->cbData];													\
		char* pcur = p;																												\
		memcpy(p, pCopyDataStruct->lpData, pCopyDataStruct->cbData);					\
																																					\
		sRDSChunk chunk;																											\
		memcpy(&chunk, pcur, sizeof(chunk));		pcur += sizeof(chunk);				\
																																					\
		GetLocalTime(&m_stLastMessageDateTime);																\
		m_uiNumberOfMessagesReceived++;																				\
																																					\
		switch(chunk.ChunkID)																									\
		{																																			\
			case RDSCI_POSITION:																								\
				sRDSPosition pos;																									\
				memcpy(&pos, &p[sizeof(chunk)], sizeof(pos));											\
				if(pos.EventType&EVENT_EASY)																			\
					RDS_SYNCED_DATA(pos);																						\
				else																															\
					RDS_UNSYNCED_DATA(pos);																					\
				break;																														\
																																					\
			case RDSCI_PROBE:																										\
				sRDSProbe probe;																									\
				memcpy(&probe, &p[sizeof(chunk)], sizeof(probe));									\
				RDS_PROBE_DATA(probe);																						\
				break;																														\
																																					\
			case RDSCI_ARM:																											\
				sRDSArm arm;																											\
				memcpy(&arm, &p[sizeof(chunk)], sizeof(arm));											\
				RDS_ARM_DATA(arm);																								\
				break;																														\
																																					\
			case RDSCI_SCANNER_DATA:																						\
			{																																		\
				sRDSScannerHeader* pHeader = (sRDSScannerHeader*)pcur;						\
				pcur += sizeof(sRDSScannerHeader);																\
				sRDSScannerPoint* pPoints = (sRDSScannerPoint*)pcur;							\
				RDS_SCANNER_DATA(pHeader, pPoints);																\
				break;																														\
			}																																		\
																																					\
			case RDSCI_RESULT:																									\
			{																																		\
				dword dwResultSize = pCopyDataStruct->cbData - sizeof(chunk);			\
																																					\
				RDS_ON_RESULT((eRDSChunkID)chunk.Parameter1,											\
											(eRDSReturnCode)chunk.Parameter2,										\
											(dwResultSize == 0) ? NULL : (void*)pcur,						\
											pCopyDataStruct->cbData - sizeof(chunk));						\
				break;																														\
			}																																		\
																																					\
			case RDSCI_SERVER_SHUTTING_DOWN:																		\
				RDS_INIT_VARS;																										\
				RDS_SERVER_DOWN();																								\
				break;																														\
		}																																			\
		delete[] p;																														\
	}																																				\
																																					\
	return Base::OnCopyData(pWnd, pCopyDataStruct);													\
}


/// Same as RDS_ON_COPY_DATA_EASY macro but we added the support of integrated RA7 arm USB camera 

#define RDS_ON_COPY_DATA_EASY_V2(Class, Base)															\
BOOL Class::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct)				\
{																																					\
	if(pCopyDataStruct->cbData != 0)																				\
	{																																				\
		char* p = new char[pCopyDataStruct->cbData];													\
		char* pcur = p;																												\
		memcpy(p, pCopyDataStruct->lpData, pCopyDataStruct->cbData);					\
																																					\
		sRDSChunk chunk;																											\
		memcpy(&chunk, pcur, sizeof(chunk));		pcur += sizeof(chunk);				\
																																					\
		GetLocalTime(&m_stLastMessageDateTime);																\
		m_uiNumberOfMessagesReceived++;																				\
																																					\
		switch(chunk.ChunkID)																									\
		{																																			\
			case RDSCI_POSITION:																								\
				sRDSPosition pos;																									\
				memcpy(&pos, &p[sizeof(chunk)], sizeof(pos));											\
				if(pos.EventType&EVENT_EASY)																			\
					RDS_SYNCED_DATA(pos);																						\
				else																															\
					RDS_UNSYNCED_DATA(pos);																					\
				break;																														\
																																					\
			case RDSCI_PROBE:																										\
				sRDSProbe probe;																									\
				memcpy(&probe, &p[sizeof(chunk)], sizeof(probe));									\
				RDS_PROBE_DATA(probe);																						\
				break;																														\
																																					\
			case RDSCI_ARM:																											\
				sRDSArm arm;																											\
				memcpy(&arm, &p[sizeof(chunk)], sizeof(arm));											\
				RDS_ARM_DATA(arm);																								\
				break;																														\
																																					\
			case RDSCI_SCANNER_DATA:																						\
			{																																		\
				sRDSScannerHeader* pHeader = (sRDSScannerHeader*)pcur;						\
				pcur += sizeof(sRDSScannerHeader);																\
				sRDSScannerPoint* pPoints = (sRDSScannerPoint*)pcur;							\
				RDS_SCANNER_DATA(pHeader, pPoints);																\
				break;																														\
			}																																		\
																																					\
			case RDSCI_VIDEO_CAPTURE_FRAME:																			\
			{																																		\
				sRDSVideoCaptureFrame* pHeader = (sRDSVideoCaptureFrame*)pcur;		\
				pcur += sizeof(sRDSVideoCaptureFrame);														\
				byte* pFrame = (byte*)pcur;																				\
				RDS_VIDEO_CAPTURE_FRAME(pHeader, pFrame);													\
				break;																														\
			}																																		\
																																					\
			case RDSCI_RESULT:																									\
			{																																		\
				dword dwResultSize = pCopyDataStruct->cbData - sizeof(chunk);			\
																																					\
				RDS_ON_RESULT((eRDSChunkID)chunk.Parameter1,											\
											(eRDSReturnCode)chunk.Parameter2,										\
											(dwResultSize == 0) ? NULL : (void*)pcur,						\
											pCopyDataStruct->cbData - sizeof(chunk));						\
				break;																														\
			}																																		\
																																					\
			case RDSCI_SERVER_SHUTTING_DOWN:																		\
				RDS_INIT_VARS;																										\
				RDS_SERVER_DOWN();																								\
				break;																														\
		}																																			\
		delete[] p;																														\
	}																																				\
																																					\
	return Base::OnCopyData(pWnd, pCopyDataStruct);													\
}


/// Same as RDS_ON_COPY_DATA_EASY_v2 macro but we added the support of references

#define RDS_ON_COPY_DATA_EASY_V2_2(Class, Base)														\
BOOL Class::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct)				\
{																																					\
	if(pCopyDataStruct->cbData != 0)																				\
	{																																				\
		char* p = new char[pCopyDataStruct->cbData];													\
		char* pcur = p;																												\
		memcpy(p, pCopyDataStruct->lpData, pCopyDataStruct->cbData);					\
																																					\
		sRDSChunk chunk;																											\
		memcpy(&chunk, pcur, sizeof(chunk));		pcur += sizeof(chunk);				\
																																					\
		GetLocalTime(&m_stLastMessageDateTime);																\
		m_uiNumberOfMessagesReceived++;																				\
																																					\
		switch(chunk.ChunkID)																									\
		{																																			\
			case RDSCI_POSITION:																								\
				sRDSPosition pos;																									\
				memcpy(&pos, &p[sizeof(chunk)], sizeof(pos));											\
				if(pos.EventType&EVENT_EASY)																			\
					RDS_SYNCED_DATA(pos);																						\
				else																															\
					RDS_UNSYNCED_DATA(pos);																					\
				break;																														\
																																					\
			case RDSCI_PROBE:																										\
				sRDSProbe probe;																									\
				memcpy(&probe, &p[sizeof(chunk)], sizeof(probe));									\
				RDS_PROBE_DATA(probe);																						\
				break;																														\
																																					\
			case RDSCI_REFERENCE_EVENT:																					\
				sRDSReference ref;																								\
				memcpy(&ref, &p[sizeof(chunk)], sizeof(ref));											\
				RDS_REFERENCE_DATA(ref);																					\
				break;																														\
																																					\
			case RDSCI_ARM:																											\
				sRDSArm arm;																											\
				memcpy(&arm, &p[sizeof(chunk)], sizeof(arm));											\
				RDS_ARM_DATA(arm);																								\
				break;																														\
																																					\
			case RDSCI_SCANNER_DATA:																						\
			{																																		\
				sRDSScannerHeader* pHeader = (sRDSScannerHeader*)pcur;						\
				pcur += sizeof(sRDSScannerHeader);																\
				sRDSScannerPoint* pPoints = (sRDSScannerPoint*)pcur;							\
				RDS_SCANNER_DATA(pHeader, pPoints);																\
				break;																														\
			}																																		\
																																					\
			case RDSCI_VIDEO_CAPTURE_FRAME:																			\
			{																																		\
				sRDSVideoCaptureFrame* pHeader = (sRDSVideoCaptureFrame*)pcur;		\
				pcur += sizeof(sRDSVideoCaptureFrame);														\
				byte* pFrame = (byte*)pcur;																				\
				RDS_VIDEO_CAPTURE_FRAME(pHeader, pFrame);													\
				break;																														\
			}																																		\
																																					\
			case RDSCI_RESULT:																									\
			{																																		\
				dword dwResultSize = pCopyDataStruct->cbData - sizeof(chunk);			\
																																					\
				RDS_ON_RESULT((eRDSChunkID)chunk.Parameter1,											\
											(eRDSReturnCode)chunk.Parameter2,										\
											(dwResultSize == 0) ? NULL : (void*)pcur,						\
											pCopyDataStruct->cbData - sizeof(chunk));						\
				break;																														\
			}																																		\
																																					\
			case RDSCI_SERVER_SHUTTING_DOWN:																		\
				RDS_INIT_VARS;																										\
				RDS_SERVER_DOWN();																								\
				break;																														\
																																					\
		}																																			\
		delete[] p;																														\
	}																																				\
																																					\
	return Base::OnCopyData(pWnd, pCopyDataStruct);													\
}

/// OnCopyData is called every time RDS Service sends a message to client. 
/// OnCopyData is responsible for analysing this message and call the right user function.
/// Use this version of macro if you want to receive via RDS_SYNCED_DATA function all reliable positions 
/// even an arm button has been pressed by user or not. Using this macro, you'll be responsible for determining 
/// if received position is interesting for you or not, by analyzing EventType flags.
///
/// We strongly suggest the use of RDS_ON_COPY_DATA_EASY macro instead.
///
#define RDS_ON_COPY_DATA(Class, Base)																			\
BOOL Class::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct)				\
{ 																																				\
	if(pCopyDataStruct->cbData != 0) 																				\
	{ 																																			\
		char* p = new char[pCopyDataStruct->cbData];													\
		char* pcur = p;																												\
		memcpy(p, pCopyDataStruct->lpData, pCopyDataStruct->cbData);					\
																																					\
		sRDSChunk chunk;																											\
		memcpy(&chunk, pcur, sizeof(chunk));		pcur += sizeof(chunk);				\
																																					\
		GetLocalTime(&m_stLastMessageDateTime);																\
		m_uiNumberOfMessagesReceived++;																				\
																																					\
		switch(chunk.ChunkID)																									\
		{																																			\
			case RDSCI_POSITION:																								\
				sRDSPosition pos;																									\
				memcpy(&pos, pcur, sizeof(pos));																	\
				if(pos.EventType&EVENT_SYNC) 																			\
					RDS_SYNCED_DATA(pos); 																					\
				else 																															\
					RDS_UNSYNCED_DATA(pos); 																				\
				break; 																														\
																																					\
			case RDSCI_PROBE:																										\
				sRDSProbe probe;																									\
				memcpy(&probe, pcur, sizeof(probe));															\
				RDS_PROBE_DATA(probe);																						\
				break;																														\
																																					\
			case RDSCI_REFERENCE_EVENT:																					\
				sRDSReference ref;																								\
				memcpy(&ref, &p[sizeof(chunk)], sizeof(ref));											\
				RDS_REFERENCE_DATA(ref);																					\
				break;																														\
																																					\
			case RDSCI_ARM:																											\
				sRDSArm arm;																											\
				memcpy(&arm, pcur, sizeof(arm));																	\
				RDS_ARM_DATA(arm);																								\
				break;																														\
																																					\
			case RDSCI_SCANNER_DATA:																						\
			{																																		\
				sRDSScannerHeader* pHeader = (sRDSScannerHeader*)pcur;						\
				pcur += sizeof(sRDSScannerHeader);																\
				sRDSScannerPoint* pPoints = (sRDSScannerPoint*)pcur;							\
				RDS_SCANNER_DATA(pHeader, pPoints);																\
				break;																														\
			}																																		\
																																					\
			case RDSCI_RESULT:																									\
			{																																		\
				dword dwResultSize = pCopyDataStruct->cbData - sizeof(chunk);			\
																																					\
				RDS_ON_RESULT((eRDSChunkID)chunk.Parameter1,											\
											(eRDSReturnCode)chunk.Parameter2,										\
											(dwResultSize == 0) ? NULL : (void*)pcur,						\
											pCopyDataStruct->cbData - sizeof(chunk));						\
				break;																														\
			}																																		\
																																					\
			case RDSCI_SERVER_SHUTTING_DOWN:																		\
				RDS_INIT_VARS;																										\
				RDS_SERVER_DOWN();																								\
				break;																														\
		} 																																		\
		delete[] p;  																													\
	} 																																			\
																																					\
	return Base::OnCopyData(pWnd, pCopyDataStruct);													\
}																																					


// **************************************************************************************************************************************************************************
// ****************************************************************  Macros available for 3rd parties  **********************************************************************
// **************************************************************************************************************************************************************************


/// \note JPDL 06/25/2008 : Not used anymore, replaced with empty content for compatibility
#define RDS_ON_REMOVE_FROM_SERVER(ClassName)


/// \note JPDL 06/25/2008 : We execute directly request to avoid useless call to RemoveFromRDS
#define RDS_REMOVE_FROM_SERVER																									\
	if(m_bRDSConnected)																														\
	{																																							\
		KillTimer(RDS_TM_CHECK_STATE_ID);																						\
		RDS_SendRequest(RDSCI_DISCONNECT);																					\
	}																																							\
																																								\
	m_bRDSConnected = FALSE;																											\
	m_bRDSCommEnabled = FALSE;
						

/// \note JPDL 29/05/2009 : New macro to allow third party to reenable comm if RDS_REMOVE_FROM_SERVER is called
#define RDS_CONNECT_TO_SERVER																										\
	if(m_bRDSInitOK)																															\
	{																																							\
		m_bRDSCommEnabled = TRUE;																										\
	}


// If method is CM_UNKNOWN, RDS displays a dialog box of available methods regarding probe type. 
// By default, a method is selected in dialog box
// If x, y, width, height are zero, window will take all screen
#define RDS_CALIBRATE_PROBE(x, y, width, height, method)												\
{																																								\
	if(m_bRDSConnected)																														\
	{																																							\
		sRDSScript		script;																												\
		script.Init();																															\
		script.ScriptType = ST_PROBE_CALIBRATION;																		\
		script.X = x; script.Y = y; script.Width = width; script.Height = height;		\
		script.Method = method;																											\
																																								\
		RDS_SendRequest(RDSCI_EXECUTE_SCRIPT, &script, sizeof(script));							\
	}																																							\
}																																							


#define RDS_EXECUTE_SCRIPT(x, y, width, height, name)														\
{																																								\
	if(m_bRDSConnected)																														\
	{																																							\
		sRDSScript		script;																												\
		script.Init();																															\
		script.ScriptType = name;																										\
		script.X = x; script.Y = y; script.Width = width; script.Height = height;		\
																																								\
		RDS_SendRequest(RDSCI_EXECUTE_SCRIPT, &script, sizeof(script));							\
	}																																							\
}																																							


// This macro displays RDS Control Panel.
#define RDS_SETTINGS																													\
{																																							\
	if(m_bRDSConnected)																													\
		RDS_SendRequest(RDSCI_DISPLAY_CONTROL_PANEL);															\
}																																							

/// \note JPDL 10/08/2009 : Section to display added
#define RDS_SETTINGS_V2(section)																							\
{																																							\
	if(m_bRDSConnected)																													\
		RDS_SendRequest(RDSCI_DISPLAY_CONTROL_PANEL, NULL, 0, section);						\
}


// This macro executes specific arm control panel. For instance, if a Sigma is used, macro displays 
// GDS control panel.
#define RDS_ADVANCED_SETTINGS																									\
{																																							\
	if(m_bRDSConnected)																													\
		RDS_SendRequest(RDSCI_DISPLAY_ADVANCED_CONTROL_PANEL);										\
}																																							


#define RDS_REMOTE_MOUSE(Enabled)																							\
{																																							\
	if(m_bRDSConnected)																													\
		RDS_SendRequest(RDSCI_CONTROL_REMOTE_MOUSE, NULL, 0, Enabled);						\
}																																							


/// \note JPDL 30/11/2010 : Adds a way to set options during remote mouse
/// \param bits is a set of eRDSRemoteMouseBits bits
#define RDS_REMOTE_MOUSE_V2(Enabled, Bits)																		\
{																																							\
	if(m_bRDSConnected)																													\
		RDS_SendRequest(RDSCI_CONTROL_REMOTE_MOUSE, NULL, 0, Enabled, Bits);			\
}																																							


/// \note JPDL 11/25/2008 : ClassName::OnWndMsg : Don't forward our timer to parent window
/// \note JPDL 29/05/2009 : Check if init has been done, needed by RDS_REMOVE_FROM_SERVER (Jim Li)
#define RDS_ON_SERVER_AVAILABLE(ClassName, RemoteMouse)										\
LRESULT ClassName::OnServerAvailable(WPARAM wParam, LPARAM lParam)				\
{																																					\
																																					\
	m_bRDSServerAvailable = TRUE;																						\
																																					\
	if(m_bRDSConnected)																											\
		return RDSRC_SUCCESS;																									\
																																					\
	if(m_bRDSCommEnabled == FALSE)																					\
		return RDSRC_SUCCESS;																									\
																																					\
	if(m_bRDSInitOK == FALSE)																								\
		return RDSRC_NOT_INITIALIZED;																					\
																																					\
	m_hRDSWindow = (HWND)lParam;																						\
																																					\
	sRDSConnect connect;																										\
	uchar				ucPlainValue[8];																						\
	cDES				des;																												\
	SYSTEMTIME	now;																												\
																																					\
	GetLocalTime(&now);																											\
																																					\
	ucPlainValue[0] = (byte)now.wDay;																				\
	ucPlainValue[1] = (byte)now.wMonth;																			\
	ucPlainValue[2] = (byte)now.wHour;																			\
	ucPlainValue[3] = (byte)now.wMinute;																		\
	ucPlainValue[4] = 0;																										\
	ucPlainValue[5] = 0;																										\
	ucPlainValue[6] = 0;																										\
	ucPlainValue[7] = 0;																										\
																																					\
	connect.Init();																													\
	des.SetKey(m_ucAuthenticationKey, cDES::DESM_ENCRYPTION);								\
	des.Compute(ucPlainValue, connect.AuthenticationValue);									\
	connect.ExtendedData = m_uiExtendedData;																\
	connect.RDSAPIVersion = RDS_API_VERSION_VALUE;													\
																																					\
	LRESULT rc = RDS_SendRequest(RDSCI_CONNECT, &connect, sizeof(connect));	\
	if(rc != RDSRC_SUCCESS)																									\
		return rc;																														\
																																					\
	m_bRemoteMouseIsOn = (bool)RemoteMouse;																	\
	RDS_REMOTE_MOUSE(RemoteMouse)																						\
	m_bRDSConnected = TRUE;																									\
	GetLocalTime(&m_stConnectionDateTime);																	\
																																					\
	RDS_SERVER_UP();																												\
																																					\
	SetTimer(RDS_TM_CHECK_STATE_ID, RDS_TM_CHECK_STATE_TIMER, NULL);				\
																																					\
	return RDSRC_SUCCESS;																										\
}																																					\
																																					\
void ClassName::CheckState(void)																					\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		if(::IsWindow(m_hRDSWindow) == FALSE)																	\
		{																																			\
			RDS_INIT_VARS;																											\
			RDS_SERVER_DOWN();																									\
		}																																			\
	}																																				\
}																																					\
																																					\
BOOL ClassName::OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult)	\
{																																												\
	if(message == WM_TIMER && wParam == RDS_TM_CHECK_STATE_ID)														\
	{																																											\
		CheckState();																																				\
		return TRUE;																																				\
	}																																											\
																																												\
	return __super::OnWndMsg(message, wParam, lParam, pResult);														\
}
			

/// \note JPDL 09/04/2008 : Parameters fixed in RDS API v0.3.10				
//	Frequency (1 to 5631 Hz)
//	Duration (1 to 255), 1 unit = 4ms. 
//	Volume (0 to 100)
#define RDS_PLAY_SOUND(frequency, duration, volume)												\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSSound sound;																											\
		sound.Init();																													\
		sound.bStored = FALSE;																								\
		sound.Frequency = frequency;																					\
		sound.Duration = duration;																						\
		sound.Volume = volume;																								\
		sound.Priority = RDSPSP_PLAY_NOW;																			\
																																					\
		RDS_SendRequest(RDSCI_PLAY_MELODY, &sound, sizeof(sound));						\
	}																																				\
}


/// \note JPDL 06/05/2009 : priority added
//	Frequency (1 to 5631 Hz)
//	Duration (1 to 255), 1 unit = 4ms. 
//	Volume (0 to 100)
#define RDS_PLAY_SOUND_V2(frequency, duration, volume, priority)					\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSSound sound;																											\
		sound.Init();																													\
		sound.bStored = FALSE;																								\
		sound.Frequency = frequency;																					\
		sound.Duration = duration;																						\
		sound.Volume = volume;																								\
		sound.Priority = priority;																						\
																																					\
		RDS_SendRequest(RDSCI_PLAY_MELODY, &sound, sizeof(sound));						\
	}																																				\
}


#define RDS_MAKE_CANNED_SOUND(soundindex)																	\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSSound sound;																											\
		sound.Init();																													\
		sound.bStored = TRUE;																									\
		sound.Sound = soundindex;																							\
																																					\
		RDS_SendRequest(RDSCI_PLAY_MELODY, &sound, sizeof(sound));						\
	}																																				\
}


#define RDS_PLAY_MELODY(soundindex)																				\
	RDS_MAKE_CANNED_SOUND(soundindex);


/// \note JPDL 2012-09-05 priority added
#define RDS_MAKE_CANNED_SOUND_V2(soundindex, priority)										\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSSound sound;																											\
		sound.Init();																													\
		sound.bStored = TRUE;																									\
		sound.Sound = soundindex;																							\
		sound.Priority = priority;																						\
																																					\
		RDS_SendRequest(RDSCI_PLAY_MELODY, &sound, sizeof(sound));						\
	}																																				\
}


/// \note JPDL 2012-09-05 priority added
#define RDS_PLAY_MELODY_V2(soundindex, priority)													\
	RDS_MAKE_CANNED_SOUND_V2(soundindex, priority);


/// \note JPDL 10/10/2008 : Added in RDS API v0.4.7
#define RDS_GET_SETTING(id)																								\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_GET_SETTING, NULL, 0, (long)id);								\
	}																																				\
}


#ifdef RDS_API_UNICODE

	/// \note JPDL 07/07/2008 : Added in RDS API v0.3.4
	#define RDS_UPDATE_SETTING(id, value)																			\
	{																																					\
		if(m_bRDSConnected)																											\
		{																																				\
			sRDSSettingW setting;																									\
			setting.Init();																												\
			setting.ID = id;																											\
			wcscpy(setting.Value, value);																					\
																																						\
			RDS_SendRequest(RDSCI_UPDATE_SETTING, &setting, sizeof(setting));			\
		}																																				\
	}

#else

	#define RDS_UPDATE_SETTING(id, value)																			\
	{																																					\
		if(m_bRDSConnected)																											\
		{																																				\
			sRDSSettingA setting;																									\
			setting.Init();																												\
			setting.ID = id;																											\
			strcpy(setting.Value, value);																					\
																																						\
			RDS_SendRequest(RDSCI_UPDATE_SETTING, &setting, sizeof(setting));			\
		}																																				\
	}

#endif


/// \note JPDL 27/07/2010 : version 2 of RDS_GET_SETTING/RDS_UPDATE_SETTING macros
/// \note JPDL 27/07/2010 : Structure used with  RDSCI_UPDATE_SETTING_V2 and RDSCI_GET_SETTING_V2 chunks
struct sRDSSettingV2
{
	eRDSSetting	ID;															// Setting we want to retrieve or update
	int					iParam1;
	int					iParam2;
	eRDSSettingValueType	ReturnedValueType;		// Type of value returned by RDS and following this structure.
	byte				Reserved[32];

	void Init(void) { memset(this, 0, sizeof(*this)); }
};

RDS_CHECK_STRUCT_SIZE(sRDSSettingV2, 48)


/// \note JPDL 27/07/2010 : Added in RDS API v2.4.0.10
#define RDS_GET_SETTING_V2(id, param1, param2)														\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSSettingV2 Setting;																								\
		Setting.Init();																												\
		Setting.ID = id;																											\
		Setting.iParam1 = param1;																							\
		Setting.iParam2 = param2;																							\
		RDS_SendRequest(RDSCI_GET_SETTING_V2, &Setting, sizeof(Setting));			\
	}																																				\
}


/// \note JPDL 2012-05-29 RDS_UPDATE_SETTING_V2 macro: param1 and param2 were missing
/// \note JPDL 27/07/2010 : Added in RDS API v2.4.0.10
#define RDS_UPDATE_SETTING_V2(id, param1, param2, pvalue, sizeofvalue)							\
{																																										\
	if(m_bRDSConnected)																																\
	{																																									\
		int iSizeOfStruct = sizeof(sRDSSettingV2)+sizeofvalue;													\
		sRDSSettingV2* psetting = (sRDSSettingV2*)new byte[iSizeOfStruct];							\
		psetting->Init();																																\
		psetting->ID = id;																															\
		psetting->iParam1 = param1;																												\
		psetting->iParam2 = param2;																												\
		memcpy(((byte*)psetting) + sizeof(sRDSSettingV2), (byte*)pvalue, sizeofvalue);	\
																																										\
		RDS_SendRequest(RDSCI_UPDATE_SETTING_V2, psetting, iSizeOfStruct);							\
	}																																									\
}


/// \note JPDL 10/10/2008 : Added in RDS API v0.4.7
#define RDS_GET_AVAILABLE_PROBES()																				\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_GET_AVAILABLE_PROBES);													\
	}																																				\
}


/// \note JPDL 10/27/2008 : Added in RDS API v0.4.8
#define RDS_RESET_ARM()																										\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_RESET_ARM);																			\
	}																																				\
}


/// \note JPDL 12/02/2008 : Added in RDS API v0.4.10
/// \param id = 64 bits id of probe to force. 0 = automatic mode
#define RDS_FORCE_PROBE(id)																								\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		sRDSForceProbe probe;																									\
		probe.Init();																													\
		probe.ProbeID = id;																										\
																																					\
		RDS_SendRequest(RDSCI_FORCE_PROBE, &probe, sizeof(probe));						\
	}																																				\
}


/// \note JPDL 01/07/2009 : Added in RDS API v0.4.12
#define RDS_VIDEO_CAPTURE_START()																					\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_VIDEO_CAPTURE_START);														\
	}																																				\
}


/// \note JPDL 01/07/2009 : Added in RDS API v0.4.12
#define RDS_VIDEO_CAPTURE_STOP()																					\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_VIDEO_CAPTURE_STOP);														\
	}																																				\
}


/// Use this function, if you want to receive all frames captured using video capture device
/// \note JPDL 01/07/2009 : Added in RDS API v2.0
#define RDS_VIDEO_CAPTURE_SUBSCRIBE()																			\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_VIDEO_CAPTURE_SUBSCRIBE);												\
	}																																				\
}


/// This macro stops subscription done by client before. Client will not receive grabbed frames anymore.
/// \note JPDL 01/07/2009 : Added in RDS API v2.0
#define RDS_VIDEO_CAPTURE_UNSUBSCRIBE()																		\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_VIDEO_CAPTURE_UNSUBSCRIBE);											\
	}																																				\
}


/// Ask service for a video capture frame
/// \note JPDL 01/07/2009 : Added in RDS API v0.4.12
#define RDS_VIDEO_CAPTURE_GRAB_FRAME(destination)																				\
{																																												\
	if(m_bRDSConnected)																																		\
	{																																											\
		sRDSVideoCaptureGrabFrame grabframe;																								\
		grabframe.Init();																																		\
		grabframe.Destination = destination;																								\
																																												\
		RDS_SendRequest(RDSCI_VIDEO_CAPTURE_GRAB_FRAME, &grabframe, sizeof(grabframe));			\
	}																																											\
}


/// Use this function if you want to receive arm data
/// \note JPDL 16/04/2009 : Added in RDS API v2.0.1
#define RDS_GET_ARM_DATA()																								\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_GET_ARM_DATA);																	\
	}																																				\
}


/// Use this function if you want to receive probe data
/// \note JPDL 16/04/2009 : Added in RDS API v2.0.1
#define RDS_GET_PROBE_DATA()																							\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_GET_PROBE_DATA);																\
	}																																				\
}


/// If available, resets internal arm trigger counter. Useful for scanning
/// \note JPDL 30/06/2009 : Added in RDS API v2.0.4
#define RDS_RESET_TRIGGER_COUNTER()																				\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_RESET_TRIGGER_COUNTER);													\
	}																																				\
}


/// \note JPDL 01/07/2010 : Switch on head light if available
#define RDS_HEAD_LIGHT_ON()																								\
{																																					\
	if(m_bRDSConnected) { RDS_SendRequest(RDSCI_HEAD_LIGHT_ON); }						\
}


/// \note JPDL 01/07/2010 : Switch off head light if available
#define RDS_HEAD_LIGHT_OFF()																							\
{																																					\
	if(m_bRDSConnected) { RDS_SendRequest(RDSCI_HEAD_LIGHT_OFF); }					\
}

/// Use this function if you want to receive camera spec.'s
/// \note DRR 2/11/2014 : Added in RDS API v?.?.?
#define RDS_GET_CAMERA_SPECS()																								\
{																																					\
	if(m_bRDSConnected)																											\
	{																																				\
		RDS_SendRequest(RDSCI_GET_CAMERA_SPECS);																	\
	}																																				\
}

/// \note JPDL 2012-09-13 RDS_FILTER_EVENTS allows 3rd party to disable/enable some events sent by RDS.
/// The corresponding melodies are also disabled.
/// Please note that today, this function only works with EVENT_BEAMxxx events.
/// By default, when connecting to an arm, tube events will be disabled by RDS. If no client is connected, tube events will be disabled again.
#define RDS_FILTER_SOUND_ON_EVENT(dwEvents, bEnable)																																\
{																																																						\
	if(m_bRDSConnected) { RDS_SendRequest(RDSCI_FILTER_SOUND_ON_EVENT, NULL, 0, (long)dwEvents, (long)bEnable); }		\
}


/// \note JPDL, 03/09/2007 : For compatibility with other environment like VB6
#pragma pack(pop)


/// Returns RDS API return code description
/// Include RDS_API_Tools.cpp in your project to use it
LPCTSTR	GetRDSAPIErrorDescription(eRDSReturnCode rc);

