/////////////////////////////////////////////////////////////////////////////
//	ArmManager.cpp
//
//		Make the interface between any of the KREON arm dll and your program.
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ArmManager.h"
#include <stdio.h>
#include <tchar.h>


/////////////////////////////////////////////////////////////////////////////
CArmManager::CArmManager(const TCHAR* DLLName)
{
	m_hModule			= NULL;
	m_bDllOk			= false;
	m_NbButtons			= -1;
	m_TriggerDuration	= 0xFFFF;
	m_DllName			= NULL;

	m_lpfnDllIsConnected						= NULL;
	m_lpfnDllBeginArm							= NULL;
	m_lpfnDllBeginArmWithCalibration			= NULL;
	m_lpfnDllEndArm								= NULL;
	m_lpfnDllBeginAcquisition					= NULL;
	m_lpfnDllGetAcquisitionMode					= NULL;
	m_lpfnDllEndAcquisition						= NULL;
	m_lpfnDllGetCurrentPosition					= NULL;
	m_lpfnDllGetCurrentPositionAndJointAngles	= NULL;
	m_lpfnDllGetTriggeredPosition				= NULL;
	m_lpfnDllGetTriggeredPositionAndJointAngles	= NULL;
	m_lpfnDllGetButtonState						= NULL;
	m_lpfnDllGetParameter						= NULL;
	m_lpfnDllSetParameter						= NULL;
	m_lpfnDllHasPropertiesWindow				= NULL;
	m_lpfnDllPropertiesWindow					= NULL;
	m_lpfnDllProbeCalibration					= NULL;
	m_lpfnDllOpenKreonArmProbesManagement		= NULL;
	m_lpfnDllGenerateKreonArmReport				= NULL;
	m_lpfnDllDeleteKreonArmProbeProfile			= NULL;
	m_lpfnDllUpdateKreonArmProbeProfile			= NULL;
	m_lpfnDllAddKreonArmProbeProfile			= NULL;

	//	Loads the DLL
	m_hModule = ::LoadLibraryEx(DLLName,NULL,LOAD_WITH_ALTERED_SEARCH_PATH);
	if (m_hModule)
	{
		size_t nameSize = _tcslen(DLLName)+1;
		m_DllName = new TCHAR[nameSize];
		_tcscpy_s(m_DllName,nameSize,DLLName);

		m_lpfnDllIsConnected						= (LPFNDLLISCONNECTED)							GetProcAddress(m_hModule,"IsConnected");
		m_lpfnDllBeginArm							= (LPFNDLLBEGINARM)								GetProcAddress(m_hModule,"BeginArm");
		m_lpfnDllBeginArmWithCalibration			= (LPFNDLLBEGINARMWITHCALIBRATION)				GetProcAddress(m_hModule,"BeginArmWithCalibration");
		m_lpfnDllEndArm								= (LPFNDLLENDARM)								GetProcAddress(m_hModule,"EndArm");
		m_lpfnDllBeginAcquisition					= (LPFNDLLBEGINACQUISITION)						GetProcAddress(m_hModule,"BeginAcquisition");
		m_lpfnDllGetAcquisitionMode					= (LPFNDLLGETACQUISITIONMODE)					GetProcAddress(m_hModule,"GetAcquisitionMode");
		m_lpfnDllEndAcquisition						= (LPFNDLLENDACQUISITION)						GetProcAddress(m_hModule,"EndAcquisition");
		m_lpfnDllGetCurrentPosition					= (LPFNDLLGETCURRENTPOSITION)					GetProcAddress(m_hModule,"GetCurrentPosition");
		m_lpfnDllGetCurrentPositionAndJointAngles	= (LPFNDLLGETCURRENTPOSITIONANDJOINTANGLES)		GetProcAddress(m_hModule,"GetCurrentPositionAndJointAngles");
		m_lpfnDllGetTriggeredPosition				= (LPFNDLLGETTRIGGEREDPOSITION)					GetProcAddress(m_hModule,"GetTriggeredPosition");
		m_lpfnDllGetTriggeredPositionAndJointAngles	= (LPFNDLLGETTRIGGEREDPOSITIONANDJOINTANGLES)	GetProcAddress(m_hModule,"GetTriggeredPositionAndJointAngles");
		m_lpfnDllGetButtonState						= (LPFNDLLGETBUTTONSTATE)						GetProcAddress(m_hModule,"GetButtonState");
		m_lpfnDllGetParameter						= (LPFNDLLGETPARAMETER)							GetProcAddress(m_hModule,"GetParameter");
		m_lpfnDllSetParameter						= (LPFNDLLSETPARAMETER)							GetProcAddress(m_hModule,"SetParameter");
		m_lpfnDllHasPropertiesWindow				= (LPFNDLLHASPROPERTIESWINDOW)					GetProcAddress(m_hModule,"HasPropertiesWindow");
		m_lpfnDllPropertiesWindow					= (LPFNDLLPROPERTIESWINDOW)						GetProcAddress(m_hModule,"PropertiesWindow");
		m_lpfnDllProbeCalibration					= (LPFNDLLPROBECALIBRATION)						GetProcAddress(m_hModule,"ProbeCalibration");
		m_lpfnDllOpenKreonArmProbesManagement		= (LPFNDLLOPENKREONARMPROBESMANAGEMENT)			GetProcAddress(m_hModule,"OpenKreonArmProbesManagement");
		m_lpfnDllGenerateKreonArmReport				= (LPFNDLLGENERATEKREONARMREPORT)				GetProcAddress(m_hModule,"GenerateKreonArmReport");
		m_lpfnDllDeleteKreonArmProbeProfile			= (LPFNDLLDELETEKREONARMPROBEPROFILE)			GetProcAddress(m_hModule,"DeleteKreonArmProbeProfile");
		m_lpfnDllUpdateKreonArmProbeProfile			= (LPFNDLLUPDATEKREONARMPROBEPROFILE)			GetProcAddress(m_hModule,"UpdateKreonArmProbeProfile");
		m_lpfnDllAddKreonArmProbeProfile			= (LPFNDLLADDKREONARMPROBEPROFILE)				GetProcAddress(m_hModule,"AddKreonArmProbeProfile");

		if(	m_lpfnDllIsConnected			&&
			m_lpfnDllBeginArm				&&	m_lpfnDllEndArm					&&
			m_lpfnDllBeginAcquisition		&&	m_lpfnDllEndAcquisition			&&
			m_lpfnDllGetAcquisitionMode		&&
			m_lpfnDllGetCurrentPosition		&&	m_lpfnDllGetTriggeredPosition	&&
			m_lpfnDllGetButtonState			&&
			m_lpfnDllGetParameter			&&	m_lpfnDllSetParameter			&&
			m_lpfnDllHasPropertiesWindow	&&	m_lpfnDllPropertiesWindow		&&
			m_lpfnDllProbeCalibration)
			m_bDllOk = true;
		else
			printf("Error in CArmManager::CArmManager: Invalid arm DLL\n");
	}
	else
		printf("Error in CArmManager::CArmManager: LoadLibrary failed\n");
}

/////////////////////////////////////////////////////////////////////////////
CArmManager::~CArmManager()
{
	printf("CArmManager::~CArmManager\n");

	if (m_hModule)
	{
		if (m_lpfnDllIsConnected && IsConnected() && m_lpfnDllEndArm)
			EndArm();

		// Free the library unless it is KRMicroScribe.dll (because it uses FlyCapture2.dll that may crash when freed).
		if ( _tcsicmp((LPCTSTR)m_DllName, _T("KRMicroScribe.dll")) )
			::FreeLibrary(m_hModule);
		m_hModule = NULL;
	}

	if (m_DllName)
		delete [] m_DllName;
}

/////////////////////////////////////////////////////////////////////////////
int CArmManager::GetNbButtons(void)
{
	if ( (m_NbButtons==-1) && m_lpfnDllGetParameter )
		GetParameter("NbButtons",&m_NbButtons);
	return m_NbButtons;
}

/////////////////////////////////////////////////////////////////////////////
void CArmManager::SetFrequency(int Frequency)
{
	if (m_lpfnDllSetParameter)
		SetParameter("AcquisitionFrequency",&Frequency);
}

/////////////////////////////////////////////////////////////////////////////
WORD CArmManager::GetTriggerDuration(void)
{
	if (m_TriggerDuration==0xFFFF)
	{
		if (m_lpfnDllGetParameter)
			GetParameter("TriggerDuration",&m_TriggerDuration);

		if (m_TriggerDuration==0xFFFF)
			return 200;		//	Default value for most arms
	}

	return m_TriggerDuration;
}
