#include "stdafx.h"
#include "../DPCMM.h"
#include "KreonCmdWnd.h"
#include "../KreonScannerArm.h"
#include "ArmManager.h"
#include "KreonServer.h"
#include "SocketManager.h"
#include "KR_Specific.h"
#include "KreonProbeCalibrations.h"
#include "../DSelectCalibration.h"

//	Arm's buttons states polling
#define	ARM_BUTTONS_TIMER_ID		1
//#define	ARM_BUTTONS_SCANNER_CAL_TIMER_ID		3
#define	ARM_BUTTONS_TIMER_PERIOD	50		//	ms
//	Arm's touch probe position polling
#define	ARM_TOUCHPROBE_TIMER_ID		2
#define	ARM_TOUCHPROBE_TIMER_PERIOD	100		//	ms



#define	ARM_NAME			_T("KreonArm")	// Kreon Ace/Baces
//#define	ARM_NAME			_T("Cimcore")	// Romer-Cimcore Stinger2/3000i/Infinite
//#define	ARM_NAME			_T("Faro")		// Faro Silver/Gold/Millenium
//#define	ARM_NAME			_T("FaroUSB")	// Faro Platinum/Titanium/Advantage/Quantum/Fusion
//#define	ARM_NAME			_T("Garda")		// Garda
//#define	ARM_NAME			_T("Romer")		// Romer France System6/Armony (Serial)
//#define	ARM_NAME			_T("RomerEth")	// Romer France Sigma (Ethernet)
//#define	ARM_NAME			_T("Simu")		// Kreon's arm simulator -- does NOT simulate the scanner, besides, you need a dongle for the scanner :)
//#define	ARM_NAME			_T("Baces3D")



KR_CALIBRATION_FILE_LIST*	volatile st_CalibrationFileList;
CArmManager					*ArmManager;
CSocketManager				*SocketManager;
//CChildView					CMainFrame::m_wndView;
HWND						KreonCmdWnd::m_StatichWnd;
KreonScannerArm*		pOwner;
CriticalSectionWrapper* kcsw;
SAIPScanStripeCloud* scanStripePkt;
DWORD						StartTime;
int							NbLines;
double						PositioningMatrix[16];
bool						g_bButtonState[4]={0,0,0,0};
bool						isScanning=false;
bool						probeCalWindowClosed = false;
int							g_nWaitForInitKreonCallback=0;
static bool					st_bEndKreon = false;

IMPLEMENT_DYNAMIC(KreonCmdWnd, CWnd)

KreonCmdWnd::KreonCmdWnd(KreonScannerArm* _owner, bool _simulate)
: owner(*_owner)
, simulate(_simulate)
, scannerCS(*new CriticalSectionWrapper)
{
	windowValid = false;
	ArmManager=NULL;
	m_bKreonInitialized=false;
	pOwner = _owner;
	kcsw = &scannerCS;
	scanStripePkt = &pOwner->scanStripePkt;
}

KreonCmdWnd::~KreonCmdWnd(void)
{
	ShutDown();
	delete &scannerCS;
}
void KreonCmdWnd::ShutDown()
{
	if (isScanning)
	{
		SocketManager->EndAcquisition();
		ArmManager->EndAcquisition();
		isScanning=false;
		ArmManager->EndArm();

		MSG Msg;
		while (PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
		{
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
			Sleep(1);
		}
		Sleep(500);
	}

	if (SocketManager)
	{
		st_bEndKreon = false;

		// End the KREON system and the thread
		BYTE SensorOff=0;
		SocketManager->EndKreon(SensorOff);

		while (!st_bEndKreon && SocketManager->IsSocketConnected())
		{
			MSG Msg;
			while (PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
			{
				TranslateMessage(&Msg);
				DispatchMessage(&Msg);
			}
			Sleep(10);
		}

		delete SocketManager;
		SocketManager=NULL;
	}
}

bool KreonCmdWnd::MakeValidWindow()
{
	// create our window
	CString strClassName;
	strClassName = AfxRegisterWndClass( NULL, 0, 0, 0 );
	if ( !CreateEx(0, strClassName, _T("KreonCmdWnd"), WS_POPUP, 0, 0, 0, 0, NULL, NULL) )
		return false;
	windowValid = true;
	m_StatichWnd = m_hWnd;
	pOwner = &owner;
	return true;
}

BOOL KreonCmdWnd::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// TODO: Add your specialized code here and/or call the base class

	return CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}
BEGIN_MESSAGE_MAP(KreonCmdWnd, CWnd)
	ON_MESSAGE(WM_SOCKET_MESSAGE, OnDataSocket)
	ON_WM_TIMER()
END_MESSAGE_MAP()

void KreonCmdWnd::OnTimer(UINT_PTR nIDEvent)
{
	if(nIDEvent==ARM_BUTTONS_TIMER_ID)	// used ONLY for scanning
	{
		int NbButtons = 0;
		ArmManager->GetParameter("NbButtons",&NbButtons);

		if(NbButtons>0)
		{
			bool* bButtonState = new bool [NbButtons];
			if(ArmManager->GetButtonState(bButtonState)==KR_ARM_OK)
			{
				bool bButtonPressed = false;
				for (int i=0; i<NbButtons ; i++)
					if (bButtonState[i])
					{
						bButtonPressed = true;
						break;
					}

					if (bButtonPressed)
					{
						//	Wait until the button is released
						while (bButtonPressed)
						{
							bButtonPressed = false;
							if (ArmManager->GetButtonState(bButtonState)==KR_ARM_OK)
								for (int i=0; i<NbButtons ; i++)
									if (bButtonState[i])
									{
										bButtonPressed = true;
										break;
									}
						}

						//	Tell the positioning window that the user wants
						//	to start the acquisition
						SocketManager->BeginAcquisition();
					}
			}
			delete [] bButtonState;
		}
	}
	else if(nIDEvent==ARM_TOUCHPROBE_TIMER_ID)
	{
		int NbButtons = 0;
		ArmManager->GetParameter("NbButtons",&NbButtons);

		if(NbButtons>0)
		{
			bool*	bButtonState = new bool [NbButtons];
			memset(bButtonState,0,sizeof(bool)*NbButtons);
			double	Matrix[16];
			TRACE0("GetCurPos 1\n");
			int Result = ArmManager->GetCurrentPosition(Matrix,bButtonState);
			if(Result==KR_ARM_OK  ||  Result==KR_ARM_ONLY_BUTTONS_ARE_VALID)
			{
				bool bFrontButton,bBackButton;
				if (NbButtons>=2)
				{
					bFrontButton = bButtonState[1];
					bBackButton  = bButtonState[0];
				} 
				else if (NbButtons==1)
				{
					bFrontButton = bButtonState[0];
					bBackButton  = false;
				}
				else
				{
					bFrontButton = false;
					bBackButton  = false;
				}
				// It is recommended to wait here for the button to be released when using sphere calibration method
				/*
				bool bButtonPressed = bFrontButton || bBackButton;
				if (bButtonPressed)
				{
				double	dummyMatrix[16];
				memset(bButtonState,0,sizeof(bool)*NbButtons);
				int waitResult;
				do
				{
				waitResult = ArmManager->GetCurrentPosition(dummyMatrix,bButtonState);
				bButtonPressed = false;
				if (Result==KR_ARM_OK  ||  Result==KR_ARM_ONLY_BUTTONS_ARE_VALID)
				{
				for (int i=0; i<NbButtons; i++)
				{
				if (bButtonState[i])
				{
				bButtonPressed = true;
				break;
				}
				}
				}
				} while (bButtonPressed);
				}
				*/
				if(bBackButton)
					SocketManager->ProbeDelLastPosition(true);
				else if(bFrontButton  &&  Result==KR_ARM_OK)
					SocketManager->ProbeAddPosition(Matrix,false);
			}
			delete [] bButtonState;
		}
	}

	CWnd::OnTimer(nIDEvent);
}

void KreonCmdWnd::OpenLicenseDlg()
{
	SocketManager->OpenLicenseDialog();
}


/////////////////////////////////////////////////////////////////////////////
void OnKreonCallback(char *Command)
{
	//	When the server receives a command, it runs it, and then, sends an ack to the client
	//	You should always read this ack to know if the command ran correctly or not.
	//	The answer is composed of a minimum of 3 bytes (except KR_SOCKET_CALLBACK):
	//	Command[0] = Name of the command (e.g. KR_SOCKET_INITKREON, KR_SOCKET_GETXYZLASERLINE...)
	//	Command[1] = Return value of the command (most of the time: true if ok, else false)
	//	Command[2] = New value of the "LastErrorCode"
	if(Command[1] == 0)
	{
		switch(Command[2])
		{
		case KR_ERR_UNAUTHORIZED_SOFTWARE:
		case KR_ERR_UNAUTHORIZED_OPTION:
			SocketManager->OpenLicenseDialog();
			break;
		}
	}

	// init variables for the very involved process of getting the current probe!
	double probeDia = 0.0;
	CString currentProbeName;
	CString currentProbe(_T("CurrentProbe"));
	//HKEY hKey;
	CString regentry ("Software\\Kreon Technologies\\vVERSION\\Config");

	switch(Command[0])
	{
	case KR_SOCKET_INITKREON:
		if (!Command[1])
		{
			SocketManager->GetLastErrorString((BYTE)Command[2], true);
			g_nWaitForInitKreonCallback = -1;
		}
		else
			g_nWaitForInitKreonCallback = 0;
		break;

	case KR_SOCKET_GETLASTERRORSTRING:
		if(Command[1])
		{
			CString tmp;
#ifdef _UNICODE
			tmp.Format(_T("Error: %s"),(TCHAR*)&Command[3]);
#else
			tmp.Format(_T("Error: %s"),&Command[3]);
#endif
			//tmp += (_T("\nLicense info will now be shown."));
			AfxMessageBox(tmp);
			//SocketManager->OpenLicenseDialog();	// this lets the user enter a code to update their dongle in case that is the problem.
		}
		break;

	case KR_SOCKET_SPHERECALIBRATION:
		if (Command[1])
		{
			// If you called this:
			// SocketManager->SphereCalibration(CalibrationScan,CalibrationMatrix,Type5axes,SupportDirection,SphereDiameter);
			// then, the following data items are returned here:
			double SphereCenter[3];
			double Sigma;
			double CalibrationMatrix[16];
			memcpy(SphereCenter,&Command[3],3*sizeof(double));
			memcpy(&Sigma,&Command[3+3*sizeof(double)],sizeof(double));
			memcpy(CalibrationMatrix,&Command[3+sizeof(double)+3*sizeof(double)],16*sizeof(double));

			CString output;
			output.Format(_T("SphereCalibration results:\nSigma = %f"),Sigma);
			AfxMessageBox(output);
		}
		else
		{
			AfxMessageBox(_T("SphereCalibration failed"));
			SocketManager->GetLastErrorString((BYTE)Command[2], true);
		}
		break;

	case KR_SOCKET_GETCALIBRATIONFILELIST:
		{
			// Allocate memory for the list
			KR_CALIBRATION_FILE_LIST* list = new KR_CALIBRATION_FILE_LIST;
			memcpy(list,&Command[3],sizeof(KR_CALIBRATION_FILE_LIST));

			if (Command[1])
			{
				/*
				// This sample code displays the calibration file list in a message box
				CString output = "Calibration file list:\n";
				int offsetFilename = 0;	// point towards the first file name
				for (UINT i=0; i<list->nbFiles; i++)
				{
				output += list->list+offsetFilename; // Add the file name to the output string
				if (list->offsetCurrentFile==offsetFilename)
				output += " (current)";
				output += "\n";
				offsetFilename += strlen(list->list+offsetFilename)+1; // point towards the next file name
				}
				AfxMessageBox(output);
				*/
			}

			st_CalibrationFileList = list; // tells CMainFrame::Init() to stop waiting for a reply from the server
		}
		break;

	case KR_SOCKET_OPENVIDEOSETTINGWINDOW:
	case KR_SOCKET_OPENPOSITIONINGWINDOW:
	case KR_SOCKET_OPENPOSITIONINGWINDOWEXT:
	case KR_SOCKET_PROBEOPENCALIBRATIONWINDOW:
		if(Command[1])
		{
			//	You can get the HWND of the windows created by the Kron SDK here.
			HWND hwnd;
			memcpy(&hwnd,&Command[3],sizeof(HWND));
		}
		break;

	case KR_SOCKET_GETCALIBRATIONMATRIX:
		if(Command[1])
		{
			//	you should save somewhere the calibration matrix
			//	(not done here as it is just an example)
			double PositioningMatrix[16];
			memcpy(PositioningMatrix,&Command[3],16*sizeof(double));

			// 2009.04.27 - we have to set the calibration matrix!
			double calMatrix[4][4];
			calMatrix[0][0] = PositioningMatrix[0];	calMatrix[0][1] = PositioningMatrix[4];	calMatrix[0][2] = PositioningMatrix[8];	calMatrix[0][3] = PositioningMatrix[12];
			calMatrix[1][0] = PositioningMatrix[1];	calMatrix[1][1] = PositioningMatrix[5];	calMatrix[1][2] = PositioningMatrix[9];	calMatrix[1][3] = PositioningMatrix[13];
			calMatrix[2][0] = PositioningMatrix[2];	calMatrix[2][1] = PositioningMatrix[6];	calMatrix[2][2] = PositioningMatrix[10];	calMatrix[2][3] = PositioningMatrix[14];
			calMatrix[3][0] = PositioningMatrix[3];	calMatrix[3][1] = PositioningMatrix[7];	calMatrix[3][2] = PositioningMatrix[11];	calMatrix[3][3] = PositioningMatrix[15];

			pOwner->SetScannerCalMatrix(calMatrix);

			//	you can also get the standard deviation from the toolkit if
			//	you need it
			double Sigma;
			memcpy(&Sigma,&Command[3 + 16*sizeof(double)],sizeof(double));

			//	and the shape default...
			double ShapeDefault;
			memcpy(&ShapeDefault,&Command[3 + 17*sizeof(double)],sizeof(double));

			double StatisticShapeDefault;
			memcpy(&StatisticShapeDefault,&Command[3 + 18*sizeof(double)],sizeof(double));
		}
		break;

	case KR_SOCKET_PROBEGETCALIBRATION:
		if(Command[1])
		{
			//BYTE ProbeIndex = Command[3];
			KR_TOUCH_PROBE	Probe;
			memcpy(&Probe,&Command[4],sizeof(KR_TOUCH_PROBE));

			//	Some devices need to know the calibration of the probe
			ArmManager->SetParameter("Probe",&Probe);

			pOwner->AddOrEditProbeCal(Probe);
		}
		break;

	case KR_SOCKET_STARTSCAN:	// 2016.03.29 - added for meshing - need to know when to start/stop mesh based on scan passes
		pOwner->SendOnScanPassStarted();
		break;

	case KR_SOCKET_PAUSESCAN:	// 2016.03.29 - added for meshing - need to know when to start/stop mesh based on scan passes
		//pOwner->SendOnScanPassStopped();
		if (app.saarm.Data.voxelizationEnabled)
			pOwner->ClearScanClouds();
		break;

	case KR_SOCKET_GETXYZLASERLINE:
		if (Command[1])
		{

			int		n;

			memcpy(&n,&Command[3],sizeof(int));
			TRACE1("%d scan lines in buffer\n", n);

			if (n>0)
			{
				//	Get the laser line
				DWORD LaserLineSizeA,LaserLineSizeB,LaserLineSize;
				DWORD LaserLineSizeAInBytes,LaserLineSizeBInBytes,LaserLineSizeInBytes;
				memcpy(&LaserLineSizeA,&Command[3+sizeof(int)],sizeof(DWORD));
				memcpy(&LaserLineSizeB,&Command[3+sizeof(int)+sizeof(DWORD)+(LaserLineSizeA*3*sizeof(double))+(16*sizeof(double))],sizeof(DWORD));
				LaserLineSize = LaserLineSizeA+LaserLineSizeB;
				double * LaserLine = new double[3*LaserLineSize];
				LaserLineSizeAInBytes = LaserLineSizeA*3*sizeof(double);
				LaserLineSizeBInBytes = LaserLineSizeB*3*sizeof(double);
				LaserLineSizeInBytes = LaserLineSizeAInBytes + LaserLineSizeBInBytes;
				if (LaserLineSizeA)
					memcpy(LaserLine,&Command[3+sizeof(int)+sizeof(DWORD)],LaserLineSizeAInBytes);
				if (LaserLineSizeB)
					memcpy(LaserLine+(3*LaserLineSizeA),&Command[3+sizeof(int)+sizeof(DWORD)+LaserLineSizeAInBytes+(16*sizeof(double))+sizeof(DWORD)],LaserLineSizeBInBytes);

				TRACE1("laser line size = %d\n", LaserLineSize);

				//double	MachineMatrix[16];
				//memcpy(MachineMatrix,&Command[3+sizeof(int)+sizeof(DWORD)+LaserLineSizeAInBytes],16*sizeof(double));
				/*
				The Machine Matrix gives the position and direction of the machine
				MachineMatrix[ 0] == Xleft
				MachineMatrix[ 1] == Yleft
				MachineMatrix[ 2] == Zleft
				MachineMatrix[ 3] == 0
				MachineMatrix[ 4] == Xup
				MachineMatrix[ 5] == Yup
				MachineMatrix[ 6] == Zup
				MachineMatrix[ 7] == 0
				MachineMatrix[ 8] == Xdirection
				MachineMatrix[ 9] == Ydirection
				MachineMatrix[10] == Zdirection
				MachineMatrix[11] == 0
				MachineMatrix[12] == Xposition
				MachineMatrix[13] == Yposition
				MachineMatrix[14] == Zposition
				MachineMatrix[15] == 1
				*/
				//txt.Format(_T("LaserLine has %d points"), LaserLineSize);
				//TRACE1("%s\n", txt);
				//if (LaserLineSizeA)
				//	txt.Format(_T("A: (%8.3f , %8.3f , %8.3f) -> (%8.3f , %8.3f , %8.3f)"),
				//	LaserLine[0],LaserLine[1],LaserLine[2],LaserLine[((LaserLineSizeA-1)*3)+0],LaserLine[((LaserLineSizeA-1)*3)+1],LaserLine[((LaserLineSizeA-1)*3)+2]);
				//else
				//	txt = (_T("No points on camera A"));
				//TRACE1("%s\n", txt);
				//if (LaserLineSizeB)
				//	txt.Format(_T("B: (%8.3f , %8.3f , %8.3f) -> (%8.3f , %8.3f , %8.3f)"),
				//	LaserLine[(LaserLineSizeA*3)+0],LaserLine[(LaserLineSizeA*3)+1],LaserLine[(LaserLineSizeA*3)+2],LaserLine[((LaserLineSize-1)*3)+0],LaserLine[((LaserLineSize-1)*3)+1],LaserLine[((LaserLineSize-1)*3)+2]);
				//else
				//	txt = (_T("No points on camera B"));
				//TRACE1("%s\n", txt);

				if(LaserLineSize > 0)
				{
					if(app.saarm.Data.useScanStripeClouds && pOwner->performingStdScannerMeas)
					{
						pOwner->AddScannerCloudStripe();

						SAIPScanStripePoint sspt;
						Vector xyzKreon;

						for(DWORD i=0; i<(LaserLineSize*3); i+=3)
						{
							if(i % pOwner->thinningFactor != 0)	
								continue;
							xyzKreon.x = LaserLine[i];
							xyzKreon.y = LaserLine[i+1];
							xyzKreon.z = LaserLine[i+2];
							xyzKreon.Scale(1.0/25.4);

							sspt.x = (float)xyzKreon.x;
							sspt.y = (float)xyzKreon.y;
							sspt.z = (float)xyzKreon.z;

							pOwner->AddScannerCloudStripePt(sspt);
						}

						Transform scannerXform;
						double	MachineMatrix[16];
						memcpy(MachineMatrix,&Command[3+sizeof(int)+sizeof(DWORD)+LaserLineSizeAInBytes],16*sizeof(double));

						double X = MachineMatrix[12];
						double Y = MachineMatrix[13];
						double Z = MachineMatrix[14];

						X /= 1000.0;
						Y /= 1000.0;
						Z /= 1000.0;

						scannerXform.T[0][0] = MachineMatrix[0];
						scannerXform.T[1][0] = MachineMatrix[1];
						scannerXform.T[2][0] = MachineMatrix[2];

						scannerXform.T[0][1] = MachineMatrix[4];
						scannerXform.T[1][1] = MachineMatrix[5];
						scannerXform.T[2][1] = MachineMatrix[6];

						scannerXform.T[0][2] = MachineMatrix[8];
						scannerXform.T[1][2] = MachineMatrix[9];
						scannerXform.T[2][2] = MachineMatrix[10];

						scannerXform.T[0][3] = X;
						scannerXform.T[1][3] = Y;
						scannerXform.T[2][3] = Z;

						CriticalSectionLock lock( *kcsw );
						scannerXform.Scale(app.saarm.LocalToInches);
						scanStripePkt->SetCurrentLineTransform(scannerXform);
					}
					else
					{
						SAIPCloudBlockPt cbpt;
						Vector xyzKreon;

						for(DWORD i=0; i<(LaserLineSize*3); i+=3)
						{
							if(i % pOwner->thinningFactor != 0)	
								continue;
							xyzKreon.x = LaserLine[i];
							xyzKreon.y = LaserLine[i+1];
							xyzKreon.z = LaserLine[i+2];
							xyzKreon.Scale(1.0/25.4);

							cbpt.x = (float)xyzKreon.x;
							cbpt.y = (float)xyzKreon.y;
							cbpt.z = (float)xyzKreon.z;

							pOwner->AddScannerCloudPt(cbpt);
						}
					}
				}

				//#define MAX_LINES_TO_COMPUTE_FREQUENCY 100
				//static CList<DWORD,DWORD> Times;
				//DWORD CurrentTime = GetTickCount();
				//if (NbLines==0)
				//{
				//	StartTime=CurrentTime;
				//	Times.RemoveAll();
				//}
				//Times.AddTail(CurrentTime);
				//NbLines++;
				//if (NbLines>2)
				//{
				//	DWORD OldestTime,NbLinesForFrequency;
				//	if (NbLines<MAX_LINES_TO_COMPUTE_FREQUENCY)
				//	{
				//		OldestTime = Times.GetHead();
				//		NbLinesForFrequency = NbLines;
				//	}
				//	else
				//	{
				//		OldestTime = Times.RemoveHead();
				//		NbLinesForFrequency = MAX_LINES_TO_COMPUTE_FREQUENCY;
				//	}
				//	double Time=(CurrentTime-OldestTime)/1000.0;
				//	int ElapsedTime=(CurrentTime-StartTime)/1000;
				//	static DWORD LastDisplayTime=StartTime;
				//	static CString DisplayedString(_T(""));
				//	if ( ((CurrentTime-LastDisplayTime)>500) && (Time>0) )
				//	{
				//		LastDisplayTime = CurrentTime;
				//		DisplayedString.Format(_T("Received lines: %03d, time: %03d, Freq. over last %3u lines: %2.1f"),
				//			NbLines,ElapsedTime,NbLinesForFrequency,(double)NbLinesForFrequency/Time);
				//	}
				//	TRACE1("%s\n", txt);
				//}

				//txt.Format(_T("Buttons state: %d %d %d %d"),g_bButtonState[0],g_bButtonState[1],g_bButtonState[2],g_bButtonState[3]);
				//TRACE1("%s\n", txt);

				delete [] LaserLine;
			}
		}
		break;

	case KR_SOCKET_GETLASERPLANECORNERS:
		if(Command[1])
		{
			double corners[4*2];	//	4 UV coordinates
			memcpy(corners,&Command[3],4*2*sizeof(double));
			CString strCorners;
			strCorners.Format(_T("Corners:\n")
				_T("   (%7.3f,%7.3f) (%7.3f,%7.3f)\n")
				_T("(%7.3f,%7.3f)       (%7.3f,%7.3f)\n"),
				corners[6],corners[7], corners[4],corners[5],
				corners[0],corners[1], corners[2],corners[3]);
			AfxMessageBox(strCorners);
		}
		break;

	case KR_SOCKET_CALLBACK:
		{
			UCHAR Cmd;
			Cmd=Command[1];
			switch(Cmd)
			{
			case KR_CMD_POSITIONING_WINDOW_CLOSED:
				{
					::KillTimer(KreonCmdWnd::m_StatichWnd,ARM_BUTTONS_TIMER_ID);
					SocketManager->EndAcquisition();
					ArmManager->EndAcquisition();
					DWORD  Param;
					memcpy(&Param,&Command[2],sizeof(DWORD));
					if (Param==1)
					{
						//	The positioning process returned 1 index
						//	To get its resulting matrix, just send a "KR_SOCKET_GETCALIBRATIONMATRIX"
						//	command to the server.
						BYTE Index = 0;
						SocketManager->GetCalibrationMatrix(Index);
					}
					isScanning=false;
					ArmManager->BeginAcquisition(KR_ARM_MODE_CURRENT_POS);
					pOwner->RestartQueryTimer();
				}
				break;

			case KR_CMD_VIDEOSETUP_WINDOW_CLOSED:
				//	The video setup window has been closed by the user
				//	(or by a call to the KR_SOCKET_CLOSEVIDEOSETTINGWINDOW command)
				break;

			case KR_CMD_SCAN_FREQUENCY:
				{
					//	Some arms need to know the frequency they will work at
					DWORD  Param;
					memcpy(&Param,&Command[2],sizeof(DWORD));
					ArmManager->SetParameter("AcquisitionFrequency",&Param);
				}
				break;

			case KR_CMD_SCAN_STARTED:
				//	The scan is starting...
				//	So, we can stop polling the arm buttons
				::KillTimer(KreonCmdWnd::m_StatichWnd,ARM_BUTTONS_TIMER_ID);
				ArmManager->EndAcquisition();
				//	And start the acquisition mode of the arm
				ArmManager->BeginAcquisition(KR_ARM_MODE_SYNCHRONOUS);
				//	Finally, as the arm is ready, start the scanner digitization.
				SocketManager->StartScannerAcquisitionInPositioningWindow();
				break;

			case KR_CMD_SCAN_PAUSED:
				ArmManager->EndAcquisition();
				ArmManager->BeginAcquisition(KR_ARM_MODE_BUTTONS);
				::SetTimer(KreonCmdWnd::m_StatichWnd,ARM_BUTTONS_TIMER_ID,ARM_BUTTONS_TIMER_PERIOD,NULL);
				break;

			case KR_CMD_SCAN_STOPPED:
				::KillTimer(KreonCmdWnd::m_StatichWnd,ARM_BUTTONS_TIMER_ID);
				ArmManager->EndAcquisition();
				ArmManager->BeginAcquisition(KR_ARM_MODE_CURRENT_POS);
				break;

			case KR_CMD_SCAN_LINE_READY:
				{
					//	Ask the laser line to the toolkit
					SocketManager->GetXYZLaserLine();
				}
				break;

			case KR_CMD_TRIGGERED_POSITION_NEEDED:
				{
					//	If your machine is fast enough, you can ask the position here,
					//	and send a "KR_SOCKET_SETMACHINEPOSITION" command to the server
					double	PositionMatrix[16];
					int nJts;
					double* jtAngles;
					TRACE0("GetTriggeredPosition 1\n"); 
					int Result=ArmManager->GetTriggeredPositionAndJointAngles (PositionMatrix, g_bButtonState, &nJts, &jtAngles, true);
					TRACE1("GetTriggeredPosition returned %d\n", Result);
					TRACE2("button0 = %d, button1 = %d\n", g_bButtonState[0], g_bButtonState[1]);
					if ((Result==KR_ARM_OK) || (Result==KR_ARM_OK_BUT_POSITION_UNAVAILABLE) || (Result==KR_ARM_MECHANICAL_STOP))
					{
						bool bPauseButton=false;
						bool bStopButton=false;
						if (Result==KR_ARM_OK)
						{
							int NbButtons = 0;
							ArmManager->GetParameter("NbButtons",&NbButtons);
							if (NbButtons>0)
							{
								if (NbButtons==1)
								{
									bStopButton = g_bButtonState[0];
								}
								else
								{
									bStopButton = g_bButtonState[0];		// trigger push, or side or yellow button
									bPauseButton = g_bButtonState[1];	// trigger pull, or center or green button
								}
							}
						}

						//	You may want to stop the acquisition if a button is pressed
						if (bPauseButton)
						{
							SocketManager->EndAcquisition();
							TRACE0("PAUSE BUTTON\n");
							pOwner->PostUpdateScanDlg(Tr("Scan Paused..."));
							while(g_bButtonState[1])	// the green button, which we are defining as the pause button
							{
								TRACE0("GetTriggeredPosition 2\n"); 
								ArmManager->GetTriggeredPositionAndJointAngles (PositionMatrix, g_bButtonState, &nJts, &jtAngles, true);
								Delay(0.1, TRUE);
							}
							pOwner->PostUpdateScanDlg(Tr("Scan Resumed.\n"));
							SocketManager->BeginAcquisition();
						}
						else if(bStopButton && isScanning)
						{
							pOwner->StopScannerMeas();
						}
						else
						{
							//	The Kreon SDK needs a triggered position from the arm
							if (Result==KR_ARM_OK)
							{
								SocketManager->SetMachinePosition(PositionMatrix);

								double X = PositionMatrix[12];
								double Y = PositionMatrix[13];
								double Z = PositionMatrix[14];

								X /= 1000.0;
								Y /= 1000.0;
								Z /= 1000.0;

								pOwner->CurrentTransform.T[0][0] = PositionMatrix[0];
								pOwner->CurrentTransform.T[1][0] = PositionMatrix[1];
								pOwner->CurrentTransform.T[2][0] = PositionMatrix[2];

								pOwner->CurrentTransform.T[0][1] = PositionMatrix[4];
								pOwner->CurrentTransform.T[1][1] = PositionMatrix[5];
								pOwner->CurrentTransform.T[2][1] = PositionMatrix[6];

								pOwner->CurrentTransform.T[0][2] = PositionMatrix[8];
								pOwner->CurrentTransform.T[1][2] = PositionMatrix[9];
								pOwner->CurrentTransform.T[2][2] = PositionMatrix[10];

								pOwner->CurrentTransform.T[0][3] = X;
								pOwner->CurrentTransform.T[1][3] = Y;
								pOwner->CurrentTransform.T[2][3] = Z;

								if(nJts == 7)
								{
									pOwner->J.x[0] = -jtAngles[0];
									pOwner->J.x[1] = -90.0-jtAngles[1];
									pOwner->J.x[2] = jtAngles[2];
									pOwner->J.x[3] = -jtAngles[3];
									pOwner->J.x[4] = jtAngles[4];
									pOwner->J.x[5] = -jtAngles[5];
									pOwner->J.x[6] = 90.0+jtAngles[6];
									//TRACE2("1st joint angle = %f\n7th joint angle = %f\n\n", -jtAngles[0], jtAngles[6]);
									//TRACE0("Our values now set to...\n");
									//J.Debug();
								}
								else if(nJts == 6)
								{
									pOwner->J.x[0] = -jtAngles[0];
									pOwner->J.x[1] = -90.0-jtAngles[1];
									pOwner->J.x[2] = jtAngles[2];
									pOwner->J.x[3] = -jtAngles[3];
									pOwner->J.x[4] = jtAngles[4];
									pOwner->J.x[5] = -jtAngles[5];
									//TRACE2("1st joint angle = %f\n6th joint angle = %f\n\n", -jtAngles[0], jtAngles[5]);
									//TRACE0("Our values now set to...\n");
									//J.Debug();
								}
							}
							else	// Or at least know that there was no position available
								SocketManager->SetNoMachinePositionAvailable();

							//	These functions automatically request for a new
							//	acquisition from the scanner (which will need
							//	a triggered position, and so on...)
						}
					}
					else
						SocketManager->EndAcquisition();
				}
				break;

			case KR_CMD_SCANNER_ERROR:
				SocketManager->EndAcquisition();
				break;

			case KR_CMD_KEY_PRESSED:
				//	A key has been pressed in the positioning window or in the video setup window
				DWORD Key;
				memcpy(&Key,&Command[3],sizeof(DWORD));
				if(Key==VK_F1)
				{
					//	...
				}
				break;

			case KR_CMD_TP_SPHERE_CALIBRATION_STARTED:
			case KR_CMD_TP_HOLE_CALIBRATION_STARTED:
				{
					ArmManager->EndAcquisition();
					//	The user wants to calibrate a touch probe
					//	Some arms/devices need to know that we are starting a probe calibration.
					//	For this, we send a blank KR_TOUCH_PROBE structure with Sigma set to a negative value.
					//KR_TOUCH_PROBE Probe = {"",0,0,0, -1.0 , 0,false};
					//ArmManager->SetParameter("Probe",&Probe);
					// let's set it to the current (0th for now) persisted cal....
					KR_TOUCH_PROBE Probe;
					int numProbes = 0;
					pOwner->GetProbeCals(Probe, numProbes);
					ArmManager->SetParameter("Probe", &Probe);

					//	We need to start a timer and the arm to give positions to the toolkit regularly.
					ArmManager->BeginAcquisition(KR_ARM_MODE_TOUCH_PROBE);
					::SetTimer(KreonCmdWnd::m_StatichWnd,ARM_TOUCHPROBE_TIMER_ID,ARM_TOUCHPROBE_TIMER_PERIOD,NULL);
				}
				break;

			case KR_CMD_TP_SPHERE_CALIBRATION_ENDED:
			case KR_CMD_TP_HOLE_CALIBRATION_ENDED:
				//	End of calibration
				::KillTimer(KreonCmdWnd::m_StatichWnd,ARM_TOUCHPROBE_TIMER_ID);
				ArmManager->EndAcquisition();
				ArmManager->BeginAcquisition(KR_ARM_MODE_CURRENT_POS);
				break;

			case KR_CMD_TP_CALIBRATION_WINDOW_CLOSED_EXT:
				int  NbProbes;
				memcpy(&NbProbes,&Command[2],sizeof(int));
				//	If NbProbes==-1, it means that the dialog has been closed without saving or with the red cross button
				//	If NbProbes==0, then the list has been saved but is empty
				//	Otherwise a probe may have been recalibrated. Read it and save it if needed
				for (int i=0 ; i<NbProbes ; i++)
					SocketManager->ProbeGetCalibration((BYTE)i);

				// forget it - none of the below actually works for getting current selected target - partly because Toolkit v.3.2.2.3 provides no way to set it!
				// So until this is supported sufficiently, we will not even call their probe cal window. The user can only select the cal. at run time, so they need to cal. probes in Kreon utilities anyway.
				// email from Karl:
				/*
				Hi Bruce,

				The CurrentProbe value only changes from the Toolkit's calibration window, which I do not recommend using for Kreon arms.

				Therefore, you could have two different configurations:

				1st: The user selects a "KreonArm" instrument
				=> You load KRKreonArm.dll
				=> You initialize InitKreon with MachineType 90 (which is for Ace... Don't worry, type 65 for old Baces3D arms or 66 for more recent Kreon Baces arms would work exactly the same; so it is not a mistake to use 90 even if you are actually using any Baces arm.
				=> You do not use the Toolkit's Probe Calibration
				=> You expect the user to select the right probe profile from the KreonArm driver.
				=> You call ArmManager->GetParameter("ProbeDiameter",&probeDia) to know the probe diameter and you know the returned positions already took the probe calibration into account

				2nd: The user selects some third-party arm
				=> You load the related KRThirdPartyArm.dll
				=> You initialize InitKreon with the MachineType corresponding the arm (for DLLs that support several arm models, just pick the most probable one: it does not really matter).
				=> You use the Toolkit's Probe Calibration window and take the secret registry key into account when it's closed
				=> You know the probe diameter and calibration from the Current Probe which is part of the list of probes you share with the Toolkit (YOU have to record the list and pass it to the Toolkit when you open the Probe Calibration and update the list when the Toolkit returns a possibly modified probe list.
				=> Be careful that the list can ben empty or the last selected probe can be "not calibrated yet".
				=> When probing, add the Probe Calibration vector to the positions returned by the arm to get the actual probe position.

				Best regards,
				Karl MARITAUD | Software Manager
				Kreon Technologies | Tel: +33 555 42 80 40
				*/
				
				// get the selected probe's diameter - this does not actually return that, and there is no way to get it!
				// Their probe cal dialog is confusing because it allows the user to select the probes and even says "Selected Probe: ...", but it
				// is self contained only, it is not really selecting the probe, but just the "selected" one to calibrate.
				//ArmManager->GetParameter("ProbeDiameter",&probeDia); // returns mm
				//app.saarm.Data.TipDia = probeDia / 1000.0;

				// get the current probe name from their registry entry...
				//if (RegOpenKeyEx(HKEY_CURRENT_USER,regentry,0,KEY_READ,&hKey) == ERROR_SUCCESS)
				//{
				//	DWORD reg_type;
				//	DWORD buf_size = MAX_PATH;
				//	TCHAR buf[MAX_PATH];
				//	buf[0]=NULL;
				//	if (!RegQueryValueEx(hKey,currentProbe,0,&reg_type,(LPBYTE)buf,&buf_size) == ERROR_SUCCESS)
				//	{
				//		AfxMessageBox(Tr("Unable to query registry entry for current probe name."));
				//		RegCloseKey(hKey);
				//	}
				//	currentProbeName = buf;
				//	// 2004.11.12 - API reported a 'bug' unable to run the latest cal.
				//	// they changed the name of the exe from TrackerCalib to TrackerCal
				//	// 2004.11.22 - they changed it back to TrackerCalib
				//	RegCloseKey(hKey);
				//}

				probeCalWindowClosed = true;
				break;

			case KR_CMD_POSITIONING_DELETED:
				//	The last positioning has been deleted in the positioning window.
				//	You should restart the timer to read the arm buttons again.
				ArmManager->BeginAcquisition(KR_ARM_MODE_BUTTONS);
				::SetTimer(KreonCmdWnd::m_StatichWnd,ARM_BUTTONS_TIMER_ID,ARM_BUTTONS_TIMER_PERIOD,NULL);
				break;
			}
		}
		break;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool KreonCmdWnd::Init()
{
	CString armName;
	if(simulate)
		armName = _T("Simu");		// Kreon's arm simulator -- does NOT simulate the scanner, besides, you need a dongle for the scanner :)
	else
		armName = _T("KreonArm");

	if(!m_bKreonInitialized)
	{
		if (SocketManager)
		{
			delete SocketManager;
			SocketManager=NULL;
		}
		if (ArmManager)
		{
			if (ArmManager->IsConnected())
				ArmManager->EndArm();
			delete ArmManager;
			ArmManager=NULL;
		}

		//	Load the library (DLL) of the arm
		TCHAR DLLName[MAX_PATH];
		_stprintf_s(DLLName,_countof(DLLName),_T("KR%s.DLL"),armName);
		ArmManager=new CArmManager(DLLName);
		if (!ArmManager->IsInitialized())
		{
			CString ErrMsg;
			ErrMsg.Format(_T("Error: unable to load library KR%s.dll."),armName);
			AfxMessageBox(ErrMsg);
			return false;
		}

		ArmManager->PropertiesWindow(m_hWnd);

//#define	KR_ARM_OK								1
//#define KR_ARM_OK_BUT_POSITION_UNAVAILABLE		2
//#define	KR_ARM_FAILED							(-1)
//#define	KR_ARM_NOT_CONNECTED					(-2)
//#define	KR_ARM_MECHANICAL_STOP					(-3)
//#define	KR_ARM_COMMUNICATION_PROBLEM			(-4)
//#define	KR_ARM_NO_RESPONSE_FROM_ARM				(-5)
//#define	KR_ARM_UNABLE_TO_INITIALIZE				(-6)
//#define	KR_ARM_FORBIDDEN_BY_DONGLE				(-7)
//#define	KR_ARM_UNABLE_TO_INITIALIZE_SOFTWARE	(-8)
//#define	KR_ARM_BAD_FIRMWARE						(-9)
//#define	KR_ARM_MODE_NOT_SUPPORTED				(-10)
//#define	KR_ARM_BAD_MODE							(-11)
		int begun = ArmManager->BeginArm();
		if(begun!=KR_ARM_OK)
		{
			CString ErrMsg;
			ErrMsg.Format(_T("Error: unable to initialize the arm. Check the connection\n\n"));

			switch (begun)
			{
				case KR_ARM_OK_BUT_POSITION_UNAVAILABLE:
					ErrMsg += (_T("KR_ARM_OK_BUT_POSITION_UNAVAILABLE"));
					break;
				case KR_ARM_FAILED:
					ErrMsg += (_T("KR_ARM_FAILED"));
					break;
				case KR_ARM_NOT_CONNECTED:
					ErrMsg += (_T("KR_ARM_NOT_CONNECTED"));
					break;
				case KR_ARM_MECHANICAL_STOP:
					ErrMsg += (_T("KR_ARM_MECHANICAL_STOP"));
					break;
				case KR_ARM_NO_RESPONSE_FROM_ARM:
					ErrMsg += (_T("KR_ARM_NO_RESPONSE_FROM_ARM"));
					break;
				case KR_ARM_UNABLE_TO_INITIALIZE:
					ErrMsg += (_T("KR_ARM_UNABLE_TO_INITIALIZE"));
					break;
				case KR_ARM_FORBIDDEN_BY_DONGLE:
					ErrMsg += (_T("KR_ARM_FORBIDDEN_BY_DONGLE"));
					break;
				case KR_ARM_BAD_FIRMWARE:
					ErrMsg += (_T("KR_ARM_BAD_FIRMWARE"));
					break;
				case KR_ARM_MODE_NOT_SUPPORTED:
					ErrMsg += (_T("KR_ARM_MODE_NOT_SUPPORTED"));
					break;
				case KR_ARM_BAD_MODE:
					ErrMsg += (_T("KR_ARM_BAD_MODE"));
					break;
				case KR_ARM_NO_TRIGGER:
					ErrMsg += (_T("KR_ARM_NO_TRIGGER"));
					break;
				case KR_ARM_MODE_BUTTONS:
					ErrMsg += (_T("KR_ARM_MODE_BUTTONS"));
					break;
				default:
					break;
			}
			AfxMessageBox(ErrMsg, MB_ICONHAND);
			return false;
		}

		if(simulate)
			return true;

		//	Initialize and run the Kreon server
		SocketManager = new CSocketManager();
		int ret = SocketManager->InitServer(OnKreonCallback,m_hWnd,WM_SOCKET_MESSAGE);
		if (ret != KR_ERR_NO_ERROR)
		{
			delete SocketManager;
			SocketManager=NULL;
			delete ArmManager;
			ArmManager=NULL;

			CString ErrMsg;
			ErrMsg.Format(_T("Error %d: unable to initialize Kreon Server.\nCheck the connection and ensure the latest Kreon SA Plugin is installed to this SA.\n\n"), ret);
			
			switch (ret)
			{
			case INVALID_SOCKET:
				ErrMsg += (_T("INVALID_SOCKET"));
				break;
			case KR_ERR_SOCKET_ERROR:
				ErrMsg += (_T("KR_ERR_SOCKET_ERROR"));
				break;
			default:
				break;
			}
			
			AfxMessageBox(ErrMsg, MB_ICONHAND);
			return false;
		}

		//	Get the calibration files list (Field "needsCalFile" from KR_CALIBRATION_FILE_LIST
		//  tells whether the scanner needs a calibration file or not so it can be called with any scanner)
		//	One or more calibration files should be present in the <Hardware>\..\Calibration directory
		// But needsCalFile is obsolete, and is always true. Also, we cannot query which "Probile" the user selected when initializing the encoders,
		// so we have absolutely no way of determining if the user is running an arm with a scanner. We have to depend on which SA model they chose.
		st_CalibrationFileList = NULL;
		SocketManager->GetCalibrationFileList();

		// Wait until we receive the calibration file list from the server
		while (!st_CalibrationFileList)
		{
			MSG Msg;
			if (PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
			{
				TranslateMessage(&Msg);
				DispatchMessage(&Msg);
			}
			Sleep(0);
		}

		char * CalibrationFile;
		CStringArray calFiles;
		int thisOffsetFilename[128];
		if (st_CalibrationFileList->needsCalFile) // 2017.10.27 - Karl Maritaud says needsCalFile is obsolete, and is always true. Too bad, that could be used to tell me if the arm has a scanner.
		{
			if (!st_CalibrationFileList->nbFiles)
				return false;	// Fail if there is no calibration file available
			if (st_CalibrationFileList->offsetCurrentFile == -1)	// No calibration file was selected?
				st_CalibrationFileList->offsetCurrentFile = 0;		// ...so we take the first one.
			CalibrationFile = st_CalibrationFileList->list+st_CalibrationFileList->offsetCurrentFile;
		
			CString calFile;
			
			DWORD numFiles = st_CalibrationFileList->nbFiles;
			int offsetFilename = 0;	// point towards the first file name
			
			for (UINT i=0; i<st_CalibrationFileList->nbFiles; i++)
			{
				calFile = st_CalibrationFileList->list+offsetFilename; // Add the file name to the output string
				calFiles.Add(calFile);
				thisOffsetFilename[i] = offsetFilename;
				offsetFilename += strlen(st_CalibrationFileList->list+offsetFilename)+1; // point towards the next file name
			}
		}
		else
			CalibrationFile = NULL;

		DSelectCalibration calSelectDlg;
		calSelectDlg.SetCals(calFiles);
		calSelectDlg.selectedCal = app.saarm.Data.KreonProbeCals.selectedScannerCal;
		if(app.saarm.CMMString.Find(_T("Sc")) != -1) // We have no way of asking the arm whether a scanner is connected, so we have to depend on the user's choice of arm and our choice of the cmmstring
		{
			if(calSelectDlg.DoModal() != IDOK)
			{
				CString ErrMsg;
				ErrMsg.Format(_T("Error: No valid scanner Calibration file selected.\nSelect a calibration even if you have no scanner.\nSee Kreon ACE Arm - User's Manual from your hardware rep. for more information.\nIt says: 'Please note that, in some applications, it may be necessary to select a scanner calibration file, first.\nWhen using the arm for probing only, any scanner calibration file will be fine.'"));
				::MessageBox(NULL, ErrMsg, _T("Probe Management"), MB_ICONERROR | MB_SETFOREGROUND | MB_TOPMOST);
				return false;
			}
			if(calSelectDlg.selectedCal < 0)
			{
				CString ErrMsg;
				ErrMsg.Format(_T("Error: No valid scanner Calibration file selected.\nSelect a calibration even if you have no scanner.\nSee Kreon ACE Arm - User's Manual from your hardware rep. for more information.\nIt says: 'Please note that, in some applications, it may be necessary to select a scanner calibration file, first.\nWhen using the arm for probing only, any scanner calibration file will be fine.'"));
				::MessageBox(NULL, ErrMsg, _T("Probe Management"), MB_ICONERROR | MB_SETFOREGROUND | MB_TOPMOST);
				return false;
			}

			for(INT_PTR i=0; i<calFiles.GetSize(); i++)
			{
				TRACE2("cal file: %s, offset = %d\n", calFiles.GetAt(i), thisOffsetFilename[i]);
			}

			CalibrationFile = st_CalibrationFileList->list + thisOffsetFilename[calSelectDlg.selectedCal];
			TRACE2("cal file offset: %d, file: %s\n", thisOffsetFilename[calSelectDlg.selectedCal], calFiles.GetAt(calSelectDlg.selectedCal));
			TRACE2("cal file: %s, char* = %s\n", calFiles.GetAt(calSelectDlg.selectedCal), CA2T(CalibrationFile));
		}
		else
		{
			// If called with NULL for CAlibrationFile, the user doesn't have to pick a scanner cal.
			// For users who don't have scanners, but Karl says there is no way to query the arm to see if a scanner is connected.
			CalibrationFile = NULL;
		}

		//	Initialize the Kreon Toolkit
		g_nWaitForInitKreonCallback = 1;
		BYTE MachineType = 90;  // See KR_SOCKET_INITKREON HTML documentation for a complete list of machine types
		UINT CodeClient  = 3497826991;	// <- insert your client code here (provided by Kron)

		SocketManager->InitKreon(MachineType, CalibrationFile, CodeClient /*,"FR"*/); // <- optional language parameter

		//	Be sure that KreonServer finished processing the initialization before going any further
		while (g_nWaitForInitKreonCallback>0)
		{
			MSG Msg;
			if (PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
			{
				TranslateMessage(&Msg);
				DispatchMessage(&Msg);
			}
			Sleep(1);
		}

		//	Free CalibrationFileList (if you don't need it any more)
		delete st_CalibrationFileList;
		st_CalibrationFileList = NULL;

		//	Initialize the positioning matrix
		PositioningMatrix[0]=1.0;	PositioningMatrix[4]=0.0;	PositioningMatrix[8]=0.0;	PositioningMatrix[12]=0.0;
		PositioningMatrix[1]=0.0;	PositioningMatrix[5]=1.0;	PositioningMatrix[9]=0.0;	PositioningMatrix[13]=0.0;
		PositioningMatrix[2]=0.0;	PositioningMatrix[6]=0.0;	PositioningMatrix[10]=1.0;	PositioningMatrix[14]=0.0;
		PositioningMatrix[3]=0.0;	PositioningMatrix[7]=0.0;	PositioningMatrix[11]=0.0;	PositioningMatrix[15]=1.0;

		if (g_nWaitForInitKreonCallback!=-1)
		{
			app.saarm.Data.KreonProbeCals.selectedScannerCal = calSelectDlg.selectedCal;
			m_bKreonInitialized=true;
		}
	}
	ArmManager->BeginAcquisition(KR_ARM_MODE_CURRENT_POS);

	return m_bKreonInitialized;
}

int KreonCmdWnd::GetButtonCount()
{
	NbButtons = 0;
	ArmManager->GetParameter("NbButtons",&NbButtons);
	return NbButtons;
}
bool* KreonCmdWnd::GetButtonStatesFromCurrentPosition()
{
	// 2015.08.03 - modified this code. For some reason, after calling BeginAcquisition(KR_ARM_MODE_CURRENT_POS),
	// the only allowed arm-interrogation command is GetCurrentPosition/*AndJointAngles*/(matrix,buttons), 
	// not GetButtonState ??  Odd since GetButtonState is a sub set of GetCurrentPosition

	bool* bButtonState = new bool [GetButtonCount()];
	//ArmManager->GetButtonState(bButtonState);
	double* matrix = new double[16];
	TRACE0("GetCurPos 2\n");
	int Result = ArmManager->GetCurrentPosition(matrix,bButtonState);
	delete [] matrix;
	return bButtonState;
}
bool* KreonCmdWnd::GetButtonStatesFromTriggeredPosition()
{
	bool* bButtonState = new bool [GetButtonCount()];
	//ArmManager->GetButtonState(bButtonState);
	double* matrix = new double[16];
	TRACE0("GetTriggeredPosition 3\n");
	int Result = ArmManager->GetTriggeredPosition(matrix,bButtonState);
	delete [] matrix;
	return bButtonState;
}
bool* KreonCmdWnd::GetButtonStatesOnly()
{
	bool* bButtonState = new bool [NbButtons];
	if(ArmManager->GetButtonState(bButtonState)==KR_ARM_OK)
	{
		return bButtonState;
	}
	return NULL;
}
bool KreonCmdWnd::GetProbeTipPos(double* matrix)	// caller must make a new double[16] for this
{
	// should not have to do the following! ...
	//KR_TOUCH_PROBE Probe;
	//int numProbes = 0;
	//pOwner->GetProbeCals(Probe, numProbes);
	//ArmManager->SetParameter("Probe", &Probe);

	bool*	bButtonState = new bool [GetButtonCount()];
	TRACE0("GetCurPos 3\n");
	int Result = ArmManager->GetCurrentPosition(matrix,bButtonState);
	delete bButtonState;
	if(Result == KR_ARM_OK)
		return true;
	
	return false;
}
bool KreonCmdWnd::GetProbeTipPosAndJointAnglesAndButtonState(double* matrix, int * nbJoints, double ** jointAngles, bool * buttons)
{
	TRACE0("GetCurPos 1\n");
	int Result = ArmManager->GetCurrentPositionAndJointAngles(matrix, buttons, nbJoints, jointAngles, true);
	if(Result == KR_ARM_OK)
	{
		double probeDia = 6.0;
		if (ArmManager->GetParameter("ProbeDiameter",&probeDia))
		{
			if (app.saarm.Data.TipDia != probeDia / 1000.0)
			{
				app.saarm.Data.TipDia = probeDia / 1000.0;
				if(app.saarm.units.toolBarModeOn)
					app.saarm.SendSAIRCAttributesToSA(); // update the toolbar in case probe was changed while in toolbar mode
			}
		}
		return true;
	}

	return false;
}
void KreonCmdWnd::PopProbeCalWindow(KR_TOUCH_PROBE* probes, int probeSize) // 2017.10.19 - we are not going to use this until it becomes useful, see comments inside case KR_CMD_TP_CALIBRATION_WINDOW_CLOSED_EXT:
{
	if (!m_bKreonInitialized)
		if (!Init())
			return;

	//	Begin the arm
	if (!ArmManager->IsConnected())
	{
		if (ArmManager->BeginArm()!=KR_ARM_OK)
		{
			CString ErrMsg;
			ErrMsg.Format(_T("Error: unable to initialize the arm. Check the connection"));
			AfxMessageBox(ErrMsg);
			return;
		}
	}

	//	Open the touch probe calibration window (in this example, 2 probes are initially created)
	//KR_TOUCH_PROBE TouchProbe[2];
	//memset(TouchProbe,0,sizeof(TouchProbe));
	//strcpy_s(TouchProbe[0].Name,_countof(TouchProbe[0].Name),"FirstTouchProbe");
	//TouchProbe[0].Diameter=6;
	//strcpy_s(TouchProbe[1].Name,_countof(TouchProbe[1].Name),"SecondTouchProbe");
	//TouchProbe[1].Diameter=3;
	//SocketManager->ProbeOpenCalibrationWindow(TouchProbe,2,true);
	SocketManager->ProbeOpenCalibrationWindow(probes,probeSize,true, 25.0, true);

	// Don't return until the probe cal window is closed
	probeCalWindowClosed = false;
	while (!probeCalWindowClosed)
	{
		MSG Msg;
		if (PeekMessage(&Msg,NULL,0,0,PM_REMOVE))
		{
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
		Sleep(0);
	}
}

void KreonCmdWnd::PopScannerSphereCalWindow()
{
	if (isScanning)
		return;
	// OnKreontoolkitPausescan();

	if (!m_bKreonInitialized)
		if (!Init())
			return;

	//	Begin the arm
	if (!ArmManager->IsConnected())
	{
		if (ArmManager->BeginArm()!=KR_ARM_OK)
		{
			CString ErrMsg;
			ErrMsg.Format(_T("Error: Unable to initialize the arm. Please check the connection"));
			AfxMessageBox(ErrMsg);
			return;
		}
	}

	//	The arm and the ECU are correctly initialized.
	//	We need to inform the ECU about the duration of the external trigger.
	//	To get this information, we use the "GetParameter" method of CArmManager
	//	(see ArmManager.h)
	WORD TriggerDuration;
	ArmManager->GetParameter("TriggerDuration",&TriggerDuration);
	SocketManager->SetExternalTriggerDuration(TriggerDuration);

	BYTE nbIndex = 1;	// For an arm, we use only one index
	KR_INDEX_EXT myIndex;
	memset(&myIndex,0,sizeof(myIndex));
	memcpy(myIndex.PositioningMatrix,PositioningMatrix,16*sizeof(double));
	strcpy_s(myIndex.Name,_countof(myIndex.Name),"MyIndex");
	SocketManager->OpenPositioningWindowExt(&myIndex,nbIndex,
		KR_SCAN_METHOD_SYNCHRONOUS,true,15,0,false,false,m_hWnd,false,0xFF);
}

bool KreonCmdWnd::VideoSetup()
{
	if(!m_bKreonInitialized)
		if(!Init())
			return false;

	SocketManager->OpenVideoSettingWindow(true);
	return true;
}
bool KreonCmdWnd::ScannerProperties()
{
	if(!m_bKreonInitialized)
		if(!Init())
			return false;

	SocketManager->OpenScannerPropertiesWindow(m_hWnd);
	return true;
}

bool KreonCmdWnd::StartScan() 
{
	if (!m_bKreonInitialized)
		if (!Init())
			return false;

	//	Begin the arm
	if (!ArmManager->IsConnected())
	{
		if (ArmManager->BeginArm()!=KR_ARM_OK)
		{
			CString ErrMsg;
			ErrMsg.Format(_T("Error : unable to initialize the arm. Check the connection"));
			AfxMessageBox(ErrMsg);
			return false;
		}
	}
	else
	{
		// If you started a timer and button interrogation when the acquisition was paused by pressing an arm button the window,
		// and if the user did not perform any action that stopped that timer/interrogation,
		// then, those should be stopped here:
		KillTimer(ARM_BUTTONS_TIMER_ID);
		if (ArmManager->GetAcquisitionMode() != KR_ARM_MODE_IDLE)
			ArmManager->EndAcquisition();
	}

	//	The arm and the ECU are correctly initialized.
	//	We need to inform the ECU about the duration of the external trigger.
	//	To get this information, we use the "GetParameter" method of CArmManager
	//	(see ArmManager.h)
	WORD TriggerDuration;
	ArmManager->GetParameter("TriggerDuration",&TriggerDuration);
	SocketManager->SetExternalTriggerDuration(TriggerDuration);

	int  Frequency = 20;
	float Step=0.2f;
	UINT  Decimation = 1;
	// 2009.04.27 - we have to set the calibration matrix!
	double calMatrix[4][4];
	pOwner->GetScannerCalMatrix(calMatrix);
	PositioningMatrix[0] = calMatrix[0][0];	PositioningMatrix[4] = calMatrix[0][1];	PositioningMatrix[8] = calMatrix[0][2];	PositioningMatrix[12] = calMatrix[0][3];
	PositioningMatrix[1] = calMatrix[1][0];	PositioningMatrix[5] = calMatrix[1][1];	PositioningMatrix[9] = calMatrix[1][2];	PositioningMatrix[13] = calMatrix[1][3];
	PositioningMatrix[2] = calMatrix[2][0];	PositioningMatrix[6] = calMatrix[2][1];	PositioningMatrix[10] = calMatrix[2][2];	PositioningMatrix[14] = calMatrix[2][3];
	PositioningMatrix[3] = calMatrix[3][0];	PositioningMatrix[7] = calMatrix[3][1];	PositioningMatrix[11] = calMatrix[3][2];	PositioningMatrix[15] = calMatrix[3][3];
	SocketManager->InitAcquisition(KR_SCAN_METHOD_SYNCHRONOUS,
		Frequency,	// not used in synchronous mode
		Step, PositioningMatrix, Decimation);
	ArmManager->BeginAcquisition(KR_ARM_MODE_SYNCHRONOUS);	// synchronous scan mode
	NbLines =0;
	isScanning = true;
	SocketManager->BeginAcquisition();
	pOwner->scanMsg = Tr("Scanning:");
	pOwner->OnUpdateScanDlg();
	pOwner->scanMsg = Tr("Pause- Pull and hold Trigger or hold Green button.");
	pOwner->OnUpdateScanDlg();
	pOwner->scanMsg = Tr("Stop- Push Trigger or press Yellow button.");
	pOwner->OnUpdateScanDlg();
	return true;
}
bool KreonCmdWnd::IsScanning()
{
	return isScanning;
}
bool KreonCmdWnd::StopScan()
{
	KillTimer(ARM_BUTTONS_TIMER_ID);
	SocketManager->EndAcquisition();
	ArmManager->EndAcquisition();
	isScanning=false;
	return true;
}
LRESULT KreonCmdWnd::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	if(message == CLOSE_SCAN_DLG)
		owner.trackScanDlg.CloseDialog();

	//if(message == SEND_BUFFERED_DATA_AND_RECONNECT)
	//	owner.SendBufferedCloudDataAndReconnect();

	if(message == UPDATE_SCAN_DLG)
		owner.OnUpdateScanDlg();

	return CWnd::WindowProc(message, wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
LRESULT KreonCmdWnd::OnDataSocket(WPARAM /*wParam*/,LPARAM /*lParam*/)
{
	SocketManager->OnDataSocket();
	return 0L;
}