#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"
#include "KreonScannerArm.h"
#include "Kreon/KreonCmdWnd.h"
#include "Kreon/DKreonScannerOptions.h"
#include "Kreon/KreonProbeCalibrations.h"
#include "DExternalScannerMeas.h"


KreonScannerArm::KreonScannerArm(void)
: cw(*new KreonCmdWnd(this, app.saarm.Data.doconnection ? false : true))
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;

	buttontime = 4294967295;
	LongDown = FALSE;

	app.saarm.SettingsFileName = _T("SA_Kreon_ArmSettings.bin");
	LogFileName = _T("KreonArm.log");

	// default hard probe calibration
	KR_TOUCH_PROBE Probe = {"Probe", -10.0,-1.0,116.0, -1.0 , 4.0, false};
	AddOrEditProbeCal(Probe);

	app.saarm.Data.autoCorrectModel = FALSE;
}

KreonScannerArm::~KreonScannerArm(void)
{
	delete &cw;
}

bool KreonScannerArm::ConnectToArm(const int dof)
{
	//if(!app.saarm.Data.doconnection || ConnectedToArm)
	//	return true;	// simulation mode

	DOF = dof;
	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);
	JShift.Create(dof,TRUE);

	J.x[0] = 124.9655;
	J.x[1] = 22.431;
	J.x[2] = 1.3425;
	J.x[3] = -170.9805;
	J.x[4] = -6.7595;
	J.x[5] = -26.324;
	J.x[6] = 0.0;

	if(!cw.windowValid)
	{
		if(!cw.MakeValidWindow())
		{
			CString m;
			m.Format(Tr("Critical Error.\nCould not start Kreon Command Message Window.\n\nEnsure the latest Kreon SA Plugin is installed to this SA.\nPlease try again."));
			AfxMessageBox(m, MB_ICONHAND);
			return false;
		}
	}

	if(!cw.Init())
	{
		CString m;
		m.Format(Tr("Critical Error.\nCould not Initialize Kreon Command Message Window.\n\nEnsure the latest Kreon SA Plugin is installed to this SA.\nPlease try again."));
		AfxMessageBox(m, MB_ICONHAND);
		cw.ShutDown();
		return false;
	}

	ConnectedToArm = TRUE;
	return true;
}
BOOL KreonScannerArm::DisconnectFromHardware()
{
	//if(!ConnectedToArm)
	//	return app.saarm.DisconnectFromHardware();

	cw.ShutDown();
	return TRUE;
}

void KreonScannerArm::DoPostInitOperations()
{
	if(app.saarm.InstrumentModel == 104 || app.saarm.InstrumentModel == 105 || app.saarm.InstrumentModel == 106 || app.saarm.InstrumentModel == 146) // Axxis arms
	{
		app.saarm.Data.autoCorrectModel = FALSE;	// we can't read the 7th axis!  Requested this 
	}
}

void KreonScannerArm::OpenKreonDongleLicenseDlg()
{
	cw.OpenLicenseDlg();
}

BOOL KreonScannerArm::GetAllData()
{
	//if( !ConnectedToArm )
	//	return ArmBase::GetAllData();

	GetPos();
	//GetButtonStates();
	return TRUE;
}
BOOL KreonScannerArm::GetPos()
{
	//if( !ConnectedToArm )
	//	return ArmBase::GetPos();

	double* matrix = new double[16];
	int nJts;
	double* jtAngles;	// points to static array in kreon's dll's
	bool*	bButtonState = new bool [cw.GetButtonCount()];
	if(cw.GetProbeTipPosAndJointAnglesAndButtonState(matrix, &nJts, &jtAngles, bButtonState))
	{
		/*
		// 2017.10.27 - KarlMaritaud says the probe cal. is already taken into account for all Kreon arms...
		// The following is only necessary for non-Kreon arms, and we only support Kreon arms in this interface.
		KR_TOUCH_PROBE probe;
		int numProbes = 0;
		GetProbeCals(probe, numProbes);

		// Position and Orientation....
		// Kreon's XYZ of the current probe cal is the offset of the
		// probe in the coordinate system of the tip of arm,
		// so we have to multiply that vector by the current position matrix...
		X = matrix[0]*probe.X + matrix[4]*probe.Y + matrix[ 8]*probe.Z
			+ matrix[12];
		Y = matrix[1]*probe.X + matrix[5]*probe.Y + matrix[ 9]*probe.Z
			+ matrix[13];
		Z = matrix[2]*probe.X + matrix[6]*probe.Y + matrix[10]*probe.Z
			+ matrix[14];
		*/

		// 2017.10.27 - so this is all we gotta do
		X = matrix[12];
		Y = matrix[13];
		Z = matrix[14];

		X /= 1000.0;
		Y /= 1000.0;
		Z /= 1000.0;

		II = matrix[8];
		JJ = matrix[9];
		KK = matrix[10];

		CurrentTransform.T[0][0] = matrix[0];
		CurrentTransform.T[1][0] = matrix[1];
		CurrentTransform.T[2][0] = matrix[2];

		CurrentTransform.T[0][1] = matrix[4];
		CurrentTransform.T[1][1] = matrix[5];
		CurrentTransform.T[2][1] = matrix[6];

		CurrentTransform.T[0][2] = matrix[8];
		CurrentTransform.T[1][2] = matrix[9];
		CurrentTransform.T[2][2] = matrix[10];

		CurrentTransform.T[0][3] = X;
		CurrentTransform.T[1][3] = Y;
		CurrentTransform.T[2][3] = Z;

		if(nJts == 7)
		{
			J.x[0] = -jtAngles[0];
			J.x[1] = -90.0-jtAngles[1];
			J.x[2] = jtAngles[2];
			J.x[3] = -jtAngles[3];
			J.x[4] = jtAngles[4];
			J.x[5] = -jtAngles[5];
			J.x[6] = 90.0+jtAngles[6];
			//TRACE2("1st joint angle = %f\n7th joint angle = %f\n\n", -jtAngles[0], jtAngles[6]);
			//TRACE0("Our values now set to...\n");
			//J.Debug();
		}
		else if(nJts == 6)
		{
			J.x[0] = -jtAngles[0];
			J.x[1] = -90.0-jtAngles[1];
			J.x[2] = jtAngles[2];
			J.x[3] = -jtAngles[3];
			J.x[4] = jtAngles[4];
			J.x[5] = -jtAngles[5];
			//TRACE2("1st joint angle = %f\n6th joint angle = %f\n\n", -jtAngles[0], jtAngles[5]);
			//TRACE0("Our values now set to...\n");
			//J.Debug();
		}

		delete [] matrix;
		ProcessButtonStates(bButtonState);
		return TRUE;
	}
	delete [] matrix;
	ProcessButtonStates(bButtonState);
	return FALSE;
}

int KreonScannerArm::GetButtonCount()
{
	//if( !ConnectedToArm )
	//	return 2;

	return cw.GetButtonCount();
}

int KreonScannerArm::ProcessButtonStates(bool* pState)
{
	buttonstate = 0;
	if(overrideArmButtons || !app.pA->armButtons.ButtonsEnabled())
		return buttonstate;
	bool* buttons = pState;

	TRACE2("button1 %d, button2 %d\n", buttons[0], buttons[1]);
	if(!(buttons[0]) && !(buttons[1]))  //no buttons
	{  
		buttonstate = 0;
		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
		if( LastButton == 1 && !LongDown)
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			Beep(1000,100);
		}
		if( LastButton == 2 && !LongDown)
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			Beep(800,100);
		}
		if(LastButton == 3 && !LongDown)
		{
			buttonstate = 4;
			buttonstate += BUTTON_SHORT;
			Beep(600,100);
		}

		LastButton = 0;
		LongDown = FALSE;
	}
	if(buttons[0])	// trigger push, side, yellow
	{  
		TRACE0("Front button hit\n");
		buttonstate = 1;
		if( LastButton == 2 || LastButton == 3 )
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			buttontime = 4294967295;
			LastButton = 1;
		}
		if( LastButton != 1 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 1;
		}
		else if( LastButton == 1 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				MessageBeep(-1);
				Beep(1200,200);
				TRACE0("LongDown just set to TRUE, front button.\n");
			}
		}
	}
	else if(buttons[1])	// trigger pull, center, or green button
	{  
		TRACE0("Middle button hit\n");
		buttonstate = 2;
		if( LastButton == 1 || LastButton == 3 )
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			buttontime = 4294967295;
			LastButton = 2;
		}
		if( LastButton != 2 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 2;
		}
		else if( LastButton == 2 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				//	TRACE0("LongDown just set to TRUE, back button.\n");
			}
		}
	}
	delete [] buttons;
	return buttonstate;
}

int KreonScannerArm::GetButtonStates()
{
	//if( !ConnectedToArm )
	//	return ArmBase::GetButtonStates();

	buttonstate = 0;
	if(overrideArmButtons || !app.pA->armButtons.ButtonsEnabled())
		return buttonstate;
	bool* buttons = cw.GetButtonStatesFromCurrentPosition();	// cw.GetButtonStates() allocates memory!
	TRACE2("button1 %d, button2 %d\n", buttons[0], buttons[1]);
	if(!(buttons[0]) && !(buttons[1]))  //no buttons
	{  
		buttonstate = 0;
		buttontime = 4294967295;	// 'filled up' DWORD, 2^(sizeof(DWORD)) - 1
		if( LastButton == 1 && !LongDown)
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			Beep(1000,100);
		}
		if( LastButton == 2 && !LongDown)
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			Beep(800,100);
		}
		if(LastButton == 3 && !LongDown)
		{
			buttonstate = 4;
			buttonstate += BUTTON_SHORT;
			Beep(600,100);
		}

		LastButton = 0;
		LongDown = FALSE;
	}
	if(buttons[0])	// trigger push, side, yellow
	{  
		TRACE0("Front button hit\n");
		buttonstate = 1;
		if( LastButton == 2 || LastButton == 3 )
		{
			buttonstate = 2;
			buttonstate += BUTTON_SHORT;
			buttontime = 4294967295;
			LastButton = 1;
		}
		if( LastButton != 1 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 1;
		}
		else if( LastButton == 1 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				MessageBeep(-1);
				Beep(1200,200);
				TRACE0("LongDown just set to TRUE, front button.\n");
			}
		}
	}
	else if(buttons[1])	// back button (green)
	{  
		TRACE0("Middle button hit\n");
		buttonstate = 2;
		if( LastButton == 1 || LastButton == 3 )
		{
			buttonstate = 1;
			buttonstate += BUTTON_SHORT;
			buttontime = 4294967295;
			LastButton = 2;
		}
		if( LastButton != 2 )
		{
			buttontime = ::GetCurrentTime();
			LastButton = 2;
		}
		else if( LastButton == 2 )
		{
			if( !LongDown && (::GetCurrentTime() - buttontime) >= LONG_DURATION )
			{
				buttonstate += BUTTON_LONG;
				buttontime = 4294967295;
				LastButton = 0;
				LongDown = TRUE;
				//	TRACE0("LongDown just set to TRUE, back button.\n");
			}
		}
	}
	delete [] buttons;
	return buttonstate;
}

void KreonScannerArm::DoCal()
{
	if( !ConnectedToArm )
	{
		ArmBase::DoCal();
		return;
	}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.1,TRUE);

	bool* buttons = cw.GetButtonStatesOnly();
	if(buttons != NULL)
	{
		while(buttons[0] || buttons[1])
		{
			delete [] buttons;
			Delay(0.1, TRUE);	// just wait - user may have started mode with a button, and we don't want to pause the scan before it gets started
			buttons = cw.GetButtonStatesOnly();
			if(buttons == NULL)
				continue;
		}
		delete [] buttons;
	}

	Delay(0.5,TRUE);

	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//d.StopQueryTimer();
	//Delay(0.5,TRUE);

	cw.PopScannerSphereCalWindow();
}
void KreonScannerArm::DoArmSettings()
{
	ProbeCalSet();
}
void KreonScannerArm::RestartQueryTimer()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	Sleep(500);
	ClearButtonState();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
}
void KreonScannerArm::ProbeCalSet()
{
	if( !ConnectedToArm )
		return ArmBase::ProbeCalSet();

	::MessageBox(NULL, _T("The .tab file and arm Profile you selected at startup defines the current probe.\nRight click the Kreon icon in the tool tray to access Probe Management.\nThen select the appropriate Profile upon arm initialization.\nSee KreonArm WIZARD User's Guide from your hardware rep. for more information.\nNOTE: If you need to change from 6dof to 7dof or vice versa, you will need to close the interface, add the appropriate arm, and restart the interface."), _T("Probe Management"), MB_ICONINFORMATION | MB_SETFOREGROUND | MB_TOPMOST);
	// 2017.10.19 - we are not going to use the below until it becomes useful, see comments inside case KR_CMD_TP_CALIBRATION_WINDOW_CLOSED_EXT:
	//KR_TOUCH_PROBE* probes = new KR_TOUCH_PROBE[app.saarm.Data.KreonProbeCals.count];
	//probes = app.saarm.Data.KreonProbeCals.probe;
	//cw.PopProbeCalWindow(probes, app.saarm.Data.KreonProbeCals.count);
}
int KreonScannerArm::AddOrEditProbeCal(KR_TOUCH_PROBE probe)
{
	app.saarm.Data.TipDia = probe.Diameter/1000.0;
	return app.saarm.Data.KreonProbeCals.AddProbe(probe);
}
void KreonScannerArm::GetProbeCals(KR_TOUCH_PROBE& probe, int& probeSize)
{
	probe = app.saarm.Data.KreonProbeCals.probe[0];
	probeSize = app.saarm.Data.KreonProbeCals.count;
	//KR_TOUCH_PROBE TouchProbe[probeSize];
	//memset(TouchProbe,0,sizeof(TouchProbe));
}

BOOL KreonScannerArm::ScannerSettings()
{
	if( !app.saarm.Data.doconnection )
		return ArmBase::ScannerSettings();

	DKreonScannerOptions dlg(*this);
	DOMODAL(dlg);
	return TRUE;
}
bool KreonScannerArm::VideoSetup()
{
	return cw.VideoSetup();
}
bool KreonScannerArm::ScannerProperties()
{
	return cw.ScannerProperties();
}

void KreonScannerArm::SetScannerCalMatrix(const double calMatrix[4][4])
{
	app.saarm.Data.kreonScannerCalMatrix.Set(calMatrix);
	app.saarm.Data.kreonScannerCalMatrix.Debug();
}
void KreonScannerArm::GetScannerCalMatrix(double calMatrix[4][4]) const
{
	app.saarm.Data.kreonScannerCalMatrix.Get(calMatrix);
	app.saarm.Data.kreonScannerCalMatrix.Debug();
}

BOOL KreonScannerArm::StartScannerMeas()
{
	//if( !app.saarm.Data.doconnection )
	//	return ArmBase::StartScannerMeas();

	if(!app.saarm.scannerLicenseValid)
	{
		AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
		return FALSE;
	}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.1,TRUE);

	bool* buttons = cw.GetButtonStatesFromTriggeredPosition();
	while(buttons[0] || buttons[1])
	{
		delete [] buttons;
		Delay(0.1, TRUE);	// just wait - user may have started mode with a button, and we don't want to pause the scan before it gets started
		buttons = cw.GetButtonStatesFromTriggeredPosition();
	}
	delete [] buttons;

	d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

	DExternalScannerMeas* pnewdlg = new DExternalScannerMeas((ArmBase&)*this, trackScanDlg);
	DOMODELESS(pnewdlg); // OR the normal way:  pnewdlg->Create(pnewdlg->IDD,NULL);

	thinningFactor = app.saarm.Data.thinningFactor;
	return cw.StartScan() ? TRUE : FALSE;
}
BOOL KreonScannerArm::StopScannerMeas()
{
	if( !app.saarm.Data.doconnection )
		return ArmBase::StopScannerMeas();

	if(!cw.IsScanning())
		return TRUE;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	CloseScanDlg();
	BOOL ret = cw.StopScan() ? TRUE : FALSE;
	CheckAndSendScannerCloud(0, true);
	Sleep(500);
	ClearButtonState();
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return ret;
}
void KreonScannerArm::PostUpdateScanDlg(CString msg)
{
	if( !ConnectedToArm )
		return ArmBase::PostUpdateScanDlg(msg);
	TRACE1("Post Message to update scan dlg with text '%s'\n", msg);
	scanMsg = msg;
	cw.PostMessage(UPDATE_SCAN_DLG, 0, 0);
}
void KreonScannerArm::OnUpdateScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	if(((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.GetCount() > 100)
		((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.ResetContent();
	int idx = ((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.AddString(scanMsg);
	((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.SetTopIndex(idx);
}
void KreonScannerArm::CloseScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	trackScanDlg.CloseDialog();
	//TRACE0("Post Message to close scan dlg...\n");
	//cw.PostMessage(CLOSE_SCAN_DLG);
}