#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMMDlg.h"

#include "MetrisGardaArm.h"
#include "MetrisMCA/cms_socket.h"

MetrisGardaArm::MetrisGardaArm(void)
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;
}

MetrisGardaArm::~MetrisGardaArm(void)
{
}

bool MetrisGardaArm::ConnectToArm(const int dof)
{
	return false;	// 2009.10.09 - Metris now supports the MCA via their HHAPI interface (our Metris K-Series interface)

	if(!app.saarm.Data.doconnection || ConnectedToArm)	// 2001.03.23
		return true;	// simulation mode

	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);

	// start up connection to MUX-X-PLUS
	if (cms_startup(NULL) != 1)
	{
		AfxMessageBox(Tr("Startup Error!\nIs GSuit installed and running?"), MB_ICONSTOP);
		return false;
	}

	Connect();
	//Delay(1.5,TRUE);


	cms_ConnectionState(pConnInfo);	// we get isConn == 1 even without an arm!!
	if(pConnInfo->isConn != 1)
	{
		AfxMessageBox(Tr("Arm Connection Error!\nIs the arm connected and on?"), MB_ICONSTOP);
		return false;
	}

	int unUsedValue = 0;
	if( !cms_Azzeramento(&unUsedValue) )
	{
		ShowTestWindow();
	}
	//if( !cms_Azzeramento(&unUsedValue) )
	//{
	//	AfxMessageBox(Tr("Arm Homing or Connection Error!"), MB_ICONSTOP);
	//	HideTestWindow();
	//	DisconnectFromHardware();
	//	return false;
	//}

	ConnectedToArm = TRUE;
	return true;
}
BOOL MetrisGardaArm::DisconnectFromHardware()
{
	Disconnect();

	cms_Stop(NULL);

	return TRUE;
}

int MetrisGardaArm::GetButtonCount()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonCount();

	return 3;
}

BOOL MetrisGardaArm::GetAllData()
{
	if( !ConnectedToArm )
		return ArmBase::GetAllData();

	GetPos();
	GetJoints();
	if(!overrideArmButtons && app.pA->armButtons.ButtonsEnabled())
		GetButtonStates();

	return TRUE;
}
BOOL MetrisGardaArm::GetPos()
{
	if( !ConnectedToArm )
		return ArmBase::GetPos();

	if(cms_xyz_PI(pCms_Xyz) <= 0)
		return FALSE;

	X = pCms_Xyz->XYZ[0]/1000.0;
	Y = pCms_Xyz->XYZ[1]/1000.0;
	Z = pCms_Xyz->XYZ[2]/1000.0;

	TRACE3("Raw Data: X=%f,Y=%f,Z=%f\n",X,Y,Z);

	// the normal vector
	Vector armNorm;
	Vector extPt(pCms_Xyz->EXT[0]/1000.0, pCms_Xyz->EXT[1]/1000.0, pCms_Xyz->EXT[2]/1000.0);
	Vector tipCenter(X, Y, Z);

	armNorm = tipCenter - extPt;
	armNorm.Normalize();
	II = armNorm.x;
	JJ = armNorm.y;
	KK = armNorm.z;

	Vector firstAxis(0.0, 1.0, 0.0);
	Vector xAxis = firstAxis * armNorm;
	Vector yAxis = armNorm * xAxis;

	CurrentTransform.SetAxes(xAxis, yAxis, armNorm);
	CurrentTransform.SetTranslation(Vector(X,Y,Z));

	return TRUE;
}
BOOL MetrisGardaArm::GetJoints(bool simulate/*=true*/)
{
	if( !ConnectedToArm )
		return ArmBase::GetJoints(simulate);

	//return TRUE;

	// no way to get arm joints
	// for now, we're using the CimCore 3024 model.  these are 'home' values....
	//J.x[0] = 0.0;
	//J.x[1] = 180.0;
	//J.x[2] = 0.0;
	//J.x[3] = 180.0;
	//J.x[4] = 0.0;
	//J.x[5] = 100.0;

	// 2008.04.16 - not any more, the guys at Garda added GetAnglesJoint....

	int angles[8];
	int joints;
	int stato=GetAnglesJoint(&angles[1],&joints);

	for(int i=1; i<7; i++)
	{
		while(angles[i] > 360.0) angles[i] -= 360.0;
		while(angles[i] < 0.0) angles[i] += 360.0;
	}

	J.x[0] = 180.0 - angles[1];
	J.x[1] = 90.0 - angles[2];
	J.x[2] = angles[3] * -1.0;
	J.x[3] = 90.0 - angles[4];
	J.x[4] = angles[5];
	J.x[5] = -angles[6];

	//app.saarm.Data.alwaysupdateSA = TRUE;

	TRACE0("Joint angles:\n");
	TRACE1("1 = %f\n", J.x[0]);
	TRACE1("2 = %f\n", J.x[1]);
	TRACE1("3 = %f\n", J.x[2]);
	TRACE1("4 = %f\n", J.x[3]);
	TRACE1("5 = %f\n", J.x[4]);
	TRACE1("6 = %f\n\n", J.x[5]);

	return TRUE;
}
int MetrisGardaArm::GetButtonStates()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonStates();

	int duration=0;
	buttonstate = 0;

	if( CheckButton(1, &duration) )
	{
		buttonstate = 1;
		buttonstate += duration;
	}
	else if( CheckButton(2, &duration) )
	{
		buttonstate = 2;
		buttonstate += duration;
	}
	else if( CheckButton(3, &duration) )
	{
		buttonstate = 4;
		buttonstate += duration;
	}	
	TRACE1("returning button state %d\n", buttonstate);
	return buttonstate;
}
BOOL MetrisGardaArm::CheckButton(int but, int* duration)
{
	if( !ConnectedToArm )
		return ArmBase::CheckButton(but, duration);

	if(IsButtonDown(but))
	{
		TRACE1("Button %d down\n",but);
		DWORD Start = ::GetCurrentTime();
		while((IsButtonDown(but))&&((::GetCurrentTime() - Start) < LONG_DURATION))
		{
		}
		if(IsButtonDown(but))
		{
			*duration = BUTTON_LONG;
			OutputSpeakerSound(3000,100);
			while(IsButtonDown(but))
			{
			}
			return TRUE;
		}
		else if(!IsRemoteMouseEnabled())
			*duration = BUTTON_SHORT;
	}
	else
		return FALSE;
	return TRUE;
}
bool MetrisGardaArm::IsButtonDown(int but)
{
	bool b1, b2, b3;
	cms_Buttons(&b1, &b2, &b3);
	if(but == 1)
		return b1;
	if(but == 2)
		return b2;
	if(but == 3)
		return b3;
	return false;
}

BOOL MetrisGardaArm::ValidateCurrentProbe()
{
	if( !ConnectedToArm )
		return ArmBase::ValidateCurrentProbe();

	return TRUE;

	ProbeInfo ArProbe[10];
	for(int c=0;c<6;c++)
	{
		ArProbe[c].Renishaw=0;
		ArProbe[c].Radius =0;
		ArProbe[c].Length=0;
	}


	cms_ProbeInfo(ArProbe);
	//	Get the active probe ID code
	//	int id = GetProbe();

	//ArProbe[0].Length
	//ArProbe[0].Renishaw
	//ArProbe[0].Radius

	app.saarm.Data.TipDia = ArProbe[0].Radius * 2.0 / 1000.0;
	TRACE1("after cms_ProbeInfo, dia. = %f\n", app.saarm.Data.TipDia);
	probeValid = true;
	return TRUE;
}

void MetrisGardaArm::DoCal()
{
	ShowTestWindow();
}

BOOL MetrisGardaArm::IsRemoteMouseEnabled()
{
	cms_ConnectionState(pConnInfo);
	if(pConnInfo->isMouseMode == 1)
		return TRUE;
	return FALSE;
}
void MetrisGardaArm::ToggleRemoteMouse()
{
	if(IsRemoteMouseEnabled())
		StopMouseMode();
	else
		StartMouseMode();
}
void MetrisGardaArm::RemoteMouse(BOOL flag)
{
	if(flag)
		StartMouseMode();
	else
		StopMouseMode();
}
void MetrisGardaArm::DoArmSettings()
{

}

BOOL MetrisGardaArm::IsRomer()
{
	return FALSE;
}
BOOL MetrisGardaArm::IsFaro()
{
	return FALSE;
}
BOOL MetrisGardaArm::IsFaroUSB()
{
	return FALSE;
}
BOOL MetrisGardaArm::IsFrenchRomer()
{
	return FALSE;
}
BOOL MetrisGardaArm::IsSNL()
{
	return FALSE;
}
BOOL MetrisGardaArm::IsMetrisGarda()
{
	return TRUE;
}