#include "stdafx.h"
#include "ModelInfo.h"
#include "../DPCMM.h"

ModelInfo::ModelInfo(DWORD_PTR _bufferSize)
: bufferSize(_bufferSize)
, offsetData(*new SAIPJointOffsetData)
{
	minCheckDelta = 0.25;	// inches
}

ModelInfo::~ModelInfo(void)
{
	ClearInfo();
	delete &offsetData;
}

void ModelInfo::ClearInfo()
{
	minCheckDelta = 0.25;
	offsetData.observations.DeleteAll();
}

bool ModelInfo::CheckUpdate(const JointOffsetCalItem& data)
{
	//TRACE0("ModelInfo::CheckUpdate, pos in:\n");
	//data.Point.Debug();
	//TRACE0("lastCheckedPos:\n");
	//lastCheckedPos.Debug();
	// if haven't moved far enough, just leave
	if( lastCheckedPos.Distance(data.Point) < minCheckDelta )
		return false;
	lastCheckedPos = data.Point;

	// if we don't have the requisite data, just add and leave
	if(offsetData.observations.GetSize() < bufferSize)
	{
		offsetData.observations.AddCopy(data);
		return false;
	}

	// currentOffsetData is our member set augmented with the input datum
	SAIPJointOffsetData currentOffsetData(offsetData);
	currentOffsetData.observations.AddCopy(data);

	// 2010.9.22 - added serial num so SA can persist model correction
	offsetData.instrumentSerialNumber = app.pA->CMMSerialNum;

	// trim back down to the correct buffer size by getting farthest Cartesian positions, and return true IF the new point was added
	VectorArray currentPoints;
	GetPoints(currentOffsetData, currentPoints);
	CDWordArray indices;
	bool ret = false;
	if(currentPoints.GetFarthestPoints(bufferSize, indices))
	{
		offsetData.observations.DeleteAll();
		for(INT_PTR i=0; i<indices.GetSize(); i++)
		{
			if(indices[i] == bufferSize)	// the new pt. is now among the farthest pts.
			{
				minCheckDelta *= 2.0;		// increase the minimum distance
				if(minCheckDelta > 12.0)	// cap it
					minCheckDelta = 12.0;
				ret = true;
			}
			offsetData.observations.AddCopy(currentOffsetData.observations[indices[i]]);
		}
	}
	currentOffsetData.observations.DeleteAll();
	return (ret && offsetData.observations.GetSize() == bufferSize);
}

void ModelInfo::GetPoints(const SAIPJointOffsetData& jOffsetData, VectorArray& pts)
{
	for(DWORD_PTR i=0; i<jOffsetData.observations.GetSize(); i++)
	{
		pts.Add(jOffsetData.observations[i].Point);
	}
}

void ModelInfo::ModelInfoUpdateToSpan()
{
	//TRACE1("sending jt. offset data packet with %d observations to SA...\n", offsetData.observations.GetSize());
	for(DWORD_PTR i=0; i<offsetData.observations.GetSize(); i++)
	{
		//TRACE1("observation %d:\n", i);
		//offsetData.observations[i].Point.Debug();
		//offsetData.observations[i].JointValues.Debug();
		Transform t;
		offsetData.observations[i].GetTransform(t);
		//t.Debug();
	}
	app.saarm.QS.SendPacket(offsetData);
}
