// RemoteUIManager.cpp: implementation of the CRemoteUIManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RemoteUIManager.h"
#include "RemoteMessage.h"
#include "UIMessage.h"
#include "StatusMessage.h"
#include "RemoteWnd.h"
#include "RemoteUIDefs.h"
#include "RemoteCommDefs.h"
#include "SocketManager.h"
#include "RemoteButton.h"
#include "RemoteEdit.h"
#include "RemoteStatic.h"
#include "RemoteListBox.h"
#include "RemoteCommand.h"
#include "BLOBMessage.h"
#include "UIReplyMessage.h"

//#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// ////////////////////////////////////////////////////////
//
// CreateRemoteWindow
//
// Construct a message to send to the remote peer,
//  instructing it to create the window based on information
//  passed in the pWndRemote parameter.
//
BOOL CRemoteUIManager::CreateRemoteWindow(CRemoteWnd *pWndRemote)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nControlType = pWndRemote->m_nWndType;
	pUI->m_nUIMessageType = CUIMessage::UMT_CREATE;
	pUI->m_nControlID = pWndRemote->m_nID;
	pUI->m_strCaption = pWndRemote->m_strCaption;
	pUI->m_nStyle = pWndRemote->m_dwStyle;
	pUI->m_wndRect = pWndRemote->m_wndRect;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	// an error occured
	return false;
}

// ////////////////////////////////////////////////////////
//
// CreateRemoteBing
//
// Send a message to 'bing' the speaker on the remote
//  system.
//
BOOL CRemoteUIManager::CreateRemoteBing()
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_BING;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	// an error occured
	return false;
}

// ////////////////////////////////////////////////////////
//
// CreateRemoteBitmap
//
// Remotely create the Bitmap passed in by pointer.
//
BOOL CRemoteUIManager::CreateRemoteBitmap( UINT nBitmapID, UINT nStaticID )
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CBitmap* pbmImage = new CBitmap;
	if ( pbmImage->LoadBitmap(nBitmapID) == 0 )
		return FALSE;
	CBLOBMessage* pBLOB = new CBLOBMessage;

	// populate the message
	pBLOB->m_pImage = pbmImage;
	pBLOB->m_nStaticID = nStaticID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pBLOB) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	// an error occured
	return false;
}

// ////////////////////////////////////////////////////////
//
// CreateRemoteBitmap
//
BOOL CRemoteUIManager::CreateRemoteBitmap( CBitmap* pBitmap, UINT nStaticID )
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CBLOBMessage* pBLOB = new CBLOBMessage;

	// populate the message
	pBLOB->m_pImage = pBitmap;
	pBLOB->m_nStaticID = nStaticID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pBLOB) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	// an error occured
	return false;
}

// ////////////////////////////////////////////////////////
//
// CreateRemoteDialog
//
// Load the dialog's template, whose ID is specified, into
//  memory.  Walk through the list of controls, and attempt
//  to creeate each one remotely.
//
BOOL CRemoteUIManager::CreateRemoteDialog(int nIDD)
{
	RemoteContext = 0;  // jmc
	HRSRC hResourceBlock;
	HGLOBAL hResource;
	BYTE* pDlgResource;

	// "load and lock" the dialog resource
	hResourceBlock = ::FindResource( NULL, MAKEINTRESOURCE(nIDD), RT_DIALOG );
	if ( hResourceBlock != NULL ) {
		hResource = ::LoadResource( NULL, hResourceBlock );
		if ( hResource != NULL ) {
			pDlgResource = (BYTE *) ::LockResource( hResource );
			if ( pDlgResource == NULL )
				return FALSE;
		}
	}

	// *** MAKE A REALLY BIG, LARGE, HUGE ASSUMPTION HERE ***
	//
	// We assume that this dialog is not an 'extended' dialog,
	//  that is, if you look in the .rc file for the defintion
	//  of the dialog, it reads as follows ...
	//
	// IDD_THEDIALOGINQUESTON DIALOG
	//
	// ... and does NOT read as follows ...
	//
	// IDD_THEDIALOGINQUESTION DIALOGEX
	//
	// In other words, it is a 'DIALOG', and not a 'DIALOGEX'.
	//
	// Really bad things will happen if this is DIALOGEX!!!!
	//

	// ***** in this section, we through the dialog header *****
	CString strCaption;
	CString strFont;
	CString strMenu;
	CString strClass;

    // grab the style for later use, then walk on ...
    DWORD style = *(LPDWORD) pDlgResource;
    pDlgResource += sizeof(DWORD);
    
    // grab the extended style for later use, then walk on ...
    DWORD exStyle = *(LPDWORD) pDlgResource;
    pDlgResource += sizeof(DWORD);
    
	// grab the number of controls, then walk on ...
    WORD nNumOfCtrls = *(LPWORD) pDlgResource;
    pDlgResource += sizeof(WORD);
    
	// walk on past the dimensions ...
	short x = *(short *) pDlgResource;
	pDlgResource += sizeof(short);
	short y = *(short *) pDlgResource;
	pDlgResource += sizeof(short);
	short w = *(short *) pDlgResource;
	pDlgResource += sizeof(short);
	short h = *(short *) pDlgResource;
	pDlgResource += sizeof(short);

	// start counting 'BYTEs', to assure that we are properly aligned on 'WORDs'
	int nByteCount = 0;
    
	// walk by the (optional) menu information
	if ( (*(WORD *) pDlgResource) == 0x0000 ) {
		pDlgResource += sizeof(WORD);					// no menu, so walk on ...
		nByteCount += 2;
	} else {
		if ( (*(WORD *) pDlgResource) == 0xFFFF ) {
			pDlgResource += sizeof(WORD);		// walk past ordinal-value of menu
			pDlgResource += sizeof(WORD);		// walk past ordinal-value of menu
			nByteCount += 4;
		} else {
			strMenu = CString((LPCWSTR) pDlgResource);
			pDlgResource += strMenu.GetLength();
//			nByteCount +=  strMenu.GetLength();
			// as long as we are using MFC, we don't
			//  appear to have to worry about this case ...
		}
	}

	// walk by the class information
	if ( (*(WORD *) pDlgResource) == 0x0000 ) {
		pDlgResource += sizeof(WORD);						// pre-defined dialog box class, so walk on ...
		nByteCount += 2;
	} else {
		if ( (*(WORD *) pDlgResource) == 0xFFFF ) {
			pDlgResource += (2 * sizeof(WORD));				// walk past ordinal-value of class
			nByteCount += 4;
		} else {
			strClass = CString((LPCWSTR) pDlgResource);
			pDlgResource += ( (strClass.GetLength() + 1) * 2 );
			nByteCount += ( (strClass.GetLength() + 1) * 2 );
		}
	}

	// walk past the caption information
	if ( (*(WORD *) pDlgResource) == 0x0000 ) {
		pDlgResource += sizeof(WORD);						// no caption specified, so walk on ...
		nByteCount += 2;
	} else {
		strCaption = CString((LPCWSTR) pDlgResource);
		pDlgResource += ( (strCaption.GetLength() + 1) * 2 );
		nByteCount += ( (strCaption.GetLength() + 1) * 2 );

		// set the text of the remote dialog to this dialog's caption
		SetRemoteWindowOwnerText( strCaption );

	}

	// walk past the font information
	if ( style &  DS_SETFONT ) {
		pDlgResource += sizeof(WORD);					// walk past the point size value
		strFont = CString((LPCWSTR) pDlgResource);
		pDlgResource += ( (strFont.GetLength() + 1) * 2 );
		nByteCount += ( (strFont.GetLength() + 1) * 2 );
	}

	// ***** in this section, we work through the dialog items *****
	WORD id;
	WORD nType;
	WORD nCreationDataSize;
	RECT rect;
	CRemoteEdit remEdit;
	CRemoteStatic remStatic;
	CRemoteListBox remListBox;
	CRemoteButton remButton;
	int nRemainder;
	for ( int i = 0; i < nNumOfCtrls; i++ ) 
	{


		// **********************************
		// ensure alignment on DWORD boundary
		// **********************************
		nRemainder = nByteCount % 4;
		pDlgResource += ( sizeof(BYTE) * nRemainder);
		nByteCount +=  ( sizeof(BYTE) * nRemainder);


		// get the style and extended style
	    style = *(DWORD *) pDlgResource;
	    pDlgResource += sizeof(DWORD);
		nByteCount += sizeof(DWORD);
	    exStyle = *(DWORD *) pDlgResource;
	    pDlgResource += sizeof(DWORD);
		nByteCount += sizeof(DWORD);

		// get the control dimensions
		x = *(short *) pDlgResource;
		pDlgResource += sizeof(short);
		nByteCount += sizeof(short);
		y = *(short *) pDlgResource;
		pDlgResource += sizeof(short);
		nByteCount += sizeof(short);
		w = *(short *) pDlgResource;
		pDlgResource += sizeof(short);
		nByteCount += sizeof(short);
		h = *(short *) pDlgResource;
		pDlgResource += sizeof(short);
		nByteCount += sizeof(short);

		// get the control id
	    id = *(WORD *) pDlgResource;
	    pDlgResource += sizeof(WORD);
		nByteCount += sizeof(WORD);

		// walk by the class information
		if ( (*(WORD *) pDlgResource) == 0xFFFF ) {
			pDlgResource += sizeof(WORD);					// walk past the filler
			nByteCount += sizeof(WORD);
			nType = *(WORD *) pDlgResource;					// grab the control type
			pDlgResource += sizeof(WORD);
			nByteCount += sizeof(WORD);
		} else {
			strClass = CString((LPCWSTR) pDlgResource);
			pDlgResource += ( (strClass.GetLength() + 1) * 2 );
			nByteCount += ( (strClass.GetLength() + 1) * 2 );
		}

		// walk by the inital text/resource id information
		if ( (*(WORD *) pDlgResource) == 0xFFFF ) {
			pDlgResource += sizeof(WORD);					// walk past the filler
			nByteCount += sizeof(WORD);
			pDlgResource += sizeof(WORD);					// walk past icon resource id
			nByteCount += sizeof(WORD);
		} else {
			strCaption = CString((LPCWSTR) pDlgResource);
			pDlgResource += ( (strCaption.GetLength() + 1) * 2 );
			nByteCount += ( (strCaption.GetLength() + 1) * 2 );
		}

		// finally, walk past the creation data, if any
		nCreationDataSize = *(WORD *) pDlgResource;
		pDlgResource += sizeof(WORD);
		nByteCount += sizeof(WORD);
		if ( nCreationDataSize != 0 )
			pDlgResource += nCreationDataSize;

		// undocumented fillers?
/*		pDlgResource += sizeof(WORD);
		pDlgResource += sizeof(WORD);*/

		// prep rect structure
		rect.left   = x;
		rect.top    = y;
		rect.right  = (x + w - 1);
		rect.bottom = (y + h - 1);

		// construct the appropriate type of object, and send it
		switch ( nType ) {

		// buttons (including checkboxes and radio buttons)
		case 0x0080:
			remButton.Create( this, strCaption, style, rect, m_pWndOwner, id );
			break;

		// edit control
		case 0x0081:
			remEdit.Create( this, style, rect, m_pWndOwner, id );
			break;

		// static control
		case 0x0082:
			remStatic.Create( this, strCaption, style, rect, m_pWndOwner, id );
			break;

		// list box
		case 0x0083:
			remListBox.Create( this, style, rect, m_pWndOwner, id );
			break;

		}

	}

	RemoteContext = nIDD;

	return TRUE;
}

// ////////////////////////////////////////////////////////
//
// SetRemoteWindowText
//
// Set the specified remote window's text.
//
BOOL CRemoteUIManager::SetRemoteWindowText( UINT nID, LPCTSTR lpszString )
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_SETTEXT;
	pUI->m_nControlID = nID;
	pUI->m_strCaption = lpszString;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// SetRemoteWindowOwnerText
//
// Set the remote window owner's text.
//
BOOL CRemoteUIManager::SetRemoteWindowOwnerText(LPCTSTR lpszString)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_SETOWNERTEXT;
	pUI->m_strCaption = lpszString;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// GetRemoteWindowText
//
// Get text from the specified remote window.
//
BOOL CRemoteUIManager::GetRemoteWindowText( UINT nID )
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_GETTEXT;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// EnableRemoteWindow
//
// Enable the specified remote windw
//
BOOL CRemoteUIManager::EnableRemoteWindow(UINT nID)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_ENABLE;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// DisableRemoteWindow
//
// Disable the specified remote windw
//
BOOL CRemoteUIManager::DisableRemoteWindow(UINT nID)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_DISABLE;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// DeleteRemoteWindow
//
// Remote the specified remote window.
//
BOOL CRemoteUIManager::DeleteRemoteWindow(UINT nID)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_DELETE;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// DeleteAllRemoteWindows
//
// Remove all windows created remotely.
//
BOOL CRemoteUIManager::DeleteAllRemoteWindows()
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_DELETEALL;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// AddRemoteString
//
// Add a string to the specified remote list box.
//
BOOL CRemoteUIManager::AddRemoteString(UINT nID, CString strText)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_ADDSTRING;
	pUI->m_nControlID = nID;
	pUI->m_strCaption = strText;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// GetCurrentRemoteWindowText
//
// Retrieve the currently selected text from a remote
//  list box.
//
BOOL CRemoteUIManager::GetCurrentRemoteWindowText(UINT nID)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_GETCURTEXT;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// GetRemoteCheckState
//
// Determine the check state of a remote button.
//
BOOL CRemoteUIManager::GetRemoteCheckState(UINT nID)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_GETCHECK;
	pUI->m_nControlID = nID;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}

// ////////////////////////////////////////////////////////
//
// SetRemoteCheckState
//
// Set the check state of a remote button.
//
BOOL CRemoteUIManager::SetRemoteCheckState(UINT nID, int nState)
{
	// verify that there is a socket manager
	if ( m_pSockMgr == NULL )
		return FALSE;

	// construct a new message, and populate it
	CUIMessage* pUI = new CUIMessage;
	pUI->m_nUIMessageType = CUIMessage::UMT_SETCHECK;
	pUI->m_nControlID = nID;
	pUI->m_nCheckState = nState;

	// add it to the queue, and send it on its way
	if ( m_pSockMgr->AddMessageToQueue(pUI) )
		if ( !m_bPostponeDelivery )
			if ( m_pSockMgr->SendMessages() )
				return true;

	return FALSE;
}
