// RemoteUIManager.cpp: implementation of the CRemoteUIManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RemoteUIManager.h"
#include "RemoteMessage.h"
#include "UIMessage.h"
#include "StatusMessage.h"
#include "RemoteWnd.h"
#include "RemoteUIDefs.h"
#include "RemoteCommDefs.h"
#include "SocketManager.h"
#include "RemoteButton.h"
#include "RemoteEdit.h"
#include "RemoteStatic.h"
#include "RemoteListBox.h"
#include "RemoteCommand.h"
#include "BLOBMessage.h"
#include "UIReplyMessage.h"

//#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRemoteUIManager::CRemoteUIManager()
{

	m_pSockMgr = NULL;
	m_bPostponeDelivery = false;

	RemoteContext = 0;		// jmc
}

CRemoteUIManager::~CRemoteUIManager()
{
	POSITION pos;
	CWnd* pWnd;
	UINT nID;
	pos = m_mapWindows.GetStartPosition();
	while ( pos ) {
		m_mapWindows.GetNextAssoc( pos, nID, pWnd );
		if ( pWnd )
			delete pWnd;
	}
	m_mapWindows.RemoveAll();
	if (m_pSockMgr)
		delete m_pSockMgr;
}

BEGIN_MESSAGE_MAP(CRemoteUIManager, CWnd)
	//{{AFX_MSG_MAP(CRemoteUIManager)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_REMOTEMESSAGE, OnRemoteMessage)
	ON_MESSAGE(WM_REMOTECONNECT, OnRemoteConnect)
END_MESSAGE_MAP()

// ////////////////////////////////////////////////////////
//
// InitToListen
//
// Create an hWnd to receive messages on, init the socket
//  manager, and instruct it to listen for a connection.
//
BOOL CRemoteUIManager::InitToListen( CWnd* pWndOwner, int nPort )
{
	// create our window
	CString strClassName;
	strClassName = AfxRegisterWndClass( NULL, 0, 0, 0 );
	if ( !CreateEx(0, strClassName, _T("RemoteUIManager(Server)"), WS_POPUP, 0, 0, 0, 0, NULL, NULL) )
		return FALSE;

	// create the socket manager, set it up to listen, and establish us as the consumer
	m_pSockMgr = new SocketManagerPDA;
	if (! m_pSockMgr->Listen(nPort) ) {
		delete m_pSockMgr;
		m_pSockMgr = NULL;
		return FALSE;
	}

	// tell the socket manager to pass its incoming messages to us
	// (we will receive them via the OnRemoteMessage method)
	m_pSockMgr->SetConsumer( this );

	// save away the window pointer to our owner, so that we can
	//  forward on messages received from the socket manager
	m_pWndOwner = pWndOwner;

	// create a system font to use on the remotely created controls
	CDC* pDC;
	pDC = pWndOwner->GetDC();
#ifdef _WIN32_WCE
	m_fntSystem.CreatePointFont( 100, _T("MS Sans Serif"), pDC );
#else
	m_fntSystem.CreatePointFont( 80, _T("MS Sans Serif"), pDC );
#endif

	return TRUE;
}

// ////////////////////////////////////////////////////////
//
// InitToConnect
//
// Create an hWnd to receive messages on, init the socket
//  manager, and instruct it to connect to the address
//  specified.
//
BOOL CRemoteUIManager::InitToConnect( CString strAddress, int nPort, CWnd* pWndOwner )
{
	// create our window
	CString strClassName;
	strClassName = AfxRegisterWndClass( NULL, 0, 0, 0 );
	if ( !CreateEx(0, strClassName, _T("RemoteUIManager(Client)"), WS_POPUP, 0, 0, 0, 0, NULL, NULL) )
		return FALSE;

	// create the socket manager, set it up to listen, and establish us as the consumer
	m_pSockMgr = new SocketManagerPDA;
	if ( !m_pSockMgr->Connect( strAddress, nPort ) )
		return FALSE;

	// tell the socket manager to pass its incoming messages to us
	// (we will receive them via the OnRemoteMessage method)
	m_pSockMgr->SetConsumer( this );

	// save away the window pointer to our owner, so that we can
	//  forward on messages received from the socket manager
	m_pWndOwner = pWndOwner;

	// create a system font to use on the remotely created controls
	CDC* pDC;
	pDC = pWndOwner->GetDC();
#ifdef _WIN32_WCE
	m_fntSystem.CreatePointFont( 100, _T("MS Sans Serif"), pDC );
#else
	m_fntSystem.CreatePointFont( 80, _T("MS Sans Serif"), pDC );
#endif

	return TRUE;
}

// ////////////////////////////////////////////////////////
//
// OnRemoteConnect
//
// "Pass-Through" method to indicate to our 'owner' that
//  the remote system has connected.
//
LRESULT CRemoteUIManager::OnRemoteConnect( WPARAM wParam, LPARAM lParam )
{
	m_pWndOwner->PostMessage( WM_REMOTECONNECT, 0, 0L );
	return 1;
}

// ////////////////////////////////////////////////////////
//
// OnRemoteMessage
//
// Handler for messages incoming from the socket manager.
//
// Interpret each incoming message, and act on it.
//
LRESULT CRemoteUIManager::OnRemoteMessage( WPARAM wParam, LPARAM lParam )
{
	CObject* pObject;
	CRemoteMessage* pRemoteMessage;
	CRemoteCommand* pCommand;
	CUIReplyMessage* pUIReplyMessage;
	CUIMessage* pUIMessage;
	CBLOBMessage* pBLOBMessage;
	CWnd* pWnd;
	CString* pString;
	CListBox* pListBox;
	CButton* pButton;

	// empty the message queue, interpreting messages, and
	//  passing them on to our owner.
	while ( ( pObject = m_pSockMgr->GetNextMessage() )) {

		// cast this to a message of our own class, and assert
		//  if it isn't ours.
		pRemoteMessage = DYNAMIC_DOWNCAST(CRemoteMessage, pObject);
		ASSERT(pRemoteMessage);

		// handle the remote message
		switch ( pRemoteMessage->m_nRemoteMessageType )
		{

		// handle remote large object transfer messages here
		// (these messages are typically sent from the server to the client)
		case CRemoteMessage::RMT_BLOB:

			// convert to the correct message type
			pBLOBMessage = DYNAMIC_DOWNCAST(CBLOBMessage, pRemoteMessage);
			ASSERT(pBLOBMessage);

			// get the associated window, previously created (of course)
			CStatic* pStatic;
			if ( m_mapWindows.Lookup(pBLOBMessage->m_nStaticID, pWnd) ) {
				pStatic = DYNAMIC_DOWNCAST(CStatic, pWnd);
				if ( pStatic ) {

					pStatic->SetBitmap( *(pBLOBMessage->m_pImage) );
				}
			}

			break;

		// handle remote UI reply messages here
		// (these messages are typically sent from the client to the server)
		case CRemoteMessage::RMT_UIREPLY:

			// convert to the correct message type
			pUIReplyMessage = DYNAMIC_DOWNCAST(CUIReplyMessage, pRemoteMessage);
			ASSERT(pUIReplyMessage);

			// post this message to our owner, and let them deal with it
			pString = new CString;
			pString->Format( _T("%s"), pUIReplyMessage->m_strMessage );
			m_pWndOwner->PostMessage( WM_UIREPLYMESSAGE, 
				MAKEWPARAM(pUIReplyMessage->m_nReplyType, pUIReplyMessage->m_nControlID) , 
				(LPARAM) pString );

			break;
			
		// handle remote UI notification messages here
		// (these messages are typically sent from the client to the server)
		case CRemoteMessage::RMT_UIC:

			// convert to the correct message type
			pCommand = DYNAMIC_DOWNCAST(CRemoteCommand, pRemoteMessage);
			ASSERT(pCommand);

			// post this message to our owner, and let them deal with it
			m_pWndOwner->PostMessage( WM_REMOTECOMMAND, pCommand->m_wParam, pCommand->m_lParam );

			break;

		// handle remote UI messages here
		// (these messages are typically sent from the server to the client)
		case CRemoteMessage::RMT_UI:

			// convert to the correct message type
			pUIMessage = DYNAMIC_DOWNCAST(CUIMessage, pRemoteMessage);
			ASSERT(pUIMessage);

			// interpret the message
			switch ( pUIMessage->m_nUIMessageType ) 
			{

			// messages typically sent from the server to the remote client

			// delete all previously created controls
			case CUIMessage::UMT_DELETEALL:
				POSITION pos;
				CWnd* pWnd;
				UINT nID;
				pos = m_mapWindows.GetStartPosition();
				while ( pos ) {
					m_mapWindows.GetNextAssoc( pos, nID, pWnd );
					delete pWnd;
				}
				m_mapWindows.RemoveAll();
				break;

			// delete a previously created control
			case CUIMessage::UMT_DELETE:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {
					m_mapWindows.RemoveKey( pUIMessage->m_nControlID );
					delete pWnd;
				}
				break;

			// 'bing' the speaker
			case CUIMessage::UMT_BING:
				::MessageBeep( -1 );
				break;

			// add a string to a list box
			case CUIMessage::UMT_ADDSTRING:
				pListBox = NULL;
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {
					pListBox = DYNAMIC_DOWNCAST(CListBox, pWnd);
					if ( pListBox ) {
						pListBox->AddString( pUIMessage->m_strCaption );
					}
				}
				break;

			// grab the text of an list box
			case CUIMessage::UMT_GETCURTEXT:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {

					// see if the selected window is a list box
					CString str;
					pListBox = NULL;
					pListBox = DYNAMIC_DOWNCAST(CListBox, pWnd);
					if ( pListBox ) {

						int nSel = pListBox->GetCurSel();
						if ( nSel != LB_ERR ) {

							pListBox->GetText( nSel, str );

							// construct a new message, and populate it
							CUIReplyMessage* pUIReply = new CUIReplyMessage;

							// grab the window text
							pUIReply->m_strMessage = str;
							pUIReply->m_nControlID = pUIMessage->m_nControlID;
							pUIReply->m_nReplyType = CUIMessage::UMT_GETCURTEXT;

							// add it to the queue, and send it on its way
							if ( m_pSockMgr->AddMessageToQueue(pUIReply) )
								m_pSockMgr->SendMessages();

						}
					}
				}
				break;

			// grab the text of an existing control
			case CUIMessage::UMT_GETTEXT:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {

					// construct a new message, and populate it
					CUIReplyMessage* pUIReply = new CUIReplyMessage;

					// grab the window text
					CString str;
					pWnd->GetWindowText( str );
					pUIReply->m_strMessage = str;
					pUIReply->m_nControlID = pUIMessage->m_nControlID;
					pUIReply->m_nReplyType = CUIMessage::UMT_GETTEXT;

					// add it to the queue, and send it on its way
					if ( m_pSockMgr->AddMessageToQueue(pUIReply) )
						m_pSockMgr->SendMessages();

				}
				break;

			// grab the check setting of a button
			case CUIMessage::UMT_GETCHECK:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {

					// see if the selected window is a button
					CString str;
					pButton = NULL;
					pButton = DYNAMIC_DOWNCAST(CButton, pWnd);
					if ( pButton ) {

						// construct a new message, and populate it
						CUIReplyMessage* pUIReply = new CUIReplyMessage;

						// get the check state
						pUIReply->m_nReplyInt = pButton->GetCheck();

						// grab the window text
						pUIReply->m_nControlID = pUIMessage->m_nControlID;
						pUIReply->m_nReplyType = CUIMessage::UMT_GETCHECK;

						// add it to the queue, and send it on its way
						if ( m_pSockMgr->AddMessageToQueue(pUIReply) )
							m_pSockMgr->SendMessages();
					}
				}
				break;

			// set the check setting of a button
			case CUIMessage::UMT_SETCHECK:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {

					// see if the selected window is a button
					CString str;
					pButton = NULL;
					pButton = DYNAMIC_DOWNCAST(CButton, pWnd);
					if ( pButton ) {
						pButton->SetCheck( pUIMessage->m_nCheckState );
					}
				}
				break;

			// enable a previously created control
			case CUIMessage::UMT_ENABLE:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {
					pWnd->EnableWindow( TRUE );
				}
				break;

			// disable a previously created control
			case CUIMessage::UMT_DISABLE:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {
					pWnd->EnableWindow( FALSE );
				}
				break;

			// set the window text of our owner
			case CUIMessage::UMT_SETOWNERTEXT:
				m_pWndOwner->SetWindowText( pUIMessage->m_strCaption );
				break;

			// set the window text of a previously created control
			case CUIMessage::UMT_SETTEXT:
				if ( m_mapWindows.Lookup(pUIMessage->m_nControlID, pWnd) ) {
					pWnd->SetWindowText( pUIMessage->m_strCaption );
				}
				break;

			// create a new control
			case CUIMessage::UMT_CREATE:

				// update the coordinates passed in with coordinates modified
				//  to match the dialog box (dialog units vs. pixels).
				::MapDialogRect( m_pWndOwner->m_hWnd, &pUIMessage->m_wndRect );

				// create the appropriate type of control
				switch ( pUIMessage->m_nControlType )
				{

				// create the button, and add it to the local windows list
				case WT_BUTTON:
					CButton* pButton;
					pButton = new CButton;
					pButton->Create( pUIMessage->m_strCaption,
						pUIMessage->m_nStyle,
						pUIMessage->m_wndRect,
						m_pWndOwner,
						pUIMessage->m_nControlID );
					pButton->SetFont( &m_fntSystem, TRUE );
					m_mapWindows.SetAt( pUIMessage->m_nControlID, pButton );
					break;

				// create the static control, and add it to the local windows list
				case WT_STATIC:
					CStatic* pStatic;
					pStatic = new CStatic;
					pStatic->Create( pUIMessage->m_strCaption,
						pUIMessage->m_nStyle,
						pUIMessage->m_wndRect,
						m_pWndOwner,
						pUIMessage->m_nControlID);
					pStatic->SetFont( &m_fntSystem, TRUE );
					m_mapWindows.SetAt( pUIMessage->m_nControlID, pStatic );
					break;

				// create the edit control, and add it to the local windows list
				case WT_EDIT:
					CEdit* pEdit;
					pEdit = new CEdit;
					pEdit->Create( pUIMessage->m_nStyle,
						pUIMessage->m_wndRect,
						m_pWndOwner, 
						pUIMessage->m_nControlID );
					pEdit->SetFont( &m_fntSystem, TRUE );
					m_mapWindows.SetAt( pUIMessage->m_nControlID, pEdit );
					break;

				// create the list box, and add it to the local windows list
				case WT_LISTBOX:
					CListBox* pListBox;
					pListBox = new CListBox;
					pListBox->Create( pUIMessage->m_nStyle,
						pUIMessage->m_wndRect,
						m_pWndOwner,
						pUIMessage->m_nControlID);
					pListBox->SetFont( &m_fntSystem, TRUE );
					m_mapWindows.SetAt( pUIMessage->m_nControlID, pListBox );
					break;

				}

				break;

			// responses typically sent from the remote client to the server
			}

			break;

		}

		// delete the object we just serviced
		delete pObject;
		
	}
	return 1;
}

// ////////////////////////////////////////////////////////
//
// ForwardCommand
//
// Take a WM_COMMAND from one of the controls in the window
//  list, and send it to the "other side" of the remote
//  connection.
//
void CRemoteUIManager::ForwardCommand( WPARAM wParam, LPARAM lParam )
{
	CRemoteCommand* pCommand = new CRemoteCommand;

TRACE3( "Forward command: HIWORD(wParam)=%d, LOWORD(wParam)=%d, lParam=%x\n", HIWORD(wParam), LOWORD(wParam), lParam);

	pCommand->m_nRemoteMessageType = CRemoteMessage::RMT_UIC;
	pCommand->m_wParam = wParam;
	pCommand->m_lParam = lParam;
	m_pSockMgr->AddMessageToQueue( pCommand );
	m_pSockMgr->SendMessages();
}

// ////////////////////////////////////////////////////////
//
// StartGrouping
//
// Set a local flag that all the other methods respect,
//  indicating that we only want to queue up commands at
//  this point, and not actually send them.
//
void CRemoteUIManager::StartGrouping()
{
	m_bPostponeDelivery = true;
}

// ////////////////////////////////////////////////////////
//
// EndGrouping
//
// Reset the local flag, and send any queued messages on
//  their way.
//
void CRemoteUIManager::EndGrouping()
{
	m_bPostponeDelivery = false;
	m_pSockMgr->SendMessages();
}
