// SocketManager.cpp: implementation of the SocketManagerPDA class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SocketManager.h"
#include "ListenSocket.h"
#include "DataSocket.h"
#include "remoteuimanager.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SocketManagerPDA::SocketManagerPDA()
{
	IsConnected = FALSE;  // jmc

	m_pListenSocket = NULL;
	m_pDataSocket   = NULL;
	m_pWndConsumer  = NULL;
}

SocketManagerPDA::~SocketManagerPDA()
{
	CObject* pMessage;

	// dump any messages currently in the queues
	while ( m_lstOutgoingMessages.GetCount() > 0 ) {
		pMessage = (CObject *) m_lstOutgoingMessages.RemoveTail();
		if ( pMessage )
			delete pMessage;
	}
	while ( m_lstIncomingMessages.GetCount() > 0 ) {
		pMessage = (CObject *) m_lstIncomingMessages.RemoveTail();
		if ( pMessage )
			delete pMessage;
	}

	// cleanup any existing objects before leaving
	if ( m_pListenSocket != NULL ) {
		delete m_pListenSocket;
		m_pListenSocket = NULL;
	}
	if ( m_pDataSocket != NULL ) {
		delete m_pDataSocket;
		m_pDataSocket = NULL;
	}

}

// ////////////////////////////////////////////////////////
//
// SetConsumer
//
// Establish the 'consumer' of this object.  The consumer
//  is notified via a PostMessage when new messages arrive.
//
void SocketManagerPDA::SetConsumer( CWnd* pWnd )
{
	m_pWndConsumer = pWnd;
}

// ////////////////////////////////////////////////////////
//
// Listen
//
bool SocketManagerPDA::Listen(int nPort)
{
	// be sure that we are not already listening
	if ( m_pListenSocket != NULL )
		return false;

	// create the listening socket object
	m_pListenSocket = new CListenSocket;
	m_pListenSocket->SetSocketManager( this );

	// create the socket, and listen on the specified port
	if ( m_pListenSocket->Create( nPort ) ) {
		if ( m_pListenSocket->Listen() ) {
			TRACE( _T("** Listening ...\n") );
			return true;
		}
	}

	// errors occurred, so cleanup and leave
	delete m_pListenSocket;
	m_pListenSocket = NULL;

	return false;
}

// ////////////////////////////////////////////////////////
//
// Connect
//
bool SocketManagerPDA::Connect(CString strAddress, int nPort)
{
	// be sure we are not already connected
	if ( m_pDataSocket != NULL )
		return false;

	IsConnected = FALSE;

	// create the data socket object
	m_pDataSocket = new CDataSocket;
	m_pDataSocket->SetSocketManager( this );

	// create the socket
	if ( m_pDataSocket->Create() ) {

		// attempt to connect to the address and port specified
		if ( m_pDataSocket->Connect(strAddress, nPort) ) {
			TRACE( _T("** Connected to server\n") );
			IsConnected = TRUE;
			return true;
		}

	}

	// errors occurred, so cleanup and leave
	delete m_pDataSocket;
	m_pDataSocket = NULL;

	return false;
}

// ////////////////////////////////////////////////////////
//
// AcceptConnection
//
// This method is called by the m_ListenSocket when it
//  accepts a connection from the client.
//
void SocketManagerPDA::AcceptConnection()
{
	// make sure we have not already accepted a connection
	if ( m_pDataSocket != NULL ) {
		delete m_pDataSocket;
		m_pDataSocket = NULL;
		TRACE0( "** Accepted ANOTHER socket\n" );
//		return;
	}

	// create a data socket that we can accept a connection on
	m_pDataSocket = new CDataSocket;
	m_pDataSocket->SetSocketManager( this );

	

	// accept the connection
	if ( m_pListenSocket->Accept( *m_pDataSocket ) ) {
		TRACE( _T("** Accepted connection from client\n") );

		IsConnected = TRUE;

		// notify the consumer that a connection has occurred
		if ( m_pWndConsumer )
			m_pWndConsumer->PostMessage( 
				WM_REMOTECONNECT, 
				0, 
				0L );

	} else {
		delete m_pDataSocket;
		m_pDataSocket = NULL;
	}

}

// ////////////////////////////////////////////////////////
//
// SendMessage
//
// Send the message object passed as a parameter across
//  the data socket.
//
bool SocketManagerPDA::SendMessage( CObArray& arObjects )
{
	if (!IsConnected)
		return FALSE;

	try
	{
		CSocketFile socketFile(m_pDataSocket);
		CArchive archiveOut(&socketFile, CArchive::store);

		// attempt to send the contents of the message queue
		arObjects.Serialize( archiveOut );
		archiveOut.Close();
		socketFile.Close();
//		TRACE( _T("** Sent message\n") );

		// dump the obj list we just sent
		for (int i = 0; i < arObjects.GetSize(); i++)
				delete arObjects[i];
		arObjects.RemoveAll();
	}
	catch(...)
	{
		TRACE(_T("PDA skt mgr: send packet EXCEPTION!\n"));
		return FALSE;
	}
	
	return true;
}

// ////////////////////////////////////////////////////////
//
// AddMessageToQueue
//
// Add a new message to the head of the outgoing message 
//  list.
//
bool SocketManagerPDA::AddMessageToQueue(CObject *pMessage)
{
	m_lstOutgoingMessages.AddHead( pMessage );

	return true;
}

// ////////////////////////////////////////////////////////
//
// SendMessages
//
// Loop through all of the messages in the outgoing message 
//  list, sending each one (and removing it).
//
bool SocketManagerPDA::SendMessages()
{
	if (!IsConnected)
		return FALSE;

	CObArray arObjects;
	CObject* pMessage;

	while ( m_lstOutgoingMessages.GetCount() > 0 ) {
		pMessage = (CObject *) m_lstOutgoingMessages.RemoveTail();
		arObjects.Add( pMessage );
	}

	return SendMessage( arObjects );
}

// ////////////////////////////////////////////////////////
//
// ReceiveMessages
//
// Receive all the messages that are currently coming in on
//  the socket, into the incoming message list.
//
void SocketManagerPDA::ReceiveMessages()
{
	// receive the messages
	CObArray* pArObjects = new CObArray;

	// transfer them to the incoming queue
	CObject* pObj;

	try
	{
		CSocketFile socketFile(m_pDataSocket);
		CArchive archiveIn(&socketFile, CArchive::load);

		// grab stuff from the archive
		pArObjects->Serialize( archiveIn );

		archiveIn.Close();
		socketFile.Close();
	}
	catch(...)
	{
		TRACE(_T("PDA skt mgr: read packet EXCEPTION!\n"));
		delete pArObjects;
		return;
	}

	TRACE1( "Received %d objects in message.\n", pArObjects->GetSize() );

	// process what is currently in the buffer
	for ( int i = 0; i < pArObjects->GetSize(); i++ )
	{
		pObj = pArObjects->GetAt( i );
		m_lstIncomingMessages.AddHead( pObj );
	}
	pArObjects->RemoveAll();

	delete pArObjects;

	// notify the consumer that more messages have arrived
	if ( m_pWndConsumer )
		m_pWndConsumer->PostMessage( 
			WM_REMOTEMESSAGE, 
			m_lstIncomingMessages.GetCount(), 
			0L );
}

// ////////////////////////////////////////////////////////
//
// GetNextMessage
//
// Retrieve the next message from the incoming message list.
//
CObject* SocketManagerPDA::GetNextMessage()
{
	CObject* pMessage;

	if ( m_lstIncomingMessages.GetCount() > 0 )
		pMessage = (CObject *) m_lstIncomingMessages.RemoveTail();
	else
		pMessage = NULL;

	return pMessage;
}

bool SocketManagerPDA::CloseDataConnection()
{
	CObject* pMessage;

	// dump any messages currently in the queues
	while ( m_lstOutgoingMessages.GetCount() > 0 ) {
		pMessage = (CObject *) m_lstOutgoingMessages.RemoveTail();
		if ( pMessage )
			delete pMessage;
	}
	while ( m_lstIncomingMessages.GetCount() > 0 ) {
		pMessage = (CObject *) m_lstIncomingMessages.RemoveTail();
		if ( pMessage )
			delete pMessage;
	}

	// cleanup any existing objects before leaving
	if ( m_pListenSocket != NULL )
		delete m_pListenSocket;
	m_pListenSocket = NULL;
	if ( m_pDataSocket != NULL ) {
		m_pDataSocket->Close();
		delete m_pDataSocket;
		m_pDataSocket = NULL;
	}

	IsConnected = FALSE;

	return true;
}
