// Pak.h: interface for the PacketSocket class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PAK_H__FB21F273_6242_11D2_BF68_00E0980086A0__INCLUDED_)
#define AFX_PAK_H__FB21F273_6242_11D2_BF68_00E0980086A0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define SOCK_BASE 700
#define MAX_PACKET_DATA 6500
#define PAK_SAFETY 10
#define PAK_UPDATE_VALUES 20

#define PAK_CLOUD_SIZE 500 
#define NORMAL_SIZE 50
#define SUFFIX_SIZE 20

typedef struct
{
	float x,y,z;
} FloatVecStruct;
/////////////////// jmc pak ///////////////////
typedef struct 
{
	double x,y,z;
} VecStruct;
typedef struct
{
	double x, y;
} VecStruct2D;
#define NOTE_LINES 10
#define NOTE_SIZE 100
#define PLANE_PERIM_PTS   20



class PacketSocket : public CSocket
{
public:
	void CStringArrayToPak(CStringArray& csa);
	void PakToCStringArray(CStringArray* pcsa);

	int sent;
	int rcvd;
	///////////////////////////////////////
	// buffering information

	//BYTE SendAck;
	DWORD NextSend;
	DWORD EndOfFile;
	BYTE WaitingForAck;
	CStdioFile SockBuf;
	BYTE BufFileOpen;
	char FileCache[MAX_PACKET_DATA];

	BYTE DrainBuffer();

	~PacketSocket();

	BOOL SetSocketOptions(DWORD SendBufSize = 16384,DWORD RecBufSize = 16384,BOOL MakeSocFile = FALSE,CString socfileprefix = _T("SOC"));
	void PacketSocket::CloseSocketAndFile();
	virtual void Close();
	virtual BOOL Connect(LPCTSTR lpszHostAddress,UINT nHostPort);
	
	///////////////////////////////////////
	virtual void OnClose( int Err );
	BYTE Active;			//TRUE if connected

	WORD HeaderSize;	 // size of header bytes.
	WORD MaxPacketSize;	 // maximum size of entire packet.
	WORD DataSize;		// user must set this when sending.  Send Packet will add the header
							// size and set the packet size!
	PacketSocket();

	BYTE PrepareToSend();		 // sets data size!
	WORD SendPacket();
	WORD ReceivePacket();
	BYTE BugPacket();		// print debugging information about the packet.
	void TkrDebug(BYTE sending);		// can't run debug code w/tracker dll's, so fprint

	virtual void OnReceive(int nErrorCode); // process incoming stuff.
	virtual int Receive(void* lpBuf, int nBufLen, int nFlags = 0);

	// Packet structure.
	//		This structure enables the raw data to be written and read in 
	// may different formats.  The application can branch to the different 
	// data molds using the command WORD.
	struct
	{
		WORD Size;
		WORD Command;
		union
		{
			char Raw[MAX_PACKET_DATA];			// used for PAK_TEXT
			struct	 // mold for command = 	PAK_UPDATE
			{
				double Values[PAK_UPDATE_VALUES];			 // three possible values.
			} Update;
			struct		// mold for command = PAK_MEASUREMENT
			{
				char ContainerName[NORMAL_SIZE];			 
				char TargetName[NORMAL_SIZE];		
				double TargetOffset;			// 4-8-97
				double Values[PAK_UPDATE_VALUES];
				char User[NORMAL_SIZE];
				char Info[NORMAL_SIZE];
				char Time[30];
			} Meas;
			struct		// mold for command = PAK_FLAGS
			{
				BYTE Bit[10];
				// BIT 0 - STOP DRAWING THE INSTRUMENT CURRENT RAY.
				// BIT 1 - send to make the instrument query.

				// Bit [2] is for interface measurement operations. action
				//		1 = delete last group
				//		2 = delete last target
				//		3 = measurement done. (sent when group is incremented)
				// Bit [3] if 1 indicates that a measurement failed.

			} Flags;
			struct		
			{
				WORD InstModel;
			} SignOn;
			struct
			{
				WORD M;
				WORD S;
				WORD InstIndex;
				double ABDist;
				double OShot[6][2];
			} OrientTheo;
			struct
			{
				WORD M;
				WORD S;
			}	BeginOrient;
			struct
			{
				WORD Serial;
			}	Ack;
			struct			// the action packet is used to control the instrument interfaces from the
								// analyzer.... it lets you initialize measurements , etc.
			{
				BYTE Mode;					// unused - set to 0.
				char Group[NORMAL_SIZE];		// GROUP name
				char Target[NORMAL_SIZE];		// target name
				WORD Quantity;								// number of points to measure.  0 means infinite.
				BYTE Method;			// 0 time, 1 space
				double Spacing;		// seconds or meters.
				double TargetOffset;			// don't use if negative.  units in meters.
			}	Action;
			struct			// the MV clr packet is specifically for the CLR instrument.  // jmc mvp
			{
				BYTE Mode;					// unused - set to 0.
				double DBL[10];
				WORD WRD[10];
			}	MVClr;
			struct					// used to send massive point clouds to SA.
			{
				WORD Count;		// number of points in cloud.
				char Name[NORMAL_SIZE];		// name of the cloud.
				FloatVecStruct Points[PAK_CLOUD_SIZE];
			}  Cloud;
			/////////////////////// jmc pak ///////////////////////////////////////////////////
			struct	// send the instrument transform and the working frame to the interface
			{
				VecStruct BaseToWorldPos;
				VecStruct BaseToWorldRot;
				char WorkingFrameName[200];
				VecStruct WorkingToWorldPos;
				VecStruct WorkingToWorldRot;
			}  BaseAndWorking;
			struct 
			{
				VecStruct PtA, PtB;
				BYTE DrawArrow;
				char Name[NOTE_SIZE];
				WORD NumNotes;
				char Notes[NOTE_LINES][NOTE_SIZE];
			} MakeLine;
			struct 
			{
				VecStruct CirclePos, CircleRot;
				double Radius;
				double Span;
				char Name[NOTE_SIZE];
				WORD NumNotes;
				char Notes[NOTE_LINES][NOTE_SIZE];
			} MakeCircle;
			struct 
			{
				VecStruct Center;
				double Radius;
				char Name[NOTE_SIZE];
				WORD NumNotes;
				char Notes[NOTE_LINES][NOTE_SIZE];
			} MakeSphere;
			struct 
			{
				VecStruct PtA, PtB;
				double Diameter;
				char Name[NOTE_SIZE];
				WORD NumNotes;
				char Notes[NOTE_LINES][NOTE_SIZE];
			} MakeCylinder;
			struct 
			{
				VecStruct PlanePos, PlaneRot;
				WORD NumPerimPts;
				VecStruct2D PerimPts[PLANE_PERIM_PTS];  // note, you need to limit the number of perimeter points!
				char Name[NOTE_SIZE];
				WORD NumNotes;
				char Notes[NOTE_LINES][NOTE_SIZE];
			} MakePlane;
			struct 
			{
				BYTE ClearFirst;  // if 1, clear the array before adding this entry.
				BYTE Done;		// if 1, this is the last point.
				char Group[NORMAL_SIZE];
				char Target[NORMAL_SIZE];
				char GroupSuffix[SUFFIX_SIZE];
				char TargetSuffix[SUFFIX_SIZE];
				VecStruct Nominal;
			} MeasGroup;
			struct
			{
				BYTE ObjectType;
			} InspectObject;
			struct
			{
				WORD Size;
				char buf[MAX_PACKET_DATA-2];		// blocks of text with null terminators after each line.
			} StringArray;

		} Data;
	} Packet;
	//////////////////////////////////////////////////////////////////////

};

#define PAK_TEXT 5
#define PAK_MEASUREMENT  10
#define PAK_UPDATE  20
#define PAK_FLAGS  30
#define PAK_SIGNON 40
//#define PAK_CONTAINER 50  // jmc pak
#define PAK_BEGIN_ORIENT 61
#define PAK_ORIENT_THEO 60
#define PAK_VSTARS		80
#define PAK_ACK			90
#define PAK_ACTION		100			/// action
#define PAK_MVCLR			102		// jmc mvp
#define PAK_CLOUD			103		/// for point clouds.
//////////// jmc pak   //////////////////////////
#define PAK_WORKING_FRAME		104	
#define PAK_MAKE_LINE			105		
#define PAK_MAKE_CIRCLE			106		
#define PAK_MAKE_SPHERE			107		
#define PAK_MAKE_CYLINDER		108
#define PAK_MAKE_PLANE			109
#define PAK_MEAS_GROUP			110
////////////////////////////////////////////
#define PAK_INSPECT_OBJECT		111		// sent from the interfaces to request a geometry selection for inspection.
	#define INSPECT_OBJECT_CIRCLE			1
	#define INSPECT_OBJECT_PLANE			2
	#define INSPECT_OBJECT_SLOT				3
#define PAK_STRING_ARRAY		112





//////////////////////////////////////////////////////////////////
// the listening socket allows for incoming connection requests.


class CListeningSocket : public CSocket
{
	DECLARE_DYNAMIC(CListeningSocket);
public:
	CListeningSocket();
protected:
	virtual void OnAccept(int nErrorCode);
};


#endif // !defined(AFX_PAK_H__FB21F273_6242_11D2_BF68_00E0980086A0__INCLUDED_)
