// PlaneObj.cpp: implementation of the PlaneObj class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PlaneObj.h"
#include "Fit\FitPlane.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PlaneObj::PlaneObj()
{
	A =0.0;
	B =0.0;
	C =1.0;
	D =0.0;
	patchShift = 0.0;
}

PlaneObj::~PlaneObj()
{
}


BOOL PlaneObj::ComputePlane(ArmPointArray* pVA)
{
	maxerr= 0.0;
	rmserr = 0.0;
	if(pVA->GetSize()<3)
	{
		//pRomerDlg->Beep(CALC_OK);	
		return FALSE;
	}
	OptimizationParameters opt;		// could be global for app.
	FitPlane fp;

	ArmPointArray pts;
	app.pA->ScaleArmPtsArrayToInches(*pVA, &pts);
	
//	fp.pPoints = pVA;
	VectorArray va;
	if(pts.GetVectorArrayofPoints(va) <= 0)
		return FALSE;
	fp.pPoints = &va;
	fp.ShowResults = FALSE;
	fp.pOpt = &opt;
	if (!(fp.Calculate()))
		return FALSE;

	maxerr = fp.MaxError*app.saarm.InchesToLocal;
	rmserr = fp.RMSError*app.saarm.InchesToLocal;
	t = fp.Trans;

	A = fp.FittedNormal.x;// * app.saarm.units.UnitToMeters[DISTUNITS_IN];
	B = fp.FittedNormal.y;// * app.saarm.units.UnitToMeters[DISTUNITS_IN];
	C = fp.FittedNormal.z;// * app.saarm.units.UnitToMeters[DISTUNITS_IN];
	D = fp.D;// * app.saarm.units.UnitToMeters[app.saarm.units.distunits];

	Vector AveDir = app.pA->ComputeAvgDirection();

	//TRACE0("AveDir:\n");
	//AveDir.Debug();
	//TRACE0("Norm:\n");
	//fp.FittedNormal.Debug();
	//double dot = AveDir.Dot(Vector(A,B,C));
	//TRACE2("dot = %f, D = %f\n", dot, D);

	if(AveDir.Dot(Vector(A,B,C))>0)
		D -= app.saarm.Data.TipDia/2.0;
	else
		D += app.saarm.Data.TipDia/2.0;

	//now lets shift the base (this is independent of ABCD!!!)
	Vector opos,zdir;
	t.GetTranslation(&opos);
	opos.Scale(app.saarm.InchesToLocal);
	zdir = Vector(A,B,C);
	zdir.Normalize(); //to be extra safe
	double offset;
	if(app.saarm.Data.AddExtraPatchShift)
	{
		patchShift = app.saarm.Data.ExtraPatchShift;
		offset = (app.saarm.Data.TipDia/2.0)+app.saarm.Data.ExtraPatchShift;
	}
	else
	{
		patchShift = 0.0;
		offset = (app.saarm.Data.TipDia/2.0);
	}
	zdir.Scale(offset); //in meters
		
	if(offset >= 0.0)
	{
		if(AveDir.Dot(zdir)>0)
			opos.Add(zdir);
		else
			opos.Sub(zdir);
	}
	else
	{
		if(AveDir.Dot(zdir)>0)
			opos.Sub(zdir);
		else
			opos.Add(zdir);
	}

	t.SetTranslation(opos);
	normal = t.GetZAxis();

	// 2007.07.09- ensure the normal always points back at the probe ijk...
	if(AveDir.Dot(normal) > 0)
	{
		normal.Scale(-1.0);

		Vector xx, yy, zz;
		t.GetAxes(&xx, &yy, &zz);
		yy.Scale(-1.0);
		zz.Scale(-1.0);
		t.SetAxes(xx, yy, zz);
	}

	maxerrinfo.Format(_T("Max Error: %f %s"),
		maxerr/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);
	rmserrinfo.Format(_T("RMS Error: %f %s"),
		rmserr/app.saarm.units.UnitToMeters[app.saarm.units.distunits],
		app.saarm.units.DistUnitNames[app.saarm.units.distunits]);

	double Aw,Bw,Cw,Dw;
	MapCoefficientsToWorking(&Aw, &Bw, &Cw, &Dw );
	PlaneText.Format(_T("A = %6.*f; B = %6.*f; C = %6.*f; D = %8.*f"),
			app.saarm.Data.Precision,Aw,
			app.saarm.Data.Precision,Bw,
			app.saarm.Data.Precision,Cw,
			app.saarm.Data.Precision,Dw/app.saarm.units.UnitToMeters[app.saarm.units.distunits]	);

	timeStamp.Format(Tr("Created ") + CTime::GetCurrentTime().Format(_T("%H:%M:%S %m/%d/%Y")));
	TRACE1("Plane fit time stamp...%s\n", timeStamp);

	return TRUE;
}

void PlaneObj::GetSAIPGeomPlane(SAIPGeomPlane& saipPlane)
{
	saipPlane.ObjectToInst = t;
	PPts.GetVectorArrayofPoints(saipPlane.PerimPts);
	saipPlane.Name = timeStamp;
	saipPlane.Notes.Append(notes);
}

void PlaneObj::SetFromSAIPGeomPlane(const SAIPGeomPlane& saipPlane)
{
	t = saipPlane.ObjectToInst;
	Vector xa, ya, za;
	t.GetAxes( &xa, &ya, &za );
	A = za.x;
	B = za.y;
	C = za.z;
	D = -za.Dot(t.GetTranslation());

	patchShift = 0.0; // can't get this from SA

	PPts.SetFromVectorArray(saipPlane.PerimPts, t);
	samples = saipPlane.PerimPts.GetSize();
	maxerr= 0.0;
	rmserr = 0.0;

	double Aw,Bw,Cw,Dw;
	MapCoefficientsToWorking(&Aw, &Bw, &Cw, &Dw );
	PlaneText.Format(_T("A = %6.*f; B = %6.*f; C = %6.*f; D = %8.*f"),
		app.saarm.Data.Precision,Aw,
		app.saarm.Data.Precision,Bw,
		app.saarm.Data.Precision,Cw,
		app.saarm.Data.Precision,Dw/app.saarm.units.UnitToMeters[app.saarm.units.distunits]	);

	timeStamp.Format(saipPlane.Name + Tr(" from SA ") + CTime::GetCurrentTime().Format(_T("%H:%M:%S %m/%d/%Y")));
	TRACE1("Plane fit time stamp...%s\n", timeStamp);

	rmserrinfo.Empty();
	maxerrinfo.Empty();
}

void PlaneObj::MapCoefficientsToWorking(double* a, double* b, double* c, double* d)
{
	// get a point on the plane
//	Vector PtOnPlane( A, B, 0.0 );
//	PtOnPlane.z = -(A*A + B*B + D)/C;   //d was already shifted

//	CString m;
//	m.Format(_T("ABC: a=%f b=%f c=%f\n"), A,B,C);
//	AfxMessageBox(m);

	Vector PtOnPlane;
	t.GetTranslation(&PtOnPlane);
	Vector n( A, B, C );
	Vector nWrk;
	app.saarm.InstToWorking.MultiplyRot( n, &nWrk );		// map vector 

//	CString m;
//	Vector v;
//	app.saarm.InstToWorking.GetRotation(&v);
//	m.Format(_T("ABC inst: a=%f b=%f c=%f\n"
//		"n: a=%f b=%f c=%f\n"
//		"nWrk: a=%f b=%f c=%f"), A,B,C,n.x,n.y,n.z,nWrk.x,nWrk.y,nWrk.z);
//	AfxMessageBox(m);

	Vector PtOnPlanePrime;
	app.saarm.InstToWorking.Multiply( PtOnPlane, &PtOnPlanePrime );	// map point 

	nWrk.Normalize();			// safety.
	*a = nWrk.x;
	*b = nWrk.y;
	*c = nWrk.z;
	*d = -( (*a)*PtOnPlanePrime.x + (*b)*PtOnPlanePrime.y + (*c)*PtOnPlanePrime.z );

}

void PlaneObj::ProjectPointsToPlane(VectorArray& VA,VectorArray* pPtsOut)
{
	//in and out in inches since we're using arrays
//	TRACE0("PlaneObj::ProjectPointsToPlane, current plane transform:\n"));
//	Vector pos, rot;
//	t.GetTranslation(&pos);
//	t.GetRotation(&rot);
//	TRACE3("Pos: X=%f, Y=%f, Z=%f\n"), pos.x,pos.y,pos.z);
//	TRACE3("Rot: Rx=%f,Ry=%f,Rz=%f\n"), rot.x,rot.y,rot.z);
	pPtsOut->Clear();
	Vector TempPoint,PtInBase,PtOnPlane;
	Transform temp;
	t.Invert(&temp);
	for (int i=0;i<VA.GetSize();i++)
	{
		TempPoint = Vector(*VA.P[i]);
//		TRACE3("Pt in: X=%f,Y=%f,Z=%f\n"), VA.P[i]->x,VA.P[i]->y,VA.P[i]->z);
//		TempPoint.Scale(app.saarm.units.UnitToMeters[DISTUNITS_IN]); //same units as baseT
		temp.Multiply(TempPoint,&PtInBase);
		PtInBase.z = 0.0;
		t.Multiply(PtInBase,&PtOnPlane);
//		TRACE3("Pt out: X=%f,Y=%f,Z=%f\n\n"), PtOnPlane.x,PtOnPlane.y,PtOnPlane.z);
//		PtOnPlane.Scale(1.0/app.saarm.units.UnitToMeters[DISTUNITS_IN]);// back to inches
		pPtsOut->P.Add( new Vector(PtOnPlane) );
	}
}

void PlaneObj::ProjectPointToPlane(Vector& pIn,Vector* pOut)
{
	//in and out in meters
	Vector TempPoint,PtInBase,PtOnPlane;
	Transform temp;
	t.Invert(&temp);
	temp.Multiply(pIn,&PtInBase);
	PtInBase.z = 0.0;
	t.Multiply(PtInBase,pOut);
}

double PlaneObj::QueryPointToPlane(Vector& pt)
{
	//in and out in meters
	Vector PtInBase;
	Transform temp;
	t.Invert(&temp);
	temp.Multiply(pt,&PtInBase);
	return PtInBase.z;
}

void PlaneObj::operator=(const GeometryObj& cgeOb)
{
	GeometryObj::operator=(cgeOb);

	PlaneObj& geOb = (PlaneObj&)cgeOb;

	A = geOb.A;
	B = geOb.B;
	C = geOb.C;
	D = geOb.D;
	patchShift = geOb.patchShift;

	PlaneText = geOb.PlaneText;
	timeStamp = geOb.timeStamp;
}

void PlaneObj::Serialize(CArchive& ar) 
{
	GeometryObj::Serialize(ar);
	BYTE loadSchema = 0;
	BYTE schema = 1;
	schema = 2; // 2009.08.12 - added patchShift member
	schema = 3; // 2011.01.03 - added ArmPointArray to use instead of VectorArray

	if(ar.IsStoring())
	{
		ar << schema;
		ar <<  A;
		ar <<  B;
		ar <<  C;
		ar <<  D;
		ar <<  PlaneText;
		ar << timeStamp;
		oldPPts.Serialize(ar);
		ar << patchShift;
		PPts.Serialize(ar);
	}
	else
	{
		ar >> loadSchema;
		ar >>  A;
		ar >>  B;
		ar >>  C;
		ar >>  D;
		ar >>  PlaneText;
		ar >> timeStamp;
		oldPPts.Serialize(ar);

		if(loadSchema >= 2)
			ar >> patchShift;

		if(loadSchema >= 3)
			PPts.Serialize(ar);
	}
	
}
IMPLEMENT_SERIAL( PlaneObj, GeometryObj, 0 );