// RDSWindowsMessenger.cpp : implementation file
//

#include "stdafx.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "HexagonRDS.h"
#include "RDSWindowsMessenger.h"

// Register windows messages that will permit to communicate with RDS driver
RDS_ADD_PRIVATE_MESSAGES

// RDS_ON_* macros define dialog box event handlers. Be sure to indicate in parameters the correct class names.
RDS_ON_COPY_DATA_EASY(RDSWindowsMessenger, CDialog);
RDS_ON_SERVER_AVAILABLE(RDSWindowsMessenger, RDS_REMOTE_MOUSE_OFF);
RDS_ON_REMOVE_FROM_SERVER(RDSWindowsMessenger)


// RDSWindowsMessenger

IMPLEMENT_DYNAMIC(RDSWindowsMessenger, CDialog)

RDSWindowsMessenger::RDSWindowsMessenger(HexagonRDS* _owner)
: owner(*_owner)
, scannerCS(*new CriticalSectionWrapper)
{
	windowValid = false;
	firstMessageReceieved = false;
	armDisconnectedShutDownInProgress = false;
	probeType = -1;
	previousProbeType = 0;

	RDS_INITIALIZE;

	m_bButton1State = FALSE;
	m_bButton2State = FALSE;
	m_bButton3State = FALSE;
	m_bServerAvailable = FALSE;
	m_bEventEasy = FALSE;

	//TubeProbeState = 0;
	syncedScanData = false;

	configuredForTracker = false;
}

RDSWindowsMessenger::~RDSWindowsMessenger()
{
	RDS_REMOVE_FROM_SERVER;
	delete &scannerCS;
}

bool RDSWindowsMessenger::MakeValidWindow()
{
	// create our window
	CString strClassName;
	strClassName = AfxRegisterWndClass( NULL, 0, 0, 0 );
	if ( !CreateEx(0, strClassName, _T("RDSWinMessengerWnd"), WS_POPUP, 0, 0, 0, 0, NULL, NULL) )
		return false;
	windowValid = true;
	return true;
}

void RDSWindowsMessenger::Disconnect()
{
	RDS_REMOVE_FROM_SERVER;
}
void RDSWindowsMessenger::Reconnect()
{
	RDS_CONNECT_TO_SERVER;
}

BEGIN_MESSAGE_MAP(RDSWindowsMessenger, CWnd)
	// Declare RDS messages handlers
	RDS_MSGS
END_MESSAGE_MAP()

// RDSWindowsMessenger message handlers

void RDSWindowsMessenger::RunProbeCal()
{
	// Macro initiates probe calibration. Let RDS choose appropriated method
	// Override x, y, width and height
	RDS_CALIBRATE_PROBE(0, 0, 0, 0, CM_UNKNOWN);
}
void RDSWindowsMessenger::DoArmSettings()
{
	// Macro calls RDS control panel
	RDS_SETTINGS;
}

BOOL RDSWindowsMessenger::IsRemoteMouseEnabled()
{
	return m_bRemoteMouseON;
}
void RDSWindowsMessenger::ToggleRemoteMouse()
{
	if(m_bRemoteMouseON)
	{
		RDS_REMOTE_MOUSE(RDS_REMOTE_MOUSE_OFF);
	}
	else
	{
		RDS_REMOTE_MOUSE(RDS_REMOTE_MOUSE_ON);
	}
}
void RDSWindowsMessenger::RemoteMouse(BOOL flag)
{
	if(flag)
	{
		RDS_REMOTE_MOUSE(RDS_REMOTE_MOUSE_ON);
	}
	else
	{
		RDS_REMOTE_MOUSE(RDS_REMOTE_MOUSE_OFF);
	}
}

void RDSWindowsMessenger::OutputSpeakerSound( int frequency, int duration )
{
	int volumeLevel = app.saarm.Data.SpkrVolume*100/40;
	RDS_PLAY_SOUND(frequency, duration, volumeLevel);
}

void RDSWindowsMessenger::ArmBeep(int sound)
{
	int volumeLevel = app.saarm.Data.SpkrVolume*100/40;

	switch(sound)
	{
	case REC_OK:
		{
			RDS_PLAY_SOUND(1400,120, volumeLevel);
			//theApp.TTS.PlayTone(1400,120);
			break;
		}
	case CALC_OK:
		{
			RDS_PLAY_SOUND(2000,80, volumeLevel);
			//theApp.TTS.PlayTone(1400,120);
			break;
		}
	case UNDO_OK:
		{
			RDS_PLAY_SOUND(800,200, volumeLevel);
			//theApp.TTS.PlayTone(800,200);
			break;
		}
	case UNDO_FAIL:
		{
			RDS_PLAY_SOUND(600,100, volumeLevel);
			//theApp.TTS.PlayTone(600,100);
			break;
		}
	case JT_LIMIT:
		{
			RDS_PLAY_SOUND(3000,250, volumeLevel);
			Delay(0.1,TRUE);
			RDS_PLAY_SOUND(2500,250, volumeLevel);
		}
	}
}

//void RDSWindowsMessenger::Playmelody()
//{
//	RDS_PLAY_MELODY(RDS_PROBE_PULLED_OUT);
//}

void RDSWindowsMessenger::AddToLog(LPCTSTR criticity, LPCTSTR format, ...)
{	
	TRACE2("%s - %s\n", criticity, format);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// defined in Hexagon's RDS_API.h

///
/// ********************************* RDS Events *********************************************
///
// RDS_VALID_DATA is automatically called when program receives a "valid" RDS event
void RDSWindowsMessenger::	RDS_REFERENCE_DATA(sRDSReference& p)
{
	if(TOO_LOK == p.RefType)
		AddToLog(_T(""), _T("TooLOK  ID: %I64lu Engaged: %d Part Name: %s"), p.ID, p.Engaged, p.PartName);
	else if(SPACE_LOK == p.RefType)
		AddToLog(_T(""), _T("SpaceLOK  ID: %I64lu Engaged: %d"), p.ID, p.Engaged);
	else if(GRID_LOK == p.RefType)
		AddToLog(_T(""), _T("GridLOK  ID: %I64lu Engaged: %d"), p.ID, p.Engaged);
	else if(LPF_REF == p.RefType)
		AddToLog(_T(""), _T("LeapFrog  ID: %I64lu Engaged: %d"), p.ID, p.Engaged);
}


// RDS_VALID_DATA is automatically called when program receives a "valid" RDS event
void RDSWindowsMessenger::RDS_SYNCED_DATA(sRDSPosition& p)
{
	ProcessArmData(true, p);
	m_bEventEasy = TRUE;

	//CString str;
	//str.Format(_T("%.2lf"), p.ProbeMovementSpeed);
	//GetDlgItem(IDC_PROBE_SPEED)->SetWindowText(str);
}


// RDS_INVALID_DATA is automatically called when program receives an "invalid" RDS event
void RDSWindowsMessenger::RDS_UNSYNCED_DATA(sRDSPosition& p)
{	
	ProcessArmData(false, p);
	m_bEventEasy = FALSE;
}

static int updateInc = 0;
void RDSWindowsMessenger::ProcessArmData(bool reliablePos, sRDSPosition& p)
{
	if(p.EventType & EVENT_SCANNING_START)
	{
		owner.SendOnScanPassStarted(); // 2020.01.07 - XIT-1616 - put this back for new SA meshing with resharper
		syncedScanData = reliablePos;
	}
	if(p.EventType & EVENT_SCANNING_STOP)
	{
		//owner.SendOnScanPassStopped();

		if (app.saarm.Data.voxelizationEnabled)
			owner.ClearScanClouds();

		syncedScanData = false;
	}
	if(p.EventType & EVENT_OVER_LIMIT)
	{
		syncedScanData = false;
		//return; // 2016.05.13 - commented this out so we can still process updates and button states
	}
	if (p.EventType & EVENT_SCANNING)
	{
		// 2016.05.11 - below is from SA2 comments. Here, we are making our own "bRliablePos" from the 2 separate events we get in native code, RDS_SYNCED_DATA and RDS_UNSYNCED_DATA (see above)
		// 2015.10.06 - Jean-Paul says:
		// bReliablePos will tell you if position and following profile is a measurement or only previewing information. 
		// Therefore no need to test EVENT_SYNC flag, test bReliablePos instead.
		// With time, I try to remove references to word SYNC which is sometimes confusing.
		syncedScanData = reliablePos;
		//Debug.WriteLine("EVENT_SCANNING AND SYNC received");
	}
	else
	{
		syncedScanData = false;
	}

	//if(p.EventType != 0)
	//{
	//	TRACE1("event = %x\n", p.EventType);
	//}
	//
	//if(p.EventType & EVENT_BEAM1_BLCK_FWD)
	//{
	//	TRACE0("1 EVENT_BEAM1_BLCK_FWD\n");
	//	TubeProbeState += EVENT_BEAM1_BLCK_FWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);

	//	owner.X = p.ProbeXYZ[RDS_X]/1000.0;
	//	owner.Y = p.ProbeXYZ[RDS_Y]/1000.0;
	//	owner.Z = p.ProbeXYZ[RDS_Z]/1000.0;

	//	owner.II = -p.ProbeMatrix[2][0];
	//	owner.JJ = -p.ProbeMatrix[2][1];
	//	owner.KK = -p.ProbeMatrix[2][2];

	//	armFrame.T[0][0] = p.ProbeMatrix[0][0];
	//	armFrame.T[1][0] = p.ProbeMatrix[0][1];
	//	armFrame.T[2][0] = p.ProbeMatrix[0][2];

	//	armFrame.T[0][1] = -p.ProbeMatrix[1][0];
	//	armFrame.T[1][1] = -p.ProbeMatrix[1][1];
	//	armFrame.T[2][1] = -p.ProbeMatrix[1][2];

	//	armFrame.T[0][2] = -p.ProbeMatrix[2][0];
	//	armFrame.T[1][2] = -p.ProbeMatrix[2][1];
	//	armFrame.T[2][2] = -p.ProbeMatrix[2][2];

	//	//TRACE1("Beam1 Blocked Fwd, XYZ:  %f, ", p.ProbeXYZ[RDS_X]);
	//	//TRACE2("%f, %f\n", p.ProbeXYZ[RDS_Y], p.ProbeXYZ[RDS_Z]);
	//	//TRACE0("Xform:\n");
	//	//armFrame.Debug();
	//	//TRACE0("\n\n");
	//	//return;
	//}
	//if(p.EventType & EVENT_BEAM1_UNBLCK_FWD)
	//{
	//	TRACE0("2 EVENT_BEAM1_UNBLCK_FWD\n");
	//	TubeProbeState += EVENT_BEAM1_UNBLCK_FWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM1_BLCK_BCKWD)
	//{
	//	TRACE0("3 EVENT_BEAM1_BLCK_BCKWD\n");
	//	TubeProbeState += EVENT_BEAM1_BLCK_BCKWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM1_UNBLCK_BCKWD)
	//{
	//	TRACE0("4 EVENT_BEAM1_UNBLCK_BCKWD\n");
	//	TubeProbeState += EVENT_BEAM1_UNBLCK_BCKWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM2_BLCK_FWD)
	//{
	//	TRACE0("5 EVENT_BEAM2_BLCK_FWD\n");
	//	TubeProbeState += EVENT_BEAM2_BLCK_FWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM2_UNBLCK_FWD)
	//{
	//	TRACE0("6 EVENT_BEAM2_UNBLCK_FWD\n");
	//	TubeProbeState += EVENT_BEAM2_UNBLCK_FWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM2_BLCK_BCKWD)
	//{
	//	TRACE0("7 EVENT_BEAM2_BLCK_BCKWD\n");
	//	TubeProbeState += EVENT_BEAM2_BLCK_BCKWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}
	//if(p.EventType & EVENT_BEAM2_UNBLCK_BCKWD)
	//{
	//	TRACE0("8 EVENT_BEAM2_UNBLCK_BCKWD\n");
	//	TubeProbeState += EVENT_BEAM2_UNBLCK_BCKWD;
	//	TRACE1("TubeProbeState = %x\n", TubeProbeState);
	//}

	//if(TubeProbeState >= 0x7F8)
	//	TubeProbeState = 0;

	// Position and Orientation....

	owner.X = p.ProbeXYZ[RDS_X]/1000.0;
	owner.Y = p.ProbeXYZ[RDS_Y]/1000.0;
	owner.Z = p.ProbeXYZ[RDS_Z]/1000.0;

	owner.II = -p.ProbeMatrix[2][0];
	owner.JJ = -p.ProbeMatrix[2][1];
	owner.KK = -p.ProbeMatrix[2][2];

	armFrame.T[0][0] = p.ProbeMatrix[0][0];
	armFrame.T[1][0] = p.ProbeMatrix[0][1];
	armFrame.T[2][0] = p.ProbeMatrix[0][2];

	armFrame.T[0][1] = -p.ProbeMatrix[1][0];
	armFrame.T[1][1] = -p.ProbeMatrix[1][1];
	armFrame.T[2][1] = -p.ProbeMatrix[1][2];

	armFrame.T[0][2] = -p.ProbeMatrix[2][0];
	armFrame.T[1][2] = -p.ProbeMatrix[2][1];
	armFrame.T[2][2] = -p.ProbeMatrix[2][2];

	owner.CurrentTransform.MultiplyRot(armFrame, owner.NativeArmToSA);

	owner.CurrentTransform.T[0][3] = owner.X;
	owner.CurrentTransform.T[1][3] = owner.Y;
	owner.CurrentTransform.T[2][3] = owner.Z;

	//for(int i=0; i<=5; i++)
	//{
	//	TRACE2("j%d %f\n", i, p.Angles[i]/DTR);
	//}

	// joint values

	if(owner.DOF >= 6)
	{
		owner.J.x[0] = p.Angles[0]/DTR+180.0;
		owner.J.x[1] = p.Angles[1]/DTR+90.0;
		owner.J.x[2] = p.Angles[2]/DTR;
		owner.J.x[3] = p.Angles[3]/DTR+90.0;
		owner.J.x[4] = -p.Angles[4]/DTR;
		owner.J.x[5] = p.Angles[5]/DTR;
	}
	if(owner.DOF >= 7)
	{
		owner.J.x[5] = -p.Angles[5]/DTR;
		owner.J.x[6] = 90.0-p.Angles[6]/DTR;
		//TRACE2("j%d %f\n", 6, p.Angles[6]/DTR);
	}

	// button states

	m_bButton1State = p.ButtonsStates[0];
	m_bButton2State = p.ButtonsStates[1];
	m_bButton3State = p.ButtonsStates[2];

	// 2016.05.13 - update SA graphic view when scanner on, but not scanning
	updateInc++;
	if(updateInc == 20)
	{
		updateInc = 0;
		if(owner.armScannerSwitchOn && !syncedScanData)
		{
			owner.UpdateToSpan();
		}
	}

	if(probeType == PT_TRIGGER_PROBE && (p.EventType & EVENT_EASY) != 0)
	{
		// then a touch probe has been triggered...
		owner.OnTouchTriggerPressed();
	}
}
//static int scanFrameInc = 100;
void RDSWindowsMessenger::OnPostInit()
{
	if( probeType == PT_HEXAGON_SCANNER || probeType == PT_CMS_SCANNER || probeType == PT_EXTERNAL_TRIGGER)
	//	|| PT_RS6_SCANNER	 // Covers RS6
	//	|| PT_AREA_SCANNER) // Covers Zeus
	{
		owner.armScannerSwitchOn = true;
	}
	else
	{
		//scanFrameInc = 100;
		owner.armScannerSwitchOn = false;
	}

	if(app.m_pMainWnd != NULL)
	{
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		if(d.IsWindowVisible())  // note that RDS_PROBE_DATA (below) calls this
		{
			if (!owner.CMMSerialNum.IsEmpty()) // 2019.11.18 - JP says they do get this despite the Faro patent on it - BUT, only if the pc is connected to the internet!
				d.jtlimit.SetWindowText(app.pA->CMMSerialNum);
			else
				d.jtlimit.SetWindowText(probeName); // this is probably more valuable to show anyway, since we already send the s/n with all data if it is not empty, but Patrick W. asked about this, likely because we show it with Faro arms
		}
		else
			app.saarm.SendSAIRCAttributesToSA();
	}
}

// RDS_PROBE_DATA is automatically called when probe changes
void RDSWindowsMessenger::RDS_PROBE_DATA(sRDSProbe& probe)
{
	//m_strEvent = _T("RDS_PROBE_DATA event");
	TRACE1(">>>Probe Type = %d\n", probe.ProbeType);

	app.saarm.Data.TipDia = probe.BallDiameter/1000.0;
	probeType = probe.ProbeType;
	//probeName.Format(_T("%s"), CA2T(probe.Name));
	//probeName.Format(_T("%s"), probe.Name);
	probeName = CString(probe.Name);

	//CString pMess, typeName;
	//typeName.Format(_T("%s"), CA2T(probe.TypeName));
	//pMess.Format(_T("Type=%d, Index=%ld, Name=%s, TypeName=%s"), probe.ProbeType, probe.Index, probeName, typeName);
	//AfxMessageBox(pMess);
	//return;

	OnPostInit();


	//bool isScanner = probe.IsAScanner();
	//if(isScanner)
	//	AfxMessageBox(_T("IS a scanner"));
	//else
	//	AfxMessageBox(_T("NOT a scanner"));

	// ArmBase::CheckAndUpdateModelInfo() does not send the update if probeValid is false. We need to set it false when there is no valid hard probe mounted
	// because we still get probe position data from this arm when no probe is attached (or when line scanning), and that would mess up the SA graphics model auto-correction
	if(probe.ProbeType == PT_UNKNOWN || probe.ProbeType == PT_VIDEO_PROBE_V2
		|| probe.ProbeType == PT_VIDEO_PROBE_V3 || probe.ProbeType == PT_HEXAGON_SCANNER
		|| probe.ProbeType == PT_CMS_SCANNER || probe.IsAScanner()	// 2011.11.07 - added IsAScanner call in case they add another scanner type that's valid for this - redundant with scanner ProbeTypes, but doens't hurt anything
		|| probe.ProbeType == PT_EXTERNAL_TRIGGER)	// 2011.11.08 - this is how we recognize the Perceptron scanner
	//	|| PT_RS6_SCANNER	 // Covers RS6
	//	|| PT_AREA_SCANNER) // Covers Zeus
	{
		owner.probeValid = false;
	}
	else
	{
		owner.probeValid = true;
	}

	// SI or SE with CMS scanner...
	//if(probe.ProbeType == PT_HEXAGON_SCANNER || probe.ProbeType == PT_CMS_SCANNER)
	if(probe.IsAScanner())	// 2011.11.07 - change to this RDS call in case they add another scanner type that's valid for this.
	{
		//AfxMessageBox(_T("Let's Scan!"));
		owner.armScannerSwitchOn = true;
		CString err;
		if(previousProbeType != probeType && owner.ConnectedToArm && owner.mainDlgIsInitialized) // JIRA XIT-29 - 2015.05.17 - check for mainDlgIsInitialized added
		{
			TRACE0("switch to scanning...\n");
			previousProbeType = probeType;

			owner.SendOnScanPassStarted();

			// 2019.03.18 - don't increment name here - causes confustion between autoIncCloudName and incrementgroup
			app.saarm.EndMeasurement();
			//if(app.saarm.EndMeasurement())
			//{
			//	if(app.saarm.Data.incrementgroup)
			//	{
			//		app.saarm.Data.Target.Format(_T("P1"));
			//		app.pA->IncrementName(app.saarm.Data.Group, 1);
			//	}
			//}
			owner.SetMeasModeByName(_T("Scanner"), err);
			owner.StartScannerMeas();
		}
		return;
	}
	else if(probe.ProbeType != PT_EXTERNAL_TRIGGER)
	{
		owner.armScannerSwitchOn = false;
		if(previousProbeType != probeType && owner.ConnectedToArm && owner.mainDlgIsInitialized)
		{
			TRACE0("switch to probing...\n");
			previousProbeType = probeType;

			owner.SendOnScanPassStopped();

			if(owner.StopScannerMeas())
			{
				if(app.saarm.Data.autoIncCloudName)
				{
					app.saarm.Data.Target.Format(_T("P1"));
					app.pA->IncrementName(app.saarm.Data.Group, 1);
				}
			}
			owner.SetMeasMode(0);

			if(Units::toolBarModeOn)
			{
				CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
				d.RestartMeasurement();
			}
		}
		return;
	}

	// SE with Perceptron Scanner
	if(probe.ProbeType == PT_EXTERNAL_TRIGGER)
	{
		TRACE0("switch to Perceptron scanning...\n");
		owner.armScannerSwitchOn = true;
		CString err;
		if(previousProbeType != probeType && owner.ConnectedToArm && owner.mainDlgIsInitialized)
		{
			previousProbeType = probeType;
			app.saarm.EndMeasurement();
			owner.SetMeasModeByName(_T("Scanner"), err);
			owner.StartPerceptronScannerMeas();
		}
	}
	else if(!probe.IsAScanner())
	{
		TRACE0("switch OFF Perceptron Scanning...\n");
		if(previousProbeType != probeType && owner.ConnectedToArm && owner.mainDlgIsInitialized)
		{
			previousProbeType = probeType;
			owner.StopPerceptronScannerMeas();
		}
		owner.armScannerSwitchOn = false;
	}

	//m_strProbe.Format(_T("Index=%ld, Name=%s, Type=%s"), probe.Index, probe.Name, probe.TypeName);

	//UpdateData(FALSE);
}


// RDS_ARM_DATA is called every 500ms and gives interesting information about connection with the arm
void RDSWindowsMessenger::RDS_ARM_DATA(sRDSArm& arm)
{
	//m_strEvent = _T("RDS_ARM_DATA event");

	//m_strArm.Format(_T("ArmType=%ld, Status=%s, SpeakerVolume=%lu"), arm.ArmType, arm.ArmStatusText, arm.SpeakerVolume);

	if(!firstMessageReceieved)
	{
		firstMessageReceieved = true;

		if(arm.ArmType == RDSAT_LEICA_TRACKER_LMF || arm.ArmType == RDSAT_LEICA_TRACKER_EMSCON)
		{
			configuredForTracker = true;
			return;
		}
		else
			configuredForTracker = false;

		if(arm.ArmStatus == AS_CONNECTED || arm.ArmStatus == AS_CONNECTED_WARNING)
		{
			if( arm.ArmStatus == AS_CONNECTED_WARNING)
			{
				CString warning;
				warning.Format(_T("%s"), arm.ArmStatusText);
				//AfxMessageBox(warning, MB_ICONINFORMATION); // forget it - they send a warning when the arm is in the home position, and when it has no probe attached (needed state for scanning), stuff like that
			}

			if (owner.CMMSerialNum.IsEmpty())
			{
				TRACE1("arm status = %s\n", arm.ArmStatusText);
				TRACE1("arm sn = %s\n", arm.SerialNumber);
				CString armSN(arm.SerialNumber);
				owner.CMMSerialNum = armSN;
			}

			owner.ConnectedToArm = TRUE;
		}
	}
	else if(owner.ConnectedToArm && arm.ArmStatus == AS_NOT_CONNECTED && !armDisconnectedShutDownInProgress)
	{
		armDisconnectedShutDownInProgress = true;
		int ret = AfxMessageBox(Tr("Arm Disconnected! Shut down?"), MB_YESNO);
		if(ret==IDYES)
		{
			PostQuitMessage(0);
		}
		armDisconnectedShutDownInProgress = false;
	}


	m_bRemoteMouseON = arm.IsRemoteMouseON;
	m_bVCRunning = arm.bVideoCaptureRunning;
	m_bHeadLightIsOn = arm.bHeadLightOn;

	//NCAAT_RA7,	// RA7
	//NCAAT_RA7i,	// RA7i (=RA7si)
	//NCAAT_RA7m,	// RA7m (=RA7se)

	//if(arm.ArmStatus == AS_CONNECTED)
	//{
	//	if(arm.NCAArmType == NCAAT_RA7i)
	//		AfxMessageBox(_T("SI arm"));
	//	if(arm.NCAArmType == NCAAT_RA7m)
	//		AfxMessageBox(_T("SE arm"));

	//}

	UpdateData(FALSE);
}

void RDSWindowsMessenger::RDS_SCANNER_DATA(sRDSScannerHeader* pHeader, sRDSScannerPoint* pPoints)
{
	CString str;

	//TRACE1("Scan Pts: %lu\n", pHeader->NbPoints);

	//if(m_bButton2State && pHeader->NbPoints > 0)
	if(syncedScanData && pHeader->NbPoints > 0) // 2016.05.11 - see use of our syncedScanData flag above (finally got around to making this interface use the RDS Control Panel settings, "Press to start and to stop", and "Keep button pressed" for scanning)
	{
		//TRACE3("Pt 0: %.3lf, %.3lf, %.3lf\n", pPoints[0].SystemXYZ[0], pPoints[0].SystemXYZ[1], pPoints[0].SystemXYZ[2]);

		if(app.saarm.Data.useScanStripeClouds && owner.performingStdScannerMeas)
		{
			owner.AddScannerCloudStripe();

			SAIPScanStripePoint sspt;
			Vector xyzRDS;

			for(DWORD k=0; k<pHeader->NbPoints; k++)
			{
				if((int)k % app.saarm.Data.thinningFactor != 0)
					continue;

				xyzRDS.x = pPoints[k].SystemXYZ[0];
				xyzRDS.y = pPoints[k].SystemXYZ[1];
				xyzRDS.z = pPoints[k].SystemXYZ[2];
				xyzRDS.Scale(1.0/25.4);

				sspt.x = xyzRDS.x;
				sspt.y = xyzRDS.y;
				sspt.z = xyzRDS.z;

				owner.AddScannerCloudStripePt(sspt);
			}
			CriticalSectionLock lock( scannerCS );

			Transform scannerXform = owner.CurrentTransform;
			scannerXform.Scale(app.saarm.LocalToInches);
			owner.scanStripePkt.SetCurrentLineTransform(scannerXform);
		}
		else
		{
			SAIPCloudBlockPt cbpt;
			Vector xyzRDS;

			for(DWORD i=0; i<pHeader->NbPoints; i++)
			{
				if((int)i % app.saarm.Data.thinningFactor != 0)
					continue;

				xyzRDS.x = pPoints[i].SystemXYZ[0];
				xyzRDS.y = pPoints[i].SystemXYZ[1];
				xyzRDS.z = pPoints[i].SystemXYZ[2];
				xyzRDS.Scale(1.0/25.4);

				cbpt.x = xyzRDS.x;
				cbpt.y = xyzRDS.y;
				cbpt.z = xyzRDS.z;

				owner.AddScannerCloudPt(cbpt);
			}
		}

		//if(scanFrameInc % 100 == 0)
		//{
		//	owner.FrameToSpan(owner.CurrentTransform, _T("scan frame"));
		//}
		//scanFrameInc++;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 2020.04.06 - RS-Squared (Zeus)
//void RDSWindowsMessenger::RDS_SCANNER_DATA_TILE(RDS_API_NAMESPACE sRDSTileHeader* pHeader, RDS_API_NAMESPACE sRDSTilePoint* pPoints)
//{
//	if(pHeader->NbPoints > 0)
//	{
//		//TRACE3("Pt 0: %.3lf, %.3lf, %.3lf\n", pPoints[0].SystemXYZ[0], pPoints[0].SystemXYZ[1], pPoints[0].SystemXYZ[2]);
//
//		if(app.saarm.Data.useScanStripeClouds && owner.performingStdScannerMeas)
//		{
//			SAIPScanStripePoint sspt;
//			Vector xyzRDS;
//
//			for (int i = 0; i < pHeader->Width; i++)
//			{
//				// make a line for this row
//				owner.AddScannerCloudStripe();
//
//				for (int j = 0; j < pHeader->Height; j++)
//				{
//					if (pPoints[i * pHeader->Width + j].SystemXYZ[2] < 10000.0) // this is Farmid's way of sending a missing point
//					{
//						if((int)k % app.saarm.Data.thinningFactor != 0)
//							continue;
//
//						xyzRDS.x = pPoints[i * pHeader->Width + j].SystemXYZ[0];
//						xyzRDS.y = pPoints[i * pHeader->Width + j].SystemXYZ[1];
//						xyzRDS.z = pPoints[i * pHeader->Width + j].SystemXYZ[2];
//						xyzRDS.Scale(1.0/25.4);
//
//						sspt.x = xyzRDS.x;
//						sspt.y = xyzRDS.y;
//						sspt.z = xyzRDS.z;
//
//						owner.AddScannerCloudStripePt(sspt);
//					}
//					//else
//					//    line.points.Add(new NPoint(null));
//				}
//				CriticalSectionLock lock( scannerCS );
//
//				Transform scannerXform = owner.CurrentTransform;
//				scannerXform.Scale(app.saarm.LocalToInches);
//				owner.scanStripePkt.SetCurrentLineTransform(scannerXform);
//			}
//		}
//		else
//		{
//			SAIPCloudBlockPt cbpt;
//			Vector xyzRDS;
//
//			for(DWORD i=0; i<pHeader->NbPoints; i++)
//			{
//				if((int)i % app.saarm.Data.thinningFactor != 0)
//					continue;
//
//				xyzRDS.x = pPoints[i].SystemXYZ[0];
//				xyzRDS.y = pPoints[i].SystemXYZ[1];
//				xyzRDS.z = pPoints[i].SystemXYZ[2];
//				xyzRDS.Scale(1.0/25.4);
//
//				cbpt.x = xyzRDS.x;
//				cbpt.y = xyzRDS.y;
//				cbpt.z = xyzRDS.z;
//
//				owner.AddScannerCloudPt(cbpt);
//			}
//		}
//}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// RDS_SERVER_UP is called when application is connected to service
void RDSWindowsMessenger::RDS_SERVER_UP(void)
{
	TRACE0("Server is up !");
}


// RDS_SERVER_DOWN is called when application is disconnected from service
void RDSWindowsMessenger::RDS_SERVER_DOWN(void)
{
	TRACE0("Server is down !");

	owner.ConnectedToArm = FALSE;
}


void RDSWindowsMessenger::RDS_VIDEO_CAPTURE_FRAME(sRDSVideoCaptureFrame* pFrame, byte* pFrameData)
{
	//m_strEvent = _T("RDS_VIDEO_CAPTURE_FRAME event");

	CString str;

	str.Format(_T("FrameData=0x%0lx, FrameSize=%lu, ImageFormat=%lu"), pFrameData, pFrame->dwFrameSize, pFrame->ImageFormat);

	AddToLog(_T(""), str);

	UpdateData(FALSE);
}


void RDSWindowsMessenger::RDS_ON_RESULT(eRDSChunkID RequestID, eRDSReturnCode ReturnCode, void* lpResult, dword dwResultSize)
{
	//AddToLog(ReturnCode == 0 ? _T("RE") : _T("CR"), _T("Result chunk received: ID=%lu, rc=%lu (%s), result=0x%lx, resultsize=%ld"), RequestID, ReturnCode, GetRDSAPIErrorDescription(ReturnCode), lpResult, dwResultSize);

	switch(RequestID)
	{
	case RDSCI_GET_AVAILABLE_PROBES:

		if(ReturnCode == RDSRC_SUCCESS)
		{
			if(dwResultSize%sizeof(sRDSProbe) != 0)
			{
				AddToLog(_T("CR"), _T("Invalid structure size ! %lu "), dwResultSize);
				return;
			}

			//dword line = 0;
			//Lc_Probes.InsertItem(line, _T("0"));
			//Lc_Probes.SetItemText(line, 1, _T("Automatic selection"));
			//line++;

			//CString str;
			//TCHAR	szBuffer[128+1];
			//dword dwNbAvailableProbes = dwResultSize/sizeof(sRDSProbe);

			//for(dword p=0 ; p<dwNbAvailableProbes ; p++, line++)
			//{
			//	sRDSProbe* pProbe = &((sRDSProbe*)lpResult)[p];
			//	Lc_Probes.InsertItem(line, _T(""));

			//	int subitem = 0;

			//	_stprintf(szBuffer, _T("%I64lu"), pProbe->ID);
			//	Lc_Probes.SetItemText(line, subitem++, szBuffer);

			//	Lc_Probes.SetItemText(line, subitem++, pProbe->Name);

			//	str.Format(_T("%s"), pProbe->TypeName);		Lc_Probes.SetItemText(line, subitem++, str);
			//	str.Format(_T("%.3lf"), pProbe->BallDiameter);		Lc_Probes.SetItemText(line, subitem++, str);

			//	str.Format(_T("%.3lf"), pProbe->CalculationResult);		

			//	Lc_Probes.SetItemText(line, subitem++, str);
			//	Lc_Probes.SetItemText(line, subitem++, datetimeToCString(pProbe->CreationDateTime));
			//	Lc_Probes.SetItemText(line, subitem++, datetimeToCString(pProbe->UpdateDateTime));

			//	_stprintf(szBuffer, _T("%I64lu"), pProbe->ID);
			//	Lc_Probes.SetItemText(line, subitem++, szBuffer);

			//	_stprintf(szBuffer, _T("0x%lx"), pProbe->StatusBits);
			//	Lc_Probes.SetItemText(line, subitem++, szBuffer);
			//}
		}

		break;


	case RDSCI_GET_SETTING:

		if(ReturnCode == RDSRC_SUCCESS)
		{
			if(dwResultSize != sizeof(sRDSSetting))
			{
				AddToLog(_T("CR"), _T("Invalid structure size ! %lu "), dwResultSize);
				return;
			}
			sRDSSetting* pSetting = (sRDSSetting*)lpResult;

			//GetDlgItem(IDC_SETTING_VALUE)->SetWindowText(pSetting->Value);
		}
		break;

	case RDSCI_GET_SETTING_V2:

		if(ReturnCode == RDSRC_SUCCESS)
		{
			if(dwResultSize < sizeof(sRDSSettingV2))
			{
				AddToLog(_T("CR"), _T("Invalid structure size ! %lu "), dwResultSize);
				return;
			}
			sRDSSettingV2* pSetting = (sRDSSettingV2*)lpResult;
			byte*	pbyValue = (byte*)lpResult + sizeof(sRDSSettingV2);
			int		iSizeOfValue = dwResultSize - sizeof(sRDSSettingV2);

			//sRDSSettingCheck* pCheck = GetRDSSettingChecks(pSetting->ID);
			//if(pCheck == NULL)	return;

			CString str;

			// Extract result
			//switch(pCheck->Type)
			//{
			//case SVT_WSTRING:			str.Format(_T("%ws"), (WCHAR*)pbyValue);	break;
			//case SVT_NVALUES:
			//	{
			//		sRDSAvailableValue*	pValues = (sRDSAvailableValue*)pbyValue;

			//		str = _T("Please check log for result");

			//		for(int i=0 ; i<iSizeOfValue/sizeof(sRDSAvailableValue) ; i++)
			//			AddToLog(_T(""), _T("Available value %ld, %ws "), pValues[i].iValue, pValues[i].sName);
			//		break;
			//	}
			//case SVT_INT:					str.Format(_T("%ld"), *(int*)pbyValue);	break;
			//case SVT_DOUBLE:			str.Format(_T("%lf"), *(double*)pbyValue);	break;
			//case SVT_KEYSTONE:
			//	{
			//		sRDSKeyStone	Keystone;
			//		memcpy(&Keystone, pbyValue, sizeof(sRDSKeyStone));
			//		str.Format(_T("%lf,%lf,%lf,%lf"), Keystone.XLeft, Keystone.XRight, Keystone.ZNear, Keystone.ZFar);	
			//		break;
			//	}
			//default:	AddToLog(_T("CR"), _T("Unknown type %lu"), pCheck->Type);
			//}

			//GetDlgItem(IDC_SETTING_VALUE2)->SetWindowText(str);
		}
		break;
	}

}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// For SE with Perceptron Scanner
LRESULT RDSWindowsMessenger::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	if(message == CLOSE_SCAN_DLG)
		owner.trackScanDlg.CloseDialog();

	if(message == SEND_BUFFERED_DATA_AND_RECONNECT)
		owner.SendBufferedCloudDataAndReconnect();

	if(message == UPDATE_SCAN_DLG)
		owner.OnUpdateScanDlg();

	return CDialog::WindowProc(message, wParam, lParam);
}
