/////////////////////////////////////////////////////////////////////////////
//
// RDSAPI.H
// (C) Copyright 2000 - 2003 Romer, Inc. All rights reserved.
// This file defines the data structures and API to access WINRDS.DLL
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _RDSAPI_H_
#define _RDSAPI_H_

#ifdef __cplusplus
#define _RDS_EXTERN extern "C"
#else
#define _RDS_EXTERN extern
#endif

#define _RDS_API  _RDS_EXTERN

#define _RDS_CALL __stdcall

#define R6SC_DLL	1
#define RDENSC_DLL	2

#define	WINRDS_MASTER_KEY			HKEY_CURRENT_USER
#define REG_WINRDS_ROOT				_T("SOFTWARE\\Romer\\WinRDS\\")
#define	REG_SECTION_GENERAL			_T("General")
#ifdef _WIN64		// NOTE: You may have to define this yourself. The x64 compiler has a bug in VS 2005
#define	REG_VALUE_INSTALL_DIRECTORY	_T("InstallDirectory64")
#define WINRDS_DLL_NAME	_T("WinRDS64")
#else
#define	REG_VALUE_INSTALL_DIRECTORY	_T("InstallDirectory")
#define WINRDS_DLL_NAME	_T("WinRDS")
#endif
//***************************************************************************
//*                     rds_record_functions ENUMERATOR
//***************************************************************************

enum rds_record_functions
{
	RDS_RECORD_NOW =								0x00000000,
	RDS_RECORD_ON_BUTTON_ONE_GOING_ON =				0x00000001,
	RDS_RECORD_ON_BUTTON_TWO_GOING_ON =				0x00000002,
	RDS_RECORD_ON_BUTTON_THREE_GOING_ON =			0x00000004,	
	RDS_RECORD_ON_BUTTON_ONE_ON =					0x00000008, // 4/19/00
	RDS_RECORD_ON_BUTTON_TWO_ON =					0x00000010, // 4/19/00
	RDS_RECORD_ON_BUTTON_THREE_ON =					0x00000020,	// 4/19/00
	RDS_RECORD_ON_EVENT_ONE_GOING_ACTIVE =			0x00000040,
	RDS_RECORD_ON_EVENT_TWO_GOING_ACTIVE =			0x00000080,
	RDS_RECORD_ON_EVENT_ONE_GOING_INACTIVE =		0x00000100,
	RDS_RECORD_ON_EVENT_TWO_GOING_INACTIVE =		0x00000200,
	RDS_RECORD_ON_POWER_PROBE_ACTIVE =				0x00000400,
	RDS_RECORD_ON_LASER_ANALOG_PROBE_ACTIVE =		0x00000800,
	RDS_RECORD_ON_LASER_TRIGGER_PROBE_ACTIVITY =	0x00001000,

	RDS_RECORD_ON_CLOCK_GOING_ACTIVE =              0x00002000,
	RDS_RECORD_ON_CLOCK_GOING_INACTIVE =            0x00004000,

	RDS_RECORD_TREAT_POWER_PROBE_AS_RIGID =			0x00008000,

	RDS_RECORD_ON_TUBE_PROBE_ACTIVITY =				0x00FF0000,

	RDS_RECORD_WHEN_INSIDE_GUIDED_PLANE_RANGE = 	0x01000000,
	RDS_RECORD_WHEN_INSIDE_GUIDED_CIRCLE_RANGE =	0x02000000,
	RDS_RECORD_WHEN_INSIDE_GUIDED_RING_RANGE =		0x04000000,

	RDS_RECORD_USING_PROBE_DETECTED =				0x20000000,	
	RDS_RECORD_USING_SCAN_MODE =					0x40000000,
	RDS_RECORD_CALLBACK_ERROR =						0x80000000,
};


//***************************************************************************
//*                 rds_callback_return_states ENUMERATOR
//***************************************************************************

enum rds_callback_return_states
{
	RDS_ABORT_RECORDING,
	RDS_CONTINUE_RECORDING,
	RDS_STOP_RECORDING,		// Added in 2.3.7 to quit WAROPFEx, but not clear arm state.
};


//***************************************************************************
//*                    rds_record_activity ENUMERATOR
//***************************************************************************

enum rds_record_activity
{
	RDS_NO_ACTIVITY =						0x00000000,
	RDS_BUTTON_ONE_WENT_ON =				0x00000001,
	RDS_BUTTON_TWO_WENT_ON =				0x00000002,
	RDS_BUTTON_THREE_WENT_ON =				0x00000004,
	RDS_BUTTON_ONE_IS_ON =					0x00000008, 	// 4/19/00
	RDS_BUTTON_TWO_IS_ON =					0x00000010,		// 4/19/00
	RDS_BUTTON_THREE_IS_ON =				0x00000020,		// 4/19/00	
	RDS_EVENT_ONE_WENT_ACTIVE =				0x00000040,
	RDS_EVENT_TWO_WENT_ACTIVE =				0x00000080,
	RDS_EVENT_ONE_WENT_INACTIVE =			0x00000100,
	RDS_EVENT_TWO_WENT_INACTIVE =			0x00000200,
	RDS_POWER_PROBE_IS_ACTIVE =				0x00000400,
	RDS_LASER_ANALOG_PROBE_IS_ACTIVE =		0x00000800,
	RDS_LASER_TRIGGER_PROBE_WENT_ACTIVATE =	0x00001000,

	RDS_CLOCK_WENT_ACTIVE =              0x00002000,
	RDS_CLOCK_WENT_INACTIVE =            0x00004000,

	RDS_TUBE_PROBE_BEAM_ONE_BLOCKED_MOVING_FORWARD =	0x00010000,
	RDS_TUBE_PROBE_BEAM_TWO_BLOCKED_MOVING_FORWARD =	0x00020000,
	RDS_TUBE_PROBE_BEAM_ONE_UNBLOCKED_MOVING_FORWARD =	0x00040000,
	RDS_TUBE_PROBE_BEAM_TWO_UNBLOCKED_MOVING_FORWARD =	0x00080000,
	RDS_TUBE_PROBE_BEAM_ONE_BLOCKED_MOVING_BACKWARD =	0x00100000,
	RDS_TUBE_PROBE_BEAM_TWO_BLOCKED_MOVING_BACKWARD =	0x00200000,
	RDS_TUBE_PROBE_BEAM_ONE_UNBLOCKED_MOVING_BACKWARD =	0x00400000,
	RDS_TUBE_PROBE_BEAM_TWO_UNBLOCKED_MOVING_BACKWARD =	0x00800000,

	RDS_INSIDE_GUIDED_PLANE_RANGE =						0x01000000,
	RDS_INSIDE_GUIDED_CIRCLE_RANGE =					0x02000000,
	RDS_INSIDE_GUIDED_RING_RANGE =						0x04000000,

	RDS_RECORD_STOP =									0x10000000,	// WAROPFEx only
	RDS_RECORD_ERROR =									0x80000000,
	RDS_RECORDING_ABORTED =								0xC0000000,		// 4/27/00
};

//***************************************************************************
//*                     rds_record_functions ENUMERATOR
//***************************************************************************

enum rds_record_functionsEx
{
	RDS_TURN_OFF_FAST_SCAN =								0x00000001,
	RDS_SCANNER_KEEP_GOING =								0x00000002,
};
//***************************************************************************
//*                    rds_record_activityEx ENUMERATOR	//2.3.5
//***************************************************************************

enum rds_record_activityEx
{
	RDS_TRIGGER_READY =									0x00000001,	
};

//***************************************************************************
//*                      power_probe_states ENUMERATOR
//***************************************************************************

enum power_probe_states 
{ 
	POWER_PROBE_START, 
	POWER_PROBE_REPEAT,
	POWER_PROBE_RESET, 
	POWER_PROBE_READY, 
	POWER_PROBE_ACTIVE
};


//***************************************************************************
//*                          event_codes ENUMERATOR
//***************************************************************************

enum event_codes 
{ 
	NO_EVENT, 
	EVENT_ONE, 
	EVENT_TWO
};


//***************************************************************************
//*                      WINRDS_VERSION_INFO STRUCTURE
//***************************************************************************

typedef struct
{
	DWORD dwSize;
	DWORD dwMajor;
	DWORD dwMinor;
	DWORD dwRevision;
	DWORD dwBuild;
	DWORD dwRxxcDLLType; // R6SC_DLL, RDENSC_DLL
	DWORD dwRxxcMajor;
	DWORD dwRxxcMinor;
	DWORD dwRxxcRevision;
	DWORD dwRxxcBuild;
} WINRDS_VERSION_INFO;


//***************************************************************************
//*                          WINRDS_VERSION_INFOEX STRUCTURE
//***************************************************************************

typedef struct
{
	DWORD dwSize;
	DWORD dwMajor;
	DWORD dwMinor;
	DWORD dwRevision;
	DWORD dwBuild;
	DWORD dwRxxcDLLType; // R6SC_DLL, RDENSC_DLL
	DWORD dwRxxcMajor;
	DWORD dwRxxcMinor;
	DWORD dwRxxcRevision;
	DWORD dwRxxcBuild;
	DWORD dwXdatMajor;
	DWORD dwXdatMinor;
	DWORD dwXdatRevision;
	DWORD dwXdatBuild;
	char  build[8];
	BOOL  beta;
} WINRDS_VERSION_INFOEX;


//***************************************************************************
//*                          RDS_POSITION STRUCTURE
//***************************************************************************

typedef struct
{
	double dX, dY, dZ;		// probe origin
	double dXI, dXJ, dXK;	// probe x-axis vector
	double dYI, dYJ, dYK;	// probe y-axis vector
	double dZI, dZJ, dZK;	// probe z-axis vector
	double dDist;			// distance to guide origin
} RDS_POSITION;


//***************************************************************************
//*                           RDS_GUIDE STRUCTURE
//***************************************************************************

typedef struct
{
	double dX, dY, dZ;	// guide origin
	double dI, dJ, dK;	// guide vector
	double dDia, dWth;	// guide diameter and guide window width
} RDS_GUIDE;


//***************************************************************************
//*                       RDS_CALLBACK_DATA STRUCTURE
//***************************************************************************

#define MAX_PROBE_BUTTONS 3
#define MAX_ARM_AXIS      6		// Warning -- Changing this value will break existing code.

typedef struct
{
	DWORD dwWnd;						// active window
	WORD wButtons[MAX_PROBE_BUTTONS];	// three probe buttons
	DWORD dwStatusCode; 				// combo of buttons and events
	double dArmAngles[MAX_ARM_AXIS];	// arm angles ( 0 to 2 PI) 	// 4/19/00	
	RDS_POSITION stProbePosition;		// used to update probe to screen presentation
	LONG lSoundFrequency;				// new sound frequency
	LONG lSoundDuration;				// new sound duration
} RDS_CALLBACK_DATA;


//***************************************************************************
//*                        RDS_RECORD_DATA STRUCTURE
//***************************************************************************

typedef WORD ( CALLBACK *CALLBACK_PROBE_POSITION) // returns rds_callback_return_states	
(
	RDS_CALLBACK_DATA* pstCallbackData
);


typedef struct
{
	DWORD dwRecordFunction;	// rds_record_functions above
	RDS_GUIDE stProbeGuide;
	LONG lSoundFrequency;
	LONG lSoundDuration;
	CALLBACK_PROBE_POSITION pfnCallbackProbePosition;	// pointer to callback function
	DWORD dwWnd;	// active window passed to callback function
	WORD wButtons[MAX_PROBE_BUTTONS];	// three probe buttons
	RDS_POSITION stProbePosition;	// probe xyz, ijk, ijk, ijk, dimn 
} RDS_RECORD_DATA;


//***************************************************************************
//*                       RDS_CALLBACK_DATAEX STRUCTURE	//2.3.5
//***************************************************************************
#define MAX_ARM_AXISEX     8		
#define ROTARY_TABLE_AXIS  7		// index in dArmAngles of Rotary Table		

typedef struct
{
	DWORD dwWnd;						// active window
	WORD wButtons[MAX_PROBE_BUTTONS];	// three probe buttons
	DWORD dwStatusCode; 				// combo of buttons and events
	DWORD dwStatusCodeEx; 				// Extended status information
	double dArmAngles[MAX_ARM_AXISEX];	// arm angles ( 0 to 2 PI) 	// 4/19/00	
	RDS_POSITION stProbePosition;		// used to update probe to screen presentation
	LONG lSoundFrequency;				// new sound frequency
	LONG lSoundDuration;				// new sound duration
} RDS_CALLBACK_DATAEX;


//***************************************************************************
//*                        RDS_RECORD_DATAEX STRUCTURE //2.3.5
//***************************************************************************

#define INTERRUPT_TICK_VALUE	4.4	// Each InterruptTick equals 4.4 milliseconds

typedef WORD ( CALLBACK *CALLBACK_PROBE_POSITIONEX) // returns rds_callback_return_states	
(
	RDS_CALLBACK_DATAEX* pstCallbackDataEx
);


typedef struct
{
	DWORD dwRecordFunction;	// rds_record_functions above
	DWORD dwRecordFunctionEx;	// rds_record_functionsEx above // 2.3.5
	RDS_GUIDE stProbeGuide;
	LONG lSoundFrequency;
	LONG lSoundDuration;
	CALLBACK_PROBE_POSITIONEX pfnCallbackProbePositionEx;	// pointer to callback function
	DWORD dwWnd;	// active window passed to callback function
	WORD wButtons[MAX_PROBE_BUTTONS];	// three probe buttons
	RDS_POSITION stProbePosition;	// probe xyz, ijk, ijk, ijk, dimn 
	DWORD InterruptTick;	// Arm-relative time of last trigger. //2.3.5
	double dArmAngles[MAX_ARM_AXISEX];	// arm angles ( 0 to 2 PI) 	//2.3.5	
} RDS_RECORD_DATAEX;


//***************************************************************************
//*                        RDS_SCANNING STRUCTURE //7.0
//***************************************************************************

typedef struct
{
	DWORD dwRecordFunction;	// rds_record_functions above
	void* pVoid;	// optional pointer for callback use
	WORD wButtons[MAX_PROBE_BUTTONS];	// three probe buttons
	RDS_POSITION stProbePosition;	// probe xyz, ijk, ijk, ijk, dimn 
	RDS_POSITION stArmPosition;	// arm xyz, ijk, ijk, ijk, dimn 
	DWORD InterruptTick;	// Arm-relative time of last trigger. //2.3.5
	DWORD TriggerCount;	// Trigger count
	double dArmAngles[MAX_ARM_AXISEX];	// arm angles ( 0 to 2 PI) 	//2.3.5	
	DWORD error;
	BYTE	reserved[128];
} RDS_SCANNING;

typedef WORD ( CALLBACK *CALLBACK_SCANNING) // returns scanning data	
(
	RDS_SCANNING* pstScanning
);

enum RDS_SCAN_OPTIONS
{
	RDS_START_SCANNING = 1,
	RDS_STOP_SCANNING,

// OR in if falling edge trigger ie. RDS_START_SCANNING|RDS_TRIGGER_ON_FALLING
	RDS_TRIGGER_ON_FALLING = 0x20,
};

enum RDS_SCAN_ERRORS
{
	RDS_SCAN_DATA_ERROR = 1,


	RDS_A_AXIS_OVERLIMIT = 0x0103,
	RDS_B_AXIS_OVERLIMIT = 0x0203,
	RDS_C_AXIS_OVERLIMIT = 0x0303,
	RDS_D_AXIS_OVERLIMIT = 0x0403,
	RDS_E_AXIS_OVERLIMIT = 0x0503,
	RDS_F_AXIS_OVERLIMIT = 0x0603,
	RDS_G_AXIS_OVERLIMIT = 0x0703,
	RDS_H_AXIS_OVERLIMIT = 0x0803,
};

_RDS_API BOOL _RDS_CALL RDS_Scan( RDS_SCAN_OPTIONS option, CALLBACK_SCANNING callback, void* pVoid /*returned via callback*/);




//***************************************************************************
//*                        ARM_DETAILS STRUCTURE
//***************************************************************************

typedef struct
{
	unsigned char	cKernels[4];	
	unsigned char	cRunCodes[4];
	unsigned char	cUSBFirmware;
	unsigned char	reserveda;
	unsigned char	reservedb;
	unsigned char	reservedc;
	DWORD	reserved2;
	DWORD	reserved3;
} ARM_DETAILS;

//***************************************************************************
//*                        RDS_PROBE_DATA STRUCTURE
//***************************************************************************

typedef struct
{
	char cPROBE_NAME_ID[40];
	UINT iPROBE_TYPE_ID;
	double dPROBE_CODE_ID;		
	double dPROBE_DIAMETER_ID;
	double dBEAM_DIAMETER_ID;
	double dBALL_INNER_COMP_ID;
	double dBALL_OUTER_COMP_ID;
} RDS_PROBE_DATA;

enum probe_types {
	NO_PROBE =							0x00,
	BALL_PROBE =						0x01,
	DUAL_BEAM_PROBE =					0x02, // changed 09-05-01
	DUEL_BEAM_PROBE =					0x02,
	LASER_TRIGGER_PROBE =				0x04,
	LASER_ANALOG_PROBE =				0x08,
	LASER_SENSOR_PROBE =				0x10, // changed 09-05-01
	STEM_PROBE =						0x20,
	HOLE_PROBE =						0x40,
	CAMERA_PROBE =						0x80,
	POWER_PROBE =						0x100,
	CHECK_PROBE =						0x200,
	BALL_PROBE_CONTACT_ON =				0x400,
	BALL_PROBE_CONTACT_OFF =			0x800, 
	VIDEO_CAMERA_PROBE =				0x1000, 
	GAP_BLADE_PROBE =					0x2000, // new 05-03-02
};


typedef struct
{
	RDS_PROBE_DATA rProbeData;
	double dPROBE_PTMX_ID;
	double dPROBE_PTMY_ID;
	double dPROBE_EDGE_ID;
	double dBEAM_B1RX_ID;
	double dBEAM_B2RX_ID;
	char cPROBE_ID[17];
	double dProbe_LENGTH_ID;
	char RESERVED[64];
} RDS_PROBE_DATAEX;





//***************************************************************************
//*                                 RDS API
//***************************************************************************

// Notation:
//
// If you are new to WinRDS, ignore the following notation. If you used 
// CRDServer before and you are converting your existing code, the
// notation that gives you mapping from CRDServer to new WinRDS functions
// will help you the transition process.
//
// S - Same function as in CRDServer
// N - New function in 2.3.1
// C - Same function as in CRDServer with some change
// O - Obsolete function
// V - Function derived from a previous public member variable in CRDServer
// N2.3.3 - New function in 2.3.3 / 2.3.4
// N2.3.5 - New function in 2.3.5

// ----------------------------- General ------------------------------------

/* C */ _RDS_API void _RDS_CALL RDS_GetVersion(WINRDS_VERSION_INFO* pVersionInfo);
/* N2.3.3 */	_RDS_API void _RDS_CALL RDS_GetVersionEx(WINRDS_VERSION_INFOEX* pVersionInfoEx);
/* N */ _RDS_API void _RDS_CALL RDS_GetErrorMessage( LPTSTR pszErrMsgBuf, DWORD dwErrMsgBuf );

// ----------------------------- Settings -----------------------------------

/* V */ _RDS_API void _RDS_CALL RDS_GetRDSDirectory( LPTSTR pszPathBuffer, DWORD dwPathBufferSize );
/* V */ _RDS_API void _RDS_CALL RDS_SetRDSDirectory( LPCTSTR pszPathBuffer );

/* N3.0 */
#define	USB_COMM_PORT	9
#define	WIFI_COMM_PORT	10

/* V */ _RDS_API void _RDS_CALL RDS_SetCommPort( int iCommPort );
/* V */ _RDS_API int _RDS_CALL RDS_GetCommPort();

/* C */ _RDS_API BOOL _RDS_CALL RDS_SetVolumeLevel( int iVolumeLevel );
/* V */ _RDS_API int _RDS_CALL RDS_GetVolumeLevel();

/* V */ _RDS_API BOOL _RDS_CALL RDS_SetRMouseLButton( int iRMouseLButton );
/* V */ _RDS_API int _RDS_CALL RDS_GetRMouseLButton();

/* V */ _RDS_API BOOL _RDS_CALL RDS_SetRMouseMButton( int iRMouseMButton );
/* V */ _RDS_API int _RDS_CALL RDS_GetRMouseMButton();

/* V */ _RDS_API BOOL _RDS_CALL RDS_SetRMouseRButton( int iRMouseRButton );
/* V */ _RDS_API int _RDS_CALL RDS_GetRMouseRButton();

/* V */ _RDS_API BOOL _RDS_CALL RDS_SetRecordButton( int iRecordButton );
/* V */ _RDS_API int _RDS_CALL RDS_GetRecordButton();

/* V */ _RDS_API void _RDS_CALL RDS_SetProbeAutoDetect( BOOL bAutoDetect );
/* V */ _RDS_API BOOL _RDS_CALL RDS_IsProbeAutoDetect();

/* V */ _RDS_API void _RDS_CALL RDS_SetProbeNumber( int iProbeNumber );
/* V */ _RDS_API int _RDS_CALL RDS_GetProbeNumber();
/* N2.3.3 */	_RDS_API BOOL _RDS_CALL RDS_GetProbeData(RDS_PROBE_DATA* pProbeData);
/* N5.2 */	_RDS_API BOOL _RDS_CALL RDS_GetProbeDataEx(RDS_PROBE_DATAEX* pProbeDataEx);

/* S */ _RDS_API BOOL _RDS_CALL RDS_RemoteMouseFunction( BOOL bEnableRemoteMouse );
/* N */ _RDS_API BOOL _RDS_CALL RDS_IsRemoteMouseEnabled();

/* S */ _RDS_API BOOL _RDS_CALL RDS_BoostRemoteMouseThreadPriority( BOOL bAboveNormal );
/* N */ _RDS_API BOOL _RDS_CALL RDS_IsRemoteMouseThreadPriorityBoosted();

/* S */ _RDS_API BOOL _RDS_CALL RDS_SetLinearAxis( BOOL bEnableLinearAxis );
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsLinearAxisEnabled();

/* N2.3.5 */ _RDS_API BOOL _RDS_CALL RDS_SetRotaryAxis( BOOL bEnableRotaryAxis );
/* N2.3.5 */ _RDS_API BOOL _RDS_CALL RDS_IsRotaryAxisEnabled();

/* S */ _RDS_API BOOL _RDS_CALL RDS_SetStaticXrefNumber( int iStaticXrefNumber );
/* S */ _RDS_API int _RDS_CALL RDS_GetStaticXrefNumber();

/* S */ _RDS_API BOOL _RDS_CALL RDS_ArchiveProfileSettings( BOOL bIsStoring );

/* N */ _RDS_API void _RDS_CALL RDS_SetLGPSCommPort( int iCommPort );
/* N */ _RDS_API int _RDS_CALL RDS_GetLGPSCommPort();

/* N */ _RDS_API BOOL _RDS_CALL RDS_SetLGPSVolumeLevel( int iVolumeLevel );
/* N */ _RDS_API int _RDS_CALL RDS_GetLGPSVolumeLevel();

/* N2.3.3 */_RDS_API double _RDS_CALL RDS_GetScalingFactor();
/* N2.3.3 */_RDS_API void _RDS_CALL RDS_SetScalingFactor( double dScale, BOOL bDisableUI );


// -------------------------- Arm Attributess --------------------------------

/* S */ _RDS_API int _RDS_CALL RDS_GetButtonCount();
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsLinearAxisAvailable();
/* N2.3.5 */ _RDS_API BOOL _RDS_CALL RDS_IsRotaryAxisAvailable();
/* S */ _RDS_API BOOL _RDS_CALL RDS_AreStaticXrefsAvailable();
/* N2.3.7 */ _RDS_API BOOL _RDS_CALL RDS_GetArmDetails(ARM_DETAILS* pDetails);

// -------------------------- Arm Operations --------------------------------

/* S */ _RDS_API BOOL _RDS_CALL RDS_InitializeAndConnect(BOOL bBlockRemoteMouse);
/* S */ _RDS_API BOOL _RDS_CALL RDS_Disconnect();
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsConnectionEstablished();

/* S */ _RDS_API BOOL _RDS_CALL RDS_ResetArm( BOOL bForceReset, BOOL bDisplayCompletionMsg, HWND hParentWnd );
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsResetDone();

/* S */ _RDS_API BOOL _RDS_CALL RDS_ValidateCurrentProbe();
/* S */ _RDS_API double _RDS_CALL RDS_GetBallProbeDiameter();

/* S */ _RDS_API BOOL _RDS_CALL RDS_GetSystemUpdate( int iUpdateFlag );
/* V */ _RDS_API void _RDS_CALL RDS_GetCurrentPosition( RDS_POSITION* pPosition );
/* V */ _RDS_API void _RDS_CALL RDS_GetEvent1Position( RDS_POSITION* pPosition );
/* V */ _RDS_API void _RDS_CALL RDS_GetEvent2Position( RDS_POSITION* pPosition );
/* S */ _RDS_API double _RDS_CALL RDS_GetAxisAngles( int iAxis );
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsButtonNDown( int iButton );
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsRMouseLButtonDown();
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsRMouseMButtonDown();
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsRMouseRButtonDown();
/* S */ _RDS_API BOOL _RDS_CALL RDS_IsRecordButtonDown();
/* N */ _RDS_API BOOL _RDS_CALL RDS_IsBeamNOn( int iBeam );

/* S */ _RDS_API BOOL _RDS_CALL RDS_EnableSpecialRecordMode( WORD wEventCode );
/* S */ _RDS_API BOOL _RDS_CALL RDS_SpecialRecordAndReenable( WORD wEventCode );
/* S */ _RDS_API BOOL _RDS_CALL RDS_EnableRecord( WORD wEventCode );
/* S */ _RDS_API WORD _RDS_CALL RDS_GetActiveEvents();
/* S */ _RDS_API WORD _RDS_CALL RDS_GetRecordedEvents();
/* S */ _RDS_API BOOL _RDS_CALL RDS_EnableToggleRecord(WORD wEventCode);
/* S */ _RDS_API WORD _RDS_CALL RDS_GetRecordedToggleEvents();
/* S */ _RDS_API WORD _RDS_CALL RDS_GetRecordedEdges();
/* S */ _RDS_API BOOL _RDS_CALL RDS_StopRecord();

/* S */ _RDS_API BOOL _RDS_CALL RDS_OutputSpeakerSound( int iFrequency, int iDuration );

/* S */ _RDS_API WORD _RDS_CALL RDS_GetInterruptTick();
/* N2.3.7 */_RDS_API DWORD _RDS_CALL RDS_GetArmElapsedTime();

/* N */ _RDS_API WORD _RDS_CALL RDS_ArmLimitStatus( );

/* N */ _RDS_API DWORD _RDS_CALL RDS_WaitAndRecordOnProbeFunction( RDS_RECORD_DATA* pstRecordData );
/* N2.3.5 */ _RDS_API DWORD _RDS_CALL RDS_WaitAndRecordOnProbeFunctionEx( RDS_RECORD_DATAEX* pstRecordDataEx );

// -------------------------------- UI --------------------------------------

/* S */ _RDS_API BOOL _RDS_CALL RDS_EditProfileSettings( HWND hParentWnd, UINT uSelectPage );
/* N */ _RDS_API BOOL _RDS_CALL RDS_ArmCheckoutPoint( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_ArmCheckoutLength( HWND hParentWnd );
/* N2.3.5 */ _RDS_API BOOL _RDS_CALL RDS_ProbeCheckout( HWND hParentWnd );
/* N */ _RDS_API void _RDS_CALL RDS_MonitorArm( HWND hParentWnd );
/* N */ _RDS_API void _RDS_CALL RDS_VerifyProbeRecording( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_PointProbeCalibration( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_BallProbeCalibration( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_PowerProbeCalibration( HWND hParentWnd );
/* N2.3.3 */_RDS_API BOOL _RDS_CALL RDS_TubeProbeCalibration( HWND hParentWnd );
/* N2.3.3 */_RDS_API BOOL _RDS_CALL RDS_CalBarProcedure( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_PowerProbeBiasCalibration( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_LeapFrog( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_GridLock( HWND hParentWnd );
/* N */ _RDS_API void _RDS_CALL RDS_LGPS( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_DownloadFirmware( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_ArmDiagnosis( HWND hParentWnd );

/* N */ _RDS_API void _RDS_CALL RDS_ArmCalibrations( HWND hParentWnd  );
/* N */ _RDS_API BOOL _RDS_CALL RDS_EncoderCalibrations( HWND hParentWnd  );

/* N */ _RDS_API void _RDS_CALL RDS_ArmAccuracy( HWND hParentWnd  );

/* N */	_RDS_API void _RDS_CALL RDS_ArmLimitSettings( HWND hParentWnd );
/* N */	_RDS_API void _RDS_CALL RDS_EncoderOffsetSettings( HWND hParentWnd )  ;
/* N */ _RDS_API BOOL _RDS_CALL RDS_WristDataRecording( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_StaticDataRecording( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_ShelbowRecording( HWND hParentWnd );
/* N */ _RDS_API BOOL _RDS_CALL RDS_TuneUpArm( HWND hParentWnd );
/* N2.3.3 */_RDS_API BOOL _RDS_CALL RDS_LinearAxisCalibration( HWND hParentWnd );
/* N2.3.5 */_RDS_API BOOL _RDS_CALL RDS_RotaryAxisCalibration( HWND hParentWnd );
														
/* C */ _RDS_API void _RDS_CALL RDS_SpecialFunctionsDialog( HWND hParentWnd );

_RDS_API BOOL _RDS_CALL RDS_SetCommunicationTimeout(double hertz);
/* N2.3.6 */_RDS_API BOOL _RDS_CALL RDS_EnterUnlockCode(BOOL bTestForEnable);

#define WM_ARM_LEFT_BUTTON_DOWN WM_APP + 10
#define WM_ARM_LEFT_BUTTON_UP WM_APP + 11
#define WM_ARM_MID_BUTTON_DOWN WM_APP + 12
#define WM_ARM_MID_BUTTON_UP WM_APP + 13
#define WM_ARM_RIGHT_BUTTON_DOWN WM_APP + 14
#define WM_ARM_RIGHT_BUTTON_UP WM_APP + 15
#define WM_ARM_LEFT_BUTTON_DBCLK WM_APP + 16
#define WM_ARM_MID_BUTTON_DBCLK WM_APP + 17
#define WM_ARM_RIGHT_BUTTON_DBCLK WM_APP + 18

/* N2.3.7 */_RDS_API void _RDS_CALL RDS_GetArmButtonsOnly( HWND hParentWnd );

/* N3.1 */
_RDS_API void _RDS_CALL RDS_GetLOKXRef( double dXRef[4][4], BOOL bInvert);
_RDS_API void _RDS_CALL RDS_GetCurrentArmPosition( RDS_POSITION* pPosition );
_RDS_API WORD _RDS_CALL RDS_GetSIDBits( void );

/* N4.0 */
_RDS_API BOOL _RDS_CALL RDS_SaveArmspecs(void);
_RDS_API BOOL _RDS_CALL RDS_LoadArmspecs(void);

/* N4.2 */
_RDS_API void _RDS_CALL RDS_SetHeadLights(BOOL bOn);
_RDS_API BOOL _RDS_CALL RDS_GetHeadLights(void);

_RDS_API BOOL _RDS_CALL RDS_GetArmSerialNumber( LPTSTR pszMsgBuf, DWORD dwMsgBuf );


#endif

//================================== EOF ====================================
