// RomerArm.cpp: implementation of the RomerArm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "nrkapp.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "RomerArm.h"
#include "DSerialPortSettings.h"
#include "DPipeProbeMeas.h"
#include "CimCoreWindowsMessenger.h"

// Perceptron ScanWorks tcp/ip interface...
#include "ScanWorks_tcpip/tools.h"
#include "ScanWorks_tcpip/tcpip.h"
#include "DExternalScannerMeas.h"

// Perceptron ToolKit...
//#include "Acq/AcqSyncGroup.h"
//#include "Acq/AcqContinuousMode.h"
//#include "Acq/AcqBrightestCGFinder.h"
//#include "Acq/AcqSimulatedTriCamSensor.h"
//#include "Acq/AcqActiveTriCamSensor.h"
//#include "Acq/AcqHardwareConfig.h"
//#include "Acq/AcqThresholdFrameCGFinder.h"
//#include "Feat/FeatPatch.h"
//#include "Util/ETKException.h"
//#include "Scan/ScanFrameTimeData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//RDS_RECORD_DATA g_RDSRecordData;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CriticalSectionWrapper* csw;
SAIPScanStripeCloud* ssPkt;

RomerArm::RomerArm()
: wm(*new CimCoreWindowsMessenger(this))
, scannerCS(*new CriticalSectionWrapper)
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;
	initialOneShot = true;

	app.saarm.CMMString = _T("CimCore");
	app.saarm.SettingsFileName = _T("SA_Romer_ArmSettings.bin");
	LogFileName = _T("Romer.log");

	Vector rot(90.0, -180.0, 270.0);
	scannerXform.SetRotation(rot);
	scannerRegKeySet = false;

	m_hThreadOne = NULL;
	csw = &scannerCS;
	ssPkt = &scanStripePkt;
}

RomerArm::~RomerArm()
{
	CleanUpPerceptron();
	delete &scannerCS;
	delete &wm;
}
bool RomerArm::PopCommSettingsDlg()
{
	DSerialPortSettings dlg(app.saarm.arm_Romer);
	dlg.m_CommPort = app.saarm.Data.CommPort-1;
	if(DOMODAL(dlg) != IDOK)
		return false;
	app.saarm.Data.CommPort = dlg.m_CommPort+1;
	return true;
}
#define BASEADD 0x25145000
bool RomerArm::ConnectToArm(const int dof)
{
	if(!app.saarm.Data.doconnection)
		return true;	// simulation mode

	if(!app.saarm.AutoSocket.Valid)
		if(!PopCommSettingsDlg())
			return false;
	// 9 usb, 10 wi-fi

	DOF = dof;

	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);
	JShift.Create(dof,TRUE);

	//// as of ScanWorks v. 5, we no longer set the DelayConnect reg key, Perceptron now provides a 'moverconnect' and 'moverdisconnet' methods for more absolute control of the ScanWorks connection
	//scannerRegKeySet = SetScannerRegKey();	// this sets ScanWorks behavior per our requested Perceptron code mods - it now 'shares' the WinRDS connection nicely!

	//if (!TCPIPInitialize())	// 2005.10.12 - initiate use of WS2_32.DLL for ScanWorks connection - only once
	//{
	//	TRACE0("TCPIPInitialize failed.\n");
	//	//AfxMessageBox(_T("Could not initialize tcp\ip connection for ScanWorks.  Scanner will not be able to run."), MB_ICONERROR);
	//}

	RDS_SetCommPort(app.saarm.Data.CommPort);
	app.saarm.TTS.TextToSpeech(_T("Initializing Romer Arm, Please stand by."));
	if(!RDS_InitializeAndConnect(FALSE))
	{
		// 2003.10.23 - this is too hard and not complete...
		//CString Msg;
		//int err = GetLastError();
		//Msg.Format("Error Initializing Arm \n\tError = 0x%X \n\t\tCheck Comm Port Specified",err);
		//if((err >= BASEADD+0x0000)&&(err<BASEADD+0x0800))
		//	Msg.Format("Controller or Comm Error - \n\tCheck Comm Port ID \n\t\tError = 0x%X",err);
		//if((err >= BASEADD+0x0800)&&(err<BASEADD+0x0900))
		//	Msg.Format("WinRDS Error - \n\tIs WinRDS Installed and in the Machine Path?  \n\t\tError = 0x%X",err);
		//if(err >= BASEADD+0x0900 )
		//	Msg.Format("Security Error - \n\tCheck WinRDS Hardware Lock   \n\t\tError = 0x%X",err);
		//int ans = AfxMessageBox(Msg + "\nRun in Simulation mode?",MB_YESNO);

		// 2003.10.23 - ...this is easier
		char errmsg[MAX_PATH];
		RDS_GetErrorMessage((LPTSTR)errmsg, MAX_PATH);
		CString Msg;
		Msg.Format(Tr("WinRDS Error - \n\n%s"),CA2T(errmsg));
		int ans = AfxMessageBox(Msg + Tr("\n\nRun in Simulation mode?"),MB_YESNO);

		if(ans==IDNO)
		{
			
//			Span.CloseSocketAndFile();
//			SetTimer(INITIAL,1000,NULL);		// loop back into conenction dialog.
			PostQuitMessage(0);	// 001.03.23
			return false;
		}
		else
			return true;	// simulation mode
	}
	// 2004.09.02 - now the main dlg does not exist when we connect to the arm...
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//RDS_ResetArm(FALSE,FALSE,d.m_hWnd);
	RDS_ResetArm(FALSE,FALSE,NULL);
	if(app.saarm.Data.AutoDetectProbe)
	{
		RDS_SetProbeAutoDetect(TRUE);
		probeValid = RDS_ValidateCurrentProbe() ? true : false;
	}
	else
		RDS_SetProbeAutoDetect(FALSE);

	app.saarm.TTS.TextToSpeech(_T("Start up Successful."));
	ConnectedToArm = TRUE;
	TRACE0("Romer connected\n");
//	SetTimer(QUERY2,2000,NULL);

	RemoteMouse(FALSE);	// 2003.02.11 - start with it off

	if(!wm.windowValid)	// 2005.09.27 - added for scanner dlg
	{
		if(!wm.MakeValidWindow())
		{
			CString m;
			m.Format(Tr("Critical Error.\nCould not start CimCore Message Window.\n\nPlease try again."));
			AfxMessageBox(m, MB_ICONHAND);
			return false;
		}
	}

	char sn[MAX_PATH];
	if(RDS_GetArmSerialNumber((LPTSTR)sn, MAX_PATH))	// this command added with WinRDS v.7.1.4
		CMMSerialNum.Format(_T("%s"), CA2T(sn));
	
	return true;
}
bool RomerArm::ReConnectToArm()
{
	RDS_SetCommPort(app.saarm.Data.CommPort);
	app.saarm.TTS.TextToSpeech(_T("Initializing Romer Arm, Please stand by."));
	Delay(1.0,TRUE);
	Sleep(500);
	if(!RDS_InitializeAndConnect(FALSE))
	{
		char errmsg[MAX_PATH];
		RDS_GetErrorMessage((LPTSTR)errmsg, MAX_PATH);
		CString Msg;
		Msg.Format(Tr("WinRDS Error - \n\n%s"),CA2T(errmsg));
		int ans = AfxMessageBox(Msg + Tr("\n\nRun in Simulation mode?"),MB_YESNO);

		if(ans==IDNO)
		{
			PostQuitMessage(0);	// 001.03.23
			return false;
		}
		else
			return true;	// simulation mode
	}
	RDS_ResetArm(FALSE,FALSE,NULL);
	//if(app.saarm.Data.AutoDetectProbe)
	//{
	//	RDS_SetProbeAutoDetect(TRUE);
	//	probeValid = RDS_ValidateCurrentProbe() ? true : false;
	//}
	//else
	//	RDS_SetProbeAutoDetect(FALSE);

	app.saarm.TTS.TextToSpeech(_T("Start up Successful."));
	RDS_OutputSpeakerSound(1400,120);
	ConnectedToArm = TRUE;
	TRACE0("Romer connected\n");
	//	SetTimer(QUERY2,2000,NULL);

	RemoteMouse(FALSE);	// 2003.02.11 - start with it off

	return true;
}
//////////////////////////////////////////////////////////////////////////
// RDS callback...
static WORD CALLBACK RecordDataCallback(RDS_CALLBACK_DATAEX *pstCallbackData)
{
	CDPCMMDlg *window = (CDPCMMDlg *)pstCallbackData->dwWnd;

	if(pstCallbackData->dwStatusCode&RDS_RECORD_CALLBACK_ERROR)
		return RDS_ABORT_RECORDING;

	//	app.pA->X = pstCallbackData->stProbePosition.dX/1000.0;
	//	app.pA->Y = pstCallbackData->stProbePosition.dY/1000.0;
	//	app.pA->Z = pstCallbackData->stProbePosition.dZ/1000.0;

	//	app.pA->II = pstCallbackData->stProbePosition.dZI;
	//	app.pA->JJ = pstCallbackData->stProbePosition.dZJ;
	//	app.pA->KK = -pstCallbackData->stProbePosition.dZK;

	//Vector pos(pstCallbackData->stProbePosition.dX/1000.0, 
	//	pstCallbackData->stProbePosition.dX/1000.0,
	//	pstCallbackData->stProbePosition.dZ/1000.0);
	//TRACE0("WAROPF pos (xyz):\n");
	//pos.Debug();

	//TRACE1("pstCallbackData->dwStatusCode = %d\n", pstCallbackData->dwStatusCode);

	window->jtlimit.SetWindowText(_T(""));
	if (RDS_ArmLimitStatus() != 0)
	{
		pstCallbackData->lSoundFrequency = 3000;
		pstCallbackData->lSoundDuration = 100;
		window->jtlimit.SetWindowText(Tr("AJoint Limit Exceeded!"));
		app.pA->jtlimitbad = true;
		return RDS_CONTINUE_RECORDING;
	}
	window->jtlimit.SetWindowText(_T(""));
	app.pA->jtlimitbad = false;

	if ( (pstCallbackData->dwStatusCode &  RDS_POWER_PROBE_IS_ACTIVE) > 0)
	{
		TRACE0("Got A ProbeActive!\n");
	}
	//if ( pstCallbackData->wButtons[0] )
	//{
	//	TRACE0("Button 1 Pressed\n");
	//	TRACE1("pstCallbackData->dwStatusCode = %d\n", pstCallbackData->dwStatusCode);
	//}
	//if ( pstCallbackData->wButtons[1] )
	//{
	//	TRACE0("Button 2 Pressed\n");
	//	TRACE1("pstCallbackData->dwStatusCode = %d\n", pstCallbackData->dwStatusCode);
	//}
	//if ( pstCallbackData->wButtons[2] )
	//{
	//	TRACE0("Button 3 Pressed - KILL waropf THREAD and stop scanner\n");
	//	app.pA->StopScannerMeas();
	//}

	//MSG msg;
	//// We have to do some window messages stuff
	//  while ( ::PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) 
	//{
	//    ::TranslateMessage( &msg );
	//	  	::DispatchMessage( &msg );
	// 	}

	return RDS_CONTINUE_RECORDING;
}
////////////////////////////////////////////////////////////////////////////////////
BOOL RomerArm::DisconnectFromHardware()
{
	BOOL ret = FALSE;
	if(RDS_IsConnectionEstablished())
	{
		Sleep(500);

		while (RDS_IsButtonNDown(2) || RDS_IsButtonNDown(1) || RDS_IsButtonNDown(3))
		{
			RDS_GetSystemUpdate(TRUE);
			Sleep(20);
		}

		RDS_RECORD_DATAEX data2;
		memset (&data2, 0, sizeof(data2));
		data2.dwRecordFunction              = RDS_RECORD_NOW;
		data2.dwRecordFunctionEx            = RDS_TURN_OFF_FAST_SCAN;
		data2.lSoundFrequency               = -1;
		data2.lSoundDuration                = -1;
		data2.pfnCallbackProbePositionEx    = NULL; //RecordDataCallback;

		RDS_WaitAndRecordOnProbeFunctionEx(&data2);

		if (RDS_Disconnect())
		{
			while (RDS_IsConnectionEstablished())
			{
				Sleep(20);
				continue;
			}
			ret = TRUE;
		}
	}

	if ( m_hThreadOne != NULL)		// if Thread already running
		KillThreadOne();

	return ret;
}
/*
int RomerArm::ProcessPendingRead()
{	
	// bug("processing a read!\n");
	int BytesRead = Span.ReceivePacket();
	if (BytesRead == 0)
		return 0;
	// precess the command here!
	if (Span.Packet.Command == PAK_TEXT)
	{
		CString t(Span.Packet.app.saarm.Data.Raw);;	// transfer to dialog.
		AfxMessageBox(t);
	}

	else if (Span.Packet.Command == PAK_WORKING_FRAME)
	{
		TRACE(_T("Received Working Frame Packet!!\n"));
		SockGeom.ProcessBaseAndWorkingPacket();
		Transform WorldToWork;
		SockGeom.WorkingToWorld.Invert(&WorldToWork);
		ArmToWorking.Multiply(WorldToWork,SockGeom.BaseToWorld);
//		ArmToWorking.Debug();
	}
	else if (Span.Packet.Command == PAK_MEAS_GROUP)
	{
		CString PGroup,PTarget, Gs, Ts;
		Vector Point;
		BYTE first=0;
		BYTE last=0;
		SockGeom.ProcessBatchMeasurePacket(&PGroup,&PTarget,&Gs,&Ts,&Point,&first,&last);
		if(first)
		{
			//clear the array then add the point
			ClearGroupMeas();
		}
		BatchMeasItem* pBMI = new BatchMeasItem;
		pBMI->Group = PGroup;
		pBMI->Target = PTarget;
		pBMI->GSuffix = Gs;
		pBMI->TSuffix = Ts;
		pBMI->Coord = Point;
		GroupMeas.Add(pBMI);
		if(last)
			m_GuideButton.EnableWindow(TRUE);
		
	}

	else if (Span.Packet.Command == PAK_FLAGS)
	{
		if (Span.Packet.app.saarm.Data.Flags.Bit[0] == 1)
			TRACE("Was Query Inst"); //currently no queryinst for romer
		//rjs QueryInst();   
	}
	else if (Span.Packet.Command == PAK_MEASUREMENT)//automeasure
	{
		TRACE("recvd meas packet.\n");
		double XDrive,YDrive,ZDrive;
		XDrive = Span.Packet.app.saarm.Data.Meas.Values[0];  //get the angles from the packet
		YDrive = Span.Packet.app.saarm.Data.Meas.Values[1];
		ZDrive = Span.Packet.app.saarm.Data.Meas.Values[2];

	
		Vector Point(XDrive,YDrive,ZDrive);
		//wipe out any old guides
		ClearGroupMeas();
		m_GuideButton.EnableWindow(FALSE);
		
		BatchMeasItem* pBMI = new BatchMeasItem;
		pBMI->Group = Span.Packet.app.saarm.Data.Meas.ContainerName;
		pBMI->Target = Span.Packet.app.saarm.Data.Meas.TargetName;;
		pBMI->GSuffix = "";
		pBMI->TSuffix = "";
		pBMI->Coord = Point;
		GroupMeas.Add(pBMI);

		DGuideMeas diag;
		diag.pRomerDlg = this;
		if(diag.DoModal()==IDCANCEL)
			SendFailedMeasFlag();
		ClearGroupMeas();
	}
	else if (Span.Packet.Command == PAK_MAKE_CIRCLE)
	{
		if(!((LastInspect==HOLEINSPECT)||(LastInspect==PININSPECT)))
			return 0;
		TRACE("CIRCLE PACKET\n");
		Transform NomTrans;
		NomTrans.SetTranslation(Vector(Span.Packet.app.saarm.Data.MakeCircle.CirclePos.x,	
						Span.Packet.app.saarm.Data.MakeCircle.CirclePos.y,
						Span.Packet.app.saarm.Data.MakeCircle.CirclePos.z));
		NomTrans.SetRotation(Vector(Span.Packet.app.saarm.Data.MakeCircle.CircleRot.x,
						Span.Packet.app.saarm.Data.MakeCircle.CircleRot.y,
						Span.Packet.app.saarm.Data.MakeCircle.CircleRot.z));
		double NomRad = Span.Packet.app.saarm.Data.MakeCircle.Radius;
	
		if(LastInspect == HOLEINSPECT)
		{
			DHoleMeas diag;
			diag.pRomerDlg = this;
			diag.Pin = FALSE;
			diag.Inspect = TRUE;
			diag.NominalTrans = NomTrans;
			diag.NominalRadius = NomRad;
			diag.DoModal();
		}
		else if(LastInspect == PININSPECT)
		{
			DHoleMeas diag;
			diag.pRomerDlg = this;
			diag.Pin = TRUE;
			diag.Inspect = TRUE;
			diag.NominalTrans = NomTrans;
			diag.NominalRadius = NomRad;
			diag.DoModal();
		}
		
	}
	else if (Span.Packet.Command == PAK_MAKE_PLANE)
	{
		if(LastInspect!=PLANEINSPECT)
			return 0;
		TRACE("PLANE PACKET\n");
		Transform NomTrans;
		NomTrans.SetTranslation(Vector(Span.Packet.app.saarm.Data.MakePlane.PlanePos.x,	
						Span.Packet.app.saarm.Data.MakePlane.PlanePos.y,
						Span.Packet.app.saarm.Data.MakePlane.PlanePos.z));
		NomTrans.SetRotation(Vector(Span.Packet.app.saarm.Data.MakePlane.PlaneRot.x,
						Span.Packet.app.saarm.Data.MakePlane.PlaneRot.y,
						Span.Packet.app.saarm.Data.MakePlane.PlaneRot.z));

		DPlaneMeas diag;
		diag.pRomerDlg = this;
		diag.Inspect = TRUE;
		diag.NominalTrans = NomTrans ;
		diag.DoModal();
		
	}
	TRACE("Ending ProcessPend\n");
	
	receivedSignOn = TRUE;
	return BytesRead;
}
*/
BOOL RomerArm::GetAllData()
{
	if( !ConnectedToArm )
		return ArmBase::GetAllData();

	RDS_GetSystemUpdate(TRUE);  //read all values

	GetPos();
	GetJoints();
	if(!overrideArmButtons && app.pA->armButtons.ButtonsEnabled())
		GetButtonStates();
	GetTemp();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//TRACE("Checking ArmLimit Status:\n");
	if(RDS_ArmLimitStatus() != 0)
	{
		//TRACE("RDA_ArmLimitStatus() != 0, i.e. BAD\n");
		d.jtlimit.SetWindowText(Tr("Joint Limit Exceeded!"));
		if(!jtlimitbad)
			ArmBeep(JT_LIMIT);
		jtlimitbad = true;
		return FALSE;
	}
	//TRACE("Jt Limit Status OK\n");
	d.jtlimit.SetWindowText(_T(""));
	jtlimitbad = false;

	return TRUE;
}
BOOL RomerArm::GetPos()
{
	if( !ConnectedToArm )
		return ArmBase::GetPos();

	CriticalSectionLock lock( scannerCS );
	//transform into consistent coords
	//and scale to meters
	RDS_POSITION Pos;
	RDS_GetCurrentPosition(&Pos);

	if(Pos.dX==0.0 && Pos.dY==0.0 && Pos.dZ==0.0)
	{
		//TRACE3("Raw Data: X=%f,Y=%f,Z=%f\n",Pos.dX,Pos.dY,Pos.dZ);
		return false;
	}

	X = (Pos.dX/1000.0);
	Y = (Pos.dY/1000.0);
	Z = (Pos.dZ/1000.0);

	//TRACE3("Raw Data: X=%f,Y=%f,Z=%f\n",X,Y,Z);

	// 2003.10.24 - have a 1028 from Romer for testing, bet we can get rid of NativeArmToSA xform
	// 2003.10.29 - not according to Rich Steckel's testing with his (old) 3000 series arm!
	Vector RomerCoord( X,Y,Z ); //raw data
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	NativeArmToSA.Multiply( RomerCoord, &AnalyzerCoord );
	X = AnalyzerCoord.x;
	Y = AnalyzerCoord.y;
	Z = AnalyzerCoord.z;

//	TRACE3("3000 Data: X=%f,Y=%f,Z=%f\n",X,Y,Z);

	// 2007.07.09 - no longer use this as the probe tip vector, see below...
//	II = -Pos.dZI;
//	JJ = -Pos.dZJ;
	//II = Pos.dZI;	// 2001.02.14 3000 only?
	//JJ = Pos.dZJ;	// 2001.02.14 3000 only?
	//KK = -Pos.dZK;

	// 2001.04.20 begin
	if(app.saarm.InstrumentModel==13||app.saarm.InstrumentModel==14) // 2500,2200
		CurrentTransform.SetRotation(Vector(Pos.dYI,Pos.dYJ,Pos.dYK),Vector(Pos.dXI,Pos.dXJ,Pos.dXK),Vector(-Pos.dZI,-Pos.dZJ,-Pos.dZK));
	// 2001.04.20 end
	// 2001.02.14 3000 only?
	else
		CurrentTransform.SetRotation(Vector(-Pos.dYI,-Pos.dYJ,Pos.dYK),Vector(-Pos.dXI,-Pos.dXJ,Pos.dXK),Vector(Pos.dZI,Pos.dZJ,-Pos.dZK));
	CurrentTransform.SetTranslation(Vector(X,Y,Z));

	// 2007.07.09 - get the normal from the transform...
	Vector zz = CurrentTransform.GetZAxis();
	zz.Normalize();
	II = zz.x;
	JJ = zz.y;
	KK = zz.z;
	
	return TRUE;
}
BOOL RomerArm::GetJoints(bool simulate/*=true*/)
{
	if( !ConnectedToArm )
		return ArmBase::GetJoints(simulate);

	if( !app.saarm.receivedSignOn )	// assigns size of J GenVec per arm type
		return FALSE;

	if(initialOneShot)
	{
		initialOneShot = false;
		// set NativeArmToSA transform
		Vector rot(0.0,0.0,180.0);
		// 2001.04.19 begin

		// 2003.10.24 - added 1000 series, have a 1028 from Romer for testing, bet we can do
		// this for ALL - only ones left are 3000's - if so, we could just get rid of NativeArmToSA xform
		// 2003.10.29 - not according to Rich Steckel's testing with his (old) 3000 series arm!
		if(app.saarm.InstrumentModel==11 || app.saarm.InstrumentModel==12 || app.saarm.InstrumentModel==22 || app.saarm.InstrumentModel==13 || app.saarm.InstrumentModel==14) //1024, 1028, 1030, 2500,2200
			rot.z = 0.0;
		// 2001.04.19 end
		NativeArmToSA.SetRotation(rot);

		//assign mouse buttons
		if(app.saarm.InstrumentModel==13||app.saarm.InstrumentModel==14) //2500,2200
		{
			RDS_SetRMouseLButton(1);
			RDS_SetRMouseMButton(3);
			RDS_SetRMouseRButton(2);
		}
		else
		{
			RDS_SetRMouseLButton(1);  // was 1 2 3
			RDS_SetRMouseMButton(2);
			RDS_SetRMouseRButton(3);
		}
	}
	
	if(app.saarm.InstrumentModel==13 || app.saarm.InstrumentModel==14) //2500,2200
	{
		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)+90.0;
		J.x[1] = -(RDS_GetAxisAngles(1)/app.saarm.units.DTR)-90.0;
		J.x[2] = (RDS_GetAxisAngles(2)/app.saarm.units.DTR);
		J.x[3] = (RDS_GetAxisAngles(3)/app.saarm.units.DTR)+270.0;
		J.x[4] = (RDS_GetAxisAngles(4)/app.saarm.units.DTR);
		J.x[5] = (RDS_GetAxisAngles(5)/app.saarm.units.DTR);
	}
	// 2003.10.24 - added 1000 series, have a 1028 from Romer for testing, bet we can do
	// this for  3000's too - if so, we could just get rid of NativeArmToSA xform
	// NOTE: this used to be what we did for the 3000's, notice the commented out line in the 'else' below
	// NOTE: ALL this requires a corrected instrument.lst as well - added to the coop project today
	// 2003.10.28 - we do not appear to need the corrections in the Inst.lst - just these code changes
	// 2003.10.29 - not neede for 3000's according to Rich Steckel's testing with his (old) 3000 series arm!
	else if(app.saarm.InstrumentModel==11 || app.saarm.InstrumentModel==12|| app.saarm.InstrumentModel==22) //1024, 1028, 1030
	{
		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)+90.0;
		J.x[1] = (RDS_GetAxisAngles(1)/app.saarm.units.DTR)+90.0;
		J.x[2] = (RDS_GetAxisAngles(2)/app.saarm.units.DTR)+180.0;
		J.x[3] = (RDS_GetAxisAngles(3)/app.saarm.units.DTR)+90.0;
		J.x[4] = -(RDS_GetAxisAngles(4)/app.saarm.units.DTR);
		J.x[5] = -(RDS_GetAxisAngles(5)/app.saarm.units.DTR);
	}
	else if( app.saarm.InstrumentModel>=79 && app.saarm.InstrumentModel<=84 )	// Infinite (5000 series) arms
	{
		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)-90.0;
		J.x[1] = (RDS_GetAxisAngles(1)/app.saarm.units.DTR)+90.0;
		J.x[2] = (RDS_GetAxisAngles(2)/app.saarm.units.DTR);
		J.x[3] = (RDS_GetAxisAngles(3)/app.saarm.units.DTR)+90.0;
		J.x[4] = -(RDS_GetAxisAngles(4)/app.saarm.units.DTR);
		J.x[5] = (RDS_GetAxisAngles(5)/app.saarm.units.DTR);
		if(DOF == 7)
			J.x[6] = -(RDS_GetAxisAngles(6)/app.saarm.units.DTR);
	}
	else if( app.saarm.InstrumentModel>=92 && app.saarm.InstrumentModel<=103 )	// Infinite 2 (5100 series) arms
	{
		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)-90.0;
		J.x[1] = (RDS_GetAxisAngles(1)/app.saarm.units.DTR)+90.0;
		J.x[2] = (RDS_GetAxisAngles(2)/app.saarm.units.DTR);
		J.x[3] = (RDS_GetAxisAngles(3)/app.saarm.units.DTR)+90.0;
		J.x[4] = -(RDS_GetAxisAngles(4)/app.saarm.units.DTR);
		J.x[5] = (RDS_GetAxisAngles(5)/app.saarm.units.DTR);
		if(DOF == 7)
			J.x[6] = -(RDS_GetAxisAngles(6)/app.saarm.units.DTR)-90.0;	// 2008.11.25 - tested with 5128Sc serial # 5128Sc-113
	}
	else	// 3000's, etc.
	{
		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)-90.0;
//		J.x[0] = (RDS_GetAxisAngles(0)/app.saarm.units.DTR)+90.0;
		J.x[1] = (RDS_GetAxisAngles(1)/app.saarm.units.DTR)+90.0;
		J.x[2] = (RDS_GetAxisAngles(2)/app.saarm.units.DTR)+180.0;
		J.x[3] = (RDS_GetAxisAngles(3)/app.saarm.units.DTR)+90.0;
		J.x[4] = -(RDS_GetAxisAngles(4)/app.saarm.units.DTR);
		J.x[5] = -(RDS_GetAxisAngles(5)/app.saarm.units.DTR);
		if(DOF == 7)
			J.x[6] = -(RDS_GetAxisAngles(6)/app.saarm.units.DTR)+180.0;
	}

	return TRUE;
}
int RomerArm::GetButtonStates()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonStates();

	int duration=0;
	buttonstate = 0;

	if( CheckButton(1, &duration) )
	{
		buttonstate = 1;
		buttonstate += duration;
	}
	else if( CheckButton(2, &duration) )
	{
		buttonstate = 2;
		buttonstate += duration;
	}
	else if( CheckButton(3, &duration) )
	{
		buttonstate = 4;
		buttonstate += duration;
	}	
	return buttonstate;
}
BOOL RomerArm::CheckButton(int but, int* duration)
{
	if( !ConnectedToArm )
		return ArmBase::CheckButton(but, duration);


	if(RDS_IsBeamNOn(1))
	{
		TRACE0("BEAM ONE ON!\n");
		OutputSpeakerSound(1200,50);
	}
	if(RDS_IsBeamNOn(2))
	{
		TRACE0("BEAM TWO ON!\n");
		OutputSpeakerSound(1600,50);
	}
	if(RDS_IsBeamNOn(1) && RDS_IsBeamNOn(2))
	{
		TRACE0("BEAMs ON!\n");
		OutputSpeakerSound(2000,50);
	}

	if(RDS_IsButtonNDown(but))
	{
		TRACE1("Button %d down\n",but);
		DWORD Start = ::GetCurrentTime();
		while((RDS_IsButtonNDown(but))&&((::GetCurrentTime() - Start) < LONG_DURATION))
			RDS_GetSystemUpdate(TRUE);
		if(RDS_IsButtonNDown(but))
		{
			*duration = BUTTON_LONG;
			OutputSpeakerSound(3000,100);
			while((RDS_IsButtonNDown(but)))
				RDS_GetSystemUpdate(TRUE);
			return TRUE;
		}
		else if(!IsRemoteMouseEnabled())
			*duration = BUTTON_SHORT;
	}
	else
		return FALSE;
	return TRUE;
}
BOOL RomerArm::GetTemp()
{
	if( !ConnectedToArm )
		return ArmBase::GetTemp();
	
	return TRUE;
}

int RomerArm::GetProbeNumber()
{
	if( !ConnectedToArm )
		return ArmBase::GetProbeNumber();
	
	probeValid = RDS_ValidateCurrentProbe() ? true : false;
	int probenum = 0;
	probenum = RDS_GetProbeNumber();
	app.saarm.Data.TipDia = RDS_GetBallProbeDiameter()/1000.0;
	return probenum;
}

int RomerArm::GetButtonCount()
{
	if( !ConnectedToArm )
		return ArmBase::GetButtonCount();

	return RDS_GetButtonCount();
}

void RomerArm::SetProbeNumber(int ProbeNum)
{
	if( !ConnectedToArm )
	{
		ArmBase::SetProbeNumber(ProbeNum);
		return;
	}

	RDS_SetProbeNumber(ProbeNum);
}

BOOL RomerArm::ValidateCurrentProbe()
{
	if( !ConnectedToArm )
		return ArmBase::ValidateCurrentProbe();

	TRACE1("calling RDS_ValidateCurrentProbe, dia. = %f\n", app.saarm.Data.TipDia);
	if (!RDS_ValidateCurrentProbe())
	{
		char errmsg[MAX_PATH];
		RDS_GetErrorMessage((LPTSTR)errmsg, MAX_PATH);
		CString Msg;
		Msg.Format(_T("%s"),tr(CA2T(errmsg)));

		NRKProgress valProbeErr;
		valProbeErr.Start(_T("Could not Validate Probe."));
		valProbeErr.SetMsg(Msg);
		RDS_OutputSpeakerSound(3000,250);
		Delay(0.1,TRUE);
		RDS_OutputSpeakerSound(2500,250);
		Delay(2.0,TRUE);
		valProbeErr.Kill();
		probeValid = false;
		return FALSE;
	}
	// 2003.04.25 - just to be safe, reset the probe dia.
	app.saarm.Data.TipDia = RDS_GetBallProbeDiameter()/1000.0;
	TRACE1("after RDS_ValidateCurrentProbe, dia. = %f\n", app.saarm.Data.TipDia);
	probeValid = true;
	return TRUE;
}

void RomerArm::SetProbeAutoDetect(BOOL ad)
{
	if( !ConnectedToArm )
	{
		ArmBase::SetProbeAutoDetect(ad);
		return;
	}

	RDS_SetProbeAutoDetect(ad);
}

BOOL RomerArm::SetVolumeLevel( int VolumeLevel )
{
	return RDS_SetVolumeLevel(VolumeLevel);
}
void RomerArm::OutputSpeakerSound( int frequency, int duration )
{
	RDS_OutputSpeakerSound(frequency, duration);
}

BOOL RomerArm::IsRomer()
{
	return TRUE;
}
BOOL RomerArm::IsFaro()
{
	return FALSE;
}
BOOL RomerArm::IsFaroUSB()
{
	return FALSE;
}
BOOL RomerArm::IsFrenchRomer()
{
	return FALSE;
}
BOOL RomerArm::IsSNL()
{
	return FALSE;
}
BOOL RomerArm::IsMetrisGarda()
{
	return FALSE;
}

BOOL RomerArm::IsRemoteMouseEnabled()
{
	if( !ConnectedToArm )
		return ArmBase::IsRemoteMouseEnabled();

	return RDS_IsRemoteMouseEnabled();
}

void RomerArm::ToggleRemoteMouse()
{
	if( !ConnectedToArm )
	{
		ArmBase::ToggleRemoteMouse();
		return;
	}

	if(RDS_IsRemoteMouseEnabled())
	{
		Delay(0.75);
		RemoteMouse(FALSE);
		app.saarm.TTS.TextToSpeech(_T("Romer mouse disabled"));
	}	
	else
	{
		Delay(0.75);
		RemoteMouse(TRUE);
		app.saarm.TTS.TextToSpeech(_T("Romer mouse enabled"));
	}
}
void RomerArm::RemoteMouse(BOOL flag)
{
	if( !ConnectedToArm )
	{
		ArmBase::RemoteMouse(flag);
		return;
	}
	RDS_RemoteMouseFunction(flag);
}

void RomerArm::ArmBeep(int sound)
{
	switch(sound)
	{
	case REC_OK:
		{
			RDS_OutputSpeakerSound(1400,120);
			//theApp.TTS.PlayTone(1400,120);
			break;
		}
	case CALC_OK:
		{
			RDS_OutputSpeakerSound(2000,80);
			//theApp.TTS.PlayTone(1400,120);
			break;
		}
	case UNDO_OK:
		{
			RDS_OutputSpeakerSound(800,200);
			//theApp.TTS.PlayTone(800,200);
			break;
		}
	case UNDO_FAIL:
		{
			RDS_OutputSpeakerSound(600,100);
			//theApp.TTS.PlayTone(600,100);
			break;
		}
	case JT_LIMIT:
		{
			RDS_OutputSpeakerSound(3000,250);
			Delay(0.1,TRUE);
			RDS_OutputSpeakerSound(2500,250);
		}
	}
}
void RomerArm::ProcessPipeTransform(const CString& name)
{
	X = (recdata.stProbePosition.dX/1000.0);
	Y = (recdata.stProbePosition.dY/1000.0);
	Z = (recdata.stProbePosition.dZ/1000.0);

	//TRACE1("ProcessPipeTransform - %s\n", name);
	//TRACE3("Raw Data: X=%f,Y=%f,Z=%f\n",X,Y,Z);
	Vector RomerCoord( X,Y,Z ); //raw data
	Vector AnalyzerCoord;
	//rot and trans to analyzer coords
	NativeArmToSA.Multiply( RomerCoord, &AnalyzerCoord );
	X = AnalyzerCoord.x;
	Y = AnalyzerCoord.y;
	Z = AnalyzerCoord.z;

	Transform t;
	t.SetRotation(
		Vector(-recdata.stProbePosition.dYI,-recdata.stProbePosition.dYJ,recdata.stProbePosition.dYK),
		Vector(-recdata.stProbePosition.dXI,-recdata.stProbePosition.dXJ,recdata.stProbePosition.dXK),
		Vector(recdata.stProbePosition.dZI,recdata.stProbePosition.dZJ,-recdata.stProbePosition.dZK));
	t.SetTranslation(AnalyzerCoord);

	FrameToSpan(t,name);
}

//////////////////////////////////////////////////////////////////////////
// for scanner with tcp/ip to ScanWorks...
SOCKET		hSocket;
CWinThread* pThread;
void ScanWorksListen(LPVOID ptr);

BOOL RomerArm::ScannerSettings()
{
	TRACE0("CimCoreArm::ScannerSettings...\n");
	if(!ConnectedToArm)
		return ArmBase::ScannerSettings();

	return TRUE;
}
BOOL RomerArm::StartScannerMeas()
{
	TRACE0("CimCoreArm::StartScannerMeas...");

	if(!app.saarm.scannerLicenseValid)
	{
		AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
		return FALSE;
	}

	if (trackScanDlg.IsAlreadyPopped())
	{
		//TRACE0("trackScanDlg.IsAlreadyPopped() = TRUE, returning...\n");
		return TRUE;
	}

	if(!ConnectedToArm)
		return ArmBase::StartScannerMeas();

	//////////////////////////////////////////////////////////////////////////////////////////////////
	// 2008.07.21
	// Ralph Stephan @ ScanWorks says: "We are moving away from using only probe number 8 and this is 
	// probably the safest way to detect if the scanner is active.  So, we'll verify the scanner once
	// ScanWorks has control - see ScanWorksCheckForProbe().

	// make sure user does not try to scan when scanner is not connected...
	//probeValid = RDS_ValidateCurrentProbe() ? true : false;
	//int probenum = RDS_GetProbeNumber();
	//if(probenum != 8)	// the scanner probe number is ALWAYS 8 according to Mike Hunwick @ Perceptron
	//{
	//	NRKProgress noScanner;
	//	noScanner.Start(_T("No Scanner Connected (Probe #8)"));
	//	noScanner.SetMsg(_T("You must connect the scanner to the arm."));
	//	RDS_OutputSpeakerSound(3000,250);
	//	Delay(0.1,TRUE);
	//	RDS_OutputSpeakerSound(2500,250);
	//	Delay(2.0,TRUE);
	//	noScanner.Kill();
	//	return FALSE;
	//}
	//////////////////////////////////////////////////////////////////////////////////////////////////


	// 2007.10.29 - reg key no longer used as of ScanWorks v5
	//if(!scannerRegKeySet)
	//{
	//	if(!SetScannerRegKey(true))
	//		return FALSE;
	//}

	// can't share the connection to winRDS with ScanWorks...
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	Delay(0.1,TRUE);
	RDS_OutputSpeakerSound(1400,120);
	DisconnectFromHardware();

	d.StartScannerTimer(app.saarm.Data.scannerTimerPeriodSec);

	DExternalScannerMeas* pnewdlg = new DExternalScannerMeas((ArmBase&)*this, trackScanDlg);
	DOMODELESS(pnewdlg); // OR the normal way:  pnewdlg->Create(pnewdlg->IDD,NULL);

	if(!pThread)
	{
		TRACE0("call begin thread...\n");
		LPVOID sptr = (LPVOID)this;
		pThread = AfxBeginThread((AFX_THREADPROC)ScanWorksListen, sptr, THREAD_PRIORITY_NORMAL, 0, 0, NULL);
		Sleep(250);
	}
	if(!hSocket || hSocket == INVALID_SOCKET)
	{
		TRACE0("Call TCPIPConnect...\n");
		if (!TCPIPConnect(&hSocket, _T("localhost"), COMM_PORT))
		{
			CString m;
			m.Format(Tr("Unable to connect tcp/ip.  Is ScanWorks running??\n\nBe sure to set 'Start ScanWorks on Console Startup' in Console Options if running ScanWorks v.5."));
			AfxMessageBox(m, MB_OK|MB_ICONERROR);
			trackScanDlg.CloseDialog();
			Sleep(1000);
			d.StopScannerTimer();
			ReConnectToArm();
			return FALSE;
		}
		Sleep(250);
		// 2005.10.03 - added support for ScanWorks v. 4.4 - it allows setting ScanWorks units...
		SetScanWorksUnitsToInches();
	}
	if(pThread && hSocket != INVALID_SOCKET)
	{
		Sleep(200);
		if(!ScanWorksMoverConnect())
		{
			trackScanDlg.CloseDialog();
			Sleep(1000);
			d.StopScannerTimer();
			ReConnectToArm();
			return FALSE;
		}
		if(!ScanWorksCheckForProbe())	// 2008.07.21 - insure user has the scanner active
		{
			StopScannerMeas();
			Sleep(1000);
			trackScanDlg.CloseDialog();
			d.StopScannerTimer();
			ReConnectToArm();
			return FALSE;
		}
		Sleep(2000);
		TRACE0("Call StartModel...\n");
		StartModel();
		return TRUE;
	}
	trackScanDlg.CloseDialog();
	AfxMessageBox(_T("Could not begin ScanWorks listening thread."), MB_ICONERROR);
	return FALSE;
}
BOOL RomerArm::StopScannerMeas()
{
	TRACE0("CimCoreArm::StopScannerMeas - stop scan");
	int		command;
	long	Written;
	command = SCANWORKS_STOP_SCAN;
	if(!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
		return false;
	}

	//SCANWORKS_PART_DISCONNECT


	TRACE0("CimCoreArm::StopScannerMeas - stop model");
	StopModel();
	//AfxMessageBox(_T("model stopped?"));
	CString statmsg;
	statmsg.Format(_T("Model Stopped"));
	PostUpdateScanDlg(statmsg);
	HideScanWorksToolBar();
	Sleep(200);
	ScanWorksMoverDisconnect();
	//AfxMessageBox(_T("mover diconnected?"));
	statmsg.Format(_T("Disconnected from Mover"));
	PostUpdateScanDlg(statmsg);
	Sleep(200);
	TRACE0("CimCoreArm::StopScannerMeas - close socket");
	TCPIPClose(hSocket);
	hSocket = INVALID_SOCKET;
	//AfxMessageBox(_T("tcpip closed?"));
	statmsg.Format(_T("Scanworks winRDS connection closed"));
	PostUpdateScanDlg(statmsg);
	Delay(0.5,TRUE);
	return SendBufferedCloudDataAndReconnect();
}
bool RomerArm::SendBufferedCloudDataAndReconnect()
{
	TRACE0("CimCoreArm::SendBufferedCloudDataAndReconnect...");
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopScannerTimer();
	CheckAndSendScannerCloud(0, true);

	//AfxMessageBox(_T("last cloud sent?"));
	CString statmsg;
	statmsg.Format(_T("Interface re-connecting to winRDS..."));
	PostUpdateScanDlg(statmsg);

	Sleep(100);

	// re-establish connection to arm
	if(ReConnectToArm())
	{
		d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
		return true;
	}
	return false;
}
void RomerArm::PostUpdateScanDlg(CString msg)
{
	if( !ConnectedToArm )
		return ArmBase::PostUpdateScanDlg(msg);
	TRACE1("Post Message to update scan dlg with text '%s'\n", msg);
	scanMsg = msg;
	wm.PostMessage(UPDATE_SCAN_DLG, 0, 0);
}
void RomerArm::OnUpdateScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	if(((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.GetCount() > 100)
		((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.ResetContent();
	int idx = ((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.AddString(scanMsg);
	((DExternalScannerMeas*)trackScanDlg.GetDlg())->statsList.SetTopIndex(idx);
}
void RomerArm::CloseScanDlg()
{
	if(!trackScanDlg.IsAlreadyPopped())
		return;
	TRACE0("Post Message to close scan dlg...\n");
	wm.PostMessage(CLOSE_SCAN_DLG);
}

///////////////////////////////////////////////////////////////////
// Perceptron specific
// 2007.10.29 - DelayConnect key no longer used as of SW v5
bool RomerArm::SetScannerRegKey(bool reportError/*=false*/)
{
	// create/open the key...
	HKEY hk; 
	DWORD dwDisp = 0; 
	if( RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Perceptron\\GSE\\Settings"), 
		0, NULL, REG_OPTION_NON_VOLATILE,KEY_WRITE, NULL, &hk, &dwDisp) ) 
	{
		if(reportError)
		{
			CString temp;
			StringHelper::GetLastErrorString(temp);
			AfxMessageBox(_T("Could not create/open Perceptron registry key.\n")+temp, MB_OK|MB_ICONERROR);
		}
		return false;
	}

	// set the key value...
	DWORD delayConnect = 1;
	// if set to 0, scanworks will act just like it does now
	// if set to 1, scanworks will NOT connect to the arm at startup unless TCPIP is disabled. 
	// when you check the TCPIP box, it will connect to the arm - if you uncheck the box it will disconnect from the arm. it will also connect/disconnect to the arm as you start/stop the model.
	bool ret = true;
	if( RegSetValueEx(hk, _T("DelayConnect"), 0, REG_DWORD, (LPBYTE) &delayConnect, sizeof(DWORD)) )
	{
		if(reportError)
		{
			CString temp;
			StringHelper::GetLastErrorString(temp);
			AfxMessageBox(_T("Could not Set Perceptron registry key.\n")+temp, MB_OK|MB_ICONERROR);
		}
		ret = false;
	}
	RegCloseKey(hk); 
	return ret;
}
void RomerArm::ResetSocket() 

{
	if (hSocket == INVALID_SOCKET)
	{
		return;
	}
	TCPIPClose(hSocket);
	hSocket = INVALID_SOCKET;
}
bool RomerArm::ScanWorksMoverConnect()
{
	int		command;
	long	Written;

	command = SCANWORKS_MOVER_CONNECT;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(Tr("Unable to connect ScanWorks to mover.  Is ScanWorks running??\n\nBe sure to set 'Start ScanWorks on Console Startup' in Console Options if running ScanWorks v.5."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
bool RomerArm::ScanWorksMoverDisconnect()
{
	int		command;
	long	Written;

	command = SCANWORKS_MOVER_DISCONNECT;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("unable to disconnect ScanWorks from mover"), MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
void RomerArm::ShowScanWorksToolBar()
{
	int		command;
	long	Written;

	command = SCANWORKS_SHOW_TOOLBAR;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to show ScanWorks toolbar"), MB_OK|MB_ICONERROR);
		return;
	}

}
void RomerArm::HideScanWorksToolBar()
{
	int		command;
	long	Written;

	command = SCANWORKS_HIDE_TOOLBAR;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to hide ScanWorks toolbar"), MB_OK|MB_ICONERROR);
		return;
	}
}

bool RomerArm::SetScanWorksUnitsToInches()
{
	int		command;
	long	Written;

	command = SCANWORKS_SET_UNITS;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(_T("Unable to set ScanWorks units.\nEither upgrade to ScanWorks v. 4.4 or later,\nor set units to Inches in ScanWorks"));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}

	stSWUnits unit;
	unit.state = 0;	// 1 is mm

	if (!TCPIPWrite(hSocket, (LPVOID) &unit, (long) sizeof(unit), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("Unable to set ScanWorks units.\nEnsure units are set to Inches in ScanWorks."), MB_OK|MB_ICONERROR);
		return false;
	}
	return true;
}
bool RomerArm::ScanWorksCheckForProbe()
{
	int command;
	long Written;
	long nNumberRead;

	command = SCANWORKS_GET_PROBE;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		CString m;
		m.Format(_T("Unable to check for ScanWorks probe settings."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);

		return false;
	}

	stSWProbeSettings settings;
	TCPIPRead(hSocket, &settings, sizeof(settings), &nNumberRead);
	if (nNumberRead < sizeof(settings))
	{
		CString m;
		m.Format(_T("Unable to get ScanWorks probe settings."));
		AfxMessageBox(m, MB_OK|MB_ICONERROR);
		return false;
	}

	if (settings.type == TOUCH_PROBE_TYPE)
	{
		CString m;
		m.Format(_T("ScanWorks: Touch Probe is active.\nNeed to activate the Scanner to run ScanWorks."));
		AfxMessageBox(m, MB_OK|MB_ICONINFORMATION);
		return false;
	}
	else if(settings.type == SCAN_PROBE_TYPE)
	{
		return true;
	}
	return false;
}
bool RomerArm::StartModel()
{
	int		command;
	long	Written;

	command = SCANWORKS_MODEL_START;
	if (!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		AfxMessageBox(_T("ScanWorks unable to start model."), MB_OK|MB_ICONERROR);
		trackScanDlg.CloseDialog();
		return false;
	}
	return true;
}
bool RomerArm::StopModel()
{
	int		command;
	long	Written;

	command = SCANWORKS_MODEL_STOP;
	if(!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	{
		ResetSocket();
		PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
		return false;
	}
	return true;
}
void RomerArm::CleanUpPerceptron()
{
	if(pThread)
		delete pThread;
}
// the ScanWorks tcp/ip thread function...
void ScanWorksListen(LPVOID ptr)
{
	TRACE0("ScanWorksListen thread started.\n");
	RomerArm* pParent = (RomerArm*)ptr;

	stSWPartConnect			Part;
	//stSWTouchprobeConnect	Touchprobe;
	stSWPhase				Phase;
	stSWScanRegion			Region;
	stSWScanLine			Line;
	//stSWGridHeader			Grid;

	stSWPartConnectHS		PartHS;
	//stSWTouchprobeConnectHS	TouchprobeHS;
	stSWPhaseHS				PhaseHS;
	stSWScanRegionHS		RegionHS;

	int				nTotalXYZs = 0;
	int				nTotalReadXYZs = 0;
	int				nTotalLines = 0;
	int				nXYZs = 0;
	float			szScanData[12000][3];
	int				nData;
	sockaddr_in		sin;
	bool			loop=true;
	//char			lpszBuf[256];


	bool m_bHandshakes = false;
	bool m_bTagdata = true;

	while(1)
	{
		SOCKET		hListenSocket;

		if (!TCPIPInitialize())	// 2005.10.12 - initiate use of WS2_32.DLL for ScanWorks connection - only once now (See ConnectToArm)
		{
			TRACE0("TCPIPInitialize failed.\n");
			return;
		}

		hListenSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (hListenSocket == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Cannot create socket to ScanWorks. ") + err);
			TRACE1("Cannot create socket to ScanWorks. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		int i = TRUE;	// 2005.10.12 - set to reuse address in case user would like to scan more than once during the life of this app!
		i = setsockopt(hListenSocket, SOL_SOCKET, SO_REUSEADDR, (LPSTR)&i, sizeof(i));
		if (i == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Failed to set sock option - SO_REUSEADDR ") + err);
			TRACE1("Failed to set sock option - SO_REUSEADDR. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		memset(&sin, 0, sizeof(sin));
		sin.sin_family = AF_INET;
		sin.sin_port   = htons(DATA_PORT);
		if (bind(hListenSocket, (sockaddr*)&sin, sizeof(sin)) == SOCKET_ERROR)
		{
			CString err;
			err.Format(_T("[%d]"), WSAGetLastError());
			pParent->PostUpdateScanDlg(_T("Could not bind to ScanWorks port. ") + err);
			TRACE1("Cannot bind to ScanWorks port. [%d]\n", WSAGetLastError());
			Delay(5.0);
			pParent->CloseScanDlg();
			return;
		}

		bool connected = true;
		//pParent->PostUpdateScanDlg(_T("Start sock set thread loop"));
		while (1)
		{
			TRACE("Top of while loop\n");
			//Beep(900,100);
			if (listen(hListenSocket, SOMAXCONN) == SOCKET_ERROR)	// 2005.10.13 - set backlog of pending connections to the max value
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("ScanWorks Could not listen. ") + err);
				TRACE1("ScanWorks Could not listen for tcp/ip packets. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			sockaddr_in		sinfrom;
			int				nLength = sizeof(sinfrom);

			TRACE("1\n");

			SOCKET	hSock;
			hSock = accept(hListenSocket, (sockaddr*)&sinfrom, &nLength);
			if (hSock == INVALID_SOCKET )
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Cannot create linkage socket to ScanWorks. ") + err);
				TRACE1("Cannot create linkage socket to ScanWorks. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			TRACE("1a\n");

			//int ret = TRUE;	// 2005.10.12 - set to reuse address in case user would like to scan more than once during the life of this app!
			//ret = setsockopt(hSock, SOL_SOCKET, SO_REUSEADDR, (LPSTR)&ret, sizeof(ret));
			//if (ret == SOCKET_ERROR)
			//{
			//	CString err;
			//	err.Format(_T("[%d]"), WSAGetLastError());
			//	pParent->PostUpdateScanDlg(_T("Failed to set sock option for linkage socket - SO_REUSEADDR ") + err);
			//	TRACE1("Failed to set sock option for linkage socket - SO_REUSEADDR. [%d]\n", WSAGetLastError());
			//	Delay(5.0);
			//	pParent->DoEndScanOperations();
			//	return;
			//}

			TRACE("2\n");

			if (hSock == INVALID_SOCKET)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("ScanWorks Failed to accept socket connection. ") + err);
				TRACE1("scanworks Failed to accept. [%d]\n", WSAGetLastError());
				Delay(5.0);
				pParent->CloseScanDlg();
				TCPIPCleanup();
				AfxEndThread(0);
			}

			DWORD dwAddr = ntohl(sinfrom.sin_addr.s_addr);
			int   i      = 0;

			i = 8 * 1024;
			i = setsockopt(hSock, SOL_SOCKET, SO_SNDBUF, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - SendBuffer ") + err);
				TRACE0("scanworks Failed to set sock options.\n");
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			i = 8 * 1024;
			i = setsockopt(hSock, SOL_SOCKET, SO_RCVBUF, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - ReceiveBuffer ") + err);
				TRACE0("scanworks Failed to set sock options.\n");
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			i = TRUE;
			i = setsockopt(hSock, IPPROTO_TCP, TCP_NODELAY, (LPSTR)&i, sizeof(i));
			if (i == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - TCP_NODELAY ") + err);
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}

			ULONG j = 0; // Zero means block
			if (ioctlsocket(hSock, FIONBIO, &j) == SOCKET_ERROR)
			{
				CString err;
				err.Format(_T("[%d]"), WSAGetLastError());
				pParent->PostUpdateScanDlg(_T("Failed to set sock options - Blocking ") + err);
				Delay(5.0);
				pParent->CloseScanDlg();
				return;
			}
			Vector xyzCimCore, xyzArm, xyzSA;

			TRACE("Bottom of while loop\n");
			//Beep(1000,100);
			loop = true;
			while (loop)
			{
				int nRead = recv(hSock, (char*)&nData, sizeof(nData), 0);

				if (nRead == SOCKET_ERROR)
				{
					pParent->PostUpdateScanDlg(_T("Error reading data"));
					break;
				}

				long nNumberRead=0;
				long nNumberWritten=0;
				int command;
				CString str;

				switch (nData)
				{
				case (SCANWORKS_PART_CONNECT): 
					TCPIPRead(hSock, &Part, sizeof(stSWPartConnect), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("ScanWorks Part Connected"));
					TRACE0("Part connect...\n");

					command = SCANWORKS_PART_CONNECT_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PartHS.model_number = Part.model_number;
					strcpy(PartHS.model_name, Part.model_name);
					PartHS.serial_number = Part.serial_number;
					PartHS.status = ALL_OK;
					TCPIPWrite(hSock, &PartHS, sizeof(stSWPartConnectHS), &nNumberWritten);

					break;


				case (SCANWORKS_PART_DISCONNECT):
					TCPIPRead(hSock, &Part, sizeof(stSWPartConnect), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("ScanWorks Closing, Please Wait..."));
					TRACE0("Part disconnect.\n");

					command = SCANWORKS_PART_DISCONNECT_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PartHS.model_number = Part.model_number;
					strcpy(PartHS.model_name, Part.model_name);
					PartHS.serial_number = Part.serial_number;
					PartHS.status = ALL_OK;
					TCPIPWrite(hSock, &PartHS, sizeof(stSWPartConnectHS), &nNumberWritten);

					loop = false;
					break;


				case (SCANWORKS_PHASE_START):  
					TCPIPRead(hSock, &Phase, sizeof(stSWPhase), &nNumberRead);
					pParent->PostUpdateScanDlg(_T("ScanWorks Connected..."));
					TRACE0("Phase start.\n");

					command = SCANWORKS_PHASE_START_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PhaseHS.phase_number = Phase.phase_number;
					strcpy(PhaseHS.phase_name, Phase.phase_name);
					PhaseHS.status = ALL_OK;
					TCPIPWrite(hSock, &PhaseHS, sizeof(stSWPhaseHS), &nNumberWritten);

					break;


				case (SCANWORKS_PHASE_STOP):
					TCPIPRead(hSock, &Phase, sizeof(stSWPhase), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("ScanWorks Phase Stop"));
					TRACE0("Phase stop.\n");

					command = SCANWORKS_PHASE_STOP_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					PhaseHS.phase_number = Phase.phase_number;
					strcpy(PhaseHS.phase_name, Phase.phase_name);
					PhaseHS.status = ALL_OK;
					TCPIPWrite(hSock, &PhaseHS, sizeof(stSWPhaseHS), &nNumberWritten);

					break;


				case (SCANWORKS_SCAN_REGION_START): 
					TCPIPRead(hSock, &Region, sizeof(stSWScanRegion), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("Scan Start"));
					pParent->SendOnScanPassStarted();	// 2016.03.29 - added for meshing - need to know when to start/stop mesh based on scan passes

					command = SCANWORKS_SCAN_REGION_START_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					RegionHS.scan_number = Region.scan_number;
					RegionHS.process_switch = Region.process_switch;
					strcpy(RegionHS.scan_name, Region.scan_name);
					RegionHS.status = ALL_OK;
					TCPIPWrite(hSock, &RegionHS, sizeof(stSWScanRegionHS), &nNumberWritten);

					break;


				case (SCANWORKS_SCAN_REGION_STOP):
					TCPIPRead(hSock, &Region, sizeof(stSWScanRegion), &nNumberRead);
					//pParent->PostUpdateScanDlg(_T("Scan Stop"));
					pParent->SendOnScanPassStopped();	// 2016.03.29 - added for meshing - need to know when to start/stop mesh based on scan passes

					if (app.saarm.Data.voxelizationEnabled)
						pParent->ClearScanClouds();

					command = SCANWORKS_SCAN_REGION_STOP_HS;
					TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);

					RegionHS.scan_number = Region.scan_number;
					RegionHS.process_switch = Region.process_switch;
					strcpy(RegionHS.scan_name, Region.scan_name);
					RegionHS.status = ALL_OK;
					TCPIPWrite(hSock, &RegionHS, sizeof(stSWScanRegionHS), &nNumberWritten);

					nTotalXYZs =0;
					nTotalReadXYZs =0;

					break;

				case (SCANWORKS_SCAN_LINE):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);

					//str.Format(_T("lines: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					//TRACE1("%s\n", str);
					//str.Format(_T("lines: %d"), nTotalLines++);
					//pDlg->SetDlgItemText(IDC_TOTALLINES, str);
					TRACE1("%s\n", str);
					if (m_bHandshakes)
					{
						command = SCANWORKS_SCAN_LINE_HS;
						TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
					}
					break;

				case (SCANWORKS_CURSOR_SCANLINE):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);
					//str.Format(_T("lines: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					break;

				case (SCANWORKS_SCAN_DATA):
					if (m_bTagdata)
					{
						TCPIPRead(hSock, szScanData, sizeof(float)*(Line.num_xyzs+1)*3, &nNumberRead);
						nTotalXYZs += Line.num_xyzs;
						nTotalReadXYZs += nNumberRead/3/(sizeof(float));
						//str.Format(_T("points: %d (%d) (#%d)"), nTotalXYZs, nTotalReadXYZs, (int) szScanData[Line.num_xyzs][0]);
						//pDlg->SetDlgItemText(IDC_POINTS, str);
						//TRACE1("%s\n", str);
						nXYZs += nTotalXYZs;
						//str.Format(_T("points: %d"), nXYZs);
						//pDlg->SetDlgItemText(IDC_TOTALPOINTS, str);
						//TRACE1("%s\n", str);

						if(app.saarm.Data.useScanStripeClouds && pParent->performingStdScannerMeas)
						{
							pParent->AddScannerCloudStripe();

							SAIPScanStripePoint sspt;
							Vector xyzRDS;

							for(int j=0; j<Line.num_xyzs-20; j++)	// 2005.10.04 - Yes, the -20 is deliberate.  These last points in the line tend to be bogus.
							{
								if(j % app.saarm.Data.thinningFactor != 0)	// 2006.03.02
									continue;
								if(j >= 12000)
									continue;

								try
								{
									xyzCimCore.x = szScanData[j][1];
									xyzCimCore.y = szScanData[j][2];
									xyzCimCore.z = szScanData[j][3];
								}
								catch(...)
								{
									continue;
								}

								//xyzCimCore.Scale(1.0/25.4);	2005.10.03 - we can set units to inches in ScanWorks as of ScanWorks v. 4.4
								pParent->NativeArmToSA.Multiply( xyzCimCore, &xyzArm );
								pParent->scannerXform.Multiply(xyzArm, &xyzSA);
								sspt.x = xyzSA.x;
								sspt.y = xyzSA.y;
								sspt.z = xyzSA.z;

								pParent->AddScannerCloudStripePt(sspt);
							}

							//TRACE3("Scan Line XYZ: x=%f, y=%f, z=%f\n",
							//	Line.machine_X,
							//	Line.machine_Y,
							//	Line.machine_Z);
							//TRACE3("Scan Line Roll, Pitch, Yaw: r=%f, p=%f, y=%f\n",
							//	Line.machine_Roll,
							//	Line.machine_Pitch,
							//	Line.machine_Yaw);

							Transform scannerXform;
							scannerXform.SetRotation(Vector(Line.machine_Roll, Line.machine_Pitch, Line.machine_Yaw));
							scannerXform.SetTranslation(Vector(Line.machine_X, Line.machine_Y, Line.machine_Z));

							CriticalSectionLock lock( *csw );
							scannerXform.Scale(app.saarm.LocalToInches);
							ssPkt->SetCurrentLineTransform(scannerXform);

							ssPkt->name = app.saarm.Data.Group;
							ssPkt->collectionName = app.saarm.Data.Collection;
						}
						else
						{
							SAIPCloudBlockPt cbpt;
							for(int j=0; j<Line.num_xyzs-20; j++)	// 2005.10.04 - Yes, the -20 is deliberate.  These last points in the line tend to be bogus.
							{
								if(j % app.saarm.Data.thinningFactor != 0)	// 2006.03.02
									continue;
								if(j >= 12000)
									continue;

								try
								{
									xyzCimCore.x = szScanData[j][1];
									xyzCimCore.y = szScanData[j][2];
									xyzCimCore.z = szScanData[j][3];
								}
								catch(...)
								{
									continue;
								}

								//xyzCimCore.Scale(1.0/25.4);	2005.10.03 - we can set units to inches in ScanWorks as of ScanWorks v. 4.4
								pParent->NativeArmToSA.Multiply( xyzCimCore, &xyzArm );
								pParent->scannerXform.Multiply(xyzArm, &xyzSA);
								cbpt.x = xyzSA.x;
								cbpt.y = xyzSA.y;
								cbpt.z = xyzSA.z;
						
								pParent->AddScannerCloudPt(cbpt);
							}
						}
						//pParent->CheckAndSendScannerCloud(2000);
						if (m_bHandshakes)
						{
							command = SCANWORKS_SCAN_DATA_HS;
							TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
						}
					}
					else
					{
						TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
						nTotalXYZs += Line.num_xyzs;
						nTotalReadXYZs += nNumberRead/3/(sizeof(float));
						str.Format(_T("points: %d (%d)"), nTotalXYZs, nTotalReadXYZs);
						//pDlg->SetDlgItemText(IDC_POINTS, str);
						nXYZs += nTotalXYZs;
						str.Format(_T("points: %d"), nXYZs);
						//pDlg->SetDlgItemText(IDC_TOTALPOINTS, str);
						if (m_bHandshakes)
						{
							command = SCANWORKS_SCAN_DATA_HS;
							TCPIPWrite(hSock, &command, sizeof(command), &nNumberWritten);
						}
					}
					break;

				case (SCANWORKS_SCANLINE_CURSOR_DATA):
					TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
					str.Format(_T("points: %d"), Line.num_xyzs);
					//pDlg->SetDlgItemText(IDC_POINTS, str);
					break;

				case (SCANWORKS_TOUCH_HEADER):
					TCPIPRead(hSock, &Line, sizeof(stSWScanLine), &nNumberRead);
					str.Format(_T("touch: %d"), Line.acquisition_number+1);
					//pDlg->SetDlgItemText(IDC_LINES, str);
					break;

				case (SCANWORKS_TOUCH_DATA):
					TCPIPRead(hSock, szScanData, sizeof(float)*Line.num_xyzs*3, &nNumberRead);
					str.Format(_T("touch: %d"), Line.num_xyzs);
					//pDlg->SetDlgItemText(IDC_POINTS, str);
					break;

				case (SCANWORKS_STOP_SCAN):
					pParent->PostUpdateScanDlg(_T("ScanWorks button press (stop)"));
					//pDlg->SetDlgItemText(IDC_POINTS, "");
					TRACE0("Stop Scan.\n");
					break;

				}
				if (!loop)
					break;
			}
			//Sleep(500);
			TRACE("out of loop - calling TCPIPClose...\n");
			//pParent->PostUpdateScanDlg(_T("Closing ScanWorks Connection, Please Wait..."));
			pParent->StopScannerMeas();
			Sleep(2000);
			//TCPIPClose(hSock);	
			//pParent->SendBufferedCloudDataAndReconnect();
			pParent->CloseScanDlg();
			//break;	// 2005.10.12
		}
		TRACE("Out of while 2...\n");
		//break;	// 2005.10.12
	}
	TCPIPCleanup();	// 2005.10.12
	//pParent->DoEndScanOperations();
	TRACE0(">>>>>>>>Out of while 1 - leaving ScanWorks thread.\n");
	//AfxEndThread(0);
}
//////////////////////////////////////////////////////////////////////////
// for scanner with ToolKit...
//BOOL RomerArm::ScannerSettings()
//{
//	TRACE0("Romer::ScannerSettings...\n");
//	if(!ConnectedToArm)
//		return ArmBase::ScannerSettings();
//
//	TRACE0("Got no settings (yet)!\n");
//	return TRUE;
//}
//
////static AcqSyncGroup testSyncGroup1;
//
//// Here's what happens: 
//// the user hits the Scan button in the main dlg
//// via ArmBase, StartScannerMeas gets called
//// this override of StartScannerMeas stops the QueryTimer in the main dlg, and...
//// pops the scan meas dialog, which does the following...
//// sets parameters for the RDS_WaitAndRecordOnProbeFunctionEx (WAROPF) thread (lives in the dialog, for the life of the dialog)
//// tells the thread to record using scan mode AND on the clock trigger from the scanner
//// starts the WAROPF thread
//BOOL RomerArm::StartScannerMeas()
//{
//	if(!ConnectedToArm)
//		return ArmBase::StartScannerMeas();
//
//	AfxMessageBox(_T("Scanner not yet supported for Romer Arm."), MB_ICONHAND);
//	return FALSE;
//
//	// check the probe ID (8 is the number for the Perceptron Contour probe) to be sure a scanner is connected.
//
//	TRACE0(">>>>>>>>PUT BACK SCANNER LICENSE CHECK<<<<<<<<!!!!\n");
//	//if(!app.saarm.scannerLicenseValid)
//	//{
//	//	AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
//	//	return FALSE;
//	//}
//	//if(lineScanning)
//	//	return TRUE;
//
//	TRACE0(">>>>>>>>PUT BACK SCANNER TIMER SET<<<<<<<<!!!!\n");
//	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
//	d.StopQueryTimer();
//	//Delay(0.5,TRUE);
//	//d.StartScannerTimer(0.25);
//
//	TRACE0("Romer::StartScannerMeas...");
//
//	// set callback params for rds and kick off the thread!
//	recdata.dwRecordFunction = RDS_RECORD_ON_BUTTON_TWO_GOING_ON; //RDS_RECORD_ON_BUTTON_THREE_GOING_ON;	// RDS_RECORD_USING_SCAN_MODE;
//	recdata.lSoundFrequency = 0;
//	recdata.lSoundDuration = 0;
//	recdata.pfnCallbackProbePositionEx = RecordDataCallback;
//	recdata.dwWnd = (DWORD)app.m_pMainWnd;
//	m_enControlOne = CONTROL_ON;
//
//	if(LaunchThreadOne())
//		ResumeThreadOne();
//
//	///////////////////////////////////////////////////////////////////////////////
//	// Perceptron Scanner stuff
//	//// This is where the data will be placed by the Acq component.
//	//ScanSet *dataCollected;
//	//// Setup the sensor.
//	//AcqContinuousMode acqMode;
//	//acqMode.setAcquisitionFrequency( 30 );	// Hz
//	//testSyncGroup1.setAcquisitionMode( &acqMode );
//	//AcqLogicalSensor aLogicalSensor1( *config->getPhysicalSensors().begin() );
//	//const AcqBrightestCGFinder brightestFinder( 5 );
//	//aLogicalSensor1.setCGFinder( &brightestFinder );				// use the "Brightest CG" finder
//	//aLogicalSensor1.setProcessingBox(								// Set image processing window.
//	//	AcqProcessingBox( AcqImagePoint( 100, 100 ), AcqImagePoint( 400, 300 ) ) );
//	//aLogicalSensor1.setRectificationUsage( RECTIFY_ON_HOST );		// perform rectification on Host PC
//	//testSyncGroup1.getLogicalSensors().push_back( &aLogicalSensor1 );
//	//// Tell the Acq component to begin the physical acquisition of data.
//	//dataCollected = testSyncGroup1.beginAcquire();
//	/////////////////////////////////////////////////////////////////////////////////
//
//	//CheckAndSendScannerCloud();
//	return TRUE;
//}
//BOOL RomerArm::StopScannerMeas()
//{
//	KillThreadOne();
//	TRACE0(">>>>>>>>PUT BACK SCANNER TIMER STOP<<<<<<<<!!!!\n");
//	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
//	//d.StopScannerTimer();
//	//Delay(0.5,TRUE);
//	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
//	//testSyncGroup1.endAcquire();
//	return TRUE;
//}
/////////////////////////////////////////////////////////////////////////////
BOOL RomerArm::DoPipeProbeMeas()
{
	if(!ConnectedToArm)
	{
		AfxMessageBox(Tr("No Simulation for Pipe Probe Measurement."), MB_ICONEXCLAMATION);
		return FALSE;
	}
	//if(!ValidateCurrentProbe())
	//	return FALSE;

	StartPipeProbe();
	// TESTING programmatic start scan...
	//TRACE0("CimCoreArm::StartScannerMeas - start scan");
	//int		command;
	//long	Written;
	//command = SCANWORKS_START_SCAN;
	//if(!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	//{
	//	PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
	//	return false;
	//}

	TRACE0("pop pipe probe dlg...\n");
	DPipeProbeMeas dlg;
	BOOL ret = dlg.DoModal() == IDOK;

	StopPipeProbe();
	// TESTING programmatic stop scan...
	//TRACE0("CimCoreArm::StopScannerMeas - stop scan");
	//command = SCANWORKS_STOP_SCAN;
	//if(!TCPIPWrite(hSocket, (LPVOID) &command, (long) sizeof(command), &Written))
	//{
	//	PostUpdateScanDlg(_T("ScanWorks unable to stop model."));
	//	return false;
	//}

	return ret;
}
bool RomerArm::StartPipeProbe()
{
	if(!ConnectedToArm)
		return false;

	return true;	// 2009.01.27 - pending support from CimCore

	// check the probe ID

	// may be a license check later...
	//if(!app.saarm.scannerLicenseValid)
	//{
	//	AfxMessageBox(_T("No Scanner License found.  Please contact your SA sales representative."), MB_ICONHAND);
	//	return FALSE;
	//}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.StopQueryTimer();
	//Delay(0.5,TRUE);
	//d.StartScannerTimer(0.25);

	TRACE0("Romer::StartPipeProbe...");

	// set callback params for rds and kick off the thread!
	recdata.dwRecordFunction = RDS_RECORD_ON_TUBE_PROBE_ACTIVITY; //RDS_RECORD_ON_TUBE_PROBE_ACTIVITY; //RDS_RECORD_ON_BUTTON_TWO_GOING_ON;	// RDS_RECORD_USING_SCAN_MODE;
	recdata.lSoundFrequency = 0;
	recdata.lSoundDuration = 0;
	recdata.pfnCallbackProbePositionEx = RecordDataCallback;
	recdata.dwWnd = (DWORD)app.m_pMainWnd;
	m_enControlOne = CONTROL_ON;

	if(LaunchThreadOne())
		ResumeThreadOne();
	return true;
}
bool RomerArm::StopPipeProbe()
{
	TRACE0("RomerArm::StopPipeProbe\n");

	return true;	// 2009.01.27 - pending support from CimCore

	KillThreadOne();
	Delay(0.2,TRUE);
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//d.StopScannerTimer();
	//Delay(0.5,TRUE);
	d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
	return true;
}
/////////////////////////////////////////////////////////////////////////////
// WAROPF thread stuff...
static BOOL ThreadProcOne( RomerArm *arm)
{
	// This is the main thread where all of the arm communication is done
	// using the 'WaitAndRecord' function. It follows the operating mode
	// set by SetMode. It is continuously running and syncs itself with
	// the arm communication speed of up to 70 points per second. 
	// 'WaitAndRecord' uses the callback function at the bottom of this file
	// to update the screen.

	DWORD dwResult;

	// wait until the Initial Suspension is off
	TRACE0("Wait for arm->m_hOnOffEventOne...\n");
	::WaitForSingleObject( arm->m_hOnOffEventOne, INFINITE );
	::SetEvent( arm->m_hThreadResumedEventOne );
	TRACE0("now go into the thread loop...\n");

	// Main loop.
	while( arm->m_enControlOne != arm->CONTROL_OFF )
	{
		if (::WaitForSingleObject( arm->m_hOnOffEventOne, 0 ) == WAIT_OBJECT_0 )
		{
			// release main thread
			::SetEvent( arm->m_hSuspendedEventOne );
			::WaitForSingleObject( arm->m_hOnOffEventOne, INFINITE );
			::SetEvent( arm->m_hThreadResumedEventOne );
		}

		// this is where the main arm communication action is initiated.
		dwResult = RDS_WaitAndRecordOnProbeFunctionEx(&arm->recdata);

		if(dwResult&RDS_BUTTON_ONE_WENT_ON)
		{
			TRACE0("b1 went on...\n");
		}
		if(dwResult&RDS_BUTTON_TWO_WENT_ON)
		{
			TRACE0("b2 went on...\n");
			//arm->StopScannerMeas();
		}
		if(dwResult&RDS_BUTTON_THREE_WENT_ON)
		{
			TRACE0("b3 went on...\n");
		}

		if(dwResult&RDS_TUBE_PROBE_BEAM_ONE_BLOCKED_MOVING_FORWARD)
		{
			arm->ProcessPipeTransform(_T("B1_Blocked_Forward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_TWO_BLOCKED_MOVING_FORWARD)
		{
			arm->ProcessPipeTransform(_T("B2_Blocked_Forward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_ONE_UNBLOCKED_MOVING_FORWARD) 
		{
			arm->ProcessPipeTransform(_T("B1_Unblocked_Forward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_TWO_UNBLOCKED_MOVING_FORWARD) 
		{
			arm->ProcessPipeTransform(_T("B2_Unblocked_Forward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_ONE_BLOCKED_MOVING_BACKWARD) 
		{
			arm->ProcessPipeTransform(_T("B1_Blocked_Backward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_TWO_BLOCKED_MOVING_BACKWARD)
		{
			arm->ProcessPipeTransform(_T("B2_Blocked_Backward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_ONE_UNBLOCKED_MOVING_BACKWARD)
		{
			arm->ProcessPipeTransform(_T("B1_Unblocked_Backward"));
		}
		else if(dwResult&RDS_TUBE_PROBE_BEAM_TWO_UNBLOCKED_MOVING_BACKWARD)
		{
			arm->ProcessPipeTransform(_T("B2_Unblocked_Backward"));
		}

		// Check for an error.
		if(dwResult&RDS_RECORD_ERROR)
		{
			DWORD save = ::GetLastError();
			TCHAR szMsg [MAX_PATH];
			RDS_GetErrorMessage(szMsg, MAX_PATH);
			TRACE1("error from winrds: %s\n", szMsg);
			//testBenchDlg->m_sErrorMsg = szMsg;
			//if(save == WINRDS_ERROR_ARM_OVER_LIMIT)
			//{
			//	RDS_OutputSpeakerSound( 100, 100 );
			//}
			arm->recdata.dwRecordFunction = RDS_RECORD_NOW;
			RDS_WaitAndRecordOnProbeFunctionEx(&arm->recdata);	
		}
		//arm->ResetCallBackClocks();

		::Sleep(40); // Let other threads run
	} // MAIN loop.
	TRACE0("Thread exiting...\n");
	return 0;

}	// ThreadProcOne

/////////////////////////////////////////////////////////////////////////////

BOOL RomerArm::ResumeThreadOne(void)
{
	// boiler plate stuff
	if ( m_enControlOne == CONTROL_SUSPENDED)
	{
		m_enControlOne = CONTROL_ON;
		::SetEvent( m_hOnOffEventOne );		// signal Mouse Thread
		::WaitForSingleObject( m_hThreadResumedEventOne, INFINITE);
		//m_bNewRecordData = FALSE;
	}

	return TRUE;

}	// ResumeThreadOne

/////////////////////////////////////////////////////////////////////////////

BOOL RomerArm::SuspendThreadOne()
{
	// boiler plate stuff
	if ( m_enControlOne != CONTROL_ON)
		return TRUE;

	recdata.dwRecordFunction = RDS_STOP_RECORDING;
	m_enControlOne = CONTROL_SUSPENDED;
	::SetEvent( m_hOnOffEventOne );		// signal Mouse Thread
	if ( ::WaitForSingleObject( m_hSuspendedEventOne, 60000 ) !=
		WAIT_OBJECT_0 )
	{
		KillThreadOne();
		return FALSE;
	}

	return TRUE;

}	// SuspendMouseThreadOne

/////////////////////////////////////////////////////////////////////////////

BOOL RomerArm::LaunchThreadOne()
{
	// boiler plate stuff
	if ( m_hThreadOne != NULL)		// if Thread already running
		KillThreadOne();

	// create 3 interthread synchronization event objects
	m_hOnOffEventOne = ::CreateEvent( NULL,		// no security
		FALSE,		// auto-reset
		FALSE,		// initially unsignalled
		NULL );		// no name

	m_hSuspendedEventOne = ::CreateEvent( NULL,		// no security
		FALSE,		// auto-reset
		FALSE,		// initially unsignalled
		NULL );		// no name

	m_hThreadResumedEventOne = ::CreateEvent( NULL,		// no security
		FALSE,		// auto-reset
		FALSE,		// initially unsignalled
		NULL );		// no name

	m_enControlOne = CONTROL_SUSPENDED;	// start with the mouse suspended
	//m_bNewRecordData = FALSE;

	DWORD dwThreadID;
	// create a secondary thread to do the I/O
	if ( NULL == ( m_hThreadOne =
		::CreateThread( NULL,		// no security atts
		0,			// default stack
		( LPTHREAD_START_ROUTINE)ThreadProcOne,
		this,		// argument: pointer to object
		0,			// creation flags
		&dwThreadID )))
	{
		TRACE0("****Could NOT Launch Thread!*******\n");
		::CloseHandle( m_hOnOffEventOne );
		::CloseHandle( m_hSuspendedEventOne );
		::CloseHandle( m_hThreadResumedEventOne );
		m_enControlOne = CONTROL_OFF;
		return FALSE;
	}
	TRACE0("Thread Launched.\n");
	return TRUE;

}	// LaunchThreadOne

/////////////////////////////////////////////////////////////////////////////

void RomerArm::KillThreadOne()
{
	// boiler plate stuff
	if ( m_hThreadOne == NULL)		// if Thread already not running
		return;

	recdata.dwRecordFunction = RDS_STOP_RECORDING;
	m_enControlOne = CONTROL_OFF;

	::TerminateThread( m_hThreadOne, 0 );
	::CloseHandle( m_hOnOffEventOne );
	::CloseHandle( m_hSuspendedEventOne );
	::CloseHandle( m_hThreadResumedEventOne );
	::CloseHandle( m_hThreadOne );

	m_enControlOne = CONTROL_OFF;
	m_hThreadOne = NULL;

}	// KillThreadOne

/////////////////////////////////////////////////////////////////////////////
void RomerArm::Test()
{
	RDS_VerifyProbeRecording( app.m_pMainWnd->m_hWnd );
}
/////////////////////////////////////////////////////////////////////////////
// Articulator
bool RomerArm::ReadArmTransform(Transform& t)
{
	if( !ConnectedToArm )
		return ArmBase::ReadArmTransform(t);
	TRACE0("RomerArm::ReadArmTransform called...\n");
	return false;
}
bool RomerArm::DriveToArmTransform(const Transform& t)
{
	if( !ConnectedToArm )
		return ArmBase::DriveToArmTransform(t);
	TRACE0("RomerArm::DriveToArmTransform called, transform:\n");
	t.Debug();
	return false;
}
bool RomerArm::ReadJointValues(GenVec& jv)
{
	if( !ConnectedToArm )
		return ArmBase::ReadJointValues(jv);
	TRACE0("RomerArm::ReadJointValues called...\n");
	return false;
}
bool RomerArm::DriveToJointValues(const GenVec& jv)
{
	if( !ConnectedToArm )
		return ArmBase::DriveToJointValues(jv);
	TRACE0("RomerArm::DriveToJointValues called, values:\n");
	for(int i=0;i<jv.nx;i++)
		TRACE(_T("		%f \n"),jv.x[i]);
	return false;
}
bool RomerArm::DriveToCommaDelimitedJointString(const CString& jv)
{
	if( !ConnectedToArm )
		return ArmBase::DriveToCommaDelimitedJointString(jv);
	return false;
}
bool RomerArm::DriveToNamedJtSet(const CString& name)
{
	if( !ConnectedToArm )
		return ArmBase::DriveToNamedJtSet(name);
	return false;
}
//////////////////////////////////////////////////
