// RomerArm.h: interface for the RomerArm class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ROMERARM_H__E446419F_1E04_42B3_A1DD_00757EA53DEF__INCLUDED_)
#define AFX_ROMERARM_H__E446419F_1E04_42B3_A1DD_00757EA53DEF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ArmBase.h"
#include "rdsapi.h"

class RomerArm : public ArmBase  
{
public:
	RomerArm();
	virtual ~RomerArm();

	friend class CimCoreWindowsMessenger;

	virtual bool ConnectToArm(const int dof);
	virtual bool PopCommSettingsDlg();
	virtual BOOL DisconnectFromHardware();
	virtual BOOL GetAllData();
	virtual BOOL GetPos();
	virtual BOOL GetJoints(bool simulate=true);
	virtual int GetButtonStates();
	virtual BOOL CheckButton(int but, int* duration);
	virtual BOOL GetTemp();
	virtual int GetProbeNumber();
	virtual int GetButtonCount();

	virtual void SetProbeNumber(int ProbeNum);
	virtual BOOL ValidateCurrentProbe();
	virtual void SetProbeAutoDetect(BOOL ad);

	virtual BOOL SetVolumeLevel( int VolumeLevel );
	virtual void OutputSpeakerSound( int frequency, int duration );
	virtual void ArmBeep(int sound);

	virtual BOOL IsRemoteMouseEnabled();
	virtual void ToggleRemoteMouse();
	virtual void RemoteMouse(BOOL flag);

	void ProcessPipeTransform(const CString& name);

	//////////////////////////////////////////////////
	// Scanner stuff
	virtual BOOL ScannerSettings();	// 2004.09.03
	virtual BOOL StartScannerMeas();	// 2004.09.03
	virtual BOOL StopScannerMeas();	// 2004.09.07
	// Perceptron specific
	Transform scannerXform;
	bool SetScannerRegKey(bool reportError=false);	// 2005.11.03 - requested code changes from Perceptron - ScanWorks now 'shares' the WinRDS connection nicely!
	bool SetScanWorksUnitsToInches();	// 2005.10.03 - added in ScanWorks v. 4.4
	bool ScanWorksCheckForProbe();		// 2008.07.21 - They say, "We are moving away from using only probe number 8 and this is probably the safest way to detect if the scanner is active."
	bool StartModel();
	bool StopModel();
	void ResetSocket();
	bool ScanWorksMoverConnect();
	bool ScanWorksMoverDisconnect();
	void ShowScanWorksToolBar();
	void HideScanWorksToolBar();
	void CleanUpPerceptron();
	bool ReConnectToArm();
	bool SendBufferedCloudDataAndReconnect();
	virtual void PostUpdateScanDlg(CString msg);
	void OnUpdateScanDlg();
	void CloseScanDlg();

	virtual BOOL DoPipeProbeMeas();
	virtual bool StartPipeProbe();
	virtual bool StopPipeProbe();

	//////////////////////////////////////////////////
	// RDS WAROPF thread stuff...
	RDS_RECORD_DATAEX recdata;
	enum ControlFlag
	{
		CONTROL_OFF,
		CONTROL_ON,
		CONTROL_SUSPENDED,
	};
	HANDLE m_hOnOffEventOne;
	HANDLE m_hSuspendedEventOne;
	HANDLE m_hThreadResumedEventOne;
	HANDLE m_hThreadOne;
	ControlFlag	m_enControlOne;
	BOOL ResumeThreadOne( void);
	BOOL SuspendThreadOne();
	BOOL LaunchThreadOne();
	void KillThreadOne();
	//////////////////////////////////////////////////
	// Articulator
	virtual bool ReadArmTransform(Transform& t);
	virtual bool DriveToArmTransform(const Transform& t);
	virtual bool ReadJointValues(GenVec& jv);
	virtual bool DriveToJointValues(const GenVec& jv);
	virtual bool DriveToCommaDelimitedJointString(const CString& jv);
	virtual bool DriveToNamedJtSet(const CString& name);
	//////////////////////////////////////////////////
	
	virtual BOOL IsRomer();
	virtual BOOL IsFaro();
	virtual BOOL IsFaroUSB();
	virtual BOOL IsFrenchRomer();
	virtual BOOL IsSNL();
	virtual BOOL IsMetrisGarda();
	virtual void Test();

protected:
	CimCoreWindowsMessenger& wm;
	bool initialOneShot;
	CString scanMsg;
	CriticalSectionWrapper& scannerCS;

private:
	bool scannerRegKeySet;
};

#endif // !defined(AFX_ROMERARM_H__E446419F_1E04_42B3_A1DD_00757EA53DEF__INCLUDED_)
