//______________________________________________________________________________

// File: RomErrs.h


#ifndef __ROMERRS_H__		//{
#define __ROMERRS_H__

#define ROMER_ERROR_BASE	0x25145000		// For User codes Bit 29 must be on

enum r6sc_error_labels								// Range: 000 - 1FF
{
	R6SC_ERROR_BASE = ROMER_ERROR_BASE + 0x000,

	R6SC_ERROR_CONTROLLER_ERROR = R6SC_ERROR_BASE,

	R6SC_ERROR_CONNECTION_ALREADY_OPEN = R6SC_ERROR_BASE + 0x100,
	R6SC_ERROR_MEMORY_ALLOCATION,
	R6SC_ERROR_CTS_OFF,
	R6SC_ERROR_CONNECTION_ALREADY_CLOSED,
	R6SC_ERROR_RECEIVE_WAIT_FAILURE,
	R6SC_ERROR_RECEIVE_WAIT_TIMEOUT,
	R6SC_ERROR_SEND_WAIT_FAILURE,
	R6SC_ERROR_SEND_WAIT_TIMEOUT,
	R6SC_ERROR_CONNECTION_NOT_OPEN,
	R6SC_ERROR_DISCONNECTED,

	R6SC_ERROR_MISSING_MSG_HEADER,
	R6SC_ERROR_UNEXPECTED_MSG_LENGTH,
	R6SC_ERROR_INCORRECT_MSG_CHECKSUM,
	R6SC_ERROR_UNRECOGNIZED_MSG,
	R6SC_ERROR_NO_ACTIVE_COUNTER,

	R6SC_ERROR_MAX_ERROR,
};

enum EThernet_error_labels								// Range: 000 - 1FF
{
	ETHERNET_ERROR_BASE = ROMER_ERROR_BASE + 0x200,
	ETHERNET_ERROR_SOCKETINIT=ETHERNET_ERROR_BASE,
	ETHERNET_ERROR_DURING_CONNECTION,
	ETHERNET_ERROR_DURING_DISCONNECTION,
	ETHERNET_MAX_PARAM_PER_COMMAND_REACHED,
	ETHERNET_MAX_COMMAND_PER_MACROCOMMAND_REACHED,
	ETHERNET_COMMAND_ALLOCATION_ERROR,
	ETHERNET_ERROR_DURING_ARM_STRUCTURE_INITIALISATION,
	ETHERNET_ERROR_ANSWER_SIZE_NOT_LOGICAL,
	ETHERNET_ERROR_ANSWER_DO_NOT_CONTAINS_ACK_OR_NACK,
	ETHERNET_ERROR_COMMAND_IN_THE_ANSWER_IS_DIFFERENT_OF_THE_QUESTION,
	ETHERNET_ERROR_DIFFERENCE_BETWEEN_THE_NUMBER_OF_QUESTIONS_AND_ANSWERS,
	ETHERNET_ERROR_DURING_RECEPTION,
	ETHERNET_ERROR_WHEN_SENDING,
	ETHERNET_ERROR_HEX_FILE_INCCORECT,
};


enum Maths_error_labels								// Range: 800 - FFF
{
	MATHS_ERROR_BASE = ROMER_ERROR_BASE + 0x300,
	MATHS_NULL_VECTOR=MATHS_ERROR_BASE,
	MATHS_DROITE_AND_PLANE_ARE_PARALLELE,
	MATHS_DROITES_PARALLELES,
	MATHS_DROITE_PERPENDICULAIRE_AU_PLAN,
};

enum LaserProbe_error_labels								// Range: 000 - 1FF
{
	LP_NO_ERROR=0,
	LP_ERROR_BASE = ROMER_ERROR_BASE + 0x700,				//0x25145700
	LP_DIGITIZER_ALLOCATION_ERROR = LP_ERROR_BASE,			//0x25145701
	LP_SYSTEM_ALLOCATION_ERROR,								//0x25145702
	LP_APPLICATION_ALLOCATION_ERROR,						//0x25145703
	LP_BUFFER1_ALLOCATION_ERROR,							//0x25145704
	LP_BUFFER2_ALLOCATION_ERROR,							//0x25145705
	LP_DISPLAY_ALLOCATION_ERROR,							//0x25145706
	LP_GRAPHIQUECONTEXT_ALLOCATION_ERROR,					//0x25145707
	LP_CONTROL_BOARD_COMMAND_ERROR,							//0x25145708
	LP_CONTROL_BOARD_TIMEOUT,								//0x25145709
	LP_END_OF_GRAB_NOT_REACH,								//0x2514570A
	LP_CALIBRATION_ALLOCATION_ERROR,						//0x2514570B
	LP_LIBRARY_NOT_STARTED,									//0x2514570C
	LP_ETHERTNET_PROTOCOL_ISNULL,							//0x2514570D
	LP_NO_ANSWER_FOR_EXTERNAL_COMMAND,						//0x2514570E
	LP_NACK_RETURNED,										//0x2514570F
	LP_ISO_TRANSFERT_PROTOCOL_NOT_STARTED,					//0x25145710
	LP_NO_CAMERA_FOUND,										//0x25145711
	LP_BUS_THREAD_NOT_STARTED,								//0x25145712
	LP_SEVERAL_BOARDS_FOUNDS,								//0x25145713
};

enum HASP_error_labels								// Range: 000 - 1FF
{
   HASPERRL_SUCCESS =  0,                        /* Operation successful                                                           */
   HASPERRL_INVALID_SERVICE = ROMER_ERROR_BASE + 0x500 -999,              /* HASP Invalid Service                                                           */
   HASPERRL_CANT_FREE_DOSMEM = ROMER_ERROR_BASE + 0x500 -121,             /* Cannot free DOS Memory                                                         */
   HASPERRL_CANT_ALLOC_DOSMEM = ROMER_ERROR_BASE + 0x500 -120,            /* Cannot allocate DOS Memory                                                     */
   HASPERRL_DOS_CANT_CLOSE_HDD = ROMER_ERROR_BASE + 0x500 -112,           /* Cannot close the HASP Device Driver                                            */
   HASPERRL_DOS_CANT_READ_HDD = ROMER_ERROR_BASE + 0x500 -111,            /* Cannot read the HASP Device Driver                                             */
   HASPERRL_DOS_CANT_OPEN_HDD = ROMER_ERROR_BASE + 0x500 -110,            /* Cannot open the HASP Device Driver                                             */
   HASPERRL_CANT_CLOSE_HDD = ROMER_ERROR_BASE + 0x500 -102,               /* Cannot close the HASP Device Driver                                            */
   HASPERRL_CANT_READ_HDD = ROMER_ERROR_BASE + 0x500 -101,                /* Cannot read the HASP Device Driver                                             */
   HASPERRL_CANT_OPEN_HDD = ROMER_ERROR_BASE + 0x500 -100,                /* Cannot open the HASP Device Driver                                             */
   HASPERRL_NOT_A_TIMEHASP = ROMER_ERROR_BASE + 0x500 -29,                /* A HASP was found but it is not a TimeHASP                                      */
   HASPERRL_TH_INVALID_PASSWORDS = ROMER_ERROR_BASE + 0x500 -28,          /* A HASP with specified passwords was not found                                  */
   HASPERRL_TH_TIMEOUT = ROMER_ERROR_BASE + 0x500 -27,                    /* Timeout - Write operation failed                                               */
   HASPERRL_TH_INVALID_ADDRESS = ROMER_ERROR_BASE + 0x500 -26,            /* Invalid address - Address is not in 0 - 15                                     */
   HASPERRL_TH_INVALID_HOUR = ROMER_ERROR_BASE + 0x500 -25,               /* Invalid Hour                                                                   */
   HASPERRL_TH_INVALID_MINUTE = ROMER_ERROR_BASE + 0x500 -24,             /* Invalid Minute                                                                 */
   HASPERRL_TH_INVALID_SECOND = ROMER_ERROR_BASE + 0x500 -23,             /* Invalid Second                                                                 */
   HASPERRL_TH_INVALID_YEAR = ROMER_ERROR_BASE + 0x500 -22,               /* Invalid year                                                                   */
   HASPERRL_TH_INVALID_MONTH = ROMER_ERROR_BASE + 0x500 -21,              /* Invalid month                                                                  */
   HASPERRL_TH_INVALID_DAY =ROMER_ERROR_BASE + 0x500 -20,                /* Invalid day                                                                    */
   HASPERRL_INVALID_PARAMETER = ROMER_ERROR_BASE + 0x500 -12,             /* Invalid parameter                                                              */
   HASPERRL_TS_SP3_FOUND = ROMER_ERROR_BASE + 0x500 -11,                  /* Terminal Server under SP3 is not supported                                     */
   HASPERRL_TS_FOUND = ROMER_ERROR_BASE + 0x500 -10,                      /* Terminal Server was found.                                                     */
   HASPERRL_INVALID_POINTER = ROMER_ERROR_BASE + 0x500 -9,                /* Invalid pointer used by Encode Data                                            */
   HASPERRL_HARDWARE_NOT_SUPPORTED = ROMER_ERROR_BASE + 0x500 -8,         /* The hardware does not support the service                                      */
   HASPERRL_DATA_TOO_SHORT = ROMER_ERROR_BASE + 0x500 -7,                 /* The data length is too short                                                   */
   HASPERRL_PORT_BUSY = ROMER_ERROR_BASE + 0x500 -6,                      /* Parallel port is busy.                                                         */
   HASPERRL_MH_WRITE_FAIL = ROMER_ERROR_BASE + 0x500 -5,                  /* Unsuccessful Write operation                                                   */
   HASPERRL_NOT_A_MEMOHASP = ROMER_ERROR_BASE + 0x500 -4,                 /* A HASP was found but it is not a MemoHASP                                      */
   HASPERRL_HASP_NOT_FOUND       = ROMER_ERROR_BASE + 0x500 -3,           /* A HASP with specified passwords was not found                                  */
   HASPERRL_MH_INVALID_ADDRESS = ROMER_ERROR_BASE + 0x500 -2,             /* Address out of range                                                           */
   HASPERRL_MH_TIMEOUT = ROMER_ERROR_BASE + 0x500 -1,                     /* Timeout - Write operation failed                                               */
   HASPERRL_NO_PROTOCOLS = ROMER_ERROR_BASE + 0x500 +1,                    /* IPX, NetBIOS, or TCP/IP protocols have not been installed properly.            */
   HASPERRL_NO_SOCKET_NUMBER = ROMER_ERROR_BASE + 0x500+ 2,                /* Communication Error - unable to get the socket number (TCP/IP, IPX)            */
   HASPERRL_COMM_ERROR = ROMER_ERROR_BASE + 0x500 +3,                      /* Communication Error.                                                           */
   HASPERRL_NO_NHLM = ROMER_ERROR_BASE + 0x500 +4,                         /* No NetHASP License Manager was found.                                          */
   HASPERRL_NO_NHLM_ADDRFILE = ROMER_ERROR_BASE + 0x500+ 5,                /* Cannot read NetHASP License Manager address file                               */
   HASPERRL_CANT_CLOSE_NHLM_ADDRFILE = ROMER_ERROR_BASE + 0x500+ 6,        /* Cannot close NetHASP License Manager address file                              */
   HASPERRL_CANT_SEND_PACKET = ROMER_ERROR_BASE + 0x500+ 7,                /* Communication error - failed to send packet (IPX, NetBIOS                      */
   HASPERRL_SILENT_NHLM = ROMER_ERROR_BASE + 0x500 +8,                     /* No answer from the NetHASP License Manager.                                    */
   HASPERRL_NO_LOGIN = ROMER_ERROR_BASE + 0x500 +10,                       /* Service requested before LOGIN                                                 */
   HASPERRL_ADAPTER_ERROR = ROMER_ERROR_BASE + 0x500 +11,                  /* NetBIOS: Communication error - adapter error                                   */
   HASPERRL_NO_ACTIVE_NHLM = ROMER_ERROR_BASE + 0x500 +15,                 /* No active NetHASP Licence Manager was found                                    */
   HASPERRL_SSBN_FAILED = ROMER_ERROR_BASE + 0x500 +18,                    /* Cannot perform LOGIN - SetServerByName failed                                  */
   HASPERRL_NHCF_SYNTAX_ERROR = ROMER_ERROR_BASE + 0x500 +19,              /* NetHASP configuration file syntax error                                        */
   HASPERRL_NHCF_GENERIC_ERROR = ROMER_ERROR_BASE + 0x500+ 20,             /* Error handling NetHASP configuration file                                      */
   HASPERRL_NH_ENOMEM = ROMER_ERROR_BASE + 0x500 +21,                      /* Memory allocation error                                                        */
   HASPERRL_NH_CANT_FREE_MEM = ROMER_ERROR_BASE + 0x500 +22,               /* Memory release error                                                           */
   HASPERRL_NH_INVALID_ADDRESS = ROMER_ERROR_BASE + 0x500 +23,             /* Invalid NetHASP memory address                                                 */
   HASPERRL_NH_ENCDEC_ERR = ROMER_ERROR_BASE + 0x500 +24,                  /* Error trying to Encrypt/Decrypt                                                */
   HASPERRL_CANT_LOAD_WINSOCK = ROMER_ERROR_BASE + 0x500 +25,              /* TCP/IP: failed to load WINSOCK.DLL                                             */
   HASPERRL_CANT_UNLOAD_WINSOCK = ROMER_ERROR_BASE + 0x500 +26,            /* TCP/IP: failed to unload WINSOCK.DLL                                           */
   HASPERRL_WINSOCK_ERROR = ROMER_ERROR_BASE + 0x500 +28,                  /* TCP/IP: WINSOCK.DLL startup error                                              */
   HASPERRL_CANT_CLOSE_SOCKET = ROMER_ERROR_BASE + 0x500 +30,              /* TCP/IP: Failed to close socket.                                                */
   HASPERRL_SETPROTOCOL_FAILED = ROMER_ERROR_BASE + 0x500 +33,             /* SetProtocol service requested without performing LOGOUT                        */
   HASPERRL_NH_NOT_SUPPORTED = ROMER_ERROR_BASE + 0x500 +40,               /* NetHASP services are not supported                                             */
   HASPERRL_NH_HASPNOTFOUND = ROMER_ERROR_BASE + 0x500 +129,               /* NetHASP key is not connected to the NetHASP Licence Manager                    */
   HASPERRL_INVALID_PROGNUM = ROMER_ERROR_BASE + 0x500 +130,               /* Program Number is not in the Program List of the NetHASP memory                */
   HASPERRL_NH_READ_ERROR = ROMER_ERROR_BASE + 0x500 +131,                 /* Error reading NetHASP memory                                                   */
   HASPERRL_NH_WRITE_ERROR = ROMER_ERROR_BASE + 0x500 +132,                /* Error writing NetHASP memory                                                   */
   HASPERRL_NO_MORE_STATIONS = ROMER_ERROR_BASE + 0x500 +133,              /* Number of stations exceeded                                                    */
   HASPERRL_NO_MORE_ACTIVATIONS = ROMER_ERROR_BASE + 0x500 +134,           /* Number of activations exceeded                                                 */
   HASPERRL_LOGOUT_BEFORE_LOGIN = ROMER_ERROR_BASE + 0x500 +135,           /* LOGOUT called without LOGIN                                                    */
   HASPERRL_NHLM_BUSY = ROMER_ERROR_BASE + 0x500 +136,                     /* NetHASP Licence Manager is busy                                                */
   HASPERRL_NHLM_FULL = ROMER_ERROR_BASE + 0x500 +137,                     /* No space in NetHASP Log Table                                                  */
   HASPERRL_NH_INTERNAL_ERROR =ROMER_ERROR_BASE + 0x500 + 138,             /* NetHASP Internal error                                                         */
   HASPERRL_NHLM_CRASHED = ROMER_ERROR_BASE + 0x500 +139,                  /* NetHASP Licence Manager crashed and reactivated                                */
   HASPERRL_NHLM_UNSUPPORTED_NETWORK = ROMER_ERROR_BASE + 0x500 +140,      /* NetHASP Licence Manager does not support the station's network                 */
   HASPERRL_NH_INVALID_SERVICE_2 = ROMER_ERROR_BASE + 0x500 +141,          /* Invalid Service NetHASP Licence Manager does not support the station's network */
   HASPERRL_NHCF_INVALID_NHLM = ROMER_ERROR_BASE + 0x500 +142,             /* Invalid NetHASP Licence Manager name in configuration file                     */
   HASPERRL_SSBN_INVALID_NHLM = ROMER_ERROR_BASE + 0x500 +150,             /* Invalid NetHASP Licence Manager name in SetServerByName                        */
   HASPERRL_ENC_ERROR_NHLM = ROMER_ERROR_BASE + 0x500 +152,                /* Error trying to encrypt by the LM                                              */
   HASPERRL_DEC_ERROR_NHLM = ROMER_ERROR_BASE + 0x500 +153,                /* Error trying to decrypt by the LM                                              */
   HASPERRL_OLD_LM_VERSION_NHLM = ROMER_ERROR_BASE + 0x500 +155            /* LM old version was found                                                       */

};
enum Gds_error_labels								// Range: 800 - FFF
{
	SERIAL_ERROR_BASE = ROMER_ERROR_BASE + 0x000,
	SERIAL_WRITTING_ERROR,					//0x25145001
	SERIAL_CONFIGURATION_MASK_ERROR,		//0x25145002
	SERIAL_CONFIGURATION_TIMEOUTS_ERROR,	//0x25145003
	SERIAL_CONFIGURATION_STATE_ERROR,						//0x25145004
	SERIAL_WAIT_ERROR,										//0x25145005	
	SERIAL_TIMEOUT_ERROR,									//0x25145006
	SERIAL_READING_ERROR,									//0x25145007
	SERIAL_ANSWERCRC16_ERROR,								//0x25145008
	SERIAL_ANSWERCOMMAND_ERROR,								//0x25145009
	SERIAL_DECODAGE_ERROR,									//0x2514500A
	SERIAL_NUMCOUNTER_ERROR,								//0x2514500B
	SERIAL_NUMBUTTON_ERROR,									//0x2514500C
	SERIAL_EVENTDIFFERENT_ERROR,							//0x2514500D
	SERIAL_B1B2SAMEEVENT_ERROR,								//0x2514500E
	SERIAL_EVENTNOTRECOGNIZED_ERROR,						//0x2514500F
	SERIAL_FREQUENCYVALUE_ERROR,							//0x25145010
	SERIAL_DURATIONVALUE_ERROR,								//0x25145011
	SERIAL_VOLUMEVALUE_ERROR,								//0x25145012
	SERIAL_ANSWER_ERROR,									//0x25145013
	SERIAL_EVENT_ERROR,										//0x25145014
	SERIAL_LOSTPOINT_ERROR,									//0x25145015
	GDS_ERROR_BASE = ROMER_ERROR_BASE + 0x800,	
	GDS_NO_COMMUNCATION_ESTABLISHED= GDS_ERROR_BASE,		//0x25145800
	GDS_ERROR_PROBE_FILE_NOT_OPENED,						//0x25145801
	GDS_ERROR_PROBE_FILE_FORMAT,							//0x25145802
	GDS_ERROR_ARMSPECS_FILE_NOT_OPENED,						//0x25145803
	GDS_ERROR_ARMSPECS_FILE_FORMAT,							//0x25145804
	GDS_ERROR_RESET_CANCELLED,								//0x25145805
	GDS_ERROR_MOUSE_SUSPENSION,								//0x25145806
	GDS_ERROR_RESET_NOT_DONE,								//0x25145807
	GDS_ERROR_CONNECTION_NOT_ESTABLISHED,					//0x25145808
	GDS_ERROR_CONNECTION_ALREADY_ESTABLISHED,				//0x25145809
	GDS_ERROR_PROBE_ID_MISMATCH,							//0x2514580a
	GDS_ERROR_ENCODER_INDEX,								//0x2514580b
	GDS_INCORRECT_PASSWORD,									//0x2514580c
	GDS_MECHANICAL_STOP,									//0x2514580d
	GDS_INCORRECT_SID,										//0x2514580e
	GDS_ERROR_SID_NOT_FOUND,								//0x2514580f
	GDS_PROTECTION_FILE_NOT_FOUND,							//0x25145810
	GDS_PROTECTION_FILE_ERROR,								//0x25145811
	GDS_OUT_OF_LIMITS,										//0x25145812
	GDS_CHECK_YOUR_SYSTEM_DATE,								//0x25145813
	GDS_INVALIDE_USER,										//0x25145814
	GDS_OPTION_NOT_FOUND,									//0x25145815
	GDS_PROTECTION_ERROR,									//0x25145816
	GDS_PROTECTION_OVERLAPPED,								//0x25145817
	GDS_INTERPOLATION_FILE_TOO_LONG,						//0x25145818
	GDS_INTERPOLATION_FILE_NOT_FOUND,						//0x25145819
	GDS_ACTIVEREFRENCE_NOT_FOUND,							//0x2514581a
	GDS_BOARD_NOT_FOUND,									//0x2514581b
	GDS_REFRENCEACTION_NOT_REFERENCED,						//0x2514581c
	GDS_ERROR_OPEN_FILE,									//0x2514581d
	GDSC_ERROR_FILE_NOT_OPENED,								//0x2514581e
	GDSC_ERROR_TYPE_NOT_REFERENCED,							//0x2514581f
	GDSC_ERROR_PROBE_FILE_NOT_OPENED,						//0x25145820
	GDSC_ERROR_INCORRECT_RELATION_FILE,						//0x25145821
	GDSC_ERROR_ALLOCATION_MEMORY1,							//0x25145822
	GDSC_ERROR_ALLOCATION_MEMORY_POSITION,					//0x25145823
	GDSC_ERROR_NO_CORRECTION_FOUND,							//0x25145824
	GDSC_ERROR_COUNTERS_FILE_NOT_OPENED,					//0x25145825
	GDSC_ERROR_CAPACITY_MEMORY_POSITION,					//0x25145826
	GDSC_CANCELED_BY_USER,									//0x25145827
	GDSC_ERROR_ALLOCATION_MEMORY_REFERENCE,					//0x25145828
	GDSC_ERROR_CAPACITY_MEMORY_REFERENCE,					//0x25145829
	GDSC_ERROR_COMMAND_NOT_UNDERSTAND_BY_BOARD,				//0x2514582a
	GDSC_ERROR_MACHINE_NOT_CORRECTLY_ASSEMBLED,				//0x2514582b
	SERIAL_OPEN_ERROR,										//0x2514582c
	GDS_WRONG_SID_SERIAL_NUMBER,							//0x2514582d
	GDS_WRONG_FILE_FORMAT,
	GDS_WRONG_FILE_VERSION,
};

#endif		//}
