#pragma once
#include "afx.h"

class EasyPadWrapper;
class PadButton;
class PadCalPoint;
class PadRegionRectangular;

class EasyPad :
	public CObject
{
public:
	EasyPad(EasyPadWrapper& _parent);
	EasyPad(); // default ctor for serialization
	EasyPad(EasyPadWrapper& _parent, const EasyPad& source);
	void operator=(const EasyPad& source);
	virtual ~EasyPad(void);

	// build pad...
	void SetBoundary(const Vector& topLeft, const Vector& bottomRight);
	void SetHeights(const double& workingHeight, const double& hotHeight);
	DWORD_PTR AddRectangularButton(const Vector& topLeft, const Vector& bottomRight, const double& pressHeight);

	// inputs...
	bool Calibrate();
	bool IsCalibrated();
	void CheckPosition(Vector instXYZ, double instOffset);

	// attributes
	EasyPadWrapper& parent;

	// utility
	void SetAllButtonZonesDead();
	void SetAllButtonsUnPressed();
	void SetAllButThisButtonUnPressed(const DWORD_PTR& idx);

	Transform instToPad;
	double workingHeight;
	double hotHeight;
	CRect padRect;
	PadRegionRectangular& padBoundary;
	NRKObArray<PadButton> Buttons;
	NRKObArray<PadCalPoint> CalPoints;

	virtual void Serialize(CArchive& ar);
protected:
	DECLARE_SERIAL( EasyPad );

private:
	bool wasInPadZone;
};
