#include "stdafx.h"
#include "EasyPadArmInterface.h"
#include "../DPCMM.h"
#include "../DPCMMDlg.h"
#include "../DGenMeas.h"

EasyPadArmInterface::EasyPadArmInterface(void)
{
	///////////////////////////////////////////
	// build a pad:

	// SA ArmPad, iteration 2 - Todd's first cool looking pad...
	Vector padTL(-0.75, 8.59, 0.0);
	Vector padBR(6.913, -0.5, 0.0);

	Vector growPad(-1.0, 1.0, 0.0);
	growPad.Scale(3.0);
	padTL += growPad;
	padBR -= growPad;

	double workingHeight = 4.0;
	double hotHeight = 2.5;
	SetPadBoundaries(padTL, padBR, workingHeight, hotHeight);

	double currentProbeRadiusInches = app.saarm.Data.TipDia*app.saarm.LocalToInches/2.0;
	double buttonPressHeight = 0.2 + currentProbeRadiusInches;

	Vector bTL(-0.359, 7.280, 0.0);
	Vector bBR(0.900, 6.033, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Discrete Pts"

	bTL = Vector(1.600, 7.280, 0.0);
	bBR = Vector(2.863, 6.033, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Stream Pts"

	bTL = Vector(-0.359, 3.110, 0.0);
	bBR = Vector(0.900, 1.856, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Patch"


	bTL = Vector(-0.359, 4.505, 0.0);
	bBR = Vector(0.900, 3.252, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Pin"

	bTL = Vector(1.600, 4.505, 0.0);
	bBR = Vector(2.863, 3.252, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Hole"

	bTL = Vector(3.562, 4.505, 0.0);
	bBR = Vector(4.818, 3.252, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Slot"


	bTL = Vector(5.510, 7.280, 0.0);
	bBR = Vector(6.763, 6.033, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Line"

	bTL = Vector(-0.359, 5.895, 0.0);
	bBR = Vector(0.900, 4.637, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Circle"

	bTL = Vector(1.600, 5.895, 0.0);
	bBR = Vector(2.863, 4.637, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Plane"


	bTL = Vector(3.562, 5.895, 0.0);
	bBR = Vector(4.818, 4.637, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Sphere"

	bTL = Vector(1.600, 3.110, 0.0);
	bBR = Vector(2.863, 1.856, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Cross"

	bTL = Vector(5.510, 4.505, 0.0);
	bBR = Vector(6.763, 3.252, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Frame"


	bTL = Vector(-1.0, -2.0, 0.0);	// No Batch button in this pad... added to conserve order of meas modes
	bBR = Vector(-2.0, -1.0, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// "Batch"

	bTL = Vector(5.510, 3.110, 0.0);
	bBR = Vector(6.763, 1.856, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Scan"

	bTL = Vector(3.562, 7.280, 0.0);
	bBR = Vector(4.818, 6.033, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Avg Pts"


	bTL = Vector(3.562, 3.110, 0.0);
	bBR = Vector(4.818, 1.856, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Geom Trig"

	bTL = Vector(5.510, 5.895, 0.0);
	bBR = Vector(6.763, 4.637, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "Cone"


	bTL = Vector(0.632, 0.708, 0.0);
	bBR = Vector(1.710, -0.160, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Previous feature

	bTL = Vector(4.711, 0.719, 0.0);
	bBR = Vector(5.795, -0.147, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Next feature


	bTL = Vector(3.333, 0.719, 0.0);
	bBR = Vector(4.423, -0.147, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Accept (Done)

	bTL = Vector(1.990, 0.719, 0.0);
	bBR = Vector(3.077, -0.147, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Cancel

	bTL = Vector(5.589, 1.710, 0.0);
	bBR = Vector(6.676, 0.844, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Delete Last Point


	bTL = Vector(-0.366, 1.731, 0.0);
	bBR = Vector(.55, 0.812, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Top

	bTL = Vector(0.634, 1.731, 0.0);
	bBR = Vector(1.554, 0.812, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Front

	bTL = Vector(1.631, 1.731, 0.0);
	bBR = Vector(2.546, 0.812, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Side

	bTL = Vector(2.633, 1.731, 0.0);
	bBR = Vector(3.54, 0.812, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Oblique

	bTL = Vector(3.730, 1.710, 0.0);
	bBR = Vector(4.819, 0.844, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * Autoscale


	bTL = Vector(0.649, 8.334, 0.0);
	bBR = Vector(1.816, 7.580, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "SA" on the top of the pad

	bTL = Vector(2.009, 8.334, 0.0);
	bBR = Vector(4.594, 7.580, 0.0);
	AddRectangularPadButton(bTL, bBR, buttonPressHeight);	// * "ARM" on the top of the pad
	///////////////////////////////////////////

	// Arm Pad Button, iteration 1...
	//Vector padTL(8.0, 6.75, 0.0);
	//Vector padBR(-0.5, -0.5, 0.0);
	//double workingHeight = 10.0;
	//double hotHeight = 1.5;
	//SetPadBoundaries(padTL, padBR, workingHeight, hotHeight);

	//Vector bTL(7.85094896, 4.86815599, 0.0);
	//Vector bBR(7.04775913, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Discrete Pts"

	//bTL = Vector(7.85094896, 3.79285483, 0.0);
	//bBR = Vector(7.04318679, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Stream Pts"

	//bTL = Vector(7.85094896, 2.70872556, 0.0);
	//bBR = Vector(7.03816882, 1.76363936, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Patch"


	//bTL = Vector(6.90387482, 4.87130801, 0.0);
	//bBR = Vector(6.1610553, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Pin"

	//bTL = Vector(6.90387482, 3.79285483, 0.0);
	//bBR = Vector(6.1610553, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Hole"

	//bTL = Vector(6.90387482, 2.70872556, 0.0);
	//bBR = Vector(6.1610553, 1.76363936, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Slot"


	//bTL = Vector(5.9731075, 4.87130801, 0.0);
	//bBR = Vector(5.22843232, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Line"

	//bTL = Vector(5.9731075, 3.79285483, 0.0);
	//bBR = Vector(5.22843232, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Circle"

	//bTL = Vector(5.9731075, 2.70872556, 0.0);
	//bBR = Vector(5.22843232, 1.76363936, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Plane"


	//bTL = Vector(5.08820969, 4.87130801, 0.0);
	//bBR = Vector(4.32769959, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Sphere"

	//bTL = Vector(5.08820969, 3.79285483, 0.0);
	//bBR = Vector(4.32769959, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Cross"

	//bTL = Vector(5.08820969, 2.70872556, 0.0);
	//bBR = Vector(4.32769959, 1.76363936, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Frame"


	//bTL = Vector(4.15189812, 4.87130801, 0.0);
	//bBR = Vector(3.38817227, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Batch"

	//bTL = Vector(4.15189812, 3.79285483, 0.0);
	//bBR = Vector(3.38817227, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Scan"

	//bTL = Vector(4.15189812, 2.70872556, 0.0);
	//bBR = Vector(3.38817227, 1.76363936, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Avg Pts"


	//bTL = Vector(3.26446212, 4.87130801, 0.0);
	//bBR = Vector(2.48066001, 3.94347743, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Geom Trig"

	//bTL = Vector(3.26446212, 3.79285483, 0.0);
	//bBR = Vector(2.48066001, 2.85667569, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "Cone"


	//bTL = Vector(2.06172923, 4.58521422, 0.0);
	//bBR = Vector(1.14952776, 3.58568631, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Previous feature

	//bTL = Vector(2.06548634, 3.03146868, 0.0);
	//bBR = Vector(1.14438329, 2.02448587, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Next feature


	//bTL = Vector(0.84276045, 5.0638372, 0.0);
	//bBR = Vector(-0.00074648, 4.08923539, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Accept

	//bTL = Vector(0.84276045, 3.84345042, 0.0);
	//bBR = Vector(-0.00074648, 2.87071126, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Cancel

	//bTL = Vector(0.84276045, 2.61266268, 0.0);
	//bBR = Vector(-0.00074648, 1.65362167, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Delete Last Point
	///////////////////////////////////////////

	//// NRK mouse pad...
	//Vector padTL(7.25, 8.5, 0.0);
	//Vector padBR(0.0, 0.0, 0.0);
	//double workingHeight = 10.0;
	//double hotHeight = 0.75;
	//SetPadBoundaries(padTL, padBR, workingHeight, hotHeight);

	//Vector bTL(7.0, 7.5, 0.0);
	//Vector bBR(6.5, 6.25, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "New River Kinematics"

	//bTL = Vector(3.375, 2.75, 0.0);
	//bBR = Vector(2.125, 0.75, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// "SA"

	//bTL = Vector(5.5, 7.5, 0.0);
	//bBR = Vector(4.5, 6.25, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Next feature

	//bTL = Vector(4.0, 7.5, 0.0);
	//bBR = Vector(3.0, 6.25, 0.0);
	//AddRectangularPadButton(bTL, bBR, 0.05);	// Previous feature
	/////////////////////////////////////////////
}

EasyPadArmInterface::~EasyPadArmInterface(void)
{
}

//////////////////////////////////////////////////////////////////////////////////////
// Outputs
void EasyPadArmInterface::OnEnterPadWorkingZone()
{
	//TRACE0("Enter pad working zone\n");
	MessageBeep(MB_ICONINFORMATION);
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.HighlightPadButton();
}
void EasyPadArmInterface::OnEnterPadHotZone()
{
	//TRACE0("Enter pad hot zone\n");
	//MessageBeep(MB_ICONEXCLAMATION);
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//d.HighlightPadButton();
}
void EasyPadArmInterface::OnExitPadZone()
{
	//TRACE0("Exit pad zone\n");
	//MessageBeep(MB_ICONHAND);
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.UnHighlightPadButton();
}
void EasyPadArmInterface::OnEnterButtonZone(DWORD_PTR buttonIdx)
{
	//TRACE1("Enter PAD button %d zone\n", buttonIdx);
	app.pA->SetMeasMode((int)buttonIdx);
	//MessageBeep(MB_ICONEXCLAMATION);
}
void EasyPadArmInterface::OnExitButtonZone(DWORD_PTR buttonIdx)
{
	TRACE1("Exit PAD button %d zone\n", buttonIdx);
	//MessageBeep(MB_ICONSTOP);
}
void EasyPadArmInterface::OnPadButtonHit(DWORD_PTR buttonIdx)
{
	MessageBeep(MB_OK);
	TRACE1("PAD Button %d hit\n", buttonIdx);
	// Arm Pad, iteration 1...
	if((int)buttonIdx <= UPPERPADMODE)
	{
		if(app.measDlg && app.pA->Measuring)
			app.measDlg->EndCancel();	// end the current meas dialog, so a pad button always works
		app.pA->SetMeasMode((int)buttonIdx);

		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

		//SetForegroundWindow(d.GetSafeHwnd());
		app.GetMainWnd()->SetWindowPos(&CWnd::wndTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
		app.GetMainWnd()->SetWindowPos(&CWnd::wndNoTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
		
		d.DoMeasurement();
		//if(buttonIdx != 10 && buttonIdx != 11)	// don't do the old Cross Section or Frame meas with the pad
		//{
		//	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		//	d.DoMeasurement();
		//}
		//else if(buttonIdx == 10 || buttonIdx == 11)	// ...but acknowledge the button hit
		//{
		//	MessageBeep(MB_ICONSTOP);
		//}
	}
	else	// feature, delete last, or Done button hit
	{
		if((int)buttonIdx == UPPERPADMODE+1)	// Previous Feature
		{
			SAIPInspectionNavigation in;
			in.navMode = SAIPInspectionNavigation::NAV_PREV;
			app.saarm.SendInspectionNavigationToSA(in);
		}

		if((int)buttonIdx == UPPERPADMODE+2)	// Next Feature
		{
			SAIPInspectionNavigation in;
			in.navMode = SAIPInspectionNavigation::NAV_NEXT;
			app.saarm.SendInspectionNavigationToSA(in);
		}

		if((int)buttonIdx == UPPERPADMODE+3)	// Accept
		{
			if(app.measDlg && app.pA->Measuring)
			{
				app.measDlg->ProcessAccept();
			}
			else if(!app.pA->Measuring)
			{
				SAIPInspectionNavigation in;
				in.navMode = SAIPInspectionNavigation::NAV_NONE;
				app.saarm.SendInspectionNavigationToSA(in);
			}
		}

		if((int)buttonIdx == UPPERPADMODE+4)	// Cancel
		{
			if(app.measDlg && app.pA->Measuring)
			{
				app.measDlg->EndCancel();
			}
			else if(!app.pA->Measuring)
			{
				SAIPInspectionNavigation in;
				in.navMode = SAIPInspectionNavigation::NAV_NONE;
				app.saarm.SendInspectionNavigationToSA(in);
			}
		}

		if((int)buttonIdx == UPPERPADMODE+5)	// Delete Last Pt.
		{
			app.saarm.RemoveLastMeasurementInSA(true);
		}
		
		if((int)buttonIdx == UPPERPADMODE+6)	// Top
		{
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::ChangeView;
			vc.ViewName.Format(_T("Top"));
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}
	
		if((int)buttonIdx == UPPERPADMODE+7)	// Front
		{
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::ChangeView;
			vc.ViewName.Format(_T("Front"));
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}

		if((int)buttonIdx == UPPERPADMODE+8)	// Side
		{
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::ChangeView;
			vc.ViewName.Format(_T("Obverse"));
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}

		if((int)buttonIdx == UPPERPADMODE+9)	// Oblique
		{
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::ChangeView;
			vc.ViewName.Format(_T("Oblique"));
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}

		if((int)buttonIdx == UPPERPADMODE+10)	// Autoscale
		{
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::Autoscale;
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}

		if((int)buttonIdx == UPPERPADMODE+11)	// "SA" on the top of the pad
		{
			//TRACE(_T("SA\n"));
			SAPPViewControl vc;
			vc.ViewMode = SAPPViewControl::ScheduleRedraw;
			vc.makeForegroundWindow = TRUE;
			app.saarm.SendViewControlToSA(vc);
		}

		if((int)buttonIdx == UPPERPADMODE+12)	// "Arm" on the top of the pad
		{
			//TRACE(_T("Arm\n"));
			app.GetMainWnd()->SetWindowPos(&CWnd::wndTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
			app.GetMainWnd()->SetWindowPos(&CWnd::wndNoTopMost,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
		}
	}
}
void EasyPadArmInterface::SetAllButtonZonesDead()
{
	// be sure to un-hilight them if there's UI showing

	// but let's not do the following, which sets all our meas mode buttons un-pressed - we should just leave the last one pressed, since that shows the 'active' meas mode
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//d.SetAllButtonStatesFalse();
}

void EasyPadArmInterface::Serialize(CArchive& ar)
{
	EasyPadWrapper::Serialize(ar);
	BYTE Schema = 1;
	BYTE LoadSchema = 0;

	if (ar.IsStoring())
	{ 
		ar << Schema;
	}
	else
	{
		ar >> LoadSchema;
	}
} 
IMPLEMENT_SERIAL( EasyPadArmInterface, EasyPadWrapper, 0 );