#include "stdafx.h"
#include "EasyPadWrapper.h"
#include "EasyPad.h"
#include "PadButton.h"
#include "PadCalPoint.h"

EasyPadWrapper::EasyPadWrapper()
: pad(*new EasyPad(*this))
{
	isCalibrating = false;
	isOn = false;
}

EasyPadWrapper::~EasyPadWrapper(void)
{
	delete &pad;
}

void EasyPadWrapper::SetPadBoundaries(const Vector& topLeft, const Vector& bottomRight, const double& workingHeight, const double& hotHeight)
{
	pad.SetBoundary(topLeft, bottomRight);
	pad.SetHeights(workingHeight, hotHeight);
}

DWORD_PTR EasyPadWrapper::AddRectangularPadButton(const Vector& topLeft, const Vector& bottomRight, const double& pressHeight)
{
	return pad.AddRectangularButton(topLeft, bottomRight, pressHeight);
}

//////////////////////////////////////////////////////////////////////////////////////
// Inputs
void EasyPadWrapper::AddCalPt(const Vector& instXYZ, const double& instPlanarOffset)
{
	pad.CalPoints.Add(*new PadCalPoint(instXYZ, instPlanarOffset));
}

void EasyPadWrapper::ClearCalPts()
{
	pad.CalPoints.DeleteAll();
}

void EasyPadWrapper::BeginCal()
{
	pad.CalPoints.DeleteAll();
	isCalibrating = true;
}

void EasyPadWrapper::CheckPosition(Vector instXYZ, double instOffset)
{
	if(isOn)
		pad.CheckPosition(instXYZ, instOffset);
}

bool EasyPadWrapper::CalibratePad()
{
	bool ret = pad.Calibrate();
	isCalibrating = false;
	return ret;
}

bool EasyPadWrapper::IsPadCalibrated()
{
	return pad.IsCalibrated();
}

bool EasyPadWrapper::IsOn()
{
	return isOn;
}
void EasyPadWrapper::SwitchOn()
{
	isOn = true;
}
void EasyPadWrapper::SwitchOff()
{
	isOn = false;
}
//////////////////////////////////////////////////////////////////////////////////////
// Outputs
void EasyPadWrapper::OnEnterPadWorkingZone()
{
	TRACE0("Enter pad working zone\n");
	MessageBeep(MB_ICONINFORMATION);
}
void EasyPadWrapper::OnEnterPadHotZone()
{
	TRACE0("Enter pad hot zone\n");
	MessageBeep(MB_ICONEXCLAMATION);
}
void EasyPadWrapper::OnExitPadZone()
{
	TRACE0("Exit pad zone\n");
	MessageBeep(MB_ICONHAND);
}
void EasyPadWrapper::OnEnterButtonZone(DWORD_PTR buttonIdx)
{
	TRACE1("Enter PAD button %d zone\n", buttonIdx);
	MessageBeep(MB_ICONERROR);
}
void EasyPadWrapper::OnExitButtonZone(DWORD_PTR buttonIdx)
{
	TRACE1("Exit PAD button %d zone\n", buttonIdx);
	MessageBeep(MB_ICONSTOP);
}
void EasyPadWrapper::OnPadButtonHit(DWORD_PTR buttonIdx)
{
	MessageBeep(MB_OK);
	TRACE1("PAD Button %d hit\n", buttonIdx);
	// derived class has button handler
}
void EasyPadWrapper::SetAllButtonZonesDead()
{
	// be sure to un-hilight them if there's UI showing
}

void EasyPadWrapper::Serialize(CArchive& ar) 
{
	BYTE Schema;
	BYTE LoadSchema = 0;
	Schema = 1;
	if (ar.IsStoring())
	{
		ar << Schema;
		pad.Serialize(ar);
	}
	else
	{
		ar >> LoadSchema;
		pad.Serialize(ar);
	}
}
IMPLEMENT_SERIAL( EasyPadWrapper, CObject, 0 );