#include "stdafx.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "DLogon2.h"
#include "GeomTriggerWrapper.h"
#include "DCALCHECK.h"
#include "DCalFaro.h"

SAArm::SAArm(void)
: SockGeom(*new SocketGeometry)
, Data(*new Store)
, units(*new Units)
, screamer(*new AudibleError)
, gp(*new SAIPGeomPlane)
, gl(*new SAIPGeomLine)
, gCyl(*new SAIPGeomCylinder)
, gs(*new SAIPGeomSphere)
, geomTriggerWrap(*new GeomTriggerWrapper)
{
	DOF = 6;
	J.Create(DOF, TRUE);		// set default size of Joint GenVec
	lastJ.Create(DOF, TRUE);
	LocalToInches = 1.0/units.UnitToMeters[DISTUNITS_IN];
	InchesToLocal = units.UnitToMeters[DISTUNITS_IN];
	receivedSignOn = false;
	scannerLicenseValid = FALSE;
	CMMString = _T("PCMM");
	SettingsFileName = _T("SA_Simulation_ArmSettings");
	arm_type = -1;
	waitingForProjectionPlaneFromSA = false;
	invalidateRefPlanesAfterStartup = false;
	waitingForLineFromSA = false;
	waitingForPointFromSA = false;
	waitingForCylinderFromSA = false;
	waitingForSphereFromSA = false;
	scaleFromInstBase = 1.0;
	trackArmTrackerInstSerialNumber = 0;
	waitingForGeomArrayFromSA = false;

	Data.controlPadEnabled = true;	// 2010.09.03 - patent is pending, so ArmPad is out of the closet!

	toolBarDevFlagSet = false;
}
						  
SAArm::~SAArm(void)
{
	delete &Data;
	delete &units;
	delete &SockGeom;
	delete &screamer;
	delete &gp;
	delete &gl;
	delete &gCyl;
	delete &gs;
	delete& geomTriggerWrap;
}

// Connections
/////////////////////////////////////////////////////////////////////////////
BOOL SAArm::DoInitInstanceStuff()	// NO LoadSettings in this overridden version
{
	// locate directories!
	CString Path(AfxGetApp()->m_pszHelpFilePath);
	int slash = Path.ReverseFind('\\');
	ProgramDirectory = Path.Left(slash);
	ProgramDirectory += "\\";

	DataDirectory = ProgramDirectory;
	FindSADir( _T("Data Files"), &DataDirectory );

	PersistenceDirectory = ProgramDirectory;
	FindSADir( _T("Persistence"), &PersistenceDirectory );

	// check if the program is running in a directory OTHER THAN the SA dir.  If so,
	// make the settings file in the same dir as the interface.
	CString SAExeFromRegistry;
	if (FindSAExe(&SAExeFromRegistry))
	{
		// NOTE: SA 64-bit runs in a different directory (x64\Spatial Analyzer64.exe) than the
		//  32-bit version.  Need to account for that here...
		CString SAExeDir, SAExeFile;
		StringHelper::ExtractDirectoryAndFilenameFromPath(SAExeFromRegistry,SAExeDir,SAExeFile);
		// Remove the 64-bit version (if it exists)
		SAExeDir.Replace(_T("x64\\"), _T(""));
		CString exepath = ProgramDirectory /*+  _T("Spatial Analyzer.exe")*/;

		TRACE2("Registry: %s\nCurrent: %s\n",SAExeFromRegistry,exepath);

		// Just compare paths here, don't care about executable naming...
		if ( StringHelper::ComparePaths(exepath,SAExeDir) != 0 )
		{
			TRACE0("ProgramDir used for path\n");
			PersistenceDirectory = ProgramDirectory;
		}
	}

	CheckSpeechSettings();

	return TRUE;
}
BOOL SAArm::PopLogonDialog()
{
	DLogon2 diag;
	diag.SAComputer = SAComputer;
	diag.SAName = SAName;
	diag.m_User = Data.UserName;
	diag.SAPort = SAPort;
	diag.doconnection = Data.doconnection;
	if(diag.DoModal()!=IDOK)
	{
		PostQuitMessage(0);
		return FALSE;
	}
	SAComputer = diag.SAComputer;
	Data.UserName = diag.m_User;
	SAPort = diag.SAPort;
	SAName = diag.SAName;
	Data.doconnection = diag.doconnection;
	//	int space1 = SAName.Find(' ');
	//	int len = SAName.GetLength();
	//	CMMString = SAName.Mid(space1+1, len-space1);
	//	SettingsFileName = CMMString;
	//	LogFileName = CMMString;

	return TRUE;
}
BOOL SAArm::Logon()
{
	CString ELogon(Tr("Could not connect to SpatialAnalyzer."));
	BOOL GoFlag = FALSE;
	receivedSignOn = false;
	while(!GoFlag)
	{
		if (AutoSocket.Valid)
		{
			SAComputer = AutoSocket.Computer;
			SAPort = AutoSocket.Port;
		}
		else		// try with the logon dialog.
		{
			if(!PopLogonDialog())
			{
				PostQuitMessage(0);
				return FALSE;
			}
		}
		GoFlag = TRUE;		// assume we are go!
		if(!ConnectToSA())
		{
			AfxMessageBox(ELogon);
			GoFlag = FALSE;
			continue;
		}
	}
	if (!ActivateInstQuickSock())
	{
		AfxMessageBox(Tr("Could not connect socket to SpatialAnalyzer."));
		PostQuitMessage(0);
		return FALSE;
	}
	//QS.SetBufferPackets(FALSE); // 2016.07.07 - experimenting with ways to help with packet error 10061 (actively refused packet receipt)
	//QS.SetSilentMode(FALSE);

	TRACE0("calling wait for signon...\n");
	// need a time out here!
	while(!receivedSignOn)
	{
		Delay(0.1,TRUE);
		Sleep(10);
	}

	BOOL ret = FALSE;
	if(EstablishArmFromArmType())
	{
		LoadSettings();	// connection settings, like comm port, etc.
		ret =  ConnectToHardware();
		if(invalidateRefPlanesAfterStartup)
		{
			Data.scaleFromInstBase = scaleFromInstBase;
			Data.InvalidateReferencePlanes();
		}
		if(ret)
		{
			CStringArray otherInfo;
			OnHardwareConnected(app.pA->CMMSerialNum, otherInfo);	// 2010.9.22 - send serial num so SA can identify persisted model correction
		}
	}
	return ret;
}
// Connection to Instrument
BOOL SAArm::ConnectToHardware()
{
	return app.pA->ConnectToArm(DOF);
}
BOOL SAArm::DisconnectFromHardware()
{
	return TRUE;
}
void SAArm::OnHeartbeatStopped()
{
	TRACE0("SAArm::OnHeartbeatStopped() called!!\n");
	app.StopNow = TRUE;

	if(app.pA->IsFaroUSB())
		app.pA->DisconnectFromHardware(); // 2016.05.10 - need to shut down the scanner if it is active - otherwise, rangefinder is hidden but scanner is active upon next startup

	//Delay(2.5, TRUE);
	SAInstBase::OnHeartbeatStopped();

	// 2012.07.07 - blt - Mantis #5314 - arm interface crashes if SA is closed when a meas dialog is up
	// Art added the below code for Mantis #3486 back on 2010.09.29, but you can't just simulate a main dlg OK button press if a meas is active...

	// Mantis #3486 -- interface does not always shutdown properly... the PostQuitMessage(0)
	//  performed in SAInstBase::OnHeartbeatStopped is sometimes ignored.  So, when the heartbeat
	//  stops... simulate IDOK button press to make the dialog go away.
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(&d)
	{
		d.PostMessage(WM_COMMAND, MAKELONG(IDOK, BN_CLICKED), 0);
	}


}
///////////////////////////////////////////////////////////////////////
// socket stuff

void SAArm::OnSAIPSignOn( SAIPSignOn& p )
{
	TRACE1("signon packet! Inst model from SA = %d\n", p.InstModel);
	InstrumentModel = p.InstModel;
	InstrumentArrayIndex = p.InstArrayIndex;
	instrumentCollection = p.instCollection;
	instrumentFullName = p.instFullName;

	if(!SetArmTypeFromSAInstModel(InstrumentModel))
	{
		DisconnectFromSA();
		CString t;  //superceeded by new test method
		t.Format(Tr("Spatial Analyzer Closed the port due to\nInstrument compatibility.  Check the instrument\ntype in Spatial Analyzer."));
		AfxMessageBox(t);
		PostQuitMessage(0);
	}
	TRACE0("Creating Joint GenVec...\n");
	//app.pA->J = J;

	//toolBarDevFlagSet = p.LicenseInfo.IsDevFlagSet(_T("INSTRUMENT_BAR_PI")) ? true : false;
	toolBarDevFlagSet = true;

	scannerLicenseValid = p.LicenseInfo.CheckModuleLicense(14);	// MODULE_ARM_SCANNER = 14

	// If this running in CHINA mode, enforce the language file that is used or exit
	// MODULE_SA_CHINA = 33
	if (p.LicenseInfo.CheckModuleLicense(33))
	{
		EnableLanguageEnforcer(_T("ArmOptiParam.tvo"), _T("aRmS"));
		if (!EnforceLanguage(true))
		{
			PostQuitMessage(0);
		}
	}

	// blt - 2008.05.12 - update inst index display for jump location in SA...
	if(app.m_pMainWnd != NULL)
	{
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		d.DisplayInstColNameIdx(instrumentFullName);
	}

	receivedSignOn = true;
}
bool SAArm::SetArmTypeFromSAInstModel(const int saInstModel)
{
	arm_type = -1;
	/////////////////////////////////////////////////////////
	// Romer Cimcore
	if (InstrumentModel == 11) // Romer 1028
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 1028 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 1028");
		//			SettingsFileName.Format(_T("Romer 1028"));
		//			LogFileName.Format(_T("Romer 1028"));
	}
	//else if(InstrumentModel == 14) // 2005.08.19 - Romer CimCore Articulator (TEMPORARILLY USING THE 2500 MODEL)
	//{
	//	arm_type = arm_RomerArticulator;
	//	TRACE0("CimCore Articulator Arm Connected\n");
	//	DOF = 7;
	//	CMMString = _T("CimCore Articulator");
	//}
	else if(InstrumentModel == 12) // Romer 1024
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 1024 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 1024");
		//			SettingsFileName.Format(_T("Romer 1024"));
		//			LogFileName.Format(_T("Romer 1024"));
	}
	else if(InstrumentModel == 13) // Romer 2200
	{
		arm_type = arm_Romer;
		TRACE0("Romer 2200 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Romer 2200");

		//arm_type = arm_Hexagon_RDS;
		//TRACE0("Hexagon Multi-Gage Arm Connected\n");
		//DOF = 6;
		//CMMString = _T("Hexagin Multi-Gage");

	}
	else if(InstrumentModel == 14) // Romer 2500
	{
		arm_type = arm_Romer;
		TRACE0("Romer 2500 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Romer 2500");
		//			SettingsFileName.Format(_T("Romer 2500"));
		//			LogFileName.Format(_T("Romer 2500"));
	}
	else if(InstrumentModel == 22) // Romer 1030
	{
		arm_type = arm_Romer;
		TRACE("CimCore 1030 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 1030");
		//			SettingsFileName.Format(_T("Romer 1030"));
		//			LogFileName.Format(_T("Romer 1030"));
	}
	else if(InstrumentModel == 31) // Romer 3018
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 3018 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 3018");
	}
	else if(InstrumentModel == 58) // Romer 3024
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 3024 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 3024");
	}
	else if(InstrumentModel == 32) // Romer 3028
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 3028 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 3028");
	}
	else if(InstrumentModel == 33) // Romer 3036
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 3036 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 3036");
	}
	// 2001.03.23 end
	else if(InstrumentModel == 46) // Romer 3012
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 3012 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 3012");
	}
	else if(InstrumentModel == 79) // Romer 5030
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5030 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5030");
	}
	else if(InstrumentModel == 80) // Romer 5028
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5028 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5028");
	}
	else if(InstrumentModel == 81) // Romer 5024
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5024 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5024");
	}
	else if(InstrumentModel == 82) // Romer 5018
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5018 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5018");
	}
	else if(InstrumentModel == 83) // Romer 5012
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5012 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5012");
	}
	else if(InstrumentModel == 84) // Romer 5036
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5036 Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5036");
	}
	else if(InstrumentModel == 94) // Romer 5130Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5130Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5130Sc");
	}
	else if(InstrumentModel == 95) // Romer 5128Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5128Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5128Sc");
	}
	else if(InstrumentModel == 96) // Romer 5124Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5124Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5124Sc");
	}
	else if(InstrumentModel == 93) // Romer 5118Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5118Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5118Sc");
	}
	else if(InstrumentModel == 92) // Romer 5112Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5112Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5112Sc");
	}
	else if(InstrumentModel == 97) // Romer 5136Sc
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5136Sc Arm Connected\n");
		DOF = 7;
		CMMString = _T("CimCore 5136Sc");
	}
	else if(InstrumentModel == 100) // Romer 5130
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5130 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5130");
	}
	else if(InstrumentModel == 101) // Romer 5128
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5128 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5128");
	}
	else if(InstrumentModel == 102) // Romer 5124
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5124 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5124");
	}
	else if(InstrumentModel == 99) // Romer 5118
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5118 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5118");
	}
	else if(InstrumentModel == 98) // Romer 5112
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5112 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5112");
	}
	else if(InstrumentModel == 103) // Romer 5136
	{
		arm_type = arm_Romer;
		TRACE0("CimCore 5136 Arm Connected\n");
		DOF = 6;
		CMMString = _T("CimCore 5136");
	}
	///////////////////////////////////////////////////////
	// Hexagon
	else if(InstrumentModel == 117) // Hexagon ROMER Multi-Gage
	{
		arm_type = arm_Hexagon_MultiGage;
		TRACE0("Hexagon Multi-Gage Arm Connected\n");
		DOF = 6;
		CMMString = _T("Multi-Gage");

	}
	else if(InstrumentModel == 118) // Hexagon Absolute 7x20SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x20SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x20SI");

	}
	else if(InstrumentModel == 125) // Hexagon Absolute 7x25SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x25SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x25SI");

	}
	else if(InstrumentModel == 126) // Hexagon Absolute 7x30SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x30SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x30SI");

	}
	else if(InstrumentModel == 127) // Hexagon Absolute 7x35SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x35SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x35SI");

	}
	else if(InstrumentModel == 128) // Hexagon Absolute 7x40SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x40SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x40SI");

	}
	else if(InstrumentModel == 129) // Hexagon Absolute 7x45SI 7 dof
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 7x45SI Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 7x45SI");

	}
	else if(InstrumentModel == 130) // Hexagon Absolute 7315 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7315 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7315");

	}
	else if(InstrumentModel == 119) // Hexagon Absolute 7x20 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x20 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x20");

	}
	else if(InstrumentModel == 131) // Hexagon Absolute 7x25 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x25 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x25");

	}
	else if(InstrumentModel == 132) // Hexagon Absolute 7x30 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x30 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x30");

	}
	else if(InstrumentModel == 133) // Hexagon Absolute 7x35 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x35 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x35");

	}
	else if(InstrumentModel == 134) // Hexagon Absolute 7x40 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x40 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x40");

	}
	else if(InstrumentModel == 135) // Hexagon Absolute 7x45 6 dof
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 7x45 6dof Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 7x45");

	}
	///////////////////////////////////////////////////////////////
	// Hexagon Series 8 arms
	else if(InstrumentModel == 169) // Hexagon Absolute 8 7dof-2m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-2m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-2m");
	}
	else if(InstrumentModel == 170) // Hexagon Absolute 8 7dof-2.5m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-2.5m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-2.5m");
	}
	else if(InstrumentModel == 171) // Hexagon Absolute 8 7dof-3m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-3m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-3m");
	}
	else if(InstrumentModel == 172) // Hexagon Absolute 8 7dof-3.5m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-3.5m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-3.5m");
	}
	else if(InstrumentModel == 173) // Hexagon Absolute 8 7dof-4m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-4m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-4m");
	}
	else if(InstrumentModel == 174) // Hexagon Absolute 8 7dof-4.5m
	{
		arm_type = arm_Hexagon_Absolute_SI;
		TRACE0("Hexagon Absolute 8 7dof-4.5m Arm Connected\n");
		DOF = 7;
		CMMString = _T("Absolute 8 7dof-4.5m");
	}
	else if(InstrumentModel == 175) // Hexagon Absolute 8 6dof-1.2m Compact
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-1.2m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-1.2m");
	}
	else if(InstrumentModel == 176) // Hexagon Absolute 8 6dof-2m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-2m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-2m");
	}
	else if(InstrumentModel == 177) // Hexagon Absolute 8 6dof-2.5m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-2.5m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof2.5m");
	}
	else if(InstrumentModel == 178) // Hexagon Absolute 8 6dof-3m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-3m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-3m");
	}
	else if(InstrumentModel == 179) // Hexagon Absolute 8 6dof-3.5m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-3.5m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-3.5m");
	}
	else if(InstrumentModel == 180) // Hexagon Absolute 8 6dof-4m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-4m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-4m");
	}
	else if(InstrumentModel == 181) // Hexagon Absolute 8 6dof-4.5m
	{
		arm_type = arm_Hexagon_Absolute;
		TRACE0("Hexagon Absolute 8 6dof-4.5m Arm Connected\n");
		DOF = 6;
		CMMString = _T("Absolute 8 6dof-4.5m");
	}
	///////////////////////////////////////////////////////
	// Faro Serial...
	else if(InstrumentModel == 5) // Faro ? (old)
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro");
		//			SettingsFileName.Format(_T("Faro"));
		//			LogFileName.Format(_T("Faro"));
	}
	else if(InstrumentModel == 10) // Faro S08
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro S08 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro S08");
		//			SettingsFileName.Format(_T("Faro S08"));
		//			LogFileName.Format(_T("Faro S08"));
	}
	else if(InstrumentModel == 26) // Faro S12
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro S12 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro S12");
		//			SettingsFileName.Format(_T("Faro S12"));
		//			LogFileName.Format(_T("Faro S12"));
	}
	else if(InstrumentModel == 25) // Faro G08
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G08 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro G08-02");
		//			SettingsFileName.Format(_T("Faro G08"));
		//			LogFileName.Format(_T("Faro G08"));
	}
	else if(InstrumentModel == 27) // Faro G12
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G12 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro G12-02");
		//			SettingsFileName.Format(_T("Faro G12"));
		//			LogFileName.Format(_T("Faro G12"));
	}
	else if(InstrumentModel == 34) // Faro G08-05 (7 dof)
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G08-05 Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro G08-05");
	}
	else if(InstrumentModel == 35)	// Faro G12-05 (7 dof)
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G12-05 Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro G12-05");
	}
	else if(InstrumentModel == 40)	// Faro G04-02
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G04-02 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro G04-02");
	}
	else if(InstrumentModel == 41)	// Faro G04-05 (7 dof)
	{
		arm_type = arm_FaroSerial;
		TRACE0("Faro G04-05 Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro G04-05");
	}
	////////////////////////////////////////////////////////////////////
	// Faro USB
	else if(InstrumentModel == 47)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 4 ft. Arm Connected\n");
		DOF = 6;
		CMMString = _T("FaroUSB4ft");
	}
	else if(InstrumentModel == 48)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 6 ft. Arm Connected\n");
		DOF = 6;
		CMMString = _T("FaroUSB6ft");
	}
	else if(InstrumentModel == 49)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 8 ft. Arm Connected\n");
		DOF = 6;
		CMMString = _T("FaroUSB8ft");
	}
	else if(InstrumentModel == 50)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 10 ft. Arm Connected\n");
		DOF = 6;
		CMMString = _T("FaroUSB10ft");
	}
	else if(InstrumentModel == 51)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 12 ft. Arm Connected\n");
		DOF = 6;
		CMMString = _T("FaroUSB12ft");
	}
	else if(InstrumentModel == 52)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 4 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroUSB4ft7dof");
	}
	else if(InstrumentModel == 53)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 6 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroUSB6ft7dof");
	}
	else if(InstrumentModel == 54)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 8 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroUSB8ft7dof");
	}
	else if(InstrumentModel == 55)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 10 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroUSB10ft7dof");
	}
	else if(InstrumentModel == 56)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro USB 12 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroUSB12ft7dof");
	}
	else if(InstrumentModel == 124)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Edge 9 ft. (7dof) Arm Connected\n");
		DOF = 7;
		CMMString = _T("FaroEdge9ft7dof");
	}
	else if(InstrumentModel == 162)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro Quantum 2.5m");
	}
	else if(InstrumentModel == 163)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro Quantum 1.5m");
	}
	else if(InstrumentModel == 164)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro Quantum 3.5m");
	}
	else if(InstrumentModel == 165)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 7;
		CMMString = _T("Faro Quantum 4m");
	}
	else if(InstrumentModel == 166)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro Quantum 2.5m");
	}
	else if(InstrumentModel == 167)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro Quantum 3.5m");
	}
	else if(InstrumentModel == 168)
	{
		arm_type = arm_FaroUSB;
		TRACE0("Faro Quantum SorM Arm Connected\n");
		DOF = 6;
		CMMString = _T("Faro Quantum 4m");
	}
	////////////////////////////////////////////////////////////////////////////
	// Sandia National Labs
	else if(InstrumentModel == 38)
	{
		arm_type = arm_SNL;
		TRACE0("SNL Arm Connected\n");
		DOF = 3;
		CMMString = _T("Sandia");
	}
	////////////////////////////////////////////////////////////////////////////
	// French Romer
	else if(InstrumentModel == 76) // Romer Sigma 2022
	{
		arm_type = arm_RomerEurope;
		TRACE0("Romer Sigma 2022 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Romer Sigma 2022");
	}
	////////////////////////////////////////////////////////////////////////////
	// Metris MCA (Garda)
	else if(InstrumentModel == 86) // Metris MCA
	{
		return false;	// 2009.10.09 - Metris now supports the MCA via their HHAPI interface (our Metris K-Series interface)

		arm_type = arm_MetrisMCA;
		TRACE0("Metris MCA (Garda) Arm Connected\n");
		DOF = 6;
		CMMString = _T("Metris MCA");
	}
	////////////////////////////////////////////////////////////////////////////
	// API Axxis (Baces by Friul Robotics)
	else if(InstrumentModel == 104) // Axxis 6-100 Arm (2.6m 6 dof)
	{
		arm_type = arm_Axxis_KreonScanner;
		TRACE0("API Axxis 6-100 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Axxis 6-100");
	}
	else if(InstrumentModel == 105) // Axxis 7-100 Arm Scanner (2.6m 7 dof)
	{
		arm_type = arm_Axxis_KreonScanner;
		TRACE0("API Axxis 7-100 Arm Scanner Connected\n");
		DOF = 7;
		CMMString = _T("Axxis 7-100 Sc");
	}
	else if(InstrumentModel == 106) // Axxis 6-200 Arm (3.2m 6 dof)
	{
		arm_type = arm_Axxis_KreonScanner;
		TRACE0("API Axxis 6-200 Arm Connected\n");
		DOF = 6;
		CMMString = _T("Axxis 6-200");
	}
	else if(InstrumentModel == 146) // Axxis 7-100 Arm Probe (2.6m 7 dof)
	{
		//arm_type = arm_Axxis_KreonScanner;
		arm_type = arm_Axxis_KreonScanner;
		//Data.autoCorrectModel = FALSE;	// we can't read the 7th axis!  Requested this --moved to APIBacesArm::DoPostInitOperations()
		TRACE0("API Axxis 7-100 Arm Probe Connected\n");
		DOF = 7;
		CMMString = _T("Axxis 7-100");
	}

	if(arm_type == -1)
		return false;
	return true;
}
bool SAArm::EstablishArmFromArmType()
{
	if(arm_type<0 || arm_type>MAX_ARM_TYPE)
		return false;

	switch (arm_type)
	{
	case arm_FaroUSB:
		TRACE0("callling EstablishArm_FaroUSB\n");
		app.EstablishArm_FaroUSB();
		Data.ButtonThreeMode = TOUCH_PROBE;
		break;
	case arm_Romer:
		TRACE0("callling EstablishArm_Romer\n");
		app.EstablishArm_Romer();
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.ButtonThreeMode = bf_next_previous;
		Data.scannerTimerPeriodSec = 1.0;
		break;
	case arm_Axxis_KreonScanner:
		TRACE0("callling EstablishArm_API_Baces_Kreon_Scanner\n");
		//AfxMessageBox(_T("not supported yet!"), MB_ICONINFORMATION);
		//return false;
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.scannerTimerPeriodSec = 1.0;
		app.EstablishArm_API_Baces_Kreon_Scanner();
		break;
	case arm_Axxis_Arm:
		// 2017.10.19 - using Kreon for all Kreon/Baces/Axxis arms now
		//Data.ButtonOneMode = bf_record_accept;
		//Data.ButtonTwoMode = bf_deleteLast_cancel;
		//Data.ButtonThreeMode = bf_record_accept;
		//TRACE0("callling EstablishArm_API_Baces\n");
		//app.EstablishArm_API_Baces();
		//break;
		TRACE0("callling EstablishArm_API_Baces_Kreon_Scanner\n");
		//AfxMessageBox(_T("not supported yet!"), MB_ICONINFORMATION);
		//return false;
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.scannerTimerPeriodSec = 1.0;
		app.EstablishArm_API_Baces_Kreon_Scanner();
		break;
	case arm_FaroSerial:
		TRACE0("callling EstablishArm_FaroSerial\n");
		app.EstablishArm_FaroSerial();
		break;
	case arm_SNL:
		TRACE0("callling EstablishArm_SNL\n");
		app.EstablishArm_SNL();
		break;
	case arm_RomerArticulator:
		TRACE0("callling EstablishArm_RomerArticulator\n");
		app.EstablishArm_RomerArticulator();
		break;
	case arm_RomerEurope:
		TRACE0("calling EstablishArm_RomerEurope\n");
		app.EstablishArm_RomerEurope();
		break;
	case arm_MetrisMCA:
		TRACE0("calling EstablishArm_MetrisMCA\n");
		app.EstablishArm_MetrisMCA();
		break;
	case arm_Hexagon_MultiGage:
		TRACE0("calling EstablishArm_Hexagon_RDS\n");
		app.EstablishArm_Hexagon_RDS();
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.ButtonThreeMode = bf_next_previous;
		break;
	case arm_Hexagon_Absolute_SI:
		TRACE0("calling EstablishArm_Hexagon_RDS\n");
		app.EstablishArm_Hexagon_RDS();
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.ButtonThreeMode = bf_next_previous;
		Data.scannerTimerPeriodSec = 0.1;
		break;
	case arm_Hexagon_Absolute:
		TRACE0("calling EstablishArm_Hexagon_RDS\n");
		app.EstablishArm_Hexagon_RDS();
		Data.ButtonOneMode = bf_deleteLast_cancel;
		Data.ButtonTwoMode = bf_record_accept;
		Data.ButtonThreeMode = bf_next_previous;
		break;
	default:
		TRACE0("Unsupported Arm type - returning false by default\n");
		return false;
	}

	Data.buttonOneMain = Data.ButtonOneMode;
	Data.buttonTwoMain = Data.ButtonTwoMode;
	Data.buttonThreeMain = Data.ButtonThreeMode;

	return true;
}

void SAArm::OnInstToWorkingChanged()
{
	TRACE0("******Got OnInstToWorkingChanged\n");
	if(Data.scaleFromInstBase != GetInstrumentScaleFactor())
	{
		Data.InvalidateReferencePlanes();
		invalidateRefPlanesAfterStartup = true;
	}
	Data.scaleFromInstBase = GetInstrumentScaleFactor();
	scaleFromInstBase = Data.scaleFromInstBase;	// for startup condition -- this packet is received BEFORE we load settings
}

void SAArm::OnSAIPGeom( SAIPGeom& g )
{
	TRACE0("Got a Geom request from SA!\n");
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomPlane)))
	{
		if(waitingForProjectionPlaneFromSA)
		{
			gp = *(SAIPGeomPlane*)&g;
			Vector pos;
			gp.ObjectToInst.GetTranslation(&pos);
			pos.Scale(InchesToLocal);
			gp.ObjectToInst.SetTranslation(pos);
			gp.PerimPts.Scale(InchesToLocal);
			waitingForProjectionPlaneFromSA = false;
		}
		else if(!app.pA->Measuring)
		{
			SAIPGeomPlane& igp = *(SAIPGeomPlane*)&g;
			app.pA->DoPlaneInspect(igp);
		}
	}
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomCircle)))
	{
		if(app.pA->Measuring)
			return;
		SAIPGeomCircle& gc = *(SAIPGeomCircle*)&g;
		if( app.pA->inspectreq == PININSPECT )
			app.pA->DoPinInspect(gc);
		else if( app.pA->inspectreq == HOLEINSPECT )
			app.pA->DoHoleInspect(gc);
	}
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomFrame)))
	{
		TRACE0("Got a frame request from SA!\n");
		if(app.pA->Measuring)
			return;
	}
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomLine)))
	{
		TRACE0("Got a Line from SA...\n");
		gl = *(SAIPGeomLine*)&g;
		gl.PtA.Scale(InchesToLocal);
		gl.PtB.Scale(InchesToLocal);
		waitingForLineFromSA = false;
	}	
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomCylinder)))
	{
		TRACE0("Got a Cylinder from SA...\n");
		Vector pos;
		gCyl = *(SAIPGeomCylinder*)&g;
		gCyl.ObjectToInst.GetTranslation(&pos);
		pos.Scale(InchesToLocal);
		gCyl.ObjectToInst.SetTranslation(pos);
		gCyl.PtA.Scale(InchesToLocal);
		gCyl.PtB.Scale(InchesToLocal);
		gCyl.Diameter *= InchesToLocal;
		waitingForCylinderFromSA = false;
	}	
	if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomSphere)))
	{
		TRACE0("Got a Sphere from SA...\n");
		gs = *(SAIPGeomSphere*)&g;
		gs.Center.Scale(InchesToLocal);
		gs.Radius *= InchesToLocal;
		waitingForSphereFromSA = false;
	}
}

void SAArm::OnSAIPGeomArray( SAIPGeomArray& geomArray )
{
	for (DWORD_PTR geomIdx = 0; geomIdx < geomArray.geoms.GetSize(); geomIdx++)
	{
		SAIPGeom& g = geomArray.geoms[geomIdx];

		TRACE1("got a geom from SA, name = '%s'", g.Name);
		// for now, the only thing we're doing with these is populating geom triggers...
		// later, we'll have to add an SAIPGeomArray member, add an =operator, and set the member
		if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomPlane)))
		{
			gp = *(SAIPGeomPlane*)&g;

			Vector pos;
			gp.ObjectToInst.GetTranslation(&pos);
			pos.Scale(InchesToLocal);
			gp.ObjectToInst.SetTranslation(pos);
			gp.PerimPts.Scale(InchesToLocal);

			geomTriggerWrap.AddSelectedPlane(gp);
		}
		if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomCylinder)))
		{
			TRACE0("Got a Cylinder from SA...\n");
			gCyl = *(SAIPGeomCylinder*)&g;

			Vector pos;
			gCyl.ObjectToInst.GetTranslation(&pos);
			pos.Scale(InchesToLocal);
			gCyl.ObjectToInst.SetTranslation(pos);
			gCyl.PtA.Scale(InchesToLocal);
			gCyl.PtB.Scale(InchesToLocal);
			gCyl.Diameter *= InchesToLocal;

			geomTriggerWrap.AddSelectedCylinder(gCyl);
		}	
		if(g.IsKindOf(RUNTIME_CLASS(SAIPGeomSphere)))
		{
			TRACE0("Got a Sphere from SA...\n");
			gs = *(SAIPGeomSphere*)&g;

			gs.Center.Scale(InchesToLocal);
			gs.Radius *= InchesToLocal;

			geomTriggerWrap.AddSelectedSphere(gs);
		}
	}
	waitingForGeomArrayFromSA = false;
}

// added 2001.11.11
void SAArm::OnSAIPMeasRequest(SAIPMeasRequest& p )
{
	if( p.RequestType == UPDATE_FRAME )
	{
		TRACE0("Frame update request from SA\n");
		app.pA->FrameUpdateRequested = TRUE;
		if(!app.pA->isMeasuringFrames)
			app.pA->DoFrameMeas();
	}
}
void SAArm::OnSAIPGroupTarget(SAIPGroupTarget& p)	// 2004.10.08 - added
{
	if(app.m_pMainWnd == NULL)	// Mantis# 5869: Instrument Interface will crash if trying to do things in toolkit before it has completely started 
	{
		AfxMessageBox(Tr("Cannot set Collection::Group::Target. Interface is not started!"), MB_ICONERROR);
		return;
	}

	app.pA->ClearData();

	//if(app.pA->Measuring)
	//	return;
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	//AfxMessageBox(p.collectionName);
	d.SetCollectionName(p.collectionName);
	d.SetGroupName(p.GroupName);
	d.SetTargetName(p.TargetName);	
}
BOOL SAArm::SetMeasurementMode(CString& mode, CString* pErrorString)	// 2004.10.08 - added
{
	if(app.pA->Measuring)
	{
		// 2016.12.08 - JIRA XIT-175 - Error when using Configure Instrument [from relationship fit's "Congigure Instrument" command] - the guys were just missing the following messagebox...
		//CString m;
		//m.Format(Tr("Received measurement mode set request.\nStop current measurement?"));
		//if(AfxMessageBox(m,MB_YESNO) == IDYES)
		//{
			app.StopNow = TRUE;
			Delay(1.0,TRUE);
		//}
		//else
		//{
		//	pErrorString->Format(Tr("User elected NOT to stop current measurement."));
		//	return FALSE;
		//}
	}

	//TRACE1("SATracker::SetMeasurementMode called with mode '%s'...\n", mode);
	CString _mode = mode;
	if(!app.pA->SetMeasModeByName(_mode, *pErrorString))
		return FALSE;
	return TRUE;
}
BOOL SAArm::GetCloudDataSendingPeriod(double& periodSec)
{
	periodSec = Data.scannerTimerPeriodSec;
	return TRUE;
}
BOOL SAArm::OpCheck(CString& command)
{
	if(StringHelper::NameCompare(command, _T("Send Measured Points")))
	{
		Data.FirePointsToo = TRUE;
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Don't Send Measured Points")))
	{
		Data.FirePointsToo = FALSE;
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Calibrate")))
	{
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		d.calbutton.HandleLButton();
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Auto-Prox with Scanner")))
	{
		Data.useScannerForAutoProxMeas = true;
		Data.useShankForAutoProxMeas = false;
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Auto-Prox with Probe")))
	{
		Data.useScannerForAutoProxMeas = false;
		Data.useShankForAutoProxMeas = false;
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Auto-Prox with Shank")))
	{
		Data.useScannerForAutoProxMeas = false;
		Data.useShankForAutoProxMeas = true;
		return TRUE;
	}
	if(StringHelper::NameCompare(command.Left(14), _T("Set Geom Name ")))
	{
		app.saarm.Data.geomName = command.Mid(14);
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		d.GetDataParams();
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Mouse Mode On")))
	{
		app.pA->RemoteMouse(TRUE);
		return TRUE;
	}
	if(StringHelper::NameCompare(command, _T("Mouse Mode Off")))
	{
		app.pA->RemoteMouse(FALSE);
		return TRUE;
	}
	if(StringHelper::NameCompare(command.Left(43), _T("Set Stream Points Spatial Increment Inches "))) // added 2016.11.29 by request from Scott Leedy
	{
		double inc = FIXROUND(_wtof(command.Mid(43)));
		if(inc > 0.0)
		{
			Data.scan_spatial_inc = inc * InchesToLocal;
			Data.scan_spatial = TRUE;
			return TRUE;
		}
	}

	return FALSE;
}
void SAArm::OnSAIPHitMeasureButton(SAIPHitMeasureButton& p)	// 2004.10.08 - added
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(!&d)
		return;
	d.SetTopMost();	// 2005.02.17

	if(p.Command == SAIPHitMeasureButton::HIT_MEASURE)
	{
		if(app.pA->Measuring)
			return;
		d.DoMeasurement();
	}
	else if(p.Command == SAIPHitMeasureButton::HIT_STOP)
	{
		TRACE0("\n************SAIPHitMeasureButton::HIT_STOP received************\n\n");
		app.StopNow = TRUE;
		if(app.pA->IsFaroUSB())
			app.pA->StopScannerMeas(); // it does nothing if not already scanning
		Delay(1.0,TRUE);
		OnMeasurementDone(MPResult::DoneSuccess);
	}
	d.SetNoTopMost();	// 2005.02.17
	app.StopNow = FALSE;	// 2005.07.18 - when either watches are closed or MP's are ended, we're getting a HIT_STOP
}
void SAArm::OnSAIPMeasureSinglePointHere(SAIPMeasureSinglePointHere& p)
{
	if(app.pA->Measuring)
	{
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(!&d)
		return;
	d.SetTopMost();	// 2005.03.23
	d.SetCollectionName(p.collectionName);
	d.SetGroupName(p.GroupName);
	d.SetTargetName(p.TargetName);
	if(app.pA->DoSAIPPointMeas())
	{
		OnMeasurementDone(MPResult::DoneSuccess);
	}
	else
		OnMeasurementDone(MPResult::DoneFatalError);
	d.SetNoTopMost();	// 2005.03.23
}
void SAArm::OnSAIPMeasureSinglePointThere(SAIPMeasureSinglePointThere& p)
{
	if(app.pA->Measuring)
	{
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}
	if(p.NominalLocation.MeasData.GetSize() <= 0)
	{
		TRACE0("Meas Pt. there failed (No nom pt.!), sending DoneFatalError\n");
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(!&d)
		return;
	d.SetTopMost();	// 2005.03.23
	d.SetCollectionName(p.NominalLocation.collection);

	// just make a batch meas group consisting of only our single pt. there...
	// first, get the nominal for the 'batch'
	SAIPMeasDataXYZ& md = *(SAIPMeasDataXYZ*)(p.NominalLocation.MeasData[0]);
	//TRACE0("OnSAIPMeasureSinglePointThere meas data in xyz...\n");
	//md.Point.Debug();
	
	SAIPMeasGroupPt* pGP = new SAIPMeasGroupPt;
	pGP->Nominal = md.Point;
	pGP->Group = p.NominalLocation.Group;;
	pGP->Target = p.NominalLocation.Target;
	app.pA->GuidedMeasGroup.Points.Add( pGP );

	Data.GoodProximity = 1.5*units.UnitToMeters[DISTUNITS_IN];	// force these to be within this sphere radius
	//Data.proxWorkingField = p.toneGenStartZone*units.UnitToMeters[DISTUNITS_IN];

	app.pA->MeasureGuidePts();	// it takes care of the MPResults
	d.SetNoTopMost();	// 2005.03.23
}
void SAArm::OnSAIPMeasGroup(SAIPMeasGroup& p)
{
	if(app.pA->Measuring)
	{
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}
	TRACE1("Got a Batch Meas packet with %d points", p.Points.GetSize());

	if( p.Points.GetSize() <= 0 )
		return;
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(!&d)
		return;
	d.SetTopMost();	// 2005.03.23
	app.pA->GuidedMeasGroup.ClearPoints();
	app.pA->GuidedMeasGroup.TargetSuffix = p.TargetSuffix;
	app.pA->GuidedMeasGroup.GroupSuffix = p.GroupSuffix;
	if(p.DriveTol > 0.0)	// 2004.10.01 - member added to SAIPMeasGroup packet for Measure Plan 'Initiate Servo-Guide' command
	{
		Data.GoodProximity = p.DriveTol*units.UnitToMeters[DISTUNITS_IN];
		Data.proxWorkingField = p.toneGenStartZone*units.UnitToMeters[DISTUNITS_IN];
	}
	for( int i=0; i < p.Points.GetSize(); i++ )
	{
		SAIPMeasGroupPt* pGP = new SAIPMeasGroupPt;
		pGP->Group = p.Points[i]->Group;
		pGP->Target = p.Points[i]->Target;
		pGP->Nominal = p.Points[i]->Nominal;
		app.pA->GuidedMeasGroup.Points.Add( pGP );
	}
	app.pA->MeasureGuidePts();	// 2004.08.24
	d.SetNoTopMost();	// 2005.03.23
}

void SAArm::OnNonStandardPacket( CObject& pkt )
{
	if (pkt.IsKindOf(RUNTIME_CLASS(SAIPAutoMeasProximity)))
	{
		OnSAIPAutoMeasProximity((SAIPAutoMeasProximity&)pkt);
	}
}
void SAArm::OnSAIPAutoMeasProximity(SAIPAutoMeasProximity &p)
{
	if(Data.useScannerForAutoProxMeas)
	{
		if(p.processDONE)
		{
			app.saarm.StopScreaming();
			app.StopNow = TRUE;
			return;
		}
		app.saarm.EndMeasurement();
		app.pA->UpdateAutoProxSamp(p);
		app.pA->performingStdScannerMeas = false;
		app.pA->StartScannerMeas();
	}
	else
	{
		// 2007.08.13 - set the current samp to the contents of p with an = operator
		// since the settings may be changed by SA while measuring...
		app.pA->UpdateAutoProxSamp(p);

		 // 2020.04.20 - XIT-1779 - if we're already measuring auto-prox, then all we need to do is call UpdateAutoProxSamp (set the new parameters that have been sent), and leave
		if(app.pA->Measuring && app.pA->pAutoProxDlg != NULL)
			return;

		bool resetMeas = false;
		if(units.toolBarModeOn)
		{
			// we've already checked to see if we're measuring autoprox (null check on pAutoProxDlg above), so if there is an active measurement (always is with toolbar), end it first
			lastToolBarMeasModeName = app.pA->CurrentMeasModeName();
			if(app.measDlg && app.pA->Measuring)
			{
				resetMeas = true;
				app.measDlg->EndCancel();
			}
		}
		// we've already checked to see if we're measuring autoprox (null check on pAutoProxDlg above), so if there is an active measurement, end it first
		else if (app.pA->Measuring)
			EndMeasurement();

		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		if(!&d)
			return;
		if(!p.processDONE)
		{
			d.SetCollectionName(p.destCollection);
			d.SetGroupName(p.destGroup);
			CString tgtName(_T("P1"));
			if (p.elements.GetSize()>0)
				tgtName = p.elements[0].name;
			d.SetTargetName(tgtName);	

			lastCollectionNameFromSA = p.destCollection;
			lastGroupNameFromSA = p.destGroup;
		}
		else
		{
			d.SetCollectionName(lastCollectionNameFromSA);
			StringHelper::IncrementName(lastGroupNameFromSA);
			d.SetGroupName(lastGroupNameFromSA);
			d.SetTargetName(_T("P1"));
		}

		app.pA->MeasureAutoProximityPoints(p);
	}
}

void SAArm::OnSAIPMeasurement( SAIPMeasurement& m )	// used by SA Inst->Locate->Measure Nom's AND SA Inst->Drift Check
{
	if(app.pA->Measuring)
	{
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	if(!&d)
		return;
	d.SetTopMost();	// 2005.03.23
	d.SetCollectionName(m.collection);
	d.SetGroupName(m.Group);
	d.SetTargetName(m.Target);
	if(app.pA->DoSAIPPointMeas())
	{
		OnMeasurementDone(MPResult::DoneSuccess);
	}
	else
		OnMeasurementDone(MPResult::DoneFatalError);
	d.SetNoTopMost();	// 2005.03.23
}
void SAArm::ApplyCribSheet(SAIPCribSheet& pkt)	// 2005.04.12 - for crib sheet support
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	OnSAIPGroupTarget(pkt.naming);
	//d.SetGeomName(pkt.naming.TargetName);
	BOOL modeSet = true;
	if(!pkt.measMode.ModeString.IsEmpty())
	{
		modeSet = SetMeasurementMode(pkt.measMode.ModeString, &pkt.measMode.ModeSetErrorString);
	}
	pkt.measMode.ModeSet = modeSet;
	if(!modeSet)
	{
		OnMeasurementDone(MPResult::DoneFatalError);
		return;
	}
	if(units.toolBarModeOn)
	{
		SendSAIRCAttributesToSA(); // 2016.12.09 - JIRA XIT-175 - new "Configure Instrument" command from SA Relationship Fits (just right click the relationship, and the configure option is in the pop-up menu)
	}
	if(!pkt.waitForCompletion)	// Mantis #7801 - 2015.06.30 - need this now that SA's CribSheetProcessHandler::OnMeasDonePacket is waiting for an MPResult
	{
		OnMeasurementDone(MPResult::DoneSuccess);
	}
	if(pkt.measure)
	{
		d.SetTopMost();
		d.DoMeasurement();
		d.SetNoTopMost();
	}
	if(pkt.waitForCompletion)	// Mantis #7801 - 2015.06.30
	{
		if(pkt.timeOutSeconds <= 0.0)	// when timeout is 0, ignore the timeout argument as we do in other MP steps
		{
			return;
		}
		DWORD Start, Delta;
		Start = ::GetCurrentTime();
		Delta = (DWORD) (pkt.timeOutSeconds*1000.0f);	// timeout after this many seconds.
		do
		{
			Sleep(1);
			if(!IsCribSheetMeasPending())
			{
				return;
			}
			ProcessWaitingMessages();
		} while ((::GetCurrentTime() - Start) < Delta);

		if(IsCribSheetMeasPending())
		{
			OnMeasurementDone(MPResult::DoneFatalError);
		}
	}
}
void SAArm::OnSAIPMeasData( SAIPMeasData& m )			// update packet (point at)
{
	TRACE0("Got a SAIPMeasData (update) packet\n");
}

void SAArm::OnSAIPFlags( SAIPFlags& p )
{
	if(app.pA->Measuring)
		return;
	if( p.Type == SAIPFlagQuery)
	{
		TRACE0("Frame update request from SA\n");
		CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
		if(!&d)
			return;
		d.SetTopMost();	// 2005.03.23
		d.StopQueryTimer();
		app.pA->FrameUpdateRequested = TRUE;
		if(!app.pA->isMeasuringFrames)
			app.pA->DoFrameMeas();
		d.SetNoTopMost();	// 2005.03.23
	}
}

void SAArm::OnSAIPInfo(SAIPInfo& p)
{
	switch(p.LengthUnits)
	{
	case SAIPInfo::LU_METERS:
		units.distunits = DISTUNITS_M;
		break;

	case SAIPInfo::LU_CENTIMETERS:
		units.distunits = DISTUNITS_CM;
		break;

	case SAIPInfo::LU_MILLIMETERS:
		units.distunits = DISTUNITS_MM;
		break;

	case SAIPInfo::LU_FEET:
		units.distunits = DISTUNITS_FT;
		break;

	case SAIPInfo::LU_INCHES:
		units.distunits = DISTUNITS_IN;
		break;

	default:
		break;
	}

	switch(p.TempUnits)
	{
	case SAIPInfo::TU_DEGS_C:
		units.tempunits = TEMPUNITS_C;
		units.pressunits = PRESSUNITS_MBAR;
		break;

	case SAIPInfo::TU_DEGS_F:
		units.tempunits = TEMPUNITS_F;
		units.pressunits = PRESSUNITS_INHG;
		break;

	default:
		break;
	}
}

BOOL SAArm::SendReportItem(CString& Title, CString& Type, CStringArray& BodyText)
{	
	SAIPInstReportItem rep;
	//	rep.Title.Format(_T("Tracker SN %d %s %s %s %s"),
	//		d.Data.TrackerSN, app.VString, d.InstIndex, CTime::GetCurrentTime().Format(_T("(%c)"), Title);
	rep.Title.Format(_T("%s %s"), Title, CTime::GetCurrentTime().Format(_T("(%c)")));
	rep.Type.Format(_T("%s"), Type);
	for( int i=0; i<BodyText.GetSize(); i++ )
		rep.BodyText.Add(BodyText[i]);

	return QS.SendPacket(rep);
}

BOOL SAArm::SendReportItem(CString& Title, CString& Type, CString& BodyText)
{	
	SAIPInstReportItem rep;
	//	rep.Title.Format(_T("Tracker SN %d %s %s %s %s"),
	//		d.Data.TrackerSN, app.VString, d.InstIndex, CTime::GetCurrentTime().Format(_T("(%c)"), Title);
	rep.Title.Format(_T("%s %s"), Title, CTime::GetCurrentTime().Format(_T("(%c)")));
	rep.Type.Format(_T("%s"), Type);
	rep.BodyText.Add(BodyText);

	return QS.SendPacket(rep);
}

BOOL SAArm::SendReportItem(const char* Title, const char* Type, CStringArray& BodyText)
{	
	SAIPInstReportItem rep;
	//	rep.Title.Format(_T("Tracker SN %d %s %s %s %s"),
	//		d.Data.TrackerSN, app.VString, d.InstIndex, CTime::GetCurrentTime().Format(_T("(%c)"), Title);
	rep.Title.Format(_T("%s %s"), Title, CTime::GetCurrentTime().Format(_T("(%c)")));
	rep.Type.Format(_T("%s"), Type);
	for( int i=0; i<BodyText.GetSize(); i++ )
		rep.BodyText.Add(BodyText[i]);

	return QS.SendPacket(rep);
}

BOOL SAArm::SendReportItem(const char* Title, const char* Type, CString& BodyText)
{	
	SAIPInstReportItem rep;
	//	rep.Title.Format(_T("Tracker SN %d %s %s %s %s"),
	//		d.Data.TrackerSN, app.VString, d.InstIndex, CTime::GetCurrentTime().Format(_T("(%c)"), Title);
	rep.Title.Format(_T("%s %s"), Title, CTime::GetCurrentTime().Format(_T("(%c)")));
	rep.Type.Format(_T("%s"), Type);
	rep.BodyText.Add(BodyText);

	return QS.SendPacket(rep);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 2006.06.07 - The Screamin' Weasel
void SAArm::SetScreamerParams()
{
	TRACE2("set screamer params: good prox = %f, working field = %f\n",Data.GoodProximity, Data.proxWorkingField);
	screamer.SetParams(Data.GoodProximity, Data.proxWorkingField);
}
void SAArm::SetScreamerParams(const double& targetAreaThreshold, const double& workingFieldThreshold)
{
	TRACE2("set screamer params: good prox = %f, working field = %f\n",targetAreaThreshold, workingFieldThreshold);
	screamer.SetParams(targetAreaThreshold, workingFieldThreshold);
}
void SAArm::ScreamThis(const double& error)
{
	screamer.SetError(error);
}
void SAArm::StartScreaming()
{
	screamer.Start();
}
void SAArm::StopScreaming()
{
	screamer.Stop();
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool SAArm::SelectPlaneFromSA(SAIPGeomPlane& p)
{
	SAIPRequestObject obReq;
	obReq.PromptUser = TRUE;
	obReq.Type = obReq.Plane;
	waitingForProjectionPlaneFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Plane From SA"));
	prog.Start(Tr("Selecting Plane..."));
	prog.SetMsg(Tr("Selecting Plane..."));
	prog.StepBar(0);
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForProjectionPlaneFromSA = false;
			return false;
		}
		if(!waitingForProjectionPlaneFromSA)
		{
			p = gp;
			return true;
		}
	}
	return false;
}
bool SAArm::SelectLineFromSA(SAIPGeomLine& l)
{
	SAIPRequestObject obReq;
	obReq.PromptUser = TRUE;
	obReq.Type = obReq.Line;
	waitingForLineFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Line From SA"));
	prog.Start(Tr("Selecting Line..."));
	prog.SetMsg(Tr("Selecting Line..."));
	prog.StepBar(0); // this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForLineFromSA = false;
			return false;
		}
		if(!waitingForLineFromSA)
		{
			l = gl;
			return true;
		}
	}
	return false;
}
bool SAArm::SelectCylinderFromSA(SAIPGeomCylinder& c)
{
	SAIPRequestObject obReq;
	obReq.PromptUser = TRUE;
	obReq.Type = obReq.Cylinder;
	waitingForCylinderFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Cylinder From SA"));
	prog.Start(Tr("Selecting Cylinder..."));
	prog.SetMsg(Tr("Selecting Cylinder..."));
	prog.StepBar(0); // this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForCylinderFromSA = false;
			return false;
		}
		if(!waitingForCylinderFromSA)
		{
			c = gCyl;
			return true;
		}
	}
	return false;
}
bool SAArm::SelectSphereFromSA(SAIPGeomSphere& s)
{
	SAIPRequestObject obReq;
	obReq.PromptUser = TRUE;
	obReq.Type = obReq.Sphere;
	waitingForSphereFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Sphere From SA"));
	prog.Start(Tr("Selecting Sphere..."));
	prog.SetMsg(Tr("Selecting Sphere..."));
	prog.StepBar(0); // this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForSphereFromSA = false;
			return false;
		}
		if(!waitingForSphereFromSA)
		{
			s = gs;
			return true;
		}
	}
	return false;
}
//////////////////////////////////////////////////////////////////////////////////////
// 2010.04.20 - added multiple geom selection for GeomTriggers...
bool SAArm::SelectMultiplePlanesFromSA()
{
	SAIPRequestMultipleObjects obReq;
	obReq.Type = obReq.Plane;
	waitingForGeomArrayFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Planes From SA"));
	prog.Start(Tr("Selecting Planes..."));
	prog.SetMsg(Tr("Selecting Planes..."));
	prog.StepBar(0); // 2009.04.02 - this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForGeomArrayFromSA = false;
			return false;
		}
		if(!waitingForGeomArrayFromSA)
		{
			//geomArray = planes;
			return true;
		}
	}
	return false;
}
bool SAArm::SelectMultipleCylindersFromSA()
{
	SAIPRequestMultipleObjects obReq;
	obReq.Type = obReq.Cylinder;
	waitingForGeomArrayFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Spheres From SA"));
	prog.Start(Tr("Selecting Spheres..."));
	prog.SetMsg(Tr("Selecting Spheres..."));
	prog.StepBar(0); // this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForGeomArrayFromSA = false;
			return false;
		}
		if(!waitingForGeomArrayFromSA)
		{
			//geomArray = spheres;
			return true;
		}
	}
	return false;
}
bool SAArm::SelectMultipleSpheresFromSA()
{
	SAIPRequestMultipleObjects obReq;
	obReq.Type = obReq.Sphere;
	waitingForGeomArrayFromSA = true;
	QS.SendPacket(obReq);

	NRKProgress prog(Tr("Selecting Spheres From SA"));
	prog.Start(Tr("Selecting Spheres..."));
	prog.SetMsg(Tr("Selecting Spheres..."));
	prog.StepBar(0); // this is to suppress the new autoStepping in nrkprog
	while(1)
	{
		Sleep(50);
		ProcessWaitingMessages();
		if(prog.cancelWasPressed)
		{
			waitingForGeomArrayFromSA = false;
			return false;
		}
		if(!waitingForGeomArrayFromSA)
		{
			//geomArray = spheres;
			return true;
		}
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////////
void SAArm::SendViewControlToSA(SAPPViewControl& vc)
{
	vc.InterpView = TRUE;
	QS.SendPacket(vc);

	if(vc.ViewMode != SAPPViewControl::Autoscale)
	{
		SAPPViewControl as;
		as.ViewMode = SAPPViewControl::Autoscale;
		QS.SendPacket(as);
	}
}
void SAArm::SendInspectionNavigationToSA(SAIPInspectionNavigation &in)
{
	QS.SendPacket(in);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FaroUSB TrackArm
bool SAArm::SelectTrackArmTrackerFromSA()
{
	WORD selectedInstType;
	if( !SelectInstrumentInSA(trackArmTrackerInstSerialNumber, selectedInstType) )
		return false;

	if(selectedInstType != 42 && selectedInstType != 108)	// 42 = INST_FAROSMX_TRACKER, 108 is Faro Ion
	{
		AfxMessageBox(Tr("You MUST select a Faro Tracker."), MB_ICONSTOP);
		trackArmTrackerInstSerialNumber = 0;
		return false;
	}
	
	return true;
}
void SAArm::OnTrackArmTransformChanged(const Transform& trackArmTransform, const bool makeCopyOfInstrument)
{
	TRACE0("Ready to send this transform to SA...\n");
	trackArmTransform.Debug();

	if( InitiateInstrumentJumpFromInterface(trackArmTrackerInstSerialNumber, trackArmTransform, makeCopyOfInstrument) )
		Data.InvalidateReferencePlanes();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//>> SA Toolbar
void SAArm::GetInterfaceRemoteControlCapabilities(BOOL& supported, int& interfaceType)
{
	supported = true;
	interfaceType = SAIRCInitialization::REMOTE_ARM;
}
//void SAArm::OnSAIRCMeasure(SAIRCMeasure& po)		// measurement triggered from toolbar.
//void SAArm::OnSAIRCStatus(SAIRCStatus& po)		// status update requested for toolbar
void SAArm::OnSAIRCAction(SAIRCAction& po)		// interface action requested by SA toolbar
{
	SAIRCActionArm* pActionArm = dynamic_cast<SAIRCActionArm*>(&po);
	if(!pActionArm)
		return;

	SetInterfaceBusy(true);

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	if(app.measDlg && app.pA->Measuring)
		app.measDlg->EndCancel();

	if(app.pA->IsFaroUSB())
		app.pA->StopScannerMeas();

	switch (pActionArm->action)
	{
	case SAIRCActionArm::ACTION_MFCR_CAL:
		{
			if(app.pA->IsFaro())
			{
				d.WriteHead("_S0 1 0100 0700");
				Delay(0.5,TRUE);
			}
			d.StopQueryTimer();
			if( app.pA->IsRomer() )
			{
				DCALCHECK diag;
				diag.DoModal();
			}
			else if( app.pA->IsFaro() )
			{
				d.CloseComm();
				DCalFaro dlg;
				dlg.DoModal();
				Delay(0.25, TRUE);	// 2001.04.20
				if( d.InitComm(FALSE) )
					d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
				else
				{
					AfxMessageBox(Tr("Could not re-start serial communications\nwith arm.  You will need to re-start the interface."));
					d.CloseComm();
					PostQuitMessage(0);
				}
			}
			else if(app.pA->IsFaroUSB() || app.pA->IsMetrisGarda() || app.pA->IsAPIBacesAxxis() || app.pA->IsKreonScannerArm() || app.pA->IsHexagonRDS())
				app.pA->DoCal();

			if(!app.pA->IsKreonScannerArm() && !app.pA->trackArmActive)
				d.StartQueryTimer(app.saarm.Data.updatetimerperiodsec);
		}
		break;

	case SAIRCActionArm::ACTION_MFCR_SETTINGS:
		app.pA->DoArmSettings();
		break;
	case SAIRCActionArm::ACTION_SCANNER_SETTINGS:
		app.pA->ScannerSettings();
		break;
	case SAIRCActionArm::ACTION_SET_STREAM_PTS_PARAMS:
		app.pA->StreamPtsSettings();
		break;
	case SAIRCActionArm::ACTION_SET_MEAS_MODE:
		app.pA->SetMeasMode(pActionArm->GetArmAction());
		break;
	default:
		break;
	}

	d.RestartMeasurement();	// SI arms have the scanning switch on them
	SetInterfaceBusy(false);
}
//void SAArm::OnSAIRCAttributes(SAIRCAttributes& po)	// SA wants to know your attributes.
void SAArm::GetInterfaceCollectionGroupTarget(CString& col, CString& grp, CString& tgt) // SA toolbar needs current CGT
{
	col = Data.Collection;
	grp = Data.Group;
	tgt = Data.Target;
}
void SAArm::ApplyInterfaceVisibility(const bool visible)
{
	if(app.measDlg && app.pA->Measuring)
		app.measDlg->EndCancel();

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	if(visible)
	{
		units.toolBarModeOn = false;

		RestoreArmButtonsForMainInterface();

		if(!app.pA->armScannerSwitchOn)
		{
			app.pA->SetMeasMode(0);
		}
		else 
		{
			app.pA->SetMeasMode(13);
		}
		
		d.MakeVisible();
	}
	else
	{
		units.toolBarModeOn = true;

		//SendSAIRCAttributesToSA();	// 2013.09.26 - we need attributes to show up before status in the toolbar - otherwise, stuff like battery icons could just show defaults instead of the actual current state
		
		SetArmButtonsForToolbar();
		d.MakeInvisible();
		d.SetDataParams();	// get C::G::T names from main dlg
		OnCGTargetNameChanged();
		
		if(!app.pA->armScannerSwitchOn)
		{
			app.pA->SetMeasMode(0);
			d.RestartMeasurement();
		}
		else
		{
			app.pA->SetMeasMode(13);
		}
			
		SendSAIRCAttributesToSA();
		//SendSAIRCStatusToSA();
	}

	//app.ub.SetWasInToolBarMode(units.toolBarModeOn);
}
int SAArm::GetMeasMode()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	return d.measmode;
}
void SAArm::RestartMeasurement()
{
	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	d.RestartMeasurement();
}
bool SAArm::EndMeasurement()
{
	if(app.measDlg && app.pA->Measuring)
	{
		try
		{
			app.measDlg->EndCancel();
			return true;
		}
		catch (CMemoryException* e)
		{
			return false;
		}
		catch (CFileException* e)
		{
			return false;
		}
		catch (CException* e)
		{
			return false;
		}
		catch(...)
		{
			return false;
		}
	}
	return false;
}
bool SAArm::SendSAIRCAttributesToSA()
{
	if(!units.toolBarModeOn)
		return false;

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);

	SAIRCAttributesArm att;
	//att.SetNeedsScannerMeasButton(arm_type == arm_Hexagon_Absolute_SI ? false : true); // app.pA->IsHexagonRDS() // decided to just show the scan icon - if clicked, it tells the user to use the arm's switch, when using the arm's switch, we set the meas mode (active radio in toolbar) via arm attributes
	
	att.SetActiveMeasMode(d.measmode);

	att.SetProbeDiaInches(FIXROUND(app.saarm.Data.TipDia/app.saarm.units.UnitToMeters[DISTUNITS_IN]));

	CString buttonFcns;

	switch (arm_type)
	{
	case arm_FaroUSB:
		buttonFcns.Format(Tr("Button1 - short/Record, long/Delete Last\r\nButton2 - short/Next Feature, long/Iterate Measure Mode"));
		break;
	case arm_Romer:
		buttonFcns.Format(Tr("Button1 - short/Next Feature, long/Iterate Measure Mode\r\nButton2 - short/Record, long/Delete Last\r\nButton3 - short/Next Feature, long/Iterate Measure Mode"));
		break;
	case arm_Axxis_KreonScanner:
		buttonFcns.Format(Tr("Button1 - short/Record, long/Delete Last\r\nButton0 - short/Next Feature, long/Iterate Measure Mode"));
		if(CMMString.Find(_T("Sc")) < 0)
			att.SetHasScanner(false);
		break;
	case arm_Axxis_Arm:
		buttonFcns.Format(Tr("Button1 - short/Record, long/Delete Last\r\nButton 2 - short/Next Feature, long/Iterate Measure Mode\r\nButton3 - short/Record, long/Delete Last"));
		break;
	case arm_FaroSerial:
		break;
	case arm_SNL:
		break;
	case arm_RomerArticulator:
		break;
	case arm_RomerEurope:
		break;
	case arm_MetrisMCA:
		break;
	case arm_Hexagon_MultiGage:
		buttonFcns.Format(Tr("Button1 - short/Next Feature, long/Iterate Measure Mode\r\nButton2 - short/Record, long/Delete Last\r\nButton3 - short/Next Feature, long/Iterate Measure Mode"));
		break;
	case arm_Hexagon_Absolute_SI:
		buttonFcns.Format(Tr("Button1 - short/Next Feature, long/Iterate Measure Mode\r\nButton2 - short/Record, long/Delete Last\r\nButton3 - short/Next Feature, long/Iterate Measure Mode"));
		break;
	case arm_Hexagon_Absolute:
		buttonFcns.Format(Tr("Button1 - short/Next Feature, long/Iterate Measure Mode\r\nButton2 - short/Record, long/Delete Last\r\nButton3 - short/Next Feature, long/Iterate Measure Mode"));
		break;
	default:
		TRACE0("Unsupported Arm type - SAArm::SetArmButtonsForToolbar()\n");
		break;
	}
	att.SetArmButtonFunctions(buttonFcns);
	
	return SendSAIRCPacket(att);
}
void SAArm::SetArmButtonsForToolbar()
{
	Data.buttonOneMain = Data.ButtonOneMode;
	Data.buttonTwoMain = Data.ButtonTwoMode;
	Data.buttonThreeMain = Data.ButtonThreeMode;

	switch (arm_type)
	{
	case arm_FaroUSB:
		Data.ButtonOneMode = bf_record_deleteLast;
		Data.ButtonTwoMode = bf_next_iterateMeasMode;
		Data.ButtonThreeMode = TOUCH_PROBE;
		break;
	case arm_Romer:
		Data.ButtonOneMode = bf_next_iterateMeasMode;
		Data.ButtonTwoMode = bf_record_deleteLast;
		Data.ButtonThreeMode = bf_next_iterateMeasMode;
		break;
	case arm_Axxis_KreonScanner:
		Data.ButtonOneMode = bf_next_iterateMeasMode;
		Data.ButtonTwoMode = bf_record_deleteLast;
		break;
	case arm_Axxis_Arm:
		Data.ButtonOneMode = bf_record_deleteLast;
		Data.ButtonTwoMode = bf_next_iterateMeasMode;
		Data.ButtonThreeMode = bf_record_deleteLast;
		break;
	case arm_FaroSerial:
		break;
	case arm_SNL:
		break;
	case arm_RomerArticulator:
		break;
	case arm_RomerEurope:
		break;
	case arm_MetrisMCA:
		break;
	case arm_Hexagon_MultiGage:
		Data.ButtonOneMode = bf_next_iterateMeasMode;
		Data.ButtonTwoMode = bf_record_deleteLast;
		Data.ButtonThreeMode = bf_next_iterateMeasMode;
		break;
	case arm_Hexagon_Absolute_SI:
		Data.ButtonOneMode = bf_next_iterateMeasMode;
		Data.ButtonTwoMode = bf_record_deleteLast;
		Data.ButtonThreeMode = bf_next_iterateMeasMode;
		break;
	case arm_Hexagon_Absolute:
		Data.ButtonOneMode = bf_next_iterateMeasMode;
		Data.ButtonTwoMode = bf_record_deleteLast;
		Data.ButtonThreeMode = bf_next_iterateMeasMode;
		break;
	default:
		TRACE0("Unsupported Arm type - SAArm::SetArmButtonsForToolbar()\n");
		break;
	}
}
void SAArm::RestoreArmButtonsForMainInterface()
{
	Data.ButtonOneMode = Data.buttonOneMain;
	Data.ButtonTwoMode = Data.buttonTwoMain;
	Data.ButtonThreeMode = Data.buttonThreeMain;
}
//<< SA Toolbar
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SAArm::Serialize(CArchive& ar) 
{
	SAInstBase::Serialize(ar);
	Data.Serialize(ar);
	units.Serialize(ar);
	if(arm_type == arm_FaroUSB)
		Data.ButtonThreeMode = TOUCH_PROBE;
}

IMPLEMENT_SERIAL( SAArm, SAInstBase, 0 );