#pragma once

#include "DataStore.h"
#include "SocketGeometry.h"
#include "Units.h"
#include "GeomTriggerWrapper.h"

class GeomTriggerWrapper;

class SAArm :
	public SAInstBase
{
public:
	SAArm(void);
	virtual ~SAArm(void);

	Store& Data;
	Units& units;
	SocketGeometry& SockGeom;
	GeomTriggerWrapper& geomTriggerWrap;

	bool receivedSignOn;
	BOOL scannerLicenseValid;
	CString CMMString;

	enum ArmTypes
	{
		arm_FaroUSB = 0,
		arm_Romer = 1,
		arm_FaroSerial = 2,
		arm_SNL = 3,
		arm_RomerArticulator = 4,
		arm_RomerEurope = 5,
		arm_MetrisMCA = 6,
		arm_Axxis_Arm = 7,
		arm_Axxis_KreonScanner = 8,
		arm_Hexagon_MultiGage = 9,
		arm_Hexagon_Absolute_SI = 10,
		arm_Hexagon_Absolute = 11
	};
#define MAX_ARM_TYPE 11

	int arm_type;
	bool SetArmTypeFromSAInstModel(const int saInstModel);
	bool EstablishArmFromArmType();
	int DOF;
	GenVec J;	// joints
	GenVec lastJ;
	GenVec JShift; // for Faro's

	CString lastToolBarMeasModeName;

	enum buttonFunction
	{
		bf_none = 0,
		bf_record_accept = 1,
		bf_deleteLast_mouse = 2,
		bf_deleteLast_cancel = 3,
		bf_record_deleteLast = 4,
		bf_accept_cancel = 5,
		bf_next_previous = 6,
		bf_next_iterateMeasMode = 7,
		// add new ones above here, and bump TOUCH_PROBE
 		bf_touch_probe = 8,
	};

	bool toolBarDevFlagSet;

	virtual bool PopCommSettingsDlg() {TRACE0("SAArm PopCommSettingsDlg called.\n"); return true;}

	// from SAIB
	virtual BOOL Logon();
	virtual BOOL PopLogonDialog();
	virtual BOOL DoInitInstanceStuff();	// overridden so I don't have to work around the fact that the base class has stuff like paths AND a call to LoadSettings - I need 
	virtual BOOL ConnectToHardware();
	// HOWEVER, still have to override it here because SAInstBase version sets simila
	virtual BOOL DisconnectFromHardware();
	virtual void OnHeartbeatStopped();	// 2001.11.19

	// Socket stuff
	virtual void OnInstToWorkingChanged();
	virtual void OnSAIPSignOn( SAIPSignOn& p );	// overridden due to many arm types - don't want a separate build for, say, all the Romers
	virtual void OnSAIPMeasurement( SAIPMeasurement& m );	// measurement packet (automeasure)
	virtual void OnSAIPMeasData( SAIPMeasData& m );			// update packet (point at)
	virtual void OnSAIPFlags( SAIPFlags& p );
	virtual void OnSAIPMeasGroup(SAIPMeasGroup& p);
	virtual void OnSAIPGeom( SAIPGeom& g );		// for inspections
	virtual void OnSAIPGeomArray( SAIPGeomArray& geoms ); // initially added for GeomTriggers
	// added 2001.11.11
	virtual void OnSAIPMeasRequest(SAIPMeasRequest& p );			// was QSMeasRequest
	// added 2004.10.08
	virtual void OnSAIPGroupTarget(SAIPGroupTarget& p);	// was QSGroupTarget
	virtual BOOL SetMeasurementMode(CString& mode, CString* pErrorString);
	virtual BOOL GetCloudDataSendingPeriod(double& periodSec);  // get the current rate at which scan cloud data is being sent in seconds
	virtual void OnSAIPHitMeasureButton(SAIPHitMeasureButton& p);
	virtual void OnSAIPMeasureSinglePointHere(SAIPMeasureSinglePointHere& p);
	virtual void OnSAIPMeasureSinglePointThere(SAIPMeasureSinglePointThere& p);
	///////////////////////////////////////////////////
	// 2006.06.09
	virtual void OnNonStandardPacket( CObject& pkt );
	void OnSAIPAutoMeasProximity(SAIPAutoMeasProximity &p);
	///////////////////////////////////////////////////

	virtual void OnSAIPInfo(SAIPInfo& p);	// 2013.04.19 - Mantis #1329 - Want function to always push unit settings to instruments 

	virtual BOOL OpCheck( CString& Command );	// need to override because the base class pops a message!
	virtual void ApplyCribSheet(SAIPCribSheet& pkt);	// 2005.04.12 - added

	BOOL SendReportItem(CString& Title, CString& Type, CStringArray& BodyText);
	BOOL SendReportItem(CString& Title, CString& Type, CString& BodyText);
	BOOL SendReportItem(const char* Title, const char* Type, CStringArray& BodyText);
	BOOL SendReportItem(const char* Title, const char* Type, CString& BodyText);

	// 2006.06.07 - The Screamin' Weasel...
	void SetScreamerParams();
	void SetScreamerParams(const double& targetAreaThreshold, const double& workingFieldThreshold);
	void StartScreaming();
	void StopScreaming();
	void ScreamThis(const double& error);

	// 2009.01.13
	bool SelectPlaneFromSA(SAIPGeomPlane& p);
	// 2010.01.21
	bool SelectLineFromSA(SAIPGeomLine& l);
	bool SelectCylinderFromSA(SAIPGeomCylinder& c);
	bool SelectSphereFromSA(SAIPGeomSphere& s);

	bool SelectMultiplePlanesFromSA();
	bool SelectMultipleCylindersFromSA();
	bool SelectMultipleSpheresFromSA();

	// 2010.09.02
	void SendViewControlToSA(SAPPViewControl& vc);
	void SendInspectionNavigationToSA(SAIPInspectionNavigation &in);

	// 2009.09.25
	DWORD_PTR trackArmTrackerInstSerialNumber;
	bool SelectTrackArmTrackerFromSA();
	void OnTrackArmTransformChanged(const Transform& trackArmTransform, const bool makeCopyOfInstrument);

	//>> SA Toolbar
	virtual void GetInterfaceRemoteControlCapabilities(BOOL& supported, int& interfaceType);	// override to enable remote for your interface.
	//virtual void OnSAIRCMeasure(SAIRCMeasure& po);		// measurement triggered from toolbar.
	//virtual void OnSAIRCStatus(SAIRCStatus& po);		// status update requested for toolbar
	virtual void OnSAIRCAction(SAIRCAction& po);		// interface action requested by SA toolbar
	//virtual void OnSAIRCAttributes(SAIRCAttributes& po);	// SA wants to know your attributes.
	virtual void GetInterfaceCollectionGroupTarget(CString& col, CString& grp, CString& tgt);  // SA toolbar needs current CGT
	virtual void ApplyInterfaceVisibility(const bool visible);
	void SetArmButtonsForToolbar();
	void RestoreArmButtonsForMainInterface();
	bool SendSAIRCAttributesToSA();
	int GetMeasMode();
	void RestartMeasurement();
	bool EndMeasurement();
	//<< SA Toolbar

	void Serialize(CArchive& ar);

protected:   
	DECLARE_SERIAL( SAArm );

private:
	AudibleError& screamer;
	bool waitingForProjectionPlaneFromSA;
	bool waitingForLineFromSA;
	bool waitingForPointFromSA;
	bool waitingForCylinderFromSA;
	bool waitingForSphereFromSA;

	bool waitingForGeomArrayFromSA;

	SAIPGeomPlane& gp;
	SAIPGeomLine& gl;
	SAIPGeomCylinder& gCyl;
	SAIPGeomSphere& gs;
	bool invalidateRefPlanesAfterStartup;
	double scaleFromInstBase;

	CString lastCollectionNameFromSA;
	CString lastGroupNameFromSA;
};
