// SNLArm.cpp: implementation of the SNLArm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DPCMM.h"
#include "DPCMMDlg.h"
#include "SNLArm.h"
#include "DSerialPortSettings.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SNLArm::SNLArm()
{
	ConnectedToArm = FALSE;
	Measuring = FALSE;

	app.saarm.CMMString = _T("SNL");
	app.saarm.SettingsFileName = _T("SA_SandiaNatLabs_ArmSettings.bin");
	LogFileName = _T("SNLArm.log");

	DOF = 3;
}

SNLArm::~SNLArm()
{

}

bool SNLArm::PopCommSettingsDlg()
{
	DSerialPortSettings dlg(app.saarm.arm_SNL);
	dlg.m_CommPort = app.saarm.Data.CommPort-1;
	if(DOMODAL(dlg) != IDOK)
		return false;
	app.saarm.Data.CommPort = dlg.m_CommPort+1;
	return true;
}
bool SNLArm::ConnectToArm(const int dof)
{
	J.Create(dof, TRUE);	// create joint vector
	lastJ.Create(dof, TRUE);
	JShift.Create(dof,TRUE);

	if(!app.saarm.AutoSocket.Valid)
		if(!PopCommSettingsDlg())
			return false;

	app.saarm.Data.TipDia = 0.0254;
	// Initialize vairable for the GALIL MOTION CONTROLLER and the VDC
	// 2004.09.02 - now the main dlg does not exist when we connect to the arm...
	//CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	CDMCWinRegistry DMCWinRegistry;
	GALILREGISTRY galilregistry;
	char Enc1OSTxt[20], Enc2OSTxt[20], Enc3OSTxt[20];
	char* ptrEnc1OSTxt=Enc1OSTxt;
	char* ptrEnc2OSTxt=Enc2OSTxt;
	char* ptrEnc3OSTxt=Enc3OSTxt;

	// Setup the VDC member variables
	// Declaration of VDCurPos occurred in VDCDLG.h
	VDCCurPos.VDCZeroValue();
	VDCCurPos.VDCLoadKP("VDC_KP_1.dat");
	VDCCurPos.VDCDispOS(ptrEnc1OSTxt, ptrEnc2OSTxt, ptrEnc3OSTxt);
//	m_enc1os.SetWindowText(Enc1OSTxt);
//	m_enc2os.SetWindowText(Enc2OSTxt);
//	m_enc3os.SetWindowText(Enc3OSTxt);

	// Is the controller registered?
	if (DMCWinRegistry.GetGalilRegistryInfo(nController, &galilregistry) != 0)
	{
//		d.status.SetWindowText("Could not find controller in the Windows registry");
		TRACE("### Galil Controller was not found.\n");
		return false;
	}
	
	// Initialize the DMCWin object
	// Declaration of m_DMCWin occurred in VDCDlg.h
	m_DMCWin.SetController(nController);	// Controller 1 in the Windows registry

	// 2004.09.02 - now the main dlg does not exist when we connect to the arm...
	m_DMCWin.SethWnd(NULL);
	//m_DMCWin.SethWnd(d.GetSafeHwnd());			// Our hWnd;

	// Open a connection to the controller 
	m_DMCWin.Open();

	//
	// NOTE: We could have omitted the calls to SetController and SethWnd and
	//       called Open with arguments:
	//
	//    m_DMCWin.Open(nController, GetSafeHwnd());
	//

	if (m_DMCWin.IsConnected())
	{
		// Display the controller version information
		char szVersion[128];

		if (m_DMCWin.GetVersion(szVersion, sizeof(szVersion)) == 0)
//			d.status.SetWindowText(szVersion);
			TRACE("Connected: %s", szVersion);
	}
	else
	{
		// Report error
		char szMessage[128];
		CString TempString;

		m_DMCWin.GetErrorText(m_DMCWin.GetLastError(), szMessage, sizeof(szMessage));
		TempString = "Could not connect to controller. ";
		TempString += szMessage;
		AfxMessageBox(TempString);

//		d.status.SetWindowText("Not connected with controller");
		return false;
	}

	ConnectedToArm = TRUE;

	// set NativeArmToSA transform
	Vector rot(0.0,0.0,24.0);
	NativeArmToSA.SetRotation(rot);
	Vector trans(-0.5,-3.0,0.0);
	trans.Scale(app.saarm.InchesToLocal);
	NativeArmToSA.SetTranslation(trans);

	return true;
}

BOOL SNLArm::GetAllData()
{
	if( !ConnectedToArm )
		return ArmBase::GetAllData();
	
	TRACE("****GetAllData!!!\n");
	if( !m_DMCWin.IsConnected() )
	{
		TRACE("DMCWin is not connected.\n");
		return FALSE;
	}
	char Enc1CntsTxt[20], Enc2CntsTxt[20], Enc3CntsTxt[20];
	char* ptrEnc1CntsTxt=Enc1CntsTxt;
	char* ptrEnc2CntsTxt=Enc2CntsTxt;
	char* ptrEnc3CntsTxt=Enc3CntsTxt;
	char Theta1Txt[20], Theta2Txt[20], Theta3Txt[20];
	char* ptrTheta1Txt=Theta1Txt;
	char* ptrTheta2Txt=Theta2Txt;
	char* ptrTheta3Txt=Theta3Txt;
	char XTxt[20],YTxt[20],ZTxt[20];
	char* ptrXTxt=XTxt;
	char* ptrYTxt=YTxt;
	char* ptrZTxt=ZTxt;
	char szMessage[1000];
	
	// Galil controller is instructed to "Tell Position -TP"
	m_DMCWin.Command("TP",szMessage,sizeof(szMessage));

	CDPCMMDlg& d = *((CDPCMMDlg*)app.m_pMainWnd);
	
	//Message is sent to VDCPosition class member function to be parsed and the data stored.
	//Text values for the encoder counts GUI variables are returned for display.
	VDCCurPos.VDCLoadCnts(szMessage, ptrEnc1CntsTxt, ptrEnc2CntsTxt, ptrEnc3CntsTxt);
//	m_enc1cnts.SetWindowText(Enc1CntsTxt);
//	m_enc2cnts.SetWindowText(Enc2CntsTxt);
//	m_enc3cnts.SetWindowText(Enc3CntsTxt);

	//The angular positions are then calculated by a VDCPosition class member functnion.
	//Text values for the joint angles GUI variables are returned for display.
	VDCCurPos.VDCCalcAng(ptrTheta1Txt, ptrTheta2Txt, ptrTheta3Txt);
	sscanf(Theta1Txt,"%lf",&J.x[0]);
	sscanf(Theta2Txt,"%lf",&J.x[1]);
	sscanf(Theta3Txt,"%lf",&J.x[2]);
	J.x[0] += 24.0;
	J.x[1] *= -1.0;
	J.x[2] *= -1.0;
//	d.jt1.SetWindowText(Theta1Txt);
//	d.jt2.SetWindowText(Theta2Txt);
//	d.jt3.SetWindowText(Theta3Txt);
	
	//The Cartesian coordinates of the probr tip are calcualted by a VDCPosition class member function.
	//Text values for the Cartesian position GUI variables are returned for display.
	VDCCurPos.VDCCalcPos(ptrXTxt, ptrYTxt, ptrZTxt);
	sscanf(XTxt,"%lf",&X);
	sscanf(YTxt,"%lf",&Y);
	sscanf(ZTxt,"%lf",&Z);
	TRACE3("SNL Cartesian Data: X=%f,Y=%f,Z=%f millimeters\n",X,Y,Z);

//	CString m;
//	m.Format("X=%4.1f,Y=%4.1f,Z=%4.1fmm\n",X,Y,Z);
//	d.jtlimit.SetWindowText(m);

	X /= 1000.0f;
	Y /= 1000.0f;
	Z /= 1000.0f;

	Vector xyzArm(X,Y,Z), xyzSA;
	NativeArmToSA.Multiply(xyzArm, &xyzSA);
	X = xyzSA.x;
	Y = xyzSA.y;
	Z = xyzSA.z;

//	d.x.SetWindowText(XTxt);
//	d.y.SetWindowText(YTxt);
//	d.z.SetWindowText(ZTxt);

	return TRUE;
}

BOOL SNLArm::IsRomer()
{
	return FALSE;
}
BOOL SNLArm::IsFaro()
{
	return FALSE;
}
BOOL SNLArm::IsFaroUSB()
{
	return FALSE;
}
BOOL SNLArm::IsFrenchRomer()
{
	return FALSE;
}
BOOL SNLArm::IsSNL()
{
	return TRUE;
}
BOOL SNLArm::IsMetrisGarda()
{
	return FALSE;
}