#include "stdafx.h"

#include "tcpip.h"
#pragma comment(lib, "ws2_32.lib")

static WSADATA g_wsaData;

const int TCPIP_MAX_BUFFER = 8192;//16 * 1024 * 1024;
const int MAX_STRING       = 1024;

/////////////////////////////////////////////////////////////////////////////
// Initialize TCPIP

bool TCPIPInitialize(void)
{
   WORD    wVersionRequested;
   WSADATA wsaData;
   int     err; 

   wVersionRequested = MAKEWORD(2, 2); 
   err = WSAStartup(wVersionRequested, &wsaData);
   if ( err != 0 ) 
   {
      printf("WSAStartup failed [%d]\n", WSAGetLastError());
      return false;
   }

   return true;
}

/////////////////////////////////////////////////////////////////////////////
// Cleanup TCPIP

void TCPIPCleanup(void)
{
    WSACleanup();
}

/////////////////////////////////////////////////////////////////////////////
// TCPIPConnect - connect

bool TCPIPConnect(SOCKET* phSocket, TCHAR* pszHost, WORD wPort)
{
   struct sockaddr_in saHost;
   struct hostent*    phost;
   int    i;

   if (pszHost == NULL)
   {
      printf("Invalid parameter 'Host Name'\n");
      return false;
   }

   *phSocket = socket(AF_INET, SOCK_STREAM, 0);
   if(*phSocket == INVALID_SOCKET)
   {
      printf("Cannot initialize socket [0x%08X]\n", WSAGetLastError());
      return false;
   }

   i = TCPIP_MAX_BUFFER;
   i = setsockopt(*phSocket, SOL_SOCKET, SO_SNDBUF, (LPSTR)&i, sizeof(i));
   if (i == SOCKET_ERROR)
   {
      closesocket(*phSocket);
      *phSocket = INVALID_SOCKET;
      printf("Cannot set socket options [0x%08X]\n", WSAGetLastError());
      return false;
   }

   i = TCPIP_MAX_BUFFER;
   i = setsockopt(*phSocket, SOL_SOCKET, SO_RCVBUF, (LPSTR)&i, sizeof(i));
   if (i == SOCKET_ERROR)
   {
      closesocket(*phSocket);
      *phSocket = INVALID_SOCKET;
      printf("Cannot set socket options [0x%08X]\n", WSAGetLastError());
      return false;
   }

	//i = 10000;
	//i = setsockopt(*phSocket, SOL_SOCKET, SO_RCVTIMEO, (LPSTR)&i, sizeof(i));
	//if (i == SOCKET_ERROR)
	//{
    //  closesocket(*phSocket);
    //  *phSocket = INVALID_SOCKET;
    //  printf("Cannot set socket options [0x%08X]\n", WSAGetLastError());
    //  return false;
	//}
	ULONG j = 0; // Zero means block
	if (ioctlsocket(*phSocket, FIONBIO, &j) == SOCKET_ERROR)
	{
		printf("Cannot set socket options");
		return false;
	}

   i = TRUE;
   i = setsockopt(*phSocket, IPPROTO_TCP, TCP_NODELAY, (LPSTR)&i, sizeof(i));
   if (i == SOCKET_ERROR)
   {
      closesocket(*phSocket);
      *phSocket = INVALID_SOCKET;
      printf("Cannot set socket options [0x%08X]\n", WSAGetLastError());
      return false;
   }

   memset(&saHost, 0, sizeof(saHost));
   char  szHost[MAX_STRING];

   for(i = 0; (i < MAX_STRING) && (pszHost[0] != '\0'); i++)
      szHost[i] = (char)pszHost[i];

   if (inet_addr(szHost) != INADDR_NONE)
   {
      saHost.sin_family           = AF_INET;
      saHost.sin_port             = htons((u_short)wPort);
      *(DWORD*)&(saHost.sin_addr) = inet_addr(szHost);
   }
   else
   {
      phost = gethostbyname(szHost);
      if (phost == NULL)
      {
         closesocket(*phSocket);
         *phSocket = INVALID_SOCKET;
         printf("Cannot find host address [0x%08X]\n", WSAGetLastError());
         return false;
      }
      saHost.sin_family = (BYTE)phost->h_addrtype;
      saHost.sin_port   = htons((u_short)wPort);
      memcpy((char *)&saHost.sin_addr, phost->h_addr, phost->h_length);
   }

   if (connect(*phSocket, (struct sockaddr*)&saHost, sizeof(saHost)) != 0)
   {
      closesocket(*phSocket);
      *phSocket = INVALID_SOCKET;
      printf("Cannot connect to host [0x%08X]\n", WSAGetLastError());
      return false;
   }

   return true;
}

/////////////////////////////////////////////////////////////////////////////
// TCPIPConnect - close

bool TCPIPClose(SOCKET hSocket)
{
   if (shutdown(hSocket, 1) != 0)
   {
      printf("Error shutting down socket [0x%08X]\n", WSAGetLastError());
      return true;
   }

   if (closesocket(hSocket) != 0)
   {
      printf("Error closing socket [0x%08X]\n", WSAGetLastError());
      return true;
   }

   return true;
}

/////////////////////////////////////////////////////////////////////////////
// TCPIPConnect - listen

bool TCPIPListen(SOCKET* phSocket, WORD wPort)
{
   SOCKET      hListen;
   sockaddr_in sin;

   hListen = socket(AF_INET, SOCK_STREAM, 0);
   if (hListen == SOCKET_ERROR)
   {
      printf("Cannot create socket [%d]", WSAGetLastError());
      return false;
   }

   memset(&sin, 0, sizeof(sin));
   sin.sin_family = AF_INET;
   sin.sin_port   = htons(wPort);
   if (bind(hListen, (sockaddr*)&sin, sizeof(sin)) == SOCKET_ERROR)
   {
      printf("Could not bind to port [%d]", WSAGetLastError());
      return false;
   }

   if (listen(hListen, 5) == SOCKET_ERROR)
   {
      printf("Could not listen [%d]", WSAGetLastError());
      return false;
   }

   printf("Listening on port %d\n", (DWORD)wPort);

   for(;;)
   {
      SOCKET      hConnect;
      sockaddr_in sinfrom;
      int         nLength = sizeof(sinfrom);

      hConnect = accept(hListen, (sockaddr*)&sinfrom, &nLength);

      if (hConnect == 0)
      {
         printf("Failed to accept [%d]", WSAGetLastError());
         break;
      }

      DWORD dwAddr = ntohl(sinfrom.sin_addr.s_addr);
      int   i      = 0;

      i = TCPIP_MAX_BUFFER;
      i = setsockopt(hConnect, SOL_SOCKET, SO_SNDBUF, (LPSTR)&i, sizeof(i));
      if (i == SOCKET_ERROR)
      {
         printf("Cannot set socket options [%d]", WSAGetLastError());
         break;
      }

      i = TCPIP_MAX_BUFFER;
      i = setsockopt(hConnect, SOL_SOCKET, SO_RCVBUF, (LPSTR)&i, sizeof(i));
      if (i == SOCKET_ERROR)
      {
         printf("Cannot set socket options [%d]", WSAGetLastError());
         break;
      }

      i = TRUE;
      i = setsockopt(hConnect, IPPROTO_TCP, TCP_NODELAY, (LPSTR)&i, sizeof(i));
      if (i == SOCKET_ERROR)
      {
         printf("Cannot set socket options [%d]", WSAGetLastError());
         break;
      }

      printf("Accepted connection from %d.%d.%d.%d\n", ((dwAddr >> 24) & 255), ((dwAddr >> 16) & 255), ((dwAddr >> 8) & 255), ((dwAddr) & 255));

      *phSocket = hConnect;

      return true;
   }

   TCPIPClose(hListen);
   return false;
}

bool TCPIPRead(SOCKET hSocket, LPVOID pData, long nSize, long* pnRead)
{
	long	nRead    = 0;
    char*	pszData  = (char*)pData;
	int		nGulp;

	*pnRead = 0;
	while(*pnRead < nSize)
	{
        nGulp = ((nSize - *pnRead) > TCPIP_MAX_BUFFER) ? TCPIP_MAX_BUFFER : (nSize - *pnRead);

		nRead = recv(hSocket, pszData + *pnRead, nGulp, 0);
		
		if (nRead == SOCKET_ERROR)
		{
			printf("error reading %d\n", WSAGetLastError());
			return false;
		}
		else if (nRead == 0) 
            return true;

		*pnRead += nRead;
	}

    return true;
}

bool TCPIPWrite(SOCKET hSocket, LPVOID pData, long nSize, long* pnWrite)
{
	long	nWrite  = 0;
	int		nGulp;

	*pnWrite = 0;
    while (*pnWrite < nSize)
	{
        nGulp = ((nSize - *pnWrite) > TCPIP_MAX_BUFFER) ? TCPIP_MAX_BUFFER : (nSize - *pnWrite);
		nWrite    = send(hSocket, ((const char*)pData) + *pnWrite, nGulp, 0);
 
		if (nWrite == SOCKET_ERROR)
		{
				return false;
		}
        else if (nWrite == 0)
            return true;

		*pnWrite += nWrite;
	}

	return true;
}


