// Units.cpp : implementation file
//

#include "stdafx.h"
#include "Units.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Units:  Interface to a general units class

bool Units::toolBarModeOn = false;

Units::Units()
{
	UnitToMeters[DISTUNITS_M] = 1.0;
	UnitToMeters[DISTUNITS_CM] = 1.0/100.0;
	UnitToMeters[DISTUNITS_MM] = 1.0/1000.0;
	UnitToMeters[DISTUNITS_FT] = 30.48/100.0;
	UnitToMeters[DISTUNITS_IN] = 2.54/100.0;

	InchesToLocal = 2.54/100.0;	// 'Local' is meters

	DistUnitNames[DISTUNITS_M] = "m";	//"Meters";
	DistUnitNames[DISTUNITS_CM] = "cm";	//"Centimeters";
	DistUnitNames[DISTUNITS_MM] = "mm";	//"Millimeters";
	DistUnitNames[DISTUNITS_FT] = "ft";	//"Feet";
	DistUnitNames[DISTUNITS_IN] = "in.";	//"Inches";

	AngUnitNames[ANGUNITS_D] = "deg";			// Decimal Degrees
	AngUnitNames[ANGUNITS_DMS] = "dms";	// Deg:Min:Sec
	AngUnitNames[ANGUNITS_R] = "rad";			// Radians
	AngUnitNames[ANGUNITS_GG] = "gons";			// Gons/Grad
	AngUnitNames[ANGUNITS_M] = "mil";			// Mils

	DTR = acos(-1.0) / 180.0;

	PressUnitNames[PRESSUNITS_HPA] = "hPa";
	PressUnitNames[PRESSUNITS_MMHG] = "mmHg";
	PressUnitNames[PRESSUNITS_INH20] = "inH20";
	PressUnitNames[PRESSUNITS_INHG] = "inHg";
	PressUnitNames[PRESSUNITS_PSIA] = "psi";
	PressUnitNames[PRESSUNITS_MBAR] = "mBar";

	UnitTommHg[PRESSUNITS_HPA] = 760.0/1010.0;
	UnitTommHg[PRESSUNITS_MMHG] = 1.0;
	UnitTommHg[PRESSUNITS_INH20] = 25.4/13.6;
	UnitTommHg[PRESSUNITS_INHG] = 25.4;
	UnitTommHg[PRESSUNITS_PSIA] = 760.0/14.7;
	UnitTommHg[PRESSUNITS_MBAR] = 760.0/1010.0;

	distunits = 0;	//inches
	angunits = 0;	//degrees
	tempunits = 1; // fahrenhiet
	pressunits = 3; //inHg
}

Units::~Units()
{
}

void Units::MakeAngleDisplayString(double& degaz, double& degel, CString* azout, CString* elout)
{
	char c1[200];
	char c2[200];
	
		switch(angunits)
		{
			case 0:				//decimal degrees
			{
				sprintf(c1,"%9.6lf"),degaz;
				sprintf(c2,"%9.6lf"),degel;
				break;
			}
			case 1:				//dms
			{
				int hd,hm,vd,vm;
				double hs,vs,rem;
				hd = (int)degaz;
				rem = degaz-(double)hd;
				hm = (int)(rem*60.0);
				rem = rem-(double)hm/60.0;
				hs = rem*3600.0;

				vd = (int)degel;
				rem = degel-(double)vd;
				vm = (int)(rem*60.0);
				rem = rem-(double)vm/60.0;
				vs = rem*3600.0;

				sprintf(c1,"%3d  %02d'  %04.1f''",hd,hm,hs);
				sprintf(c2,"%3d  %02d'  %04.1f''",vd,vm,vs);
				break;
			}
			case 2:				//radians
			{
				double dtr = (acos(-1.0))/180.0;
				sprintf(c1,"%9.6lf",degaz*dtr);
				sprintf(c2,"%9.6lf",degel*dtr);
				break;
			}
			case 3:				//gons
			{
				double dtg = 400.0/360.0;
				sprintf(c1,"%10.4lf",degaz*dtg);
				sprintf(c2,"%10.4lf",degel*dtg);
				break;
			}
			case 4:				//mils
			{
				double dtm = 6400.0/360.0;
				sprintf(c1,"%11.3lf",degaz*dtm);
				sprintf(c2,"%11.3lf",degel*dtm);
				break;
			}
	}
	*azout = c1;
	*elout = c2;
}

double Units::ScaleDistUnits(int From, int To)
{
	double CurrentToNew = UnitToMeters[ From ];
	CurrentToNew /= UnitToMeters[ To ];
	return ( (From==To) ? 1.0 : CurrentToNew );
}

double Units::DegreesToUnit()
{
	switch(angunits)
	{
	case 0:				//decimal degrees
		{
			return 1.0;
			break;
		}
	case 1:				//dms
		{
			return 1.0;	// they'll just get degrees in this case
			break;
		}
	case 2:				//radians
		{
			double dtr = (acos(-1.0))/180.0;
			return dtr;
			break;
		}
	case 3:				//gons
		{
			double dtg = 400.0/360.0;
			return dtg;
			break;
		}
	case 4:				//mils
		{
			double dtm = 6400.0/360.0;
			return dtm;
			break;
		}
	case 5:				//arcseconds
		{
			double dta = 3600.0;
			return dta;
			break;
		}
	default:
		{
			return 1.0;
			break;
		}
	}
}
double Units::InchesToUnit()
{
	return InchesToLocal/UnitToMeters[distunits];
}

void Units::Serialize(CArchive& ar) 
{
	BYTE LoadSchema, Schema = 0;        // version detection
	
	if (ar.IsStoring())
	{
		if(distunits<0 || distunits>=MAX_DISTUNITS)
			distunits = 0;	//inches
		if(angunits<0 || angunits>=MAX_ANGUNITS)
			angunits = 0;	//degrees
		if(tempunits<0 || tempunits>=MAX_TEMPUNITS)
			tempunits = 1; // fahrenhiet
		if(pressunits<0 || pressunits>=MAX_PRESSUNITS)
			pressunits = 3; //inHg

		ar << Schema;
		ar << distunits;
		ar << angunits;
		ar << tempunits;
		ar << pressunits;
	}
	else
	{
		ar >> LoadSchema;
		ar >> distunits;
		ar >> angunits;
		ar >> tempunits;
		ar >> pressunits;

		if(distunits<0 || distunits>=MAX_DISTUNITS)
			distunits = 0;	//inches
		if(angunits<0 || angunits>=MAX_ANGUNITS)
			angunits = 0;	//degrees
		if(tempunits<0 || tempunits>=MAX_TEMPUNITS)
			tempunits = 1; // fahrenhiet
		if(pressunits<0 || pressunits>=MAX_PRESSUNITS)
			pressunits = 3; //inHg
	}
}

IMPLEMENT_SERIAL( Units, CObject, 0 );